'use strict';
class ActivityObjectWrapper {
    constructor(p) {
        this.activity = p;
        this.activity.addEventListener('itemsChanged', this.processChildren.bind(this));
    }

    get Position() {
        if (!this.parent) {
            return "center";
        }

        var item = this;

        while (!item.position) {
            item = item.parent;
        }

        return item.position;
    }

    processChildren(e) {
        var splices = e.detail;

        if (this._children) {
            this.children = this._children;
        }

        if (!this.children) {
            this.children = this.activity.Items.map(function(c) { return new ActivityObjectWrapper(c); });
        } else {
            splices.forEach(function(splice) {
                this.children.splice(splice.index, splice.removed.length);

                for (var i = splice.index; i < splice.index + splice.addedCount; i++) {
                    this.children.splice(i, 0, new ActivityObjectWrapper(this.activity.Items[i]));
                }
            }.bind(this));
        }
        this.updateLeftAndRight();
    }

    getAllChildren() {
        return new Promise(function(resolve) {
            this.getChildren()
                .then(function(children) {
                    if (children) {
                        var p = children.map(function(i) { return i.getAllChildren(); });

                        Promise.all(p).then(resolve);
                    } else {
                        resolve();
                    }
                });
        }.bind(this));
    }

    getChildren() {
        return new Promise(function(resolve) {
            if (this._children) {
                this.children = this._children;
                this._children = null;
                resolve(this.children);
            }
            if (this.activity.HasChildren) {
                this.activity.getChildren()
                    .then(function(children) {
                        if (!this.children || this.children.length == 0) {
                            this.children = children.map(function(c) { return new ActivityObjectWrapper(c); });
                        }
                        this.updateLeftAndRight();
                        resolve(this.children);
                    }.bind(this));
            } else {
                resolve([]);
            }
        }.bind(this));
    }

    updateLeftAndRight() {
        if (!this.activity.Parent) {
            var i = 0, l = this.children.length;
            this.left = [];
            this.right = [];
            for(; i < l; i++){
                if (i % 2) {
                    this.left.push(this.children[i]);
                    this.children[i].position = 'left';
                } else {
                    this.right .push(this.children[i]);
                    this.children[i].position = 'right';
                }
            }
        }
    }
}