'use strict';

class ActivityObject extends odaObject {
    constructor (p, parent, data) {
        super(p);

        if(p){
            for(var i in p){
                this[i] = p[i];
            }
        }

        this._parent = parent;

        if (data) {
            this._data = data;
        }

        if (parent) {
            this._taskClass = parent.TaskClass;
        } else {
            var owner = this.$class, isAction = false;

            do {
                if (owner.id === '1D16BABB8061FA6') { //todo УБРАТЬ ID
                    isAction = true;
                }

                owner = owner.$owner;
            } while (owner && !isAction && owner.prefix === 'C');

            if (isAction) {
                this.$domain.getClass('1D199697E7BFA08').then(c => {
                    this._taskClass =  c;
                });
            }
        }
    }

    get ActivityClass() {
        if (this._activityClass) {
            return this._activityClass;
        }

        var cls = this.$class;

        while (cls.id != '1D16259B365BA98') {
            cls = cls.$owner;
        }

        this._activityClass = cls;

        return this._activityClass;
    }

    get Head() {
        return this.IsHead ? this : this.Parent.Head;
    }

    get AllItems() {
        var head = this.Head;

        if (!head._itemsMap) {
            head._itemsMap = new Map();
        }

        return head._itemsMap
    }

    get TaskClass() {
        return this._taskClass || this.$class;
    }

    get HasChildren() {
        var children = this.Root.$children;

        return children && children.length > 0 || this.Items && this.Items.length > 0;
    }

    get HasConditions() {
        var children = this.Root.$conditions;

        return children && children.length > 0;
    }

    get HasParent() {
        return !!(this.Parent || this.Root.$parent && this.Root.$parent);
    }

    get Conditions() {
        if (!this.HasConditions) {
            return;
        }

        return this.Root.$conditions.map(function(c) {
            var id = c.$activity[0].link;
            var index = id.lastIndexOf(':');

            return id.substring(index + 1);
        });
    }

    get Items() {
        return this._items;
    }

    get IsHead() {
        return !this.Parent;
    }

    get IsLeaf() {
        return !this.HasChildren;
    }

    get Parent() {
        return this._parent;
    }

    _appendLink(item, index) {
        if (!item) {
            return;
        }

        var children = this.Root.$children;

        if (!children) {
            children = this.Root.$children = [];
        }

        var data =  {
            activity : item.name,
            time : item.Root.Time,
            $activity: [
                {
                    link : this.getRelativeId(item),
                    name : item.name
                }
            ]
        };

        if (!index) {
            children.push(data);
        } else {
            children.splice(index, 0, data);
        }

        return data;
    }

    _removeLink(item) {
        if (!item) {
            return;
        }

        var children = this.Root.$children;
        var index = children.findIndex(function(c){ return c.$activity[0].link.includes(item.id); });

        if (index < 0) {
            return;
        }

        children.splice(index, 1);
    }

    appendChild(item) {
        return new Promise(resolve => {
            if (this.readOnly) {
                throw new Error(this);
            }

            if (item) {
                if (!this._items) {
                    this.getChildren()
                    .then(function() {
                        return this.appendChild(item)
                    }.bind(this))
                    .then(resolve);

                    this._items = this._items || [];
                }
                else {
                    var data = this._appendLink(item);
                    var activity = new ActivityObject(item, this, data);

                    activity.isNew = !(item instanceof ActivityObject);

                    this.AllItems.set(activity.id, activity);

                    var index = this.Items.push(activity);
                    this.fire('itemsChanged', [
                        {
                            addedCount : 1,
                            index: index - 1,
                            removed : [],
                            type: 'splice'
                        }
                    ]);

                    this.propertyChanged('State', this.State);
                    this.propertyChanged('Percent', this.Percent);
                    resolve(activity);
                }
            }
            else {
                this.TaskClass.createObject().then(this.appendChild.bind(this));
            }
        });
    }

    appendChildren(children) {
        return new Promise(resolve => {
            if (!children) {
                resolve();
            }

            var p = children.map(this.appendChild.bind(this));

            Promise.all(p).then(resolve);
        });
    }

    childOf(item) {
        if (item.id === this.id) {
            return false;
        }

        let parent = this.Parent;

        while (parent) {
            if (item.id === parent.id) {
                return true;
            }

            parent = parent.Parent;
        }

        return false;
    }

    getAllChildren() {
        return new Promise(function(resolve) {
            this.getChildren()
            .then(function(children) {
                if (children) {
                    var p = children.map(function(i) { return i.getAllChildren(); });
                    Promise.all(p).then(resolve);
                } else {
                    resolve();
                }
            });
        }.bind(this));
    }

    getChildren() {
        return new Promise((resolve, reject) =>{
            if (!this.HasChildren) {
                resolve();
                return;
            }

            if (this._items) {
                resolve(this._items);
                return;
            }

            var children = [];

            this.Root.$children.forEach(item => {
                var id = item.$activity[0].link;
                var index = id.lastIndexOf(':');

                id = id.substring(index + 1);
                id = this.ActivityClass.fullId + "/O:" + id;
                children.push(ODANT.findItem(id));
            });

            Promise.all(children).then(results => {
                results.forEach((r, index) =>{
                    if (!this._items) {
                        this._items = [];
                    }

                    var activity = new ActivityObject(r, this, this.Root.$children[index]);
                    this.AllItems.set(activity.id, activity);
                    this._items.push(activity);
                });
                resolve(this._items);
            })
            .catch(reject);
        });
    }

    getParentObject() {
        return new Promise(function(resolve) {
            if (!this.HasParent) {
                throw new Error('There is no parent');
            }

            ODANT.findItem(this.$domain.fullId + '/' + this.Root.$parent[0].link).then(resolve);
        }.bind(this))
    }

    moveTo(target) {
        return new Promise(function(resolve, reject) {
            if (this.readOnly) {
                throw new Error(this);
            }

            var index = this.Parent.Items.indexOf(this);

            if (index < 0) {
                throw new Error('Moving is failed');
            }

            this.Parent.Items.splice(index, 1);
            this.Parent._removeLink(this);

            this.Parent.fire('itemsChanged', [
                {
                    addedCount : 0,
                    index: index,
                    removed : [ this ],
                    type: 'splice'
                }]
            );

            target.appendChild(this, false)
            .then(function(newItem) {
                newItem.isNew = false;
                resolve({ item : newItem, removed: index });
            })
            .catch(reject);
        }.bind(this));
    }

    shift() {
        return new Promise(function(resolve) {
            if (this.readOnly) {
                throw new Error(this);
            }

            if (!this.Parent) {
                throw new Error('Parent is not defined');
            }

            this.Parent.removeChild(this);

            var copy = function(items) {
                var promises = items.map(function(i) { return this.Parent.appendChild(i); }.bind(this));

                Promise.all(promises).then(resolve);
            }.bind(this);

            if (this.Items) {
                copy(this.Items);
            } else {
                this.getChildren()
                .then(function() {
                    copy(this.Items);
                }.bind(this))
            }
        }.bind(this));
    }

    remove(withoutChildren) {
        return new Promise(function(resolve) {
            if (this.readOnly) {
                throw new Error(this);
            }

            if (!this.Parent) {
                resolve();
            }

            this.Parent.removeChild(this, withoutChildren).then(resolve);
        }.bind(this));
    }

    removeChild(child, withoutChildren) {
        return new Promise(function(resolve) {
            if (this.readOnly) {
                throw new Error(this);
            }

            var index = this.Items.indexOf(child);

            if (index < 0) {
                resolve();
            }

            this.Items.splice(index, 1);
            this._removeLink(child);

            this.AllItems.delete(child.id);

            this.fire('itemsChanged', [
                {
                    addedCount : 0,
                    index: index,
                    removed : [ child ],
                    type: 'splice'
                }]
            );

            this.propertyChanged('State', this.State);
            this.propertyChanged('Percent', this.Percent);

            if (!withoutChildren) {
                resolve(index);
            } else {
                if (child.Items) {
                    this.appendChildren(child.Items).then(resolve);
                } else {
                    child.getChildren().then(function(children) {
                        this.appendChildren(children).then(resolve);
                    }.bind(this));
                }
            }
        }.bind(this));
    }

    save() {
        if (this._parentObj) {
            this._parentObj.save();
        }

        var items = this.Items;

        if (items) {
            items.forEach(function(i) { i.save(); });
        }

        super.save();
    }

    ___appendChild() {
        return {
            access: 'C',
            icon: 'add',
            text: 'Append Child',
            mode : 'appendChild',
            method: this.appendChild.bind(this)
        }
    }

    ___complete() {
        return {
            access: 'W',
            icon: 'ok',
            text: 'Complete',
            mode : 'complete',
            method: this.complete.bind(this)
        }
    }

    ___start() {
        return {
            access: 'W',
            icon: 'ok',
            text: 'Start',
            mode : 'start',
            method: this.start.bind(this)
        }
    }

    ___abort() {
        return {
            access: 'W',
            icon: 'del',
            text: 'Abort',
            mode : 'abort',
            method: this.abort.bind(this)
        }
    }

    ___edit() {
        return {
            access : 'W',
            icon: 'edit',
            text : 'Edit',
            mode : 'edit',
            method : function () {

            }
        }
    }

    ___remove(){
        return {
            access: 'W',
            icon: 'del',
            text: 'Remove',
            mode : 'remove',
            method: this.remove.bind(this)
        }
    }

    get Methods() {
        return new Promise(function(resolve) {
            var checkAccess = (method) => {
                return this.access.allow(method.access);
            };

            var mas = [], m = this['___appendChild'];

            if (checkAccess(m)) {
                mas.push(this.___appendChild());
            }

            m = this['___edit'];

            if (checkAccess(m)) {
                mas.push(this.___edit());
            }

            m = this['___remove'];

            if (checkAccess(m) && this.Parent) {
                mas.push(this.___remove());
            }

            resolve(mas);
        }.bind(this));
    }

    get name() {
        return this._getProperty('name', 'New');
    }

    set name(value) {
        this._setProperty('name', value);

        if (this.isNew) {
            this.Root['name'] = value;
            this.isNew = false;
        }

        this.propertyChanged('Name');
    }

    get ReadOnly() {
        return this.State == 'Completed' || this.State == "Cancelled";
    }

    get State() {
        if (this.state) {
            return this.state;
        }

        if (this.IsHead) {
            return "Ready";
        }
        else {
            if (this.Parent.State == "Work") {
                return "Ready";
            }
        }
    }

    getStateActions() {
        var actions = [];

        switch (this.State) {
            case "Ready" : actions = [ this.___start() ]; break;
            case "CanComplete" :
            case "Work" :
                var canComplete = function(item) {
                    if (!item.HasChildren) {
                        return item.State == "Cancelled" || item.State == "Completed";
                    }

                    return item.Items.every(canComplete);
                };

                if (!this.HasChildren || canComplete(this)) {
                    actions.push(this.___complete());
                }

                actions.push(this.___abort());
                break;
        }

        return actions;
    }

    abort() {
        if (this.State != "Cancelled" && this.State != "Completed") {
            this.state = "Cancelled";
            this.propertyChanged("State", this.state);

            if (this.Items) {
                this.Items.forEach(function (item) {
                    item.abort();
                });
            }
        }
    }

    complete() {
        if (this.State != "Cancelled" && this.State != "Completed") {
            this.state = "Completed";
            this.propertyChanged("State", this.state);

            if (this.Parent) {
                this.Parent.computePercent(true);
            }
        }
    }

    start() {
        if (this.State == "Ready") {
            var changeState = function() {
                this.state = this.HasChildren ? "Work" : "CanComplete";
                this.propertyChanged("State", this.state);
                if (this.Items) {
                    this.Items.forEach(function(item) {
                        item.propertyChanged("State", item.State);
                    });
                }
            };

            if (this.HasChildren) {
                this.getAllChildren().then(function() {
                    var tree = document.createElement("activity-tree");
                    tree.checkable = true;
                    tree.items = this.Items;

                    return ODA.showDialog({ label : "Start activities", component : tree });
                }.bind(this))
                .then(function(result) {
                    changeState.call(this);
                    var checkedItems = result.ctrl.checkedItems;

                    if (Array.isArray(checkedItems)) {
                        checkedItems.forEach(function(item) {
                            changeState.call(item);
                        })
                    }
                }.bind(this));
            } else {
                changeState.call(this);
            }
        }
    }

    _getProperty(name, def, asNumber) {
        var v = this._data;

        if (v) {
            v = v[name];
        }

        if (asNumber) {
            v = +v;
        }

        if (!v || v == 0) {
            v = this.Root[name];
        }

        if (asNumber && v) {
            v = +v;
        }

        return v || def;
    }

    _setProperty(name, value) {
        if (this.readOnly) {
            return;
        }

        if (this._data) {
            this._data[name] = value;
        } else {
            this.Root[name] = value;
        }
    }

    propertyChanged(name, value) {
        if (!name) {
            return;
        }

        this.fire('propertyChanged', { property : name, value : value || this[name] });
    }

    showActionsMenu(pos) {
        return  this.Methods.then(methods => {
            return ODA.showContextMenu({ label: this.label, icon: this.icon, items: methods, pos: pos });
        })
    }

    showStateMenu(pos) {
        return new Promise(function(resolve) {
            var actions = this.getStateActions();

            if (!actions || actions.length == 0) {
                resolve();
            } else {
                if (actions.length == 1) {
                    var action = actions[0];

                    if (action.mode == "abort") {
                        ODA.confirm("Abort '" + this.name + "'?", "Activities").then(function() {
                            action.method();
                            resolve();
                        });
                    }

                    actions[0].method();
                    resolve();
                } else {
                    ODA.showContextMenu({label: this.name, icon: this.icon, items: actions, pos: pos })
                    .then(resolve);
                }
            }
        }.bind(this))
    }

    get ComputedTime() {
        if (this._computedTime) {
            return this._computedTime + (this._overload || 0);
        }

        return this.computeTime();
    }

    get Plan() {
        return this._getProperty('Time', 0, true);
    }

    get Percent() {
        if (this.IsLeaf) {
            return 0;
        }

        return this.computePercent();
    }

    get Start() {
        if (item._start) {
            return item._start;
        }

        return this.computeStart();
    }

    get Time() {
        return this._getProperty('Time', 0, true);
    }

    set Time(value) {
        this._setProperty('Time', value);
        this.propertyChanged('Time');
    }

    get TotalTime() {
        return Math.max(this.ComputedTime, this.Plan);
    }

    clear() {
        let properties = ['_computedTime', '_start', '_offset', '_downTime', '_lose', '_lag', '_overload'];

        properties.forEach(function(p) {
            this[p] = null;
        });

        let items = this.Items;

        if (Array.isArray(items)) {
            items.forEach(function(i) { i.clear(); });
        }
    }

    computePercent(notify) {
        var items = this.Items || this.Root.$children;

        var value = items.reduce(function(sum, i) {
            var time = +i.Time;

            if (i.State == 'Completed') {
                sum.time += time;
                sum.count++;
            }

            sum.totalTime += time;
            sum.totalCount++;

            return sum;
        }, { count : 0, time : 0, totalTime : 0, totalCount : 0});

        if (value.count == value.totalCount) {
            this._percent = 1;
            this.state = "CanComplete";
        } else {
            value = value.totalTime != 0 ? value.time / value.totalTime : 0;

            this._percent = value;
        }

        if (notify) {
            this.propertyChanged('Percent');
            this.propertyChanged('State');
        }

        return this._percent;
    }

    computeTime() {
        if (!this.Items) {
            this._computedTime = this.Plan;

            return this._computedTime;
        }

        var map = new Map();

        this.Items.forEach(function(item) {
            var conditions = item.Conditions;

            if (conditions) {
                conditions = conditions.map(function(x) { return x.id; }).sort().join();
            }

            var group = [];

            if (map.has(conditions)) {
                group = map.get(conditions);
            }

            group.push(item);

            map.set(conditions, group);
        });

        var time = 0;

        map.forEach(function(value) {
            time += value.reduce(function(a, b) {
                return Math.max(a, b.TotalTime);
            }, 0);
        });

        this._computedTime = time;

        return this._computedTime;
    }

    computeStart() {
        if (!this.Parent) {
            this._start = 0;

            return this._start;
        }

        var conditions = item.Conditions;

        if (!conditions) {
            this._start = this.Parent.Start;
        } else {
            this._start = conditions.reduce(function(sum, i) {
                return Math.max(sum, i.Start + i.TotalTime);
            }, 0);
        }

        return this._start;
    }
}