(class extends odaService {

    static get properties() {
        return {
            allowAcces: 'R',
            allowUse: true,
            isActive: true,
            label: 'Filling',
            subMenu: async function() {
                if (!this.$owner || !this.$owner.Root
                    || !this.$owner.Root.$SERVICES || this.$owner.Root.$SERVICES.length <= 0
                    || !this.$owner.Root.$SERVICES[0].$BIND || this.$owner.Root.$SERVICES[0].$BIND.length <= 0
                    || !this.$owner.Root.$SERVICES[0].$BIND[0].$DOC || this.$owner.Root.$SERVICES[0].$BIND[0].$DOC.length <= 0)
                    return [];

                const promArr = this.$owner.Root.$SERVICES[0].$BIND[0].$DOC.map(async (doc) => {
                    try {
                        const obj = await this.$owner.findItem(doc.$link[0].link);
                        return { 'label': obj.name, 'execute': (e) => obj.show() };
                    } catch (e) {
                        return undefined;
                    }
                });
                const objArr = await Promise.all(promArr);
                return objArr.filter(e => e);
            }
        }
    }

    async execute(params) {
        const url = this.resolveUrl('template-selector/fill-template-selector.html');
        const selector = await ODA.createComponent(url);
        selector.contextItem = this.$owner;

        const dialog = {
            title: 'Select template',
            icon: 'image:center-focus-strong',
            dblclick: true,
            ok: {
                show: true,
                disable: false
            },
            buttons: [
                { label: 'New', icon: 'icons:add', special: 'add' },
                { label: 'Remove', icon: 'icons:remove', special: 'del' },
                { label: 'Settings', icon: 'icons:settings', special: 'set' },
                { label: 'OK', special: 'ok' }
            ]
        };
        const dialogResult = await ODA.showDialog(selector, dialog);
        if (!dialogResult.button) {
            if (!dialogResult.focusedItem || !dialogResult.focusedItem.item)
                return;

            const bindResult = await ODANT.API.post(dialogResult.focusedItem.item.url,
                { execute: 'methods/auto-fill', restart: 'true' },
                "'" + this.$owner.fullId + "'");

            if (!dialogResult.focusedItem.item.Root.HiddenBind
                || (dialogResult.focusedItem.item.Root.HiddenBind.toLowerCase() !== "true")) {

                if (Array.isArray(bindResult)) {
                    const promises = bindResult.map((url) => this.$owner.$class.findItem(url));
                    return Promise.all(promises).then((bindObjs) => bindObjs.map((obj) => obj.show()));
                }

                return this.$owner.$class.findItem(bindResult).then(bindObj => bindObj.show());
            }
        } else {
            switch (dialogResult.button.special) {
                case 'add':
                    const fillClsPath = "M:1CD85C475AC7F6F/C:143574104309355";
                    const fillCls = await this.$owner.$base.findItem(fillClsPath);
                    const newTmpObj = await fillCls.createObject();

                    if (newTmpObj.Root) {
                        const component = await ODA.createComponent('/web/lib/dialogs/select-class/select-class.html',
                            { contextItem: this.$owner.$base });
                        const result = await ODA.showDialog(component, { title: 'Select destination Class', icon: 'odant:class' });
                        if (result && result.item) {
                            newTmpObj.Root.Bind = `${this.$owner.$class.label} -> ${result.item.label}`;
                            newTmpObj.Root.$Bind = [];
                            const bind = {
                                SrcClass: fillCls.getRelativeId(this.$owner.$class),
                                DstClass: fillCls.getRelativeId(result.item),
                                Relative: true
                            };
                            newTmpObj.Root.$Bind.push(bind);

                            newTmpObj.show();
                        }
                    }
                    break;
                case 'del':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        if (await ODA.confirm('Are you sure?')) {
                            dialogResult.control.focusedItem.item.remove();
                        }
                    }
                    break;
                case 'set':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        dialogResult.control.focusedItem.item.show();
                    }
                    break;
                case 'ok':
                    if (!dialogResult.control.focusedItem || !dialogResult.control.focusedItem.item)
                        return;

                    return dialogResult.control.focusedItem.item.execute('run-fill', { srcObj: this.$owner.fullId });
                default:
                    alert('Я таких значений не знаю');
            }
        }
    }
}).register();