const Module = require('module');
const path = require('path');

(class extends odaAction {
    execute() {
        if (this.bot) {
            delete this.bot;
            this.bot = null;
        }

        this.bot = new Bot(this.$item);
        ODANT.BotInterface = this.bot;

        return this.bot.init()
            .then(() => `Bot started: ${new Date().toLocaleString()} (${this.$item.fullId})`)
    }
}).register();


class Bot {
    constructor(cls) {
        this.$class = cls;
        this.Sessions = [];
        slugify.config({separator: '_'});

        if (typeof EXTERNALORIGIN === 'undefined') {
            global.EXTERNALORIGIN = 'https://dev.botincubator.ru';
        }

        this.Platforms
            .then(platforms => {
                platforms.forEach(p => p.Message = this.messageHandler.bind(this));
            })
    }

    get rootContext() {
        return this.$class.$base;
    }

    get Platforms() {
        return (this._platforms)
            ? Promise.resolve(this._platforms)
            : this.$class.$domain.getClass('1D290FE398AA470').then(platforms =>
                platforms.Classes.then(classes => {
                    const proms = classes
                        .map(i => i.executeAction('init')
                            .then(platform => platform)
                            .catch(() => null));
                    return Promise.all(proms).then(res => this._platforms = res.filter(i => i))
                })
            )
    }

    init() {
        return Promise.all([
                this._initBotGuestSolution(),
                this._initClsBotUsers()
            ]
        )
    }

    _initBotGuestSolution() {
        return this.$class.$base.Solutions
            .then(solutions => {
                const guestSolution = solutions
                    .find(solution => solution.id === '1D330750F66A380');

                return (guestSolution)
                    ? this._setBotGuestSolution(guestSolution)
                    : this._createBotGuestSolution()
                        .then(this._setBotGuestSolution.bind(this))
            });
    }

    _setBotGuestSolution(guestSolution) {
        this.GuestSolution = guestSolution;
    }

    _createBotGuestSolution() {
        return this.$class.$base.API.POST(
            'create_domain',
            {},
            `{"$CLASS":[{"Name":"Bot guest","ClassId":"1D330750F66A380","Label":"Bot guest","Type":"SOLUTION"}]}`
        )
            .then(() => this.$class.$base.findItem('S:1D330750F66A380'))
    }

    _initClsBotUsers() {
        return this._getClsBotUsers()
            .then(clsUsers => (clsUsers)
                ? this._setClsBotUsers(clsUsers)
                : this._createClassBotUsers()
                    .then(this._setClsBotUsers.bind(this))
            );
    }

    _getClsBotUsers() {
        return new Promise((resolve, reject) =>
            this.$class.$base.findItem('S:000000000000000/C:000000000000210/C:1D2FA412CBB8A00')
                .then(resolve)
                .catch(() => resolve())
        );
    }

    _setClsBotUsers(users) {
        this.Users = users;
    }


    _createClassBotUsers() {
        return this.$class.$base.findItem('S:000000000000000/C:000000000000210')
            .then(users => users.API
                .POST(
                    'create_class',
                    {},
                    `{"$CLASS":[{"Author":"SYS","ClassId":"1D2FA412CBB8A00","Name":"Bot-users","Label":"bot-users","Parent":"000000000000210","Update":"2017-07-11T14:25:53", "i":"True","$SF":[{"i":"True"}],"$METADATA":[{"i":"True","$ATTR":[{"Name":"FirstName","Label":"FirstName","List":"False","i":"True"},{"Name":"LastName","Label":"LastName","List":"False","i":"True"} ,{"Name":"phone","List":"False","i":"True"},{"Name":"email","List":"False","i":"True"},{"Name":"uid","List":"False","i":"True"},{"Name":"card","Label":"card","List":"False","i":"True"},{"Name":"role","Label":"role","List":"False","i":"True"},{"Name":"platforms","List":"True","i":"True","$ATTR":[{"Name":"platform","Label":"Name","List":"False","i":"True"}]}]}]}]}`
                )
                .then(res => users.getClass(res.$C[0].i))
            )
    }

    messageHandler(from, msg) {
        this.getSession(from)
            .then(session => session.command(msg))
    }

    getSession(from) {
        return this._getUserId(from)
            .then(xqResult => {
                const oid = xqResult.$result[0].oid;

                if (oid) {
                    const foundSession = this.Sessions.find(session => session.User.id === oid);

                    if (foundSession) {
                        return foundSession
                    } else {
                        return this.Users.getObject(oid)
                            .then(user => this._initSession(new Session(this, user))
                                .then(session => {
                                    this.Sessions.push(session);
                                    return session;
                                })
                            );
                    }
                } else {
                    return this._createUser(from)
                        .then(user => this._initSession(new Session(this, user))
                            .then(session => {
                                this.Sessions.push(session);
                                return session;
                            })
                        )
                }
            })
    }

    _getUserId(from) {
        const xq = `
                let $oids :=//OBJECT[platforms[@platform="${from.platform}" and @uid = "${from.id}"]]/@oid
    
                return element result{
                    attribute oid {if ($oids[1]) then (string($oids[1])) else ('')}
                }
            `;

        return this.Users.getIndex('pack')
            .then(idx => idx.POST('xquery_index', {async: false}, xq))
    }

    _createUser(from) {
        return this.Users.createObject()
            .then(obj => {
                const platformRow = obj.addRow('platforms');
                const rolesRow = obj.addRow('roles');

                return Promise.all([
                    obj.setValue('FirstName', from.firstName),
                    obj.setValue('LastName', from.lastName),
                    obj.setValue('platform', from.platform, platformRow),
                    obj.setValue('role', this.GuestSolution.id, rolesRow),
                    obj.setValue('name', this._generateUserName(from)),
                    obj.setValue('uid', from.id, platformRow),
                ])
                    .then(() => obj.save()
                        .then(() => obj))
            })
    }

    _generateUserName(user) {
        let name = user.firstName || '';

        name = (name.length > 0)
            ? `${name} ${user.lastName || ''}`
            : user.lastName || '';

        return name;
    }

    notifyUsers(param) {
        console.log(param.roles);

        if (param.roles) {
            return this._getUsersIdForRoles(param.roles)
                .then(this._getSessions.bind(this))
                .then(sessions => this._sendNotifyMessage(sessions, param))
        } else {
            return this._getSessions(param.uids)
                .then(sessions => this._sendNotifyMessage(sessions, param))
        }
    }

    _sendNotifyMessage(sessions, param) {
        if (param.item) {
            return Promise.all(sessions
                .map(session => session.SetCurrentContext(param.item))
            )
                .then(() => {
                    return Promise.all(sessions.map(session => session.Message
                        .then(res => session.platform.send(res, session.CurrentPlatformUser))
                    ))
                })
        } else {
            return Promise.all(sessions
                .map(session => session.platform.send({
                    text: param.msg
                }, session.CurrentPlatformUser))
            )
        }
    }

    _getUsersIdForRoles(rolesName) {
        return this.$class.$base.Solutions
            .then(workPlaces => {
                const wpIDs = workPlaces
                    .filter(wp => rolesName.some(roleName => roleName === wp.name))
                    .map(wp => `'${wp.id}'`)
                    .join(',');

                const xq = ` 
                                <x>
                                    { 
                                        for $oid in (oda:select(//PACK/OBJECT/roles, 'role', (${wpIDs}), '')/../@oid) 
                                        return <o oid="{$oid}"/>
                                    } 
                                </x>
                            `;

                return this.Users.XQuery(xq, {index: 'Pack'})
                    .then(res => res && res.$x && res.$x[0].$o && res.$x[0].$o.map(u => u.oid) || []);
            })
    }

    _getSessions(uids) {
        console.log(`uids = ${uids}`);

        if (uids) {
            return this.Users.getObjects()
                .then(users => Promise.all(
                    users
                        .filter(user => uids.some(uid => uid === user.id))
                        .reduce((p, user) => {
                            let foundSession = this.Sessions.find(session => session.User === user);
                            if (foundSession) {
                                p.push(Promise.resolve(foundSession))
                            } else {
                                p.push(this._initSession(new Session(this, user))
                                    .then(session => {
                                        this.Sessions.push(session);
                                        return session;
                                    }))
                            }
                            return p;
                        }, []))
                )
        } else {
            return this.Users.getObjects()
                .then(users => Promise.all(
                    users
                        .reduce((p, user) => {
                            if (!this.Sessions.some(session => session.User === user)) {
                                p.push(this._initSession(new Session(this, user))
                                    .then(session => this.Sessions.push(session)))
                            }
                            return p;
                        }, []))
                    .then(() => this.Sessions)
                )
        }
    }

    _initSession(session) {
        return this._setSessionCurrentPlatform(session)
            .then(this._setUserId);
    }

    _setSessionCurrentPlatform(session) {
        return Bot._sessionCurrentPlatformName()
            .then(platformName =>
                this.Platforms
                    .then(platforms => {
                        const platform = platforms.find(platform => platform.__proto__.constructor.name === platformName);

                        if (platform) {
                            session.platform = platform;
                        }
                        return session;
                    })
            )
    }

    _setUserId(session) {
        session.CurrentPlatformUser = {
            id: session.User.Root.$platforms.find(platform => platform.platform === session.platform.__proto__.constructor.name).uid
        };

        return session;
    }

    static _sessionCurrentPlatformName() {
        //TODO get current platform
        return Promise.resolve('Telegram');
    }

}

class Session {
    constructor(bot, user) {
        this.Bot = bot;
        this.User = user;
        this.LimitShowObject = 10;
        this.SearchPhrase = '';
    }

    get currentContext() {
        return this._current || this.Bot.rootContext;
    }

    _userHasPhone() {
        return this.User.fields
            .then(fields => fields
                .find(f => f.name === 'phone')
                .getValue(this.User)
            )
    }


    _userHasFIO() {
        return this.User.fields
            .then(fields => fields
                .find(f => f.name === 'name')
                .getValue(this.User)
            )
    }

    _initMiddleware() {
        this._middleware = {
            after: [],
            before: []
        };
    }

    _handlingEvent(eventName) {
        return this.currentContext.getActions('bot-events').then(events => {
            const event = events.find(e => e.name.toLowerCase() === eventName);
            return (event)
                ? event.execute.call(event, this).then(res => res)
                : Promise.resolve(true)
        })
    }


    static _fieldValue(param) {
        return new Promise((resolve, reject) => {
            return Promise.resolve(param['field'].getValue(param['obj'], param['root']))
                .then(res => resolve(res))
                .catch(err => resolve());
        })
    }


    _syncExecutePromises(promises) {
        return new Promise((resolve) => {
            if (Array.isArray(promises)) {
                promises.push(() => new Promise(() => {
                    promises.pop();
                    resolve()
                }));

                promises.reduce(
                    (p, i) => p.then(
                        () => new Promise((resolve, reject) => i().then(() => resolve()))
                    ),
                    Promise.resolve()
                );
            } else {
                resolve(true);
            }
        });
    }

    SetCurrentContext(contextItem) {
        if (this.currentContext) {
            return this._handlingEvent('on-exit')
                .then(res => {
                    if (res) {
                        this._clearTemporaryData(contextItem);
                        this.currentContext = contextItem;

                        return this._loadSettings()
                            .then(() => this._processingSettings('read'))
                            .then(() => this._executeTasks()
                                .then(() => this._handlingEvent('on-enter')
                                    .then(() => {
                                        if (this.currentContext.prefix === 'C') {
                                            return this._objectCount(this.currentContext, this._mask)
                                                .then(count => this.CountObjects = count)
                                        }
                                    })
                                    .then(() => Promise.resolve(true))
                                ))
                    } else {
                        return Promise.resolve(false);
                    }
                })
        } else {
            this.currentContext = contextItem;
            return this._handlingEvent('on-enter')
                .then(() => Promise.resolve(true));
        }
    }

    createObject(item){
        if (item){
            return this.SetCurrentContext(item)
                .then((res) => {
                    this._createObject();
                    return false;
                })
        } else {
            return this._createObject();
        }
    }

    _processingSettings(mode) {
        return this._processingSettingsTimeOut(mode)
            .then(() => this._processingSettingsRecalc())
            .then((recalc) => this._processingSettingsSave(recalc))
    }

    _processingSettingsTimeOut() {
        return new Promise((resolve, reject) => {
            const timeoutValue = CommonUtils.findPropertyValue(this._settings, `timeout.mode.${this._objectMode}.value`, 300);

            if (typeof timeoutValue === 'function') {
                return Promise.resolve(timeoutValue.call(this.currentContext))
                    .then(resolve);
            }

            setTimeout(resolve, timeoutValue);
        });
    }

    _processingSettingsRecalc(item = this.currentContext) {
        return CommonUtils.getValueFromProperty(
            CommonUtils.findPropertyValue(this._settings, `recalc.mode.${this._objectMode}`, true),
            {
                context: item,
                params: {session: this},
            })
            .then((res) => {
                if (res) {
                    if (item.prefix === 'O') {
                        return item.API.POST('recalc', {}, JSON.stringify(item.body))
                            .then((result) => {
                                return item._updateRoot(result).then(() => {
                                    item.reset();
                                    item.fireChanged();
                                    return res;
                                })
                            })
                    }
                }

                return (res);
            });
    }

    _processingSettingsSave(item = this.currentContext, recalc) {
        if (recalc) {
            return item.save();
        }
    }

    _executeTasks() {
        return this.currentContext.getActions('bot-tasks')
            .then(events => {
                if (events.length > 0) {
                    return Promise.all(events.map(e => e.execute.call(e, this)
                    ))
                } else {
                    return Promise.resolve();
                }
            }).catch((err) => {
                console.log(err);
            })
    }

    set currentContext(v) {
        this._initMiddleware();
        this._settings = undefined;
        this.handler = undefined;

        if (v.prefix !== 'O') {
            this.SearchPhrase = '';
        }

        if (v.prefix === 'B'
            || v.prefix === 'S'
            || (this._current && this._current.prefix === 'C' && v.prefix === 'C')
            || (this._current && this._current.prefix === 'O' && v.prefix === 'O' && this._current.$class.fullId !== v.$class.fullId)) {
            this._mask = undefined;
        }

        this._updatedObjects = true;
        this._current = v;
    }

    _setUserPhone(phone) {
        return this.User.$class.fields.then(fields => {
            return fields.find(f => f.name === 'phone').setValue(this.User, phone)
                .then(() => this.User.save());
        })
    }


    contact(contact) {
        this.User.$class.fields.then(fields => {
            for (let prop in contact) {
                let foundField = fields.find(f => f.name === prop);

                if (foundField) {
                    this.User.setValue(foundField, contact[prop]);
                }
            }

            return this.User.save()
                .then(() =>
                    (this.handler)
                        ? this.handler()
                        : this.Message
                            .then(res => this.platform.send(res, this.CurrentPlatformUser)))
        })
    }

    command(msg) {
        if (msg.value === '/start') {
            this._structure = null;
            return this.defaultHandler(msg);

        }

        return this._handleMsgType(msg)
            .then(this._handleMsg.bind(this))
    }

    _handleMsgType(msg) {
        if (msg.type === 'phone') {
            return this._setUserPhone(msg.value)
                .then(() => msg)
        } else {
            return Promise.resolve(msg)
        }

    }

    _handleMsg(msg) {
        if (this.handler) {
            const item = this._findItemInStructure({text: msg.value, onlyCommand: true});

            if (item && item.run) {
                const run = item.run;

                return Promise.resolve(run.call(this, {item, session: this, msg}))
                    .then(res => {
                        if (res) {
                            return this.Message
                                .then(() => this.platform.send(this._structure, this.CurrentPlatformUser));
                        }
                    })
            } else {
                return this.handler(msg.value, msg.type);
            }

        } else {
            return this.defaultHandler(msg);
        }
    }

    _findItemInStructure(param) {
        if (!param.text || !this._structure) {
            return;
        }

        let item;
        let groupItems;
        let searchValue = param.text.trim();

        for (let groupName in this._structure.groups) {
            if (item) {
                break;
            }

            if (param.onlyCommand && groupName !== 'commands') {
                continue;
            }

            groupItems = this._treeToList(this._structure.groups[groupName]);

            item = item || groupItems && groupItems.find(i => (i.command && i.command.trim() === searchValue));
            item = item || groupItems && groupItems.find(i => (i.label && i.label.trim() === searchValue));
        }

        return item;
    }

    _treeToList(tree) {
        const items = [];
        const stack = [];

        stack.unshift(...tree.items);

        while (stack.length !== 0) {
            let node = stack.shift();

            if (node.items) {
                stack.unshift(...[].concat(...node.items));
            }

            items.push(node);
        }

        return items;
    }

    defaultHandler(msg) {
        return this.init()
            .then(() => {
                let item = this._findItemInStructure({text: msg.value});

                if (item && item.run) {
                    const run = item.run;

                    return Promise.resolve(run.call(this, {item, session: this, msg}))
                        .then(res => {
                            if (!res) {
                                return;
                            }
                            return this.Message
                                .then(() => this.platform.send(this._structure, this.CurrentPlatformUser));
                        })
                } else {
                    return (this._structure)
                        ? this.platform.send(this._structure, this.CurrentPlatformUser)
                        : this.Message
                            .then(res => this.platform.send(res, this.CurrentPlatformUser))
                }

            })
    }

    _loadSettings(item = this.currentContext) {
        return this._getItemSettings(item)
            .then(settings => this._settings = settings);
    }

    init() {
        if (this._structure) {
            return Promise.resolve(this._structure);
        }

        return this.SetCurrentContext(this.Bot.rootContext)
            .then(() => {
                return this.Message
                    .then(() => this._structure)
            });
    }

    get Message() {
        return Promise.all([
            this.Structure,
            this.Objects,
            this.Methods,
            this.Commands,
            this.fields,
            this.Packets,
            this.Custom
        ])
            .then(this._mergeGroups)
            .then(this._restoreGroupMsgId.bind(this))
            .then(this._buildStructure.bind(this))
    }

    get Custom() {
        return (this._customSource)
            ? this._generateCustomGroup()
            : Promise.resolve();
    }

    _generateCustomGroup() {
        let promises = [
            Array.isArray(this._customSource.structure)
                ? Promise.resolve(this._customSource.structure)
                : Promise.resolve(this._customSource.structure())
            ,
            (typeof this._customSource.label === 'function')
                ? Promise.resolve(this._customSource.label())
                : Promise.resolve(this._customSource.label)
        ];

        return Promise.all(promises)
            .then(res => {
                const cg = {
                    name: 'custom',
                    text: res[1],
                    items: res[0],
                    param: this._customSource.param
                };

                return cg;
            });
    }

    _mergeGroups(groups) {
        return groups
            .reduce((res, gr) => {
                if (gr) {
                    res[gr.name] = gr;
                }

                return res
            }, {})
    }

    _restoreGroupMsgId(groups) {
        if (this._structure) {
            for (let prop in groups) {
                let foundGroup = this._structure.groups[prop];

                if (foundGroup && foundGroup.msgId) {
                    groups[prop].msgId = foundGroup.msgId;
                }
            }
        }

        return groups;
    }

    _buildStructure(groups) {
        return Promise.all([
            this.Text,
            this.icon,
        ])
            .then(res => {
                this._structure = {
                    text: res[0],
                    image: res[1],
                    groups
                };
                this._updatedObjects = false;

                return this._syncExecutePromises(this._middleware['after'])
                    .then(res => this._structure);
            })
    }

    get Structure() {
        if (this.currentContext.prefix === 'O') {
            return Promise.resolve();
        }

        if (!this.currentContext.config.$C || this.currentContext.config.$C.length === 0) {
            return Promise.resolve();
        }

        return this._availableLinks()
            .then(() => this._availableItems())
            .then((items) => this._buildStructureItemsJSON(items));
    }

    _availableLinks() {
        return this.currentContext.$base.Solutions
            .then(solutions => {
                const rolesId = this.User.Root.$roles
                    .map(role => role.role);

                const userSolutionsLinks = solutions
                    .filter(solution => rolesId.includes(solution.id))
                    .map(userSolution => Promise.all(userSolution.Links));


                return Promise.all(userSolutionsLinks)
                    .then(res => {
                        this._links = []
                            .concat(...res)
                            .filter(link => {
                                return link.sourceItem && link.sourceItem.prefix === 'C'
                            })
                            .sort((a, b) => (a.label < b.label) ? -1 : 1)
                            .reduce((p, n) => {
                                let linkIdx = p.findIndex(link => link.id === n.id);
                                if (linkIdx === -1) {
                                    p.push(n);
                                } else if (linkIdx > -1 && p[linkIdx].Root['Access'] < n.Root['Access']) {
                                    p[linkIdx] = n
                                }
                                return p;
                            }, [])
                    })
            });
    }

    _availableItems() {
        return this.currentContext.Classes
            .then(classes => {
                return classes.filter(item => {
                    return this._links.some(link => {
                        if (link.Root.Hide) {
                            return;
                        }

                        let _item = item;
                        let result = false;

                        while (_item.prefix !== 'B' && _item.$owner) {
                            if (link.sourceItem.fullId === _item.fullId) {
                                result = true;
                                break;
                            }

                            _item = _item.$owner;
                        }

                        return result;
                    });
                });
            })
    }

    _buildStructureItemsJSON(items) {
        return Promise
            .all(items.map((item, idx) => this._buildStructureItemJSON({
                item,
                idx,
            })))
            .then((structureItemJSONs) => {
                    return this.Text
                        .then(text => {
                                return ({
                                    name: 'structure',
                                    items: structureItemJSONs,
                                    text: text,
                                    updated: true
                                })
                            }
                        )
                }
            )
            .catch(err => console.log(err))
    }

    _buildStructureItemJSON(param) {
        const item = param.item;
        return this._getItemSettings(item)
            .then(settings => {
                return this._getLabelForStructure({
                    item,
                    settings
                })
                    .then(label => {
                        return {
                            icon: (item.hasChildren) ? '+' : null,
                            label: label,
                            id: item.id,
                            command: `/str${param.idx}`,
                            run: this._structureItemHandler.bind(this)
                        }
                    });
            })
    }

    //TODO TT
    _getLabelForStructure(param) {
        const settings = param.settings;
        const item = param.item;

        return this._objectCount(item)
            .then(objectCounts => {
                if (settings && settings.label) {
                    return CommonUtils.getValueFromProperty(
                        settings.label, {
                            context: item,
                            params: {
                                session: this
                            }
                        });
                }

                if (settings && settings.hideCountObjects) {
                    return item.label;
                }

                return Promise.resolve(`${item.label} ${objectCounts > 0 ? `[${objectCounts}]` : ''}`)
            });
    }

    _getItemSettings(item) {
        return item.getFile('settings/bot.config.js').then((file) => file.load())
            .then(txt => {
                return CommonUtils.requireFromString(txt);
            })
            .catch((err) => {
                console.log(err);
                return null;
            })
    }


    _structureItemHandler({item}) {
        return this.currentContext.findItem(`C:${item.id}`)
            .then((odaItem) => this.SetCurrentContext(odaItem))
    }

    get Packets() {
        if (this.currentContext.prefix === 'C' || this.currentContext.prefix === 'O') {
            const classContext = (this.currentContext.prefix === 'O')
                ? this.currentContext.$class
                : this.currentContext;

            return classContext.Items
                .then(items => {
                        if (items.length === 0
                            && (classContext.Root.$Pack && classContext.Root.$Pack.length > 0)
                            && ((this._mask && this._mask.length < classContext.Root.$Pack.length) || !this._mask)) {

                            return classContext.API.GET('get_pack_list',
                                {
                                    'by_user': this.accessLevelForLink() === 6 ? '*' : this.User.id
                                }).then(json => {
                                let packets = json.$FOLDER[0].$FILE;

                                if (packets) {
                                    this._mask = this._mask || [];

                                    if (this._mask.length > 0) {
                                        let packFilter = this._mask
                                            .reduce((a, c) => a += `${c}#`, '');

                                        packets = packets
                                            .filter(pack => pack.name.startsWith(packFilter));
                                    }

                                    const currentGroupPackets = packets
                                        .reduce(
                                            (prev, current) => {
                                                let packGroup = current.name.split('#')[this._mask && this._mask.length || 0];

                                                if (!prev.includes(packGroup)) {
                                                    prev.push(current.name.split('#')[this._mask && this._mask.length || 0])
                                                }

                                                return prev;
                                            }, []
                                        );

                                    return Promise.all(
                                        currentGroupPackets
                                            .map(packet =>
                                                this._objectCount(this.currentContext, this._mask.concat([packet]))
                                            )
                                    )
                                        .then(objectCounts => {
                                            const items = currentGroupPackets
                                                .map((packet, idx) => ({
                                                        label: `${packet} ${objectCounts[idx] > 0 ? `[${objectCounts[idx]}]` : ''}`,
                                                        id: packet,
                                                        command: `/_${slugify(packet)}`,
                                                        run: (param) => Promise.resolve(this._mask.push(param.id))
                                                    })
                                                );

                                            return ({
                                                name: 'packets',
                                                items: items,
                                                text: this.currentContext.label,
                                                updated: true
                                            });
                                        });
                                } else {
                                    return {
                                        name: 'packets',
                                        items: [],
                                        text: this.currentContext.label,
                                        updated: true
                                    }
                                }
                            });
                        } else {
                            return Promise.resolve();
                        }
                    }
                )
        }
        else {
            return Promise.resolve();
        }
    }

    get Objects() {
        if (this.currentContext.prefix === 'C' || this.currentContext.prefix === 'O') {
            const classContext = this.currentContext.prefix === 'O' ? this.currentContext.$class : this.currentContext;

            return classContext.Items
                .then(items => {
                    if ((this.SearchPhrase && this.SearchPhrase.length > 0)
                        || items.length === 0
                        && (!classContext.Root.$Pack)
                        || (classContext.Root.$Pack && this._mask && classContext.Root.$Pack.length === this._mask.length)) {
                        return classContext.getIndex('botgrid', 'element x { for $o in //OBJECT order by $o/@date descending return element O { $o/@oid, $o/@name, $o/@date, $o/@Author }}')
                            .then(index => {
                                const qbyUser = this._byUser(classContext);

                                const xq = this.SearchPhrase.length > 0
                                    ? `
                                            element x {
                                                attribute cnt { count(x/O[${(qbyUser === '') ? qbyUser : `${qbyUser} and`} contains(lower-case(@name), lower-case("${this.SearchPhrase}"))]) }, 
                                                for $O in subsequence(//x/O[${(qbyUser === '') ? qbyUser : `${qbyUser} and`} contains(lower-case(@name), lower-case("${this.SearchPhrase}"))], 1, ${this.CountObjects}) 
                                                return $O 
                                             }
                                      `
                                    : `
                                            element x { 
                                                for $O in subsequence(//x/O${(qbyUser === '') ? qbyUser : `[${qbyUser}]`}, ${this.currentShowObject - this.LimitShowObject + 1}, ${this.currentShowObject}) 
                                                return $O
                                            }
                                      `;

                                let mask = '*';

                                if (this._mask) {
                                    mask = this._mask
                                        .reduce((a, c) => a += `${c}/`, '');

                                    mask = mask
                                        .slice(0, mask.length - 1);

                                    this._updatedObjects = true;
                                }

                                // const byuser = this.accessLevelForLink() === 6 ? '*' : this.User.Id;

                                return index.POST('xquery_index', {
                                    loadmask: mask,
                                    // by_user: byuser,
                                    async: false,
                                }, xq)
                                    .then(res => {
                                        let items = [];

                                        if (res && res.$x[0] && res.$x[0].$O) {
                                            items = res.$x[0].$O
                                                .map((item, i) => ({
                                                        label: item.name || item.oid,
                                                        id: item.oid,
                                                        command: `/obj${i}`,
                                                        n: i,
                                                        run: () =>
                                                            classContext.getObject(item.oid)
                                                                .then(obj => this.SetCurrentContext(obj)
                                                                    .then(() => {
                                                                        this._updatedObjects = false;
                                                                        return obj;
                                                                    })
                                                                )
                                                    }
                                                ))
                                        }

                                        return this.Text
                                            .then((text) => {
                                                return ({
                                                    name: 'objects',
                                                    items,
                                                    text,
                                                    updated: this._updatedObjects
                                                });
                                            })


                                    })
                                    .catch(err => {
                                        console.log(err);
                                    })

                            })
                            .catch(err => {
                                console.log(err);
                            })

                    } else {
                        return Promise.resolve();
                    }
                })

        } else {
            return Promise.resolve();
        }
    }

    UseAfter(fn) {
        if (this._validateMiddleware(fn)) {
            this._middleware['after'].push(fn);
        }
    }

    UseBefore(fn) {
        if (this._validateMiddleware(fn)) {
            this._middleware['before'].push(fn);
        }
    }

    _byUser(item) {
        if (!item.Root.ByUser) {
            return ''
        }

        return parseInt(this.accessLevelForLink(item)) === 6 ? '' : `@Author=\'${this.User.id}\'`
    }

    _validateMiddleware(fn) {
        if (!fn) {
            throw new Error('abyss!');
        }

        return true;
    }

    accessLevelForLink(item) {
        if (!item && this.currentContext.prefix == 'B') {
            return 0;
        }

        let _owner = item || this.currentContext;

        while (_owner.$owner.prefix !== 'B' && _owner.$owner.prefix !== 'M') {
            _owner = _owner.$owner;
        }

        const linkSource = this._links && this._links
            .find(link => link.sourceItem === _owner);

        return CommonUtils.findPropertyValue(linkSource, 'Root.Access', 0);
    }

    get Methods() {
        let methods = [];

        if (this.currentContext.prefix === 'C'
            && !this.currentContext.hasChildren
            && this.accessLevelForLink() >= 4
            && this._getVisibilityInlineMethod('create')) {
            methods.push(
                {
                    label: this._getLabelForInlineMethod('create', 'Создать'),
                    id: 'Create object',
                    command: '/create-object',
                    run: this._createObject.bind(this)
                });
        } else if (this.currentContext.prefix === 'O'
            && this.accessLevelForLink() >= 3
            && this._getVisibilityInlineMethod('edit')) {
            methods.push(
                {
                    label: this._getLabelForInlineMethod('edit', 'Изменить'),
                    id: 'Edit object',
                    command: '/edit-object',
                    run: this.editObject.bind(this)
                });
        }

        return this.currentContext.getActions('bot-methods')
            .then(botMethods => {
                return this._visibleMethods(botMethods)
                    .then(visibleMethods => {
                        visibleMethods.forEach(m => {
                            methods.push({
                                label: m.label,
                                icon: m.icon,
                                id: m.name,
                                method: m,
                                command: '/exec_' + m.name,
                                run: () => m.execute.call(m, this)
                            });
                        });

                        const labelGroup = CommonUtils.findPropertyValue(this._settings, 'methods.label', 'Доступные действия');

                        return Promise.resolve((methods.length > 0) ? {
                            name: 'methods',
                            items: methods,
                            text: labelGroup
                        } : undefined);

                    });
            })
    }

    _getLabelForInlineMethod(methodName, label) {
        if (!this._settings || !this._settings.methods || !this._settings.methods.items) {
            return label;
        }

        const foundMethod = this._settings.methods.items.find(m => m.name === methodName);
        return (foundMethod && foundMethod.hasOwnProperty('label'))
            ? foundMethod.label
            : label;
    }

    _getVisibilityInlineMethod(methodName) {
        if (!this._settings || !this._settings.methods || !this._settings.methods.items) {
            return true;
        }

        const foundMethod = this._settings.methods.items.find(m => m.name === methodName);
        return (foundMethod && foundMethod.hasOwnProperty('enabled'))
            ? foundMethod.enabled
            : true;
    }

    _objectCount(cls, packets) {
        let mask = '*';

        if (packets) {
            mask = packets.join('/');
        }

        return cls.getIndex('Pack')
            .then(index => {
                let byUser = this._byUser(cls);

                if (byUser !== '') {
                    byUser = `[${byUser}]`
                }

                return index.XQuery(`count(//PACK/OBJECT${byUser})`, mask, 'json')
                    .then(res => res[0] || 0)
            });
    }

    _visibleMethods(methods) {
        return Promise.all(methods
            .map(bm => (bm.isVisible)
                ? bm.isVisible(this)
                : Promise.resolve(true))
        )
            .then(function (methodsVisibility) {
                const visibleMethods = [];

                this.forEach((m, idx) => {
                    if (methodsVisibility[idx]) {
                        visibleMethods.push(this[idx])
                    }
                });

                return visibleMethods;
            }.bind(methods))
    }

    _createObject() {
        this._objectMode = 'create';
        return this._createNewObject()
            .then((newObj) => this._loadSettings(newObj))
            .then((settings) => this._emptyFieldsStructure({destination: [], settings: settings}))
            .then(this._setFieldsStructure.bind(this))
            .then(this._initCreateHandler.bind(this))
            .then(() => this._iterationByFields())
            .then(() => false)
    }


    _setMode(mode) {
        return Promise.resolve(this.mode = mode);
    }

    _emptyFieldsStructure(param) {
        if (param.field) {
            if (param.field.isTable) {
                const ds = new DataSet(param.field, {dataRoot: param.dataRoot || this.currentContext.Root});

                return Promise.all([
                    ds.metadata,
                    ds.Items
                ]).then((dsRes) => {
                    return this._fieldStructureValue(param)
                        .then(() => {
                            const displayedFields = dsRes[0];

                            if (displayedFields && displayedFields.length > 0) {
                                const items = param.destination.items = [];

                                return Promise.all(dsRes[1]
                                    .map(row => {
                                            return Promise.all(displayedFields
                                                .map(f => {
                                                        return this._emptyFieldsStructure({
                                                            destination: items[items.push({}) - 1],
                                                            field: f.field,
                                                            root: row,
                                                            settings: f.settings
                                                        })
                                                    }
                                                )
                                            )
                                        }
                                    ))
                                    .then(Promise.resolve());
                            }
                        })
                })
            } else {
                let destination = param.destination;
                destination.id = param.field.name;
                destination.field = param.field;
                destination.settings = param.settings;
                return;
            }
        }

        return this.currentContext.fields
            .then((fields) => this._mergeFieldsWithSettings({
                fields,
                settings: param.settings
            }))
            .then((displayedFields) => {
                if (displayedFields && displayedFields.length > 0) {
                    return Promise.all(displayedFields.map((f) => {
                        return this._emptyFieldsStructure(
                            {
                                destination: param.destination[param.destination.push({}) - 1],
                                field: f.field,
                                dataRoot: this.currentContext.Root,
                                settings: f.settings
                            })
                    }))
                }
            })
            .then(() => {
                return param.destination
            })
    }

    _mergeFieldsWithSettings(param) {
        const fieldsSettings = param.settings;
        const fields = param.fields;

        if (fieldsSettings && fieldsSettings.fields) {
            return fields
                .map(field => ({field, settings: fieldsSettings.fields.find(fs => fs.name === field.name)}))
        }

        return fields
            .map(field => ({field}))
    }

    _setFieldsStructure(fieldsStructure) {
        this._fieldsStructure = fieldsStructure;
        return Promise.resolve();
    }

    _createNewObject() {
        return this.currentContext.createObject()
            .then((newObj) => {
                this._newObject = newObj;

                return Promise.all([
                    this._newObject.setValue('Author', this.User.id),
                    this._newObject.setValue('Owner', this.User.id)
                ])
                    .then(() => this._newObject)
            })
    }

    _initCreateHandler() {
        this.handler = (command, type) => {
            if (command) {
                const validationResult = Session.validationFieldValue(this._currentFillField.field, command);
                return (validationResult.error)
                    ? this.platform.send({text: validationResult.error}, this.CurrentPlatformUser)
                    : this.saveFieldValue(this._newObject, this._currentFillField.field, command)
                        .then(this._iterationByFields.bind(this))
            }
        };


        return Promise.resolve();
    }

    _iterationByFields() {
        const fieldItem = this._nextField();

        if (fieldItem) {
            return this._getVisibilityField(fieldItem)
                .then((visible) => {
                    if (visible) {
                        this._currentFillField = fieldItem;
                        return this._requestFieldValue();
                    }

                    return this._iterationByFields()
                })
        } else {
            return this._processingSettingsRecalc(this._newObject)
                .then(() => this._newObject.save())
                .then(() => this.SetCurrentContext(this._newObject)
                    .then(() => {
                        this._clearTemporaryData();
                        this._newObject = undefined;
                        this._updatedObjects = true;
                        this.Message.then(res => this.platform.send(res, this.CurrentPlatformUser))
                    })
                )
                .catch(err => {
                    this._clearTemporaryData();
                    console.log(err);
                })
        }
    }

    _getVisibilityField(fieldItem) {
        let visible = CommonUtils.findPropertyValue(fieldItem, `settings.mode.${this._objectMode}.visible`);

        if (visible === undefined) {
            visible = CommonUtils.findPropertyValue(this._settings, `defaults.fields.mode.${this._objectMode}.visible`, true);
        }

        return CommonUtils.getValueFromProperty(
            visible,
            {
                context: (this._objectMode === 'create') ? this._newObject : this.currentContext,
                params: {
                    session: this
                }
            }
        );
    }

    createNewObject(cls) {
        if (!cls) {
            throw new Error('cls undefined' + typeof code);
        }

        return cls.createObject()
            .then(obj => this._newObject = obj)
            .then(obj => Promise.all([
                    obj.setValue('Author', this.User.id),
                    obj.setValue('Owner', this.User.id),
                ])
                    .then(() => obj)
            )
    }

    newObject() {
        return Promise.resolve(this._newObject);
    }

    _requestFieldValue() {
        const obj = this._newObject || this.currentContext;
        const fieldType = (this._currentFillField && this._currentFillField.settings && this._currentFillField.settings.type);
        let defaultValue = this._currentFillField.field.Root.Default;

        function saveFieldValue(value) {
            return this.saveFieldValue(obj, this._currentFillField.field, value)
                .then(() => {
                        return (this._newObject)
                            ? this._iterationByFields()
                            : this._saveCurrentObject()
                    }
                )
        }

        const _saveFieldValue = saveFieldValue.bind(this);

        if (defaultValue) {
            if (this._currentFillField.field.Root.ValueList) {
                defaultValue = defaultValue.substr(1, defaultValue.length - 2)
            }

            return _saveFieldValue(defaultValue);
        }

        switch (fieldType) {
            case 'phone':
                return this._userHasPhone()
                    .then(phone => (phone)
                        ? _saveFieldValue(phone)
                        : this.requestPhone()
                    );
                break;
            case 'location':
                this.requestLocation();
                break;
            case 'fio':
                return this._userHasFIO()
                    .then(fio => (fio)
                        ? _saveFieldValue(fio)
                        : this.requestFIO()
                    );
                break;
            default:
                this.sendFieldMessage(this._currentFillField.field);
        }
    }

    _clearTemporaryData(item) {
        if (this.currentContext.prefix !== 'O' && this.currentContext.$owner !== item) {
            this.currentShowObject = this.LimitShowObject;
        }

        this.handler = undefined;
        this._mode = undefined;
        this._objectMode = 'read';
        this._pathField = undefined;
        this._currentFillField = undefined;
        this._fieldsStructure = undefined;
        this._customSource = undefined;
        this._tempData = undefined;
    }

    _saveCurrentObject() {
        this._clearTemporaryData();
        return this.currentContext.save();
    }

    _nextField() {
        this._pathField = this._nextPathField();

        if (this._pathField) {
            return this._fieldsStructure[this._pathField]
        }
    }

    _nextPathField() {
        if (this._pathField) {
            const lastIdxDot = this._pathField.lastIndexOf('.');
            if (lastIdxDot === -1) {
                let itemIdx = parseInt(this._pathField);

                if (this._fieldsStructure.length - 1 === itemIdx) {
                    return;
                }

                itemIdx++;
                return itemIdx.toString();

            } else {
                //TODO Table
            }
        } else {
            return (this._fieldsStructure.length === 0)
                ? ''
                : '0';
        }
    }

    editObject() {
        this._objectMode = 'write';

        return Promise.all([
            this.fields,
            this.Commands
        ])
            .then(this._mergeGroups)
            .then(this._restoreGroupMsgId.bind(this))
            .then(this._buildStructure.bind(this))
            .then(res => {
                this._objectMode = 'read';
                this.platform.send(res, this.CurrentPlatformUser);
            });
    }

    _requestEditField(opt) {
        this._currentFillField = opt.item;
        this.handler = this._editField.bind(this);
        return this._requestFieldValue();
    }

    _editField(command) {
        const validationResult = Session.validationFieldValue(this._currentFillField.field, command);

        if (validationResult.error) {
            return this.platform.send({text: validationResult.error}, this.CurrentPlatformUser);
        }

        this.handler = undefined;
        return this.saveFieldValue(this.currentContext, this._currentFillField.field, command)
            .then(() => this.currentContext.save())
            .then(() => this.platform.send({text: 'Объект обновлен'}, this.CurrentPlatformUser)
                .then(() => {
                    setTimeout(() => {
                            this._updatedObjects = true;
                            this.Message
                                .then(res => this.platform.send(res, this.CurrentPlatformUser))
                        }
                        , 300)
                })
            )
    }

    static validationFieldValue(field, value) {
        let res = {};

        const type = (typeof field.type === 'string')
            ? field.type.toLowerCase()
            : field.type;

        switch (type) {
            case 'image':
            case 'рисунок':
            case 'файл':
            case 'file':
                const re = /^(?:https?|ftp):\/\/(?:(?:(?:[\w\.\-\+!$&'\(\)*\+,;=]|%[0-9a-f]{2})+:)*(?:[\w\.\-\+%!$&'\(\)*\+,;=]|%[0-9a-f]{2})+@)?(?:(?:[a-z0-9\-\.]|%[0-9a-f]{2})+|(?:\[(?:[0-9a-f]{0,4}:)*(?:[0-9a-f]{0,4})\]))(?::[0-9]+)?(?:[\/|\?](?:[\w#!:\.\?\+=&@!$'~*,;\/\(\)\[\]\-]|%[0-9a-f]{2})*)?$/i;
                if (!re.test(value)) {
                    res = {error: 'Введите корректное значение'};
                }
                break;
        }

        return res;
    }

    sendFieldMessage(field) {
        return this.messageField(field)
            .then(msgField => this.platform.send(msgField, this.CurrentPlatformUser))
    }

    saveFieldValue(object, field, value) {
        const type = (typeof field.type === 'string')
            ? field.type.toLowerCase()
            : field.type;

        switch (type) {
            case 'file':
            case 'рисунок':
            case 'файл':
            case 'image':
                return fetch(value)
                    .then(res => res.buffer()
                        .then(buffer => field.setValue(object, new File({name: value, buffer})))
                    );
            default:
                if (field.TypeClass && !field.TypeClass.isAbstract) {
                    return field.TypeClass.getObject(value)
                        .then(obj => field.setValue(object, obj))
                } else if (field.Root.ValueList) {
                    let valueList = field.Root.ValueList
                        .split(',')
                        .map(value => ({
                            label: value.substring(value.lastIndexOf('=') + 1, value.length - 1),
                            id: value.substring(1, value.lastIndexOf('='))
                        }));

                    let foundValue = valueList.find(val => val.label === value);

                    return object.setValue(field, foundValue.label)
                        .then(() => object.Root[`$${field.name}`] = [{key: foundValue.id}])
                } else {
                    return object.setValue(field, value);
                }
        }
    }

    messageField(field) {
        const type = (typeof field.type === 'string')
            ? field.type.toLowerCase()
            : field.type;

        switch (type) {
            case 'image':
            case 'рисунок':
            case 'файл':
            case 'file':
                return Promise.resolve({text: `Выберите ${field.label || field.name}`});
            default:
                if (field.TypeClass && !field.TypeClass.isAbstract) {
                    const byUser = this._byUser(field.TypeClass);

                    return field.TypeClass.getIndex('botgrid', 'element x { for $o in //OBJECT order by $o/@date descending return element O { $o/@oid, $o/@name, $o/@date, $o/@Author }}')
                        .then(index => {
                            return index.POST('xquery_index', {
                                async: false,
                            }, `for $a in //O${(byUser === '') ? '' : `[${byUser}]`} return element O {$a/@oid}`)
                                .then(res => {
                                        return Promise.all(res.$O.map(el => field.TypeClass.getObject(el.oid)))
                                            .then(objects =>
                                                Promise.resolve(Session._buildCustomGroup(
                                                    field.label || field.name,
                                                    objects.map(obj => ({label: obj.name, command: obj.id})))
                                                ))
                                    }
                                )
                        })
                        .catch(err => [])
                } else if (field.Root.ValueList) {
                    const items = field.Root.ValueList
                        .split(',')
                        .map(value => ({
                            label: value.substring(value.lastIndexOf('=') + 1, value.length - 1),
                            command: value.substring(value.lastIndexOf('=') + 1, value.length - 1)
                        }));
                    return Promise.resolve(Session._buildCustomGroup(field.label || field.name, items));
                } else {
                    return Promise.resolve({text: `Введите: ${field.label || field.name}`});
                }
                break;
        }
    }

    static _buildCustomGroup(text, items) {
        let res = {
            groups: {}
        };

        res.groups['custom'] = {items: items, text: text};
        return res
    }

    get Commands() {
        return this._availabilityOfCommands()
            .then((availabilityOfCommands) => {
                if (!availabilityOfCommands) {
                    return Promise.resolve({name: 'commands', items: [], text: 'Commands'});
                }

                return this._buildCommands()
                    .then((commands) => {
                        if (commands.length > 0) {
                            return Promise.resolve({name: 'commands', items: commands, text: 'Commands'});
                        }

                        return Promise.resolve();
                    })
            });
    }

    _availabilityOfCommands() {
        if (this.currentContext.prefix === 'B') {
            return Promise.resolve(false);
        }

        return CommonUtils.getValueFromProperty(
            CommonUtils.findPropertyValue(this._settings, `commands.enabled`, true),
            {
                context: this.currentContext,
                params: {session: this},
            });
    }

    _buildCommands() {
        return Promise.all(
            [
                this._buildCommandsCommon(),
                this._buildCommandsCustom()
            ]
        ).then((commandsGroups) => []
            .concat(...commandsGroups)
            .filter(command => command));
    }

    _buildCommandsCustom() {
        const rawCommands = CommonUtils.findPropertyValue(this._settings, 'commands.custom.items');

        if (!rawCommands) {
            return Promise.resolve([]);
        }

        return Promise.all(
            rawCommands.map(
                rawCommand => this._buildCustomCommand(rawCommand))
        )
    }

    _buildCustomCommand(rawCommand) {
        return CommonUtils.getValueFromProperty(
            CommonUtils.findPropertyValue(rawCommand, `enabled`, true),
            {
                context: rawCommand,
                params: {session: this},
            })
            .then((enabled) => {
                if (!enabled) {
                    return;
                }

                return CommonUtils.getValueFromProperty(
                    CommonUtils.findPropertyValue(rawCommand, `label`),
                    {
                        context: rawCommand,
                        params: {session: this},
                    })
                    .then((label) => {
                        if (!label) {
                            return;
                        }

                        return {
                            id: rawCommand.name,
                            label: label,
                            command: `/${rawCommand.name}`,
                            run: rawCommand.run
                        }
                    })
            })
    }


    _buildCommandsCommon() {
        const commonCommandsSettings = CommonUtils.findPropertyValue(this._settings, `commands.common.items`);
        const commonCommands = [];

        commonCommands.push(
            {
                id: 'back',
                label: '🔙',
                command: '/back',
                run: () => {
                    this._clearTemporaryData();

                    if (this._mask && this._mask.length > 0) {
                        this._mask = this._mask.length === 1 ? undefined : this._mask.pop();

                        if (this.currentContext.prefix === 'O') {
                            return this.SetCurrentContext(this.currentContext.$class);
                        }

                        return Promise.resolve(true);
                    } else if (this._links.some(link => link.sourceItem === this.currentContext)) {
                        return this.SetCurrentContext(this.currentContext.$base)
                    } else {
                        return this.SetCurrentContext(
                            (this.currentContext.prefix === 'O')
                                ? this.currentContext.$class.$owner
                                : this.currentContext.$owner);
                    }
                }
            },
            {
                label: '🏠',
                id: 'home',
                command: '/home',
                run: () => this.SetCurrentContext(this.Bot.rootContext)
            }
        );

        this.currentShowObject = this.currentShowObject || this.LimitShowObject;

        if (this.CountObjects > this.LimitShowObject && this.currentShowObject - this.LimitShowObject !== 0) {
            commonCommands.push({
                label: 'Пред',
                id: 'prev',
                command: '/prev',
                run: () => {
                    this.currentShowObject -= this.LimitShowObject;
                    this._updatedObjects = true;

                    if (this.currentContext.prefix === 'O') {
                        return this.SetCurrentContext(this.currentContext.$class)
                            .then(() => Promise.resolve(true))
                    } else {
                        return true
                    }
                }
            });
        }

        if (this.currentShowObject < this.CountObjects && !this.currentContext.hasChildren) {
            commonCommands.push({
                label: 'След',
                id: 'next',
                command: '/next',
                run: () => {
                    this.currentShowObject += this.LimitShowObject;
                    this._updatedObjects = true;

                    if (this.currentContext.prefix === 'O') {
                        return this.SetCurrentContext(this.currentContext.$class)
                            .then(() => Promise.resolve(true))
                    } else {
                        return true
                    }
                }
            });
        }

        commonCommands.push({
            label: '🔍',
            id: 'find',
            command: '/find',
            run: () => {
                this.handler = (command) => {
                    this.SearchPhrase = command;
                    this.handler = undefined;
                    this._updatedObjects = true;

                    let xq = `count(x/O[contains(lower-case(@name), lower-case("${this.SearchPhrase}"))])`;
                    const classContext = (this.currentContext.prefix === 'O') ? this.currentContext.$class : this.currentContext;

                    return classContext.XQuery(xq, {
                        byuser: (this.accessLevelForLink() == 6) ?
                            '*'
                            : this.User.id
                    })
                        .then(res => {
                            this.CountObjects = res[0];
                            return Promise.all([
                                this.Commands,
                                this.Objects
                            ])
                                .then(this._mergeGroups)
                                .then(this._restoreGroupMsgId.bind(this))
                                .then(this._buildStructure.bind(this))
                                .then(res => this.platform.send(res, this.CurrentPlatformUser));
                        });
                };
                return this.platform.send({text: 'Введите текст для поиска'}, this.CurrentPlatformUser)
            }
        });

        return Promise.all(
            commonCommands
                .map(commonCommand => this._buildCommonCommand(commonCommand, commonCommandsSettings && commonCommandsSettings
                    .find(commonCommandSettings => commonCommandSettings.name === commonCommand.id))
                )
        );
    }

    _buildCommonCommand(rawCommand, commandSettings) {
        if (!commandSettings) {
            return Promise.resolve(rawCommand);
        } else {
            return CommonUtils.getValueFromProperty(
                CommonUtils.findPropertyValue(commandSettings, `enabled`, true),
                {
                    context: rawCommand,
                    params: {session: this},
                })
                .then((enabled) => {
                    if (!enabled) {
                        return;
                    }

                    return CommonUtils.getValueFromProperty(
                        CommonUtils.findPropertyValue(commandSettings, `label`, rawCommand.label),
                        {
                            context: rawCommand,
                            params: {session: this},
                        })
                        .then((label) => {
                            if (!label) {
                                return;
                            }

                            return {
                                id: rawCommand.name,
                                label: label,
                                command: `/${rawCommand.name}`,
                                run: rawCommand.run
                            }
                        })
                })
        }
    }

    get fields() {
        return (this.currentContext.prefix === 'O')
            ? this._getFieldsGroup()
            : Promise.resolve();
    }

    _getFieldsGroup() {
        return this._buildFieldsStructure({
            destination: [],
            settings: this._settings && this._settings.fields
        })
            .then(res => {
                    return (res && res.destination.length > 0)
                        ? ({name: 'fields', items: res.destination, editMode: (this._objectMode === 'write')})
                        : Promise.resolve()
                }
            )
    }

    _buildFieldsStructure(param) {
        if (param.field) {
            if (param.field.isTable) {
                const ds = new DataSet(param.field, {dataRoot: param.dataRoot || this.currentContext.Root});

                return Promise.all([
                    ds.metadata,
                    ds.Items
                ]).then((dsRes) => {
                    return this._fieldStructureValue(param)
                        .then(() => {
                            return this._filterFields({
                                fields: dsRes[0],
                                settings: param.settings && param.settings.items
                            })
                                .then((visibleFields) => {
                                    if (visibleFields && visibleFields.length > 0) {
                                        const items = param.destination.items = [];

                                        return Promise.all(dsRes[1]
                                            .map(row => {
                                                    return Promise.all(visibleFields
                                                        .map(f => {
                                                                return this._buildFieldsStructure({
                                                                    destination: items[items.push({}) - 1],
                                                                    field: f.field,
                                                                    root: row,
                                                                    settings: f.settings
                                                                })
                                                            }
                                                        )
                                                    )
                                                }
                                            ))
                                            .then(Promise.resolve());
                                    }
                                })
                        })
                })
            } else {
                return this._fieldStructureValue(param)
            }
        }

        return this.currentContext.fields
            .then((fields) => {
                return this._filterFields({
                    fields,
                    settings: param.settings
                }).then((visibleFields) => {
                    if (visibleFields && visibleFields.length > 0) {
                        return Promise.all(visibleFields.map((f) => {
                            return this._buildFieldsStructure(
                                {
                                    destination: param.destination[param.destination.push({}) - 1],
                                    field: f.field,
                                    dataRoot: this.currentContext.Root,
                                    settings: f.settings
                                })
                        }))
                    }
                })
                    .then(() => {
                        return param
                    })
            });
    }

    _fieldStructureValue(param) {
        return Session._fieldValue({
            obj: this.currentContext,
            field: param.field,
            root: param.root
        })
            .then((value) => {
                const field = param.field;
                const settings = param.settings;
                const destination = param.destination;
                const isOdaClassOrOdaObject = (value instanceof odaObject || value instanceof odaClass);
                const defaultHideLabelValue = CommonUtils.findPropertyValue(this._settings, `defaults.fields.mode.${this._objectMode}.hideLabel`, false);
                const hideLabel = CommonUtils.findPropertyValue(settings, `mode.${this._objectMode}.hideLabel`, defaultHideLabelValue);

                let run;
                let command;

                if (this._objectMode === 'write') {
                    run = this._requestEditField.bind(this);
                    command = `/izm_${slugify(param.field.label)}`
                } else if (isOdaClassOrOdaObject) {
                    run = () => Promise.resolve(this.SetCurrentContext(value));
                    command = `/goto_${slugify(param.field.label)}`
                }

                destination.id = field.name;
                destination.field = field;
                destination.settings = settings;
                destination.label = field.label || field.name;
                destination.command = command;
                destination.hideLabel = hideLabel;
                destination.linkCaption = value instanceof odaFile ? 'Открыть' : undefined;
                destination.transition = isOdaClassOrOdaObject;
                destination.field = field;
                destination.run = run;
                destination.value = this._formattedFieldValue(field, value, isOdaClassOrOdaObject);
            })
            .catch(err => {
                console.log(err)
            });
    }

    _filterFields(param) {
        const fieldsSettings = param.settings;
        const fields = param.fields;
        const fieldItems = (fieldsSettings)
            ? fields.map(field => ({
                field,
                settings: fieldsSettings && fieldsSettings.find(fs => fs.name === field.name)
            }))
            : fields.map(field => ({field}));


        return Promise.all(fieldItems.map(fieldItem => this._getVisibilityField(fieldItem)))
            .then(fieldsVisibility => {
                const visibleFields = fieldItems.reduce((p, n, idx) => {
                    if (fieldsVisibility[idx]) {
                        p.push(n);
                    }
                    return p;
                }, []);

                return visibleFields;
            });
    }

    _formattedFieldValue(field, value, transition) {
        const constructorName = field.constructor.name;

        switch (constructorName) {
            case 'odaDateTimeField':
                return new Date(+new Date(value) - 10800000).toLocaleString([], {
                    year: 'numeric',
                    month: '2-digit',
                    day: '2-digit',
                    hour: '2-digit',
                    minute: '2-digit'
                });
                break;
            default:
                if (value instanceof odaItem) {
                    return (value instanceof odaFile)
                        ? `${global.EXTERNALORIGIN}/api${value.fullId}`
                        : (transition)
                            ? value.label
                            : value;
                } else {
                    return value || ''
                }
        }
    }

    get Text() {
        const settings = this._settings;

        const settingsText = settings && settings.msg || settings && settings.label;

        if (settingsText) {
            return CommonUtils.getValueFromProperty(
                settingsText,
                {
                    context: this.currentContext,
                    params: {session: this},
                });
        }

        return Promise.resolve(this.currentContext.label || this.currentContext.name)
    }

    get icon() {
        return (this.currentContext.prefix === 'O' || this._objectMode === 'write')
            ? this.currentContext.$class.fields.then(fields => {
                let photo = fields.find(item => item.name === 'photo');

                if (photo) {
                    return Promise.resolve(photo.getValue(this.currentContext))
                        .then(value => {
                            return (value) ? `${value.externalUrl}/?mode=preview256` : ''
                        })
                }
            })
            : Promise.resolve();
    }

    requestContactInformation() {
        return new Promise(resolve =>
            this.platform.requestContactInformation(this.User, `Отправить контакт`)
                .then(() => {
                    this.handler = (command) => {
                        this.handler = undefined;
                        resolve(command)
                    }
                })
        )
    }


    requestPhone() {
        return this.platform.requestPhone({
            user: this.CurrentPlatformUser,
            text: `Необходимо сообщить ваш телефонный номер`
        })
            .then(() => false)
            .catch((err) => {
                console.log(err)
            })

    }

    requestLocation() {
        return this.platform.requestLocation({
            user: this.CurrentPlatformUser,
            text: `Введите свой адрес`,
            caption: `Необходимо сообщить ваши координаты`
        })
            .then(() => false)
            .catch((err) => {
                console.log(err)
            })
    }

    requestFIO() {
        return this.platform.requestPhone({
            user: this.CurrentPlatformUser,
            text: `Необходимо сообщить ваше ФИО`
        })
            .then(() => false)
            .catch((err) => {
                console.log(err)
            })
    }

    SendMessage(txt) {
        return this.platform.send({text: txt}, this.CurrentPlatformUser);
    }

    SendRequestPhone() {
        return new Promise(resolve => {
            return this._userHasPhone()
                .then(phone => (phone)
                    ? resolve(phone)
                    : this.requestPhone()
                        .then(() => {
                            this.handler = (command) => {
                                this.handler = undefined;
                                resolve(command)
                            }
                        })
                )
        })
    }

    SendQuestion(txt) {
        return new Promise(resolve => {
            return this.platform.send({text: txt}, this.CurrentPlatformUser)
                .then(res => {
                    this.handler = (command) => {
                        this.handler = undefined;
                        resolve(command)
                    }
                })
        })
    }

    requestFieldValue(field, odaObject) {
        return new Promise((resolve, reject) => {
            return this.messageField(field)
                .then(msgField => this.platform.send(msgField, this.CurrentPlatformUser)
                    .then(res => {
                        this.handler = (command) => {
                            const validationResult = Session.validationFieldValue(field, command);
                            if (validationResult.error) {
                                return this.platform.send({text: validationResult.error}, this.CurrentPlatformUser);
                            } else {
                                this.handler = undefined;
                                return this.saveFieldValue(odaObject, field, command)
                                    .then(resolve)
                                    .catch(reject)
                            }
                        }
                    }))
        })
    }

    sendMenuMessage(txt, buttons, options) {
        return new Promise((resolve, reject) => {
            let customGroup = {
                name: 'custom',
                items: buttons,
                text: txt
            };

            const msg = {
                text: txt,
                groups: {
                    'custom': customGroup
                }
            };

            return this.platform.send(msg, this.CurrentPlatformUser)
                .then(res => {
                    this.handler = (command) => {
                        this.handler = undefined;
                        resolve(command)
                    }
                })
        });
    }


//Deprecated
    AddCommandToCommands(command) {
        if (command && command.label && command.id && command.command && command.run) {
            const commandsGroup = this._structure.groups['commands'];

            if (commandsGroup) {
                commandsGroup.items.push(command);
            }
        }

        return Promise.resolve();
    }

//Deprecated
    UpdateMethodsTitle(txt) {
        const methodsGroup = this._structure.groups['methods'];

        if (methodsGroup) {
            methodsGroup.text = txt;
        }

        return Promise.resolve();
    }

    RefreshGroup(groupName) {
        return this.Message
            .then(() => {
                const foundGroup = this._structure.groups[groupName];

                if (foundGroup) {
                    foundGroup.refresh = true;

                    const msg = {
                        groups: {
                            [groupName]: foundGroup
                        }
                    };

                    return this.platform.send(msg, this.CurrentPlatformUser)
                }
            })
    }

    RefreshMethods() {
        return new Promise(resolve => {
            setTimeout(() => this._handlingEvent('on-enter')
                    .then(() => this.Message
                        .then(() => {
                            const foundGroup = this._structure.groups['methods'];
                            if (foundGroup) {
                                foundGroup.refresh = true;
                                const msg = {groups: {}};
                                msg.groups['methods'] = foundGroup;

                                return this.platform.send(msg, this.CurrentPlatformUser)
                                    .then(() => resolve())
                            } else {
                                resolve();
                            }
                        }))
                , 300)
        })
    }
}

class CommonUtils {
    static findPropertyValue(object, path, defaultValue) {
        if (!object || !path) {
            return defaultValue;
        }

        const propsPart = path.split('.');
        let currentProperty = object;

        for (let i = 0; i <= propsPart.length - 1; i++) {
            currentProperty = currentProperty[propsPart[i]];

            if (currentProperty === undefined) {
                return defaultValue;
            }
        }

        return currentProperty;
    }

    static getValueFromProperty(property, {context, params} = {}) {
        if (typeof property === 'function') {
            return Promise.resolve(property.call(context, params));
        }

        return Promise.resolve(property);
    }


    static requireFromString(code, filename, opts) {
        if (typeof filename === 'object') {
            opts = filename;
            filename = undefined;
        }

        opts = opts || {};
        filename = filename || '';

        opts.appendPaths = opts.appendPaths || [];
        opts.prependPaths = opts.prependPaths || [];

        if (typeof code !== 'string') {
            throw new Error('code must be a string, not ' + typeof code);
        }

        let paths = Module._nodeModulePaths(path.dirname(filename));
        let parent = module.parent;
        let m = new Module(filename, parent);

        m.filename = filename;
        m.paths = [].concat(opts.prependPaths).concat(paths).concat(opts.appendPaths);
        m._compile(code, filename);

        let exports = m.exports;
        // parent.children && parent.children.splice(parent.children.indexOf(m), 1);

        return exports;
    };
}

if (typeof module !== 'undefined' && typeof module.exports !== 'undefined') {
    module.exports = Bot;
}