const TelegramBot = require('node-telegram-bot-api');


(class extends odaAction {
    execute() {
        console.log('init telegram');
        return this.contextItem.$STATIC.$object.then(obj => {
            if (obj.Root.token) {
                console.log(obj.Root.token);
                return new Telegram(obj.Root.token);
            }
            console.log('Token not found!');
            throw new Error('Token not found!');
        })
    }
}).register()

class Telegram {
    constructor(token) {
        this.BotInterface = new TelegramBot(token, {polling: true});
        console.log('Telegram is started');

        this.BotInterface.on('message', e => {
            this.sendTyping(e.from.id);
            if (e.document) {
                this.BotInterface.getFileLink(e.document.file_id)
                .then(url => this.Message(this._mainInformation(e), {type: 'text', value: url}));
            } else if (e.photo) {
                this.BotInterface.getFileLink(e.photo[e.photo.length - 1].file_id)
                .then(url => this.Message(this._mainInformation(e), {type: 'text', value: url}));
            } else if (e.contact) {
                this.Message(this._mainInformation(e), {type: 'phone', value: e.contact.phone_number});
            } else if (e.location) {
                this.Message(this._mainInformation(e), {
                    type: 'location',
                    value: `${e.location.latitude},${e.location.longitude}`
                })
            } else {
                this.Message(this._mainInformation(e), {type: 'text', value: e.text});
            }

        });

        this.BotInterface.on('callback_query', e => {
            this.sendTyping(e.from.id);
            return this.Message(this._mainInformation(e), {type: 'text', value: e.data});
        });
    }

    _mainInformation(e) {
        return {
            id: e.from.id,
            firstName: e.from.first_name,
            lastName: e.from.last_name,
            platform: this.__proto__.constructor.name
        };
    }

    send(message, user) {
        const promises = [];
        const groups = message.groups;
        let navGroupAdded = false;

        if (message.text) {
            if (message.image) {
                let pictureParam = {
                    'caption': message.text
                };

                const navGroup = this.navGroup(groups);
                if (navGroup) {
                    navGroupAdded = true;
                    pictureParam.reply_markup = this.navGroupMarkUp(navGroup, groups['commands']).reply_markup;
                }

                promises.push(() => this.sendPicture(user.id, message.image, pictureParam))
            } else {
                if (!groups || (groups && !Object.keys(groups)
                    .map(groupName => groups[groupName])
                    .some(gr => gr.text === message.text))) {
                    promises.push(() => this.sendMessage(user.id, message.text, {parse_mode: 'HTML'}))
                }
            }
        }

        if (groups) {
            if (!navGroupAdded) {
                const navGroup = this.navGroup(groups);

                if (navGroup) {
                    promises.push(() => this.sendMessage(user.id, navGroup.text, this.navGroupMarkUp(navGroup, groups['commands'])));
                }
            }

            if (groups['fields']) {
                promises.push(() => this.fields(groups['fields'], user));

                if (!groups['fields'].edit) {
                    promises.push(() => this.transitions(groups['fields'], user));
                }
            }

            if (groups['methods']) {
                promises.push(() => this.methods(groups['methods'], user));
            }

            if (groups['custom']) {
                promises.push(() => this.custom(groups['custom'], user));
            }
        }

        promises.reduce(
            (p, i) => p.then(
                () => new Promise((resolve, reject) => i().then(() => resolve()))
            ),
            Promise.resolve()
        );

        return Promise.resolve();
    }

    navGroup(groups) {
        let navGroup = groups['structure'] || groups['objects'] || groups['packets'];
        if (navGroup && navGroup.updated) {
            return navGroup
        }
    }

    navGroupMarkUp(navGroup, commands) {
        return this.structure(navGroup, commands);
    }

    custom(group, user) {
        let buttons = [];
        const layout = group.param && group.param.layout || 'vertical';

        switch (layout) {
            case 'vertical': {
                if (group.items) {
                    group.items.forEach(item => buttons.push([
                        {
                            text: item.label,
                            callback_data: item.command
                        }
                    ]))
                }
                break;
            }


            case 'custom': {
                group.items.reduce((a, c) => {
                    a.push(c.items.map(i => ({"text": i.label, "callback_data": i.command})));
                    return a
                }, buttons);
                break;
            }
        }

        return (group.refresh)
            ? this.editMessage(user.id, group.msgId, {
                text: group.text,
                replyMarkup: Telegram._replyMarkup(buttons).reply_markup
            })
            : this.sendMessage(user.id, group.text, Telegram._replyMarkup(buttons), group);
    }

    fields(group, user) {
        const stack = [];
        const fields = [];

        stack.unshift(...group.items);

        while (stack.length !== 0) {
            let node = stack.shift();

            if (node.items) {
                stack
                .unshift(...[].concat(...node.items));
            }

            fields.push(node);
        }

        const msgFields = fields.reduce((p, n) => {
            if (n.value && n.label === 'photo') {
                return p
            }

            let textValue = n.value;

            if (group.editMode) {
                textValue = (n.value instanceof odaItem)
                    ? n.value.name
                    : n.value;


            } else {
                if (n.transition) {
                    return p;
                }
            }

            if (!n.hideLabel) {
                p += `<b>${n.label}</b>: `;
            }

            p += (n.linkCaption)
                ? `<a href="${textValue}">${n.linkCaption}</a>`
                : `${textValue}`;

            p += (group.editMode)
                ? ` <a href="${n.command}">${n.command}</a> \n`
                : `\n`;

            return p;
        }, '');

        return this.sendMessage(user.id, msgFields, {parse_mode: 'HTML', disable_web_page_preview: true})
    }

    transitions(group, user) {
        if (group.editMode) {
            return Promise.resolve();
        }

        const stack = [];
        const fields = [];

        stack.unshift(...group.items);

        while (stack.length !== 0) {
            let node = stack.shift();

            if (node.items) {
                stack
                .unshift(...[].concat(...node.items));
            }

            fields.push(node);
        }

        const transitions = fields
        .reduce((p, n) => {
            if (n.transition) {
                p.push([{
                    text: n.value,
                    callback_data: n.command
                }])
            }
            return p;
        }, []);

        return (transitions.length > 0)
            ? this.sendMessage(user.id, 'Перейти', Telegram._replyMarkup(transitions))
            : Promise.resolve();
    }

    structure(group, commands) {
        const menu = [];

        if (commands) {
            menu.push(commands.items.map(command => ({
                    text: `${command.icon || ''}${command.label || ''} `,
                    callback_data: command.command
                })
            ));
        }

        group.items.forEach((item, i) => {
            if (i % 2 === 0) {
                menu.push([]);
            }

            menu[menu.length - 1].push({
                text: item.label,
                callback_data: item.command,
                request_contact: item.requiredContact
            });
        });

        return Telegram._replyMarkup(menu, true)
    }


    methods(group, user) {
        const menu = [];

        group.items.forEach((item, i) => {
            if (i % 2 === 0) {
                menu.push([]);
            }

            menu[menu.length - 1].push(
                {
                    text: item.label || item.icon,
                    callback_data: item.command
                });
        });

        return (group.refresh)
            ? this.editMessage(user.id, group.msgId, {
                text: group.text,
                replyMarkup: Telegram._replyMarkup(menu).reply_markup
            })
            : this.sendMessage(user.id, group.text, Telegram._replyMarkup(menu), group);
    }

    requestContactInformation(user, text) {
        return this.sendMessage(user.id, text, Telegram._replyMarkup([[
            {
                text: 'Сообщите свой номер телефона',
                callback_data: '/share_phone',
                request_contact: true
            }
        ]], true));
    }

    requestPhone(req) {
        return this.sendMessage(req.user.id, req.text, Telegram._replyMarkup([[
            {
                text: req.text,
                callback_data: '/share_phone',
                request_contact: true
            }
        ]], true));
    }

    requestLocation(req) {
        return this.sendMessage(req.user.id, req.text, Telegram._replyMarkup([[
            {
                text: req.caption || req.text,
                callback_data: '/share_location',
                request_location: true
            }
        ]], true));
    }

    sendMessage(userId, caption, menu, group) {
        return this.BotInterface.sendMessage(userId, caption, menu)
        .then(res => {
            if (group) {
                group.msgId = res.message_id
            }

            return true;
        });
    }

    sendPicture(userId, url, options) {
        return this.BotInterface.sendPhoto(userId, url, options);
    }

    sendTyping(userId) {
        this.BotInterface.sendChatAction(userId, 'typing');
    }

    editMessage(userId, msgId, opt) {
        return this.BotInterface.editMessageText(opt.text, {
            chat_id: userId,
            message_id: msgId,
            reply_markup: opt.replyMarkup
        });
    }

    static _replyMarkup(menu, keyboard) {
        const rp = {};
        rp[keyboard ? 'keyboard' : 'inline_keyboard'] = menu;

        return {
            parse_mode: 'HTML',
            disable_web_page_preview: false,
            force_reply: true,
            reply_markup: JSON.stringify(rp),
            resize_keyboard: true
        };
    }
}