(() => {
    'use strict';

    let action = class extends odaAction {
        execute() {
            console.log(`Viber init at ${new Date().toLocaleString()} (${this.contextItem.$base.name}, ${this.contextItem.$base.fullId}`);
            return this.contextItem.$STATIC.$object.then(staticObject =>
                    this.contextItem.$class.$STATIC.fields.then(fields => {
                            return Promise.all(
                                fields.map(field => {
                                    const _field = field;

                                    return Promise.resolve(field.getValue(staticObject))
                                        .then(fieldValue => ({
                                            field: _field,
                                            value: fieldValue
                                        }))
                                }))
                                .then(res => {
                                    const emptySettingsValues = res.filter(fr => !fr.value);

                                    if (emptySettingsValues.length > 0) {
                                        return;
                                    }

                                    const options = res.reduce((p, n) => {
                                        p[n.field.name] = n.value;
                                        return p;
                                    }, {});

                                    return new Viber(this.$item, options);
                                })
                        })
                )
        }
    };


    const ViberBot = require('viber-bot').Bot;
    const BotEvents = require('viber-bot').Events;

    //Message Objects
    const TextMessage = require('viber-bot').Message.Text;
    const UrlMessage = require('viber-bot').Message.Url;
    const ContactMessage = require('viber-bot').Message.Contact;
    const PictureMessage = require('viber-bot').Message.Picture;
    const VideoMessage = require('viber-bot').Message.Video;
    const LocationMessage = require('viber-bot').Message.Location;
    const StickerMessage = require('viber-bot').Message.Sticker;
    const RichMediaMessage = require('viber-bot').Message.RichMedia;
    const KeyboardMessage = require('viber-bot').Message.Keyboard;


class Viber {
    constructor(item, options) {
        this.BotInterface = new ViberBot({
            authToken: options.token,
            name: options.name,
            avatar: options.avatar
        });

        this.BotInterface.on(BotEvents.MESSAGE_RECEIVED, this._onMessageReceived);
        this.BotInterface.on(BotEvents.MESSAGE_SENT, this._onMessageSent);
        this.BotInterface.on(BotEvents.CONVERSATION_STARTED, this._onConversationStarted);
        this.BotInterface.on(BotEvents.ERROR, this._onError);
        this.BotInterface.on(BotEvents.UNSUBSCRIBED, this._onUnsubscribed);
        this.BotInterface.on(BotEvents.SUBSCRIBED, this._onSubscribed);

        this.BotInterface.setWebhook(this._buildWebhookUrl(item));
    }

    _buildWebhookUrl(item){
        const url = `${item.externalUrl}?execute=methods/webhook`;
        console.log(`Viber webhook url: ${url}`);
        return url;
    }

    send(message, userProfile) {
        return this.BotInterface.sendMessage(userProfile, new TextMessage('Hello'));
    }

    _onMessageReceived(message, response){
        return this.Message(this._createUserProfile(response.userProfile),  this._createMessage({
            type: 'MESSAGE_RECEIVED'
        }))
    }

    _onMessageSent(message, userProfile){
        return this.Message(this._createUserProfile(userProfile),  this._createMessage({
            type: 'MESSAGE_SENT'
        }))
    }

    _onConversationStarted(userProfile, isSubscribed, context, onFinish){
        return this.Message(this._createUserProfile(userProfile),  this._createMessage({
            type: 'CONVERSATION_STARTED'
        }))
    }

    _onError(err){
        return this.Message(this._createUserProfile(err.userProfile),  this._createMessage({
            type: 'ERROR',
            value: err
        }))
    }

    _onUnsubscribed(response){
        return this.Message(this._createUserProfile(response.userProfile),  this._createMessage({
            type: 'UNSUBSCRIBED'
        }))
    }

    _onSubscribed(response){
        return this.Message(this._createUserProfile(response.userProfile),  this._createMessage({
            type: 'SUBSCRIBED'
        }))
    }

    _createMessage(options){
        return ({

        })
    }

    // id	string	---
    // name	string	---
    // avatar	string	Optional Avatar URL
    // country	string	currently set in CONVERSATION_STARTED event only
    // language	string	currently set in CONVERSATION_STARTED event only
    _createUserProfile(userProfile){
        return ({
            avatar: userProfile.avatar,
            country: userProfile.country,
            firstName: userProfile.name,
            id: userProfile.id,
            language: userProfile.language,
            platform: this.__proto__.constructor.name,
        })
    }
}

    action.register();

    //For testing
    if (typeof module !== 'undefined' && typeof module.exports !== 'undefined'){
        module.exports = action;
    }
})();