(class extends odaClass {

    static get properties(){return {
        itemTypeLabel: function() {
            switch (this.journalLevel) {
                case 0:
                    return this.super('itemTypeLabel');
                case 1:
                    return 'journal';
                default: {
                    return 'account';
                }
            }
        },
        label:{
            get: function () {
                return this.super('label');
            },
            set: function(value) {
                this.setAttr('Label', value);

                if (this.journalLevel > 1) {
                    this.async(() => {
                        this._updateLabel();
                    }, 100)
                }
            }
        },
        icon: function() {
            switch (this.journalLevel) {
                case 0:
                    return this.super('icon');
                case 1:
                    return this.url + '/~/web/class/journal.png';
                default: {
                    return (this.config && this.config.ico && this.super('icon')) || this.url + '/~/web/class/account.png';
                }
            }
        },
        accountLabel:{
            category: 'account',
            get: function() {
                return this.Root['account-label'];
            },
            set: function(value) {
                this.setProperty('account-label', value);
                this._updateLabel();
            }
        },
        accountActive:{
            category: 'account',
            enum: ['A/P', 'A', 'P'],
            get: function() {
                return this.Root['account-active'];
            },
            set: function(value) {
                this.setProperty('account-active', value);
            }
        },
        accountCode:{
            category: 'account',
            get: function() {
                return this.Root['account-code'];
            },

            set: function(value) {
                this.setProperty('account-code', value);
                this._updateLabel();
            }
        },
        journalLevel: function() {
            if (this.$owner && this.$owner.type === this.type)
                return this.$owner.journalLevel + 1;
            return 0;
        },
        defaultViewName: function() {
            switch (this.journalLevel) {
                case 0:
                    return this.super('defaultViewName');
                case 1:
                    return 'balance';
                default:
                    return 'analytics';
            }
        }

    }}


    _getLabel(cls, name) {
        if (cls && cls.journalLevel > 1) {
            return this._getLabel(cls.$owner, `${cls.accountCode}.${name}`)
        }

        return name;
    }

    _updateLabel() {
        this.setProperty('Label', `${this._getLabel(this.$owner, this.accountCode)} ${this.accountLabel}`);
    }





    _getJournal() {
        if (!this._journal) {
            let j = this;
            while (j.journalLevel !== 1) {
                j = j.$owner;
            }
            this._journal = j;
        }
        return this._journal;
    }

    _getAccountObject(b, p) {
        let begin = b ? b.find(a => a.n === this.name) : null;
        let period = p ? p.find(a => a.n === this.name) : null;

        let db = begin ? +begin.db : 0;
        let cb = begin ? +begin.cb : 0;
        let dp = period ? +period.dp : 0;
        let cp = period ? +period.cp : 0;
        let de = db + dp;
        let ce = cb + cp;

        let obj = {account: this};

        if (db - cb === 0) {
            obj.db = 0;
            obj.cb = 0;
        } else if (db - cb > 0) {
            obj.db = db - cb;
            obj.cb = 0;
        } else {
            obj.db = 0;
            obj.cb = cb - db;
        }

        obj.dp = dp;
        obj.cp = cp;

        if (de - ce === 0) {
            obj.de = 0;
            obj.ce = 0;
        } else if (de - ce > 0) {
            obj.de = de - ce;
            obj.ce = 0;
        } else {
            obj.de = 0;
            obj.ce = ce - de;
        }

        return obj;
    }
    _getAccBalance(period, beginDate) {
        return new Promise(resolve => {
            if(this.hasChildren){

                return this.Classes.then( classes => {
                    let p = classes.map(c => c.getBalance(period, beginDate));

                    return Promise.all(p).then(res => {
                        let balance =  res.reduce((a, b) => {
                            a.db += Number(b.db);
                            a.cb += Number(b.cb);
                            a.dp += Number(b.dp);
                            a.cp += Number(b.cp);
                            a.de += Number(b.de);
                            a.ce += Number(b.ce);
                            return a;
                        }, {
                            account: this,
                            db: 0,
                            cb: 0,
                            dp: 0,
                            cp: 0,
                            de: 0,
                            ce: 0
                        })

                        resolve(balance);
                    })
                })
            } else {
                let accs = this._getUpBalance(period, beginDate, true);

                if(accs.then){
                    accs.then(res => {
                        let balance = this._getAccountObject(...res);
                        resolve(balance);
                    })
                } else {
                    let balance = this._getAccountObject(...accs);
                    resolve(balance);
                }
            }
        })
    }

    _getBalanceObject(begin, period) {
        let beg = {db: 0, cb: 0};
        begin && begin.reduce((a, b) => {
            a.db += Number(b.db);
            a.cb += Number(b.cb);
            return a;
        }, beg);

        let per = {dp: 0, cp: 0};
        period && period.reduce((a, b) => {
            a.dp += Number(b.dp);
            a.cp += Number(b.cp);
            return a;
        }, per);

        return {
            account: this,
            db: beg.db,
            cb: beg.cb,
            dp: per.dp,
            cp: per.cp,
            de: (beg.db + per.dp),
            ce: (beg.cb + per.cp)
        };
    }
    _getUpBalance(period, beginDate, cache) {
        let _journal = this._getJournal();

        if(!cache){
            _journal.reset('_upBalance');
        }

        return _journal.$('_upBalance', () => {
            let p = [_journal.getBeginBalance(beginDate), _journal.getPeriodBalance(period)];

            return Promise.all(p).then(res => {
                let begin = res[0];
                let period = res[1];

                return [begin.$X[0].$A, period.$X[0].$A];
            })
        })
    }

    getBalance(period, beginDate, cache){
        return new Promise(resolve => {
            if(this.journalLevel > 1) {
                this._getAccBalance(period, beginDate)
                .then( balance => {
                    resolve(balance);
                })
            }

            if(this.journalLevel === 1){
                let upBalance = this._getUpBalance(period, beginDate, cache);

                if(upBalance.then){
                    upBalance.then(res =>{
                        resolve(this._getBalanceObject(...res));
                    })
                } else {
                    resolve(this._getBalanceObject(...upBalance));
                }

            }
        })
    }

    getBeginBalance(beginDate) {
        let xqBegin = "element X { for $a in //D/A group by $a/@n order by $a/@n return  element A {$a[1]/@n, attribute db {sum($a/@d)}, attribute cb {sum($a/@c)}}}";
        return this.XQuery(xqBegin, {
            index: 'Balance/Cascade',
            mask: beginDate
        })
    }

    getPeriodBalance(periodDate) {
        let xqPeriod = "element X { for $a in //D/A order by $a/@n group by $a/@n return  element A {$a[1]/@n, attribute dp  {sum($a/@d)}, attribute cp  {sum($a/@c)}}}";
        return this.XQuery(xqPeriod, {
            index: 'Balance',
            mask: periodDate
        })
    }

    getAnalyticsIndex(name, xq, mask) {
        return this.XQuery(xq, {
            index: name,
            mask: mask
        })
    }

}).register();