(class extends odaAction {
    static get properties() {
        return {
            allowAccess: 'A',
            label () {
                switch (this.$item.journalLevel) {
                    case 0:
                        return 'Add journal';
                    case 1:
                        return 'Add account';
                    default:
                        return 'Add sub account';
                }
            },
            icon () {
                switch (this.$item.journalLevel) {
                    case 0:
                        return 'enterprise:balance';
                    default:
                        return 'account';
                }

            },
            subIcon: 'plus'
        }
    }

    execute() {
        switch (this.$item.journalLevel) {
            case 0:
                return this._createJournal();
            default:
                return this._createAccount();
        }
    }

    _createAccount() {
        return new Promise(resolve => {
            ODA.createComponent(this.resolveUrl('odant-create-account-view.html'))
            .then(control => {
                let prefix = this._getLabel(this.$item, '');
                control.set('parentCode', prefix);

                return ODA.showDialog({
                    title: this.label,
                    subTitle: this.$item,
                    icon: this.$item.icon,
                    component: control
                })
            })
            .then(res => {
                if (res.code && !isNaN(res.code)) {
                    let label = `${res.parentCode}${res.code}`;
                    let name = 'A' + label.replace(/\./g, '');

                    this.$item._createClass(name)
                    .then(cls => {
                        cls.accountLabel = `${res.account}`;
                        cls.accountCode = res.code;
                        cls.accountActive = res.active;
                        cls.save()
                        .then(result => {
                            if (result) {
                                ODA.push("Action create class", {
                                    tag: "create-class",
                                    body: '"' + cls.name + '" was created',
                                    icon: this.$item.icon
                                });
                                resolve(cls);
                            }
                        })
                    })
                }
            })
        });
    }

    _createJournal() {
        return this.$item.Classes.then(clss => {
            return ODA.inputText({title: this.label, value: "Class " + (clss.length + 1)}).then(name => {
                return this.$item._createClass(name).then(cls => {
                    ODA.push("Action create class", {
                        tag: "create-class",
                        body: '"' + name + '" was created',
                        icon: this.$item.icon
                    });
                    return cls;
                })
            });
        })
    }

    _getLabel(cls, name) {
        if (cls && cls.journalLevel > 1) {
            return this._getLabel(cls.$owner, `${cls.accountCode}.${name}`)
        }

        return name;
    }

}).register();