(class extends odaClass {
    static get properties() {
        return {
            defaultViewName: 'documentation',
            icon: 'documents/books',
            itemTypeLabel() {
                return this.hasChildren ? 'library' : 'book';
            },
            async docItems() {
                try {
                    const items = await this.Items;
                    if (items.length > 0) {
                        return items;
                    }

                    return await this.getObjects();

                    // let objects = await this.getObjects();
                    // let p = objects.map(obj => obj.label);
                    // let subs = await Promise.all(p);
                    // let collator = new Intl.Collator(undefined, {numeric: true, sensitivity: 'base'});
                    // objects.sort((a, b) => {
                    //     let x = a.label.toLowerCase();
                    //     let y = b.label.toLowerCase();
                    //     return collator.compare(x, y);
                    // });
                    //
                    // return objects;

                } catch (e) {
                    return []
                }

            },
            async aboutItems() {
                try {
                    const items = await this.Items;
                    const structure = await this.getFile('book/structure.json');
                    const rootFile = await structure.loadRootFile();
                    return [rootFile , ...items]
                } catch (err) {
                    console.error(err);
                    return []
                }
            },
            DefaultContent() {
                let html = document.implementation.createHTMLDocument();
                let meta = html.createElement('meta');
                meta.setAttribute("charset", "UTF-8");
                html.head.appendChild(meta);
                return html.documentElement.outerHTML;
            }
        }
    }

    async nextItem(item, recurs) {
        try {
            let items = await this.Items;
            if (items.length === 0) {
                items = await this.getObjects();
            }
            if (items.length > 0) {
                if (!item) {
                    return {
                        parent: this,
                        item: items[0]
                    }
                } else {
                    let idx = items.findIndex(i => i.id === item.id);
                    if(items[idx + 1]) {
                        return {
                            parent: this,
                            item: items[idx + 1]
                        };
                    }
                }
            }
            if(!recurs) {
                return null;
            }
            let parent = await this.getParentItem();
            return parent.nextItem(this, true)

        } catch (e) {
            return null
        }
    }

    getFirstObject() {
        return this.XQuery('string(functx:first-node(x/O)/@oid)')
            .then(result => {
                if (!result || !result[0]) {
                    throw new Error('Last object not found')
                }

                return this.getObject(result[0]);
            })
    }

    async createObject(label) {
        try {
            const json = await this.API.POST('create_object', {});
            const obj = await this.getObject(json.$OBJECT[0].oid);
            obj.Root.name = label;
            await obj.save();
            return this.async(async () => {
                let root = {
                    label: obj.name,
                    name: 'book',
                    locale: ODANT.language,
                    author: this.userId,
                    items: [],
                    locals: [
                        {
                            language: ODANT.language,
                            label: obj.name,
                            author: this.userId,
                        }
                    ]
                };

                const folder = await obj.getFolder('book', true);
                const structure = await  folder.getFile('structure.json', JSON.stringify(root));
                const folderLang = await folder.getFolder(ODANT.language, true);


                let html = document.implementation.createHTMLDocument();
                let meta = html.createElement('meta');
                meta.setAttribute("charset", "UTF-8");
                html.head.appendChild(meta);
                const bookFile = await folderLang.getFile('book.html', html.documentElement.outerHTML);

                this.fire('create', obj);
                ODA.push("chapter created", {
                    tag: "create-object",
                    body: 'chapter "' + label + '" has been created',
                    icon: this.icon
                });
                return obj;
            }, 1000);
        } catch (error) {
            console.error(error);
        }
    }

    async getParentItem() {
        return this.$owner
    }
    async hasDocChildren(){
        return this.hasChildren ||  !!(await this.getObjectCount('*'))
    }

}).register();


