(class extends odaFile {
    static get properties() {
        return {
            defaultViewName() {
                if (this.isArticle) {
                    return 'documentation';
                }
                return this.super('defaultViewName');
            },
            icon() {
                if (this.isArticle) {
                    return 'document'
                }
                return this.super('icon');
            },
            itemTypeLabel() {
                if (this.isArticle) {
                    return 'article'
                }
                return this.super('itemTypeLabel');
            },
            label: {
                get() {
                    if (!this.isArticle)
                        return this.id;
                    if (!this._label) {
                        this.getLabel().then(label => {
                            this._label = label;
                            this.fireChanged();
                            return label;
                        });
                        return 'Loading...';
                    }
                    return this._label;
                },
                set(v) {
                    (async () => {
                        if (this.isArticle) {
                            const structure = await this.Structure;
                            this._label = await structure.setLabel(this.name, v);
                            this.fireChanged();
                        } else {
                            this.super('label', v);
                        }
                    })()
                }
            },
            Structure: {
                get() {
                    return (async () => {
                        if (this.json) {
                            return this.json
                        }
                        this.json = await this.$owner.$owner.getFile('structure.json');
                        return this.json
                    })()
                },
            },

            async Files() {
                const structure = await this.Structure;
                return structure.loadFiles(this);
            },
            async docItems() {
                try {
                    return await this.Files;
                } catch (err) {
                    return []
                }
            },
            async aboutItems() {
                return this.docItems;
            },
            DefaultContent: function () {
                let html = document.implementation.createHTMLDocument();
                let meta = html.createElement('meta');
                meta.setAttribute("charset", "UTF-8");
                html.head.appendChild(meta);
                return html.documentElement.outerHTML;
            },
            isArticle() {
                return this.$('is-article', () => {
                   return  this.$owner.$owner.name === 'book';
                })
            }
        }
    }

    async nextItem(item, recurs) {
        try {
            let items = await this.Files;
            if (items.length > 0) {
                if (!item) {
                    return {
                        parent: this,
                        item: items[0]
                    }
                } else {
                    let idx = items.findIndex(i => i.id === item.id);
                    if(items[idx + 1]) {
                        return {
                            parent: this,
                            item: items[idx + 1]
                        };
                    }
                }
            }
            if(!recurs) {
                return null;
            }
            let parent =  await this.getParentItem();
            return parent.nextItem(this, true);
        } catch (e) {
            return null
        }
    }

    async getLabel() {
        const structure = await this.Structure;
        const label = await structure.getFileLabel(this.name);
        return label ? label : this.name;
    }

    async rename(name) {
        if (this.isArticle) {
            this.label = name;
        } else {
            await super.rename(name);
        }
    }

    async remove() {
        if (this.isArticle) {
            const structure = await  this.Structure;
            const root = await structure.findFileAsync(this, structure.json);
            if (root) {
                root.items = root.items || [];
                if (root.items.length > 0) {
                    const folder = await structure.getFolderLang();
                    let files = root.items.map(i => folder.getFile(i.name + '.html'));
                    let list = await Promise.all(files);
                    list.forEach(i => i.remove());
                }
                const success = await structure.removeFileStructure(this.name);
                if (success) {
                    super.remove();
                }
            }
        } else {
            super.remove();
        }
    }


    async getParentItem() {
        let structure = await this.Structure;
        return structure.getParentItem(this);
    }

    async hasDocChildren() {
        const structure = await this.Structure;
        return await structure.hasDocChildren(this);
    }

    async execute() {
        if (this.isArticle) {
            const structure = await this.Structure;
            const data = await structure.findFileAsync(this);
            if (!data.readOnly) {
                this.show('docs-editor');
            }
        } else {
            super.execute();
        }
    }
}).register();