(class extends odaFile {
    static get properties() {
        return {
            Json: {
                set(v) {
                    this._json = v;
                    return this.save(JSON.stringify(v));
                },
                get() {
                    return (async () => {
                        if (this._json) {
                            return this._json;
                        } else {
                            this._json = await this.load();
                            return this._json
                        }
                    })();
                }
            },
            async docItems() {
                await this.Json();
                return this.loadFiles();
            }
        }
    }

    getFolderLang(lang = ODANT.language) {
        return this.$('lang-folder: ' + lang, () => this.$owner.getFolder(lang, true));
    }

    async loadFiles(file) {
        const json = await this.Json;
        const data = this.findFile(file, json);
        const items = data && data.items || [];
        const map = items.map(item => this.getArticleFile(item));
        const res = await Promise.all(map);
        return res.filter(i => i);
    }

    async getArticleFile(item) {
        const getFile = async (locale) => {
            try {
                const folder = await this.getFolderLang(locale);
                const file = item.name && await folder.getFile(item.name + '.html');
                file.json = this;
                return file;
            }
            catch (e) {
                console.warn(e);
                if (!locale && (item.language || item.locale)) {
                    return getFile(item.language || item.locale)
                }
            }
        };
        return getFile();
    }

    async loadRootFile() {
        const json = await this.Json;
        let data = this.findFile('', json);
        return this.getArticleFile(data);
    }

    findFile(file, json) {
        let name = file instanceof odaFile ? file.name : null;
        let funcFind = (json, name) => JSON.find(json, i => {
            if (!name) {
                return i;
            }
            if (i.name === name) {
                return i;
            }
        });

        return funcFind(json, name);
    }

    async findFileAsync(file) {
        const json = await this.Json;
        return this.findFile(file, json);
    }

    async move(moveData, source, dropMode) {
        const json = await this.Json;
        let node = JSON.find(json, i => {
            if (i.name === moveData.name) {
                return i;
            }
        });
        if (node) {
            const success = await  this.removeFileStructure(moveData.name);

            if (success) {
                let dataNode = JSON.find(json, i => {
                    if (i.name === source.name) {
                        return i;
                    }
                });
                switch (dropMode) {
                    case 'in':
                        if (dataNode) {
                            dataNode.items = dataNode.items || [];
                            dataNode.items.push(node);
                        }
                        break;
                    case 'bottom':
                        if (dataNode) {
                            let parent = JSON.find(json, i => {
                                if (i.items && i.items.includes(dataNode)) {
                                    return i
                                }
                            });
                            let index = parent.items.indexOf(dataNode);
                            parent.items.splice(index + 1, 0, node)
                        }
                        break;
                    case 'top':
                        if (dataNode) {
                            let parent = JSON.find(json, i => {
                                if (i.items && i.items.includes(dataNode)) {
                                    return i
                                }
                            });
                            let index = parent.items.indexOf(dataNode);
                            parent.items.splice(index, 0, node)
                        }
                        break;
                }
                this.Json = json;
            }
        }
    }

    async removeFileStructure(name) {
        const json = await this.Json;
        let node = JSON.find(json, i => {
            if (i.name === name) {
                return i
            }
        });
        if (node) {
            let parent = JSON.find(json, i => {
                if (i.items && i.items.includes(node)) {
                    return i
                }
            });
            let index = parent.items.indexOf(node);
            parent.items.splice(index, 1);
            this.Json = json;
            return true
        }
    }

    async getFileLabel(name) {
        try {
            const json = await this.Json;
            let root = JSON.find(json, i => {
                if (i.name === name) {
                    return i;
                }
            });
            if (root) {
                if ((ODANT.language.includes(root.locale || root.language) || (root.locale && root.locale.includes(ODANT.language))
                    || root.language && root.language.includes(ODANT.language)) && root.label) {
                    return root.label;
                }
                let locale = root.locals.find(i => {
                    if (ODANT.language.includes(i.language) || i.language.includes(ODANT.language)) {
                        return i;
                    }
                });
                if (locale) {
                    return locale.label;
                } else {
                    return root.label;
                }
            }
        } catch (err) {
            console.error(error);
            return name;
        }
    }

    async setLabel(fileLabel, value) {
        try {
            const json = await this.Json;
            let root = JSON.find(json, i => {
                if (i.name === fileLabel) {
                    return i;
                }
            });
            if (root) {
                if ((ODANT.language.includes(root.locale || root.language) || (root.locale && root.locale.includes(ODANT.language))
                    || root.language && root.language.includes(ODANT.language)) && root.label) {
                    root.label = value;
                }
                let locale = root.locals.find(i => {
                    if (ODANT.language.includes(i.language) || i.language.includes(ODANT.language)) {
                        return i;
                    }
                });

                if (locale) {
                    locale.label = value;
                }
            }
            this.Json = json;
            return value
        } catch (err) {
            console.error(error);
        }
    }

    async addFilToStructure(ownerFileName, fileName, label) {
        const json = await this.Json;
        let root = JSON.find(json, i => {
            if (i.name === ownerFileName) {
                return i;
            }
        });
        if (root) {
            root.items = root.items || [];
            let data = {
                name: fileName, language: ODANT.language, author: this.userId,
                locals: [{
                    language: ODANT.language,
                    label: label,
                    author: this.userId
                }]
            };
            root.items.push(data);
            this.Json = json;
            return true;
        } else {
            throw new Error('failed to add file in json');
        }
    }

    async getParentItem(file) {
        let json = await this.Json;
        let article = this.findParent(json, (i, parent) => {
            if (i.name === file.name) {
                return parent
            }
        }, null);

        if (article) {
            if (article.name === 'book') {
                return file.$dataOwner;
            } else {
                return this.getArticleFile(article);
            }
        } else {
            return null;
        }
    }


    findParent(json, callback, parent) {
        let res = callback(json, parent);
        if (!res) {
            res = json.items && json.items.find(i => {
                return this.findParent(i, callback, json);
            });
            if (res)
                res = this.findParent(res, callback, json);
        }
        return res;
    }

    async hasDocChildren(name) {
        const json = await this.Json;
        const data = this.findFile(name, json);
        const items = data && data.items || [];
        return items.length > 0;
    }

}).register();