(class extends odaAction {
    async execute(params) {
        try {
            console.log('Start HTML to PDF');
            console.log(`Params: ${JSON.stringify(params)}`);

            if (!params.cls || !params.oids)
                return;

            const cls = await this.$owner.findItem(params.cls);
            if (!cls) {
                console.log(`Can't find class`);
                return;
            }

            console.log(`ActionSource = ${this.ActionSource.url}`);
            console.log(`ActionSource.access = ${JSON.stringify(this.ActionSource.access)}`);

            console.log(`cls = ${cls.url}`);
            console.log(`cls.access = ${JSON.stringify(cls.access)}`);

            const fields = await cls.fields;
            console.log(`fields = ${fields}`);

            const fieldsLabels = fields.map(f => f.label)
            console.log(`fieldsLabels = ${fieldsLabels}`);

            const visibleFields = fields.map(f => f.name)
            console.log(`visibleFields = ${visibleFields}`);

            const attrs = visibleFields.reduce((accumulator, currentValue) => {
                const xq = `attribute ${currentValue} { if (@${currentValue}) then @${currentValue} else ' ' }`;
                return accumulator + (accumulator && accumulator.length > 0)
                    ? accumulator + ', ' + xq
                    : xq;
            }, '');
            console.log(`attrs = ${attrs}`);

            const filter = (params.oids === '*') ? 'x/O' : `oda:select(x/O, 'oid', '${params.oids}', ' ')`
            const xq = `for $a in ${filter} return element O { $a/(${attrs}) }`;
            console.log(`xq = ${xq}`);

            const url = `${cls.url}/I:table`;
            console.log(`url = ${url}`);

            const json = await ODANT.API.post(url, { method: 'xquery' }, xq);

            console.log(`json = ${json}`);
            console.log(`stringify = ${JSON.stringify(json)}`);

            let html = this.getHtmlHead();

            html += fieldsLabels.reduce((acc, cur, i, arr) => this.getFieldsTags(acc, cur, i, arr), '');
            html += `</tr>`;
            html += json.$O.reduce((acc, cur) => {
                const rowsTags = visibleFields.reduce((accF, curF, iF) => {
                    const cls = (iF === 0) ? 'td-left' : 'td-right';
                    return accF + `<td class="${cls}">${cur[curF]}</td>`
                }, '');
                return acc + `<tr>${rowsTags}</tr>`
            }, '');
            html += `</table></body></html>`

            console.log(`html = ${html}`);

            const htmlPdf = await import('html-pdf-chrome');
            const options = {
                port: 9222, // port Chrome is listening on
              };
              
            const pdf = await htmlPdf.default.create(html, options).catch((err) => console.error(err));
            const base64 = pdf && pdf.toBase64();
            cls.saveFile(base64, 'test.pdf');

            //return pdf && pdf.toBuffer();
            return base64;
        } catch (e) {
            console.log(e);
            return null;
        }
    }

    getFieldsTags(acc, cur, i, arr) {
        const cls = (i === 0) ? 'th-left' : 'th-right';
        return acc + `<th class="${cls}">${cur}</th>`
    }

    getHtmlHead() {
        return `<!DOCTYPE html>
<html>
    <head>
    <mate charest="utf-8" />
    <title></title>
    <style>
        .th-right {
        border-bottom: 1px solid black;
        border-top:    1px solid black;
        border-right:  1px solid black;
        }

        .th-left {
        border-bottom: 1px solid black;
        border-top:    1px solid black;
        border-left:   1px solid black;
        border-right:  1px solid black;
        }

        .td-right {
        border-bottom: 1px solid black;
        border-right:  1px solid black;
        }

        .td-left {
        border-bottom: 1px solid black;
        border-left:   1px solid black;
        border-right:  1px solid black;      
        }
    </style>
    </head>
    <body>
    <table align="center" cellspacing="0" cellpadding="5">
        <tr>`;
    }
}).register();