(class extends odaAction {
    async execute(params) {
        try {
            console.log('Start JSON to CSV');
            console.log(`$owner = ${this.$owner.url}`);
            console.log(`Params: ${JSON.stringify(params)}`);

            if ((!params.cls || !params.oids) && (!params.objId || !params.table))
                return;

            console.log(`ActionSource = ${this.ActionSource.url}`);
            console.log(`ActionSource.access = ${JSON.stringify(this.ActionSource.access)}`);


            let fields;
            let item = await this.$owner.$base.findItem(params.cls);
            if (!item) {
                console.log(`Can't find class`);
                return;
            }

            if (params.objId) {
                item = await item.getObject(params.objId);
                if (!item) {
                    console.log(`Can't find object`);
                    return;
                }

                fields = await item.$class.fields;
                const table = fields.find((f) => f.name === params.table);
                fields = await table.fields;

            } else {                
                fields = await item.fields;
            }

            const paramFields = params.fields && params.fields.split('|');
            if (paramFields) fields = fields.filter((fld => paramFields.includes(fld.name)))
            console.log(`fields = ${fields}`);

            const fieldsLabels = fields.reduce((xq, f) => {
                const xpath = `attribute ${f.name} { '${f.label}' }`;
                return xq + (xq && xq.length > 0)
                    ? xq + ', ' + xpath
                    : xpath;
            }, '')
            console.log(`fieldsLabels = ${fieldsLabels}`);

            const visibleFields = fields.map(f => f.name)
            console.log(`visibleFields = ${visibleFields}`);

            const attrs = visibleFields.reduce((xq, name) => {
                const xpath = `attribute ${name} { if (@${name}) then @${name} else ' ' }`;
                return xq + (xq && xq.length > 0)
                    ? xq + ', ' + xpath
                    : xpath;
            }, '');
            console.log(`attrs = ${attrs}`);

            let json;
            if (params.objId) {
                const xq = `for $a in OBJECT/${params.table} return element O { $a/(${attrs}) }`;
                console.log(`xq = ${xq}`);

                json = await ODANT.API.post(item.url, { method: 'xquery' }, xq);
            } else {
                const filter = (params.oids === '*') ? 'x/O' : `oda:select(x/O, 'oid', '${params.oids}', ' ')`
                const xq = `element O { ${fieldsLabels} }, for $a in ${filter} return element O { $a/(${attrs}) }`;
                console.log(`xq = ${xq}`);

                const url = `${item.url}/I:table`;
                console.log(`url = ${url}`);

                json = await ODANT.API.post(url, { method: 'xquery' }, xq);
            }

            console.log(`json = ${json}`);
            console.log(`stringify = ${JSON.stringify(json)}`);

            const converter = await import('json-2-csv');

            const options = {
                delimiter: {
                    wrap: '"', // Double Quote (") character
                    field: ';', // Comma field delimiter
                    eol: '\n' // Newline delimiter
                },
                excelBOM: true,
                prependHeader: false
            }

            const csv = converter.default.json2csvAsync(json && json.$O, options)
                .catch((err) => console.error(err));

            return csv;
        } catch (e) {
            console.log(e);
            return null;
        }
    }
}).register();