(class extends odaAction {
    async execute(params) {
        try {
            console.log('Start JSON to XLSX');
            console.log(`$owner = ${this.$owner.url}`);
            console.log(`Params: ${JSON.stringify(params)}`);

            if (!params.cls || (!params.oids && (!params.objId || !params.table)))
                return;

            console.log(`ActionSource = ${this.ActionSource.url}`);
            console.log(`ActionSource.access = ${JSON.stringify(this.ActionSource.access)}`);


            let fields;
            let item = await this.$owner.$base.findItem(params.cls);
            if (!item) {
                console.log(`Can't find class`);
                return;
            }

            if (params.objId) {
                item = await item.getObject(params.objId);
                if (!item) {
                    console.log(`Can't find object`);
                    return;
                }

                fields = await item.$class.fields;
                const table = fields.find((f) => f.name === params.table);
                fields = await table.fields;

            } else {                
                fields = await item.fields;
            }

            console.log(`item = ${item.url}`);
            console.log(`item.access = ${JSON.stringify(item.access)}`);

            const paramFields = params.fields && params.fields.split('|');
            if (paramFields) fields = fields.filter((fld => paramFields.includes(fld.name)));
            console.log(`fields = ${fields}`);

            let hasTypes = false;
            const fieldsType = await Promise.all(fields.map(async (f) => {
                let odaType = 'string';
                let typeCls;
                if (f.type) {
                    typeCls = await f.TypeClass;
                    if (typeCls) {
                        switch (typeCls.name.toLowerCase()) {
                            case 'bool':
                                odaType = 'bool';
                                hasTypes = true;
                                break;
                            case 'numeric':
                                odaType = 'number';
                                break;
                            default:
                                break;
                        }
                    }
                }
                return { name: f.name, label: f.label, type: odaType };
            }));

            const fieldsObject = fieldsType.reduce((obj, fld) => {
                obj[fld.name] = fld.type;
                return obj;
            }, {});

            console.log(`hasTypes = ${hasTypes}`);
            console.log(`fieldsObject = ${JSON.stringify(fieldsObject)}`);
            console.log(`fieldsType = ${JSON.stringify(fieldsType)}`);

            const fieldsName = fieldsType.map(f => f.label || f.name)
            console.log(`fieldsName = ${fieldsName}`);

            let attrs;
            await Promise.all(fieldsType.map(async (f) => {
                const xq = `attribute ${f.name} { if (@${f.name}) then @${f.name} else ' ' }`;
                attrs = attrs && (attrs.length > 0)
                        ? attrs + ', ' + xq
                        : xq;
                return ''
            }));
            console.log(`attrs = ${attrs}`);

            let json;
            if (params.objId) {
                const xq = `for $a in OBJECT/${params.table} return element O { $a/(${attrs}) }`;
                console.log(`xq = ${xq}`);

                json = await ODANT.API.post(item.url, { method: 'xquery' }, xq);
            } else {
                const filter = (params.oids === '*') ? 'x/O' : `oda:select(x/O, 'oid', '${params.oids}', ' ')`
                const xq = `for $a in ${filter} return element O { $a/(${attrs}) }`;
                console.log(`xq = ${xq}`);

                const url = `${item.url}/I:table`;
                console.log(`url = ${url}`);

                json = await ODANT.API.post(url, { method: 'xquery' }, xq);
            }

            console.log(`stringify = ${JSON.stringify(json)}`);

            const json2xlsx = await import('node-json-xlsx');
            const data = (!hasTypes)
                         ? json && json.$O
                         : json && json.$O.map((i) => {
                                Object.keys(i).forEach((attr) => {
                                    const typeObj = fieldsType.find(o => o.name === attr);
                                    if (typeObj) {
                                        switch (typeObj.type) {
                                            case 'bool':
                                                i[attr] = i[attr] === 'True';
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                });
                                return i;
                           });

            console.log(`data.stringify = ${JSON.stringify(data)}`);

            let xlsx
            try {
                xlsx = json2xlsx.default(data, { fields: fieldsObject, fieldNames: fieldsName});
            } catch (e) {
                console.log(e);
            }

            return xlsx;
        } catch (e) {
            console.log(e);
            return null;
        }
    }
}).register();