(class extends odaService {
    static get properties() {
        return {
            allowAccess: 'R',
            allowExport: true,
            allowUse: true,
            icon: 'odant:grid',
            isActive: true,
            label: 'Export from table',
            title: 'Export from table to XLSX',
            subMenu: function () {
                return [
                    {
                        'icon': 'files:xlsx',
                        'label': 'Export from table to XLSX',
                        'execute': (e) => this.execute(e)
                    },
                    {
                        'icon': 'files:csv',
                        'label': 'Export from table to CSV',
                        'execute': (e) => this.exportToCsv(e)
                    },
                    {
                        'icon': 'files:xml',
                        'label': 'Export from table to XML',
                        'execute': (e) => this.exportToXml(e)
                    }
                ];
            }
        }
    }

    async chooseTable(item) {
        const fields = (await (item.fields || item.$class.fields)).filter((fld) => fld && fld.isTable);
        if (fields && fields.length === 1)
            return fields[0];

        const listbox = await ODA.createComponent('/web/lib/lists/listbox/listbox.html');
        listbox.contextItem = item;
        listbox.items = fields;

        const dialog = {
            title: 'Select table field',
            icon: 'odant:grid',
            ok: {
                show: true,
                disable: false
            }
        };
        const dialogResult = await ODA.showDialog(listbox, dialog).catch((e) => {});
        if (dialogResult) {
            return listbox.focusedItem;
        }

        return null;
    }

    async chooseFields(item) {
        const fields = (await (item.fields || item.$class.fields)).filter((fld) => fld && !fld.getBoolAttr('HideOnGrid'));

        const url = this.resolveUrl('../../../lib/filter-listbox/filter-listbox.html');
        const listbox = await ODA.createComponent(url);
        listbox.contextItem = item;
        listbox.comboMode = true;
        listbox.items = [...fields];
        listbox.selection = [...fields];

        const dialog = {
            title: 'Select fields',
            icon: 'odant:fields',
            ok: {
                show: true,
                disable: false
            }
        };
        const dialogResult = await ODA.showDialog(listbox, dialog).catch((e) => {});
        if (dialogResult) {
            return listbox.selection;
        }

        return null;
    }

    async execute(params) {
        const table = await this.chooseTable(this.$owner);
        if (!table) return;

        const fields = await this.chooseFields(table);
        if (!fields) return;

        const paramFields = fields && fields.reduce((acc, fld) => { 
            if (acc && (acc.length > 0)) acc += '|';
            return acc + fld.name;
        }, '');

        const actionSource = await this.$owner.$base.findItem('M:1D35C51D39FF410/C:1D5A8D25F133B8A');

        try {
            const result = await ODANT.API.post(actionSource.url,
                { execute: 'methods/json-to-xlsx', cls: this.$owner.$class.id, objId: this.$owner.id, fields: paramFields, table: table.name });

            const l = result.length;
            const array = new Uint8Array(l);
            for (let i = 0; i < l; i++) {
                array[i] = result.charCodeAt(i) & 0xFF;
            }

            this.openFile(array, this.$owner, 'xlsx');
        } catch (e) {
            console.error(e);
        }
    }

    async exportToCsv(params) {
        const table = await this.chooseTable(this.$owner);
        if (!table) return;

        const fields = await this.chooseFields(table);
        if (!fields) return;

        const paramFields = fields && fields.reduce((acc, fld) => { 
            if (acc && (acc.length > 0)) acc += '|';
            return acc + fld.name;
        }, '');

        const actionSource = await this.$owner.$base.findItem('M:1D35C51D39FF410/C:1D5A8D25F133B8A');

        try {
            const result = await ODANT.API.post(actionSource.url,
                { execute: 'methods/json-to-csv', cls: this.$owner.$class.id, objId: this.$owner.id, fields: paramFields, table: table.name });

            const bytes = new TextEncoder().encode(result);
            this.openFile(Uint8Array.from([239, 187, 191, ...bytes]), this.$owner, 'csv', 'text/plain;charset=UTF-8');
        } catch (e) {
            console.error(e);
        }
    }    

    async exportToXml(params) {
        const table = await this.chooseTable(this.$owner);
        if (!table) return;

        const fields = await this.chooseFields(table);
        if (!fields) return;

        const paramFields = fields && fields.reduce((acc, fld) => { 
            if (acc && (acc.length > 0)) acc += '|';
            return acc + fld.name;
        }, '');

        try {
            const xq = `element x { for $r in OBJECT/${table.name} return element O { for $a in $r/@*[contains('${paramFields}', name())] return $a } }`;
            const xml = '<?xml version="1.0" encoding="UTF-8" ?>' + await ODANT.API.post(this.$owner.url, { method: 'xquery', format: 'xml' }, xq);

            console.log(`xml = ${xml}`);

            this.openFile(xml, this.$owner, 'xml');
        } catch (e) {
            console.error(e);
        }
    }

    openFile(data, $owner, ext, type = '') {
        const blob = new Blob([data], { encoding: "UTF-8", type: type })
        const link = document.createElement('a');
        const linkHref = window.URL.createObjectURL(blob);
        link.setAttribute('href', linkHref);
        link.setAttribute('download', `${$owner.$class.label}.${ext}`);
        link.click();
    }
}).register();