(class extends odaService {

    static get properties() {
        return {
            allowAccess: 'R',
            allowArrayContext: true,
            allowExport: true,
            allowUse: true,
            icon: 'files:xlsx',
            isActive: true,
            label: 'Export',
            title: 'Export to XLSX',
            subMenu: function () {
                return [
                    {
                        'icon': 'files:xlsx',
                        'label': 'Export to XLSX',
                        'execute': (e) => this.execute(e)
                    },
                    {
                        'icon': 'files:csv',
                        'label': 'Export to CSV',
                        'execute': (e) => this.exportToCsv(e)
                    },
                    // {
                    //     'icon': 'files:pdf',
                    //     'label': 'Export to PDF',
                    //     'execute': (e) => this.exportToPdf(e)
                    // },
                    {
                        'icon': 'files:xml',
                        'label': 'Export to XML',
                        'execute': (e) => this.exportToXml(e)
                    }
                ];
            }
        }
    }

    async chooseFields(item, selected) {
        const fields = (await (item.fields || item.$class.fields)).filter((fld) => fld && !fld.getBoolAttr('HideOnGrid'));

        const url = this.resolveUrl('../../../lib/filter-listbox/filter-listbox.html');
        const listbox = await ODA.createComponent(url);
        listbox.contextItem = item;
        listbox.comboMode = true;
        listbox.items = [...fields];
        const selectedFields = (selected) ? fields.filter((f) => selected.includes(f.name)) : fields;
        listbox.selection = [...selectedFields];

        const dialog = {
            title: 'Select fields',
            icon: 'odant:fields',
            ok: {
                show: true,
                disable: false
            }
        };
        const dialogResult = await ODA.showDialog(listbox, dialog).catch((e) => {});
        if (dialogResult) {
            return listbox.selection;
        }

        return null;
    }

    async chooseTemplate(item) {
        const url = this.resolveUrl('../../../lib/template-selector/export-template-selector.html');
        const selector = await ODA.createComponent(url);
        selector.fromTable = false;
        selector.contextItem = item.$class;

        const dialog = {
            title: 'Select template',
            icon: 'image:center-focus-strong',
            dblclick: true,
            ok: {
                show: true,
                disable: false
            },
            buttons: [
                { label: 'New', icon: 'icons:add', special: 'add' },
                { label: 'Remove', icon: 'icons:remove', special: 'del' },
                { label: 'Settings', icon: 'icons:settings', special: 'set' },
                { label: 'OK', special: 'ok' }
            ]
        };
        const dialogResult = await ODA.showDialog(selector, dialog);
        if (!dialogResult.button) {
            return dialogResult.focusedItem && dialogResult.focusedItem.item;
        } else {
            switch (dialogResult.button.special) {
                case 'add':
                    const exportClsPath = "M:1D35C51D39FF410/C:1D5A8D25F133B8A";
                    const exportCls = await item.$base.findItem(exportClsPath);
                    
                    const tmpName = await ODA.inputText({ title: 'Input template name', value: 'Export from ' + item.label });

                    const fields = await this.chooseFields(item);
                    if (!fields) return undefined;
                    
                    const newTmpObj = await exportCls.createObject();
                    if (newTmpObj.Root) {                        
                        newTmpObj.Root.name = tmpName;
                        newTmpObj.Root.source = item.$class.label;
                        newTmpObj.Root.$source = [{ link: exportCls.getRelativeId(item.$class) }];
                        const paramFields = fields.reduce((acc, fld) => { 
                            if (acc && (acc.length > 0)) acc += '|';
                            return acc + fld.name;
                        }, '');
                        newTmpObj.Root.fromTable = false;
                        newTmpObj.Root.fields = paramFields;
                        await newTmpObj.save();

                        return newTmpObj;
                    }
                    break;
                case 'del':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        if (await ODA.confirm('Are you sure?')) {
                            dialogResult.control.focusedItem.item.remove();
                        }
                    }
                    break;
                case 'set':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        const fields = await this.chooseFields(item, dialogResult.control.focusedItem.item.Root.fields);
                        if (fields) {
                            const paramFields = fields.reduce((acc, fld) => { 
                                if (acc && (acc.length > 0)) acc += '|';
                                return acc + fld.name;
                            }, '');
                            dialogResult.control.focusedItem.item.Root.fields = paramFields;
                            await dialogResult.control.focusedItem.item.save();
                        }
                    }
                    break;
                case 'ok':
                    return dialogResult.control.focusedItem && dialogResult.control.focusedItem.item;
                default:
                    alert('Я таких значений не знаю');
            }
        }
        return undefined;
    }

    async execute(params) {
        const oids = this.getOids(this.$owner);

        if (oids) {
            try {
                // TODO исправить ActionSource (подменяется на класс, от которого вызывается)
                const templateObject = await this.chooseTemplate(this.$owner);
                if (!templateObject) return;
                const actionSource = await this.$owner.$base.findItem('M:1D35C51D39FF410/C:1D5A8D25F133B8A');
                const clsId = this.$owner.$class.id;//actionSource.getRelativeId(this.$owner);
                const paramFields = templateObject.Root.fields;
                let result = await ODANT.API.post(actionSource.url,
                    { execute: 'methods/json-to-xlsx', cls: clsId, fields: paramFields, oids: oids });

                const l = result.length;
                const array = new Uint8Array(l);
                for (let i = 0; i < l; i++) {
                    array[i] = result.charCodeAt(i) & 0xFF;
                }

                this.openFile(array, this.$owner, 'xlsx');
            } catch (e) {
                console.error(e);
            }
        }
    }

    async exportToCsv(params) {
        const oids = this.getOids(this.$owner);

        if (oids) {
            try {
                // TODO исправить ActionSource (подменяется на класс, от которого вызывается)
                const templateObject = await this.chooseTemplate(this.$owner);
                if (!templateObject) return;
                const actionSource = await this.$owner.$base.findItem('M:1D35C51D39FF410/C:1D5A8D25F133B8A');
                const clsId = this.$owner.$class.id;//actionSource.getRelativeId(this.$owner);
                const paramFields = templateObject.Root.fields;
                const result = await ODANT.API.post(actionSource.url,
                    { execute: 'methods/json-to-csv', cls: clsId, fields: paramFields, oids: oids });

                const bytes = new TextEncoder().encode(result);
                //bytes.set([239, 187, 191], 0);
                this.openFile(Uint8Array.from([239, 187, 191, ...bytes]), this.$owner, 'csv', 'text/plain;charset=UTF-8');
            } catch (e) {
                console.error(e);
            }
        }
    }

    async exportToPdf(params) {
        const oids = this.getOids(this.$owner);

        if (oids) {
            try {
                // TODO исправить ActionSource (подменяется на класс, от которого вызывается)
                const templateObject = await this.chooseTemplate(this.$owner);
                if (!templateObject) return;
                const actionSource = await this.$owner.$base.findItem('M:1D35C51D39FF410/C:1D5A8D25F133B8A');
                const clsId = this.$owner.$class.id;//actionSource.getRelativeId(this.$owner);
                const paramFields = templateObject.Root.fields;
                let result = await ODANT.API.post(actionSource.url,
                    { execute: 'methods/html-to-pdf', cls: clsId, fields: paramFields, oids: oids });

                //const url = 'data:application/pdf;base64,' + result;
                const blobPdfFromBase64String = (base64String) => {
                    const byteArray = Uint8Array.from(
                        atob(base64String)
                            .split('')
                            .map(char => char.charCodeAt(0))
                    );
                    return new Blob([byteArray], { type: 'application/pdf' });
                };

                const link = document.createElement('a');
                const linkHref = window.URL.createObjectURL(blobPdfFromBase64String(result));
                link.setAttribute('href', linkHref);
                link.setAttribute('download', `${this.$owner.$class.label}.pdf`);
                link.click();
                // const l = result.length;
                // const array = new Uint8Array(l);
                // for (let i = 0; i < l; i++) {
                //     array[i] = result.charCodeAt(i);
                // }

                // this.openFile(array, this.$owner, 'pdf');
            } catch (e) {
                console.error(e);
            }
        }
    }

    async exportToXml(params) {
        const oids = this.getOids(this.$owner);

        if (oids) {
            try {
                const cls = await this.$owner.$class;
                const fields = await this.chooseFields(cls);
                console.log(`cls = ${cls.url}`);
                console.log(`cls.access = ${JSON.stringify(cls.access)}`);

                const xpath = fields && fields.reduce((xpath, fld) => `${xpath + (xpath ? ', @' : '@') + fld.name}`, '');
                let xq = (xpath) ? `element x { for $a in oda:select(x/O, 'oid', '${oids}', ' ') return element O { $a/(${xpath})} }`: `element x { oda:select(x/O, 'oid', '${oids}', ' ') }`;
                console.log(`xq = ${xq}`);

                if (await ODA.confirm(xq, 'Хотите изменить запрос?').catch(e => {})) {
                    const url = this.resolveUrl('../../../../../../../../../../web/lib/ace-editor/ace-editor.html');
                    const editor = await ODA.createComponent(url);
                    editor.mode = 'xquery';
                    editor.theme = 'eclipse';
                    editor.value = xq;
                    editor.style = 'width: 600px; height: 600px;';
    
                    const dialog = {
                        title: 'XQuery',
                        icon: 'icons:code'
                    };
                    const dialogResult = await ODA.showDialog(editor, dialog).catch((e) => {});
                    if (dialogResult) xq = editor.value;
                }

                const url = `${cls.url}/I:table`;
                console.log(`url = ${url}`);

                const xml = '<?xml version="1.0" encoding="UTF-8" ?>' + await ODANT.API.post(url, { method: 'xquery', format: 'xml' }, xq);

                console.log(`xml = ${xml}`);

                this.openFile(xml, this.$owner, 'xml');
            } catch (e) {
                console.error(e);
            }
        }
    }

    getOids(item) {
        let oids;
        if (item instanceof odaItemList) {
            oids = item.items.map(o => o.oid)
                .join(' ');
        } else if (item instanceof odaObject) {
            oids = item.id;
        }

        return oids;
    }

    openFile(data, $owner, ext, type = '') {
        const blob = new Blob([data], { encoding: "UTF-8", type: type })
        const link = document.createElement('a');
        const linkHref = window.URL.createObjectURL(blob);
        link.setAttribute('href', linkHref);
        link.setAttribute('download', `${$owner.$class.label}.${ext}`);
        link.click();
    }
}).register();