(class extends odaService {
    static get properties() {
        return {
            label: 'Import',
            isActive: true,
            allowArrayContext: true,
            allowAccess: 'C'
        }
    }


    execute() {
        return this.getSourceFile()
        .then(this.getValidationSourceSettings.bind(this))
        .then(this.validationSource.bind(this))
        .then(this.handlingValidationSourceResult.bind(this))
        .then(this.getRules.bind(this))
        .then(this.execRules.bind(this))
        .then(this.validationRulesResult.bind(this))
        .then(this.handlingValidationRulesResult.bind(this))
        .then(this.saveObjectsFromSourceRules.bind(this))
        .then(this.updateImportObjectInfo.bind(this))
        .then(this.responseWriter.bind(this))
        .catch(this.errorWriter.bind(this))
    }


    getSourceFile() {
        return this.$item.fields
        .then(fields => {
            const sourceField = fields.find(f => f.name === 'source');
            return sourceField.getValue(this.$item)
            .then(file => this.source = file)
        });
    }

    getValidationSourceSettings() {
        return this.$item.$class.$STATIC.$object.then(staticObj => {
            return this.$item.$class.$STATIC.fields.then(fields => {
                const validateField = fields.find(f => f.name === 'validate');
                const dsValidate = new DataSet(validateField, {dataRoot: staticObj.Root});

                return Promise.all([
                    dsValidate.metadata,
                    dsValidate.Items,
                ])
                .then(res => {
                    const meta = res[0];
                    const rows = res[1];

                    return Promise.all(
                        rows.map(r => {
                            const vs = {};

                            return Promise
                            .all(meta.map(f => {
                                return Promise.resolve(f.getValue(staticObj, r))
                                .then(fv => vs[f.name] = fv)
                            }))
                            .then(() => vs)
                        })
                    ).then(res => this.validationSourceSettings = res)
                })
            })
        })
    }

    validationSource() {
        const validationPromises = this.validationSourceSettings
        .filter(vs => vs.use && vs.xq)
        .map(vs => {
            return new Promise((resolve, reject) => {
                return this.source.API.POST('xquery', {format: 'XML'}, vs.xq || '*')
                .then(() => {
                    resolve();
                })
                .catch(err => {
                    if (err.status === 204) {
                        resolve();
                    }

                    vs.err = {
                        type: vs.name,
                        message: err.message
                    };

                    resolve();
                })
            });
        });

        if (validationPromises.length > 0) {
            return Promise.all(validationPromises)
        }

        return Promise.resolve();
    }

    handlingValidationSourceResult() {
        if (this.validationSourceSettings.some(s => s.use && s.required && s.err)) {
            throw new Error('VALIDATION_SOURCE_ERROR');
        }

        return Promise.resolve();
    }

    getRules() {
        return this.$item.$class.$STATIC.$object.then(staticObj => {
            return this.$item.$class.$STATIC.fields.then(fields => {
                const validateField = fields.find(f => f.name === 'rules');
                const dsRules = new DataSet(validateField, {dataRoot: staticObj.Root});

                return Promise.all([
                    dsRules.metadata,
                    dsRules.Items,
                ])
                .then(res => {
                    const meta = res[0];
                    const rows = res[1];
                    const cls = meta.find(f => f.name === 'cls');
                    const useField = meta.find(f => f.name === 'use');
                    const fields = meta.find(f => f.name === 'fields');
                    const xq = meta.find(f => f.name === 'xq');
                    const validation = meta.find(f => f.name === 'validate');

                    return Promise.all(
                        rows.map(r =>
                            Promise.all([
                                cls.getValue(staticObj, r),
                                xq.getValue(staticObj, r),
                                useField.getValue(staticObj, r),
                                this._getTableFields(staticObj, fields, r),
                                this._getTableFields(staticObj, validation, r)
                            ])
                            .then((res) => ({
                                cls: res[0],
                                xq: res[1],
                                use: res[2],
                                fields: res[3],
                                validations: res[4]
                            }))
                        )
                    )
                    .then(rules => this.rules = rules.filter(rule => rule.use))
                })
            })
        })
    }


    _getTableFields(staticObj, fields, r) {
        const ds = new DataSet(fields, {dataRoot: r});
        return Promise.all([
            ds.metadata,
            ds.Items,
        ])
        .then((res) => {
            const meta = res[0];
            const rows = res[1];

            return Promise.all(
                rows.map(r => {
                    const vs = {};

                    return Promise
                    .all(meta.map(f => {
                        return Promise.resolve(f.getValue(staticObj, r))
                        .then(fv => vs[f.name] = fv)
                    }))
                    .then(() => vs)
                })
            )
        });
    }

    execRules() {
        return this.$item.fields
        .then(fields => {
                const logField = fields.find(f => f.name === 'log');

                return logField.fields
                .then(fields => {
                    let sourceField = fields.find(f => f.name === 'source');

                    return Promise.all(this.rules
                    .map(rule => this._execRule({rule, logField, sourceField})))
                    .then(() => {
                        return this.$item.save()
                    })
                });
            }
        )
    }

    _execRule(param) {
        return new Promise((resolve, reject) => {
            return this.source.API.POST('xquery', {
                'format': 'XML',
                'stream': {
                    'gzip': false
                }
            }, this._buildRuleQuery(param.rule))
            .then(xqResultStream => {
                let row = this.$item.addRow(param.logField.name);
                return param.sourceField.setValue(this.$item, new File({
                    'name': 'result.xml',
                    'stream': xqResultStream
                }), row)
                .then((file) => {
                    console.log(this.$item);
                    param.rule.source = file;
                    resolve();
                });


                // return lk.$class.findField('MyAdresses/Address')
                //     .then(fieldAddress =>
                //         fieldAddress.setValue(lk, this.$item, lk.addRow('MyAdresses'))
                //             .then(() => lk.save())
                //     );

                // let newRow = param.logField.addRow({odaObject: this.$item});
                // return Promise.all([
                //     param.sourceField.setValue(this.$item, new File({
                //         'name': 'result.xml',
                //         'stream': xqResultStream
                //     }), newRow),
                //
                // ])
                //     .then((file) => {
                //         param.rule.source = file;
                //         let d = this.$item;
                //         resolve();
                //     });


            })
            .catch(err => {
                rule.err = err;
                resolve();
            })
        });
    };


    _buildRuleQuery(rule) {
        if (!rule.xq && rule.fields.length === 0) {
            throw new Error('rule.xq and rule.fields not found');
        }

        //merge xq with fields
        if (rule.xq && rule.fields.length > 0) {
            let fieldsXMLTree = this._buildFieldsXML(rule.fields);

            return `
                                let $source := ${qr.xqResult}
                                let $host := 'H:${qr.destination.$host.id}'
                                let $classFullId := '${qr.destination.fullId}'
                                
                                let $src := if ($source/name() = DATASET)
                                                then 
                                                    DATASET/DATA/*
                                                else 
                                                    $source
                                                    
                                let $addedObjCount := count($source/OBJECT[not(@oid)])
                                let $updatedObjCount := count($source/OBJECT[@oid])
                                let $oids := oda:gen-oids($host, $addedObjCount)

                             let $addedObjects := 
                                <OBJECTS>
                                   {
                                         for $r at $i in $src/*[not(@oid)]
                                         ${objectXML}
                                   },
                                </OBJECTS>
                                
                                let $updatedObjects := 
                                <OBJECTS>
                                   {
                                         for $r at $i in $src/*[(@oid)]
                                         ${objectXML}
                                   },
                                </OBJECTS>     
                                


                                $res := 
                                 <OBJECTS>
                                    {
                                        $addedObjects/*,
                                        $updatedObjects/*,                         
                                    }
                                  </OBJECTS>
 
                                  return element result {
                                    attribute addedObjCount {$addedObjCount},
                                    attribute updatedObjCount {$updatedObjCount},
                                    $res
                                  }      
                                    
                         `;
        }

        //only xq
        if (rule.xq && rule.fields.length === 0) {
            let xqueryExpression =
                `
                        let $hostFullId := 'H:${this.$item.$host.id}'
                        
                        let $source := doc(concat('<?xml version="1.0"?>', oda:command('xquery?id=
                        ${this.source.fullId.replace('~/', 'F:')}',"${rule.xq.replace(/"/g, '&#34;')}")))    
                       
                        let $src := if ($source/name() = DATASET)
                        then DATASET/DATA/*
                        else $source
                                                        
                        let $objectCountWithoutOID := count($src//*[not(@oid)])
                        let $oids := oda:gen-oids($hostFullId, $objectCountWithoutOID)	
                        
                           let $objects := 
                           <OBJECTS>
                           {
                                for $r at $i in $src//OBJECT
                                return element OBJECT {
                                    attribute oid {if ($r/@oid) then $r/@oid else $oids[$i]},
                                    $r/@*[not((name() = 'oid'))],
                                    $r/*
                                }
                            }			   
                         </OBJECTS>														
                        
                        return $objects
                    `
            ;
            console.log(xqueryExpression);
            return xqueryExpression;
        }

        //only fields
        return '';
    }

    _buildFieldsXML(source) {
        let result = [];

        source.forEach(function (a, idx) {
            let keys = a.src.split('/');
            let last = keys.pop();

            keys
            .reduce(function (r, k) {
                if (!r[k]) {
                    r[k] = {_: []};
                    r._.push({name: k, children: r[k]._});
                }
                return r[k];
            }, this)._.push({name: last.split('@')[1], source: source[idx].dest});
        }, {_: result});

        return [...result.filter((a) => !a.children)]
        .map(a => `attribute ${a.name} {$src/${a.source}}`, '')
        .join(',');
    }

    validationRulesResult() {
        return Promise.all(this.rules
        .map(rule => this._validationRuleResult(rule)));
    }

    _validationRuleResult(rule) {
        const validationPromises = rule.validations
        .filter(v => v.use && v.xq)
        .map(v => {
            return new Promise((resolve, reject) => {
                return rule.source.API.POST('xquery', {format: 'XML'}, v.xq)
                .then(() => {
                    resolve();
                })
                .catch((err) => {
                    if (err.status === 204) {
                        resolve();
                    }

                    v.err = {
                        type: v.name,
                        message: err.message
                    };

                    resolve();
                })
            });
        });

        if (validationPromises && validationPromises.length > 0) {
            return Promise.all(validationPromises);
        } else {
            return Promise.resolve();
        }
    }

    handlingValidationRulesResult() {
        const hasValidationRulesError = this.rules
        .some(r => r.validations.some(s => s.use && s.required && s.err));

        if (hasValidationRulesError) {
            throw new Error('VALIDATION_RULES_RESULT_ERROR');
        }

        return Promise.resolve();
    }

    saveObjectsFromSourceRules() {
        return Promise.all(
            this.rules.map(rule => this._saveObjectsFromSourceRule(rule)
            .then(res => rule.result = res))
        )
        .then(this._saveObjectsLink.bind(this));
    }

    _saveObjectsFromSourceRule(rule) {
        return new Promise((resolve, reject) => {
            const xq =
                `   
                        let $classFullId := '${rule.cls.fullId}'                 
                        let $doc := doc(oda:command('get_file?id=${rule.source.fullId.replace('~/', 'F:')}'))
                        let $saveDocs := oda:command(concat('save_objects?id=', $classFullId, '&amp;async=false'), oda:xml($doc))
                        return $saveDocs  
                    `
            ;

            return rule.cls.XQuery(xq)
            .then(res => resolve(Boolean(res)))
        })
    }

    _saveObjectsLink() {
        let xqHeader = '';

        if (this.rules && this.rules.length > 0) {
            xqHeader = this.rules.reduce((a, rule) =>
                a += `<source class-fullid='${rule.cls.$base.getRelativeId(rule.cls)}' class-name='${rule.cls.label}' file='${rule.source.fullId.replace('~/', 'F:')}'/>`, '')
        }

        if (xqHeader.length > 0) {
            let xq = `
                    let $sources := 
                        <sources>
                            ${xqHeader}
                        </sources>
                    
                    let $classFullId := '${this.$item.$class.fullId}'    
                    let $obj := doc('${this.$item.fullId}')/OBJECT   
         
                      let $newObj := 
                        element OBJECTS{
                            element OBJECT{
                              $obj/@*,
                              $obj/*[not((name() = 'log'))],
                        
                                for $source in $sources/source
                                let $loadedDoc := doc(oda:command(concat('get_file?id=', $source/@file)))
                                
                                return element log{
                                                attribute cls {$source/@class-name},
                                                attribute objs {count($loadedDoc/OBJECTS/OBJECT)},
                                                element cls{ 
                                                    attribute link {$source/@class-fullid}
                                                },
                                                for $obj in $loadedDoc/OBJECTS/OBJECT
                                                return element objs{
                                                    attribute obj {$obj/@oid},
                                                    element obj{
                                                        attribute link {concat($source/@class-fullid, '/O:', $obj/@oid)}
                                                    }
                                                }
                                        }
                          }
                      }
               
                      return oda:command(concat('save_objects?id=', $classFullId, '&amp;async=false'), oda:xml($newObj)) 
                `;

            return this.$item.API.POST('xquery', {format: 'XML'}, xq)
            .then(res => this.result = res)
            .catch(err => console.log(err))
        } else {
            return Promise.reject('');
        }
    }


    updateImportObjectInfo() {
        return this.$item._reload()
        .then(() =>
            this.$item.fields
            .then(fields => {
                const statusField = fields.find(f => f.name === 'status');
                return statusField.setValue(this.$item, 'Завершен')
                .then(() => this.$item.save())
            })
        )
    }

    responseWriter() {
        console.log(`Import done (${new Date().toLocaleString()})`)
    }

    errorWriter(e) {
        console.log(e.message);

        return this.$item.reload().then(() => {
            switch (e.message) {
                case 'VALIDATION_SOURCE_ERROR': {
                    return this._handlingValidationSourceError(e);
                }
                case 'VALIDATION_RULES_RESULT_ERROR' : {
                    return this._handlingValidationRulesResultError(e);
                }
                default: {
                    return this._handlingUnidentifiedError(e);
                }
            }
        });
    }

    _handlingValidationSourceError() {
        return this._setErrorResult('_handlingValidationSourceError')
        .then(() => this.$item.save());
    }

    _handlingValidationRulesResultError() {
        return this._setErrorResult(`\n${this._getSourceValidationResultText()}\r\n${this._getRulesValidationResultText()}`)
        .then(() => this.$item.save());
    }

    _handlingUnidentifiedError(e) {
        return this._setErrorResult(e.message)
        .then(() => this.$item.save());
    }

    _setErrorResult(text) {
        return this.$item.fields
        .then((fields) => {
            return Promise.all([
                fields.find(f => f.name === 'result').setValue(this.$item, text || ''),
                fields.find(f => f.name === 'status').setValue(this.$item, 'ошибка'),
            ]);
        })
    }

    _getRulesValidationResultText() {
        const resultRules = this.rules
        .reduce((pr, nr) => pr += nr.validations
        .reduce((pv, nv) => {
            let state = 'OK';

            if (!nv.use) {
                state = 'Not use'
            }

            if (nv.err && nv.err.message) {
                state = `error (${nv.err.message})`;
            }

            pv += `${nv.name}: ${state} \r\n`;

            return pv;
        }, ''), '');

        return `Validation result of rules: ${(resultRules) ? `\r\n${resultRules}` : 'not specified'}`
    }

    _getSourceValidationResultText() {
        let sourceValidationMessage = this.validationSourceSettings
        .reduce((pv, nv) => {
            let state = 'OK';

            if (!nv.use) {
                state = 'Not use'
            }

            if (nv.err && nv.err.message) {
                state = `error (${nv.err.message})`;
            }

            pv += `${nv.name}: ${state} \r\n`;

            return pv;
        }, '');

        return `Validation source: ${(sourceValidationMessage) ? `\r\n${sourceValidationMessage}` : 'not specified'}`
    }
}).register();

