(() => {
    'use strict';
    let backend = class extends odaService {

        static get properties() {
            return {
                allowAccess: 'W',
                allowExport: true,
                allowSave: true,
                label: "Backend"
            }
        }

        async execute(soapXml) {
            console.log((new Date()).toISOString() + " - SOAP Start");
            console.log("soapXml = " + soapXml);
            let i = soapXml.indexOf('?>');
            if (i > 0) {
                soapXml = soapXml.substr(i + 2);
                console.log("soapXml = " + soapXml);
            }

            console.log((new Date()).toISOString() + ' - Create XML file');
            let file = await this.$item.getFile('soap/' + ODANT.getGuid() + '.xml', soapXml);
            console.log((new Date()).toISOString() + ' - XQ to XML file');
                
            let method = await file.API.POST('xquery', { format: 'json' }, 'element name { *:Envelope/*:Body/*/name() }');
            console.log((new Date()).toISOString() + " - JSON method = " + method);

            let name = method.$name[0].$TEXT$[0];
            console.log("Method name = " + name);

            if (name.includes('ping')) {
                console.log((new Date()).toISOString() + " - SOAP End ping");
                return '<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope" >\n' +
                    '   <soapenv:Header/>\n' +
                    '   <soapenv:Body>\n' +
                    '      <pingResponse xmlns="http://eai.azurewebsites.net">\n' +
                    '         <pingResult>\n' +
                    '            ' + (new Date()).toISOString() + '\n' +
                    '         </pingResult>\n' +
                    '      </pingResponse>\n' +
                    '   </soapenv:Body>\n' +
                    '</soapenv:Envelope>';
            }

            if (name.includes('SaveObjects')) {
                return this.saveObjects(soapXml, this.$item);
            }

            if (name.includes('GetObjects')) {
                return this.getObjects(soapXml, this.$item);
            }

            if (name.includes('DeleteObjects')) {
                return this.deleteObjects(soapXml, this.$item);
            }

            // if (name.includes('SetStaticData')) {
            //     return this.setStaticData(soapXml, this.$item);
            // }

            // if (name.includes('SetDynamicData')) {
            //     return this.setDynamicData(soapXml, this.$item);
            // }

            // if (name.includes('DeleteStaticData')) {
            //     return this.deleteStaticData(soapXml, this.$item);
            // }

            // if (name.includes('DeleteDynamicData')) {
            //     return this.deleteDynamicData(soapXml, this.$item);
            // }

            return args;
        }

        async saveObjects(xml, context) {
            console.log((new Date()).toISOString() + " - Start SaveObjects");
            console.log((new Date()).toISOString() + " - context = " + context);

            let xq = `let $xml := ${xml}
return
    element OBJECTS {
        for $dt in $xml/*:Body/*:SaveObjects/*:dt
        return 
            element OBJECT
            {
                for $a in $dt/*
                return attribute { substring-after($a/name(), ':') } { $a/text() }
            }
    }`;

            console.log((new Date()).toISOString() + " - xq = " + xq);

            const objXml = await context.XQuery(xq, { format: 'xml' });
            console.log((new Date()).toISOString() + " - objXml: " + objXml);

            const result = await context.API.POST('save_objects', {}, objXml);
            console.log((new Date()).toISOString() + " - result: " + JSON.stringify(result));

            xq = `let $xml := ${xml}
return $xml/*:Body/*:SaveObjects/*:transportguid/text()`;            
            const transportguid = await context.XQuery(xq, { format: 'json' });
            console.log((new Date()).toISOString() + " - transportguid: " + JSON.stringify(transportguid));

            return `<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">
    <soapenv:Header/>
    <soapenv:Body>
        <SaveObjectsResponse xmlns="http://eai.azurewebsites.net">
            <transportguid>${transportguid.$TEXT$}</transportguid>
            <SaveObjectsResult>Всё хорошо</SaveObjectsResult>
        </SaveObjectsResponse>
    </soapenv:Body>
</soapenv:Envelope>`;
        }

        async getObjects(xml, context) {
            console.log((new Date()).toISOString() + " - Start GetObjects");
            console.log((new Date()).toISOString() + " - context = " + context);

            let xq = `let $xml := ${xml}
return $xml/*:Body/*:GetObjects/*:dt/*:oid/text()`;
            console.log((new Date()).toISOString() + " - xq = " + xq);

            let oids = await context.XQuery(xq, { format: 'json' });
            console.log((new Date()).toISOString() + " - oids: " + oids);

            let oidsStr = oids.$TEXT$.join(' ');
            console.log((new Date()).toISOString() + " - oidsStr: " + oidsStr);

            let objXQ = `for $a in oda:select(x/O, 'oid', '${oidsStr}', ' ')
return element dt {
    for $b in $a/@*
    return element { $b/name() }
    {
        string($b)
    }
}`;
            console.log((new Date()).toISOString() + " - objXQ: " + objXQ);

            let objXML = await context.XQuery(objXQ, { format: 'xml' });
            console.log((new Date()).toISOString() + " - objXML: " + objXML);

            xq = `let $xml := ${xml}
return $xml/*:Body/*:GetObjects/*:transportguid/text()`;            
            const transportguid = await context.XQuery(xq, { format: 'json' });
            console.log((new Date()).toISOString() + " - transportguid: " + JSON.stringify(transportguid));
            return `<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">
    <soapenv:Header/>
    <soapenv:Body>
        <GetObjectsResponse xmlns="http://eai.azurewebsites.net">
            <transportguid>${transportguid.$TEXT$}</transportguid>
            <GetObjectsResult>
                ${objXML}
            </GetObjectsResult>
        </GetObjectsResponse>
    </soapenv:Body>
</soapenv:Envelope>`;
        }

        async deleteObjects(xml, context) {
            console.log((new Date()).toISOString() + " - Start DeleteObjects");
            console.log((new Date()).toISOString() + " - context = " + context);

            let xq = `let $xml := ${xml}
return $xml/*:Body/*:DeleteObjects/*:dt/*:oid/text()`;
            console.log((new Date()).toISOString() + " - xq = " + xq);

            let oids = await context.XQuery(xq, { format: 'json' });
            console.log((new Date()).toISOString() + " - oids: " + oids);

            let oidsStr = oids.$TEXT$.join(' ');
            console.log((new Date()).toISOString() + " - oidsStr: " + oidsStr);

            context.removeObjects(oidsStr);

            xq = `let $xml := ${xml}
return $xml/*:Body/*:DeleteObjects/*:transportguid/text()`;            
            const transportguid = await context.XQuery(xq, { format: 'json' });
            console.log((new Date()).toISOString() + " - transportguid: " + JSON.stringify(transportguid));

            return `<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">
    <soapenv:Header/>
    <soapenv:Body>
        <DeleteObjectsResponse xmlns="http://eai.azurewebsites.net">
            <transportguid>${transportguid.$TEXT$}</transportguid>
            <DeleteObjectsResult>Всё хорошо</DeleteObjectsResult>
        </DeleteObjectsResponse>
    </soapenv:Body>
</soapenv:Envelope>`;
        }

        // setStaticData(xml, context) {
        //     console.log((new Date()).toISOString() + " - Start SetStaticData");
        //     console.log("context = " + context);
        //     return context.findItem('C:1D374D4BD6271FE').then(cls => {
        //         console.log((new Date()).toISOString() + " - Class: " + cls.label);

        //         return cls.getIndex('FindExist').then(idx => {
        //             console.log((new Date()).toISOString() + " - Index: " + idx.label);
        //             let xq = 'let $xml := ' +
        //                 xml +
        //                 'return element OBJECTS {\n' +
        //                 '\tfor $dt in ($xml/*:Body/*:SetStaticData/*:dt, x/O)\n' +
        //                 '\tgroup by $dt/*:internal_id/text()\n' +
        //                 '\treturn if ($dt/*:branch)\n' +
        //                 '\t\t   then element OBJECT {\n' +
        //                 '\t\t\t\t\t($dt/@oid)[1],\n' +
        //                 '\t\t\t\t\tattribute Bran  { $dt/*:branch/text() },\n' +
        //                 '\t\t\t\t\tattribute Subs  { $dt/*:subsidiary/text() },\n' +
        //                 '\t\t\t\t\tattribute Nameo { $dt/*:name/text() },\n' +
        //                 '\t\t\t\t\tattribute Type  { $dt/*:type/text() },\n' +
        //                 '\t\t\t\t\tattribute Inter { ($dt/*:internal_id/text())[1] },\n' +
        //                 '\t\t\t\t\tfor $lst in $dt/*:attributes/*:attribute\n' +
        //                 '\t\t\t\t\treturn element Attr {\n' +
        //                 '\t\t\t\t\t\tfor $a in $lst/*\n' +
        //                 '\t\t\t\t\t\tlet $name := string($a/name())\n' +
        //                 '\t\t\t\t\t\tlet $rN := if (contains($name, \'id\'))\n' +
        //                 '\t\t\t\t\t\t\t\t   then \'Ido\'\n' +
        //                 '\t\t\t\t\t\t\t\t   else if (contains($name, \'value\'))\n' +
        //                 '\t\t\t\t\t\t\t\t\t\tthen \'Valueo\'\n' +
        //                 '\t\t\t\t\t\t\t\t\t\telse if (contains($name, \'valid_since\'))\n' +
        //                 '\t\t\t\t\t\t\t\t\t\t\t then \'Valid\'\n' +
        //                 '\t\t\t\t\t\t\t\t\t\t\t else $name\n' +
        //                 '\t\t\t\t\t\treturn attribute { $rN } { $a/text() }\n' +
        //                 '\t\t\t\t\t}\n' +
        //                 '\t\t\t\t}\n' +
        //                 '\t\t   else \'\'\n' +
        //                 '}';
        //             console.log("xq: " + xq);
        //             return idx.XQuery(xq, { format: 'xml' }).then(objXml => {
        //                 console.log((new Date()).toISOString() + " - XML to save: " + objXml);
        //                 return cls.API.POST('save_objects', {}, objXml).then(result => {
        //                     console.log("Result: " + result);
        //                     console.log((new Date()).toISOString() + " - SOAP End SetStaticData");
        //                     return '<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">\n' +
        //                         '   <soapenv:Header/>\n' +
        //                         '   <soapenv:Body>\n' +
        //                         '      <SetStaticDataResponse xmlns="http://eai.azurewebsites.net">\n' +
        //                         '         <SetStaticDataResult>\n' +
        //                         '            Всё хорошо\n' +
        //                         '         </SetStaticDataResult>\n' +
        //                         '      </SetStaticDataResponse>\n' +
        //                         '   </soapenv:Body>\n' +
        //                         '</soapenv:Envelope>';
        //                 })
        //             });
        //         });
        //     });
        // }

        // setDynamicData(xml, context) {
        //     console.log((new Date()).toISOString() + " - Start SetDynamicData");
        //     console.log("context = " + context);
        //     return context.findItem('C:1D374D4B0EF1F6F')
        //         //return context.findItem('H:1D340F0CC923F35/P:WORK/B:1D3548A01D71580/C:1D374D4B0EF1F6F')
        //         .then(cls => {
        //             console.log((new Date()).toISOString() + " - Class: " + cls.label);
        //             return cls.getIndex('FindExist').then(idx => {
        //                 console.log((new Date()).toISOString() + " - Index: " + idx.label);
        //                 let xq = 'let $xml := ' +
        //                     xml +
        //                     '\nreturn element OBJECTS {\n' +
        //                     '\tfor $dt in ($xml/*:Body/*:SetDynamicData/*:dt, x/O)\n' +
        //                     '\tgroup by $dt/*:internal_id/text()\n' +
        //                     '\treturn if ($dt/*:branch)\n' +
        //                     '\t\t   then element OBJECT {\n' +
        //                     '\t\t\t\t\t($dt/@oid)[1],\n' +
        //                     '\t\t\t\t\tattribute Bran  { $dt/*:branch/text() },\n' +
        //                     '\t\t\t\t\tattribute Subs  { $dt/*:subsidiary/text() },\n' +
        //                     '\t\t\t\t\tattribute Nameo { $dt/*:name/text() },\n' +
        //                     '\t\t\t\t\tattribute Type  { $dt/*:type/text() },\n' +
        //                     '\t\t\t\t\tattribute Inter { ($dt/*:internal_id/text())[1] },\n' +
        //                     '\t\t\t\t\tattribute PeriodType  { $dt/*:period_type/text() },\n' +
        //                     '\t\t\t\t\tattribute PeriodStart { $dt/*:period_start/text() },\n' +
        //                     '\t\t\t\t\tattribute PeriodEnd\t  { $dt/*:period_end/text() },\n' +
        //                     '\t\t\t\t\tfor $lst in $dt/*:attributes/*:attribute\n' +
        //                     '\t\t\t\t\treturn element Attr {\n' +
        //                     '\t\t\t\t\t\tfor $a in $lst/*\n' +
        //                     '\t\t\t\t\t\tlet $name := string($a/name())\n' +
        //                     '\t\t\t\t\t\tlet $rN := if (contains($name, \'id\'))\n' +
        //                     '\t\t\t\t\t\t   then \'IdO\'\n' +
        //                     '\t\t\t\t\t\t   else if (contains($name, \'value\'))\n' +
        //                     '\t\t\t\t\t\t\t\tthen \'ValueO\'\n' +
        //                     '\t\t\t\t\t\t\t\telse $name\n' +
        //                     '\t\t\t\t\t\treturn attribute { $rN } { $a/text() }\n' +
        //                     '\t\t\t\t\t}\n' +
        //                     '\t\t\t\t}\n' +
        //                     '\t\t   else \'\'\n' +
        //                     '}';
        //                 console.log("xq: " + xq);
        //                 return idx.XQuery(xq, { format: 'xml' }).then(objXml => {
        //                     console.log((new Date()).toISOString() + " - XML to save: " + objXml);
        //                     return cls.API.POST('save_objects', {}, objXml).then(result => {
        //                         console.log("Result: " + result);
        //                         console.log((new Date()).toISOString() + " - SOAP End SetDynamicData");
        //                         return '<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">\n' +
        //                             '   <soapenv:Header/>\n' +
        //                             '   <soapenv:Body>\n' +
        //                             '      <SetDynamicDataResponse xmlns="http://eai.azurewebsites.net">\n' +
        //                             '         <SetDynamicDataResult>\n' +
        //                             '            Всё хорошо\n' +
        //                             '         </SetDynamicDataResult>\n' +
        //                             '      </SetDynamicDataResponse>\n' +
        //                             '   </soapenv:Body>\n' +
        //                             '</soapenv:Envelope>';
        //                     })
        //                 });
        //             });
        //         });
        // }

        // deleteStaticData(xml, context) {
        //     console.log((new Date()).toISOString() + " - Start DeleteStaticData");
        //     console.log("context = " + context);
        //     return context.findItem('C:1D374D4BD6271FE')
        //         .then(cls => {
        //             console.log((new Date()).toISOString() + " - Class: " + cls.label);
        //             return cls.getIndex('FindExist').then(idx => {
        //                 console.log((new Date()).toISOString() + " - Index: " + idx.label);
        //                 let xq = 'let $xml := ' +
        //                     xml +
        //                     'return for $dt in $xml/*:Body/*:DeleteStaticData/*:internal_id/text()\n' +
        //                     '\t   return x/O[*:internal_id/text() = $dt]/string(@oid)';
        //                 console.log("xq: " + xq);
        //                 return idx.XQuery(xq, { format: 'json' }).then(oids => {
        //                     console.log((new Date()).toISOString() + " - oids: " + oids);

        //                     cls.removeObjects(oids.join(' '));

        //                     console.log((new Date()).toISOString() + " - SOAP End DeleteStaticData");
        //                     return '<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">\n' +
        //                         '   <soapenv:Header/>\n' +
        //                         '   <soapenv:Body>\n' +
        //                         '      <DeleteStaticDataResponse xmlns="http://eai.azurewebsites.net">\n' +
        //                         '         <DeleteStaticDataResult>\n' +
        //                         '            Всё хорошо\n' +
        //                         '         </DeleteStaticDataResult>\n' +
        //                         '      </DeleteStaticDataResponse>\n' +
        //                         '   </soapenv:Body>\n' +
        //                         '</soapenv:Envelope>';
        //                 });
        //             });
        //         });
        // }

        // deleteDynamicData(xml, context) {
        //     console.log((new Date()).toISOString() + " - Start DeleteStaticData");
        //     console.log("context = " + context);
        //     return context.findItem('C:1D374D4B0EF1F6F')
        //         .then(cls => {
        //             console.log((new Date()).toISOString() + " - Class: " + cls.label);
        //             return cls.getIndex('FindExist').then(idx => {
        //                 console.log((new Date()).toISOString() + " - Index: " + idx.label);
        //                 let xq = 'let $xml := ' +
        //                     xml +
        //                     'return for $dt in $xml/*:Body/*:DeleteDynamicData/*:internal_id/text()\n' +
        //                     '\t   return x/O[*:internal_id/text() = $dt]/string(@oid)';
        //                 console.log("xq: " + xq);
        //                 return idx.XQuery(xq, { format: 'json' }).then(oids => {
        //                     console.log((new Date()).toISOString() + " - oids: " + oids);

        //                     cls.removeObjects(oids.join(' '));

        //                     console.log((new Date()).toISOString() + " - SOAP End DeleteDynamicData");
        //                     return '<soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope">\n' +
        //                         '   <soapenv:Header/>\n' +
        //                         '   <soapenv:Body>\n' +
        //                         '      <DeleteDynamicDataResponse xmlns="http://eai.azurewebsites.net">\n' +
        //                         '         <DeleteDynamicDataResult>\n' +
        //                         '            Всё хорошо\n' +
        //                         '         </DeleteDynamicDataResult>\n' +
        //                         '      </DeleteDynamicDataResponse>\n' +
        //                         '   </soapenv:Body>\n' +
        //                         '</soapenv:Envelope>';
        //                 });
        //             });
        //         });
        // }
    };

    module.exports = backend;
    backend.register()
})();
