globalThis.TEST_OBJECT_VIEW_DESIGNER_DRAG_TYPES = globalThis.TEST_OBJECT_VIEW_DESIGNER_DRAG_TYPES || {
    field: {
        type: 'odant/designer-field',
        effect: 'copy',
    },
    container: {
        type: 'odant/designer-container',
        effect: 'copy',
    },
    unit: {
        type: 'odant/designer-unit',
        effect: 'move',
    }
};
globalThis.FieldItem = globalThis.FieldItem || class FieldItem {
    constructor($field, $level) {
        this.$field = $field;
        this.$level = $level || 0;
        if ($field.path.split('/').length < 2) {
            FieldItem.fieldItems.push(this);
        }
        this.id = ODANT.getGuid();
    }
    get icon() { return this.$field.icon }
    get label() { return this.$field.label }
    get path() { return this.$field.path.replace('[fields]', '[FIELDS]') }
    get fields() { return this.$field.fields }

    static fieldItems = [];
    static clear() {
        this.fieldItems.splice(0, this.fieldItems.length);
    }
    static async fill(fields = [], level = 0) {
        const flatify = async (res, f) => {
            res = await res;
            res.push(new FieldItem(f, level));
            if (!f.isTable && !f.isLink && !(f.TypeClass && f.TypeClass.isAbstract)) {
                const fields = await f.fields;
                if (fields && fields.length) {
                    res.push(...(await fields.reduce(flatify, [], level + 1)));
                }
            }
            return res;
        }
        this.fieldItems = await fields.reduce(flatify, []);
    }
    static async getField(path, fields = this.fieldItems) {
        const fullPath = path.startsWith('[FIELDS]') ? path : `[FIELDS]/${path}`;
        const fieldItem = fields.find(f => fullPath === f.path);
        if (fieldItem) {
            return fieldItem instanceof odaField ? fieldItem : fieldItem.$field;
        }
        else {
            const owner = fields.find(f => fullPath.includes(f.path));
            if (owner) {
                const fields = await owner.fields;
                return this.getField(path, fields);
            } else {
                return null;
            }
        }
    }
}
globalThis.designController = globalThis.designController || {
    focusedUnit: undefined,
    selection: [],
    _handlers: {select:[], focus: []},
    focus(unit) {
        this.focusedUnit = unit;
        this.fire('focus');
    },
    select(units) {
        units = Array.isArray(units) ? units : [units];
        this.selection.splice(0, this.selection.length, ...units);
        this.fire('select');
    },
    addSelect(units) {
        units = Array.isArray(units) ? units : [units];
        this.selection.push(...units);
        this.fire('select');
    },
    deselect(units) {
        units = Array.isArray(units) ? units : [units];
        for (const unit of units) {
            const idx = this.selection.indexOf(unit);
            if (!idx) this.selection.slice(idx, 1);
        }
        this.fire('select');
    },
    checkSelect(unit) {
        return this.selection.includes(unit);
    },
    checkFocus(unit) {
        return this.focusedUnit === unit;
    },
    on(type, cb) {
        const list = this._handlers[type];
        if (!list.includes(cb)) list.push(cb);
    },
    off(type, cb) {
        const list = this._handlers[type];
        const idx = list.indexOf(cb);
        if (~idx) list.splice(idx, 1);
    },
    fire(type) {
        for (const cb of this._handlers[type]) {
            cb(new CustomEvent(type, { detail: this}));
        }
    }
};
globalThis.UnitItem = globalThis.UnitItem || class UnitItem {
    constructor(unit) {
        this.unit = unit;
        this.item = this.unit.item;
        this.host = this.unit.host;
        this.domHost = unit.domHost;
        this.hostItems = this.domHost.item?.items || this.domHost.items;
        if (unit.dropSide) {
            this.side = this.unit.dropSide;
            this.align = this.side === 'inside' ? '' : this.unit.sides[this.side].align;
            this.idxMod = this.side === 'inside' ? 0 : this.unit.sides[this.side].idxMod;
        }
    }
    get idx() {
        return this.hostItems.indexOf(this.item);
    }
}