(class extends odaService {

    static get properties() {
        return {
            label: 'Register',
            allowArrayContext: true,
            isActive: true,
            allowAccess: 'C',
            subMenu: function(){
                return [
                    {
                        'icon': 'icons:cancel',
                        'label': 'Cancel',
                        'execute': function(e){
                            const postBody = JSON.stringify(
                                {
                                    'clsId': this.contextItem.$class.fullId,
                                    'items': this.contextItem.Items ? this.contextItem.Items.join(' ') : [this.contextItem.id],
                                }
                            );

                            return this.ActionSource.EXECUTE('methods/unreg-doc', postBody)
                                .then((result) => {
                                    ODA.notify(result);
                                })
                                .catch((err) => {
                                    console.log(err);
                                });

                        }.bind(this)
                    },
                    {
                        'icon': 'icons:autorenew',
                        'label': 'Renew transaction',
                        'execute': (e) => ODA.notify('Renew transaction')
                    },
                    {
                        'icon': 'icons:settings',
                        'label': 'Configure transactions',
                        'execute': (e) => ODA.notify('Configure transactions')
                    }
                ]
            }
        }
    }

    execute() {
            const xq = `
                        let $oids := '${this.$item.Items ? this.$item.Items.join(' ') : [this.$item.id]}'
                        let $docClassId := '${this.$item.$owner.fullId}'
                        let $allObjects := oda:select(doc(concat($docClassId, '/I:PACK'))/PACK/OBJECT, 'oid', $oids)

                        let $res :=
                            for $obj in $allObjects
                                group by $obj/@bid
                                return element b {
                                    attribute bid {$obj[1]/@bid},
                                    attribute name {$obj[1]/@bid},
                                    for $o in $obj
                                        group by $o/@cid
                                            return element c{
                                                attribute name {$o[1]/@cid},
                                                attribute cid  {$o[1]/@cid},
                                                attribute countObject {count($o)},
                                                for $reg in //PACK/OBJECT[cls/contains(@link, concat('C:', $o[1]/@cid))]
                                                    return element reg {
                                                        attribute clsId {concat('H:','${this.$item.$host.id}', '/B:', $o[1]/@bid, '/', 'C:',$o[1]/@cid)},
                                                        attribute templateId {concat('B:', $o[1]/@bid, '/', 'C:',$reg/@cid, '/' ,'O:', $reg/@oid)},
                                                        attribute oids {string-join($o/@oid, ' ')},
                                                        $reg/@name
                                                    }
                                            }
                                }

								
						let $regs := $res//reg		
						let $countTemplate := count($regs)
                      
						return  element result {
							attribute count {$countTemplate},
							if ($countTemplate = 1) then (
							attribute clsId{$regs[1]/@clsId},
							attribute templateId{$regs[1]/@templateId},
							attribute oids{$regs[1]/@oids})
							else $res	
						}
                    `;

            return this.ActionSource.XQuery(xq, {index: 'Pack', format: 'json-tagged'})
                .then((xqRes) => {
                    const countRegs = parseInt(xqRes[0].count);
                    if (countRegs === 1){
                        return this._regDocs(xqRes[0])
                    } else {
                        return ODA.createComponent(this.resolveUrl('reg-selector/reg-selector.html'), {
                            dataSet: xqRes[0].items
                        })
                            .then(control =>
                                ODA.showDialog(control)
                                    .then((e) => {
                                        if (e.focusedRow) {
                                            return this._regDocs(e.focusedRow)
                                        }
                                    })
                            )
                    }
                });
    }

    _regDocs({clsId, oids, templateId}){
        return this.contextItem.$base.findItem(templateId)
            .then((template) => {
                const postBody = JSON.stringify(
                    {
                        'clsId': clsId,
                        'items': oids,
                    }
                );

                return template.EXECUTE('methods/reg-doc', postBody)
                    .then((res) => {
                        console.info(`Transaction: reg-doc execution time (hr): ${res.time[0]}s ${res.time[1]/1000000}ms`);
                    })
                    .catch((err) => {
                        console.error(`Transaction: error ${err}`)
                    })
            })
    }
}).register();