(class extends odaService {

    static get properties() {
        return {
            allowAcces: 'RW',
            allowArrayContext: true,
            allowUse: true,
            icon: 'enterprise:calculator',
            isActive: true,
            label: 'Recalc'
        }
    }

    async execute(params) {
        const url = this.resolveUrl('recalc-template-selector/recalc-template-selector.html');
        const selector = await ODA.createComponent(url);
        selector.contextItem = this.$owner;

        const dialog = {
            title: 'Select template',
            icon: 'image:center-focus-strong',
            ok: {
                show: true,
                disable: false
            },
            buttons: [
                { label: 'New', icon: 'icons:add', special: 'add' },
                { label: 'Remove', icon: 'icons:remove', special: 'del' },
                { label: 'Settings', icon: 'icons:settings', special: 'set' },
                { label: 'OK', special: 'ok' }
            ]
        };
        const dialogResult = await ODA.showDialog(selector, dialog);
        if (!dialogResult.button) {
            if (!dialogResult.focusedItem || !dialogResult.focusedItem.item)
                return;

            const recalcResult = await ODANT.API.post(dialogResult.focusedItem.item.url,
                { execute: 'methods/run-recalc', restart: 'true' },
                `'${this.getOids(this.$owner)}'`);
        } else {
            switch (dialogResult.button.special) {
                case 'add':
                    const recalcClsPath = "M:1CF5DF22BB9FFCB/C:397547091253375";
                    const recalcCls = await this.$owner.$base.findItem(recalcClsPath);
                    const newTmpObj = await recalcCls.createObject();

                    if (newTmpObj.Root) {
                        newTmpObj.Root.cls = this.$owner.$class.label;
                        newTmpObj.Root.$cls = [ { link: recalcCls.getRelativeId(this.$owner.$class) } ];
                        newTmpObj.show();
                    }
                    break;
                case 'del':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        if (await ODA.confirm('Are you sure?')) {
                            dialogResult.control.focusedItem.item.remove();
                        }
                    }
                    break;
                case 'set':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        dialogResult.control.focusedItem.item.show();
                    }
                    break;
                case 'ok':
                    if (!dialogResult.control.focusedItem || !dialogResult.control.focusedItem.item)
                        return;

                    const tmpObj = dialogResult.control.focusedItem.item;
                    const recalcResult = await ODANT.API.post(tmpObj.url,
                        { execute: 'methods/run-recalc', restart: 'true' },
                        `'${this.getOids(this.$owner)}'`);
                    break;
                default:
                    alert('Я таких значений не знаю');
            }
        }
    }

    getOids(item) {
        if (item instanceof odaItemList) {
            return item.items.map(i => i.oid).join(' ');
        }

        return item.id;
    }
}).register();