const UserSession = require('./user-session');
const EventEmitter = require('events').EventEmitter;

class sessionList extends EventEmitter{
    constructor(){
        super();
        this.sessions = new Map();
    }
    get(profile){
        return this.sessions.get(profile.id)
    }
    add(profile, item){
        if (this.get(profile))
            this.delete(profile);

        const userSession = new UserSession(profile);
        userSession.MainContextItem = item;

        userSession.on('sessionExpired', this._onSessionExpired.bind(this, userSession));
        userSession.on('contextItemChanged', this._onContextItemChanged.bind(this, userSession));

        this.sessions.set(profile.id,  userSession);
        this.emit('sessionStart', userSession);
    }
    delete(profile){
        this.sessions.delete(profile.id);
    }
    getSessionMainContext(profile){
        return this.get(profile).MainContextItem;
    }
    getSessionContext(profile){
        return this.get(profile).ContextItem;
    }
    setSessionContext(profile, item){
        this.get(profile).ContextItem = item;
    }
    getSessionStructure(profile){
        return this.get(profile).Structure;
    }

    _onSessionExpired(userSession){
        this.delete(userSession);
        this.emit('sessionClose', userSession);
    }
    _onContextItemChanged(userSession){
        this.emit('sessionContextItemChanged', userSession);
    }
}

module.exports = sessionList;
