'use strict';


const proxyActionHandler = {
    get:(target, key)=>{
        const prop = Object.getOwnPropertyDescriptor(target.properties, key);
        if (!prop)
            return target[key];
        return target.getAttr(key, target[key], prop.value && prop.value.type);
    },
    set:(target, key, value)=>{
        const prop = Object.getOwnPropertyDescriptor(target.properties, key);
        if (prop && !prop.readOnly && (prop.set || prop.value) && target.$class) {
            if (!target.Root){
                if (!target.$class.Root.$METHODS){
                    target.$class.Root.$METHODS = [{$METHOD:[]}];
                }
                target.$class.Root.$METHODS[0].$METHOD.push({name: target.id})
            }
            target.setAttr(key,value)
            target.$class.fireChanged();
        }
        else{
            target[key] = value;
        }
        return true;
    }
}

globalThis.odaAction = globalThis.odaAction || class odaAction extends odaItem {
    get Root(){
        return this.$class && this.$class.Root.$METHODS && this.$class.Root.$METHODS[0].$METHOD
            && this.$class.Root.$METHODS[0].$METHOD.find(m => (m.name === this.id))
    }

    //todo static create odaAction
    static create(owner, file, source) {
        return owner.$(file.path, async () => {
            const load = async (url, error) => {
                try {
                    const script = await ODANT.loadScript(url);
                    const p = {
                        scriptFolder: file.$owner,
                        group: file.$owner.$owner.name,
                        $owner: owner,
                        ActionSource: source,
                        $sourceItem: source,
                        error: error,
                        _scriptFile: file
                    };
                    let  action = odaItem.factory.create(script.src, p);
                    action = new Proxy(action, proxyActionHandler)
                    return action;
                }
                catch (e) {
                    console.log(e, url);
                    if (url !== '/web/core/script.js') {
                        return load('/web/core/script.js', e);
                    }
                }
            };
            return load(file.url);
        })
    }
    async createComponent(path) {
        path = path || this.defaultViewName;
        const control = await ODA.createComponent(this.resolveUrl(path));
        control.contextItem = this.$item;
        control.sourceItem = this.$sourceItem;
        return control;
    }
    static get properties() {return {
        // allow access level for contextItem
        // 0 - '' : no access
        // 1 - 'P': Preview
        // 2 - 'R': Read
        // 3 - 'W': Write
        // 4 - 'C': Create
        // 5 - 'D': Delete
        // 6 - 'A': Admin
        shortcut: '',
        _color(){
            return this.inherit.color;
        },
        methods:()=>[{
                name: 'edit-code',
                label: 'Edit code',
                icon: 'icons:settings',
                execute(){
                    ODA.push("Вызван метод edit-code");
                },
                allowAccess: 'A'
            }
        ],
        locked: false,
        allowUse: {
            type: Boolean,
            get() { return true },
            set(v) { console.log('set allowUse: ', v) }
        },
        subIcon() {
            if (this.$item instanceof odaItemList)
                return this.$item.subIcon;
        },
        allowExport: false, // allow export this action to any classes in current base
        allowArrayContext: false, // allow action odaItemList as contextItem
        Hidden: false,
        itemType: odaItem.Types.odaAction,
        description() {
            return 'Action for ' + this.$item;
        },
        label: {
            get() { return (this.scriptFolder && this.scriptFolder.label); },
            set(v) { console.log('set label: ', v) }
        },
        icon: {
            get() { return this.resolveUrl('icon.png'); },
            set(v) { console.log('set icon: ', v) }
        },
        defaultIcon: 'icons:settings-applications',
        inherit (){  // todo Проверить
            return this.$('inherit', ()=>{
                return  (this.scriptFolder && this.scriptFolder.inherit) || new odaInherit();
            })
        },
        id () {
            return (this.scriptFolder && this.scriptFolder.name) || this.constructorName;
        },
        path () {
            return this.$("path", () => {
                if (this.src) {
                    const array = this.src.split('/');
                    array.pop();
                    return array.reduce((res, i)=>{
                        if (res.length > 0 || (i === (ODANT.webMode?'client':'server')))
                            res.push(i);
                        return res;
                    }, []).join('/');
                }
                return null;
            })
        },
        url () {
            return this.contextItem.url + '/~/' + this.path + '/index.html';
        },
        ActionSourceClass() { //todo remove
            if (this.inherit.level === odaInherit.none || this.inherit.level === odaInherit.core || !this.$class.$owner)
                return Promise.resolve(this.$class);
            return this.$class.$owner.findAction(this.scriptFolder.$owner.label + '/' + this.scriptFolder.label).then(action => {
                return action && action.ActionSourceClass;
            })
        },
        $class() {
            return (this.contextItem instanceof odaClass) ? this.contextItem : this.contextItem.$class;
        },
        group () {
            return this.$('group', this.scriptFolder.$owner.name);
        },
        $item () {
            return this.$owner;
        },
        contextItem () {
            return this.$owner;
        },
        isActive: true,
        defaultViewName () {
            return this.name.toLowerCase()+'.html';
        },
        type(){
            return this.constructorName.toLowerCase();
        }
    }}
    // set Folder(v){
    //     this.$set('folder', v);
    // }
    showSettings(){
        return this.scriptFolder.open('app-studio');
    }
    get MethodPrefix() {
        return this.$('MethodPrefix', () => {
            if (this.src.includes('/web/class'))
                return '$C';
            if (this.src.includes('/web/object'))
                return '$O';
            if (this.src.includes('/web/file'))
                return '$F';
            else
                return undefined;
        })
    }
    // get MethodRoot() {
    //     if (!this.$class.METHODS)
    //         this.$class.Root.$METHODS = [{}];
    //
    //
    //     let methodRoot;
    //     if (this.$class.METHODS.$METHOD) {
    //         methodRoot = this.$class.METHODS.$METHOD.find((m) => (m.name === this.name) && (m.cid === this.ActionSource.id))
    //     } else {
    //         this.$class.METHODS.$METHOD = [];
    //     }
    //
    //     if (!methodRoot) {
    //         methodRoot = { name: this.name, cid: this.ActionSource.id}
    //         this.$class.METHODS.$METHOD.push(methodRoot);
    //     }
    //
    //     return methodRoot;
    // }
    execute(params) {
        ODA.notify(this);
        ODA.notify(this.src);
        ODA.notify(this.$owner);
        ODA.notify(params);
    }
    showDialog(params) {
        return this.createComponent(this.resolveUrl(this.defaultViewName)).then(component => {
            component.contextItem = this.$item;
            if (params) {
                for (let p in params) {
                    component[p] = params[p];
                }
            }
            return ODA.showDialog(component)
        })
    }

    // saveSettingsFile(body, key, inItemsOwner) {
    //     var path = inItemsOwner ? 'web/' + this.$owner.ItemType + '/' + this.ItemType + '/' + this.Name + '/_settings/' : 'web/' + this.ItemType + '/' + this.Name + '/_settings/';
    //     path = key ? path + key + '_settings.json' : path + 'settings.json';
    //     var storingItem = inItemsOwner ? this.$owner.$owner : this.$owner;
    //     return storingItem.saveFile(body, path);
    // }
    // loadSettingsFile(key, fromItemsOwner) {
    //     var path = fromItemsOwner ? 'web/' + this.$owner.ItemType + '/' + this.ItemType + '/' + this.Name + '/_settings/' : 'web/' + this.ItemType + '/' + this.Name + '/_settings/';
    //     path = key ? path + key + '_settings.json' : path + 'settings.json';
    //     var storingItem = fromItemsOwner ? this.$owner.$owner : this.$owner;
    //     return storingItem.loadFile(path);
    // }
    // get settingsFiles() {
    //     return new Promise((resolve, reject) => {
    //         //todo
    //     })
    // }
};
odaAction.register();
// odaView -------------------------------------------------------------------------------------------------------------



globalThis.odaView = globalThis.odaView || class odaView extends odaAction {
    static get properties() {return {
        // itemType: 'view',
        category: 'views',
        allowSave: false,
        allowButtons: false,
        allowServices: false,
        hideToolbar: false,
        hideViews: false
        // defaultIcon: 'icons:visibility'
    }}
    execute(parent) {
        while (parent && !parent.showView) {
            parent = parent.domHost;
        }
        if (parent){
            parent.showView(this);
        }
        else{
            window.open(ODANT.URL(this.contextItem.url + '/~/client/form/index.html?view=' + this.id));
        }

    }
    get _control() {
        // return this.$('_control', ()=>{
            return this.createComponent();
        // })
    }
    createComponent(component) {
        return ODA.createComponent(this.resolveUrl(component || this.defaultViewName)).then(control => {

            control.contextItem = this.$item;
            control.sourceItem = this.$sourceItem;
            return control;
        })
    }

    create (layoutOnly = false) {
        return ODA.createComponent(this.resolveUrl(this.defaultViewName)).then((viewControl) => {
            const control = viewControl.cloneNode();

            control.contextItem = this.$item;

            if (!layoutOnly) {
                return control;
            }

            const selectors = [
                '.side-bar',
                'view-toolbar',
                '.service-bar',
            ];

            let node;

            selectors.forEach((selector) => {
                node = control.querySelector(selector);

                if (node) {
                    node.remove();
                }
            });

            return control;
        });
    }

    getModalResult(control) {
        control = control ? Promise.resolve(control) : this._control;
        return control.then(ctrl => {
            return ctrl;
        })
    }
    open(form = this.name) {
        this.$item.open(undefined, form);
    }

    show(form = this.name) {
        this.$item.show(form);
    }
    showDialog(params) {
        return this.$item.showDialogView(this.name, params);
    }

    navigate(form = this.name) {
        this.$item.navigate(form);
    }
};
odaView.register();

// odaPage -------------------------------------------------------------------------------------------------------------

globalThis.odaPage = globalThis.odaPage || class odaPage extends odaAction {
    execute(args) {
        // let _window = window.open("about:blank", '_blank');
        // window.variables = args;
        // _window.document.location = this.url;
        window.open(this.url, '_blank');
    }
    static get properties() { return {
        category: 'pages',
        defaultIcon: 'icons:open-in-browser'
    }}
};
odaPage.register();

// odaService ----------------------------------------------------------------------------------------------------------

globalThis.odaService = globalThis.odaService || class odaService extends odaAction {
    static get properties() {return {
        allowExport: true,
        category: 'services',
        defaultIcon: 'icons:launch',

    }}
    execute(params) {
        ODA.notify(this.ContextItem);
        ODA.notify(params.item);
        ODA.notify(this.src);
    }
};
odaService.register();

// odaButton ----------------------------------------------------------------------------------------------------------

globalThis.odaButton = globalThis.odaButton || class odaButton extends odaAction {
    static get properties() {return {
        defaultIcon: 'icons:extension',
        category: 'buttons'

    }}
    execute(params) {
        ODA.notify(this.ContextItem);
        ODA.notify(params.item);
        ODA.notify(this.src);
    }
};
odaButton.register();