'use strict';

globalThis.odaConnection = globalThis.odaConnection || class odaConnection extends odaItem {
    static get properties() { return {
        isSystem: true,
        // defaultForm: '',
        pathLabel: null,
        url(){
            return this.id + "/api";
        },
        _actionsPath: 'client',
        userId: getCache('userId', '')
    }}
    static async _findAccountServer(login, type) {
        return ODANT.API.request(ODANT.origin + '/api', {method: 'find_account', login: login, type: type});
    }
    static async _authorize(ticket) {
        const ts = JSON.stringify(ticket);
        const security = new odantLocalStorage('security');
        try{
            const response = await ODANT.API.request(ODANT.origin +'/api', {method: 'authorize'}, ts);
            security.setValue('ssid', response.result);
            security.setValue('ticket', ticket);
            security.setValue('time', Date.now());
            return this.login();
        }
        catch (e) {
            ODANT.push(e);
            security.clear();
        }
    }
    static async _checkLogin(login){
        try{
            let res = await odaConnection._findAccountServer(login);
            await ODANT.API.request(res.result+'/api', {method: 'check_login', login: login});
            return true;
        }
        catch (e) {
            return false;
        }

    }
    static async _registration(login, password, type) {
        if (await odaConnection._checkLogin(login))
            throw new Error ('login is busy');
        let res = await odaConnection._findAccountServer(login);
        const p = Sha256.hash(password);
        return ODANT.API.request(res.result+'/api', {method: 'registration', login: login, type: type, referrer: document.URL}, p);
    }
    static async _authentication(login, password) {
        let res = await odaConnection._findAccountServer(login);
        const p = Sha256.hash(password);
        res = await ODANT.API.request(res.result+'/api', {method: 'authentication', login: login}, p);
        new odantLocalStorage('security').setValue('login', login);
        res = await odaConnection._authorize(res);
        ODANT.restart();
        return res;
    }
    static async _enterWithoutAuth(security = new odantLocalStorage('security')) {
        security.clear();
        security.setValue('login', 'GUEST');
        ODANT.restart();
    }
    static async login(force) {
        if (ODANT.clientSide) {
            try {
                return await ODANT.API.request(ODANT.origin + '/api', { method: 'check_ssid' });
            }
            catch (e) {
                async function authorize(login, ticket) {
                    if (!login && ticket) {
                        try {
                            login = ticket.$AUTHORIZATION[0].$INFO[0].$OBJECT[0].name;
                        } catch (err) {
                            console.warn(err);
                        }
                    }
                    if (login && login !== 'guest') {
                        if (ticket) {
                            const ts = JSON.stringify(ticket);
                            const security = new odantLocalStorage('security');
                            try {
                                const res = await ODANT.API.request(ODANT.origin + '/api', { method: 'authorize' }, ts);
                                security.setValue('ssid', res.result);
                                security.setValue('ticket', ticket);
                                security.setValue('time', Date.now());
                                security.setValue('login', login);
                                return res;
                            }
                            catch (e) {
                                console.warn(e);
                                ODANT.push(e);
                                security.clear();
                                return odaConnection._findAccountServer('guest');
                            }
                        } else {
                            return odaConnection._findAccountServer(login);
                        }
                    } else if (window.top === window) {
                        const storage = new odantLocalStorage('security');
                        odaConnection._enterWithoutAuth(storage);
                    }
                }
                const storage = new odantLocalStorage('security');
                const login = force ? '' : storage.getValue('login');
                const ticket = storage.getValue('ticket');
                if (login) {
                    return authorize(login, ticket);
                } else {
                    const res = await ODANT.API.request(ODANT.origin + '/api', { method: 'version' }).catch(e => console.warn(e));
                    if (!res || !res.result || res.result === '1.0') {
                        try {
                            return await ODANT.showLogin();
                        }
                        catch (err) {
                            return ODANT._enterWithoutAuth(security);
                        }
                    }
                    return new Promise(async (resolve, reject) => {
                        document.body.style.pointerEvents = 'none';
                        document.body.style.filter = 'blur(1px) brightness(0.8)';
                        const authOrigin = (await ODANT.API.request(ODANT.origin + '/api', { method: 'find_account' })).result;
                        function getPath(method = 'pap') {
                            return encodeURI(`${authOrigin}/api/~/client/auth/index.html?redirect_url=${location.href}`);
                        }
                        const width = 500;
                        const height = 500;
                        const left = Math.floor((window.innerWidth - width) / 2 + window.screenLeft);
                        const top = Math.floor((window.innerHeight - height) / 2 + window.screenTop);
                        const wnd = window.open(getPath(), 'odant-authentication','', true);
                        // wnd.resizeTo(width, height);
                        // wnd.resizeTo(left, top);
                        wnd.name = 'odant-authentication';
                        const Transmitter = (await import('../host/client/auth/Transmitter.js')).default;
                        const transmitter = new Transmitter(wnd);

                        const closeWnd = () => {
                            if (wnd) {
                                transmitter.send('close', undefined, getPath());
                                wnd.close();
                            }
                        };
                        transmitter.on('loginAsGuest', (e) => {
                            close({ login: 'guest' });
                        });
                        transmitter.on('authorize', (e) => {
                            close(e.data.data);
                        });

                        const close = async ({login, ticket } = {}) => {
                            closeWnd();
                            // document.body.style.pointerEvents = '';
                            // document.body.style.filter = '';
                            if (!login) {
                                try {
                                    login = ticket.$AUTHORIZATION[0].$INFO[0].$OBJECT[0].name;
                                } catch (err) {
                                    console.warn(err);
                                }
                            }
                            const res = await authorize(login, ticket);
                            resolve(res);
                            location.reload();
                        }
                        window.addEventListener('click', () => close({ login: 'guest' }));
                        window.addEventListener('beforeunload', closeWnd);
                        window.addEventListener('close', closeWnd);
                        wnd.focus();
                    });
                }

                // let security = new odantLocalStorage('security');
                // let login = force?'':security.getValue('login');
                // if (login) {
                //     const ticket = security.getValue('ticket');
                //     if (ticket)
                //         return odaConnection._authorize(ticket);
                //     return odaConnection._findAccountServer(login);
                // }
                // else if (ODA.mainWindow === window) {
                //     try{
                //         return await ODANT.showLogin();
                //     }
                //     catch (err) {
                //         ODANT._enterWithoutAuth(security);
                //     }
                // }
            }
        }
    }
    async logout(){
        if (ODANT.clientSide) {
            try {
                await ODANT.API.request(ODANT.origin + '/api', { method: 'logout' });
            } catch (err) { console.log(err) };
            new odantLocalStorage('security').clear();
            ODANT.restart();
        }
    }
    listen(item, callback){
        this.EventSource.addEventListener(item.oldFullId, callback, false);
    }
    unlisten(item, callback){
        this.EventSource.removeEventListener(item.oldFullId, callback);
    }

    get EventSource() {
        return this.$('EventSource', () => {
            let parser;

            if (ODANT.webMode){
                parser = document.createElement('a');
                parser.href = this.name;
            } else {
                parser = new URL(this.name);
            }

            let url;
            url = (parser.protocol === 'https:')
                ? 'wss://'
                : 'ws://';

            url += parser.hostname;

            if (parser.port && parser.port.length && parser.port.length > 0) {
                url += ':' + parser.port;
            }

            url += '/ws';
            return new WebSocketEvents(url);
        })
    }

    async REQUEST(item, params = {}, post) {
        // console.time('request async');
        let result = await ODANT.API.request(item.url, params, post);
        // console.timeEnd('request async');
        if (params.async){
            // console.time('request');
            const checkAsyncStatus = async (handle, delay) => {

                let command = "status";
                switch (handle.status){
                    case 'ready':{
                        command = 'load'
                    } break;
                    case 'break':{
                        command = 'break'
                    } break;
                }
                try{
                    const result = await ODANT.API.request(ODANT.origin + handle.url+'/'+command);
                    if (result === undefined || result === null)
                        return  result;
                    // if(ODANT.clientSide)
                    //     ODANT.mainWindow.dispatchEvent(new CustomEvent('async-request', {detail: handle}));

                    handle.status = result.status;
                    switch (handle.status){
                        case 'ready':{
                            return checkAsyncStatus(handle);
                        }
                        case undefined:{
                            return result;
                        }
                        default:{
                            return new Promise((resolve, reject)=>{
                                const callback = async ()=>{
                                    // delay = delay < 1000?delay * 1.1:delay;
                                    try{
                                        const result = await checkAsyncStatus(handle, delay);
                                        resolve(result);
                                    }
                                    catch(err){
                                        console.error(err);
                                        reject(err);
                                    }
                                };
                                setTimeout(callback, delay);
                            });
                        }
                    }
                }
                catch(err){
                    handle.status = 'break';
                    // if (ODANT.clientSide){
                    //     ODANT.mainWindow.dispatchEvent(new CustomEvent('async-request', {detail: handle}));
                    // }
                    console.error(this, err);
                    return null;
                }
            };
            result = await checkAsyncStatus(result, 100);
        }
        // console.timeEnd('request');
        return result;
    }
    _fireAsyncEvent(handle){
        if(ODANT.clientSide){
            ODA.mainWindow.dispatchEvent(new CustomEvent('async-request', {target: handle}));
        }
    }
    async getItem(path, find = true) {
        return this.$('get-item:' + path, async () => {
            if (!path) return this;
            else {
                if (find) {
                    try {
                        const json = await this.API.GET('find_item', { path: path, chain: "full", prefix: true });
                        path = decodeURI(json.result);
                    } catch (e) { console.warn(e); }
                }
                let s = path.split('/');
                if (s.length < 4 && (s[0] === 'http:' || s[0] === 'https:')) {
                    return ODANT.getConnection(path);
                }
                let id, prefix, oid;
                if (s.length) {
                    id = s.pop();
                    if (id.indexOf(':') === 1) {
                        prefix = id[0];
                        id = id.substr(2);
                    }
                    else if (id !== 'api') prefix = id === '~' || id.indexOf('.') < 0 ? 'DIR' : 'FILE';
                    oid = s.join('/');
                    try {
                        const owner = await this.getItem(oid, prefix.length !== 1);
                        if (!owner) throw new Error(`error on getting owner of ${path}`);
                        switch (prefix) {
                            case 'H': return owner.getHost(id);
                            case 'P':
                            case 'B':
                            case 'S':
                            case 'W':
                            case 'M': return owner.getDomain(id, prefix);
                            case 'D': console.warn("D: deprecated prefix!!! Use P: or B: or M: or :W or S:"); return owner.getDomain(id);
                            case 'C': return owner.getClass(id);
                            case 'O': return owner.getObject(id);
                            case 'I': return owner.getIndex(id);
                            case 'U': return owner.getUser(id);
                            case 'DIR': return owner.getFolder(id);
                            case 'FILE': return owner.getFile(id);
                            default: return ODANT.getConnection(oid);
                        }
                    } catch (err) {throw err;}
                }
            }
        })
    };
    get $host(){
        return this.$('host');
    }


    async getActions(type, context) {
        const host = await this.getHost();
        return host.getActions(type);
    }

    getHost(id) {
        let idx = id ? id.indexOf('H:'):-1;
        if (idx > -1)
            id = id.substr(idx + 2, 15);
        return this.$('host-' + id, async () => {
            let params = id?{path: 'H:'+id}:{};
            params.format='json';
            params.method  = 'find_item';
            params.prefix = true;
            const json = await ODANT.API.get(this.url, params);
            let path = json.result;
            let hostItem = ODANT.items[path];
            if(hostItem)
                return hostItem;
            else{
                hostItem =  new odaHost({$owner: this, id: path.substring(3)});
                const cfg = await hostItem.getConfig();
                // hostItem.$('config', cfg.$H[0]);
                ODANT.items[path] = hostItem;
                hostItem.PublicPart = await hostItem.getDomain('ROOT', 'P');
                return hostItem;
            }

        })
    }

    get config() {
        return this.$host.config;
    }
    getUser(id = this.userId) {
        return this.$("User-" + id, () => {
            return odaUser.create(this, id);
            // return Promise.resolve(new odaUser({$owner: this, id: id}));
        });
    }

    get Items() {
        return this.$host.Items;
    }

    getItems(prefixFilter, inherit) {
        return this.$host.getItems(prefixFilter, inherit);
    }
}


ODANT.Connections = new Map();
ODANT.getConnection = async function (url, uid) {
    return ODANT.cache('get-connection' + url + uid, async() => {
        if (!url || url === '') {
            url = ODANT.origin;
        }

        url = url.split('/');
        while (url.length > 3)
            url.pop();
        url = url.join('/');
        if (ODANT.clientSide){  // <-- client side
            try{
                const uid = await odaConnection.login();
                ODANT.$connection = ODANT.$connection || new odaConnection({id: url, userId: uid.result});
                const host = await ODANT.$connection.getHost();
                host.addServerEventListener((event) => {
                    alert('The session has expired');
                    document.location.reload();
                });
                ODANT.$connection.$('host', host);
                return ODANT.$connection;
            }
            catch(err){  // <-- web guest connection
                ODANT.$connection = new odaConnection({id: url, userId: "GUEST"});
                const host = await ODANT.$connection.getHost();
                ODANT.$connection.$('host', host);
                return ODANT.$connection;
            }
        }
        else{  // <-- server side
            if (!uid){
                if (!ODANT.$connection){

                    //base connection
                    ODANT.$connection = ODANT.$connection || new odaConnection({id: url});
                    const host = await ODANT.$connection.getHost();
                    ODANT.$connection.$('host', host);
                    return ODANT.$connection;
                }
            }
            else{  // <-- user connection
                let connection = ODANT.Connections.get(uid);
                if (!connection){
                    connection = new odaConnection({id: url, userId: uid});
                    ODANT.Connections.set(uid, connection);
                    const host = await connection.getHost();
                    connection.$set('Host', host);
                }
                return connection;
            }

        }
        return ODANT.$connection;
    });
};



ODANT.checkSsid = function() {
    return ODANT.API.get(document.origin + '/api', {method: 'check_ssid'});
};



ODANT.restart = () => {
    let url = new URL(document.location);
    if (url.searchParams.get('authMode') === 'pin') {
        url.searchParams.delete('authMode');
        document.location.replace(url);
    } else {
        document.location.reload();
    }
};

ODANT.showLogin = async function () {
    const form = await ODA.createComponent('/web/core/user/lib/user-button/login-form.html');
    return ODA.showDialog(form);
};
if (ODANT.clientSide && !globalThis.odaAuth) {
    globalThis.odaAuth = {
        loginServerUrl: '',
        storage: new odantLocalStorage('security'),
        goToLogin() {
            const params = new URLSearchParams(location.search);
            const redirect_url = params.get('redirect_url') || document.URL;
            document.location.replace(`${ODANT.origin}/web/core/host/client/login-form/index.html${redirect_url ? `?redirect_url=${redirect_url}` : ''}`);
        },
        async checkLogin(login, loginOrigin) {
            try {
                loginOrigin = loginOrigin || await this.findAccount(login);
                await ODANT.API.request(loginOrigin + '/api', { method: 'check_login', login: login });
                return true;
            }
            catch (e) {
                return false;
            }
        },
        checkSsid() {
            return ODANT.API.get(ODANT.origin + '/api', { method: 'check_ssid' });
        },
        async registration(login, password, type) {
            const loginOrigin = await this.findAccount(login);
            try {
                await this.checkLogin(login, loginOrigin);
            } catch (e) {
                throw new Error('login is busy');
            };
            const passHash = Sha256.hash(password);

            const res = await ODANT.API.request(loginOrigin + '/api', { method: 'registration', login: login, type: type, referrer: document.URL }, passHash);
            this._redirectBack();
        },
        async authentication(login, password) {
            const loginOrigin = await this.findAccount(login);
            const passHash = Sha256.hash(password);

            let ticket = await ODANT.API.request(loginOrigin + '/api', { method: 'authentication', login: login }, passHash);
            this.storage.setValue('login', login);
            ticket = await this.authorize(ticket);
            this._redirectBack();
        },
        async loginIn(force) {
            try {
                const res = await ODANT.API.request(ODANT.origin + '/api', { method: 'check_ssid' });
                // this._redirectBack();
            }
            catch (e) {
                const login = force ? '' : this.storage.getValue('login');
                if (login) {
                    const ticket = this.storage.getValue('ticket');
                    if (ticket) return this.authorize(ticket);
                    return this.findAccount(login);
                }
                else if (window.top === window) {
                    try {
                        return await this.goToLogin();
                    }
                    catch (err) {
                        this.loginAsGuest(this.storage);
                    }
                }
            }
        },
        async loginAsGuest() {
            this.storage.clear();
            this.storage.setValue('login', 'GUEST');
            this._redirectBack();
        },
        async authorize(ticket) {
            const ts = JSON.stringify(ticket);
            try {
                const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'authorize' }, ts);
                const ssid = response.result;
                this.storage.setValue('ssid', ssid);
                this.storage.setValue('ticket', ticket);
                return this.loginIn();
            }
            catch (e) {
                ODANT.push(e);
                this.storage.clear();
            }
        },
        async findAccount(login, type = 'mail') {
            const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'find_account', login: login, type: type });
            return response.result;
        },
        async logout() {
            try {
                await ODANT.API.request(ODANT.origin + '/api', { method: 'logout' });
            } catch (err) { console.log(err) };
            this.storage.clear();
            this._redirectBack();
        },
        _redirectBack() {
            const params = new URLSearchParams(location.search);
            const redirect_url = params.get('redirect_url') || '/web';
            if (document.URL !== redirect_url) location.replace(redirect_url);
        },
        async getMethods() {
            try {
                return await ODANT.API.request(ODANT.origin + '/api', { method: 'get_methods' });
                // return ['pap', 'ldap'];
            } catch (err) {
                console.warn(err);
                return null;
            }
        }
    };
    (async () => {
        odaAuth.loginServerUrl = await odaAuth.findAccount();
    })()
}