'use strict';
globalThis.odaDomain =  globalThis.odaDomain ||  class odaDomain extends odaClass {
    static get properties() {return {
        isAbstract:{
            hide: true
        },
        prefix: 'D',
        zIndex: 60,
        _color: 'gray',
        subIcon: null,
        defaultViewName: {
            category: 'Views',
            get() {
                return this.getAttr('default-class-view', 'desktop');
            },
            set(v){
                this.setAttr('default-class-view', v);
            }
            // return (this.Root && this.Root['default-class-view']) || 'desktop';
        },
        itemTypeLabel () {
            return this.isSystem ? 'system' : 'domain';
        },
        icon () {
            return this.isSystem ? 'icons:lock' : this.url + '/~/icon.png';
        },
        isSystem () {
            return this.id === '000000000000000';
        },
        type: {
            get() {
                return this.getAttr('Type');
            },
        },
    }}
    get objectCount() {
        return 0;
    }
    async _createDomain(name, prefix, body) {
        let type;
        prefix = (prefix || '').toUpperCase();
        switch (prefix) {
            case 'B': {
                type = 'BASE';
            } break;
            case 'M': {
                type = 'MODULE';
            } break;
            case 'P': {
                type = 'PART';
            } break;
            // case 'S': {
            //     type = 'SOLUTION';
            // } break;
            case 'W': {
                type = 'WORKPLACE';
            } break;
        }

        const res = await this.API.POST('create_domain', {name: name || 'New domain', type: type, prefix: true}, body);
        this.reset();
        const cfg = await this.getConfig();
        this.$set('config', cfg);
        const domain = await this.getDomain(res.i, prefix);
        this.fire('update');
        this.fire('create', domain);
        return domain;

    }

    get $LINK() {
        return (this.Root.$LINKS && this.Root.$LINKS.length && this.Root.$LINKS[0].$LINK) || [];
    }
    get Links() {
        return this.$("Links", () => {
            return this.$LINK.map(i => {
                return this.$base.findItem(i.link || i.cid).then(item => {
                    return new odaLinkItem({ Root: i, sourceItem: item, $owner: this });
                }).catch(() => {
                    return new odaLinkItem({ Root: i, $owner: this });
                });
            })
        })
    }

    get $links() {
        return Promise.all(this.Links);
    }

    assignUsers(ids) {
        const body = ids.join(' ');
        if (body) return this.API.POST('add_users', {}, body);
    }

    unassignUsers(ids) {
        const body = ids.join(' ');
        if (body) return this.API.POST('remove_users', {}, body);
    }

};
odaDomain.register();

'use strict';
globalThis.odaBase =  globalThis.odaBase || class odaBase extends odaDomain {
    static get properties() {return {
        prefix: 'B',
        icon() {
            if (this.type === 'USER') {
                //TODO: получить объект пользователя, получить от него аватар
            }
            if (this.config && this.config.ico) {
                if (this.config.ico.includes(':'))
                    return this.config.ico;
                return this.url + '/~/' + this.config.ico;
            }
            if (this.config && this.config.$B)
                return 'odant:sub-base';
            return 'odant:base';
        },
        itemTypeLabel: 'base',
        _color: 'brown',
        zIndex: 70,
        subIcon() {
            return this.$('subIcon', () => {
                if (this.config && this.config.ico)
                    return this.url + '/~/'+this.config.ico;
                return ''
            })
        }
    }}
    constructor(p){
        super(p);
        if (!ODANT.webMode)
            this.$connection.unlisten(this,  this.eventHandler_);
    }
    eventHandler(event){
        if (ODANT.webMode) {
            const getText = obj => {
                return obj.value.split('\n').map(s => {
                    try {
                        if (s.includes('(')) {
                            const path = s.split('(')[1].split(')')[0];

                            let url = new URL(path);
                            if (url.protocol === 'file:')
                                url = url.pathname.substring(url.pathname.indexOf('/web/'));
                            else
                                url = url.pathname;

                            url = ODANT.origin + url;
                            return s.replace(path, url);
                        }
                    }
                    catch (e) {

                    }
                    //return s.replace(/https?\:\/\/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\:\d{1,5}/gi, ODANT.origin);
                    return s.replace(/https?\:\/\/127.0.0.1\:\d{1,5}/gi, ODANT.origin);
                }).join('\n');
            };
            switch(event.operation) {
                case 'delete':
                    this.$owner.reset();
                    this.fire('deleted', this);
                    break;
                case 'update':
                    this.load();
                    break;
                case 'create':
                    this.reset();
                    this.fire('update');
                    break;
                case 'jscript':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            const text = getText(i);
                            console.log('%coda-server', 'font: bold; color: white; background-color:blue; padding: 1px 8px;', text);
                        }
                    });
                    break;
                case 'jscript-log':
                    event.$additional.forEach(i=>{
                        if (i.value){
                            const text = getText(i);
                            if (text.includes('Error'))
                                console.error('%coda-server', 'font: bold; color: white; background-color:red; padding: 1px 8px;', text);
                            else
                                console.log('%coda-server', 'font: bold; color: white; background-color: silver; padding: 1px 8px;', text);

                        }
                    });
                    break;
                case 'jscript-warn':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            const text = getText(i);
                            console.warn('%coda-server', 'font: bold; color: white; background-color:orange; padding: 1px 8px;', text);
                        }
                    });
                    break;
                case 'jscript-error':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            const text = getText(i);
                            console.error('%coda-server', 'font: bold; color: white; background-color:red; padding: 1px 8px;', text);
                        }
                    });
                    break;
            }
        }
    }

    get $system() {
        return this.$('$system', () => {
            return this.findItem('D:000000000000000');
        })
    }

    get $security() {
        return this.$('$security', async () => {
            const sys = await this.$system;
            return sys.findItem('C:000000000000200');
        })
    }

    // get $users() {
    //   //TODO: исправить выдачу объектов пользователей на сервере
    //   //Fix
    //   const fieldId = 'i';
    //   const fieldName = 'n';
    //
    //   return this.API.GET('get_users').then(data => {
    //     const list = data.$USERS[0].$U;
    //     return this.findItem('D:000000000000000/C:000000000000200').then((usersCls) => {
    //
    //       return usersCls.$objects.then((users) => {
    //         return users.map((user) => {
    //           const data = { ...user.Root };
    //
    //           if (!data.hasOwnProperty('name')) {
    //             const userWithName = list.find(u => u[fieldId] === user.id);
    //             if (userWithName[fieldName]) data.name = userWithName[fieldName];
    //           }
    //
    //           return new odaUser({
    //             $owner: this,
    //             body: {
    //               $USER: [data],
    //             },
    //           });
    //         });
    //       });
    //     });
    //   });
    // }

    get $users() {
        return this.$security.then(async (s) => {
            return await s.$users;
        });
    }
    get $assignedUsers() {
        return this.$security.then(async (s) => {
            return await s.$assignedUsers;
        });
    }

    static create(owner, root) {
        return new odaBase(params);
    }
    getBase(id){
        // if (id === this.id)
        //     return this;
        return this.getDomain(id, 'B');
    }
    get BaseCount() {
        return this.$('BaseCount', ()=>{
            if(this.config){
                return this.config.$B && this.config.$B.length || 0;
            }
        })
    }
    get WorkPlaces() {
        return this._getItems('W');
    }
    get Bases() {
        return this._getItems('B');
    }
    get Domains() {
        return this._getItems('D');
    }
    get Modules() {
        return this._getItems('M');
    }
    get $items() {
        return this.Bases;
    }
    get $domains() {
        return Promise.all([this.Bases, this.WorkPlaces, this.Modules]).then(([bases, workPlaces, modules]) => {
            return [].concat(bases, workPlaces, modules);
        });
    }
    get $workPlaces() {
        return this.WorkPlaces;
    }
    // get Solutions() {
    //     return this._getItems('S');
    // }

    async getUser(uid){
        return (await ODANT.API.get(this.url + '/U:' + uid)).$USER[0];
    }
    getServices(context) {
        return this.$(`services: ${context.itemType}`, async ()=>{
            const modules = await this.Modules;
            const p = modules.map(module => module.getServices(context));
            let res = await Promise.all(p);
            res = res.reduce((a, b) =>{
                a.push(...b);
                return a;
            }, []);
            return res;
        })
    }
};
odaBase.register();

'use strict';
globalThis.odaModule =  globalThis.odaModule ||  class odaModule extends odaDomain {
    static get properties() { return {
        category: 'modules',
        prefix: 'M',
        icon: 'odant:module',
        zIndex: 30,
        itemTypeLabel: 'module',
        _color: 'magenta',
        defaultViewName: 'module-info'
    }}
    getServices(context) {
        return this.$('services: '+context.itemType, async ()=>{
            const classes = await this.Classes;
            let p = classes.map(cls => cls.getActions('*', context));
            const services = await Promise.all(p);
            const res = services.reduce((res, actions)=>{
                res.push(...actions.filter(i => i.allowExport));
                return res;
            }, []);
            return res;
        })
    }
};
odaModule.register();

'use strict';
globalThis.odaWorkPlace =  globalThis.odaWorkPlace || class odaWorkPlace extends odaDomain {
    static get properties() {
        return {

            prefix: 'W',
            zIndex: 50,
            _color: 'blue',
            itemTypeLabel: 'work place',
        }
    }

    get $assignedUsers() {
        const xq = `for $a in //*[roles/@role='${this.id}'] return element U { $a/(@oid) }`;

        return this.$base.$security.then((s) => {
            return s.getIndex('pack').then((i) => {
                return i.XQuery(xq).then((ls) => {
                    return Promise.all(((ls && ls.$U) || []).map((el) => s.getObject(el.oid)));
                });
            });
        });
    }

    async createLink(cls) {
        const classId = cls.id;
        const res = await this.API.POST('create_link', {classId});
        this.reset();
    }
    async setAccessLevel(links = []) {
        if (links.length > 0) {
            return await this.API.POST('set_access_level', {body: links});
        }
    }
    // added
    async setHide(item, value = "False") {
        // if (links.length > 0) {
        //     return await this.API.POST('set_hide', { body: links });
        // }
        if (this.Root && this.Root.$LINKS && this.Root.$LINKS.length) {
            const links = this.Root.$LINKS[0].$LINK;
            if (links && links.length) {
                const link = links.find(l => l.id === item.id);
                link.Hide = value;
                this.save();
            }
        }
    }
    checkHide(item) {
        if (this.Root && this.Root.$LINKS && this.Root.$LINKS.length) {
            const links = this.Root.$LINKS[0].$LINK;
            if (links && links.length) {
                const link = links.find(l => l.id === item.id);
                return ODANT.getBool(link && (link.hide || link.Hide));
            }
            return false;
        }
    }
    ///////
};
odaWorkPlace.register();
//
// 'use strict';
// globalThis.odaSolution =  globalThis.odaSolution || class odaSolution extends odaDomain {
//     static get properties() { return {
//         prefix: 'S',
//         zIndex: 5,
//         _color: 'blue',
//         defaultViewName () {
//             return this.getAttr('default-class-view', 'ribbon');
//         },
//         itemTypeLabel: 'work place',
//         icon: 'odant:workplace'
//     }}
// };
// odaSolution.register();


'use strict';
globalThis.odaPart =  globalThis.odaPart || class odaPart extends odaBase {
    static get properties() { return {
        isSystem: true,
        prefix: 'P',
        icon () {
            let lowerId = this.id.toLowerCase();
            switch (lowerId) {
                case 'root':
                    return 'odant:root';
                case 'work':
                    return 'odant:structure';
            }

            return lowerId;
        },

        itemTypeLabel () {
            return this.id.toLowerCase();
        },
    }}
    get access() {
        // if (this.id === 'ROOT')
        //     return new odaAccess('T');
        return super.access;
    }

    async getServices(type, context) {
        return [];
    }
    get $security() {
        return this.$host.$security;
    }
};
odaPart.register();
