'use strict';

globalThis.odaField = globalThis.odaField || class odaField extends odaClassItem {
    execute(params) {
        ODA.notify(this.displayLabel);
    }

    reset(prop) {
        const _name = this._cache.name;
        super.reset(prop);
        this._cache.name = _name;
        this.$owner.reset(prop);
    }

    async createField(attrs) {
        if (!attrs)
            throw new Error('No attrs for create field');
        if (!attrs.Name)
            throw new Error('No attrs.Name for create field');
        this.Root.$ATTR = this.Root.$ATTR || [];
        if (this.ATTRS.find(i => i.Name === attrs.Name))
            throw new Error(`Field named ${attrs.Name} already exists`);
        this.ATTRS.push(attrs);
        const field = await odaField.create(this, attrs);
        this.reset();
        this.fire('create', field);
        return field;
    }

    static get properties() {
        return {
            _hasList() {
                return this._isLink || (this.TypeClass && this.TypeClass.hasChildren) || this.ValueList;
            },
            _isLink() {
                return !this.isTable && this.TypeClass && !this.TypeClass.isAbstract;
            },
            _isStructure() {
                if (!this.isTable) {
                    if (this.TypeClass) {
                        if (this.TypeClass.isAbstract && this.TypeClass.ATTRS.length > 0) {
                            return true;
                        }
                    }
                    if (this.ATTRS.length > 0) return true;
                }
                return false;
            },
            // defaultEditor: '/web/lib/type-editors/string/string-editor.html',
            // oda_editorUrl: '',
            // oda_viewerUrl: '',
            // oda_extendedEditorUrl: '',
            testFormat: {
                readOnly: true
            },
            _actionsPath: 'web/class/fields',
            itemTypeLabel() {
                if (this.isTable)
                    return 'table';
                return 'field'
            },
            Config: null,
            _labelTop() {
                // return Boolean(this.validate);
                return true;
            },
            isStatic() {
                if (this.$owner instanceof odaClass) {
                    return this.$owner.STATIC && this.$owner.STATIC.$ATTR && this.$owner.STATIC.$ATTR.some(s => s === this.Root);
                }
                return this.$owner.isStatic;
            },
            icon() {
                return this.$('icon', () => {
                    if (this.isTable)
                        return 'odant:grid';
                    if (this.isLink)
                        return 'icons:link';
                    if (this.TypeClass && this.TypeClass.icon !== 'odant:class' && this.TypeClass.icon !== 'odant:abstract-class')
                        return this.TypeClass.icon;
                    if (this.hasFields)
                        return 'odant:fields';
                    return 'odant:field';
                })
            },
            hasFields() {
                return (this.TypeClass && this.TypeClass.isAbstract && this.TypeClass.$FIELDS.hasFields) || (this.ATTRS && (this.ATTRS.length > 0));
            },
            isLink() {
                return (this.TypeClass && !this.TypeClass.isAbstract);
            },
            subIcon() {
                return this.$('subIcon', () => {
                    if (this.TypeClass)
                        return this.TypeClass.icon;
                    return null;
                })
            },
            name() {
                return this.$('name', () => {
                    return this.getAttr('Name');
                })
            },
            id() {
                return 'FIELD:' + this.name;
            },
            url() {
                return this.$owner.url;
            },
            label: {
                get() {
                    return this.getAttr('Label', this.name);
                },
                set(v) {
                    this.setAttr('Label', v);
                }
            },
            placeholder: {
                get() {
                    return this.getAttr('placeholder|Prompt')
                },
                set(v) {
                    this.setAttr('placeholder', v)
                }
            },
            required: {
                type: Boolean,
                get() {
                    return this.getBoolAttr('Required');
                },
                set(v) {
                    this.setBoolAttr('Required', v);
                }
            },
            path() {
                return this.$('path', () => {
                    if (this.$owner instanceof odaField)
                        return this.$owner.path + '/' + this.name;
                    return this.name;
                })
            },
            type: {
                // type_editor: TE_TYPE_SELECTOR,
                editor: '/web/core/structure/lib/type-editor/type-editor.html',
                get() {
                    return this.getAttr('Type');
                },
                set(v) {
                    // this.$owner.reset('fields');
                    this.setAttr('Type', v);
                    this.debounce('set-TypeCLass', async () => {
                        try {
                            this.reset();
                            if (v) {
                                const cls = await this.$owner.$class.findItem(v);
                                if (cls) this.$set('TypeClass', cls);
                            }
                        } catch (e) {
                            console.error(e);
                        } finally {
                            this.fireChanged();
                        }
                    }, 300);
                    // this.fire('deleted');
                    // this.$owner.fire('create', this);
                }
            },
            TypeClass() {
                return this.$('TypeClass');
            },
            Format: {
                get() {
                    return this.getAttr('format', this.getAttr('Format'));
                },
                set(value) {
                    this.setAttr('format', value);
                }
            },
            Mode: {
                get() {
                    return this.getAttr('Mode');
                },
                set(value) {
                    this.setAttr('Mode', value);
                }
            },
            isTable: {
                type: Boolean,
                description: `When 'true' it works like data table`,
                get() {
                    return this.getBoolAttr('List');
                },
                set(v) {
                    this.setBoolAttr('List', v);
                }
            },
            autoExec: {
                type: Boolean,
                description: `If true, the field create method will be executed on “Object create”.`,
                get() {
                    return this.getBoolAttr('autoExec', false);
                },
                set(v) {
                    this.setBoolAttr('autoExec', v);
                }
            },

            // defaultViewName() {
            //     if (this.isTable)
            //         return this.type ? 'simple' : undefined;
            //     return 'simple';
            // },
            //
            // extendedViewName: {
            //     get() {
            //         if (this.isTable)
            //             return this.getAttr('TableView', 'table');
            //         if (this.ATTRS.length > 0)
            //             return 'extended';
            //         if (this.TypeClass && this.TypeClass.isAbstract && this.TypeClass.$FIELDS.hasFields)
            //             return 'extended';
            //         return '';
            //     },
            //     // noinspection JSAnnotator
            //     set(v) {
            //         this.setAttr('extended-view', v);
            //     }
            // },
            _input() {
                if (this.isTable) return '';
                if (this._isStructure) return '';
                if (this.isLink) return 'link';
                if (this.TypeClass && this.TypeClass.hasChildren) return 'link';
                return 'default';
            },
            _complex() {
                if (this.isTable)
                    return this.getAttr('TableView', 'table');
                if (this._isStructure) return 'structure';
                if (this.isLink) return 'link';
                if (this.TypeClass && this.TypeClass.hasChildren) return 'structure';
                return;
            },
            readOnly: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Read only expression',
                description: 'XQuery expression to read only state',
                get() {
                    return this.getAttr('ReadOnly');
                },
                set(v) {
                    this.setAttr('ReadOnly', v);
                }
            },
            ValueList: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Value list expression',
                description: 'XQuery expression to dropdown value list',
                get() {
                    return this.getAttr('ValueList');
                },
                set(v) {
                    this.setAttr('ValueList', v);
                }
            },
            onlyValueList: {
                type: Boolean,
                label: 'only value list values',
                description: 'allow only value list values',
                get() {
                    return this.getBoolAttr('onlyValueList');
                },
                set(v) {
                    this.setBoolAttr('onlyValueList', v);
                }
            },
            _simpleModuleName: 'field-control',
            _extModuleName() {
                return this.isTable ? 'table-field-control' : 'field-control'
            },
            // HideOnGrid: {
            //     get() {
            //         return this.getBoolAttr('HideOnGrid');
            //     },
            //     set(v) {
            //         this.setBoolAttr('HideOnGrid', v);
            //     }
            // },
            hideOnEdit: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Hide on editor expression',
                get() {
                    return (this.getAttr('meta:HideOnEdit') || this.getAttr('HideOnEdit'));
                },
                set(v) {
                    this.setAttr('meta:HideOnEdit', v);
                }
            },
            Necessary: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                description: 'Determines the need to fill this field',
                get() {
                    return this.getAttr('Necessary');
                },
                set(v) {
                    this.setAttr('Necessary', v);
                }
            },
            Note: {
                get() {
                    return this.getAttr('Hint', '');
                },
                set(v) {
                    this.setAttr('Hint', v);
                }
            },
            XValue: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Value expression',
                description: 'XQuery expression to calculate field value',
                get() {
                    return this.getAttr('XValue');
                },
                set(v) {
                    this.setAttr('XValue', v);
                    if (v && !this.readOnly) {
                        this.readOnly = true;
                    }
                }
            },
            Suffix: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Suffix field expression',
                description: 'XQuery expression for field suffix value',
                get() {
                    return this.getAttr('meta:Suffix');
                },
                set(v) {
                    this.setAttr('meta:Suffix', v);
                }
            },
            XNode: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Node build expression',
                description: 'XQuery expression for build nodes',
                get() {
                    return this.getAttr('XNode', '');
                },
                set(v) {
                    this.setAttr('XNode', v);
                }
            },
            Default: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Default expression',
                description: 'XQuery expression to calculate default field value',
                get() {
                    return this.getAttr('Default', '');
                },
                set(v) {
                    this.setAttr('Default', v);
                }
            },
            XNodeFilter: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                description: 'XQuery expression for XNode',
                get() {
                    return this.getAttr('XNodeFilter');
                },
                set(v) {
                    this.setAttr('XNodeFilter', v);
                }
            },
            validate: {
                category: 'Expressions',
                editor: '/web/lib/inputs/expression/expression.html',
                label: 'Validate expression',
                description: 'XQuery expression to validate field value',
                get() {
                    return this.getAttr('meta:validate', this.getAttr('meta:Validate'));
                },
                set(v) {
                    this.setAttr('meta:validate', v);
                }
            },
            // возможно  для слудующих 3 свойств требуется добавить проверку: является ли isTable
            disableAdd: {
                category: 'Limitations',
                type: Boolean,
                get() {
                    return this.getBoolAttr('DisableAdd');
                },
                set(v) {
                    this.setBoolAttr('DisableAdd', v);
                }
            },
            disableClone: {
                category: 'Limitations',
                type: Boolean,
                get() {
                    return this.getBoolAttr('DisableClone');
                },
                set(v) {
                    this.setBoolAttr('DisableClone', v);
                }
            },
            disableRemove: {
                category: 'Limitations',
                type: Boolean,
                get() {
                    return this.getBoolAttr('DisableRemove');
                },
                set(v) {
                    this.setBoolAttr('DisableRemove', v);
                }
            }
            ////////
        }
    }

    get Root() {
        const attr = this.$owner.Root.$ATTR;
        if (attr) {
            return attr.find(i => i.Name === this.name) || super.Root;
        }
        return super.Root;
        // // return this.$('root', ()=>{
        //     const path = this.path.split('/');
        //     const p = path.shift();
        //     let root;
        //     switch (p) {
        //         case '[FIELDS]':
        //             root = this.$class.$FIELDS.Root;
        //             break;
        //         case '[STATIC]':
        //             root = this.$class.$STATIC.Root;
        //             break;
        //         case '[DETAILS]':
        //             root = this.$class.$DETAILS.Root;
        //             break;
        //     }
        //     while (path.length){
        //         if (!root.$ATTR)
        //             return null;
        //         const step = path.shift();
        //         root = root.$ATTR.find(i => i.Name === step);
        //         console.log('get root')
        //     }
        //     return root
        //
        // // })
        //     || super.Root;
    }

    static async create(owner, attr) {
        // if (owner instanceof odaExtFields)
        //     owner = owner.$owner;
        let p = { $owner: owner, Root: attr, name: attr.Name };
        if (attr.Type) {
            let type = attr.Type;
            if(/^\/?P:ROOT/.test(type)) type = `${owner.$host.fullId}/${type}`.replace(/\/\//g, '/');
            try {
                p.TypeClass = (await owner.$base.findItem(type)) || (await ODANT.findItem(type));
            } catch (err) {
                console.warn(err);
            }
        }
        let key = (ODANT.webMode ? window.location.origin : ODANT.origin) + '/web/core/_items/field.js';
        if (p.TypeClass) {
            let link = p.TypeClass.url;
            if (p.TypeClass.isAbstract) {
                link += '/~/fields/field.js'
            }
            else {
                link += '/~/fields/links/field.js'
            }
            try {
                const script = await ODANT.loadScript(link);
                key = script.src;
            }
            catch (e) {
                console.error(this, e);
                p.error = e;
            }
        }
        return odaItem.factory.create(key, p);
    }

    get $group() {
        return this.$('$group', () => {
            return this.$owner instanceof odaExtFields ? this.$owner : this.$owner.$group
        })
    }

    get fields() {
        return this.$('fields', async () => {
            return odaClass._getFields.call(this, '$ATTR');
        })
    }

    get hasFields() {
        return this.getElements('$ATTR').length > 0 || (this.TypeClass && this.TypeClass.isAbstract && this.TypeClass.hasFields);
    }

    _getObject(root) {
        if (root && root.oid) {
            const bid = root.bid || this.$base.id;
            const cid = root.cid || this.$class.id;
            return this.$(`B:${bid}/C:${cid}/O:${root.oid}`, async () => {
                try {
                    const $class = (await this.$connection.getItem(`B:${bid}/C:${cid}`)) || this.$class;
                    const obj = await $class.getObject(root.oid);
                    return obj;
                } catch (e) {
                    try {
                        const obj = await this.TypeClass.getObject(root.oid);
                        return obj;
                    } catch (e) {
                        return null;
                    }
                }
            });
        }
    }

    getValueList(odaObject, dataRoot) {
        if (this._isLink) {
            return this.TypeClass.getIndex('table').then(idx => {
                const ds = idx.getDataSet();
                return ds.Items.then(res => {
                    return res;
                })
            })
        }
        else if (this.TypeClass && this.TypeClass.hasChildren) {
            return this.TypeClass.$classes;
        }
        return odaObject && odaObject.getValueList(this, dataRoot || odaObject.Root);
    }

    getMetadata(attr, odaObject, dataRoot, idx) {
        return odaObject && odaObject.getMetadata(this, attr, dataRoot || odaObject.Root, idx);
    }

    /** @param {odaObject} $object @param {Object} currentRoot */
    getOwnerRoot($object, currentRoot) {
        if (!currentRoot) return null;
        const getRoots = (node, path) => {
            const step = `$${path.shift()}`;
            const cPath = Array.from(path);
            const subNodes = node[step];
            if (!subNodes || path.length === 0) return subNodes;
            return subNodes.reduce((res, n) => {
                const roots = getRoots(n, cPath);
                if (roots) res.push(...roots);
                return res;
            }, []);
        };
        const path = this.path.split('/').slice(1, -1); // срезаем [FIELDS] и fieldName
        if (path.length === 1) return $object.Root;
        const name = path.pop();
        const root = $object.Root;

        const roots = getRoots(root, path);
        return roots.find(r => {
            const root = r[`$${name}`];
            if (Array.isArray(root)) {
                return root.some(r => r === currentRoot);
            }
            return false;
        });
    }

    async getReadOnly(odaObject, root) {
        if (!odaObject || odaObject.readOnly && ODANT.getBool(odaObject.readOnly))
            return true;
        const res = ODANT.getBool(await this.getMetadata('ReadOnly', odaObject, root, 0));
        if (res || this.$owner instanceof odaExtFields) return res;
        return await this.$owner.getReadOnly && this.$owner.getReadOnly(odaObject, this.getOwnerRoot(odaObject, root));
    }

    async getNecessary(odaObject, root) {
        if (this.Necessary) {
            if (this.Necessary.toLowerCase() === 'true') {
                return true;
            } else if (this.Necessary.toLowerCase() === 'false') {
                return false;
            }
        }

        if (!odaObject)
            return true;
        const res = await this.getMetadata('Necessary', odaObject, root, 0);
        return ODANT.getBool(res);
    }

    async getDescription(odaObject, root) {
        if (!odaObject)
            return '';
        return this.getMetadata('Description', odaObject, root, 0);
    }

    getSuffix(odaObject, root) {
        if (!odaObject || odaObject.readOnly)
            return '';
        return this.getMetadata('Suffix', odaObject, root, 0);
    }

    async getValidate($object, root) {
        if (!$object || $object.readOnly)
            return null;
        if (this.ValueList && this.onlyValueList) {
            const value = await this.getValue($object, root);
            if (!value) return null;
            const list = await this.getValueList($object, root);
            const fn = (v) => v === value || (typeof v === 'object' && v.value === value);
            if (!list.some(fn)) return `Only values from the value list are allowed.`;
        }
        return this.getMetadata('validate', $object, root, 0) || this.getMetadata('Validate', $object, root, 0);
    }

    async getHidden(odaObject, root) {
        if (!odaObject)
            return false;
        const res = await this.getMetadata('HideOnEdit', odaObject, root, 0);
        return ODANT.getBool(res);
    }

    getValue($object, root) {
        if ($object)
            return $object.getValue(this, root);
        return (root && root[this.name]);
    }

    getFormattedValue($object, root, format) {
        return this.getValue($object, root)
    }
    async setValue($object, value, root) {
        if ($object)
            return $object.setValue(this, value, root);
        throw new Error('No context odaObject');
    }

    getCDATA(odaObject, root) {
        if (odaObject) {
            return odaObject.getCDATA(this, root);
        }
        return JSON.selectSingleNode(root, this.name + '/CDATA()')
    }

    setCDATA(odaObject, value, root) {
        if (odaObject) {
            odaObject.setCDATA(this, value, root);
        }
    }

    addRow(param = {}) {
        if (param.odaObject) {
            return param.odaObject.addRow(this.name, param.root);
        }
    }

    copyRows(param = {}) {
        if (param.odaObject) {
            return param.odaObject.copyRows(this.name, param.rows, param.root);
        }
    }

    delRow(param = {}) {
        if (param.odaObject) {
            return param.odaObject.delRow(this.name, param.row, param.root);
        }
    }

    getDataContext(param = {}) {
        if (param.odaObject)
            return Promise.resolve(param.odaObject);
        else if (param.root && param.root.oid) {
            return this.$class.getObject(param.root.oid)
        }
    }

    remove() {
        const idx = this.$owner.ATTRS.findIndex(i => i.Name === this.name);
        if (idx >= 0) {
            this.$owner.ATTRS.splice(idx, 1);
            this.fire('deleted', this);
        }
    }

    get disabled() {
        return this.$owner.disabled || this.$owner.ATTRS.find(i => i.Name === this.name) !== this.Root;
    }

    get ATTRS() {
        return this.getAttr('$ATTR', []);
    }

    _countCalc(items) {
        return items.length;
    }
    /** @returns {string} */
    async getLinkFields() {
        const val = this.LinkFields;
        if (!val) {
            return (await this.fields).reduce((res, i) => res += `@${i.name}, `, '').slice(0, -2);
        } else if (val === '*') {
            return '@*, *';
        }
        return val;
    }
    /** @param {odaObject} $object */
    async fillLinkRoot($object, dataRoot) {
        const xq = `element OBJECT {OBJECT/(${await this.getLinkFields()})}`;
        const res = await $object.XQuery(xq);
        const data = Array.isArray(res.$OBJECT) && res.$OBJECT[0] || {};
        ODANT.mixin(data, dataRoot);
    }

    // get oda_editor(){
    //     return this.$('oda_editor', async () => {
    //         if(this.oda_editorUrl){
    //             const editor = await ODA.createComponent(this.oda_editorUrl);
    //             if (editor){
    //                 return editor.localName;
    //             }
    //         }
    //         return '';
    //     })
    // }
    //
    // get oda_viewer(){
    //     return this.$('oda_viewer', async () => {
    //         if(this.oda_viewerUrl){
    //             const editor = await ODA.createComponent(this.oda_viewerUrl);
    //             if (editor){
    //                 return editor.localName;
    //             }
    //         }
    //         return '';
    //     })
    // }
    //
    // get oda_extendedEditor(){
    //     return this.$('oda_extendedEditor', async () => {
    //         if(this.oda_extendedEditorUrl){
    //             const editor = await ODA.createComponent(this.oda_extendedEditorUrl);
    //             if (editor){
    //                 return editor.localName;
    //             }
    //         }
    //         return '';
    //     })
    // }
    //
    //
    // get editorUrl(){
    //     return `${(this.TypeClass || this).url}/~/fields/views/${this.defaultViewName}/${/* this._simpleModuleName */'filed-control'}.html`;
    // }
    // get editorId(){
    //     return this.$('editor-id', async () => {
    //         return ODA.loadComponent(this.editorUrl);
    //     })
    // }
    //

    get input() {
        return this.$('input', async () => {
            if (this._input)
                return await ODA.loadComponent(`${(this.TypeClass || this).url}/~/fields/inputs/${this._input}/${this._input}.html`);
        })
    }

    get complex() {
        return this.$('complex', async () => {
            if (this._complex)
                return await ODA.loadComponent(`${(this.TypeClass || this).url}/~/fields/complex/${this._complex}/${this._complex}.html`);
        })
    }
    //
    // get preview() {
    //     if(this.defaultViewer){
    //         return this.$('preview', async () => {
    //             return ODA.createComponent(this.defaultViewer);
    //         })
    //     }
    // }
    //
    // get newEditor() {
    //     return this.$('new-editor', async () => {
    //         return ODA.createComponent(this.defaultEditor);
    //     })
    // }
    //
    // get extEditor() {
    //     return this.$('ext-editor', async () => {
    //         return this._getEditor(true);
    //     })
    // }
    //
    // async _getEditor(extended) {
    //     const viewName = extended ? this.extendedViewName : this.defaultViewName;
    //     if (viewName) {
    //         const moduleName = extended ? this._extModuleName : this._simpleModuleName;
    //         const context = this.TypeClass || this;
    //         const url = context.url + '/~/fields/views/' + viewName + '/' + moduleName + '.html';
    //         return ODA.createComponent(url, {field: this, isExtended: extended});
    //     }
    // }

    getRelativePath(field) {
        if (!(field instanceof odaField) || field.isStatic || this.isStatic) {
            return this.path;
        }

        if (field === this) {
            return this.name;
        }

        let path = this.path;
        const contextPath = field.path;

        if (path > contextPath && path.startsWith(contextPath)) {
            return path
                .replace(contextPath, '')
                .substr(1);
        } else if (field.$owner === this.$owner) {
            return this.name;
        }

        const steps = path.split('/');
        const contextSteps = contextPath.split('/');
        let length = contextSteps.length - 1;

        while (length && steps[0] === (contextSteps[contextSteps.length - length + 1])) {
            length--;
            steps.shift();
        }

        return '../'.repeat(length) + steps.join('/');
    }

    findField(path) {
        return odaClass.findField.call(this, path);
    }

    fireChanged() {
        super.fireChanged();
        this.$class.fireChanged();
    }
    async exec(list) {
        if (this.isLink) {
            const actions = await this.getActions('links');
            const action = actions.find(a => {
                return a.name === this.linkMethod;
            });
            return action.execute(list);
        }
    }
};

odaField.register();

globalThis.odaExtFields = globalThis.odaExtFields || class odaExtFields extends odaField {
    constructor(p) {
        super(p);
    }
    get disabled() {
        return this.$owner.disabled;
    }
    static get properties() {
        return {
            name: '[FIELDS]',
            TypeClass: this.$owner,
            icon: 'fields',
            label: "FIELDS",
            url() {
                return this.$owner.url;// + '/class.oml'
            }
        }
    }
    get Root() {
        // return this.$('root', () => {
        return this.$owner.getElement('$METADATA')
        // })
    }
    async createField(attrs) {
        if (!this.Root) {
            this.$owner.Root.$METADATA = [{}];
            this.fireChanged();
        }
        return super.createField(attrs);
    }
};

odaExtFields.register();

globalThis.odaStatic = globalThis.odaStatic || class odaStatic extends odaExtFields {
    static get properties() {
        return {
            name: '[STATIC]',
            subIcon: 'system',
            label: "STATIC",
            itemTypeLabel: 'static'
        }
    }

    get $object() {
        return this.$('object', () => this.$owner.getObject(this.$owner.id))
    }

    get Root() {
        return this.$('root', () => this.$owner.getElement('$STATIC'))
    }

    async createField(attrs) {
        if (!this.Root) {
            this.$owner.Root.$STATIC = [{}];
            this.fireChanged();
        }

        if (!attrs)
            throw new Error('No attrs for create field');
        if (!attrs.Name)
            throw new Error('No attrs.Name for create field');
        this.Root.$ATTR = this.Root.$ATTR || [];
        if (this.ATTRS.find(i => i.Name === attrs.Name))
            throw new Error(`Field named ${attrs.Name} already exists`);
        this.ATTRS.push(attrs);
        const field = await odaField.create(this, attrs);
        this.reset();
        this.fire('create', field);
        return field;
    }
};

odaStatic.register();

globalThis.odaDetails = globalThis.odaDetails || class odaDetails extends odaStatic {
    static get properties() {
        return {
            name: '[DETAILS]',
            subIcon: 'info',
            label: "DETAILS",
            itemTypeLabel: 'details'
        }
    }

    get Root() {
        return this.$('root', () => this.$owner.getElement('$DETAILS'))
    }
};
odaDetails.register();