'use strict';
globalThis.odaFile = globalThis.odaFile || class odaFile extends odaClassItem {
    get $folders() {
        return [];
    }
    get $files() {
        return [];
    }
    static get properties() {
        return {
            isSystem() {
                return this.$owner.name === '~';
            },
            defaultViewName: 'code-editor',
            name() {
                return this.$('name', () => {
                    let n = this.id;
                    let i = n.lastIndexOf('/');
                    if (i > 0)
                        n = n.substring(i + 1);
                    i = n.lastIndexOf('.');
                    if (i > 0)
                        n = n.substring(0, i);
                    return n;
                })
            },
            _actionsPath() {
                return 'web/file/.' + this.ext + '/' + (ODANT.webMode ? 'client' : 'server')
            },
            label: {
                get() {
                    return this._label || this.id;
                },
                set(value) {
                    this._label = value;
                    //todo set label
                    //this.super('label', value);
                }
            },
            defaultIcon: 'files:file',
            itemTypeLabel() {
                return `file: ${this.sizeText}`;
            },
            icon() {
                return `files:${this.ext}`;
            },
            ext() {
                return this.$('ext', () => {
                    let i = this.id.lastIndexOf('.') + 1;
                    return i ? this.id.substr(i) : '';
                })
            },
            displayLabel() {
                if (this.$dataOwner)
                    return (this.itemTypeLabel + ': ' + this.path + ' [' + this.$dataOwner.pathLabel + ']');
                return (this.itemTypeLabel + ': ' + this.path);
            },
            id() {
                return this.$('id', () => {
                    return this.getAttr('name');
                })
            },
            path() {
                return this.$('path', () => {
                    let path = this.Root && this.Root.path;
                    if (!path) {
                        let i = this;
                        while (i && i instanceof odaFile) {
                            i = i.$owner;
                        }
                        path = this.url.replace(i.url + '/', '');
                    }
                    return path;
                })
            },
            size() {
                return this.$('size', () => {
                    return this.Root ? parseInt(this.Root.size || '') : '';
                })
            },
            sizeText() {
                if (this.size && this.size > 0) {
                    let v = this.size;
                    let pcs = ' b';
                    if (v > 999 && v < 1000000) {
                        v = Math.round(v / 10) / 100;
                        pcs = ' Kb'
                    }
                    else if (v < 1000000000) {
                        v = Math.round(v / 10000) / 100;
                        pcs = ' Mb'
                    }
                    else if (v < 1000000000000) {
                        v = Math.round(v / 10000000) / 100;
                        pcs = ' Gb'
                    }
                    return v.toLocaleString() + pcs;
                }
            },
            time() {
                return this.Root.time || '';
            },
            inherit() {
                return this.$('inherit', () => {
                    const root = this.Root || this.config || {};
                    let level = root.inherit || 0;
                    return new odaInherit(level);
                })
            },
            _downloadFiles() { return ['doc', 'docx', 'csv', 'xls', 'xlsx', 'ppt', 'pptx'] }
        }
    }
    get Parent() { // <-- todo непонятный код
        let source = null;
        switch (this.inherit.level) {
            case odaInherit.type: source = this.$dataOwner.TypeClass; break;
            case odaInherit.parent: source = this.$dataOwner.Parent; break;
            case odaInherit.owner: source = this.$dataOwner.$owner; break;
            case odaInherit.core: source = this.$dataOwner.Core; break;
            case odaInherit.core: source = this.$dataOwner.Core; break;
        }
        if (!source)
            return Promise.reject();
        if (source instanceof odaItem)
            source = Promise.resolve(source);
        if (!source.then)
            return Promise.reject();
        return source.then(item => {
            return item.getFile(this.path);
        })
    }
    // get XML() {
    //     return this.API.GET('xquery', { format: 'xml'});
    // }
    static create(owner, params = { id, root }) {
        // console.log('create file', owner, params.Root);
        const id = params.id || (params.root && params.root.name);
        return owner.$('file: ' + id, async () => {
            async function createFile(ext = '') {
                try {
                    const path = owner.$class.url + '/~/web/file' + ext + '/file.js';
                    const script = await ODANT.loadScript(path);
                    return odaItem.factory.create(script.src, { $owner: owner, Root: params.root, id: id });
                }
                catch (err) {
                    console.error(err, id);
                    return createFile();
                }
            }
            if (owner.$class && id !== 'file.js' && owner.path !== '~/web/file/') {
                const ext = id.includes('.') ? ('/.' + id.split('.').pop()) : '';
                return createFile(ext);
            }
            return new odaFile({ $owner: owner, Root: params.root, id: id });
        })
    }
    remove() {
        return this.API.GET('delete').then(result => {
            this.fire('deleted', this);
            return result;
        })
    }
    async rename(name) {
        await this.API.POST('rename', {}, name);
        this.fireChanged();
        return name;
    }
    async move(path) {
        await this.API.POST('move', {}, path);
        this.fireChanged();
        return path;
    }
    async copy(path) {
        await this.API.POST('copy', {}, path);
        this.fireChanged();
        return path;
    }

    _save(data) {
        data = data || this.data;
        if (data) {
            return this.$owner.saveFile(data, this.id, this.path.includes('/web/')).then(() => {
                this.reset('load');
                this.fireChanged();
                this.$owner.fireChanged(this);
                if (this.$dataOwner)
                    this.$dataOwner.fireChanged(this);
                return this;
            });
        }
        // throw new Error('no data for save');
    }
    set data(data) {
        this.$set('data', data);
        this.fireChanged();
    }

    get data() {
        return this.$('data');
    }

    async execute(downloadFileName) {
        if (this._downloadFiles.includes(this.ext.toLowerCase())) {
            return this.executeAction('download', {downloadFileName});
        }
        return this.show();
    }
}
odaFile.register();


