'use strict';
globalThis.odaFolder =  globalThis.odaFolder || class odaFolder extends odaFile {
    static get properties(){return {
        icon: 'odant:folder',
        _fill(){
            if (this.inherit.level)
                return this._color;
            return '#fdad3d'
        },
        defaultViewName: 'files',
        label(){
            if(this.$owner === this.$dataOwner)
                return this.$owner.label+'/~';
            return this.id;
        },
        _actionsPath(){
            return 'web/folder/' + (ODANT.webMode ?'client' : 'server')
        },
        subIcon(){
            return this.$('subIcon', async ()=>{
                if (this.name === 'web')
                    return 'globe.png';
                try{
                    const file = await this.getFile('icon.png');
                    return file.url;
                }
                catch(err){
                    return null;
                }
            })

        },
        path(){
            return this.$('path', ()=>{
                let i = this;
                while(i && i instanceof odaFolder){
                    i = i.$owner;
                }
                return this.url.replace(i.url+'/', '');
            })
        },
        itemTypeLabel(){ //todo бред
            if (this.$owner instanceof odaFolder){
                switch (this.name){
                    case 'server':
                    case 'components':
                    case 'client':
                    case 'fields':
                        return 'modules';

                }
            }
            switch (this.$owner.name){
                case 'server':
                case 'components':
                case 'client':
                    return 'actions';
                case 'fields':
                    return 'field module';
                case 'web':
                    return 'item folder';
            }

            switch (this.$owner.itemTypeLabel){
                case 'actions':
                    return this.$owner.name + ' action';
            }

            return 'folder';
        }
    }}
    get $folder() {
        return this;
    }
    async execute(){
        if (ODANT.webMode){
            try{
                let context = ODANT.getContextFromUrl(window.location.href);
                const item = await ODANT.findItem(context);
                if (item instanceof odaConnection){
                    window.location.href = window.location.href.replace('/~/', this.fullId+'/~/');
                }
                else if (this.$owner === item || this.$dataOwner === item){
                    window.location.href = window.location.href.replace(context, this.fullId);
                }
                else{
                    this.formsRun();
                }
            }
            catch (e) {
                this.formsRun();
            }
        }
    }
    get hasChildren(){
        return this.Root && (this.Root.files || this.Root.dirs);
    }

    get ChildrenCount(){
        return this.$('ChildrenCount', ()=>{
            let count = 0;
            if(this.Root){
                if(this.Root.$FILE)
                    count += this.Root.$FILE.length;
                if(this.Root.$DIR)
                    count += this.Root.$DIR.length;
            }
            return count || '';
        })
    }

    findByMask(mask = '*', deep = 0){
        return this.API.GET('get_dirlist', {mask: mask, deep: deep});
    }
    getFolderInfo(path = '*', deep = 0){
        return ODANT.API.get(this.url+'/'+path,  {method: 'get_dirlist', deep: deep});
    }

    getFilesAndFolders(path = '*', deep = 0) {
        return this.$(`get-files-and-folders: ${path}: ${deep}`, async () => {
            path = path.split('/');
            path = path.filter(i=>i);
            const name = path.pop();
            let folder = this;
            if (path.length)
                folder = await this.getFolder(path.join('/'), true);

            const json = (folder.Root && folder.Root.$FILE && folder.Root.$DIR && deep === 0)?folder.Root:await folder.findByMask(name, 3);
            const func = async (owner, root, deep)=>{

                try{
                    if (root){
                        let files = (root.$FILE || []).map(f => {
                            return odaFile.create(owner, {root: f})
                        });
                        files = await Promise.all(files);
                        let dirs = (root.$DIR || []).map(async f =>{
                            const dir = odaFolder.create(owner, {Root: f});
                            if (deep){
                                return await func(dir, f, deep - 1);
                            }
                            return [dir];
                        });
                        dirs = await Promise.all(dirs);
                        dirs = dirs.reduce((res, f)=>{
                            res.push(...f);
                            return res;
                        },[]);
                        dirs.push(...files);
                        return dirs;
                    }
                }
                catch (e) {
                    console.error(e);
                }

                return [];
            };
            return await func(folder, json, deep);
        });
    }
    getFiles(path = '*', deep = 0) {
        return this.$('get-files: ' + path, async () => {
            const files = await this.getFilesAndFolders(path, deep);
            return files.filter(i => !(i instanceof odaFolder));
        })
    }
    getFolders(path = '*', deep = 0) {
        return this.$(`get-folders: ${path}: ${deep}`, async () => {
            const files = await this.getFilesAndFolders(path, deep);
            return files.filter(i => i instanceof odaFolder);
        })
    }

    get $folders(){
        return this.getFolders();
    }
    get $files(){
        return this.getFiles();
    }

    getFile(path, force){
        if(this.$owner.itemTypeLabel === 'object') this.reset();
        return this.$('get-file: '+ path, async()=>{
            path = path.split('/');
            path = path.filter(i=>i);
            const name = path.pop();
            const folder = (path.length>0)?await this.getFolder(path.join('/'), force):this;
            const files = await folder.getFiles('*');
            let file =  files.find(i=> i.id.toLowerCase() === name.toLowerCase());
            if (!file) {
                if (!force)
                    throw new Error(`File '${name}' not found in ${folder.path}`);
                file = await odaFile.create(folder, {id: name});
                if (typeof force !== 'boolean'){
                    return file.save(force).then(()=>{
                        this.fire('create', file);
                        return file;
                    });
                }
            }
            return file;
        })
    }

    getFolder(path, force){
        return this.$('get-folder:'+path, async ()=>{
            path = path.split('/');
            path = path.filter(i=>i);
            const name = path.pop().toLowerCase();
            const folder = (path.length>0) ? await this.getFolder(path.join('/'), force) : this;
            const folders = await folder.getFolders('*');
            let res =  folders.find(i=> i.id.toLowerCase() === name);
            if (!res){
                if (!force)
                    throw new Error(`Folder '${name}' not found in ${folder.path}`);
                res = await odaFolder.create(folder, {id: name});
            }
            return res;
        })
    }

    static create(owner, params = {id, Root}){
        const id = params.id || (params.Root && params.Root.name);
        const folder = owner.$('folder: ' + id, ()=>{
            return new odaFolder({$owner: owner, Root: params.Root, id: id});
        });
        return folder;
    }


    async createFolder(path, inherit = false) {
        await this.saveFile('', path + '/temp', inherit);
        this.reset();
        const folder = await this.getFolder(path, true);
        this.fire('update');
        const file = await folder.getFile('temp');
        file.remove();
        this.fire('create', folder);
        return folder;

    };

    async checkFileExists(path){
        const list = await ODANT.API.request(this.url+'/'+path, {method: 'get_dirlist'});
        if(list.$FOLDER[0].$DIR || list.$FOLDER[0].$FILE)
            return true;
        throw new Error('file not found');
    }
}

