// registrator ---------------------------------------------------------------------------------------------------------
// registrator ---------------------------------------------------------------------------------------------------------
'use strict';
globalThis.ATTR_CLASS = globalThis.ATTR_CLASS || class ATTR_CLASS{
    constructor(key, def, type){
        this.key = key;
        this.default = def;
        this.type = type;
    }
};

globalThis.CACHE_CLASS = globalThis.CACHE_CLASS || class CACHE_CLASS{
    constructor(key, def){
        this.key = key;
        this.default = def;
    }
};

globalThis.odaModuleFactory = globalThis.odaModuleFactory || class odaModuleFactory{
    constructor(){
        this.map = new Map();
    }
    add(key, callback) {
        key = this._url2key(key);
        if (!this.map.get(key)) {
            this.map.set(key, callback);
        }
    }
    create(key, params) {
        if (typeof key === 'object') {
            params = key;
            key = params.key;
        }
        key = this._url2key(key || (this.webMode ? (document.currentScript.src || document.currentScript.baseURI) : root.currentScript));
        if (params)
            params.key = null;
        let createFunc = this.map.get(key);
        if(createFunc){
            return createFunc(params);
        }else{
            throw `no create func from ${key}`;
        }
    }
    _url2key (url) {
        return url && decodeURI(url).toLowerCase();
    }
};
globalThis.itemCounter = ODANT.counter();
globalThis.odaRegister = globalThis.odaRegister || class odaRegister {
    constructor(p) {
        this.constructor.register();
        this._cache = {};
        if (p) {
            for (let i in p) {
                this._setPropertyValue(i, p[i])
            }
        }
        this._itemCode = itemCounter();
    }

    static get _resetExcludes(){
        return ['$owner', 'id', 'Root', 'API', 'url', 'fullId', 'body', '_objects', '_indexes', '$folder']
    }
    _setPropertyValue(name, value) {
        let proto = this.__proto__;
        while (proto) {
            let pd = Object.getOwnPropertyDescriptor(proto, name);
            if (pd) {
                if (pd.set)
                    this[name] = value;
                else {
                    this._cache[name] = value;
                    this.$(name, value);
                }

                return;
            }
            proto = proto.__proto__;
        }
        if (this[name] === undefined) {
            Object.defineProperty(this, name, {
                enumerable: true,
                configurable: true,
                get () {
                    return this.$(name, value);
                }
            });
        }
    }
    $(key, callback) {
        let result = this._cache[key];
        if (result === undefined) {
            if (typeof callback === 'function')
                result = callback();
            else
                result = callback;
            this._cache[key] = result
        }
        return result;
    }

    $set(key, value) {
        this._cache[key] = value;
    }
    static register() {
        let myProps = this.properties;
        if (this.modules.includes(myProps))
            return;
        this.modules.push(myProps);
        let ownProps = this.prototype.properties;

        for (let i in myProps) {
            let prop = myProps[i];
            if (prop === null)
                prop = myProps[i] = {value: prop};
            // else if(prop.hide){
            //     delete myProps[i];
            //     continue;
            // }
            switch (typeof prop) {
                case 'object':{
                    let func;
                    if(prop instanceof ATTR_CLASS){
                        func = function(){
                            return this.getAttr && this.getAttr(prop.key, prop.default);
                        }
                    }
                    else if(prop instanceof CACHE_CLASS){
                        func = function(){
                            return this.$(prop.key, prop.default);
                        }
                    }
                    if(func){
                        prop = myProps[i] = {get: func, key: prop.key, default: prop.default};
                    }
                } break;
                case 'function': {
                    prop = myProps[i] = {get: prop};
                } break;
                default: {
                    prop = myProps[i] = {value: prop};
                } break;
            }
            let description = {
                enumerable: true,
                configurable: true
            };
            if (prop.get){
                if(prop.get instanceof ATTR_CLASS){
                    description.get = function(){
                        return this.getAttr && this.getAttr(prop.get.key, prop.get.default);
                    };
                }
                else if(prop.get instanceof CACHE_CLASS){
                    description.get = function(){
                        return this.$(prop.get.key, prop.get.default);
                    };
                }
                else
                    description.get = prop.get;
            }
            else if (prop.value !== undefined) {
                description.value = prop.value;
            }

            if (prop.set) {
                if(prop.set instanceof ATTR_CLASS){
                    description.set = function(v){
                        return this.setAttr && this.setAttr(description.set.key, v);
                    };
                }
                else
                    description.set = prop.set;
            }
            else{
                prop.readOnly = true;
            }

            Object.defineProperty(this.prototype, i, description);
            if (ownProps) {
                let own = ownProps[i];
                for (let j in own) {
                    if (!prop[j])
                        prop[j] = own[j];

                }
            }
        }
        if (myProps && ownProps) {
            for (let i in ownProps) {
                if (!myProps[i])
                    myProps[i] = ownProps[i];
            }
        }
        Object.defineProperty(this.prototype, 'properties', {
            enumerable: true,
            configurable: true,
            value: myProps
        });

        let key = (ODANT.root && ODANT.root.currentScript) || (document && document.currentScript && (document.currentScript.src || document.currentScript.baseURI));
        if (key) {
            key = decodeURI(key);
            this.factory.add(key.toLowerCase(), params => {
                let res = new this(params);
                res.src = key;
                res.folder = params.folder;
                return res;
            })
        }
        return this;
    }
    static get factory() {
        if (!this.__factory)
            this.__factory = new odaModuleFactory();
        return this.__factory
    }

    static get modules() {
        if (!this.__modules)
            this.__modules = [];
        return this.__modules;
    }
};




globalThis.odaInherit = globalThis.odaInherit || class odaInherit{

    constructor(level = 0){
        if (Number.isNaN(Number(level)))
            this._level = odaInherit.levels[level] || 0;
        else
            this._level = Number(level);
    }
    static get levels(){
        return {
            self: 0,
            none: 0,
            owner: 1,
            parent: 2,
            type: 3,
            core: 4,
            service: 5
        }
    }
    static get none(){
        return odaInherit.levels.none;
    }
    get yes(){
        return this.level > odaInherit.none;
    }
    static get self(){
        return odaInherit.levels.self;
    }
    static get owner(){
        return odaInherit.levels.owner;
    }
    static get parent(){
        return odaInherit.levels.parent;
    }
    static get type(){
        return odaInherit.levels.type;
    }
    static get core(){
        return odaInherit.levels.core;
    }
    static get service(){
        return odaInherit.levels.service;
    }

    get isSelf(){
        return this.level === odaInherit.levels.self;
    }
    get isOwner(){
        return this.level === odaInherit.levels.owner;
    }
    get isParent(){
        return this.level === odaInherit.levels.parent;
    }
    get isType(){
        return this.level === odaInherit.levels.type;
    }
    get isCore(){
        return this.level === odaInherit.levels.core;
    }
    get isService(){
        return this.level === odaInherit.levels.service;
    }

    get color () {
        switch(this.level){
            case odaInherit.parent:
                return 'blue';
            case odaInherit.type:
                return 'green';
            case odaInherit.core:
                return 'gray';
            case odaInherit.owner:
                return 'orangered';
            case odaInherit.service:
                return 'red';
            default:
                return 'black'
        }
        return 'transparent'
    }
    get level () {
        return this._level;
    }
    get label () {
        return odaInherit.getLabel(this.level);
    }
    static getLabel(level = odaInherit.none){
        switch (level) {
            case odaInherit.type:
                return 'type';
            case odaInherit.parent:
                return 'parent';
            case odaInherit.owner:
                return 'owner';
            case odaInherit.core:
                return 'core';
            case odaInherit.service:
                return 'service'
        }
        return 'self';
    }
};

globalThis.odaAccess = globalThis.odaAccess || class odaAccess{
    constructor(level = 0){
        if (Number.isNaN(Number(level)))
            this._level = odaAccess.getLevel(level) || 0;
        else
            this._level = Number(level);
    }
    static get levels(){
        return {
            none: 0,
            template: 1,
            read: 2,
            write: 3,
            create: 4,
            delete: 5,
            admin: 6
        }
    }
    static get none(){
        return odaAccess.levels.none;
    }
    static get template(){
        return odaAccess.levels.template;
    }
    static get read(){
        return odaAccess.levels.read;
    }
    static get write(){
        return odaAccess.levels.write;
    }
    static get create(){
        return odaAccess.levels.create;
    }
    static get delete(){
        return odaAccess.levels.delete;
    }
    static get admin(){
        return odaAccess.levels.admin;
    }
    get symbol(){
        switch (this.level) {
            case odaAccess.write:
                return 'W';
            case odaAccess.create:
                return  'C';
            case  odaAccess.delete:
                return 'D';
            case odaAccess.admin:
                return 'A';
            case odaAccess.read:
                return 'R';
        }
        return '';
    }
    get readOnly(){
        return this.level < odaAccess.getLevel('W');
    }
    allow(access){
        return this.level >= odaAccess.getLevel(access);
    }
    get allowTemplate(){
        return this.allow('T');
    }
    get allowRead(){
        return this.allow('R');
    }
    get allowWrite(){
        return this.allow('W');
    }
    get allowCreate(){
        return this.allow('C');
    }
    get allowDelete(){
        return this.allow('D');
    }
    get allowAdmin(){
        return this.allow('A');
    }
    static getLevel (val) {
        let access = odaAccess.none;
        if (val) {
            switch (typeof val) {
                case 'object':
                    return val.level;
                case 'number':
                    return val;
                case 'function':
                    return odaAccess.getLevel(val());
                case 'string': {
                    access = parseInt(val);
                    if (isNaN(access)) {
                        access = odaAccess.none;
                        val = val.toLowerCase();
                        for (let i in val){
                            let level = odaAccess.none;
                            switch (val[i]){
                                case 'a': level = odaAccess.admin; break;
                                case 'd': level = odaAccess.delete; break;
                                case 'c': level = odaAccess.create; break;
                                case 'w': level = odaAccess.write; break;
                                case 'r': level = odaAccess.read; break;
                                case 't': level = odaAccess.template; break;
                            }
                            access = Math.max(access, level);
                        }
                    }
                } break;
            }
        }
        return access;
    }
    get level () {
        return this._level;
    }
    get icon () {
        return 'symbols:'+this.symbol;
    }
    get label () {
        switch (this.level) {
            case odaAccess.template:
                return 'template';
            case odaAccess.read:
                return 'read';
            case odaAccess.write:
                return 'write';
            case odaAccess.create:
                return 'create';
            case odaAccess.delete:
                return 'delete';
            case odaAccess.admin:
                return 'admin';
        }
        return 'none';
    }
    get color () {
        switch (this.level) {
            case odaAccess.template:
                return 'gray';
            case odaAccess.read:
                return 'gray';
            case odaAccess.write:
                return 'blue';
            case odaAccess.create:
                return 'green';
            case odaAccess.delete:
                return 'red';
            case odaAccess.admin:
                return 'black';
        }
        return 'black';
    }
};

// class odaItem -------------------------------------------------------------------------------------------------------

globalThis.getCache = globalThis.getCache || function (key, def){
    return new CACHE_CLASS(key, def);
}

globalThis.odaItem = globalThis.odaItem || class odaItem  extends odaRegister {
    get textRoot(){
        return (this.Root && JSON.stringify(this.Root)) || '';
    }
    get isChanged(){
        return Boolean(this.data) || (this.currentState !== this.textRoot);
    }
    get $folders(){
        return this.$folder.$folders;
    }
    get $files(){
        return this.$folder.$files;
    }
    XQuery (xq, {format = 'json'}={}){
        return this.POST('xquery', {format: format, async: true}, xq)
    }
    static get Types(){
        return {
            //Base
            odaItem: 0,
            odaConnection: 1,
            odaHost: 2,
            odaStructure: 3,
            odaSecurity: 7,
            //classes
            odaClass: 100,
            odaDomain: 101,
            odaBase: 102,
            odaPart: 103,
            odaModule: 104,
            odaWorkPlace: 106,
            odaGroup: 107,
            //actions
            odaAction: 200,
            odaService: 201,
            odaView: 202,
            //files
            odaFile: 300,
            odaFolder: 301,
            //fields
            odaField: 400,
            odaExtField: 401,
            odaStatic: 401,
            odaDetails: 401,
            //ClassItem
            odaIndex: 500,
            odaObject: 600,
            //Security
            odaUser: 700,
            odaRole: 701,

            odaPack: 800
        }
    }
    static get properties() {return {
        _color(){
            if (this.adminMode)
                return this.inherit && this.inherit.color;
            return 'black'
        },
        _fill(){
            return this._color;
        },
        prefix: {
            category: 'system',
            value: ''
        },
        title(){
            return this.label + (this.$owner?(' [' + this.$owner.labelPath+']'):'');
        },
        defaultViewName: 'editor',
        extendedViewName: '',
        defaultPage: 'navigator',
        defaultForm: 'form',
        zIndex: 0,
        constructorName() {
            let p = this.constructor;
            let name = p.name;
            while (!name && p.__proto__) {
                p = p.__proto__;
                name = p.name;
            }
            return name;
        },
        itemType() {
            return odaItem.Types[this.constructorName];
        },
        itemTypeLabel () {
            return this.constructorName;
        },
        itemTypePath () {
            return this.constructorName.replace('oda', '').toLowerCase();
        },
        userId () {
            return this.$connection.userId;
        },
        name () {
            return this.getAttr('Name', this.id);
        },
        // '_error': {
        //     get: getCache('error'),
        //     set (error) {
        //         this.$set('error', error);
        //         this.fire('alert', {error: error});
        //     }
        // },
        // '_warning': {
        //     get: getCache('warning'),
        //     set (warning) {
        //         this.$set('warning', warning);
        //         this.fire('alert', {warning: warning});
        //     }
        // },
        // '_info': {
        //     get: getCache('info'),
        //     set (info) {
        //         this.$set('info', info);
        //         this.fire('alert', {info: info});
        //     }
        // },
        _actionsPath(){
            return 'web/'+this.itemTypePath +'/' + (ODANT.webMode ?'client' : 'server')
        },
        author: {
            category: 'system',
            get (){
                return this.getAttr('Author');
            }
        },
        label: {
            get () {
                return this.getAttr('Label', this.name);
            },
            set (v){
                this.setAttr('Label', v);
            }
        },
        pathLabel () {
            return this.$('pathLabel', () => {
                let owner = this.$owner;
                while (owner && owner.prefix === this.prefix) {
                    owner = owner.$owner;
                }
                if (owner && owner instanceof odaClass) {
                    let res = owner;
                    if (owner.pathLabel)
                        res = owner.pathLabel + '/' + res;
                    return res;
                }
                return '';
            });
        },
        displayLabel () {
            return this.itemTypeLabel + ': ' + this.label + (this.pathLabel?' [' + this.pathLabel + ']':'');
        },
        id: {
            category: 'system',
            readonly: true,
            get () {
                return this._cache.id;
            }
        },
        fullId: {
            category: 'system',
            get () {
                return this.$('fullId', () => {
                    let s = '';
                    let own = this;
                    let pref = '';
                    while (own && own.$owner) {
                        let step = (own.id ? own.id : null);
                        if (step && own.prefix)
                            step = own.prefix + ':' + step;
                        if (step) {
                            step = '/' + step;
                            if (!own.prefix || own.prefix !== pref) {
                                s = step + s;
                            }
                        }
                        pref = own.prefix;
                        own = own.$owner;
                    }
                    return s;
                })
            }
        },
        _fullPath:{
            get () {
                return this.$('_fullPath', () => {
                    return (this.$owner?(this.$owner._fullPath+'/'):'')+this.prefix+':'+this.id;
                })
            }
        },
        oldFullId: {
            category: 'system',
            label: 'Полный ID (устаревший)',
            get () {
                return this.$('oldFullId', () => {
                    let s = '';
                    let own = this;
                    let pref = '';
                    let type;
                    while (own && own.$owner) {
                        let step = (own.id ? own.id : null);
                        if (step && own.prefix) {
                            let convertedPrefix = own.prefix;
                            switch (convertedPrefix) {
                                case 'B':
                                case 'M':
                                case 'P':
                                case 'S':
                                case 'W':
                                    convertedPrefix = 'D';
                                    break;
                            }
                            step = convertedPrefix + ':' + step;
                        }
                        if (step) {
                            step = '/' + step;
                            if (!own.prefix || own.prefix !== pref) {
                                s = step + s;
                            }
                        }
                        pref = own.prefix;
                        own = own.$owner;
                    }
                    return s;
                });
            }
        },
        labelPath() {
            return this.$('labelPath', () => {
                if (!(this instanceof odaBase) && this.$owner)
                    return this.$owner.labelPath + ' / ' + this.label;
                return this.label;
            })
        },
        externalUrl() {
            return this.$('ExternalUrl', () => {
                if (typeof EXTERNALORIGIN === 'undefined' || EXTERNALORIGIN === null)
                    return this.url;
                return (EXTERNALORIGIN + "/api" + this.fullId);
            });
        },
        url() {
            return this.$('url', () => {
                if (this === this.$connection)
                    return this.id + "/api";
                return this.$connection.url + this.fullId;
            });
        },
        icon() {
            return this.url + '/~/icon.png';
        },
        iconBackground(){
            return this.$('icon-background', 'transparent');
        },
        iconEx() {
            return this.url + '/~/iconex.png';
        },
        backgroundImage(){
            return this.url + '/~/background.jpg';
        },
        subIcon: '',
        defaultIcon: '',
        access: {
            category: 'system',
            get () {
                return this.$('access', () => {
                    if (this.$part && this.$part.id === 'ROOT')
                        return this.$owner.access;
                    const root = this.config || this.Root ||  {};
                    let level =  root.access || root.a;
                    if (level === undefined && this.$owner)
                        return this.$owner.access;
                    return new odaAccess(level);
                });
            }
        },

        readOnly: {
            category: 'system',
            get () {
                return this.access.readOnly;
            }
        },
        inherit (){
            return this.$('inherit', () =>{
                const root = this.Root || this.config || {};
                let level =  root.inherit;
                if (level === undefined && this.$owner)
                    return this.$owner.inherit;
                return new odaInherit(level);
            })
        },
        logo () {
            return this.url + '/~/logo.png';
        },
        cover () {
            return this.url + '/~/cover.png';
        },
        viewUrl () {
            return ODANT.URL(this.url + `/~/client/${this.defaultViewName}/${this.defaultViewName}.html`)
        },
        formUrl () {
            return ODANT.URL(this.url + `/~/client/${this.defaultForm}/${this.defaultForm}.html`)
        },

        description(){
            return this.getAttr('descrioption|Note|Hint');
        },
        isRef() {
            return this.$owner && (this.$owner.isRef || this.$owner instanceof odaWorkPlace);
        }

    }}

    get data(){
        return this.$('data');
    }
    set data(value){
        this.$set('data', value);
        if (value)
            this.fireChanged();
    }

    get error(){
        return this.$('error')
    }
    set error(error){
        this.$set('error', error);
        this.fire('alert', error)
    }
    get info(){
        return this.$('info')
    }
    set info(info){
        this.$set('info', info);
        this.fire('alert', info)
    }
    get warning(){
        return this.$('warning');
    }
    set warning(warning){
        this.$set('warning', warning);
        this.fire('alert', warning)
    }

    async(callback, delay){
        setTimeout(callback, delay)
    }
    debounce(key = this.id, callback, delay){
        this._denounces = this._denounces || {};
        clearTimeout(this._denounces[key]);
        this._denounces[key] = setTimeout(callback, delay);
    }

    get adminMode(){
        return this.access.allowAdmin;
    }

    super(prop){
        if (prop){
            let p = this.__proto__.__proto__;
            let v;
            while(p && v === undefined){
                let o = Object.getOwnPropertyDescriptor(p, prop);
                if (o){
                    if (o.get)
                        v = o.get.call(this)
                    else
                        v = p[prop];
                }
                p = p.__proto__;
            }
            return v;
        }
    }


    toJSON(){
        let obj = {};
        for (let i in this){
            if (this[i] && typeof this[i] !== 'object')
                obj[i] = this[i];
        }
        return obj;
    }

    async GET(method, params = {async, method: method}) {
        params.method = params.method || method;
        return this.$connection.REQUEST(this, params);
    }

    async POST(method, params = {async, method: method}, post) {
        params.method = params.method || method;
        return this.$connection.REQUEST(this, params, post);
    }

    async EXECUTE(action, params = {}, async = false) {
        return this.POST(null, {execute: action, async: async}, params)
    }

    load(params, fetchParams) {
        return this.API.GET(undefined, params, fetchParams);
    }

    getElements(path) {
        path = path.split('/');
        return  path.reduce((a, b)=>{
            return a.reduce((i, j) =>{
                i.push(...(j[b] || []));
                return i;
            },[])
        }, [this.Root || {}])
    }
    getElement(path, idx = 0){
        return this.getElements(path)[idx];
    }
    forceElement(path, idx = 0){
        const el = this.getElement(path, idx);
        if (!el) {
            path = path.split('/');
            for(let i = 0; i < path.length; i++){
                if(!this.Root[path[i]]){

                }
            }
        }
        return el;
    }
    /** @param {string} name @param {any} def @param {Function} type @returns {string} */
    getAttr(name, def, type) {
        let res = undefined;
        if (this.Root){
            name = name.split('|');

            while(!res && name.length){
                const n = name.shift().trim();
                res = this.Root[n];
            }
            if (res === undefined || res === '' || res === null) return def;
            switch(type){
                case Boolean:{
                    res = String(res).toLowerCase() === 'true';
                } break;
                case Number:{
                    res = Number(res);
                } break;
                case Date:{
                    res = Date.parse(res);
                } break;
            }
            return res;
        }
        return def;
    }
    /** @param {string} name @param {any} v */
    setAttr(name, v) {
        if (this._itemCode && this.Root && this.Root[name] !== v) {
            if (v === undefined || v === null)
                delete this.Root[name];
            else
                this.Root[name] = v;
            this.fireChanged();
        }
    }
    /** @param {string} name @param {Boolean} def @returns {Boolean} */
    getBoolAttr(name, def = false) {
        return ODANT.getBool(this.getAttr(name), def);
    }
    /** @param {string} name @param {Boolean} v */
    setBoolAttr(name, v) {
        this.setAttr(name, v === undefined ? v : v ? 'True' : 'False');
    }
    /** @param {string} name @param {number} def @returns {Boolean} */
    getNumAttr(name, def = 0) {
        return Number(this.getAttr(name)) || def || 0;
    }
    /** @param {string} name @param {number} v */
    setNumAttr(name, v) {
        this.setAttr(name, v === undefined ? v : String(v).replace(',', '.'));
    }
    /** @param {string} name @param {Date} def @returns {Date} */
    getDateAttr(name, def) {
        return Date.parse(this.getAttr(name)) || def;
    }
    /** @param {string} name @param {Date} v */
    setDate(name, v) {
        this.setAttr(name, v === undefined ? v : v.toISOString());
    }

    get Children() {
        return [];
    }


    _reload() {

    }

    get _allowSave() {
        return this.access.allowWrite;
    }


    async remove() {
        const res = await this.API.GET('delete');
        this.$owner.reset();
        this.fire('deleted');
        return res;
    }

    async executeMethod(method, params) {
        return ODANT.executeMethod(this, method, params);
    }

    async findAction(action) {
        action = action.toLowerCase().split('/');
        const name = action.pop();
        const type = action.pop() || (ODANT.serverSide?('methods'):undefined);
        const actions = await this.getActions(type);
        const m = actions.find(i => {
            return i.name.toLowerCase() === name;
        });
        return m;
        // if (m)
        //     return m;
        // throw new Error("Action '" + type + "/" + name + "' not found!");
    }

    async executeAction(name, param) {
        const action = await this.findAction(name);
        if (action) {
            return action.execute(param);
        }
        console.warn(`Action "${name}" not found in ${this.itemTypeLabel} "${this.label}"`);
    }

    async executeServerAction(action, param) {
        return this.API.EXECUTE(action, param);
    }

    async execute(action, param) {
        if (action)
            return this.executeAction(action, param);
        else
            this.open();
    }

    getRelativeId(item) {
        if (!item || !item.fullId) {
            return null;
            // throw 'incorrect item';
        }

        let res = item.fullId;
        let cnt_id = this.fullId;
        while (cnt_id.indexOf("/") > -1) {
            res = res.replace(cnt_id + "/", '');
            cnt_id = cnt_id.substring(0, cnt_id.lastIndexOf("/"));
        }
        return res;
    }

    toString() {
        return (this.itemTypeLabel + ': ' + this.label);
    }

    get Root() {
        return this.$('Root');
    }


    get type() {
        return this.constructorName;
    }

    get $class() {
        return this.$owner && (this.$owner instanceof odaClass?this.$owner:this.$owner.$class);
    }

    get $domain() {
        return this.$owner && (this.$owner instanceof odaDomain?this.$owner:this.$owner.$domain);
    }

    get $base() {
        return this.$owner && (this.$owner instanceof odaBase?this.$owner:this.$owner.$base);
    }

    get $workGroup() {
        return this.$owner && (this.$owner instanceof odaWorkPlace?this.$owner:this.$owner.$workGroup);
    }

    get $module() {
        return this.$owner && (this.$owner instanceof odaModule? this.$owner:this.$owner.$module);
    }

    get $part() {
        return this.$owner && (this.$owner instanceof odaPart?this.$owner:this.$owner.$part);
    }

    get $host() {
        return this.$owner && (this.$owner instanceof odaHost?this.$owner:this.$owner.$host);
    }


    get ServerEvents() {
        return this.$('ServerEvents', () => {
            return new ServerEvents(this);
        });
    }

    get $owner() {
        return this._owner || this.$('$owner');
    }



    fireChanged() {
        this.debounce('fire-changed', ()=>{
            this.fire('changed', this);
        }, 10)
    }

    fire(event, detail, propagation) {
        if (event === 'update')
            this.reset();
        if (!this._timers) {
            this._timers = [];
        }
        this.debounce(event, ()=>{
            this._fireEvent(
                {
                    type: event,
                    target: this,
                    currentTarget: this,
                    detail: detail,
                    path: [],
                    propagation: propagation,
                    stopPropagation: () => {
                        this.propagation = false
                    }
                }, detail
            );
            this._fireEvent({
                type: 'event',
                source: event,
                target: this,
                currentTarget: this,
                detail: detail,
                path: [],
                propagation: true,
                stopPropagation: () => {
                    this.propagation = false
                }
            }, detail);
            if (event === 'update') {
                this.fire('change', this, true);
            }
        }, 50);
        return (this.events && this.events[event] && this.events[event].length > 0);
    }

    _fireEvent(customEvent, detail) {
        try {
            if (customEvent.path) {
                customEvent.path.push(this);
            }

            let event = this.events ? this.events[customEvent.type] : null;

            if (Array.isArray(event)) {
                event.slice().forEach((e) => {
                    e(customEvent, detail);
                });
            }
        } catch (ex) {
            console.error(ex);
        } finally {
            //todo stopPropagation()
            if (customEvent.propagation) {
                let _owner = this.$owner;

                if (_owner instanceof odaItem) {
                    customEvent = Object.assign(customEvent);
                    customEvent.target = _owner;
                    _owner._fireEvent(customEvent, detail);
                }
            }
        }
    }

    listen(event, callback) {
        if (!this.events) {
            if (this.onServerEvents) {
                this.onServerEvents(e => {
                    if (ODANT.webMode)
                        console.dir(e);
                });
            }
            this.events = {};
        }
        if (!this.events[event])
            this.events[event] = [];
        this.events[event].push(callback);
    }

    addEventListener(event, callback) {
        this.listen(event, callback);
    }

    unlisten(event, callback) {
        if (!this.events)
            return;
        if (!this.events[event])
            return;
        let i = this.events[event].indexOf(callback);
        if (i < 0)
            return;
        this.events[event].splice(i, 1);
    }

    removeEventListener(event, callback) {
        this.unlisten(event, callback);
    }

//todo переделать на fullId
    addServerEventListener(callback) {
        this.$connection.EventSource.addEventListener(this.oldFullId, callback, false);
    }

    removeServerEventListener(callback) {
        this.$connection.EventSource.removeEventListener(this.oldFullId, callback);
    }

    reset(key) {
        if (key) {
            if (!Array.isArray(key))
                key = key.split(' ');
            key.forEach(k => {
                this._cache[k] = undefined;
                delete this._cache[k];
            })
        }
        else {
            // var arr = [];
            for (let i in this._cache) {
                if (!odaItem._resetExcludes.includes(i)){
                    this._cache[i] = undefined;
                    delete this._cache[i];
                }

            }
        }
    }

    create(name, p) {
        let el = document.createElement(name);
        if (p) {
            for (let i in p)
                el[i] = p[i];
        }
        return el;
    }

    get $connection() {
        return this.$('connection', () => {
            let i = this.$owner && this.$owner.$connection;
            if (!i && this instanceof odaConnection)
                i = this;
            return i;
        })
    }

    get XML() {
        return this.API.GET(undefined, {format: 'xml'});
    }

    set XML(xml) {
        this._xml = xml;
        this.fire('changed', this);
    }


    async findItem(path) {
        return this.$('find-item:' + path, async () => {
            if (!path)
                throw 'unable to find item, path is empty';
            if (!path.includes('.')) {
                let s = path.split('/');
                if (s.length === 1) {
                    if (path.indexOf(':') === -1) {
                        path = 'C:' + path;
                    }
                    if (!path.includes('B:')) {
                        path = 'B:*/' + path;
                    }
                    if (!path.includes('P:')) {
                        path = 'P:*/' + path;
                    }
                    if (!path.includes('H:')) {
                        path = 'H:*/' + path;
                    }
                }
                const val = await this.API.GET("find_item", {path: path, chain: 'full', prefix: true})
                if (val)
                    return await ODANT.findItem(val.result);
                return null;
            }
            else {
                return await ODANT.findItem(this.fullId + '/' + path);
            }
        });
    }
    resolveUrl(path) {
        let a = (this.src || this.url).split('/');
        a.pop();
        a.push(path);
        return a.join('/');
    }


    // path must start with prefix
    getItem(path) {
        const slash = path.startsWith('/') ? '' : '/';

        return ODANT.findItem(this.fullId + slash + path);
    }

    getValue(root, key) {
        return (root || this.Root)[key];
    }

    setValue(root, value, key) {
        (root || this.Root)[key] = value;
    }

    get Views() {
        return this.getActions('odaView');
    }

    get Services() {
        return this.getActions('services');
    }

    getFormUrl(form, view) {
        const $form = `${form || this.defaultForm}/index.html`;
        const $view = view ? `?view=${view}` : '';
        const url = `${this.url}/~/client/${$form}${$view}`;

        return ODANT.URL(url);
    }

//todo непонятки

    // async loadComponent(path) {
    //     let url = path;
    //     if (!url.includes('/'))
    //         url = this.url + '/~/client/' + path + '/dom-module.html';
    //     const link = await ODANT.loadLink(url);
    //     return link.import.querySelector('dom-module').id;
    // }

    async createComponent(path) {
        if (!path.includes('/')) {
            path = this.url + `/~/lib/${path}/${path}.html`;
        }
        const control = await ODA.createComponent(path);
        control.contextItem = this;
        return control;
    }

    getViewUrl(view) {
        view = view || this.defaultViewName;
        return this.url + `/~/client/${view}/${view}.html`;
    }

    async getViewComponent(view) {
        let url = this.getViewUrl(view);
        const link = await ODANT.loadLink(url)
        return link.import.querySelector('dom-module').id;
    }


    async getView(view, layoutOnly) {
        const viewAction = await this.getViewAction(view);
        return viewAction.create(layoutOnly);
    }

    getViewId(view) {
        return this.$('view-id-'+view, async ()=>{
            const id = await ODA.loadComponent(`${(this.TypeClass || this).url}/~/client/${view}/${view}.html`);
            return id;
        })

    }


    async show(view, params = {}, dialog = false) {
        view = view || this.defaultViewName;
        const action = await this.getViewAction(view);
        const form = await ODA.createComponent(this.formUrl, params);
        form.viewName = view;
        form.contextItem = this;
        form.unique = this.fullId+"#"+form.viewName;
        return form.show(dialog);
    }
    async showDialog(view, params){
        view = view || this.defaultViewName;
        const action = await this.getViewAction(view);
        return action.showDialog(params);
    }
    async showViewAsDialog(action, params) {
        action = await this.findAction(action);
        return action.showDialog(params);
    }

    async showDialogView(view, params) {
        view = view || this.defaultViewName;
        const form = await ODA.createComponent(this.formUrl);
        form.viewName = view;
        form.params = params;
        form.contextItem = this;
        return form.showDialog();
    }
    //
    // async showModal(view) {
    //     const form = await ODA.createComponent(this.formUrl.replace('/index.', '/dom-module.'))
    //     form.viewName = view || this.defaultViewName;
    //     form.contextItem = this;
    //     return form.show();
    // }

    open(form, view) {
        const url = this.getFormUrl(form, view);
        window.open(url, '_blank');
    }

    navigate(form, view) {
        const url = this.getFormUrl(form, view);
        window.open(url);
    }

    formsRun(form, view) {
        const url = ODANT.URL(this.getFormUrl(form, view));
        ODA.showForm(url);
    }

    get Parent() {
        return this.$('Parent', async () => {
            const json = await this.API.GET('get_class_parent', {prefix: true});
            return ODANT.findItem(json.result);
        })
    }

    get Parents() {
        return this.$('Parents', async () => {
            const parent = await this.Parent;
            if (parent) {
                return parent.Parents;
            }
            return [];
        })
    }


    getViewAction(view) {
        return this.findAction('odaView/' + (view || this.defaultViewName));
    }

    getActions(type = '*', context) {
        return this.$class.getActions(type, context || this);
    }


// FILES ----------------------------------------------------------------------------------------------------

    get $folder() {
        return this.$('$folder', odaFolder.create(this, {id: '~'}));
    }

    async getFolder(path, force) {
        if (path === '~')
            return this.$folder;
        return this.$folder.getFolder(path, force);
    }

    getFolders(path, deep) {
        return this.$folder.getFolders(path, deep);
    }

    getFile(path, force) {
        return this.$folder.getFile(path, force);
    }

    getFiles(path, deep) {
        return this.$folder.getFiles(path, deep);
    }
    getFilesAndFolders(path) {
        return this.$folder.getFilesAndFolders(path);
    }

    get Files() {
        return this.getFiles();
    }

    get Folders() {
        return this.getFolders();
    }

    get FilesAndFolders() {
        return this.getFilesAndFolders();
    }
    loadFile(path, params, fetchParams) {
        return ODANT.API.get(this.url + '/' + path, params, fetchParams);
    }

    saveFile(fileBody, path = '', inherit = false, mime) {
        let name = path.split('/').pop();
        switch (fileBody.constructor.name.toLowerCase()) {
            case 'string':
                if (ODANT.webMode) {
                    fileBody = new File([fileBody], name, {type: mime || "text/plain"});
                }
                break;
            case 'array':
            case 'object':
                if (ODANT.webMode) {
                    fileBody = new File([JSON.stringify(fileBody)], name, {type: mime || "application/json"});
                }
                break;
            case 'odafile': // todo <-- Еще смотреть как определять тип файла (MIME)
                name = fileBody.id;
                return fileBody.load().then(fileBody=>{
                    return this.saveFile(fileBody, path || name);
                });
            case 'file':
            case 'blob':
            case 'readstream':
                break;
            default:
                throw new Error('Unknown file type')
                break;
        }
        name = name || fileBody.name;
        const formData = new FormData();
        formData.append("file", fileBody, name);
        const url = inherit?this.url:this.url.replaceAll('/~/', '/');
        return ODANT.API.post(url + '/' + path, undefined, formData).then((res)=>{
            this.reset();
            return this.getFile(path, true).then(file => {
                file.$owner.fireChanged(file);
                file.$owner.fire('create', file);
                return file;
            });
        })
    }

    deleteFile(path) {
        path = this.url + '/' + path;
        return ODANT.API.get(path, {method: 'delete'});
    }

    _save(data) {
        data = data || this.data;
        if (data) {
            return this.API.POST(undefined, undefined, data).then(res=>{
                this.data = null;
            })
        }
        throw new Error('no data for save');
    }

    save(data) {
        if (this.access.allowWrite) {
            if (!this.fire('before-save', this)) {
                return this._save(data).then(res=>{
                    this.currentState = null;//this.textRoot;
                    this.fire('saved', this);
                    this.fire('after-save', this);
                    return res
                })
            }
            else {
                let p = this.events['before-save'].map((i) => {
                    if (i.then)
                        return i;
                    return Promise.resolve(i());
                });
                return Promise.all(p).then(()=>{
                    return this._save(data).then(res=>{
                        this.currentState = null;//this.textRoot;
                        this.fire('saved', this);
                        this.fire('after-save', this);
                        return res
                    })
                })
            }
        }
    }

    get API() {
        return this.$('API', new API(this));
    }

}
odaItem.register();

function ServerEvents(item) {
    console.log("Item '" + item.fullId + "' subscribed to Server Events");
    item.$connection.EventSource.addEventListener('/' + item.fullId, item.fire);
}

function API(item) {
    this.$item = item;
}

API.prototype.GET = function (method = null, params ={}, fetchParams) {
    params.method = params.method || method;
    return ODANT.API.get(this.$item, params, fetchParams);
};

API.prototype.EXECUTE = function (action, param) {
    return ODANT.API.post(this.$item, {execute: action}, param);
};

API.prototype.POST = function (method = null, params = {}, post, fetchParams) {
    params.method = params.method || method;
    return ODANT.API.post(this.$item, params, post, fetchParams);
};

API.prototype.DELETE = function (method = null, params = {}) {
    params.method = params.method || method;
    return ODANT.API.delete(this.$item, params);
};

ODANT.findItem = async (path) => {
    try {
        return ODANT.getItem(path);
    } catch (err) {
        console.warn(err);
        try {
            /** @type {odaConnection} $connection */
            const $connection = await ODANT.getConnection();
            /** @type {odaHost} $host */
            const $host = await $connection.getHost(path);
            return $host.findItem(path);
        } catch (err) {
            console.warn(err, path);
            return null;
        }
    }
};

ODANT.getItem = async (path) => {
    try {
        const $connection = await ODANT.getConnection();
        return await $connection.getItem(path);
    } catch (err) {
        throw err;
    }
};

// odaConnection -------------------------------------------------------------------------------------------------------

globalThis.WebSocketEvents = globalThis.WebSocketEvents ||  class WebSocketEvents {
    constructor(url) {
        this._url = url;
        this._start();
    }

    _start() {
        try{
            this._webSocket = new WebSocket(this._url);
            this._webSocket.onopen = (event) => {
                this._onOpen(event);
            };
            this._webSocket.onclose = (event) => {
                this._onClose(event);
            };
            this._webSocket.onmessage = (event) => {
                this._onMessage(event);
            };
        }
        catch (e) {

        }

    }

    _onOpen(event) {
        this._recoverySubscribe();
        console.log('Open webSocket connection');
    }

    _onClose(event) {
        setTimeout(() => {
            this._start();
        }, 3000);
        console.log('Close webSocket connection');
    }

    _onError(event) {
        this._webSocket.close();
        console.log('Error in webSocket connection');
    }

    _onMessage(event) {
        let info = JSON.parse(event.data);
        if (info.event && (info.data || info.custom))
            this._fire(info.event, (info.data || info.custom));
        // console.log('Message in webSocket', event.data);
    }

    _fire(name, data) {
        this._callbacks = this._callbacks || {};
        let args = [].slice.call(arguments, 1);
        let callbacks = this._callbacks['$' + name];

        if (callbacks) {
            callbacks = callbacks.slice(0);
            for (let i = 0, len = callbacks.length; i < len; ++i) {
                callbacks[i].apply(this, args);
            }
        }

        return this;
    }

    _is_connected() {
        return this._webSocket && this._webSocket.readyState === 1;
    }

    _send(data) {
        if (this._is_connected()) {
            if (typeof data === "string" && data.length > 0)
                this._webSocket.send(encodeURIComponent(data));
        }
    }

    _sendSubscribe(name) {
        if (this._is_connected())
            this._send('+' + name);
    }

    _sendUnSubscribe(name) {
        if (this._is_connected())
            this._send('-' + name);
    }

    _sendUnSubscribeAll() {
        if (this._is_connected())
            this._send('-*');
    }

    _recoverySubscribe() {
        if (this._is_connected()) {
            if (this._callbacks) {
                let data;
                for (let name in this._callbacks) {
                    if (data)
                        data += '\r\n';
                    else
                        data = '';
                    data += '+' + name.substring(1);
                }
                if (data.length > 0)
                    this._send(data);
            }
        }
    }

    addEventListener(name, callback) {
        this._callbacks = this._callbacks || {};
        if ((this._callbacks['$' + name] = this._callbacks['$' + name] || [])
        .push(callback) === 1)
            this._sendSubscribe(name);
        return this;
    }

    removeEventListener(name, callback) {
        this._callbacks = this._callbacks || {};

        if (0 === arguments.length) return this;

        // specific event
        let callbacks = this._callbacks['$' + name];
        if (!callbacks) return this;

        // remove all handlers
        if (1 === arguments.length) {
            this._sendUnSubscribe(name);
            delete this._callbacks['$' + name];
            return this;
        }

        // remove specific handler
        let cb;
        for (let i = 0; i < callbacks.length; i++) {
            cb = callbacks[i];
            if (cb === callback) {
                callbacks.splice(i, 1);
                break;
            }
        }

        if (0 === callbacks.length)
            this._sendUnSubscribe(name);

        return this;
    }

    removeAllListeners() {
        this._callbacks = {};
        this._sendUnSubscribeAll();
        return this;
    }

    listeners(name) {
        this._callbacks = this._callbacks || {};
        return this._callbacks['$' + name] || [];
    }

    hasListeners(name) {
        return !!this.listeners(name).length;
    }

    get URL() {
        return this._url;
    }

    sendCustomEvent(name, eventObject) {
        if (typeof name !== "string") {
            throw Error("sendCustomEvent error: name is not string");
        }
        if (name.length == 0) {
            throw Error("sendCustomEvent error: name is empty");
        }
        let isDataString = (typeof eventObject === "string");
        let isDataObject = (typeof eventObject === "object");
        if (!isDataString && !isDataObject) {
            throw Error("sendCustomEvent error: eventObject must be string or object");
        }
        const msgObj = {
            root: {
                event: name,
                custom: JSON.stringify(eventObject)
            }
        };
        // let message = '{"root":{"event":"' + name + '","custom":';
        // if (isDataString) {
        //     message +='"' + eventObject + '"';
        // }
        // else {
        //     let eventString = `${JSON.stringify(eventObject)}'`;
        //     message += eventString;
        // }
        // message += '}}';
        this._send(JSON.stringify(msgObj));
    }
}


// odaItemList ---------------------------------------------------------------------------------------------------------

globalThis.odaItemList = globalThis.odaItemList || class odaItemList extends odaItem {
    static get properties(){
        return {
            prefix: 'List',
            fullId: '',
            subIcon(){
                const count = this.items.length;
                if(count<10)
                    return `symbols:d${count}`;
                return `symbols:d9+`;
            },
            icon: 'icons:list',
            id() {
                return this.itemType + 's [' + this.items.length + ']';
            },
            label(){
                return `${this.items.length} items`;
            },
            length(){
                return (this.items && this.items.length) || 0;
            },
            itemTypeLabel() {
                return this.label;
            },
            itemType:{
                get(){
                    return this._itemType;
                },
                set(v){
                    this._itemType = v;
                }
            }
        }
    }
    // get $owner(){
    //     return (this.$item && this.$item.$owner);
    // }
    async getItems(){
        return this.$('$items', ()=>{
            const items = this.items.map(async i=>{
                if (i instanceof odaItem)
                    return i;
                if (typeof i === 'string') {
                    switch (this.itemType){
                        case odaObject:
                            return this.$owner.getObject(i)
                    }

                }
                else if (typeof i === 'object') {
                    switch (this.itemType){
                        case odaObject:{
                            const base = (i.bid && await this.$owner.$base.findItem('B:'+i.bid)) || this.$owner.$base;
                            const cls = (i.cid && await base.findItem('C:'+i.cid)) || this.$owner;
                            return cls.getObject(i.oid);
                        }
                    }
                }
            });
            return Promise.all(items);
        });
    }
    get items(){
        return this.$('items', []);
    }
    async remove() {
        const items = await this.getItems();
        const item = items[0];
        if (item instanceof odaObject)
            return item.$class.removeObjects(items.map(i => (i.id || i)).join(' '));
        else {
            //todo доделать удаление из списка
            let p = items.map(i => {
                return i.remove().then(res => {
                    return res;
                }).catch(err => {
                    return err;
                })
            });
            return Promise.all(p);
        }
    }

    async getActions(type, context) {
        const items = await this.getItems();
        const item = items[0];
        if (item) {
            const methods = await item.getActions(type)
            let res = methods.filter(i => {
                return (i instanceof odaAction && i.allowArrayContext);
            });
            res = res.map(i => {
                let m = Object.create(i);
                m._owner = this.items.length > 1 ? this : item;
                return m;
            });
            return res;
        }
        return [];
    }
}

// odaLinkItem ---------------------------------------------------------------------------------------------------------

globalThis.odaLinkItem = globalThis.odaLinkItem || class odaLinkItem extends odaItem {
    static get properties() {
        return {
            Children () {
                if (this.sourceItem && this.sourceItem.Children) {
                    return this.sourceItem.Children.map(i => {
                        return i.then(item => {
                            return (new odaLinkItem({
                                Root: item.Root,
                                sourceItem: item,
                                $owner: (this instanceof odaLinkItem) ? this.$owner : this
                            }));
                        }).catch(() => {
                            return (new odaLinkItem({$owner: (this instanceof odaLinkItem) ? this.$owner : this}));
                        });
                    });
                }

                return [];
            },
            fullId () {
                return (this.$owner) ? this.$owner.fullId + "/" + this.prefix + ':' + this.id : undefined;
            },
            icon () {
                return (this.sourceItem) ? this.sourceItem.icon : 'icons:warning';
            },
            id () {
                return this.$("id", () => {
                    if (this.sourceItem) {
                        return this.sourceItem.id;
                    }

                    return this.Root.link.split('/').pop().split(':').pop();
                })
            },
            label () {
                return (this.sourceItem) ? this.sourceItem.label : this.Root.Label;
            },
            objectCount () {
                return (this.sourceItem) ? this.sourceItem.objectCount : null;
            },
            prefix () {
                return this.$("prefix", () => {
                    if (this.sourceItem) {
                        return this.sourceItem.prefix;
                    }

                    let p = this.Root.link.split('/').pop().split(':').shift();

                    if (p.length > 1) {
                        p = 'C';
                    }

                    return p;
                })
            },
            hide(){
                return ODANT.getBool(this.config && this.config.hide);
            },
            subIcon: 'editor:insert-link',
        }
    }
}
