'use strict';

globalThis.odaObjectType =  globalThis.odaObjectType || class odaObjectType{
    static get object(){
        return 0;
    }
    static get static(){
        return 1;
    }
    static get detail(){
        return 2;
    }
    static getLabel(type = 0){
        switch(type){
            case 0: return 'object';
            case 1: return 'static';
            case 2: return 'detail'
        }
        return 'unknown object type';
    }
}
globalThis.odaObject =  globalThis.odaObject || class odaObject extends odaItem {

    getIndex(name, XQ) {
        return this.$('Index:' + name, async () => {
            if (XQ)
                await this.API.POST('create_index', { name: name }, XQ);
            return new odaIndex({ id: name, $owner: this, XQ: XQ });
        })
    }
    constructor(p){
        super(p);
        if (this.$class)
            this.$class._objects.push(this);
    }
    get $objects(){
        return this.getIndex('table').then(idx=>{
            return  idx.XQuery(`for $a in //O[@parent='${this.id}'] return element O {$a/@oid}`).then(list=>{
                const p = ((list && list.$O) || []).map(el => this.$class.getObject(el.oid));
                return Promise.all(p);
            });
        });
    }
    static create(dataRoot, owner){
        return new odaObject({Root: dataRoot, $owner: owner});//$owner.getObject(dataRoot.oid);
    }
    static get properties(){ return {
        prefix: 'O',
        objectType(){
            return this.$('objectType', ()=>{
                if(this.id === this.$class.id)
                    return odaObjectType.static;
                if(this.id === this.$base.id)
                    return odaObjectType.detail;
                return odaObjectType.object;
            });
        },
        itemTypeLabel () {
            return odaObjectType.getLabel(this.objectType);
        },
        icon(){
            return this.$('icon', ()=>{
                let icon = this.Root.icon;
                if(icon)
                    return this.url+'/~/'+ icon;
                return 'odant:object';
            })
        },
        // subIcon(){
        //     return this.$class.icon;
        // },
        id (){
            return this.getAttr('oid');
        },
        defaultForm () {
            return this.$class.defaultObjectForm;
        },
        defaultPage () {
            return this.$class.defaultObjectPage;
        },
        defaultViewName () {
            return this.$class.defaultObjectView;
        },
        label(){
            switch (this.objectType){
                case odaObjectType.static:
                    return this.$class;
                case odaObjectType.detail:
                    return this.$base;
            }
            return this.name || 'No name';
        },
        access: {
            category: 'system',
            get() {
                return this.$owner.access;
            }
        },

        readOnly:{
            get(){
                return this.access.readOnly ||  this.getAttr('ro');
            },
            set (v){
                this.setBoolAttr('ro', v);
            }
        },
        name(){
            if(this.isNew)
                return 'New';
            return this.getAttr('name',  this.getAttr('oid'));
        },
        date (){
            return this.getDateAttr('date');
        }
    }}
    toString() {
        return this.label;
    }
    showSettings(){
        return this.$class.showSettings();
    }

    // get XML() {
    //     return this.$class.API.POST('recalc', {format: 'xml'}, JSON.stringify(this.body));
    // }

    get Root(){
        return this.$('Root', ()=>{
            return (this.body.$OBJECT || this.body.$STATIC || this.body.$DETAIL || [{}])[0];
        })
    }

    async load(params = {recalc: false, meta: false}){
        try {
            const res = await super.load(params);
            if (typeof res === 'object')
                this.body = res;
            // else
            //     this.data = res;
            return res;
        }
        catch (e) {
            this.error = e;
            console.error(e);
        }
    }
    reload(){
        return this.load({meta: true, recalc: true});
    }
    recalc(){
        this.debounce('recalc', async ()=>{
            if (this._stopRecalc) {
                this._stopRecalc = false;
                return;
            }
            if (this.readOnly) return;
            try {
                this.body = await this.$class._recalcObject(this, true);
            }
            catch (e) {
                this.error = e;
                console.error(e);
            }
        }, 300);
    }

    async save(data) {
        const fields = await this.fields;

        const emptyFlds = await this.checkNecessary(fields, this, this.Root);
        if (emptyFlds.length > 0) {
            const emptyFldNames = emptyFlds.map((f) => f.field.label).join(', ');
            const err = new Error(`The object has empty required fields: ${emptyFldNames}`);
            err.name = 'Warning';
            err.icon = 'icons:warning';
            throw err;
        }

        const notValidatedFields = await this.checkValidate(fields, this, this.Root);
        if (notValidatedFields.length > 0) {
            const notValidatedFldTxt = notValidatedFields.map((f) => `${f.field.label} - ${f.validateText}`).join(', ');
            const err = new Error(`The object has not validated fields: ${notValidatedFldTxt}`);
            err.name = 'Warning';
            err.icon = 'icons:warning';
            throw err;
        }

        data = data || this.data || JSON.stringify(this.body);
        const res = await super.save(data);
        this.currentState = null;
        this.fireChanged();
        return res;
    }
    async checkNecessary(fields, object, root) {
        const _requiredFields = fields.map(async(f) => ({field: f, required: (f.required || (await f.getNecessary(object, root)))}));
        const requiredFields = (await Promise.all(_requiredFields)).filter((o) => o.required);
        const _valRes = requiredFields.map(async (f) => ({ field: f.field, value: await f.field.getValue(object, root)}));
        const valRes = await Promise.all(_valRes);
        const emptyFlds = valRes.filter((fldValObj) => !fldValObj.value);

        const promFld = fields.map(async (fld) => {
            if (fld.isLink) return;

            const childFields = await fld.fields;
            if (childFields && childFields.length > 0) {
                const childRoots = root[`$${fld.name}`];
                if (childRoots && childRoots.length > 0) {
                    const promRoot = childRoots.map(async (r) => {
                        await emptyFlds.push(...await this.checkNecessary(childFields, object, r))
                    });
                    await Promise.all(promRoot);
                }
            }            
        });
        await Promise.all(promFld);

        return emptyFlds;
    }
    async checkValidate(fields, object, root) {
        const _notValidatedFields = fields.map(async(f) => ({field: f, validateText: (await f.getValidate(object, root))}));
        const notValidatedFields = (await Promise.all(_notValidatedFields)).filter((o) => o.validateText);

        const promFld = fields.map(async (fld) => {
            if (fld.isLink) return;

            const childFields = await fld.fields;
            if (childFields && childFields.length > 0) {
                const childRoots = root[`$${fld.name}`];
                if (childRoots && childRoots.length > 0) {
                    const promRoot = childRoots.map(async (r) => {
                        await notValidatedFields.push(...await this.checkValidate(childFields, object, r))
                    });
                    await Promise.all(promRoot);
                }
            }            
        });
        await Promise.all(promFld);

        return notValidatedFields;
    }
    
    get body(){
        return this.$('body')
    }
    set body(json){
        try{
            if (json){
                this.error = json.$Error;
                if (ODANT.webMode) {
                    json = this._mixin(json, this.body);
                }
                this.$set('body', json);
                this.reset('Root');
                if (!this.currentState)
                    this.currentState = this.textRoot;
                this.fireChanged();
            }
        }
        catch (e) {
            this.error = e;
            console.error(e);
        }
    }
    // _mixin(from, to){
    //     if (!to)
    //         return from;
    //     if (from){
    //         if (from === to)
    //             return to;
    //         for(let p in to){
    //             if(p[0] === '$'){
    //                 to[p].forEach((v, i) => {
    //                     if (from[p][i])
    //                         this._mixin(from[p][i], v);
    //                     else
    //                         delete to[p][i];
    //                 });
    //             }
    //             else if (to[p] !== from[p]) {
    //                 to[p] = from[p];
    //                 delete from[p];
    //             }
    //         }
    //         for(let p in from){
    //             if (from[p]) {
    //                 to[p] = from[p];
    //             }
    //             else{
    //                 delete to[p];
    //             }
    //         }
    //     }
    //     return to;
    // }
    remove() {
        this.API.POST('delete_object');
    }

    //СТАРЫЙ
    _mixin(from, to){
        if (!to)
            return from;
        if (from) {
            for (let p in to) {
                if (from[p] === '' || from[p] === undefined) delete to[p];
                if (p[0] !== '$' || p === '$CDATA$') {
                    to[p] = from[p];
                    delete from[p];
                }
                else if (from[p]) {
                    to[p].forEach((v, i) => {
                        if (from[p][0]) {
                            this._mixin(from[p][0], v);
                            from[p].shift();
                        }
                        else {
                            to[p].splice(i, to[p].length - i);
                        }
                    })
                }
                else {
                    delete to[p];
                }
            }
            for (let p in from) {
                if (from[p] === '' || from[p] === undefined) continue;
                if (p[0] !== '$')
                    to[p] = from[p];
                else if (to[p]) {
                    from[p].forEach(v => {
                        to[p].push(v);
                    })
                }
                else
                    to[p] = from[p];
            }
        }
        return to;
    }

    getViewUrl(view){
        view = view || this.defaultViewName;
        return this.$class.url + `/~/web/object/client/${view}/${view}.html`;
    }
    get isStatic(){
        return this.id === this.$class.id;
    }

    get isDetail(){
        return this.id === this.$base.id;
    }

    get $FIELDS(){
        if (this.isDetail)
            return this.$class.$DETAILS;
        if (this.isStatic)
            return this.$class.$STATIC;
        return this.$class.$FIELDS;
    }
    get fields(){
        return this.$class.fields;
    }

    getValueList(field, root = this.Root){
        let md = this.getMetadata(field, 'ValueList', root);
        if(Array.isArray(md)){
            for (let i = 0; i<md.length; i++){
                for (let j in md[i])
                    return md[i][j];
            }
        }
        return [];
    }

    getMetadata(field, attr, root = this.Root, idx){
        field = field.name || field;
        try{
            root = root['$meta:data'][0];
            root = root['$a:'+field][0];
            root =  root['$meta:'+attr];
           if (idx !== undefined)
                root = root[idx].value;
            return root;
        }
        catch (err){
            if (idx === undefined)
                return [];
            return null;
        }
    }
    /** @param {odaField} field @param {object} root @param {string} linkName */
    async restoreLink(field, root, linkName) {
        try {
            /**@type {odaObject[]} */
            const objects = await field.TypeClass.getObjects(`@name='${linkName}'`);
            if (objects && objects.length > 0) {
                const row = this.addRow(field, root);
                const object = objects[0];
                await field.fillLinkRoot(object, row);
                return object;
            } else {
                return linkName;
            }
        } catch (e) {
            return linkName;
        }
    }
    /** @param {odaField} field @param {Object} root @param {any} def */
    getValue(field, root, def){
        if (field.name) {
            if (!root){
                const path = field.path.split('/');
                root = this.Root;
                while (root && path.length>1){
                    const step = '$'+ path.shift();
                    let array = root[step];
                    if (array && array.length)
                        root = array[0];
                    else
                        root = null;
                }
                if (!root)
                    return null;
            }
            if (field.isTable)
                return root['$'+field.name];
            field = field.name;
        }
        else
            root = root || this.Root;

        let path = field.split('/');
        while (path.length > 0 && root) {
            field = path[0].replace('@', '');
            if (path.length > 1) {
                root = root['$' + field];
                root = root && root[0];
            }
            path.splice(0, 1);
        }
        const value = root && root[field];
        //if (!field.isLink) return value;

        let link = JSON.selectSingleNode(root,  field + '/@link'); //root && root['$'+field] && root['$'+field].length>0 && (root['$'+field][0].link || root['$'+field][0].file);
        if (link) return this.findItem(link);
        // if (value) return this.restoreLink(field, root, value);
        return value;
    }
    /** @param {odaField} field @param {any} value @param {Object} root @returns {any} */
    async setValue(field, value = null, root){
        if(typeof Blob !== 'undefined' && value instanceof Blob){
            const ext = (value.name.includes('.') && '.' + value.name.split('.').pop()) || '';
            const file = await this.saveFile(ODANT.webMode?value:(value.stream || value.buffer), ODANT.getGuid() + ext);
            file.label = value.name;
            const res = await this.setValue(field, file, root);
            await this.save();
            return res;
        }
        let name = field;
        if (field.name) {
            const path = field.path.split('/');
            if (!root){
                root = this.Root;
                while (path.length>1){
                    const step = '$'+ path.shift();
                    let array = root[step];
                    if (!array)
                        root[step] = array = [{}];
                    else if (!array.length)
                        array.push({});
                    root = array[0];
                }
            }
            name = field.name;
        }
        root = root || this.Root;

        if(!value){
            delete root[name];
            this.clearRows(field, root);
        }
        else if (typeof value === 'object') {
            this._stopRecalc = true;
            const row = {};
            if(value instanceof odaItem){
                this._stopRecalc = true;
                if(value instanceof odaObject){
                    if (!field || !field.isLink) Object.assign(row, value.Root)
                    else if (!this._hasOwnersFieldsIsLink(field)) await field.fillLinkRoot(value, row);
                }
                row.link = this.getRelativeId(value);
            }
            else if (typeof value === 'object') {
                ODANT.mixin(value, row);
            }
            this._stopRecalc = false;
            this.clearRows(field, root);
            root[name] = String(value && (value.label || value.name || value.value || value));
            Object.assign(this.addRow(field, root), row);
        }
        else{
            root[name] = String(value);
        }
        // if (before !== JSON.stringify(root))
        this.recalc();
        return value
    }
    /** @param {odaField} field */
    _hasOwnersFieldsIsLink(field) {
        const owner = field.$owner;
        if (owner instanceof odaField && !(owner instanceof odaExtFields)) {
            return owner.isLink || this._hasOwnersFieldsIsLink(owner);
        }
        return false;
    }
    addRow(field, root){
        let name = field;
        if (field.name) {
            const path = field.path.split('/');
            if (!root){
                root = this.Root;
                while (path.length>1){
                    const step = '$'+ path.shift();
                    let array = root[step];
                    if (!array)
                        root[step] = array = [{}];
                    else if (!array.length)
                        array.push({});
                    root = array[0];
                }
            }
            name = field.name;
        }
        root = root || this.Root;
        root = root['$'+name] = root['$'+name] || [];
        let n = {};
        root.push(n);
        if (field.isTable) {
            let max = root.reduce((res, i)=>{
                if (+i.id > res)
                    res = +i.id;
                return res;
            },0);
            root.filter(i=>(!i.id)).map(i=>{
                i.id = ++max;
            });
        }
        this.recalc();
        return  n;
    }
    delRow(name, row, root = this.Root){
        if(name && row) {
            let arr = root['$'+name];

            if(!arr || !arr.length){
                return;
            }

            let index = row;

            if (typeof row !== 'number')  {
                index = arr.indexOf(row);
            }
            if (index > -1) {
                let data = arr[index];
                arr.splice(index, 1);
                this.recalc();
                return {index: index, value: data};
            }
        }
    }
    getCDATA(field, root = this.Root){
        return JSON.selectSingleNode(root, (field.name || field)+'/CDATA()');
    }

    setCDATA(field, value, root = this.Root){
        field = field.name || field;
        root = root['$'+field] = (root['$'+field] || [{}])[0];
        root['$CDATA$'] = value;
        this.fireChanged();
    }


    clearRows(field, root = this.Root){
        field = field.name || field;
        if(root['$'+field]){
            delete root['$'+field];
            this.recalc();
        }

    }
    async setParent(parentObject){
        await this.setValue('parent', parentObject);
        await this.setValue('p', null); // убирает устаревшую работающую некорректно установку родителя
        return this.save();
    }
    get Parent(){
        return this.getValue('parent');
    }
    copyRows(name, rows, root = this.Root) {
        if (name && Array.isArray(rows)) {
            let copies = rows.map(row => {
                let copy = this.addRow(name, root);

                for (let p in row) {
                    if (p === 'id') {
                        continue;
                    }

                    copy[p] = row[p];
                }

                return copy;
            });
            return copies;
        }
    }
    getData (p, onReady){
        this.API.GET(null, p,  res=>{
            onReady(res);
        })
    }





};
odaObject.register();
