'use strict';
globalThis.odaStructureItem = globalThis.odaStructureItem || class odaStructureItem extends odaItem {
    static get properties() {
        return {
            id() {
                return this.$('id', () => (this.config && this.config.i))
            },
            name() {
                return this.$('name', () => ((this.config && this.config.n) || 'unknown'));
            },
            label() {
                return this.$('label', () => ((this.config && this.config.l) || this.name));
            },
            _createRules: [],
            _deleteRules: [],
            _copyRules: [],
            _moveRules: [],
            _linkRules: [],
            _inheritRules: [],
        }
    }

    _checkRule(ruleType, to) {
        try {
            if (this === to)
                throw new Error("Not allow self to self actions");
            if (to === this.$owner)
                throw new Error("Not allow self to parent actions");
            let rules = this[`_${ruleType}Rules`];
            if (!rules)
                throw new Error(`Not found rule '${ruleType}'`);
            if (!Array.isArray(rules))
                throw new Error(`Corrupted rule description for '${ruleType}'`);

            // console.log(rules);



            let fromFId = this.$owner._fullPath;
            let toFID = to._fullPath;
            // console.log(fromFId);
            // console.log(toFID);
            for (let i = 0; i < rules.length; i++) {
                let checkFrom = this._checkConditionRule(fromFId, rules[i].from);
                let checkTo = this._checkConditionRule(toFID, rules[i].to);
                if (checkFrom && checkTo) {
                    return true
                }
            }
            return false;
        }
        catch (err) {
            return false;
        }

    }

    _checkConditionRule(id, rules) {
        if (!rules) {
            return true;
        }
        let splitId = id.split('/');
        splitId.reverse();

        let splitRules = rules.split(',');
        for (let i = 0; i < splitRules.length; i++) {
            let rule = splitRules[i].split('/');
            rule.reverse();
            rule = rule.filter((item, index, arr) => !(!item.trim() && !(arr[index - 1].trim())));
            let data = this._checkCoincidence(rule, splitId);
            if (data) {
                return true;
            }
        }
        return false;
    }

    _checkCoincidence(rule, splitId) {

        let currentIndex = 0;
        let currentItem = null;


        for (let j = 0; j < rule.length; j++) {
            if (rule[j]) {
                if (splitId[currentIndex].startsWith(rule[j])) {
                    currentIndex++;
                    currentItem = rule[j];
                } else {
                    return false
                }
            } else {
                for (let i = currentIndex; i < splitId.length; i++) {
                    if (!splitId[i].startsWith(currentItem)) {
                        // currentIndex--;
                        break;
                    } else {
                        currentIndex = i;
                    }
                }

            }
        }
        return true;
    }

    getUser(id) {
        return this.$('getUser:' + id, async () => {
            return this.API.GET('find_item', { path: 'U:' + id, chain: "full", prefix: true });
        })
    }
    async getItem(id, prefix) {
        if (prefix && prefix !== 'C')
            return this.getDomain(id, prefix);
        return this.getClass(id);
    }
    getDomain(id, prefix) {
        prefix = (prefix || 'D').toUpperCase();
        id = id.split(':').pop();
        return this.$(prefix + ':' + id, async () => {
            if (!this.config || !this.config[`\$${prefix}`] || !this.config[`\$${prefix}`].some(i => i.i.toLowerCase() === id.toLowerCase()))
                await this.getConfig();
            let root = this.config[`\$${prefix}`] && this.config[`\$${prefix}`].find(i => i.i.toLowerCase() === id.toLowerCase());
            if (!root)
                return this.findItem(prefix + ':' + id);
            let params = { $owner: this, id: root.i, name: root.n, label: root.l || root.n, type: root.t, config: root };
            let item;
            switch (prefix) {
                case 'M':
                    item = new odaModule(params);
                    break;
                case 'B':
                    item = new odaBase(params);
                    break;
                case 'P':
                    item = new odaPart(params);
                    break;
                // case 'S':
                //     item= new odaSolution(params);
                //     break;
                case 'W':
                    item = new odaWorkPlace(params);
                    break;
                default:
                    item = new odaDomain(params);
                    break;
            }
            await item.load();
            return item;
        });
    }
    getClass(id) {
        id = id.split(':').pop();
        return this.$('C:' + id, async () => {
            try {
                if (!this.config || !this.config.$C || !this.config.$G)
                    await this.getConfig();
                let root = this.config.$C && this.config.$C.find(i => i.i === id) || await this.getAlienConfig('C', id);
                if (!root && this.config && this.config.$G && Array.isArray(this.config.$G)) {
                    for (let g of this.config.$G) {
                        if (g.$C && Array.isArray(g.$C)) {
                            for (let c of g.$C) {
                                const foundClass = g.$C.find(i => i.i === id);
                                if (foundClass) {
                                    root = foundClass;
                                    break;
                                }
                            }
                        }
                    }
                }
                // if (!root)
                //     return null;

                let params = { $owner: this, id: root.i, name: root.n, label: root.l || root.n, type: root.t, config: root };
                let url = ((this instanceof odaDomain) ? this : this.$domain).url + '/C:' + id + '/~/class.js';
                async function loadClass(url, params) {
                    params.key = (await ODANT.loadScript(url)).src;
                    const item = odaItem.factory.create(params);
                    await item.load();
                    return item;
                }
                try {
                    return await loadClass(url, params)
                }
                catch (err) {
                    params.error = err;
                    url = this.$connection.id + '/web/core/class/class.js';
                    return await loadClass(url, params)
                }
            } catch (err) {
                const title = `${this.label ? `"${this.label}" ` : ''}[${this.fullId}]`;
                console.warn(`problems on getting C:${id} in ${this.itemTypeLabel} ${title}`);
                return undefined;
            }
        })
    }

    async load(params = {}) {
        try {
            const res = await super.load(params);
            if (typeof res === 'object')
                this.body = res;
            else
                this.data = res;
            return res;
        }
        catch (e) {
            this.error = e;
            console.error(e);
        }
    }
    get body() {
        return this.$('body', () => this.load());
    }

    set body(json) {
        try {
            this.error = json.$Error;
            this.$set('body', json);
            this.reset();
            if (this.prefix && this.prefix.toUpperCase() === 'C') {
                const conf = this.config || this.$owner.getAlienConfig(this.prefix, this.id);
                if (conf) {
                    if (conf.then) {
                        conf.then(conf => {
                            this.$set('config', conf);
                        })
                    } else {
                        this.$set('config', conf);
                    }
                }
            }
            this.reset('Root');
            if (!this.currentState) {
                this.currentState = this.textRoot.toString();
            }

            this.fireChanged();
        }
        catch (e) {
            this.error = e;
            console.error(e);
        }
    }
    get Root() {
        return this.$('Root', () => {
            return ((this.body && this.body.$CLASS) || [{}])[0];
        })
    }
    getCategories() {
        if (this.prefix === "C") {
            return [];
        }

        const classes = this.config && this.config.$C;

        if (!Array.isArray(classes)) {
            return [];
        }

        const categories = [];
        const categoriesMap = classes.reduce(
            (m, c) => {
                if (c.c && !m[c.c]) {
                    m[c.c] = c.c;
                    categories.push(new odaCategory({ $owner: this, _id: c.c }))
                }

                return m;
            },
            {}
        );

        return categories;
    }

    getModuleGroup() {
        if (!"BP".includes(this.prefix) || !Array.isArray(this.config.$M) || !this.config.$M.length) {
            return;
        }

        return new odaModuleGroup({ $owner: this });
    }

    get config() {
        return this.$('config', () => {
            if (this !== this.$owner && this.$owner && this.$owner.id !== this.id && this.$owner.config) {
                let conf = this.$owner.config['$' + this.prefix];
                if (conf) return conf.find(i => i.i === this.id);
                console.warn('Config element not found!');
                return null;
            }
            return null;
        });
    }
    async getAlienConfig(prefix, id) {
        prefix = prefix.toUpperCase();
        const $owner = (this instanceof odaDomain) ? this : this.$domain;
        const url = `${$owner.url}/${prefix}:${id}`;
        const config = await ODANT.API.post(url, { method: 'get_config' });
        return config[`$${prefix}`][0];
    }

    async getConfig(xq, group = false, byType = false) {
        return this.$(`get-config:${xq ? xq : 'none'}${group ? '-grouped' : ''}${byType ? '-groupedByType' : ''}`, async () => {
            let cfg = await this.API.POST('config', { count: false, prefix: true, deep: xq ? 0 : 4, group, byType }, xq);
            if (!xq) {
                cfg = cfg['$' + this.prefix][0];
                if (!group) this.$set('config', cfg);
            }
            return cfg;
        });
    }

    get Items() {
        return this.$('Items', async () => {
            const config = await this.getConfig();
            let p = [];
            for (let i in config) {
                let group = config[i];
                if (Array.isArray(group)) {
                    let prefix = i[1];
                    p.push(...group.map(item => {
                        return this.getItem(item.i, prefix);
                    }))
                }
            }
            const items = await Promise.all(p);
            return items.sort((a, b) => {
                return ((a.zIndex < b.zIndex) || (a.zIndex === b.zIndex && a.label && b.label && a.label.toLowerCase() < b.label.toLowerCase())) ? -1 : 1;
            });
        })
    }

    getItems(prefix, inherit) {
        let p = [this.Items];
        if (inherit) {
            let id = this.prefix + ":" + this.id;
            let base = this.$base.$owner;
            while (base && base instanceof odaBase) {
                if (this instanceof odaBase) {
                    p.push(base.getItems(prefix).then(list => {
                        return list.filter(i => {
                            return !(i instanceof odaBase);
                        });
                    }));
                }
                else {
                    p.push(base.findItem(id).then(item => {
                        return item.getItems(prefix);
                    }).catch(() => {
                        return [];
                    }))
                }
                base = base.$owner;
            }
            if (this instanceof odaBase && base.$host.PublicPart !== this) {
                p.push(base.$host.PublicPart.getItems(prefix).then(list => {
                    return list.filter(i => {
                        return !(i instanceof odaBase);
                    });
                }));

            }
        }
        return Promise.all(p).then(array => {
            return array.reduce((a, b) => {
                b.forEach(bb => {
                    if (!a.find(aa => {
                        return aa.id === bb.id;
                    })) {
                        a.push(bb);
                    }
                });
                return a;
            }, []);
        });
    }
    async _getItems(prefix = 'C') {
        return this.$('get-items-' + prefix, async () => {
            if (!this.config || (this.config.ch === 'T' && !this.config.items))
                await this.getConfig();
            let array = this.config['$' + prefix];
            if (array) {
                let p = array.map(item => {
                    return this.findItem(prefix + ':' + item.i);
                });
                const items = await Promise.all(p);
                return items.sort((a, b) => {
                    return ((a.zIndex < b.zIndex) || (a.zIndex === b.zIndex && a.label && b.label && a.label.toLowerCase() < b.label.toLowerCase())) ? -1 : 1;
                });
            }
            return [];
        })
    }
    get Classes() {
        return this.Items.then(items => items.filter(i => i.prefix === 'C'));
    }
    get ItemsCount() {
        return this.$('ChildrenCount', () => (this.config && ODANT.getArrayChildren(this.config, '$').length) || 0)
    }
    async _getAllActions(context = this) {
        return this.$(`allActions: ${context.itemType}: * ${context.ext || ''}: system=${context.isSystem}`, async () => {
            const path = context._actionsPath;
            const src = (context instanceof odaField && context.TypeClass) || this;
            const deep = 3;//ODANT.webMode?2:3;
            let files = await src.getFiles(path, deep);
            files = files.filter(i => {
                return (!['views', 'pages'].includes(i.$owner.$owner.name) && i.id === i.$owner.name + '.js' || i.id === 'script.js');
            });
            let actions = files.map(f => {
                return odaAction.create(context, f, this)
            })
            actions = (await Promise.all(actions)).filter(i => i);;
            const services = await this.getServices(context);
            services.forEach(i => {
                if (i && !actions.find(a => a.scriptFolder.path === i.scriptFolder.path)) actions.push(i);
            });
            return actions;
        });
    }
    getActions(type = '*', context) {
        type = type.toLowerCase();
        context = context || this;
        return this.$(`actions: ${context.itemType}: ${type} ${context.ext || ''}: system=${context.isSystem}`, async () => {
            let actions = await this.$(`actions: ${context.itemType}: * ${context.ext || ''}: system=${context.isSystem}`, async () => {
                try {
                    let actions = await this._getAllActions(context);
                    actions = actions.filter(a => (a && context.access.allow(a.allowAccess)));
                    const p = await Promise.all(actions.map(async i => i.allowUse));
                    actions = actions.filter((item, index) => p[index]);
                    actions.sort((a, b) => ((a.zIndex <= b.zIndex && a.label.toLowerCase() < b.label.toLowerCase()) ? -1 : 1));
                    return actions;
                }
                catch (e) {
                    console.error(e);
                    return []
                }
            });
            if (type === 'other') {
                actions = actions.filter(i => {
                    return !i.category;
                });
            } else if (type !== '*') {
                actions = actions.filter(i => { // <-- filter by type
                    return type === i.type || (i.category && type.includes(i.category)) || type.includes(i.group);
                });
            }
            return actions;
        }).then(actions => {
            actions = actions.map(async i => {
                return odaAction.create(context, i._scriptFile, this);
            });
            return Promise.all(actions);
        })
    }
    getServices(context) {
        if (this.$module || !this.$base)
            return [];
        return this.$base.getServices(context);

    }

    get hasChildren() {
        return this.config && this.config.ch === 'T';
    }

    async _buildStructure(structure) {
        await this._createStructure(structure);
        const cfg = await this.getConfig();
        this.reset();
        this.$set('config', cfg);
        this.fire('reset');
        return this;

    }

    async _createStructure(structure) {
        return Promise.all(
            structure.map(async (s) => {
                const prefix = s.prefix.toUpperCase();
                const params = { name: s.name || 'New Item' };

                let command = 'create_class';

                if (prefix !== 'C') {
                    command = 'create_domain';
                    params.type = this._convertPrefixToType(prefix);
                    params.prefix = true;
                }

                const res = await this.API.POST(command, params, s.body);
                let item;
                if (prefix === 'C')
                    item = await this.getClass(res.$C[0].i);
                else
                    item = await this.getDomain(res.i, prefix);
                return item._createStructure(s.structure)

            })
        );
    }

    _convertPrefixToType(prefix) {
        switch (prefix) {
            case 'B':
                return 'BASE';
            case 'M':
                return 'MODULE';
            case 'P':
                return 'PART';
            case 'S':
                return 'SOLUTION';
            case 'W':
                return 'WORKPLACE';
            case 'C':
                return 'CLASS';
            case 'H':
                return 'HOST';
            case 'O':
                return 'OBJECT';
            default:
                return '';
        }
    }

    canDrag() {
        return this.access.admin;
    }
}
odaStructureItem.register();
