(class extends odaAction {
    static get properties(){ return {
        allowAccess: 'A'
    }}

    _getToolkit() {
        return {
            translit: function (text) {
                return slugify(text);
            },
            getHiddenFolder: function (sourceFolder, path) {
                return sourceFolder.getFolder(path, true).then(folder => {
                    return folder.Files.then(files => {
                        if (files.length)
                            return folder;
                        throw new Error(folder);
                    });
                })
            },
            getDifferentFileList: function (sourceFilder, destFolder) {
                var _sourceFiles;
                return sourceFilder.Files.then(sourceFiles => {
                    _sourceFiles = sourceFiles;
                    return destFolder.Files;
                }).then(destFiles => {
                    var result = [];
                    _sourceFiles.forEach(template => {
                        if (destFiles.find((file) => {
                                    return template.id === file.id;
                                }
                            )) {
                            return;
                        }
                        result.push(template);
                    });
                    return result;
                });
            }
        };
    }

    _getTemplatizer(templateList) {
        var _obj = {
            _templateList: templateList,
            _templateMap: new Map()
        };

        _obj._parseTemplateVariables = function (body) {
            var variables = [];

            var index;
            var last = 0;
            var inc = 0;
            while ((index = body.indexOf('%', last)) > -1) {
                inc++;

                if (inc % 2 === 0 && inc > 0) {
                    variables.push(body.substring(last, index));
                }

                last = index + 1;
            }

            return variables;
        };

        _obj._pushTemplate = (templateFile) => {
            return templateFile.load().then(body => {
                var _file = {
                    body: body,
                    context: templateFile
                };
                _obj._templateMap.set(templateFile.id, _file);
                return _file;
            });
        };

        _obj.parse = (files) => {
            return Promise.all(files.map(file => _obj._pushTemplate(file)))
        };

        _obj._compileTemplateBody = function (body, key, properties) {
            if (!properties) {
                return body;
            }

            switch (typeof body) {
                case 'string':
                    for (var e in properties) {
                        body = body.replace(
                            new RegExp('%' + e + '%', 'g'),
                            properties[e].value
                        );
                    }

                    return body;
                default:
                    return properties[key] && properties[key].value ? properties[key].value : body;
            }
        };

        _obj.getVariables = (filter) => {
            var _vars = new Map();

            _obj._templateMap.forEach(
                (value, key, map) => {
                    if (filter && !filter(value, key, map)) {
                        return;
                    }

                    switch (typeof value.body) {
                        case 'string':
                            var local = _obj._parseTemplateVariables(value.body);
                            local.forEach((n) => _vars.set(n, {label: n}));
                            break;
                        default:
                            if (value.context.id.indexOf('.png') > -1)
                                _vars.set(value.context.id, {
                                    label: value.context.id,
                                    value: value.body,
                                    icon: value.context.url
                                });
                    }
                }
            );

            return _vars;
        };
        _obj.compile = (destFolder, properties) => {
            return new Promise(
                (resolve, reject) => {
                    var promises = [];
                    _obj._templateMap.forEach(
                        (value, key, map) => {
                            promises.push(
                                destFolder.appendFile(
                                    _obj._compileTemplateBody(value.body, key, properties),
                                    key
                                )
                            );
                        }
                    );

                    Promise.all(promises).then(resolve).catch(reject);
                }
            );
        };
        return _obj;
    }

    _resolveTemplateDest(destFolder) {
        var _toolkit = this._getToolkit();
        return _toolkit.getHiddenFolder(destFolder, '_require').then(folder => ({
            action: destFolder,
            template: folder
        })).catch(() => {
            return _toolkit.getHiddenFolder(destFolder, '_template').then(folder => ({template: folder})).catch(() => {
                return _toolkit.getHiddenFolder(destFolder.$owner, '_template').then(folder => ({
                    action: destFolder,
                    template: folder
                }));
            });
        });
    }

    _getManifest() {
        return {
            _variables: {
                actionTitle: {
                    label: 'Action Title'
                },
                actionName: {
                    require: true,
                    label: 'Action Name',
                    passing: ['actionTitle'],
                    compute: (params) => {
                        return this._getToolkit().translit(params.actionTitle.value || '');
                    }
                },
                actionAccess: {
                    label: 'Action Access',
                    options: ['R', 'RW', 'RWC', 'RWCD', 'A'],
                    value: 'R'
                },
                moduleId: {
                    hidden: true,
                    passing: ['actionName'],
                    compute: (params) => {
                        return 'odant-' + params.actionName.value;
                    }
                }
            },
            set: function (key, params, overwrite) {
                var item = this._variables[key];
                if (!item) {
                    this._variables[key] = params;
                    return;
                }

                for (var k in params) {
                    if (!item[k] || overwrite)
                        item[k] = params[k];
                }
            },
            get: function (key) {
                return this._variables[key];
            },
            join: function (template) {
                template.forEach(
                    (item, key) => {
                        this.set(key, item);
                    }
                );
            },
            createVariablesList: function (template) {
                var vars = new Map();

                /* init map */
                for (var k in this._variables) {
                    var v = template.get(k);

                    if (v || this._variables[k].require) {
                        vars.set(k, this._variables[k]);
                    }
                }

                /* add requiries */
                var passingVars = new Map();
                vars.forEach(
                    (item) => {
                        (item.passing || []).forEach(
                            (pass) => {
                                passingVars.set(pass, this.get(pass));
                            }
                        );
                    }
                );

                /* merge */
                passingVars.forEach(
                    (item, key) => {
                        vars.set(key, item);
                    }
                );

                /* create object */
                var result = {};
                vars.forEach(
                    (item, key) => {
                        result[key] = item;
                    }
                );

                return result;
            }
        };
    }

    _createAction(destFolder, templateSource) {
        var _action;
        return this._resolveTemplateDest(templateSource).then(result => {
            if (result.action) {
                _action = result.action;
                return this._getToolkit().getDifferentFileList(result.template, _action);
            }
            return result.template.Files;
        }).then(diffs => {
            if (!diffs.length) {
                return _action;
            }

            var _manifest = this._getManifest();
            var _variables = {};

            if (_action) {
                _manifest.set('actionName', {value: _action.name, hidden: true, compute: null, passing: []}, true);
            }

            var templatizer = this._getTemplatizer(diffs);
            return templatizer.parse(diffs).then(result => {
                var templateVars = templatizer.getVariables();//result.getVariables();
                _manifest.join(templateVars);
                _variables = _manifest.createVariablesList(templateVars);
                return this._showActionDialog(
                    _variables,
                    (_action)
                        ? 'Need to update the template. Continue?'
                        : 'Create a new action with the following parameters?'
                );
            }).then(result => { /* dialog confirm */
                    if (!result) {
                        if (_action)
                            return _action;
                        throw new Error();
                    }
                    if (_action)
                        return _action;
                    else
                        return destFolder.createFolder(result.actionName.value);
                }
            ).then(compiledAction => {
                return templatizer.compile(compiledAction, _variables).then(result => compiledAction);
            }).catch(error => {
                if (_action && _action.inherit.none)
                    return _action;
                throw error;
            })
        });
    }

    _showActionDialog(variables, message) {
        return ODA.loadLink(this.resolveUrl('odant-create-action.html')).then(() => {
            const _component = document.createElement('odant-create-action-dialog');
            _component.set('variables', variables);
            _component.set('message', message);
            return ODA.showDialog({
                component: _component,
                title: 'Create Action'
            });
        }).then(() => {
            return variables;
        }).catch(() => {
            return null;
        });
    }

    execute(templateSource) {
        return ODANT.loadScript('/web/ext/transliteration/lib/browser/transliteration.js').then(
            (s) => {
                return this.$owner.getFolder(templateSource.path, true).then(
                    (contextFolder) => {
                        return this._createAction(contextFolder, templateSource);
                    }
                );
            }
        );
    }
}).register();