export default odaView({
    props: {
        allowButtons: true,
        allowAccess: 'R',
        label: 'Data grid',
        allowServices: true,
        allowUse() {
            return !this.$owner.isAbstract;
        },
        allowPrint: true
    }
})

ODANT({
    is: 'odant-grid-view',
    imports: [
        '/web/core/index/lib/packets-bar/packets-bar.js',
        '/web/core/class/lib/grid/grid.js'
    ],
    template: /* html*/ `
    <style>
        :host{
            @apply --vertical;
            overflow: hidden;
        }
    </style>
    <odant-packets-bar class="flex" :context-item="_contextItem" ::mask="packMask"></odant-packets-bar>
    <odant-grid style="height: 100%;" ref="grid" class="flex" :context-item="_contextItem" :filter-val :pack-mask ::focused-item></odant-grid>
    <div slot="right-tools" class="flex horizontal" style="align-items: center;">
        <input class="label content" type="text" ::value="filterVal" style="height: 25px" title="filter" placeholder="filter">
        <oda-button ~show="_contextItem?.access?.allowAdmin" icon="icons:save" :icon-size title="Share settings" @tap="_shareSettings"></oda-button>
        <oda-button ~show="_contextItem?.access?.allowAdmin && sharedSettings" icon="icons:clear" :icon-size title="Clear shared settings" @tap="_clearSharedSettings"></oda-button>
        <!--<oda-button @tap="_refresh" icon="icons:refresh" title="Refresh grid"></oda-button>-->
    </div>
`,
    dataSet: [],
    props: {
        contextItem: {
            type: Object,
            watch(n) {
                if (n) {
                    this._getSharedSettings();
                }
            }
        },
        _contextItem: Object, // что бы отсрочить загрузку данных в грид
        iconSize: 24,
        packMask: String,
        filterVal: String,
        focusedItem: {
            type: Object,
            notify: true
        },
        sharedSettings: false
    },
    async _shareSettings() {
        if (this.$refs.grid && this.$refs.grid.$refs.table) {
            const confirm = await ODA.confirm('Are you sure you want to share grid settings? This will erase the local user settings.', 'Confirm');
            if (!confirm) return;
            const $item = this.contextItem;
            const table = this.$refs.grid.$refs.table;
            const settings = {};
            const { columns } = table;
            for (const col of columns) {
                settings[`/column/${col.name}/order`] = col.order;
                settings[`/column/${col.name}/width`] = col.width;
                settings[`/column/${col.name}/hidden`] = col.hidden;
            }
            return $item.saveFile(JSON.stringify(settings), 'web/class/settings/grid/settings.json');
        }
    },
    async _getSharedSettings() {
        try {
            const $item = this.contextItem;
            // TODO
            const settings = (await $item.$class.getFile('settings/grid/settings.json'))?.load();
            if (settings && this.$refs.grid.$refs.table) {
                const table = this.$refs.grid.$refs.table;
                for (const key in settings) {
                    table.__write(`${$item.id}${key}`, settings[key]);
                }
                table.render();
                this.sharedSettings = true;
            }
        } catch (e) {     
            console.error(e);
        } finally {
            this._contextItem = this.contextItem;
        }
    },
    async _clearSharedSettings() {
        const confirm = await ODA.confirm('Erase shared settings?', 'Confirm');
        if (!confirm) return;
        const $item = this.contextItem;
        await $item.deleteFile('web/class/settings/grid/settings.json');
    }
})