ODA({
    is: 'odant-field-base',
    template: /*html*/`
    <style>
        :host{
            position: relative;
            border: none;
        }
        .dropdown {
            padding: 0px;
            opacity: .5;
        }
        .dropdown:hover {
            opacity: 1;
        }
        .warning {
            cursor: pointer;
            position: absolute;
            left: 0px;
            z-index: 1;
            opacity: .5;
        }
        .error {
            cursor: pointer;
            position: absolute;
            z-index: 1;
            opacity: .5;
        }
        oda-icon {
            align-self: center;
        }
    </style>`,
    hostAttributes: {
        tabindex: 0
    },
    _warning(e) {
        ODA.push(e.target.title);
    },
    _showDropDown(value) {
        return this.$field && !this.complex && !this.readOnly && this.$field._hasList && !(this.$field._isLink && value)
    },
    props: {
        iconSize: Number,
        complex: {
            type: Boolean,
            reflectToAttribute: true
        },
        $object: {
            type: Object,
            default: null,
            watch(n, o) {
                if (o) {
                    //this.unlisten(n, 'changed', 'getValue')
                }
                if (n) {
                    //this.listen(n, 'changed', 'getValue')
                }
            }
        },
        warning: {
            type: String,
            reflectToAttribute: true,
            watch(n, o) {
                if (o) {
                    this.style.setProperty('padding-left', '0px');
                }
                if (n) {
                    this.style.setProperty('padding-left', (this.iconSize * .5 + 12) + 'px');
                }
            }
        },
        $field: Object,
        necessary: {
            type: Boolean
        },
        dataRoot: Object,
        focused: false,
        iconSize: 24,
        value: {
            type: [String, Number, Boolean, Object],
            notify: true,
            async watch(n) {
                if (n !== undefined && this.$field && this.$object && !this.readOnly) {
                    await this.$field.setValue(this.$object, n, this.dataRoot);
                }
            }
        },
        item: {
            type: Object,
            watch(item) {
                if (item)
                    this.dataRoot = item;
            }
        },
        column: {
            type: Object,
            watch(column) {
                this.$field = column && column.$field;
            }
        },
        readOnly: Boolean
    },
    listeners: {
        input(e) {
            e.stopPropagation();
            if (this.$field && this.$field._hasList && e.detail.value)
                this._showList(e.detail.value);
            else
                this._closeDropDown();
        },
        keyup(e) {
            if (this.focused && e.ctrlKey && e.keyCode === 32) {
                e.preventDefault();
                this._showList(null);
            }
        }
    },
    observers: [
        async function _init($field, dataRoot, $object) {
            if ($field && dataRoot) {
                if (await $field.getReadOnly($object, dataRoot)) {
                    this.readOnly = true;
                }
                this.getValue();
            }
        }
    ],
    keyBindings: {
        home: '_first',
        ArrowUp: '_before',
        ArrowDown: '_next',
        enter: '_result',
    },
    _first(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.first();
        }
    },
    _result(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd._result();
        }
    },
    _next(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.next();
        }
    },
    _before(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.before();
        }
    },
    _closeDropDown() {
        this.async(() => {
            if (this.dd) {
                this.dd.close();
                this.dd = null;
            }
        }, 100)
    },
    setValue(value) {
        if (this.value === value) return;
        if (value && value.valueOf) {
            if (this.value && this.value.valueOf) {
                if (this.value.valueOf() === value.valueOf())
                    return;
            }
        }
        this.value = value === undefined ? null : value;
    },
    async getValue(e) {
        if (this.$field && this.dataRoot) {
            if (!this.$field.isTable) {
                const v = await this.$field.getValue(this.$object, this.dataRoot);
                this.setValue(v);
            }
            if (this.$object) {
                const w = await this.$field.getValidate(this.$object, this.dataRoot);
                const r = await this.$field.getReadOnly(this.$object, this.dataRoot);
                const n = await this.$field.getNecessary(this.$object, this.dataRoot);

                this.warning = w;
                this.readOnly = r;
                this.necessary = n;
            }
        }
    },
    attached() {
        //this.listen(this, 'focus', '_onFocus');
        //this.listen(this, 'blur', '_onBlur');
        let host = this.domHost;
        while (!this.$object && host) {
            this.$object = host.$object;
            host = host.domHost;
        }
    },
    detached() {
        //this.unlisten(this, 'focus', '_onFocus');
        //this.unlisten(this, 'blur', '_onBlur');
    },
    toFocus() {
        this._onFocus();
        return this.focused;
    },
    _onFocus(e) {
        if (e) e.stopPropagation();
        if (!this.readOnly) {
            this.focused = true;
        }
    },
    _onBlur() {
        this.focused = false;
        if(this.dd){
            this.async(()=>{
                this._closeDropDown();
            }, 300);
        }
    },
    async _showList(filter) {
        // TODO
        if (filter === null && this.dd) {
            this._closeDropDown();
            return;
        }
        try {
            let list = await this.$field.getValueList(this.$object, this.dataRoot);
            if (list.length) {
                if (!this.dd || !this.dd.opened) {
                    this.dd = ODA.createElement('oda-dropdown-list', { parent: this });
                    this.dd.filter = filter;
                    this.dd.items = list;
                    this.dd.showCount = true;
                    let res = await this.dd.show(this.value, !this.focused).catch(() => void (0));
                    if(!res) res = this.value;
                    this._closeDropDown();
                    this.focused = false;
                    if (res && typeof res === 'object' && 'value' in res) res = res.value;
                    if (this.$field._isLink && typeof res === 'object' && !(res instanceof odaItem)) {
                        res = await this.$field._getObject(res);
                    }
                    this.$field.setValue(this.$object, res, this.dataRoot);
                }
                else {
                    this.dd.filter = filter;
                    this.dd.items = list;
                }
            }
            else {
                this._closeDropDown();
            }
        }
        catch (e) {
            this._closeDropDown();
        }
    }
});

ODA({
    is: 'odant-field-input',
    extends: 'odant-field-base',
    template: /*html*/`
        <oda-icon ~show="warning" class="warning" :icon="warning && 'icons:warning'" :icon-size="iconSize * .5" :title="warning" :style="{padding: (iconSize * .25)+'px 2px'}" @tap.stop="_warning"></oda-icon>
        <oda-icon ~show="necessary && !value" class="error" :icon="necessary && !value && 'icons:error'" :icon-size="iconSize * .5" title="Required field is not filled"
            :style="{padding: (iconSize * .25) + 'px 2px', left: warning ? (iconSize * .5) + 'px' : '0px'}" @tap.stop="_warning"></oda-icon>
        <oda-button class="dropdown" ~if="_showDropDown(value) || false" :icon-size="iconSize" icon="icons:arrow-drop-down" @tap="_showList(null)"></oda-button>
    `,
    props: {
        iconSize: {
            default: 24,
            type: Number,
        }
    },
    listeners: {
        contextmenu: '_onContextMenu'
    },
    async _onContextMenu(e) {
        if(this.$refs.input){
            e.stopPropagation();
        }
    },
})