function getData(metaPacks, packets, lastPakcs) {
    const packs = [];
    for (let i = 0; i < metaPacks.length; i++) {
        let mode = metaPacks[i].PackMode;
        let lastPack = (lastPakcs && lastPakcs.length > 0) ? lastPakcs[i] : {};

        // ------- for Leha Bugor ---------- //
        if (lastPack && lastPack.active) {
            lastPack.active.map(a => {
                if (a.period) {
                    let period = {
                        from: new Date(a.period.from),
                        to: new Date(a.period.to)
                    }
                    a.period = period;
                    return a;
                }
            })
        }
        // ------------------------------- //

        let pack = {
            mode: mode,
            items: [],
            lastPacks: lastPack && lastPack.active,
            oldNow: lastPack && lastPack.now
        };
        if (packets && !(packets.length === 1 && packets[0].name === 'pack')) {
            packets.forEach(p => pack.items.push(p.name.split('#')[i]));
        }
        packs.push(this.getDataAdapter(pack));
    }
    return packs;
}

function getDataAdapter(pack) {
    switch (pack.mode) {
        case 'Field':
            return new packFieldAdapter(pack);
        case "Day":
        case "Month":
        case "Quart":
        case "Year":
            return new packDateAdapter(pack);
        case "Alpha":
            return new packAlphaAdapter(pack);
        default:
            return new packDataAdapter(pack);
    }
}

class packDataAdapter {

    constructor(pack) {
        if (pack) {
            this.mode = pack.mode;
            this.source = pack.items;
            this.oldNow = pack.oldNow;
            this.active = pack.lastPacks ? pack.lastPacks : [];
            this.items = this.getItems();
            if (this.items.length == 1) {
                this.active = [this.items[0]]
            }
        }
    }

    getItems() {
        const map = new Map();
        this.source.forEach(item => {
            if (!map.has(item) && item != '') {
                map.set(item, this.getItem(item));
            }
        });

        let items = Array.from(map.values());
        items.unshift({mask: '*', label: '❋', key: '*', clickable: true});
        return items;
    }

    getItem(oldItem) {
        let item = {key: oldItem, mask: `${oldItem}*`, label: oldItem};// active: this.isActive(oldItem) // active:this.lastPack.some(function(lp){return lp == oldItem}.bind(this))
        return item;
    }

    isActive(item) {
        try {
            return this.active.some(a => a.key == item);
        }
        catch (e) {
            return false;
        }
    }

    isPossible(idx) {
        if (!this.possibleItems || this.possibleItems.length == 0) {
            return false;
        }

        let m = this.items[idx].mask.replace(new RegExp('\\*', 'g'), '.*');
        let tMask = new RegExp(m);

        return this.possibleItems.some(item => tMask.test(item))
    }

    showPack(e) {
        return new Promise(resolve => {
            let target = e.currentTarget;
            let panel = target.getBoundingClientRect();
            let pos = {x: parseInt(panel.left), y: parseInt(panel.bottom)};
            let control = target.create('odant-pack-alpha', {pack: this});
            let params = {items: this.items, pos: pos, control: control};

            return ODA.showDialog(params).then(res => {
                resolve(res.pack);
            }).catch(e => {

            })

        });
    }
}

class packAlphaAdapter extends packDataAdapter {
    showPack(e) {
        return new Promise(resolve => {

            let target = e.currentTarget;
            let panel = target.getBoundingClientRect();
            let pos = {x: parseInt(panel.left), y: parseInt(panel.bottom)};
            let control = target.create('odant-pack-alpha', {pack: this});
            let params = {items: this.items, pos: pos, control: control};

            return ODA.showDialog(params).then(res => {
                resolve(res.pack);
            }).catch(e => {

            })
        });
    }
}

class packFieldAdapter extends packDataAdapter {

    getItems() {
        const map = new Map();
        this.source.forEach(i => {
            let key = i[0];
            if (!map.has(key)) {
                map.set(key, [key]);
            }
        });
        const items = [];
        map.forEach((value, key) => items.push(this.getItem(key, true)));
        items.push({mask: '*', label: '❋', key: '*'});
        return items;
    }

    getItem(oldItem, hasChildren) {
        let item = {
            key: oldItem,
            mask: `${oldItem}*`,
            // clickable: true,
            label: oldItem,
            items: hasChildren ? [] : undefined
            // ,
            // active: this.isActive(oldItem),
            // getItems: this.getSubItems.bind(this)//this.contextMenu.bind(this)
        };
        return item;
    }

    // contextMenu(e) {
    //     let target = e.currentTarget;
    //     let item = target.item;
    //     let panel = target.getBoundingClientRect();
    //     let pos = {x: parseInt(panel.left), y: parseInt(panel.bottom)};
    //     let param = {item, pos};
    //
    //     if (!item.items) {
    //         item.items = this.setSubItems(item);
    //     }
    //
    //     let dropList = target.create('odant-pack-drop-list-menu', {items: item.items});
    //     return dropList.open(param).then(i => i.item);
    // }

    getSubItems(item) {
        let key = item.key;
        return this.source
            .filter(i => i[0] === key)
            .map(it => {
                let subItem = this.getItem(it);
                subItem.owner = item;
                return subItem;
            })
    }

    showPack(e) {
        return new Promise(resolve => {

            let target = e.currentTarget;
            let panel = target.getBoundingClientRect();
            let pos = {x: parseInt(panel.left), y: parseInt(panel.bottom)};
            let control = target.create('odant-pack-field', {pack: this});
            let params = {items: this.items, pos: pos, control: control};

            return ODA.showDialog(params).then(res => {
                let selection = res.$.tree.selection;

                if(selection && selection.length > 0){
                    let active = selection.map(node => node.data);
                    this.active = active;
                    resolve(res.pack);
                }
            }).catch(e => {

            })
        });
    }
}

class packDateAdapter extends packDataAdapter {
    getItems() {
        let current = new Date();
        let key = this.getCurrentModeName();
        let nowItem = {
            mask: this.getCurrentModeMask(),
            label: this.getCurrentLabel(),//'Now',
            id: 'Now',
            // clickMethod: this.getNowItem.bind(this),
            period: this._getPeriodDate({from: key, to: key}),
            key: key
            // ,
            // active: this.isActive(this.getCurrentModeName())
        }

        if (this.active.length === 0) { //|| this.oldNow != current.toLocaleFormat('%Y-%m-%d')
            // nowItem.active = true;
            let now = Object.assign({}, nowItem);
            now.label = this.getCurrentLabel();
            this.active = [now];
        }

        const map = new Map();
        this.source.forEach(item => {
            let year;
            try {
                year = item.substring(0, 4);
            }
            catch (e) {
            }
            if (!map.has(year) && year != '')
                map.set(year, this.getYearItem(year));
        });
        const items = Array.from(map.values());
        items[0].group = true;

        items.push({
            mask: '',
            label: 'Period',
            key: 'Period',
            group: true,
            icon: 'enterprise:calendar',
            clickMethod: this.showPeriod.bind(this)
            // ,
            // active: this.isActive('Period')
        });
        let allPeriod = this.source.length > 0 ? this._getPeriodDate({
            from: this.source[0].substring(0, 4),
            to: this.source[this.source.length - 1].substring(0, 4)
        }) : {from: undefined, to: undefined, error: 'No data packets. Pack.oml only.'}
        items.push({mask: '*', label: '❋', key: '*', period: allPeriod});
        items.unshift(nowItem);
        return items;
    }

    _getPeriodDate(date) {
        const timeFrom = 'T00:00:00';
        const timeTo = 'T23:59:59';
        const timeZone = '+0300'; //TODO Receive from the server
        let period = {
            from: new Date(`${date.from}${timeFrom}${timeZone}`),
            to: new Date(`${date.to}${timeTo}${timeZone}`)
        };
        return period;
    }

    setSubItems(item) {
        let key = item.key;
        let subSource = this.source.filter(i => {
            let y = i.substring(0, 4);
            return y == key;
        });
        let subItems;
        switch (this.mode) {
            case "Day":
                subItems = this.getDayPackets(subSource, item);
                break;
            case "Month":
                subItems = this.getMonthPackets(subSource, item);
                break;
            case "Quart":
                subItems = this.getQuartPackets(subSource, item);
                break;
            case "Year":
                subItems = [];
                break;
        }
        return subItems;
    }

    getYearItem(year) {
        let period = {from: `${year}-01-01`, to: `${year}-12-31`};
        let item = {
            key: year,
            mask: `${year}*`,
            period: this._getPeriodDate(period),
            clickable: true,
            label: year
            // ,
            // active: this.isActive(year),
        };
        item.items = this.setSubItems(item);
        return item;
    }

    getPeriodItem(period) {
        let items = [];
        if (this.mode == 'Quart') {
            items = this.source.filter(i => {
                let f = new Date(period.from);
                let t = new Date(period.to);
                let y = new Date(i.substring(0, 4)); // год текущего item'a

                if (f <= y && y <= t) {
                    let fq = num2quart(f.getMonth() + 1);
                    let tf = num2quart(t.getMonth() + 1);
                    let d = parseInt(i[i.length - 1]);
                    return (fq <= d && d <= tf);
                }
                else
                    return false;
            });
        }
        else {
            items = this.source.filter(i => {
                let f = new Date(period.from);
                let b = new Date(period.to);
                let d = new Date(i);
                return (f <= d && d <= b);
            });
        }
        return items;
    }

    getQuartPackets(items, year) {
        let quarts = [];
        items.forEach(item => {
            let q;
            if (item.indexOf('q') != -1) {
                q = item[item.indexOf('q') + 1];
            }
            else {
                let date = new Date(item);
                let m = date.getMonth() + 1;
                q = this.num2quart(m);
            }
            if (!quarts.some(elem => elem.key == `${year.key}-q${q}`)) {

                // ------- for Leha Bugor ------ //
                let months = this.quart2months(q);
                let dateLastDay = new Date(this._getDateLastDay(year.key, months[months.length - 1]));
                let period = {
                    from: `${year.key}-${months[0]}-01`,
                    to: dateLastDay.toJSON().substr(0, 10)
                }
                // ---------------------------- //

                quarts.push({
                    mask: this.getQuartMask(year.key, q),
                    key: `${year.key}-q${q}`,
                    period: this._getPeriodDate(period),
                    clickable: true,
                    label: `${q}/4 ${year.key}`,
                    owner: year
                });
            }
        });
        quarts.reverse();
        return quarts;
    }

    getQuartMask(y, q) {
        if (this.mode == "Quart")
            return `${y}-q${q}`;
        else {
            switch (q) {
                case 1:
                    return `${y}-01*|${y}-02*|${y}-03*`;
                case 2:
                    return `${y}-04*|${y}-05*|${y}-06*`;
                case 3:
                    return `${y}-07*|${y}-08*|${y}-09*`;
                case 4:
                    return `${y}-10*|${y}-11*|${y}-12*`;
                default:
                    return '*';
            }
        }
    }

    getMonthPackets(items, year) {
        let quarts = this.getQuartPackets(items, year);
        items.forEach(item => {
            let date = new Date(item);
            let m = date.getMonth() + 1;
            let month = date.toLocaleString(window.navigator.language, {month: 'long'});
            let q = this.num2quart(m);
            let quart = quarts.find(elem => elem.key == `${year.key}-q${q}`);
            if (!quart.items) {
                quart.items = [];
            }
            if (!quart.items.some(elem => elem.key == date.toLocaleFormat('%Y-%m'))) {
                let dateLastDay = new Date(this._getDateLastDay(year.key, m));
                let period = {
                    from: `${date.toLocaleFormat('%Y-%m')}-01`,
                    to: dateLastDay.toJSON().substr(0, 10)
                }
                quart.items.push({
                    mask: `${date.toLocaleFormat('%Y-%m')}*`,
                    period: this._getPeriodDate(period),
                    clickable: true,
                    key: date.toLocaleFormat('%Y-%m'),
                    label: `${month} ${quart.key.substring(0, 4)}`,
                    owner: quart
                });
            }
        });
        quarts.forEach(q => q.items.reverse());
        return quarts;
    }

    getDayPackets(items, year) {
        let quarts = this.getMonthPackets(items, year);
        quarts.forEach(quart => {
            quart.items.forEach(month => {
                let dayList = items.filter(elem => {
                    let date = new Date(elem);
                    return (date.toLocaleFormat('%Y-%m') == month.key);
                });
                let map = new Map();
                dayList.forEach(d => {
                    if (!map.has(d))
                        map.set(d, d);
                });
                let days = Array.from(map.values());
                if (days.length > 0) {
                    month.items = [];
                    days.forEach(day => {
                        let period = {
                            from: day,
                            to: day
                        }
                        month.items.push({
                            mask: `${day}*`,
                            key: day,
                            period: this._getPeriodDate(period),
                            clickable: true,
                            label: new Date(day).toLocaleString(window.navigator.language, {
                                day: 'numeric',
                                year: 'numeric',
                                month: 'short'
                            }),
                            owner: month
                        });
                    });
                    month.items.reverse();
                }
            });
        })
        return quarts;
    }

    _getDateLastDay(year, month) {
        let y = parseInt(year);
        let m = parseInt(month);
        return new Date(y, m) - 86400000;
    }

    getCurrentModeName() {
        switch (this.mode) {
            case 'Day':
                return new Date().toLocaleFormat('%Y-%m-%d');
            case 'Month':
                return new Date().toLocaleFormat('%Y-%m');
            case 'Quart':
                let d = new Date();
                return `${d.getFullYear()}-${this.num2quart(d.getMonth() + 1)}`;
            case 'Year':
                return new Date().getFullYear();
            default:
                return this.mode;
        }
    }

    getCurrentModeMask() {
        switch (this.mode) {
            case 'Day':
                return new Date().toLocaleFormat('%Y-%m-%d*');
            case 'Month':
                return new Date().toLocaleFormat('%Y-%m*');
            case 'Quart':
                let d = new Date();
                return this.getQuartMask(d.getFullYear(), this.num2quart(d.getMonth() + 1), this.mode);
            case 'Year':
                return `${new Date().getFullYear()}*`;
            default:
                return undefined;
        }
    }

    getNowItem() {
        let idx = this.items.findIndex(i => i.id == 'Now');
        let item = this.items[idx];

        item.label = this.getCurrentLabel();
        item.key = this.getCurrentModeName();
        item.mask = this.getCurrentModeMask();
        item.period = this._getPeriodDate({from: item.key, to: item.key});

        return Promise.resolve(item);
    }

    num2quart(num) {
        let quarts = ['', 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4];
        return quarts[num];
    }

    quart2months(q) {
        let _q = parseInt(q)
        if (!isNaN(_q)) {
            switch (_q) {
                case 1:
                    return ['01', '02', '03'];
                case 2:
                    return ['04', '05', '06'];
                case 3:
                    return ['07', '08', '09'];
                case 4:
                    return ['10', '11', '12'];
            }
        }
        return [];
    }

    getCurrentLabel() {
        let label = '';
        switch (this.mode) {
            case "Day":
                label = this.getCurrentDay();
                break;
            case "Month":
                label = this.getCurrentMonth();
                break;
            case "Quart":
                label = this.getCurrentQuarts();
                break;
            case "Year":
                label = this.getCurrentYear();
                break;
        }
        return label;
    }

    getCurrentDay() {
        let dateNow = new Date();
        return dateNow.toLocaleString(window.navigator.language, {day: 'numeric', year: 'numeric', month: 'short'});
    }

    getCurrentMonth() {
        let dateNow = new Date();
        return dateNow.toLocaleString(window.navigator.language, {year: 'numeric', month: 'short'})
    }

    getCurrentQuarts() {
        let dateNow = new Date();
        return `${this.num2quart(dateNow.getMonth() + 1)}/4 ${dateNow.getFullYear()}`;
    }

    getCurrentYear() {
        let dateNow = new Date();
        return dateNow.getFullYear();
    }

    showPack(e) {
        return new Promise(resolve => {
            let target = e.currentTarget;
            let panel = target.getBoundingClientRect();
            let pos = {x: parseInt(panel.left), y: parseInt(panel.bottom)};

            if (this.items) {
                let p = {items: this.items, pos: pos, title: 'Select date'};
                return target.create('oda-context-menu')
                    .open(p)
                    .then(res => {
                        if (res && res.key === 'Period') {
                            this.showPeriod(e).then(r => {
                                this.active = [r];
                                resolve(this)
                            });
                        } else {
                            this.active = [res];
                            resolve(this);
                        }
                    })
                    .catch(e => {
                        resolve(e);
                    });
            }
        });
    }

    showPeriod(e) {
        let control = document.createElement('odant-pack-date-period');
        return ODA.showDialog({label: "Select period", component: control}).then(result => {
            // var control = result.ctrl;
            let period = {from: result.from, to: result.to}; //{from: control.from, to: control.to};
            let items = this.getPeriodItem(period);
            items.reverse();

            let item;
            if (items.length > 0) {
                let name = `${items[items.length - 1]} — ${items[0]}`;
                item = {mask: items.join('|'), key: 'Period', label: name};
            }
            else {
                let name = `${period.from} — ${period.to}`;
                item = {mask: `${period.from}|${period.to}`, key: 'Period', label: name};
            }

            let periodIdx = this.items.findIndex(i => i.key == 'Period');
            let periodItem = this.items[periodIdx];

            for (let p in item) {
                periodItem[p] = item[p];
            }

            periodItem.period = this._getPeriodDate(period);
            return periodItem;
        });
    }


}

if (!Date.prototype.toLocaleFormat) {
    Date.prototype.toLocaleFormat = function (format) {
        var f = {
            Y: this.getFullYear(),
            y: this.getFullYear() - (this.getFullYear() >= 2e3 ? 2e3 : 1900),
            m: this.getMonth() + 1,
            d: this.getDate(),
            H: this.getHours(),
            M: this.getMinutes(),
            S: this.getSeconds()
        }, k;
        for (k in f)
            format = format.replace('%' + k, f[k] < 10 ? "0" + f[k] : f[k]);
        return format;
    }
}

