ODANT({
    is: "odant-field-tree",
    imports: [
        '@lib/item-tree',
        '@lib/action-bar'
    ],
    extends: 'odant-item-tree',
    props: {
        itemSourceNames: 'fields',
        allowActions: false,
        allowFocus: true,
        allowSelection: true,
        allowDrag: true,
        itemsList: '$$field',
        cols: {
            get() {
                const cols = [{ name: this.keyName, treeMode: true }];
                this.groups = [{ name: 'group' }];
                return cols;
            }
        },
        // defaultTemplate: 'odant-field-tree-cell',
        // defaultGroupTemplate: 'odant-field-tree-group'
    },
    observers: [
        async function _setDataSet(contextItem, itemSourceNames) {
            if (contextItem && contextItem instanceof odaItem && itemSourceNames) {
                const _fields = await this._getFields(contextItem.$FIELDS).catch(e => []);
                const _static = await this._getFields(contextItem.$STATIC).catch(e => []);
                const _details = await this._getFields(contextItem.$DETAILS).catch(e => []);
                this.dataSet = [..._fields, ..._static, ..._details];
            }
        }
    ],
    listeners: {
        'row-dblclick'(e, d) {
            if (d && d.value && d.value.$item && d.value.$item.TypeClass) {
                d.value.$item.TypeClass.show();
            }
        }
    },
    async _getFields(source) {
        const ds = await Promise.all((await source.fields).map(async i => {
            try {
                const fields = (await i.fields || []).filter((i, n) => !n).map(() => {
                    return { template: 'oda-table-loading' };
                });
                return { $item: i, items: fields, group: this._getGroupname(i.$group.name) };
            } catch (err) {
                console.error(err);
            }
            return { $item: i, items: [], group: this._getGroupname(i.$group.name) };
        }));
        if (ds && ds.length) {
            return ds;
        } else {
            return [this._getDummy(source)];
        }
    },
    _getGroupname(name) {
        switch (name) {
            case '[FIELDS]': return '1.' + name;
            case '[STATIC]': return '2.' + name;
            case '[DETAILS]': return '3.' + name;
            default: return '4.' + name;
        }
    },
    _getDummy(source) {
        return { $item: { $group: source, label: 'NO-FIELDS', dummy: true, icon: '', color: 'gray' }, group: this._getGroupname(source.name), $allowFocus: false, };
    },
    _deleteDummy(source) {
        const group = this._getGroupname(source.name);
        this.deleteItems(i => i.$item && i.$item.dummy && i.group === group);
    },
    async _event(e, d) {
        if (e.detail && !(e.detail instanceof odaField)) return;
        const node = this.find(i => {
            return i.$item === e.currentTarget;
        });
        switch (e.source) {
            case 'changed': {
                if (node) {
                    node.items = await this._beforeExpand(node) || [];
                    if (!node.items.length) {
                        this.set(node, '$expanded', false);
                        this._refresh();
                    }
                }
            } break;
            case 'create': {
                if (node) {
                    node.items = await this._beforeExpand(node) || [];
                    this.set(node, '$expanded', true);
                } else {
                    const fields = (await d.fields || []).filter((i, n) => !n).map(() => {
                        return { template: 'oda-table-loading' };
                    });
                    this._deleteDummy(d.$group);
                    this.dataSet.push({ $item: d, items: fields, group: this._getGroupname(d.$group.name), $parent: node });
                }
                this.async(() => {
                    this.focusedNode = this.find(i => i.$item && i.$item.id === d.id) || this.focusedNode;
                    this.selection.splice(0, this.selection.length, this.focusedNode);
                }, 300);
            } break;
            case 'deleted': {
                if (node) {
                    if (!d.$group.ATTRS.length) {
                        this.dataSet.push(this._getDummy(d.$group));
                    }
                    this.deleteItems(i => i === node);
                    this.async(() => {
                        this.focusedNode = null;
                        this.selection.splice(0, this.selection.length);
                    }, 300);
                }
            } break;
        }
    },
    _drop(e, item, dropmode) {
        e.preventDefault();
        let dragItems = this.dragSources;
        if (dragItems.length > 0) {
            dragItems.forEach(item => item.$item.remove());
            switch (dropmode) {
                case 'top': {
                    let parent = this.getParentItem(item);
                    const index = parent.item.ATTRS.indexOf(item.$item.Root);
                    parent.item.ATTRS.splice(index, 0, ...dragItems.map(item => item.$item.Root));
                } break;
                case 'in': {
                    if (!item.item.Root.$ATTR) {
                        item.item.Root.$ATTR = [];
                    }
                    item.item.Root.$ATTR.push(...dragItems.map(item => item.$item.Root));
                } break;
                case 'bottom': {
                    let parent = this.getParentItem(item);
                    const index = parent.item.ATTRS.indexOf(item.$item.Root);
                    parent.item.ATTRS.splice(index + 1, 0, ...dragItems.map(item => item.$item.Root));

                } break;
            }
        }
        return true
    }
});

ODANT({
    is: 'odant-field-tree-group',
    imports: '@oda/table.js',
    extends: 'oda-table-expand',
    template: /* html */`
    <style>
        :host{
            @apply --horizontal;
            @apply --flex;
            @apply --header;
            margin-bottom: 1px;
            min-height: 32px;
            flex: 1 1 !important;
        }
        oda-item-node{
            max-width: 120px;
            min-width: 120px;
            @apply --no-flex;
            padding: 2px;
            fill: white;
            width: 120px;

        }
        *{
            color: white;
        }
        .divider{
            margin: 8px;
            border: 1px solid gray;
            width: 0px;
            transform: rotate(15deg);
        }
        :host>div>div{
            align-items: center;
        }
    </style>
    <div class="vertical flex header">
        <div class="flex horizontal">
            <odant-item-node :size="iconSize" @tap.stop="_toggleExpand()" style="font-size: x-large" hide-prefix hide-suffix :context-item="contextItem" menu-groups="admin"></odant-item-node>
            <odant-action-bar ~show="_showActions(table && table.allowActions)" :size="iconSize/1.5" auto-hide class="header flex" groups="admin" class="content" :context-item="contextItem"></odant-action-bar>
        </div>
        <div ~show="_showActions(table && table.allowActions)" ~for="i in  fields" class="horizontal  flex" @down.stop="" style="align-items: center">
            <odant-item-node :size="iconSize" hide-prefix hide-suffix disabled :context-item="i"></odant-item-node>
            <odant-action-bar :size="iconSize/1.5" auto-hide class="header flex" groups="admin" class="content" :context-item="i"></odant-action-bar>
        </div>
    </div>`,
    props: {
        contextItem: {
            type: Object,
            get() {
                return this.item.items && this.item.items[0] && this.item.items[0].$item && this.item.items[0].$item.$group || {};
            }
        },
        selectedItem: {
            type: Object,
            watch(item, old) {
                const fields = [];
                if (item) {
                    let i = item;
                    while (i && i !== this.contextItem) {
                        fields.push(i);
                        i = i.$owner;
                    }
                    fields.reverse();
                }
                this.fields = fields;
            }
        },
        fields: {
            type: Array,
            value() {
                return []
            }
        }
    },
    observers: [
        function _tableItemChanged(table, item) {
            if (!table || !(table instanceof EventSource) || !item) return;

            this.listen(table, 'selection-changed', '_setSelectedItem');
            if (table.items.indexOf(item) === 0 && !item.$expanded) {
                this._toggleExpand();
            }
        }
    ],
    _showActions() {
        return this.table.allowActions && this.table.contextItem && this.table.contextItem.adminMode;
    },
    _setSelectedItem(e, d) {
        const array = e.detail.value.filter(i => {
            let item = i.$item;
            while (item) {
                if (item === this.contextItem)
                    break;
                item = item.$owner;
            }
            return !!item;
        }).map(i => i.$item);
        if (array.length) {
            this.selectedItem = ((array.length === 1) ? array[0] : new odaItemList({ items: array }));
        }
        else {
            this.selectedItem = null;
        }
    },
    detached() {
        this.unlisten(this.table, 'selection-changed', '_setSelectedItem');
    }
});

ODANT({
    is: 'odant-field-tree-cell',
    imports: [
        '@oda/table',
        '@lib/item-node'
    ],
    extends: 'oda-table-cell-base, odant-item-node',
    props: {
        showName: true,
        hidePrefix: true,
        allowAdmin: false,
    }
});