if (!globalThis.CORE) {
    const attrMap = {
        Name: 'name',
        Label: 'label',
        Type: 'type'
    }
    const configMap = {
        i: 'id',
        l: 'label',
        n: 'name',
        t: 'type',
        a: 'accessLevel',
        v: 'version',
    }
    const oldModulesExclude = [
        'demo', 'lib', 'dist'
    ]
    const coreReservedWords = [
        'is', 'template', 'props', 'extends', 'keys', 'observers', 'listeners', 'hostAttributes'
    ]
    const prefixMap = {
        H: 'host',
        P: 'part',
        B: 'base',
        // S: 'system',
        W: 'workplace',
        D: 'domain',
        M: 'module',
        C: 'class',
        O: 'object',
        // I: 'index',
    }
    const prefMap = {
        'C:': ['B:', 'M:', 'W:', 'D:'],
        'B:': ['P:', 'B:'],
        'M:': ['B:', 'P:'],
        'W:': ['B:'],
        'D:': ['B:', 'P:'],
        'P:': ['H:'],
        'H:': null
    };
    //const prefArray = ['', 'H:', 'P:', ['B:', 'M:', 'W:', 'D:'], 'C:'];
    const targetStack = [];
    const SERVER_MODE = typeof globalThis.process !== 'undefined';
    const DEFAULTORIGIN = 'https://current.odant.org';
    // const ACTION_DIR_MAP = {
    //     'client': ['views', 'pages', 'methods', 'admin'],
    //     'server': ['methods', 'tasks', 'triggers']
    // };
    const proxyActionHandler = {
        get: (target, key) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (!prop || (typeof prop.value === 'function'))
                return target[key];
            return target.getAttr(key, target[key], prop.value && prop.value.type);
        },
        set: (target, key, value) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (prop && !prop.readOnly && (prop.set || prop.value) && target.$class) {
                // TODO
                if (!target.ROOT) {
                    let action;
                    if (target.$module) {
                        if (!target.$class.ROOT.$MODULES) {
                            target.$class.ROOT.$MODULES = [{ $MODULE: [{ id: target.$module.id, $ACTION: [] }] }];
                            action = this.$class?.ROOT?.$MODULES?.[0].$MODULE[0].$ACTION;
                        } else {
                            let module = this.$class?.ROOT?.$MODULES?.[0].$MODULE.find((m) => m.id === target.$module.id);
                            if (!module) {
                                const l = this.$class?.ROOT?.$MODULES?.[0].$MODULE.push({ id: target.$module.id, $ACTION: [] })
                                module = this.$class?.ROOT?.$MODULES?.[0].$MODULE[l - 1];
                            }
                            action = module.$ACTION;
                        }

                    } else {
                        if (!target.$class.ROOT.$ACTIONS) {
                            target.$class.ROOT.$ACTIONS = [{ $ACTION: [] }];
                        }
                        action = target.$class.ROOT.$ACTIONS[0].$ACTION;
                    }
                    action.push({ id: target.id });
                }
                // delete if equal default value
                target.setAttr(key, (target[key] !== value) ? value : undefined)
                target.$class.fireChanged();
            }
            else {
                target[key] = value;
            }
            return true;
        }
    }
    globalThis.RequestError = globalThis.RequestError || class RequestError extends Error {
        constructor(response, message) {
            super(message);
            this.errorCode = response.headers.get('odant-error-code');
            this.status = response.status;
            this.statusText = response.statusText;
            this.url = response.url;
        }
    };
    globalThis.CORE = function (prototype) {
        const parents = (typeof prototype.extends === 'string'
            ? prototype.extends.replace(/ /g, '').split(',').map((e) => {
                if (globalThis[e])
                    return globalThis[e];
                throw Error(`Parent class '${e}' for inherit to '${prototype.is}' not found!`)
            })
            : [prototype.extends]).filter(Boolean);
        prototype.listeners = prototype.listeners || {};
        prototype.props = prototype.props || {};
        for (let key in prototype) {
            if (coreReservedWords.includes(key)) continue;
            const item = Object.getOwnPropertyDescriptor(prototype, key);
            if (typeof item.value === 'function')
                continue;
            if (item.value) {
                prototype.props[key] = prototype.props[key] || item.value;
            }
            else if (item.get || item.set) {
                prototype.props[key] = prototype.props[key] || {};
                prototype.props[key].get = item.get;
                prototype.props[key].set = item.set;
            }
            delete prototype[key];
        }
        for (let key in prototype.props) {
            let prop = prototype.props[key];
            if (typeof prop === 'function')
                prop = (prop.name === key) ? { get: prop } : { type: prop };
            if (Array.isArray(prop))
                prop = { default: prop, type: Array };
            else if (prop === null || typeof prop !== "object")
                prop = { default: prop, type: prop ? prop.__proto__.constructor : Object };
            else if (Object.keys(prop).length === 0 || (!prop.get && !prop.set && prop.default === undefined && !prop.type))
                prop = { default: prop, type: Object };
            if (typeof prop.get === 'string')
                prop.get = prototype[prop.get];
            if (typeof prop.set === 'string')
                prop.set = prototype[prop.set];
            if (!prop.type && prop.default !== undefined)
                prop.type = prop.default.__proto__.constructor;
            prototype.props[key] = prop;
        }

        if (parents) {
            for (let parent of parents) {
                for (let key in parent.$prototype.props) {
                    const parentProp = parent.$prototype.props[key];
                    //if (parentProp) continue;
                    const targetProp = prototype.props[key];
                    if (targetProp) {
                        for (let i in parentProp) {
                            if (i === 'get' && 'default' in targetProp) continue;
                            if (i === 'default' && 'get' in targetProp) continue;
                            targetProp[i] = targetProp[i] || parentProp[i];
                        }
                    }
                    prototype.props[key] = targetProp || parentProp;
                }
                for (const k in parent.$prototype.listeners) {
                    if (!prototype.listeners[k])
                        prototype.listeners[k] = parent.$prototype.listeners[k];
                }
                for (let key in parent.$prototype) {
                    const targetMethod = Object.getOwnPropertyDescriptor(prototype, key);
                    if (targetMethod && targetMethod.value)
                        continue;
                    const parentMethod = Object.getOwnPropertyDescriptor(parent.$prototype, key);
                    if (parentMethod && (parentMethod.get || typeof parentMethod || typeof parentMethod.value === 'function'))
                        Object.defineProperty(prototype, key, parentMethod);
                }
            }
        }
        if (this) {
            for (const k in this.$prototype.listeners) {
                if (!prototype.listeners[k])
                    prototype.listeners[k] = this.$prototype.listeners[k];
            }
        }
        const clsName = prototype.is || (this ? ('$' + this.name) : 'cls');
        const body = `with (globalThis) {
            function ${clsName} (props){
                if(!new.target)
                    return CORE.call(${prototype.is}, props);
                // super(props);
                this.__cache__ = {};
                this.__customCache__ = {};
                this.__items__ = {};
                this.__deps__ = {};
                this.__handlers__ = {};
                
                this.constructor.defaults = this.constructor.defaults || {};
                let obj = this.constructor.prototype.__proto__.constructor;
                while (obj) {
                    for (const k in obj.defaults) {
                        if ((k in this.constructor.defaults) || p.$prototype.props[k]?.get) continue;
                        this.constructor.defaults[k] = obj.defaults[k];
                    }
                    obj = obj.prototype.__proto__?.constructor;
                }
                for (let p in this.constructor.defaults){
                    let val = this.constructor.defaults[p];
                    if (val && typeof val === 'object'){
                        if (Array.isArray(val))
                            val = Array.from(val);
                        else
                            val = Object.assign({}, val);
                    }
                    this.__cache__[p] = val;
                }
                for (let p in props){
                    let value = props[p];
                    const prop = this.props[p];
                    if (value !== undefined && prop){
                        if (prop.type === Boolean)
                            value = (value === 'true') || !!value;
                        else if (prop.type === Number)
                            value = +value;
                    }
                    this.__cache__[p] = value;
                }
                if (prototype.created)
                    prototype.created.call(this);
                if(prototype.listeners && typeof prototype.listeners === 'object'){
                    for (const k in prototype.listeners) {
                        const h = typeof prototype.listeners[k] === 'string'
                            ? this[prototype.listeners[k]]
                            : prototype.listeners[k];
                        this.listen(k, h);
                    }
                }
            }
            return ${clsName}
        }`;
        const cls = (new Function('p, prototype', body))(this, prototype);
        cls.prototype = Object.create((this || CORE).prototype);
        cls.prototype.constructor = cls;
        cls.$prototype = prototype;

        Object.defineProperties(cls.prototype, {
            listen: {
                value: function (type, handler) {
                    if (!this.__handlers__[type]) this.__handlers__[type] = [];
                    this.__handlers__[type].push(handler);
                }
            },
            unlisten: {
                value: function (type, handler) {
                    if (!this.__handlers__[type]) return;
                    const idx = this.__handlers__[type].indexOf(handler);
                    if (~idx) this.__handlers__[type].splice(idx, 1);
                }
            },
            addEventListener: {
                value: cls.prototype.listen
            },
            removeEventListener: {
                value: cls.prototype.unlisten
            },
            fire: {
                value: async function (type, detail) {
                    if (!this.__handlers__[type]) return;
                    const promises = [];
                    for (const h of this.__handlers__[type]) {
                        if (h.constructor.name === 'AsyncFunction') {
                            promises.push(h.call(this, undefined, detail));
                        } else {
                            h.call(this, undefined, detail);
                        }
                    }
                    await Promise.all(promises);
                }
            }
        })
        Object.defineProperty(cls.prototype, 'props', {
            get() {
                return prototype.props;
            }
        })
        if (prototype.is && !globalThis[prototype.is]) {
            globalThis[prototype.is] = cls;
        }
        cls.defaults = cls.defaults || {};
        for (let key in prototype.props) {
            const prop = prototype.props[key];
            if (prop.default !== undefined)
                cls.defaults[key] = prop.default;
            const description = { enumerable: !key.startsWith('_'), configurable: false };
            description.get = function () {
                const stack = targetStack ? [...targetStack].reverse() : [];
                for (let i of stack) {
                    if (i.target === this && i.prop === key)
                        continue;
                    let deps = this.__deps__[key];
                    if (!deps)
                        deps = this.__deps__[key] = [];
                    if (deps.find(d => {
                        return d.target === i.target && d.prop === i.prop;
                    })) continue;
                    deps.push(i);
                }
                if (prop.get && !(key in this.__cache__)) {
                    targetStack.push({ target: this, prop: key })
                    this.__cache__[key] = prop.get.call(this);
                    targetStack.pop();
                    const deps = this.__deps__[key] || [];
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                }
                return this.__cache__[key];
            }
            description.set = prop.set ?
                function (val) {
                    this.__cache__[key] = val;
                    prop.set.call(this, val);
                    const deps = this.__deps__[key] || []
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                } : (prop.readOnly ? undefined :
                    function (val) {
                        this.__cache__[key] = val;
                        const deps = this.__deps__[key] || []
                        for (let d of deps) {
                            delete d.target.__cache__[d.prop];
                        }
                    })
            Object.defineProperty(cls.prototype, key, description);
        }
        for (let key in prototype) {
            const d = Object.getOwnPropertyDescriptor(prototype, key);
            if (typeof d.value === 'function') {
                cls.prototype[key] = function (...args) {
                    return d.value.call(this, ...args);
                }
            }
            else if (d.get || d.set)
                Object.defineProperty(cls.prototype, key, d);
        }
        const prefix = prototype.props?.prefix?.default;
        if (prefix) {
            CORE.__factory__[prefix] = CORE.__factory__[prefix] || cls;
        }
        // console.dir(cls);
        return cls;
    }
    CORE.getContextFromUrl = function (url) {
        if (url) {
            url = decodeURI(url);
            url = url.split('?')[0];
            url = url.split('#')[0];
            let i = url.indexOf('/api/');
            if (i < 0)
                return null;
            url = url.substring(i + 4);
            i = url.lastIndexOf('/~');
            if (i < 0) {
                url = url.split('/').filter(s => {
                    return s.indexOf(':') === 1;
                }).join('/');
            }
            else {
                url = url.substring(0, i);
            }

            return url;
        }
        return '';
    }
    CORE.__factory__ = {};
    CORE.getFactory = function (prefix) {
        return CORE.__factory__[prefix];
    }
    if (!SERVER_MODE && !document.location.host.startsWith('localhost'))
        CORE.URL = new URL(document.location.origin);
    else
        CORE.URL = new URL(DEFAULTORIGIN);
    CORE({
        is: 'odaAccess',
        props: {
            level: Number,
            label() {
                switch (this.level) {
                    case 6: return 'admin';
                    case 5: return 'delete';
                    case 4: return 'create';
                    case 3: return 'write';
                    case 2: return 'read';
                    case 1: return 'preview';
                    case 0: return 'none';
                    default: return 'unknown';
                }
            },
            color() {
                switch (this.level) {
                    case 6: return 'var(--admin-color)';
                    case 5: return 'var(--rwcd-color)';
                    case 4: return 'var(--rwc-color)';
                    case 3: return 'var(--rw-color)';
                    case 2: return 'var(--r-color)';
                    case 1: return 'var(--p-color)';
                }
                return 'var(--no-access-color)';
            },
            letter() {
                return this.label[0].toUpperCase();
            }
        }
    })
    CORE({
        is: 'odaIndexDataSet',
        props: {
            $index: Object,
            mask: String,
            filter: String,
        },
        get Items() {
            return (async () => {
                let data = await this.$index.request((this.filter) ? 'search_index' : 'xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    this.filter || '*/*');

                data = data ? data[Object.keys(data)[0]] : [];

                return data;
            })();
        },
        get metadata() {
            return this.$index.$$fields;
        },
        async getItems(from = 1, length = 100) {
            try {
                const data = await this.$index.request('xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    `subsequence(*/*, ${from}, ${length})`);
                return data[Object.keys(data)[0]];
            } catch (e) {
                console.error(e);
                return [];
            }
        },
        async getRowItem(row) {
            if (row) {
                let rows = Array.isArray(row) ? row : [row];
                if (rows.length === 0)
                    return Promise.resolve(null);

                const item = await this.$index.$class.getObject(rows[0].oid);
                if (rows.length === 1)
                    return item;
                return null;
                // TODO odaItemList
                //return new odaItemList({ $owner: this.$index.$class, $item: item, $$items: rows.map(i => { return i.oid }) });
            }
            return Promise.reject();
        }
    })
    CORE({
        is: 'odaInherit',
        props: {
            name: String,
            level: Number,
            color() {
                switch (this.name) {
                    case 'owner':
                        return 'var(--inherit-owner)';
                    case 'parent':
                        return 'var(--inherit-parent)';
                    case 'type':
                        return 'var(--inherit-type)';
                    case 'core':
                        return 'var(--inherit-core)';
                    case 'service':
                        return 'var(--inherit-service)';
                }
            }
        }
    })
    CORE({
        is: 'odaItem',
        props: {
            $part() {
                return this.$owner.$path;
            },
            id: String,
            typeName: String,
            $class() {
                // No $class in host
                return this.$owner?.$class;
            },
            $module() {
                return this.$owner?.$module;
            },
            $base() {
                // No $base in host
                return this.$owner?.$base;
            },
            $host() {
                return this.$owner.$host;
            },
            $owner: null,
            name() {
                return this.id;
            },
            label() {
                return this.name;
            },
            icon: 'odant:item',
            defaultViewName: 'editor',
            defaultPage: 'navigator',
            defaultForm: 'form',
            accessLevel: undefined,
            access: {
                category: 'system',
                get() {
                    if (!this.__cache__.accessLevel && this.$owner && this.$part?.id === 'ROOT')
                        return this.$owner.access;
                    return new odaAccess({ level: this.__cache__.accessLevel });
                }
            },
            inherit() {
                const root = this.ROOT || this.config || {};
                let level = root.inherit;
                if (level === undefined && this.$owner)
                    return this.$owner.inherit;
                return new odaInherit({ name: level });
            },
            _color: 'var(--item-color)',
            color() {
                if (this.access?.level >= 6) return this.inherit?.color || this._color;
                return this._color;
            },
            fill() {
                return this.color;
            },
            actionsPath() {
                return `web/${this.typeName}/client/*/*`;
            },
            isRef() {
                return this.$owner && (this.$owner.isRef || this.$owner instanceof odaWorkPlace);
            },
            zIndex: 0,
        },
        get $$items() {
            return this.cache('$$items', async () => {
                let items = [];
                for (let i in this) {
                    if (i.startsWith('$$') && i !== '$$items')
                        items.push(...(await this[i]));
                }
                return items;
            })
        },
        get $$actions() {
            return this.getActions();
        },
        save(body) {
            if (this.access.level > 2) {
                this.fire('before-save', this);
                return this._save(body).then(res => {
                    this.fire('saved', this);
                    this.fire('after-save', this);
                    return res
                })

                // TODO events
                // if (!this.fire('before-save', this)) {
                //     return this._save(body).then(res => {
                //         this.fire('saved', this);
                //         this.fire('after-save', this);
                //         return res
                //     })
                // }
                // else {
                //     let p = this.events['before-save'].map((i) => {
                //         if (i.then)
                //             return i;
                //         return Promise.resolve(i());
                //     });
                //     return Promise.all(p).then(() => {
                //         return this._save(body).then(res => {
                //             this.fire('saved', this);
                //             this.fire('after-save', this);
                //             return res
                //         })
                //     })
                // }
            }
        },
        _save(body) {
            body = body || this.body;
            if (body) {
                return this.request(undefined, undefined, body).then(res => {
                    this.body = null;
                })
            }
            throw new Error('no data for save');
        },
        import(path) {
            return import(this.url + '/~/' + path);
        },
        getActionFiles(node) {
            return (node.$DIR || []).reduce((res, cat) => {
                const files = (cat.$DIR || []).reduce((res, d) => {
                    const f = d.$FILE?.find(f => f.id === `${d.id}.js`);
                    if (f) {
                        f.parent = d;
                        f.owner = cat;
                        res.push(f);
                    }
                    return res;
                }, [])
                res.push(...files);
                return res;
            }, [])
            // return node?.$DIR?.reduce((res, d) => {
            //     // if (map) {
            //     //     if (map === d.id || map[d.id] || map.includes?.(d.id)) {
            //     //         res.push(...this.getActionFiles(d, map[d.id] || null, d));
            //     //     }
            //     // } else {
            //         const f = d.$FILE?.find(f => f.id === `${d.id}.js`);
            //         if (f) {
            //             f.parent = d;
            //             f.owner = cat;
            //             res.push(f);
            //         }
            //     // }
            //     return res;
            // }, []);
        },
        async getActions($item = this) {
            return this.cache('getActions' + $item?.typeName, async () => {
                if (!$item.config) await $item.getConfig?.();
                const body = await $item?.$class?.body;
                let path = $item.actionsPath;
                const endIdx = path.indexOf('*');
                if (~endIdx) path = path.slice(0, endIdx - 1);
                const url = `${this.$folder.url}/${path}?get_dirlist&deep=3`;
                const dirStruct = await (await fetch(url)).json();
                //TODO: доработать для клиентской и серверной стороны
                const actFiles = this.getActionFiles(dirStruct/*, ACTION_DIR_MAP.client*/);
                let actions = actFiles.map(async f => {
                    const module = { /* $folder: f, */ id: f.parent.id, path: f.parent.path, $owner: $item, $source: this };
                    let action;
                    try {
                        const result = await import(f.path);
                        action = new result.default(module);
                    }
                    catch (e) {
                        console.error(f.path, e);
                        action = new odaActionError(module);
                    }
                    action.category = action.category || f?.owner?.id;
                    return action;
                })
                actions = (await Promise.all(actions));
                actions = actions.filter(Boolean);

                // вызывает все тормоза при открытии контекстного меню и т.п.
                console.time(`getActionsExp ${this.path}`);
                if (!(this.$owner instanceof odaModule)) {
                    const exports = (await this.$base?.getActionsExp(this)) || [];
                    // console.dir(exports);
                    actions.push(...exports);
                }
                console.timeEnd(`getActionsExp ${this.path}`);

                //todo фильтрация конечного списка перед выводом
                return actions.map(a => new Proxy(a, proxyActionHandler)).filter(a => a.active);
            });
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const actions = await this.getActions($item);
                return actions.filter(i => i.export);
            });
        },
        async findAction(name) {
            const actions = await this.getActions();
            return actions.find(i => i.name === name);
        },
        cache(key, callback) {
            let result = this.__customCache__[key];
            if (result === undefined) {
                if (typeof callback === 'function')
                    result = callback();
                else
                    result = callback;
                this.__customCache__[key] = result;
            }
            return result;
        },
        toJSON() {
            let obj = {};
            for (let i in this) {
                if (this[i] && typeof this[i] !== 'object')
                    obj[i] = this[i];
            }
            return obj;
        },
        getFormUrl(form, view) {
            const $form = `${form || this.defaultForm}/index2.html`;
            const $view = view ? `?view=${view}` : '';
            const url = `${this.url}/~/client/views/${$form}${$view}`;

            //return ODANT.URL(url);
            return url;
        },
        open(form, view) {
            const url = this.getFormUrl(form, view);
            window.open(url, '_blank');
        },
        async show(view, params = {}, dialog = false) {
            view = view || this.defaultViewName;
            const action = await this.getViewAction(view);
            const form = await ODA.createComponent('odant-form', params);
            form.viewName = view;
            form.contextItem = this;
            form.unique = this.path + "#" + form.viewName;
            return form.show(dialog);
        },
        navigate(form, view) {
            const url = this.getFormUrl(form, view);
            window.open(url);
        }
    })
    odaItem({
        is: 'odaVirtualItem'
    })
    odaItem({
        is: 'odaBodyOwner',
        props: {
            ROOT: Object,
            body() {
                return this.cache('body', async () => {
                    const body = await this.request();
                    this.ROOT = body?.[this.rootTag]?.[0];
                    await this.fire('body');
                    return body;
                })
            }
        }
    })
    odaItem({
        is: 'odaFieldsOwner',
        props: {
            _fieldRootKeys: ['METADATA'/* , 'STATIC', 'DETAILS' */]
        },
        get _$$config() {
            return this._$$fields;
        },
        get $$fields() {
            return this.cache('$$fields', async () => {
                await this.body;
                const promises = [];
                for (const k of this._fieldRootKeys) {
                    const attrs = [];
                    if (Array.isArray(this[k]?.$ATTR)) {
                        attrs.push(...(this[k]?.$ATTR));
                    }
                    if (attrs.length === 0 && this.ROOT?.$ATTR) {
                        attrs.push(...this.ROOT.$ATTR);
                    }
                    if (this.$typeClass) {
                        await this.$typeClass.body;
                        if (Array.isArray(this.$typeClass[k]?.$ATTR)) {
                            this.$typeClass[k]?.$ATTR.forEach(a => {
                                const _a = attrs.find(_a => _a.name === a.name);
                                if (_a) {
                                    // возможно изменяет исходные данные
                                    // и это будет зажигать needSave
                                    for (const k in a) {
                                        _a[k] = _a[k] || a[k];
                                    }
                                } else {
                                    attrs.push(a);
                                }
                            });
                        }
                    }
                    promises.push(...attrs.map(async a => {
                        const config = { ROOT: a, $owner: this, place: k };
                        for (let p in a) {
                            const s = attrMap[p];
                            if (s === undefined) continue;
                            config[s] = a[p];
                        }
                        if (!a.Type) return new odaField(config);
                        try {
                            const typeCls = (await this.$base.findItem(a.Type)) || (await this.$host.findItem(a.Type));
                            if (!typeCls) return new odaField(config);
                            config.$typeClass = typeCls;
                            const url = `${typeCls.url}/~/fields/${!typeCls.isAbstract ? '' : 'links/'}extend.js`;
                            const imp = await import(url);
                            return new imp.default(config);
                        }
                        catch (e) {
                            console.warn(e)
                            return new odaField(config);
                        }
                    }));
                }
                return await Promise.all(promises);
            });
        },
        get _$$fields() {
            // return this.$$fields;
            return this._get$$fields();
        },
        async _get$$fields() {
            const fields = await this.$$fields;
            if (this.typeName !== 'class') return fields;
            const groups = Object.fromEntries(this._fieldRootKeys.map(k => [k, []]));
            for (const f of fields) {
                groups[f.place].push(f);
            }
            const result = [];
            for (const k in groups) {
                result.push(new odaFieldGroup({
                    label: k === this._fieldRootKeys[0] ? 'FIELDS' : k,
                    name: k === this._fieldRootKeys[0] ? 'FIELDS' : k,
                    _$$config: groups[k],
                    ItemCount: groups[k].length,
                    $class: this,
                    place: k
                }));
            }
            return result;

            // const map = { 'M': '$$Modules', 'W': '$$Workplaces', 'C': 'other', 'P': 'other', 'B': 'other' };
            // const allowAdmin = this.access.level >= 6;
            // items: for (const i of fields) {
            //     if (!allowAdmin && (i.hidden || i.config.hidden)) continue;
            //     for (let ex of this.structureExcludes) {
            //         const parts = ex.split(':');
            //         ex = `${parts[0].replace('*', i.prefix)}:${parts[1].replace('*', i.name)}`;
            //         if (ex === `${i.prefix}:${i.name}`) continue items;
            //     }
            //     const category = i.category ? i.category : map[i.prefix];
            //     if (!(category in groups)) groups[category] = [];
            //     groups[category].push(i);
            // }
            // const result = [];
            // for (const g in groups) {
            //     if (g !== 'other') {
            //         const group = new odaGroup();
            //         group.$owner = this;
            //         switch (g) {
            //             case '$$Modules':
            //                 group.id = 'Modules';
            //                 group.type = 'module';
            //                 break;
            //             case '$$Workplaces':
            //                 group.id = 'Workplaces';
            //                 group.type = 'workplace';
            //                 break;
            //             default:
            //                 group.id = g;
            //                 group.type = 'category';
            //         }
            //         group._$$structure = groups[g];
            //         result.push(group);
            //     } else {
            //         result.push(...groups[g]);
            //     }
            // }
            // return result;
        },
        listeners: {
            body: function () {
                for (const k of this._fieldRootKeys) {
                    Object.defineProperty(this, k, {
                        get() {
                            return this.getRootElement(k);
                        }
                    });
                }
            }
        },
    })
    odaItem({
        is: 'odaIndexesOwner',
        get $$indexes() {
            return this.cache('$$indexes', async () => {
                await this.body;
                let folders = await this.getFolders('index/*')
                if (!folders) return [];
                const indexes = folders.map(async (f) => {
                    // TODO временно через файл
                    const file = await f.getFile(f.name + '.xq');
                    if (!file) return null;
                    const result = await file.request('xquery', { format: 'json' }, '*');
                    const conf = { id: f.name, name: f.name, $owner: this };
                    if (result?.$XQUERY) {
                        conf.ROOT = result.$XQUERY[0];
                        for (let p in conf.ROOT) {
                            const s = attrMap[p];
                            if (s === undefined) continue;
                            conf[s] = conf.ROOT[p];
                        }
                    }
                    return new odaIndex(conf);
                });
                return (await Promise.all(indexes)).filter(Boolean);

            });
        },
        getIndex(name, XQ, type) {
            return this.cache('Index:' + name, async () => {
                if (XQ) {
                    const param = { name: name };
                    if (type) param.type = type;
                    await this.API.POST('create_index', param, XQ);
                }
                const index = (await this.$$indexes).find(i => i.name === name)
                return index || new odaIndex({ id: name, name, $owner: this });
            })
        }
    })
    odaItem({
        is: 'odaFilesNFoldersOwner',
        props: {
            $$files: {
                type: Array,
                get() {
                    return this.getFiles();
                }
            },
            $$folders: {
                type: Array,
                get() {
                    return this.getFolders();
                }
            }
        },
        async getDirList() {
            return this.cache('get_dirlist', async () => await this.request('get_dirlist', { deep: 2 }));
        },
        async getFolders(path = '*') {
            return this.cache('get-folder-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        return (files?.$DIR || []).map(i => {
                            let folder = this.__items__[i.name];
                            if (!folder) {
                                i.$owner = this;
                                folder = this.__items__[i.name] = new odaFolder({ id: i.id, name: i.name, $owner: this, accessLevel: i.access });
                                folder.config = i;
                            }
                            return folder;
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                }
                else {
                    const steps = path.split('/');
                    path = steps.shift();
                    // todo переделать на маски
                    let folders = await this.getFolders();
                    if (path !== '*')
                        folders = folders.filter(i => i.name === path);
                    if (steps.length) {
                        path = steps.join('/');
                        folders = folders.map(async f => {
                            return (await f.getFolders(path)).flat();
                        });
                    }
                    return (await Promise.all(folders)).flat();
                }
            })
        },
        async getFiles(path = '*') {
            return this.cache('get-files-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        return (files.$FILE || []).map(i => {
                            i.$owner = this;
                            const file = new odaFile({ id: i.id, name: i.name, $owner: this, accessLevel: i.access });
                            file.config = i;
                            return file;
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                } else {
                    const steps = path.split('/');
                    const name = steps.pop();
                    path = steps.join('/');
                    const folders = await this.getFolders(path)
                    let files = folders.map(async f => (await f.getFiles('*')).flat());
                    files = (await Promise.all(files)).flat();
                    if (name === '*') return files;
                    return files.filter(i => i.name === name)
                }
            })
        },
        async getFolder(path) {
            return await this._getFromPath(path, false);
        },
        async getFile(path) {
            return await this._getFromPath(path, true);
        },
        async _getFromPath(path, isFile) {
            const steps = path.split('/');
            const name = steps.pop();
            path = steps.join('/');
            const items = await this[isFile ? 'getFiles' : 'getFolders'](path ? path + '/*' : '*');
            return items.find(i => i.name === name);

        },
        async show(view, params = {}, dialog = false) {
            view = view || this.defaultViewName;
            const action = await this.findAction(view);
            // TODO load
            const form = await ODA.createComponent('odant-form', params);
            form.viewName = view;
            form.contextItem = this;
            form.unique = this.path + "#" + form.viewName;
            return form.show(dialog);
        },
        saveFile(fileBody, path = '', inherit = false, mime) {
            let name = path.split('/').pop();
            switch (fileBody.constructor.name.toLowerCase()) {
                case 'string':
                    //if (ODANT.webMode) {
                    fileBody = new File([fileBody], name, { type: mime || "text/plain" });
                    //}
                    break;
                case 'array':
                case 'object':
                    //if (ODANT.webMode) {
                    fileBody = new File([JSON.stringify(fileBody)], name, { type: mime || "application/json" });
                    //}
                    break;
                case 'odafile': // todo <-- Еще смотреть как определять тип файла (MIME)
                    name = fileBody.id;
                    return fileBody.load().then(fileBody => {
                        return this.saveFile(fileBody, path || name);
                    });
                case 'file':
                case 'blob':
                case 'readstream':
                    break;
                default:
                    throw new Error('Unknown file type')
            }
            name = name || fileBody.name;
            const formData = new FormData();
            formData.append("file", fileBody, name);
            const url = inherit ? this.url : this.url.replaceAll('/~/', '/');
            return fetch(url + '/' + path, { method: 'POST', body: formData, credentials: 'include' }).then((res) => {
                this.reset?.();
                return this.getFile(path).then(file => {
                    file.$owner.fireChanged?.(file);
                    file.$owner.fire?.('create', file);
                    return file;
                });
            })
        }
    })
    odaItem({
        is: 'odaServerItem',
        props: {
            version: 0,
            prefix: String,
            body: Object,
            id: undefined,
            name() {
                return this.ROOT?.name || this.id;
            },
            label() {
                return this.ROOT?.label || this.name;
            },
            icon() {
                if (this.config.ico)
                    return this.url + '/~/' + this.config.ico;
                return this.__cache__.icon;
            },
            path() {
                return (this.$owner ? (this.$owner.path + '/') : '') + (this.prefix ? this.prefix + ':' : '') + this.id;
            },
            url() {
                return this.$host.origin + '/api' + (this.id ? ('/' + this.path) : '');
            },
            $folder() {
                return new odaFolder({ $owner: this, id: '~' });
            },
            defaultViewName() {
                return 'editor';
            }
        },
        resolveUrl(path) {
            return this.url + '/' + path;
        },
        async request(method = '', params = {}, postObject) {
            try {
                let url = encodeURI(this.url + (method ? '?' + method : '') + Object.keys(params).reduce((res, p) => {
                    res += ((res || method) ? '&' : '?') + p + '=' + params[p];
                    return res;
                }, ''))
                if (typeof postObject !== 'undefined' && typeof postObject !== 'string') {
                    postObject = JSON.stringify(postObject);
                }
                params = { method: postObject ? 'POST' : 'GET', body: postObject, credentials: 'include' };
                const response = await fetch(url, params)
                switch (response.status) {
                    case 200: {
                        let content_type_header = response.headers.get('Content-Type');
                        let res = content_type_header ? content_type_header.split(';')[0] : content_type_header;
                        switch (res) {
                            case 'application/x.odant.async':
                            case 'application/x.odant.async+json':
                            case 'text/x-json':
                            case 'application/json':
                                return response.json();
                            case 'text/cmd':
                            case 'text/css':
                            case 'text/csv':
                            case 'text/javascript':
                            case 'application/javascript':
                            case 'text/php':
                            case 'text/html':
                            case 'text/plain':
                            case 'text/xml':
                                return response.text();
                            case 'image/png':
                            case 'image/gif':
                            case 'image/jpg':
                            case 'image/jpeg':
                            case 'application/octet-stream':
                                return CORE.webMode ? response.blob() : response.buffer();

                            default: {
                                throw new RequestError(response, 'Unresolved fetch mime-type: ' + res);
                            }
                        }
                    }
                    case 500: {
                        const text = await response.text();
                        throw new RequestError(response, text);
                    }
                    default: {
                        // throw new RequestError(response, response.statusText);
                    }
                }
            }
            catch (e) {
                // console.error(e)
            }
        },
        async load(params, fetchParams) {
            try {
                const res = await this.request(undefined, params, fetchParams);
                if (typeof res === 'object')
                    this.body = res;
                else
                    this.data = res;
                return res;
            }
            catch (e) {
                this.error = e;
                console.error(e);
            }
        },
        async findItem(path, params = { path: path, chain: 'full', prefix: true }) {
            const cacheKey = 'find-item:' + Object.keys(params).reduce((res, key) => {
                return res + `${key}=${JSON.stringify(params[key])}`;
            }, '');
            return this.cache(cacheKey, async () => {
                try {
                    if (!params.path.includes('~')) {
                        //todo: доработать
                        if (params.path.startsWith('/')) params.path = params.path.substring(1);
                        let s = params.path.split('/');
                        let last = s.pop();

                        const prefixKeys = Object.keys(prefMap);
                        let pref = prefixKeys.find(p => last.startsWith(p));
                        if (!pref) {
                            pref = 'C:';
                            last = 'C:' + last;
                        }
                        params.path = last;

                        while (prefMap[pref]) {
                            last = s.pop();
                            if (!last) {
                                pref = prefMap[pref][0];
                                params.path = `${pref}*/${params.path}`;
                            } else {
                                const map = prefMap[pref];
                                pref = map.find(p => last.startsWith(p));
                                if (!pref) {
                                    pref = map[0];
                                    params.path = `${pref}${last}/${params.path}`;
                                } else {
                                    params.path = `${last}/${params.path}`;
                                }

                            }
                        }
                        // s = params.path.split('/');
                        // let node = this.config;
                        // let path = this.path;
                        // while (s.length > 0 && node) {
                        //     const id = s.shift();
                        //     path += '/' + id;
                        //     const prefixId = id.split(':');
                        //     node = this.findConfigNode(node, prefixId[0], prefixId[1]);
                        // }
                        //
                        // if (node) {
                        //     return this.getItem(path);
                        // }
                    }

                    path = await this.request('find_item', params);
                    return this.$host.getItem(path.result);
                }
                catch (e) {
                    console.error(e)
                }
            });
        },
        findConfigNode(node, prefix, id) {
            let result = node[`$${prefix}`]?.find(n => n.i === id);
            if (result) return result;

            let root = node;
            for (const key in root) {
                if (!key.startsWith('$')) continue;

                for (const child of root[key]) {
                    result = this.findConfigNode(child, prefix, id)
                    if (result) return result;
                }
            }

            return result;
        },
        async getItem(path) {
            return this.cache('get-item:' + path, async () => {
                if (this.config === undefined) {
                    await this.getConfig?.();
                }

                let step = this.prefix + ':' + this.id;
                let idx = path.indexOf(step);
                if (idx < 0)
                    throw new Error('Undefined search context: ' + path);
                path = path.substring(++idx + step.length);
                if (!path)
                    return this;
                if (!path.includes(':')) {
                    // File
                    return path.includes('.')
                        ? this.$folder.getFile(path.replace('~/', ''))
                        : this.$folder.getFolder(path.replace('~/', ''));
                }
                step = path.split('/')[0];
                const prefix = step.split(':')[0];
                const id = step.split(':')[1];
                let ext, config;
                if (prefix === 'O') {
                    return this.getObject(id);
                } else {
                    await this.getConfig?.();
                    const items = this.config['$' + prefix];
                    config = items?.find(i => i.i === id);
                    ext = config?.extend || `/web/core/${prefixMap[prefix]}/extend.js`;
                }
                const imp = await import(ext);
                const item = new imp.default({ id, prefix, $owner: this });
                item.config = config || (await item.getConfig?.());
                return item.getItem(path);
            });
        },
        getAttr(name, def, type) {
            let res = undefined;
            if (this.ROOT) {
                name = name.split('|');
                while (!res && name.length) {
                    const n = name.shift().trim();
                    res = this.ROOT[n];
                }

                if (res === undefined || res === '' || res === null) return def;
                switch (type) {
                    case Boolean: {
                        res = String(res).toLowerCase() === 'true';
                    } break;
                    case Number: {
                        res = Number(res);
                    } break;
                    case Date: {
                        res = Date.parse(res);
                    } break;
                }
                return res;
            }
            return def;
        },
        setAttr(name, v) {
            if (this.ROOT && this.ROOT[name] !== v) {
                if (v === undefined || v === null)
                    delete this.ROOT[name];
                else
                    this.ROOT[name] = v;
                //this.fireChanged();
            }
        }
    })
    odaServerItem({
        is: 'odaStructureItem',
        extends: 'odaFilesNFoldersOwner',
        props: {
            config: {
                set(v) {
                    if (v) {
                        for (let p in v) {
                            const s = configMap[p];
                            if (s === undefined) continue;
                            this[s] = v[p];
                        }
                    }
                }
            },
            category() {
                return this.config.c || '';
            },
            structureListNames: [
                '$$classes',
                '$$bases',
                '$$parts',
                '$$workplaces',
                '$$modules',
            ],
            structureExcludes: ['P:ROOT', 'P:CACHE']
        },
        async getConfig(params = { prefix: true, deep: 2, group: false, byType: false }) {
            return this.cache(`get-config:${JSON.stringify(params)}`, async () => {
                // try{
                const res = await this.request('get_config', params);
                this.config = res['$' + this.prefix][0];
                return this.config;
                // }
                // catch (e){
                //     console.error(e);
                // }
            })
        },
        async _getItems(prefix) {
            const config = await this.getConfig()
            return Promise.all((config['$' + prefix] || []).map(i => {
                return this.getItem(this.path + '/' + prefix + ':' + i.i);
            }));
        },
        async getNearestAccessibleItem() {
            if (this.access.level > 3) return this;

            await this.getConfig({ deep: 10 });
            const configRoot = this.config;
            let nodes = await this.getAccessibleNodes({ item: configRoot, level: 0 });
            if (!nodes || nodes.length <= 0) return null;

            let itemNode;
            if (nodes.length === 1) {
                itemNode = nodes[0];
            } else {
                const minLevel = nodes.reduce((res, n) => (res > n.level) ? n.level : res, Infinity) - 1;

                nodes = nodes.reduce((res, n) => {
                    const max = n.level - minLevel;
                    let owner = n.owner;
                    for (let i = 1; i < max; i++) {
                        owner = owner.owner;
                    }

                    res.add(owner);
                    return res;
                }, new Set());

                while (nodes.size > 1) {
                    nodes = Array.from(nodes).reduce((res, n) => {
                        if (n.item.owner) res.add(n.item.owner);

                        return res;
                    }, new Set());
                }

                itemNode = nodes?.values?.()?.next?.()?.value;
            }

            let path;
            while (itemNode) {
                path = `${itemNode.item.t?.[0] || 'H'}:${itemNode.item.i}/${path ? path : ''}`;
                itemNode = itemNode.owner;
            }

            return await this.$host.getItem(path || this.path);
        },
        async getAccessibleNodes(owner, items = []) {
            for (const key in owner.item) {
                if (!key.startsWith('$')) continue;

                const level = owner.level + 1;
                for (const item of owner.item[key]) {
                    const o = { item, owner, level };
                    if (item.a > 3 && !item.hide) {
                        items.push(o);
                    } else {
                        await this.getAccessibleNodes(o, items);
                    }
                }
            }

            return items;
        },
        get _$$structure() {
            return this._getStructure();
        },
        async _getStructure() {
            const items = (await Promise.all(this.structureListNames.map(async n => (await this[n] || [])))).flat();
            const groups = { other: [] };
            const map = { 'M': '$$Modules', 'W': '$$Workplaces', 'C': 'other', 'P': 'other', 'B': 'other' };
            const allowAdmin = this.access.level >= 6;
            items: for (const i of items) {
                if (!allowAdmin && (i.hidden || i.config.hidden)) continue;
                for (let ex of this.structureExcludes) {
                    const parts = ex.split(':');
                    ex = `${parts[0].replace('*', i.prefix)}:${parts[1].replace('*', i.name)}`;
                    if (ex === `${i.prefix}:${i.name}`) continue items;
                }
                const category = i.category ? i.category : map[i.prefix];
                if (!(category in groups)) groups[category] = [];
                groups[category].push(i);
            }
            const result = [];
            for (const g in groups) {
                if (g !== 'other') {
                    const group = new odaGroup();
                    group.$owner = this;
                    switch (g) {
                        case '$$Modules':
                            group.id = 'Modules';
                            group.type = 'module';
                            break;
                        case '$$Workplaces':
                            group.id = 'Workplaces';
                            group.type = 'workplace';
                            break;
                        default:
                            group.id = g;
                            group.type = 'category';
                    }
                    group._$$structure = groups[g];
                    result.push(group);
                } else {
                    result.push(...groups[g]);
                }
            }
            return result;
        }
    })
    odaStructureItem({
        is: 'odaHost',
        props: {
            typeName: 'host',
            origin: String,
            $host() {
                return this;
            },
            $authHost: Object,
            $user: Object,
            prefix: 'H',
            icon: 'odant:host',
            defaultViewName: 'desktop-ext',
            uid: String,
            actionsPath: 'web/core/structure/client/*/*',
            supportUrl() {
                const rootPart = this.config?.$P?.find(n => n.i === 'ROOT');
                if (rootPart && Object.keys(rootPart).find(k => k === '$M')) return `${this.url}/P:ROOT`
                return `${this.origin}/api/H:${this.config.s}/P:ROOT`;
            }
        },
        async created() {

            if (!SERVER_MODE) {
                try {
                    this.uid = (await this.request('check_ssid')).result; // todo убрать result
                }
                catch (e) {
                    const security = JSON.parse(localStorage.getItem('security'));
                    if (!security || security.login === 'GUEST') return;
                    if (security.login && security.ticket) {
                        const ts = JSON.stringify(security.ticket);
                        try {
                            const response = await this.request('authorize', {}, ts)
                            security.ssid = response.result;
                            // return this.loginIn();
                        }
                        catch (e) {
                            // ODANT.push(e);
                            localStorage.removeItem('security');
                        }
                        // this.request('check_ssid').then(res => {
                        //     this.uid = res?.result;
                        //     this.$user = new odaUser({
                        //         $owner: this,
                        //         id: this.uid || 'GUEST',
                        //         root: userRoot || {}
                        //     })
                        // })
                        // this.uid = (await this.request('check_ssid')).result; //todo убрать result
                    }
                }
                // const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'find_account', login: login, type: type });
                const find_account_res = await this.request('find_account');

                // todo:
                // try {
                //     const check_ssid_res = await this.request('check_ssid');
                //     const uid = check_ssid_res?.result;
                //     const userRoot_res = await fetch(`${this.$authHost.url}/U:${uid}`);
                //     const userRoot = await userRoot_res.json();
                //     this.$user = new odaUser({
                //         $owner: this,
                //         id: this.uid,
                //         root: userRoot
                //     });
                // } catch (err) {
                //     console.error(err);
                //     this.$user = new odaUser({
                //         $owner: this,
                //         id: 'GUEST',
                //     });
                // }
            }
            await this.getConfig();
        },
        get $$parts() {
            return this._getItems('P');
        },
    })
    odaStructureItem({
        is: 'odaClass',
        extends: 'odaBodyOwner, odaFieldsOwner, odaIndexesOwner',
        props: {
            rootTag: '$CLASS',
            _fieldRootKeys: ['METADATA', 'STATIC'/* , 'DETAILS' */],
            $class() {
                return this;
            },
            icon() {
                if (this.config?.ico) {
                    if (this.config.ico.includes(':'))
                        return this.config.ico;
                    return this.url + '/~/' + this.config.ico;
                }
                if (/*(await this.isAbstract) || */this.hasChildren) { // TODO isAbstract лезет в рут который получается асинхронно, иконка в таком случае async (продумать)
                    return 'odant:abstract-class';
                }
                return 'odant:class';
            },
            prefix: 'C',
            typeName: 'class',
            defaultViewName: 'grid-view',
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         if (this.isAbstract) {
            //             if (this.access.allowAdmin)
            //                 return 'configurator';
            //             return 'qr-code';
            //         }
            //         return this.getAttr('default-class-view', 'qr-code');//'grid');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
            defaultObjectView: {
                //category: 'Views',
                get() {
                    return this.getAttr('default-object-view', 'editor');
                },
                set(v) {
                    this.setAttr('default-object-view', v);
                }
            },
            objectCount: {
                type: Number,
                get() {
                    return this.cache('objectCount', async () => {
                        return await this.getObjectCount();
                    });
                }
            },
            zIndex: 10,
        },
        get ACTIONS() {
            return this.getRootElement('ACTIONS');
        },
        get _$$config() {
            return (async () => {
                const actions = await this.$$actions;
                return [
                    ...await this._$$fields,
                    {
                        _$$config: actions,
                        label: 'ACTIONS',
                        name: 'ACTIONS',
                        typeName: 'category',
                        icon: 'actions',
                        ItemCount: actions.length,
                    }
                ];
            })()
        },
        get $$classes() {
            return this._getItems('C');
        },
        get $$packets() {
            return (async () => {
                const json = await this.request('get_pack_list');
                return json.$FOLDER[0].$FILE ? json.$FOLDER[0].$FILE.reduce((res, v) => {
                    const names = v.Name.split('#');
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.Count;
                    if (names.length)
                        p.items.push({ count: v.Count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, ROOT: i });
                }) : [];
            })();
        },
        get hasChildren() {
            return this.config && this.config.ch === 'T';
        },
        save(body) {
            body = body || this.body || '{"$CLASS":[' + JSON.stringify(this.ROOT) + ']}';
            return super.save(body);
        },
        getRootElement(name) {
            return this.ROOT?.[`$${name}`]?.[0];
        },
        async getObjectCount(pack) {
            const json = await this.request('get_class_objects_count', { format: 'json', mask: pack || '*' });
            return json && parseInt(json.result) || 0;
        },
        async createObject(src) {
            if (src && typeof src === 'object')
                src = JSON.stringify(src);
            const json = await this.request('create_object', { recalc: true, meta: true }, src);
            // TODO load extended object script
            const root = json?.$OBJECT?.[0];
            const obj = new odaObject({ id: root?.oid, body: json, ROOT: root, $owner: this });
            obj.currentState = 'new object';
            this.fire('create', obj);
            return obj;

        },
        getObject(id, params = { meta: true, recalc: false }) {
            return this.cache(id, async () => {
                let cls = this;
                let url = `${this.url}/O:${id}`;
                if (params?.length) {
                    Object.keys(params).reduce((url, p, i) => {
                        url += i === 0 ? '?' : '&';
                        url += `${p}=${params[p]}`;
                        return url;
                    }, url);
                }
                const body = await (await fetch(url)).json();
                const ROOT = (body.$OBJECT || body.$STATIC || body.$DETAILS)[0];
                if (this.id !== ROOT.cid) {
                    try {
                        cls = (await this.findItem(ROOT.cid)) || this;
                    }
                    catch (e) {
                        cls = this;
                    }
                }
                // TODO load extended object script
                return new odaObject({ id, body, ROOT, $owner: cls });
            })
        },
        async createClass({ name, label, root = {} }) {
            if (name)root.Name = name;
            if (label)root.Label = label;
            const body = { $CLASS: [root] };
            const res = await this.request('create_class', undefined, body);
            // const cfg = await this.$base.getConfig();
            // this.$base.$set('config', cfg);
            // this.reset();
            const cid = res.result.slice(res.result.lastIndexOf(':') + 1);
            const cls = await this.findItem(`C:${cid}`);
            this.fire('create', cls);
            return cls;

        },
        getInheritRoot() {
            return {
                ClassId: this.id,
                Name: this.name,
                Label: this.label,
                Parent: this.id
            };
        }
    })
    odaServerItem({
        is: 'odaPack',
        props: {
            icon: 'av:library-books',
            ROOT() {
                return this.cache('ROOT', async () => {
                    return await this.body;
                })
            },
            mask() {
                const prefix = this.$owner.mask;
                return (prefix ? prefix + '/' : '') + this.name;
            }
        },
        get hasPacks() {
            return this.$$packets.length > 0;
        },
        get $$packets() {
            return this.cache('packets', () => {
                return this.ROOT.items.reduce((res, v) => {
                    const names = v.names;
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.count;
                    if (names.length)
                        p.items.push({ count: v.count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, ROOT: i })
                });
            });
        }
    })
    odaServerItem({
        is: 'odaGroup',
        props: {
            id() {
                return this.$owner?.id + '_G_' + this.name;
            },
            // access() {
            //     return this.$owner?.access;
            // },
            icon: 'odant:folder',
            prefix: 'G',
            type: 'CATEGORY',
            $owner: String,
            fill() {
                switch (this.type?.toLowerCase()) {
                    case 'module': return 'magenta';
                    case 'workplace': return 'darkblue';
                    case 'group':
                    case 'category': return 'orange';
                    default: return 'black'
                }
            },
            isRef: false,
            zIndex: 0,
        },
        async getItem(path) {
            return await this.$owner?.getItem(path);
        },
        async findItem(path, params) {
            return await this.$owner?.findItem(path, params);
        }
    })
    odaGroup({
        is: 'odaModuleGroup',
        props: {
            typeName: "modules",
            prefix: 'M',
            icon: 'folders/folder-magenta-open',
            zIndex: 30
        }
    })
    odaClass({
        is: 'odaDomain',
        props: {
            icon() {
                return this.isSystem ? 'icons:lock' : 'odant:domain';
            },
            prefix: 'D',
            typeName: 'domain',
            isSystem() {
                return this.id === '000000000000000';
            },
            _color: 'var(--domain-color)',
            defaultViewName: 'desktop-ext',
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         return this.getAttr('default-class-view', 'desktop');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
            zIndex: 60,
        },
        async createDomain({ name, label, prefix, root = {} }) {
            if (name) root.Name = name;
            if (label) root.Label = label;
            const body = {$CLASS: [root]};
            let type;
            prefix = (prefix || '').toUpperCase();
            switch (prefix) {
                case 'B': {
                    type = 'BASE';
                } break;
                case 'M': {
                    type = 'MODULE';
                } break;
                case 'P': {
                    type = 'PART';
                } break;
                case 'W': {
                    type = 'WORKPLACE';
                } break;
            }

            const res = await this.request('create_domain', { type: type, prefix: true }, body);
            // this.reset();
            // const cfg = await this.getConfig();
            // this.$set('config', cfg);
            const domain = await this.findItem(`${prefix}:${res.i}`);
            this.fire('update');
            this.fire('create', domain);
            return domain;

        },
        getInheritRoot() {
            return {
                ClassId: this.id,
                Name: this.name,
                Label: this.label,
                Type: this.type
            };
        }
    })
    odaDomain({
        is: 'odaBase',
        props: {
            _fieldRootKeys: ['DETAILS'],
            icon() {
                if (this.type === 'USER') {
                    //TODO: получить объект пользователя, получить от него аватар
                }
                if (this.config?.ico) {
                    if (this.config.ico.includes(':'))
                        return this.config.ico;
                    return this.url + '/~/' + this.config.ico;
                }
                if (this.config?.$B)
                    return 'odant:sub-base';
                return 'odant:base';
            },
            prefix: 'B',
            $base() {
                return this;
            },
            typeName: 'base',
            _color: 'var(--base-color)',
            $module() {
                return undefined;
            },
            zIndex: 70,
        },
        get $$modules() {
            return this._getItems('M');
        },
        get $$bases() {
            return this._getItems('B');
        },
        get $$workplaces() {
            return this._getItems('W');
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const modules = await this.$$modules;
                let actions = await Promise.all(modules.map((module) => {
                    return module.getActionsExp($item);
                }));
                actions = actions?.flat();
                // console.dir(actions);
                return actions;
            });
        },
        get baseCount() {
            return this.cache('baseCount', () => {
                if (this.config) {
                    return this.config.$B?.length || 0;
                }
            })
        },
        async inheritDomain(inheritItem) {
            const item = await this.createDomain({ name: inheritItem.name, prefix: inheritItem.prefix, root: inheritItem.getInheritRoot() });
            const classes = await inheritItem.$$classes;
            for (const child of classes) await this.inheritClass(item, child);

            this.fire('update');
            return item;
        },
        async inheritClass(inheritItem) {
            const item = await this.createClass({ name: inheritItem.name, root: inheritItem.getInheritRoot() });
            const classes = await inheritItem.$$classes;
            for (const child of classes) await this.inheritClass(item, child);
            item.save();
            return item;
        }
    })
    odaBase({
        is: 'odaPart',
        props: {
            $part() {
                return this;
            },
            icon() {
                switch (this.id) {
                    case 'ROOT':
                        return 'odant:root';
                    case 'WORK':
                        return 'odant:structure';
                }
                return this.id.toLowerCase();
            },
            prefix: 'P',
            typeName: 'part'
        }
    })
    odaDomain({
        is: 'odaModule',
        props: {
            icon: 'odant:module',
            prefix: 'M',
            typeName: 'module',
            _color: 'var(--module-color)',
            defaultViewName: 'module-info',
            $module() {
                return this;
            },
            zIndex: 30,
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const classes = await this.$$classes;
                let actions = await Promise.all(classes.map((cls) => {
                    // return cls.getActions($item);
                    return cls.getActionsExp($item);
                }))
                actions = actions?.flat();
                console.dir(actions);
                return actions;
            });
        }
    })
    odaDomain({
        is: 'odaWorkPlace',
        props: {
            icon: 'odant:workplace',
            prefix: 'W',
            typeName: 'work place',
            _color: 'var(--workplace-color)',
            zIndex: 50,
        },
        async createLink(cls) {
            const classId = cls.id;
            const res = await this.request('create_link', { classId });
            //this.reset();
        }
    })
    odaServerItem({
        is: 'odaFile',
        extends: 'odaFilesNFoldersOwner',
        props: {
            id() {
                return this.name;
            },
            icon() {
                switch (this.ext) {
                    case 'jpg':
                    case 'png':
                        return this.url + '?mode=preview64';
                }
                return 'files:' + this.ext;
            },
            ext() {
                return this.id.split('.').pop();
            },
            typeName: 'file',
            zIndex: 102,
            actionsPath() {
                return `client/*/*`;
            },
            defaultViewName() {
                switch (this.ext) {
                    case 'jpg':
                    case 'png':
                        return 'image-preview';
                    case 'pdf':
                        return '';
                }
                return 'code-editor';
            },
        },
        get time() {
            return this.config?.time;
        },
        get type() {
            return this.typeName;
        },
        get size() {
            return this.config?.size;
        },
        get sizeText() {
            if (this.size && this.size > 0) {
                let v = this.size;
                let pcs = ' b';
                if (v > 999 && v < 1000000) {
                    v = Math.round(v / 10) / 100;
                    pcs = ' Kb'
                }
                else if (v < 1000000000) {
                    v = Math.round(v / 10000) / 100;
                    pcs = ' Mb'
                }
                else if (v < 1000000000000) {
                    v = Math.round(v / 10000000) / 100;
                    pcs = ' Gb'
                }
                return v.toLocaleString() + pcs;
            }
        },
    })
    odaFile({
        is: 'odaFolder',
        extends: 'odaFilesNFoldersOwner',
        props: {
            icon: 'odant:folder',
            typeName: 'folder',
            label() {
                return this.name === '~' ? this.$owner.label + ' [~]' : this.name;
            },
            fill() {
                if (this.color === 'var(--item-color)') return 'var(--folder-color)';
                return this.color;
            },
            zIndex: 101,
            defaultViewName: 'file-explorer',
        }
    })
    odaServerItem({
        is: 'odaIndex',
        extends: 'odaFieldsOwner, odaIndexesOwner, odaFilesNFoldersOwner',
        props: {
            icon: {
                default: 'odant:index',
                get() {
                    switch (this.type) {
                        case 'Union': return 'odant:index';
                        case 'Cascade': return 'odant:index';
                        default: return 'odant:index';
                    }
                }
            },
            prefix: 'I',
            typeName: 'index',
            type: String,
            ROOT: Object,
            METADATA() {
                return this.ROOT?.$METADATA?.[0] || this.$class.METADATA;
            }
        },
        getRows(mask, from, length, post) {
            if (typeof post === 'object')
                post = JSON.stringify(post);
            return this.request('dataset', { loadmask: mask, mask: mask, from: from, length: length }, post);
        },
        getDataSet(mask = '*', filter, clear) {
            if (clear) this.reset();
            return this.$('DataSet-' + mask + filter, () => {
                return new IndexDataSet(this, mask, filter);
            }, 100000) //todo Предложение по сбросу кэша
        }
    })
    odaServerItem({
        is: 'odaObject',
        extends: 'odaBodyOwner, odaFilesNFoldersOwner, odaIndexesOwner',
        props: {
            rootTag: '$OBJECT',
            icon: 'odant:object',
            prefix: 'O',
            typeName: 'object',
            $fields() {
                this.$class.$fields;
            },
            defaultViewName() {
                return this.$class.defaultObjectView;
            },
        },
        onbody() {
            //this.ACTIONS = this.getRootElement('ACTIONS');
        },
        async getActions() {
            return this.$class.getActions(this);
        }
    })
    odaBase({
        is: 'odaUser',
        props: {
            prefix: 'U',
            icon() {
                return this.id === 'GUEST'
                    ? 'icons:perm-identity'
                    // : this.$host.$authHost.request('get_user_icon', {id: this.id})
                    : 'icons:perm-identity'
            }
        }
    })
    odaServerItem({
        is: 'odaAction',
        props: {
            active() {
                return !this.export;
            },
            icon: 'odant:action',
            typeName: 'action',
            $source: Object,
            viewName() {
                return `odant-${this.name.toLowerCase()}`;
            },
            ROOT() {
                const metaRoot = this.$class?.$module
                    ? this.$class?.ROOT?.$MODULES?.[0].$MODULE?.find(e => e.id === this.$class?.$module?.id)
                    : this.$class?.ROOT?.$ACTIONS?.[0];
                return metaRoot?.$ACTION?.find(e => e.id === this.id);
            }
        },
        execute() {
            throw new Error(`Action "${this.label}" not implemented!`)
        },
        getActions() {
            return [];
        }
    })
    odaAction({
        is: 'odaView',
        props: {
            typeName: 'view',
            url() {
                return this.path;
            }
        },
        async execute(parent) {
            while (parent && !parent.showView) {
                parent = parent.parent || parent.domHost || parent.parentElement;
            }
            if (parent) {
                parent.showView(this);
            }
            else {
                window.open(this.$owner.url + '/~/client/pages/form/index2.html?view=' + this.id);
            }
        },
        get _control() {
            return ODANT.createComponent(this.viewName);
        },
        async createComponent(component) {
            await import(`${this.url}/${this.name}.js`);
            const control = await ODA.createComponent(component || this.viewName);
            control.contextItem = this.$owner;
            control.sourceItem = this.$source;
            return control;
        }
    })
    odaAction({
        is: 'odaActionError',
        props: {
            icon: 'icons:error'
        },
        execute() {
            window.open(this.url + '/~/client/app-studio/index2.html');
        }
    })
    odaAction({
        is: 'odaPage',
        props: {
            typeName: 'page'
        },
        execute() {
            window.open(this.$owner.url + '/~/client/pages/' + this.name + '/index2.html');
            // console.warn('page not defined!!!');
        }
    })
    odaAction({
        is: 'odaMethod',
        props: {
            typeName: 'method'
        },
        execute() {
            console.warn('method not defined!!!');
        }
    })
    odaMethod({
        is: 'odaButton',
        props: {
            typeName: 'button'
        },
        execute() {
            console.warn('button not defined!!!');
        }
    })
    odaMethod({
        is: 'odaService',
        props: {
            typeName: 'service'
        },
        execute() {
            console.warn('service not defined!!!');
        }
    })
    odaVirtualItem({
        is: 'odaField',
        extends: 'odaFieldsOwner',
        props: {
            place: '',
            $folder() {
                return (this.$typeClass || this.$class).$folder
            },
            typeName: 'field',
            icon: 'odant:field',
            ROOT: Object,
            $typeClass: Object,
            actionsPath() {
                return 'fields/*/*';
            },
            isTable: false,
            hasFields() {
                return ((this.$typeClass?.isAbstract && this.$typeClass.hasFields) || (this.ATTRS?.length > 0));
            },
            isLink() {
                return (this.$typeClass && !this.$typeClass.isAbstract);
            },
            _input() {
                if (this.isTable) return '';
                //if (this._isStructure) return '';
                if (this.isLink) return 'link';
                if (this.$typeClass?.hasChildren) return 'link';
                return 'default';
            },
        },
        get input() {
            return this.cache('input', async () => {
                if (this._input)
                    await import(`${(this.$typeClass || this).url}/~/fields/inputs/${this._input}/${this._input}.js`);

                return this._input ? `oda-${this._input}` : '';
            })
        }
    })
    odaField({
        is: 'odaFieldGroup',
        props: {
            typeName: 'category',
            icon: 'fields',
        }
    })
    CORE.waitIcon = 'odant:spin';
    CORE.host = new odaHost({ origin: CORE.URL.origin });
}
export default globalThis.CORE;