import ODA from '../../../../../oda/oda.js';
import '../user-tree/user-tree.js';
ODA({
    is: 'odant-user-manager',
    template: `
    <style>
        :host{
            @apply --horizontal;
            @apply --flex;
        }
        .panel{
            @apply --vertical;
            @apply --flex;
            width: calc(50% - 48px/2);/* 48px ширина промежутка */
        }
        .btns{
            @apply --vertical;
            align-items: center;
            justify-content: center;
            padding: 8px;
        }
        .btns > oda-button{
            @apply --raised;
            margin-bottom: 8px;
            border-radius: 50%;
        }
        [disabled]{
            @apply --disabled;
        }
        .panel > oda-item-node{
            border-bottom: 1px solid gray;
            padding: 4px;
            margin-bottom: 8px;
        }
    </style>
    <div class="panel">
        <odant-user-tree ref="assignedTree" :context-item="contextItem" assigned></odant-user-tree>
    </div>
    <div class="btns header">
        <oda-button class="btn content" icon="icons:arrow-back" @tap="assignUsers" :disabled="_block"></oda-button>
        <oda-button class="btn content" icon="icons:arrow-forward" @tap="unassignUsers" :disabled="_block"></oda-button>
    </div>
    <div class="panel">
        <odant-item-node :context-item="rootContext" ::current-item="currentLevelItem" :disabled="isLoading" allow-expand></odant-item-node>
        <odant-user-tree ref="availableTree" :context-item="currentLevelItem" ::disabled="isLoading" lazy></odant-user-tree>
    </div>`,
    props: {
        contextItem: {
            type: Object,
            set(n) {
                this._block = !n;
                // if (n) this._block = false;
                // else this._block = true;
            }
        },
        rootContext: Object,
        _block: true,
        currentLevelItem: Object,
        isLoading: Boolean
    },
    async assignUsers() {
        this._block = true;
        try {
            const users = await this.$refs.availableTree.getChecked();
            const ids = this._getUsersIDs(users);
            await this.contextItem.assignUsers(ids);
            this.contextItem.reset();
            this.contextItem.fireChanged()
            return true;
        } catch (e) { }
        finally {
            this._block = false;
        }
    },
    async unassignUsers() {
        this._block = true;
        try {
            const users = await this.$refs.assignedTree.getChecked();
            const ids = this._getUsersIDs(users);
            await this.contextItem.unassignUsers(ids);
            this.contextItem.reset();
            this.contextItem.fireChanged();
            return true;
        } catch (e) { }
        finally {
            this._block = false;
        }
    },
    _getUsersIDs(users) {
        const types = [
            odaItem.Types.odaObject,
            odaItem.Types.odaUser,
            odaItem.Types.odaRole,
        ];

        return users.reduce((acc, n) => {
            if (types.includes(n.$item.itemType)) {
                acc.push(n.$item.id);
            }
            return acc;
        }, []);
    }
})