import ODA from '../../../../../oda/oda.js';
import '../../../../item/lib/item-tree/item-tree.js';
import '../../../../../lib/inputs/string/string.js';
import '../../../../item/lib/item-node/item-node.js';
ODA({
    is: 'odant-user-tree',
    extends: 'this, odant-item-tree',
    template: `
    <style>
        :host{
            @apply --vertical;
            @apply --flex;
            height: 100%;
        }
        :host > oda-string-input {
            border: 1px solid var(--border-color);
            border-radius: 3px;
            margin: 5px 0 5px;
        }
        [disabled]{
            @apply --disabled;
        }
    </style>
    <oda-string-input ~if="allowSearch" :args="inputArgs" ::value="search"></oda-string-input>`,
    props: {
        disabled: {
            type: Boolean,
            notify: true,
            reflectToAttribute: true,
            set(n){
                if(n) ODA.loader.on();
                else ODA.loader.off();
            }
        },
        contextItem: {
            type: Object,
            async set(n) {
                if (n) {
                    this.disabled = true;
                    const p_allUsers = await ODANT.API.post(n.$host, { method: 'get_users_xml' });
                    const p_allAssignedUsers = await ODANT.API.post(n, { method: 'get_users_xml', type: 1 });
                    this.allUsers = p_allUsers && p_allUsers.$USERS && p_allUsers.$USERS[0].$U;
                    this.allAssignedUsers = p_allAssignedUsers && p_allAssignedUsers.$USERS && p_allAssignedUsers.$USERS[0].$U;
                    if (n instanceof odaDomain) this._contextItem = n;
                    else if (n.$domain) this._contextItem = n.$domain;
                    this.disabled = false
                }
            }
        },
        hideRoot: true,
        _contextItem: {
            type: Object,
            async set(n, o) {
                if (o) {
                    this.unlisten(o, 'changed', '_handler');
                    this.unlisten(o, 'update', '_handler');
                }
                if (n) {
                    this.listen(n, 'changed', '_handler');
                    this.listen(n, 'update', '_handler');
                    await this._setRootItem();
                }
            }

        },
        itemSourceNames: {
            type: String,
            get() {
                return this.assigned ? '$assignedUsers' : '$classes, $objects'
            }
        },
        allowSearch: false,
        allowCheck: true,
        search: '',
        inputArgs: {
            type: Object,
            default: {
                format: '',
                mask: '',
                placeholder: 'Search...',
                title: '',
                separator: '',
            },
        },
        columns: {
            type: Array,
            default() {
                return [
                    {
                        treeMode: true,
                        name: 'label',
                    },
                ];
            },
        },
        assigned: false,
        allUsers: [],
        allAssignedUsers: [],
        defaultTemplate: 'odant-user-tree-node'
    },
    _handler(){
        this.rootItem = null;
        this._setRootItem();
    },
    async _setRootItem() {
        if (this.assigned) {
            if (this._contextItem) {
                this.rootItem = this._contextItem;
            } else {
                this.rootItem = null;
                this.dataSet = [];
            }
        } else {
            const $security = await this._contextItem.$security;
            if ($security) {
                this.rootItem = $security;
            } else {
                this.rootItem = null;
                this.dataSet = [];
            }
        }
    },
    async _beforeExpand(node, deep = 1) {
        let res = this.itemSourceNames.split(',').map(async src => {
            src = src.trim();
            return node.$item && (await node.$item[src] || []);
        });
        res = (await Promise.all(res)).flat();
        res = res.map(async i => {
            const obj = { $item: i, label: this._getUserLabel(i), items: [] };
            if (deep) {
                const ch = await this._checkChildren(obj);
                if (ch) {
                    obj.$expanded = true;
                    if (deep + 1 < 4) {
                        obj.items = await this._beforeExpand(obj, deep + 1)
                    } else {
                        obj.items = [{}]
                    }
                }
            }
            return obj;
        });
        res = (await Promise.all(res)).sort((a, b) => (a.$item.name || a.$item.label || '').localeCompare(b.$item.name || b.$item.label || ''));
        return res;
    },
    _getUserLabel(item) {
        const user = this.allUsers.find(u => u.i === item.id);
        if (user) return user.l || user.n || user.e || user.i;
        return item.label;
    }
})

ODA({
    is: 'odant-user-tree-node',
    extends: 'odant-item-node',
    async _itemChanged(e) {
        this.expanded = false;
        if (this.contextItem) {
            await this.$super('odant-item-node', '_itemChanged');
            this.label = this.item && this.item.label || this.label;
        }
    }
})