(class extends odaAction {
    static get properties() {
        return {
            icon: 'icons:add-box',
            allowAccess: 'A',
            Hidden: true,
        }
    }

    async execute(params = {}) {
        let { area, type, name, label } = params;
        let path = this.$item.path.replace(/\~(\/web\/object)?/, '');
        area = area || path.includes('/server') ? 'server' : 'client';
        if (area === 'server' && /~\/server\/.*/.test(path)) {
            switch (path) {
                case '/server/methods': type = 'method'; break;
                case '/server/triggers': type = 'trigger'; break;
                case '/server/tasks': type = 'task'; break;
                case '/server/tasks/on-start': type = 'task'; name = 'on-start'; label = 'on-start'; break;
                case '/server/tasks/on-timer': type = 'task'; name = 'on-timer'; label = 'on-timer'; break;
                case '/server/triggers/after-create': type = 'task'; name = 'on-start'; label = 'after-create'; break;
                case '/server/triggers/after-delete': type = 'task'; name = 'after-delete'; label = 'after-delete'; break;
                case '/server/triggers/after-update': type = 'task'; name = 'after-update'; label = 'after-update'; break;
                case '/server/triggers/before-create': type = 'task'; name = 'before-create'; label = 'before-create'; break;
                case '/server/triggers/before-delete': type = 'task'; name = 'before-delete'; label = 'before-delete'; break;
                case '/server/triggers/before-update': type = 'task'; name = 'before-update'; label = 'before-update'; break;
                default: break;
            }
        }
        if (!type) {
            type = await this._showSelectTypeDialog(area);
        }
        if (!name) {
            ({ name, label } = await this._showSetActionNameDialog(area, type));
        }
        if (!area || !type || !name) {
            throw new Error('It is impossible to create an action, not enough data.');
        }
        this._createAction(area, type, name, label);
    }
    _getPath(area, type) {
        if (area === 'client') {
            return `/${area}`;
        }
        return `/${area}/${type}s`;
    }
    async _showSelectTypeDialog(area) {
        if (!area) return;
        const control = await ODA.createComponent(this.resolveUrl('select-action-type.html'));
        control.area = area;
        const { type } = await ODA.showDialog(control, { title: 'Select a type for action' });
        return type;
    }
    async _showSetActionNameDialog(area, type) {
        if (!area || !type) return {};
        const control = await ODA.createComponent(this.resolveUrl('set-action-name.html'));
        control.area = area;
        control.type = type;
        const { name, label } = await ODA.showDialog(control, { title: 'Set a name for action' });
        return { name, label };
    }
    async _createAction(area, type, name, label) {
        const path = this._getPath(area, type);

        const actionRoot = path === this.$item.path ? this.$item : await this.$item.$owner.getFolder(path, true);
        const actionFolder = await actionRoot.createFolder(name);
        const protoClass = this._getProtoClass(type);
        const jsBody = this._getJSBody(protoClass, label);
        await actionFolder.saveFile(jsBody, name + '.js', true);
        if (area === 'client' && ['view', 'page'].includes(type)) {
            const htmlBody = this._getHTMLBody(type, name);
            await actionFolder.saveFile(htmlBody, name + '.html', true);
        }
    }
    _getProtoClass(type) {
        switch (type) {
            case 'button': return 'odaButton';
            case 'service': return 'odaService';
            case 'page': return 'odaPage';
            case 'view': return 'odaView';
            case 'method':
            default: return 'odaAction';
        }
    }
    _getJSBody(protoClass, label) {
        return `(class extends ${protoClass} {
    static get properties() {
        return {
            ${label ? `label: '${label}',` : ''}
            allowAccess: 'A',
            allowUse: true,
            allowExport: false,
            allowArrayContext: false,
            icon: 'icons:settings-applications',
        };
    }
}).register();`
    }
    _getHTMLBody(type, name) {
        return `<script src="/web/core/oda.js"></script>
<oda-module id="oda-${name}">
    <template>
        <style>
            
        </style>
        
    </template>
    <script>
        ODA({
            is: 'oda-${name}',
            properties: {
                
            },
            ${type === 'page' ? `show() { document.body.appendChild(this.host);}` : ''}
        })
    </script>
</oda-module>`
    }
}).register();