ODANT({
    is: 'odant-file-drag-and-drop',
    props: {
        enableDragAndDrop: true,
        __dropToNode: null,
        __dropModeBlock: false,
        __types: ['Files', 'odant/files', 'odant/folders', 'odant/items'],
        item: {
            type: Object,
            get() {
                if (this.contextItem) {
                    return {
                        $item: this.contextItem.$folder,
                    };
                };
            },
        },
    },
    listeners: {
        dragover: '_onDragOver',
        drop: '_onDrop',
        dragenter: '_onDragEnter',
        dragleave: '_onDragleave',
        dragend: '_onDragEnd',
    },
    _onDragStart: function (e) {
        if (!this.enableDragAndDrop) return;

        const dt = e.dataTransfer;
        const node = e.target.item;
        const item = node.$item;
        const isSelection = this.selection.length > 0 && this.selection.find(i => i.$item === item) !== undefined;
        const items = isSelection ? this.selection.map(({ $item }) => $item) : [item];
        const types = odaItem.Types;
        const typesByCode = {
            'none': 'Files',
            [types.odaFile]: 'odant/files',
            [types.odaFolder]: 'odant/folders',
            [types.odaItem]: 'odant/items',
        };
        const itemsByType = {};

        if (!isSelection && this.allowSelection) {
            this.selection = [node];
        }

        for (const t of this.__types) {
            itemsByType[t] = [];
        }

        for (let i of items) {
            const type = typesByCode[i.itemType];
            if (type !== undefined) itemsByType[type].push(i.toJSON());
        }

        for (let type in itemsByType) {
            if (!itemsByType[type].length) continue;
            const json = JSON.stringify(itemsByType[type]);

            dt.setData(type, json);
        }

        dt.setDragImage(null, 0, 0);
    },
    _onDragOver(e) {
        e.preventDefault();
        e.stopPropagation();

        const dt = e.dataTransfer;
        const isContainItems = this.__types.some((t) => dt.types.includes(t));

        const setDropEffect = () => {
            dt.dropEffect = e.ctrlKey ? 'copy' : 'move';
        };

        if (isContainItems) {
            let elWithItem = e.path.find(i => i.item);

            const isFolder = elWithItem.item.$item.itemType === odaItem.Types.odaFolder;

            const allowDrop =
                !this.selection.includes(elWithItem.item) &&
                isFolder &&
                !elWithItem.item.$item.readOnly;

            if (!allowDrop) {
                if (!isFolder) {
                    this.__dropToNode = this.item;
                    setDropEffect();
                    return;
                }

                dt.dropEffect = 'none';
                return;
            }

            if (elWithItem) {
                this.__dropToNode = elWithItem.item;

                const elRow = e.path.find(i => i.row);

                if (elRow) {
                    if (!elRow.row.drop === false) {
                        this.set(elRow.row, '$dropMode', '');
                        dt.dropEffect = 'none';
                        return;
                    }
                    const step = (this.treeStep * elRow.row.$level) - this.treeStep;
                    this.updateStyles({ '--margin-left-drag': (step > 0 ? step : 0) + 'px' });
                    if (this.__dropToNode.$item.itemType === odaItem.Types.odaFile) {
                        this.set(elRow.row, '$dropMode', 'bottom');
                    } else {
                        this.set(elRow.row, '$dropMode', 'in');
                    }
                }

                setDropEffect();
                return;
            }
        }

        dt.dropEffect = 'none';
    },
    _onDragEnter(e) {
        e.preventDefault();
    },
    _onDragleave(e) {
        e.preventDefault();

        const rt = e.relatedTarget;
        const dn = this.__dropToNode;
        const isCurrent =
            rt && (
                rt.item && rt.item === dn ||
                dn && rt.contextItem && rt.contextItem.fullId === dn.$item.fullId
            );

        if (!isCurrent) {
            this.__dropToNode = null;
        }

        this._resetRowDropMode(e);
    },
    async _onDrop(e) {
        e.preventDefault();
        e.stopPropagation();

        const dt = e.dataTransfer;
        const dropNode = this.__dropToNode;
        const dropItem = this.__dropToNode.$item;

        this._resetRowDropMode(e);
        this.__dropModeBlock = true;

        const isCurrent = dropItem.fullId === this.contextItem.$folder.fullId;
        const isFolder = dropItem.itemType === odaItem.Types.odaFolder;
        const allowDrop =
            isFolder &&
            !dropItem.readOnly &&
            (e.ctrlKey || !isCurrent || dt.files);

        if (!allowDrop) return;

        let newFiles = [];
        const odaTypes = this.__types.filter(i => i.startsWith('odant'));

        for (const type of odaTypes) {
            const data = dt.getData(type);
            try {
                const ls = data ? JSON.parse(data) : [];
                newFiles.push(...ls);
            } catch (err) { }
        }

        newFiles.push(...Array.from(dt.files));

        const lastFiles = (await dropItem.getFilesAndFolders()).flat();

        const newFileNames = newFiles.map(i => i.label || i.name);
        const lastFileNames = lastFiles.map(i => i.label);
        const illegalNames = [].concat(newFileNames, lastFileNames);

        const eventAction = e.ctrlKey ? 'copy' : 'move';
        let action = null;

        const doAction = async (action, file, lastFile) => {
            const isOdaItem = !!file.fullId;

            if (action === 'save') {
                const fileName = file.label || file.name;
                const newName = this._getDefaultNewName(fileName, illegalNames);

                if (isOdaItem) {
                    // file.rename(newName)
                    this._renameFile(file, newName);
                    this._doAction(eventAction, file);
                } else {
                    dropItem.saveFile(file, newName, undefined, file.type);
                }

                illegalNames.push(newName);
            } else {
                if (isOdaItem) {
                    this._doAction(action, file);
                } else {
                    dropItem.saveFile(file, file.name, undefined, file.type);
                }

                illegalNames.push(file.label || file.name);
            }
        };

        const isMultipleFiles = newFiles.length > 1;

        const openDialog = async (newFile, lastFile) => {
            const form = await ODA.createComponent('./file-conflict-dialog.js', {
                lastFile,
                dropItem,
                illegalNames,
                isMultipleFiles,
            });

            try {
                const dialogResult = await ODA.showDialog(form);
                const { control, button } = dialogResult;

                if (control.action === 'rename') {
                    const fileName = newFile.label || newFile.name;

                    const formNewName = await ODA.createComponent('./file-new-name-dialog.js', {
                        fileName,
                        illegalNames,
                    });

                    try {
                        const dialogResult = await ODA.showDialog(formNewName);
                        const newName = dialogResult.newName;

                        this._renameFile(newFile, newName);
                        await doAction(eventAction, newFile, lastFile);
                    } catch (err) {
                        await openDialog(newFile, lastFile);
                    }
                    return;
                }

                await doAction(control.action, newFile, lastFile);
                const forAllNext = button === control.buttons.forAllNext;
                if (forAllNext) action = control.action;
            } catch (err) { }
        };

        for (let newFile of newFiles) {
            const label = (newFile.label || newFile.name).toLowerCase();
            const lastFile = lastFiles.find(f => f.label.toLowerCase() === label);

            if (!lastFile) {
                await doAction(eventAction, newFile, lastFile);
                continue;
            }

            if (action) {
                await doAction(action, newFile, lastFile);
                continue;
            }

            await openDialog(newFile, lastFile);
        }

        this.__dropModeBlock = false;
        this.__dropToNode = null;

        if (this._beforeExpand) {
            dropNode.items = await this._beforeExpand(dropNode);
            this._refresh();
        }
    },
    _getDefaultNewName(fileName, illegalNames) {
        let label = fileName;
        const formatMatch = label.match(/[.][a-z0-9]+$/i);
        const format = formatMatch && formatMatch[0] || '';
        label = label.substring(0, label.length - format.length);
        const match = label.match(/_-?\d+$/);
        const isNumbered = match !== null;
        let n = 1;
        if (isNumbered) {
            label = label.substring(0, label.length - match[0].length);
            n = +match[0].match(/-?\d+/)[0] + 1;
        }
        let newName;
        const setName = () => {
            newName = `${label}_${n}${format}`;
        };
        setName();

        while (illegalNames.reduce((res, n) => res || n.toLowerCase() === newName.toLowerCase(), false)) {
            n++;
            setName();
        }

        return newName;
    },
    _resetRowDropMode(e) {
        const elRow = e.path.find(p => p.row);
        if (elRow) {
            this.set(elRow.row, '$dropMode', '');
        }
    },
    _onDragEnd(e) {
        e.preventDefault();
        if (!this.__dropModeBlock) this.__dropToNode = null;
    },
    _doAction(action, file) {
        action = action.toLowerCase();

        const dropItem = this.__dropToNode.$item;

        switch (action) {
            case 'replace':
                this._moveFile(file, dropItem);
                // file.move(dropItem);
                break;

            case 'copy':
                this._copyFile(file, dropItem);
                // file.copy(dropItem);
                break;

            case 'move':
                this._moveFile(file, dropItem);
                // file.move(dropItem);
                break;
        }
    },
    // методы, которые нужны файлам
    _renameFile(file, name) {
        file.label = name;
        file.name = name;
    },
    async _moveFile(file, target, needRemove = true) {
        const fileOfOda = await ODANT.findItem(file.fullId);


        const allowMove =
            fileOfOda.$owner.fullId !== target.fullId &&
            fileOfOda.fullId !== target.fullId;

        const allowCopy =
            allowMove || !needRemove;

        if (!allowMove && !allowCopy) return;

        // move folder
        if (fileOfOda instanceof odaFolder) {
            const newFolder = await target.createFolder(file.label);
            const files = await fileOfOda.getFilesAndFolders();
            const moves = [];
            for (const file of files) {
                moves.push(this._moveFile(file, newFolder, needRemove));
            }

            if (needRemove) await Promise.all(moves);
        } else { // move file
            await target.saveFile(fileOfOda, file.label);
        }

        if (needRemove) fileOfOda.remove();
    },
    async _copyFile(file, target) {
        this._moveFile(file, target, false);
    },
});