import Transmitter from './Transmitter.js';
export default class Authenticate{
    constructor() {
        const url = new URL(location.toString());
        this.redirect_url = url.searchParams.get('redirect_url');
        for (const c of ['form', 'loginInput', 'passInput', 'passToggleBtn', 'resetBtn', 'forgotBtn', 'registerBtn', 'loginBtn']) {
            this[c] = document.getElementById(c);
            if (this[c] === null) throw `Отсутствует элемент формы: ${c}`;
        }
        this.transmitter = new Transmitter;

        this.reset = (e) => {
            this.form.reset();
        }
        this.submit = (e) =>  {
            e.preventDefault();
            if (this.form.checkValidity()) {
                console.log('submit');
            }
        }
        this._setPinMode = (val) => {
            val = Boolean(val);
            this.loginInput.disabled = true;
            this.registerBtn.hidden = true;
            this.forgotBtn.hidden = true;
            this.loginBtn.hidden = true;
            this.passInput.value = '';
            if (typeof otherMethodsBlock !== 'undefined') {
                otherMethodsBlock.style.setProperty('opacity', '0');
                otherMethodsBlock.style.setProperty('pointerEvents', 'none');
            }
            if (val) {
                this.passInput.placeholder = 'pin';
                this.passInput.type = 'text';
            } else {
                this.loginInput.disabled = false;
                this.registerBtn.hidden = false;
                this.forgotBtn.hidden = false;
                this.loginBtn.hidden = false;
                this.passInput.placeholder = 'password';
                this.passInput.type = 'password';
                if (typeof otherMethodsBlock !== 'undefined') {
                    otherMethodsBlock.style.removeProperty('opacity');
                    otherMethodsBlock.style.removeProperty('pointerEvents');
                }
            }
            passInput.value = '';
        }
        this.forgot = (e) =>  {
            e.preventDefault();
            this._setPinMode(true);
            const login = this.login;
            this.findAccount(login).then(res => {
                ODANT.API.request(res + '/api', {
                    method: 'one_time_password',
                    login: login,
                    type: 'email',
                    referrer: document.URL,
                }).then(res => {
                    this.passInput.focus();
                    const fn = () => {
                        if (this.password.length > 4) {
                            this.passInput.value = this.passInput.value.slice(0, 4);
                            return;
                        }
                        if (this.password.length === 4) {
                            this.submit();
                            this.passInput.removeEventListener('input', fn);
                        }
                    }
                    this.passInput.addEventListener('input', fn);
                }).catch(err => {
                    this._setPinMode(false);
                });
            });
        }
        this.register = (e) =>  {
            e.preventDefault();
        }
        this.togglePassMode = () => {
            if (this.passInput.type === 'password') {
                this.passInput.type = 'text';
                passToggleBtn.icon = 'icons:visibility-off';
            } else {
                this.passInput.type = 'password';
                this.passToggleBtn.icon = 'icons:visibility';
            }
        }
        this.loginAsGuest = () => {
            this.destroy();
            this.transmitter.send('loginAsGuest');
        }
        this.clearValidate = (e) => {
            this.passInput.setCustomValidity('');
        }
        this.validateLogin = async () => {
            const acc = await this.findAccount(this.login);
            if (!acc) this.loginInput.setCustomValidity('такого пользователя не существует');
            else this.loginInput.setCustomValidity('');
            this.loginInput.reportValidity();
        }
        this.findAccount = async (login = this.login) => {
            try {
                const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'check_login', login, type: 'email' });
                return response.result;
            } catch (e) {
                return false;
            }
        }
    }
    init(params) {
        // try {
        //     const storage = new odantLocalStorage('security');
        //     const login = force ? '' : storage.getValue('login');
        //     const ticket = storage.getValue('ticket');
        //     if (login && ticket) {
        //         const res = await odaConnection._authorize(ticket);
        //         if (res) {
        //             this.transmitter.send('authorize', { login, ticket });
        //             this.destroy();
        //             window.close();
        //         }
        //     }
        // } catch (err) { }
        this._url = params && params.url;
        this._name = params && params.name;
        this.form.addEventListener('submit', this.submit);
        this.loginInput.addEventListener('input', this.clearValidate);
        this.passInput.addEventListener('input', this.clearValidate);
        this.resetBtn.addEventListener('click', this.reset);
        this.forgotBtn.addEventListener('click', this.forgot);
        this.registerBtn.addEventListener('click', this.register);
        this.passToggleBtn.addEventListener('click', this.togglePassMode);
        window.addEventListener('close', this.loginAsGuest);
        // window.addEventListener('beforeunload', this.loginAsGuest);
        this.registerBtn.hidden = false;
        this.forgotBtn.hidden = false;
    }
    destroy() {
        this.form.removeEventListener('submit', this.submit);
        this.loginInput.removeEventListener('input', this.clearValidate);
        this.passInput.removeEventListener('input', this.clearValidate);
        this.resetBtn.removeEventListener('click', this.reset);
        this.forgotBtn.removeEventListener('click', this.forgot);
        this.registerBtn.removeEventListener('click', this.register);
        this.passToggleBtn.removeEventListener('click', this.togglePassMode);
        window.addEventListener('close', this.loginAsGuest);
        // window.removeEventListener('beforeunload', this.loginAsGuest);
    }
    get password() {
        return this.passInput.value;
    }
    get login() {
        return this.loginInput.value;
    }
}