window.name = 'odant-authentication';
window.addEventListener('message', (e) => {
    switch (e.data.type) {
        case 'action':
            switch (e.data.data) {
                case 'close': window.close(); break;
                case 'focus': window.focus(); break;
                default:
                    break;
            }
            break;
        default:
            break;
    }
});
function sendClose() {
    sendTicket('guest');
}
function sendBlur() {
    window.opener.postMessage({ type: 'event', data: 'blur' }, '*');
}
window.addEventListener('beforeunload', sendClose);
window.addEventListener('blur', sendBlur);

const form = document.getElementById('form');
const loginBlock = document.getElementById('loginBlock');
const logInput = document.getElementById('logInput');
const passInput = document.getElementById('passInput');
const loginBtn = document.getElementById('loginBtn');
const buttons = document.getElementById('buttons');

async function load() {
    form.style.transition = 'all .5s ease-in-out';
    setTimeout(() => {
        form.style.opacity = '1';
        setTimeout(() => {
            logInput.select(0, -1);
            _formInput();
            form.style.transition = '';
            form.style.opacity = '';
            loadOtherMethods();
        }, 500);
    }, 100);

    form.addEventListener('submit', (e) => {
        e.preventDefault();
    });
    [logInput, passInput].forEach(i => i.addEventListener('focus',
        (e) => { e.target.select(0, -1) }
    ));
    form.addEventListener('input', _formInput);
    form.addEventListener('reset', _reset);
    loginBtn.addEventListener('click', _login);

    async function _formInput(e) {
        const login = logInput.value;
        const password = passInput.value;
        await _checkLogin({ login, password });
    }
    async function _checkLogin({ login = logInput.value, password = passInput.value }, force) {
        let checked = false;
        try {
            checked = force || await checkLogin(login);
            setVisibility(loginBtn, checked && password);
        } catch (e) {
            setVisibility(loginBtn, false);
        }
        return checked;
    }
    function setVisibility(elem, cond) {
        cond = Boolean(cond);
        if (!cond) {
            elem.setAttribute('hidden', '');
        } else {
            elem.removeAttribute('hidden');
        }
    }
    function _reset() {
        logInput.value = '';
        passInput.value = '';
        _formInput(e);
    }
    async function _login() {
        const login = logInput.value;
        const password = passInput.value;
        const loginChecked = await checkLogin(login);
        try {
            if (loginChecked) {
                await authentication(login, password);
            }
        } catch (e) {
            console.log(e);
        }
    }
    async function checkLogin(login) {
        try {
            await ODANT.API.request(ODANT.origin + '/api', { method: 'check_login', login: login });
            return true;
        }
        catch (e) {
            return false;
        }
    }
    async function authentication(login, password) {
        const ticket = await ODANT.API.request(ODANT.origin + '/api', { method: 'oauth-authentication', login: login }, password);
        sendTicket(login, ticket);
    }
    function sendTicket(login, ticket) {
        window.removeEventListener('beforeunload', sendClose);
        window.opener.postMessage({ type: 'action', data: 'authorize', params: { login, ticket } }, '*');
    }

    async function loadOtherMethods() {
        const otherMethodsBlock = document.getElementById('otherMethodsBlock');
        otherMethodsBlock.innerHTML = '';
        try {
            const methods = await ODANT.API.request(ODANT.origin + '/api', { method: 'get_methods' });
            if (methods && methods.length) {
                for (const m of methods) {
                    if (m !== 'oauth') {
                        const b = document.createElement('oda-button');
                        b.label = m;
                        b.addEventListener('click', () => {
                            window.removeEventListener('beforeunload', sendClose);
                            location.replace(getPath(m));
                        });
                        otherMethodsBlock.appendChild(b);
                    }
                }
                otherMethodsBlock.style.opacity = '';
            } else {
                otherMethodsBlock.style.opacity = '0';
            }
        } catch (err) {
            otherMethodsBlock.style.opacity = '0';
            console.warn(err);
        }
    }
    function getPath(method = 'pap') {
        const path = `${ODANT.origin}/api/~/client/auth`;
        const redirect_url = new URL(location.toString()).searchParams.get('redirect_url');
        let s = `${path}/${method}/index.html`;
        if (redirect_url) {
            s += `?redirect_url=${redirect_url}`;
        }
        return s;
    }
}
setTimeout(async () => {
    if (ODANT && ODANT.frameworkReady) load();
    else {
        await loadCore();
        load();
    }
}, 300);