window.name = 'odant-authentication';
window.addEventListener('message', (e) => {
    switch (e.data.type) {
        case 'action':
            switch (e.data.data) {
                case 'close': window.close(); break;
                case 'focus': window.focus(); break;
                default:
                    break;
            }
            break;
        default:
            break;
    }
});
function sendClose() {
    window.opener.postMessage({ type: 'action', data: 'authorize', params: {login: 'guest'} }, '*');
}
function sendBlur() {
    window.opener.postMessage({ type: 'event', data: 'blur' }, '*');
}
window.addEventListener('beforeunload', sendClose);
window.addEventListener('blur', sendBlur);

const form = document.getElementById('form');
const loginBlock = document.getElementById('loginBlock');
const logInput = document.getElementById('logInput');
const passInput = document.getElementById('passInput');
const loginBtn = document.getElementById('loginBtn');
const forgotBtn = document.getElementById('forgotBtn');
const regBtn = document.getElementById('regBtn');
const guestBtn = document.getElementById('guestBtn');
const buttons = document.getElementById('buttons');

const redirect_url = new URL(location).searchParams.get('redirect_url');
async function load() {
    form.style.transition = 'all .5s ease-in-out';
    setTimeout(() => {
        form.style.opacity = '1';
        setTimeout(() => {
            logInput.select(0, -1);
            _formInput();
            form.style.transition = '';
            form.style.opacity = '';
            loadOtherMethods();
        }, 500);
    }, 100);

    form.addEventListener('submit', (e) => {
        e.preventDefault();
    });
    [logInput, passInput].forEach(i => i.addEventListener('focus',
        (e) => { e.target.select(0, -1) }
    ));
    form.addEventListener('input', _formInput);
    form.addEventListener('reset', _reset);
    forgotBtn.addEventListener('click', _forgot);
    loginBtn.addEventListener('click', _login);
    regBtn.addEventListener('click', _login);
    guestBtn.addEventListener('click', async (e) => {
        _setInputsDisabled(true);
        sendTicket('guest');
    });

    async function _formInput(e) {
        const login = logInput.value;
        const password = passInput.value;
        if (passInput.placeholder === 'pin') {
            if (password.length < 4) return;
            else authentication(login, password);
        }
        await _checkLogin({ login, password });
    }
    function _reset() {
        logInput.value = '';
        passInput.value = '';
        _formInput(e);
    }
    async function authentication(login, password) {
        const passHash = Sha256.hash(password);
        const ticket = await ODANT.API.request(ODANT.origin + '/api', { method: 'authentication', login: login }, passHash);
        sendTicket(login, ticket);
    }
    async function _checkLogin({ login = logInput.value, password = passInput.value }, force) {
        let checked = false;
        try {
            checked = force || await checkLogin(login);
            setVisibility(loginBtn, checked && password);
        } catch (e) {
            setVisibility(loginBtn, false);
        } finally {
            setVisibility(regBtn, !checked && password);
            setVisibility(guestBtn, !checked || !login);
            setVisibility(forgotBtn, login && checked);
        }
        return checked;
    }
    function setVisibility(elem, cond) {
        cond = Boolean(cond);
        if (!cond) {
            elem.setAttribute('hidden', '');
        } else {
            elem.removeAttribute('hidden');
        }
    }
    function _setInputsDisabled(cond) {
        cond = Boolean(cond);
        const inputBlocks = [...form.querySelectorAll('.inputBlock')];
        inputBlocks.forEach(b => [...b.children].forEach(c => {
            c.disabled = cond;
            if (cond) {
                c.setAttribute('disabled', '');
            } else {
                c.removeAttribute('disabled');
            }
        }));
    }
    function _setLoginDisabled(cond) {
        cond = Boolean(cond);
        [...loginBlock.children].forEach(c => {
            c.disabled = cond;
            if (cond) {
                c.setAttribute('disabled', '');
            } else {
                c.removeAttribute('disabled');
            }
        });
    }
    function _setPinMode(cond) {
        cond = Boolean(cond);
        _setLoginDisabled(cond);
        buttons.hidden = cond;
        if (cond) {
            passInput.placeholder = 'pin';
            passInput.type = 'text';
        } else {
            buttons.hidden = false;
            passInput.placeholder = 'password';
            passInput.type = 'password';
        }
        passInput.value = '';
    }
    function _forgot() {
        _setPinMode(true);
        const login = logInput.value;
        return findAccount(login).then(res => {
            ODANT.API.request(res.result + '/api', {
                method: 'one_time_password',
                login: login,
                type: 'email',
                referrer: document.URL,
            }).then(res => {
                passInput.focus();
            }).catch(err => {
                _setPinMode(false);
            });
        });
    }
    async function _login() {
        const login = logInput.value;
        const password = passInput.value;
        const loginChecked = await checkLogin(login);
        try {
            if (loginChecked) {
                await authentication(login, password);
            }
            else {
                if (await registration(login, password)) {
                    _checkLogin(login, password, true);
                    _setPinMode(true);
                    passInput.focus();
                }
            }
        } catch (e) {
            _setPinMode(false);
        }
    }
    function _isEmail(login) {
        if (!login)
            return false;
        const re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        return re.test(login);
    }
    async function checkLogin(login) {
        try {
            await ODANT.API.request(ODANT.origin + '/api', { method: 'check_login', login: login });
            return true;
        }
        catch (e) {
            return false;
        }
    }
    async function findAccount(login, type = 'mail') {
        const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'find_account', login: login, type: 'email' });
        return response.result;
    }
    async function registration(login, password) {
        try {
            await checkLogin(login);
        } catch (e) {
            throw new Error('login is busy');
        };
        const passHash = Sha256.hash(password);

        const res = await ODANT.API.request(ODANT.origin + '/api', { method: 'registration', login: login, type: 'email', referrer: document.URL }, passHash);
        return res.result;
    }
    function sendTicket(login, ticket) {
        window.removeEventListener('beforeunload', sendClose);
        window.opener.postMessage({ type: 'action', data: 'authorize', params: { login, ticket } }, '*');
    }
    async function loadOtherMethods() {
        const otherMethodsBlock = document.getElementById('otherMethodsBlock');
        otherMethodsBlock.innerHTML = '';
        try {
            const methods = await ODANT.API.request(ODANT.origin + '/api', { method: 'get_methods' });
            if (methods && methods.length > 1) {
                for (const m of methods) {
                    if (m !== 'pap') {
                        const b = document.createElement('oda-button');
                        b.label = m;
                        b.addEventListener('click', () => {
                            window.removeEventListener('beforeunload', sendClose);
                            location.replace(getPath(m));
                        });
                        otherMethodsBlock.appendChild(b);
                    }
                }
                otherMethodsBlock.style.opacity = '';
            } else {
                otherMethodsBlock.style.opacity = '0';
            }
        } catch (err) {
            otherMethodsBlock.style.opacity = '0';
            console.warn(err);
        }
    }
    function getPath(method = 'pap') {
        const path = `${ODANT.origin}/api/~/client/auth`;
        const redirect_url = new URL(location.toString()).searchParams.get('redirect_url');
        let s = `${path}/${method}/index.html`;
        if (redirect_url) {
            s += `?redirect_url=${redirect_url}`;
        }
        return s;
    }
}
setTimeout(async () => {
    if (ODANT && ODANT.frameworkReady) load();
    else {
        await loadCore();
        load();
    }
}, 300);