'use strict';
globalThis.odaIndex =  globalThis.odaIndex || class odaIndex extends odaClassItem {
    destroy() {
        this.$connection.unlisten(this);
    }

    constructor(p){
        super(p);
        if (this.$class)
            this.$class._indexes.push(this);
        this.$connection.listen(this, event =>{
            // ODA.push('Index event: '+this);
            this.fireChanged(event);
        })
    }

    static get properties(){ return {
        prefix: 'I',
        icon: 'table'
    }}


    getRows(mask, from, length, post){
        if (typeof post === 'object')
            post = JSON.stringify(post);
        return this.POST('dataset', {loadmask: mask, mask: mask, from: from, length: length, async: true}, post);
    }
    search(mask = '*', filter= '*/*'){
        // return this.API.POST('search_index', {}, filter);
        return this.POST('search_index', {loadmask: mask, format: 'json', async: true}, filter);
    }

    get fields() {
        return this.$class.fields;
    }

    getDataSet(mask = '*', filter, clear){
        if(clear) this.reset();
        return this.$('DataSet-'+mask + filter, ()=>{
            return new IndexDataSet(this, mask, filter);
        }, 100000) //todo Предложение по сбросу кэша
    }

    XQuery (xq, {mask = '*', format = 'json', byuser = '*'}={}){
        return this.POST('xquery_index', {loadmask: mask, format: format, by_user: byuser, async: true}, xq)
    }
    get $indexes(){
        return this.$('$indexes', async ()=>{
            // let res = await this.API.GET('get_class_indexes');
            // res = res.result.split('|');
            let res = ['a','b'];
            res = res.map(i=>this.getIndex(i));
            return await Promise.all(res);
        })
    }
    get $items(){
        return this.$indexes;
    }
    getIndex(name, XQ) {
        return this.$('Index:'+name, async ()=>{
            if (XQ)
                await this.API.POST('create_index', {name: name}, XQ);
            return new odaIndex({id: name, $owner: this, XQ: XQ});
        })
    }
    async getRowItem(dataRoot){
        if (dataRoot){
            let rows = Array.isArray(dataRoot)?dataRoot:[dataRoot];
            if (rows.length>0){
                const item = await this.$class.getObject(rows[0].oid);
                if(rows.length === 1)
                    return item;
                return new odaItemList({$owner: this.$class, $item: item, items: rows.map(i =>{ return i.oid})});

            }
        }
    }
};

globalThis.odaXQuery =  globalThis.odaXQuery || class odaXQuery extends odaClassItem {

    static get properties(){ return {
            prefix: 'Q',
            icon: 'xquery',
            name (){
                return this.getAttr('Name');
            },
            XQ (){
                return this.getAttr('XQ');
            },
            note (){
                return this.getAttr('Hint');
            },
            type (){
                return this.getAttr('Type', 'Standart');
            }
        }}
    
    get HasChildren(){
        let xq = this.Root.$XQ;
        return Boolean(Array.isArray(xq) && xq.length > 0);
    }

    async getActions(){
        let src = this.TypeClass || this.$class;
        if (src){
            const methods = await src.getActions('_xqueries', this);
            return methods.map(i => {
                let m = Object.create(i);
                m._owner = this;
                return m;
            });
        }
        return [];
    }
};
globalThis.IndexDataSet =  globalThis.IndexDataSet || class IndexDataSet{
    constructor(index, mask , filter){
        this.Index = index;
        this.Mask = mask;
        this.filter = filter;
        // index.listen('changed', event=>{
        //     ODA.push(`DataSet in "${this.Index.label}" index from class "${this.Index.$class.label}" has been changed.`);
        // });
        this._itemCode = itemCounter();
    }
    async getItems(ftom = 1, length = 100){
        try{
            const data = await this.Index.POST('xquery_index', {loadmask: this.Mask, format: 'json', by_user: this.ByUser, async: true}, `subsequence(*/*, ${ftom}, ${length})`);
            return data[Object.keys(data)[0]];
        }
        catch (e) {
            console.error(e);
            return [];
        }
    }
    get Items(){
        return (async ()=>{
            let data = [];
            // console.time('IndexDataSet - items');
            if(this.filter){
                 data = await this.Index.POST('search_index', {loadmask: this.Mask, format: 'json', by_user: this.ByUser, async: true}, this.filter);
            }else{
                 data = await this.Index.POST('xquery_index', {loadmask: this.Mask, format: 'json', by_user: this.ByUser, async: true}, '*/*');
            }

            if (data){
                // console.time('IndexDataSet - data');
                data  = data[Object.keys(data)[0]];
                // console.timeEnd('IndexDataSet - data');
            }
            else
                data = [];
            // console.timeEnd('IndexDataSet - items');
            return data;
        })();
    }
    get metadata(){
        return this.Index.fields;
    }
    async getRowItem(row){
        if (row){
            let rows = Array.isArray(row)?row:[row];
            if (rows.length === 0)
                return Promise.resolve(null);

            const item = await this.Index.$class.getObject(rows[0].oid);
            if(rows.length === 1)
                return item;
            return new odaItemList({$owner: this.$class, $item: item, items: rows.map(i =>{ return i.oid})});
        }
        return Promise.reject();
    }
};
// DataSet -------------------------------------------------------------------------------------------------------------
'use strict';
globalThis.DataSet =  globalThis.DataSet || class DataSet{
    constructor(item, p){
        this.$('item', item);
        if(p){
            for(let i in p){
                let pd = Object.getOwnPropertyDescriptor(this.__proto__, i);
                if(pd && pd.set)
                    this[i] = p[i];
                else
                    this.$(i, p[i]);
            }
        }
        if(!(item instanceof odaField) &&  this.$class)
            this.$class.onServerEvents(this.ServerEvents.bind(this));
//                this.$class.addServerEventListener(this.ServerEvents.bind(this))
    }
    ServerEvents(e) {
        setTimeout(()=> {
            if (e.updates) {
                let mask = this.$('PackMask', '*');
                let pack = e.pack.replace(/#(?=.)/g, '*/') + '*';

                if(mask === '*' || mask === pack){
                    e.updates.forEach(oid=> {
                        this.$class.getObjectList(oid).then(objects=> {
                            this.updateItems(objects);
                        })
                    });
                }
            }
            else if (e.deletes && this._items) {
                let items = [];
                this._items.forEach(i=> {
                    let idx = e.deletes.indexOf(i.oid);
                    if (idx > -1) {
                        items.push(i);
                        e.deletes.splice(idx, 1);
                    }
                });
                this.removeItems(items);
            }
        }, 1)
    }
    get $class(){
        return this.$item.$class;
    }
    get $item(){
        return this.$('$item');
    }
    get Packets(){
        return this._data;
    }
    get Data(){
        return this._items;
    }
    get Items(){
        return this.$('items', ()=>{
            if(!this._items){
                this._items = [];

                if (this.$item instanceof odaObject){
                    this._items.push(this.dataRoot);
                }
                else if (this.$item instanceof odaField){
                    if (!this.dataRoot['$'+this.$item.name])
                        this.dataRoot['$'+this.$item.name] = new Array();
                    this._items = this.dataRoot['$'+this.$item.name];
                }
                else {
                    let p;
                    if (this.$item instanceof odaClass){
                        p = this.$item.XQuery();
                    }
                    else if (this.$item instanceof odaIndex){
                        p = this.$item.XQuery('*', {mask: this.$('PackMask', '*')});
                    }
                    if (p)
                        return p.then(data =>{
                            let i;
                            for(i in data){
                                break;
                            }
                            this._data = data[i];
                            this._data.forEach(x =>{
                                i = null;
                                for(i in x) {
                                    if(i.startsWith('$'))
                                        break;
                                    else i = null;
                                }
                                if(i)
                                    this._items = this._items.concat(x[i]);
                            });
                            return this._items;
                        });
                }
            }
            return Promise.resolve(this._items);
        });
    }
    get dataRoot(){
        return this.$('dataRoot', () => {
            switch(this.prefix){
                case 'O':
                    return this.$item.Root;
                case 'C':
                    break;
            }
        });
    }

    get itemType(){
        return this.$item.__proto__.constructor.name;
    }
    get fields(){
        return this.$item.fields;
    }
    get id(){
        return this.$('id', function(){
            if(this.$item instanceof odaField){
                return this.$item.fullId+'/FIELD:' + this.$item.path;
            }
            return this.$item.fullId;
        }.bind(this));
    }
    get metadata() {
        return this.$('metadata', () =>{
            return this.fields.then(fields =>{
                let metadata = fields.filter(field => {
                    if (this.$item instanceof odaIndex){
                        if (field.isTable)
                            return false;
                        // if (field.HideOnGrid)
                        //     return false;
                    }
                    return true;
                });
                if(this.$item instanceof odaIndex && this.$class.hasChildren){
                    return ODANT.loadScript("/web/core/class/fields/_class/script.js").then(script =>{
                        let p = {$owner: this.$class, DataSet: this, key: script.src};
                        let clsField = ODANT.factory.create(p);
                        metadata.push(clsField);
                        return metadata;
                    })
                }
                return metadata;
            })
        });
    }
    getRowItem(row){
        if (row){
            let rows = Array.isArray(row)?row:[row];
            if (rows.length === 0)
                return Promise.resolve(null);
            let p;
            switch(this.$item.prefix){
                case 'O':
                    p = Promise.resolve(this.$item);
                    break;
                case 'C':{
                    p = this.$item.getObject(rows[0].oid);
                } break;
                case 'I':{
                    p = this.$item.$class.getObject(rows[0].oid);
                } break;
            }
            return p.then(item=>{
                if(rows.length === 1)
                    return item;
                return new odaItemList({$owner: this.$class, $item: item, items: rows.map(i =>{ return i.oid})});
            })
        }
        return Promise.reject();
    }
    updateItems(items){
        this.Items.then(() => {
            // let filter = items.map(i => i.oid).join(' ');
            // let old = this._items.filter(i => filter.includes(i.oid));
            // this.removeItems(old);
            // this.addItems(items);

            let update = [];
            let add = [];
            items && items.forEach(item => {
                let idx = this._items.findIndex(i => i.oid === item.oid);
                idx > -1 ? update.push({idx, item}) : add.push(item);
            })

            this._updateItems(update);
            this.addItems(add);

        })
    }

    _updateItems(items){
        items && items.forEach(i => {
            let newitem = {};
            ODANT.mixin(i.item, newitem);

            let idx = i.idx ? i.idx : this._items.findIndex(item => item.oid === i.oid)
            this._items.splice(idx, 1, newitem);
        })

        this.fire('update-items', items.map(i => i.item));
    }

    addItems(rows){
        return this.Items.then(items=>{
            if(rows && rows.length>0){
                rows.forEach(r =>{
                    if (!items.includes(r))
                        items.push(r);
                })
                // items.push(...rows);
                // let added = rows.map(i=> {
                //     // let newitem = {};
                //     // ODANT.mixin(i, newitem);
                //     if (!items.includes(i))
                //         items.push(i);
                //     return i;
                // });
                if(rows.length>0)
                    this.fire('add-items', rows);
            }
        });
    }
    removeItems(items){
        return this.Items.then(()=>{
            if(items && items.length>0){
                let removed = items.filter(i =>{
                    let idx = this._items.indexOf(i);
                    if(idx>-1){
                        this._items.splice(idx, 1);
                    }
                    return idx>-1;
                });
                if(removed.length>0)
                    this.fire('remove-items', removed);
            }
        });
    }


    $(key, callback) {
        if (!this._$) {
            this._$ = new Map();
        }
        let result = this._$.get(key);
        if (result === undefined) {
            if (callback && typeof callback === 'function')
                result = callback();
            else
                result = callback;
            this._$.set(key, result);
        }
        return result;
    }
    reset(key) {
        if (this._$) {
            if (key) {
                if (!Array.isArray(key))
                    key = key.split(' ');
                key.forEach(k => {
                    this._$.delete(k);
                })
            }
            else {
                let arr =[];
                this._$.forEach((v,k)=>{
                    if(!this._resetExcludes.includes(k))
                        arr.push(k);
                });
                this.reset(arr);
            }

        }
    }


    fire(event, detail){
        if(!this.events)
            return;
        if(!this.events[event])
            return;
        let e = new CustomEvent(event, {target: this, detail: detail});
        this.events[event].forEach(i=>{
            i(e);
        })
    }
    addEventListener(event, resolve){
        if(!this.events)
            this.events = {};
        if(!this.events[event])
            this.events[event] = [];
        this.events[event].push(resolve);
    }
    removeEventListener(event, resolve){
        if(!this.events)
            return;
        if(!this.events[event])
            return;
        let i = this.events[event].indexOf(resolve);
        if(i<0)
            return;
        this.events[event].removeAt(i);
    }
}