import ODA from '/web/oda/oda.js';
import ODANT from '/web/core/odant3.js';
ODANT({
    is: 'odant-packets-bar', template: /*html*/`
    <style>
        :host {
            @apply --flex;
            @apply --horizontal;
            flex-grow: 0 !important;
            width: 100%;
            overflow-x: auto;
            border-bottom: 1px solid gray;
            box-sizing: border-box;
        }
    </style>
    <oda-packets-bar-row ref="rows" ~for="(item, key) in items" :items="item.data" :meta="item.meta" :mode="item.mode" :key="key" @mask-changed="_rowMaskChanged"></oda-packets-row>
    `,
    props: {
        contextItem: {
            type: Object,
            set(contextItem) {
                if (this.packets || !contextItem) return;
                // this.listen(contextItem, 'pack-create', '_getPackets');
                // this.listen(contextItem, 'pack-delete', '_getPackets');
                this._getPackets();
                // .catch(err => {
                //     this._hideBar();
                //     console.error(err.message);
                // });
            }
        },
        selection: Array,
        history: Array,
        packets: Array,
        items: Array,
        mask: String,
        structure: {
            type: Object,
            default: {}
        }
    },
    async _getPackets() {
        await this.contextItem.body;
        const root = this.contextItem.ROOT;
        const packets = await this.contextItem.$$packets;
        if (packets && packets.length > 0/* && !(packets.length === 1 && packets[0].name === 'pack')*/) {
            // this.packets = [packets];
            // packets.forEach(packet => {
            //     if (packet.hasPacks)
            //         this._buildPackets(packet.Packets);
            // });
            this.packets = this.packets || [];
            this._buildPackets(packets);
            this.items = root && root.$Pack ? root.$Pack.map((sPack, index) => {
                const mode = sPack.PackMode;
                const arrayOfPacketsMap = [...this.packets[index].values()];
                const row = {
                    meta: sPack,
                    mode: mode,
                    data: this._rebuildPackets(arrayOfPacketsMap, mode) || []
                };
                return row;
            }) : this.packets.map((packs, index) => ({ data: this._rebuildPackets([...packs.values()]) }));
            this.hidden = false;
        } else {
            this._hideBar();
        }
    },
    _rowMaskChanged(event) {
        this.debounce('row-mask-changed', () => {
            const masks = this.$refs.rows.map(row => row.mask ? row.mask.split('|') : ['']);
            const nMasks = [...masks[0]];
            for (let idx = 1; idx < masks.length; idx++) {
                let tMasks = nMasks.reduce((res, nm) => {
                    masks[idx].forEach(m => {
                        res.push(`${nm}/${m}`);
                    });
                    return res;
                }, []);
                if (tMasks) nMasks.splice(0, nMasks.length, ...tMasks);
            }
            this.mask = nMasks.join('|');
            ODA.success(this, 'mask changed', this.mask);
        }, 10);
    },
    _buildPackets(packets, level = 0) {
        this.packets[level] = this.packets[level] || new Map();
        packets.forEach(packet => {
            this.packets[level].set(packet.name, packet);
            // if (!this.packets[level].some(i => i.name === packet.name)) this.packets[level].push(packet);
            if (packet.hasPacks) this._buildPackets(packet.$$packets, level + 1);
        });
    },
    _rebuildPackets(packets, mode) {
        if (/day|month|quart|year/i.test(mode)) {
            let isValidDate = [];
            let isNotValidDate = [];
            packets.forEach(p => {
                if (/^(\d{4})[\/\-]?(0[1-9]|1[012]|q[1-4])?[\/\-]?(0[1-9]|[12][0-9]|3[01])?$/.test(p.name)) {
                    isValidDate.push(p);
                } else
                    isNotValidDate.push(p);
            });
            if (isValidDate.length > 0) {
                let { years, quarts, months, days } = this._getDateStructure(isValidDate, mode);
                years.forEach(year => {
                    year.items = /year/i.test(mode) ? [] : quarts.filter(quart => quart.name.substr(0, 4) === year.name);
                });
                if (!/quart|year/i.test(mode)) {
                    quarts.forEach(quart => {
                        quart.items = months.filter(month => {
                            let splits = quart.name.split('q');
                            let monthsOfQuarter = this._getMonthsOfQuarter(parseInt(splits[1]));
                            monthsOfQuarter = monthsOfQuarter.map(m => splits[0] + m);
                            return monthsOfQuarter.some(m => m === month.name);
                        });
                    });
                    months.forEach(month => {
                        month.items = days.filter(day => day.name.substr(0, 7) === month.name);
                    });
                    days.forEach(day => {
                        day.items = [];
                    });
                } else if (/quart/i.test(mode)) {
                    quarts.forEach(quart => {
                        quart.items = [];
                    });
                }
                isValidDate = [this._getItem({ name: new Date().toISOString().split('T')[0], is: 'oda-packets-bar-now-selector' }, mode), ...years];
            }
            if (isNotValidDate.length > 0) {
                isNotValidDate = this._getFieldStructure(isNotValidDate);
            }
            return [...isValidDate, ...isNotValidDate];
        } else {
            const result = this._getFieldStructure(packets);
            console.log(result);
            return result;
        }
    },
    _getFieldStructure(packs, mode) {
        let tree = null,
            arrayOfTree = [];
        for (let x = 0; x < packs.length; x++) {
            if (packs[x].name === 'pack' || packs[x].name === '') {
                arrayOfTree.push(this._getItem({ name: packs[x].name }, mode));
                continue;
            }
            let current = null,
                existing = null;
            [...packs[x].name].forEach((i, idx) => {
                const step = packs[x].name.substring(0, idx + 1);
                if (idx === 0) {
                    if (arrayOfTree.every(t => t.name !== step)) {
                        tree = this._getItem({ name: step }, mode);
                        arrayOfTree.push(tree);
                    }
                    current = tree.items;
                } else {
                    existing = current.find(cur => cur.name === step);
                    current = existing ? existing.items : current[current.push(this._getItem({ name: step }, mode)) - 1].items;
                }
            });
        }
        arrayOfTree = this._clearTree(arrayOfTree)
            .map(item => (item.name.length > 1 && item.name !== 'pack') ? this._getItem({ name: item.name.substring(0, 1), items: [item] }, mode) : item);
        return arrayOfTree;
    },
    _clearTree(items) {
        return items.map(item => {
            let current = item;
            if (item.items.length !== 1) {
                current.items = this._clearTree(item.items);
            } else {
                while (current.items.length === 1) {
                    current = current.items[0];
                }
                current.items = this._clearTree(current.items);
            }
            return current;
        });
    },
    _getDateStructure(packs, mode) {
        const yrs = {};
        const dateStructure = packs.reduce((a, c) => {
            let parts = c.name.split('-');
            let addToGroupDate = (dateArray, dateItem, groupName) => {
                let item = dateArray.find(el => el.name === groupName);
                if (!item)
                    dateArray.push(this._getItem({ name: groupName, items: [dateItem] }, mode));
                else
                    item.items.push(c);
            };
            parts.forEach((part, idx) => {
                switch (idx) {
                    case 0:
                        yrs[part] = yrs[part] || { items: [] };
                        yrs[part].items.push(c);
                        addToGroupDate(a.years, c, part);
                        break;
                    case 1:
                        const monthGroupName = c.name.substr(0, 7);
                        const splitQuartName = monthGroupName.split('-');
                        const quartGroupName = /quart/i.test(mode) ? c.name : splitQuartName[0] + '-q' + this._getQuarterOfMonth(splitQuartName[1]);
                        addToGroupDate(a.quarts, c, quartGroupName);
                        addToGroupDate(a.months, c, monthGroupName);
                        break;
                    case 2:
                        addToGroupDate(a.days, c, c.name.substr(0, 10));
                        break;
                }
            });
            return a;
        }, { years: [], quarts: [], months: [], days: [] });
        return { years: dateStructure.years.sort((a, b) => parseInt(b.name) - parseInt(a.name)), quarts: this._sortArray(dateStructure.quarts), months: this._sortArray(dateStructure.months), days: this._sortArray(dateStructure.days) };
    },
    _getItem(preset, mode) {
        let item = {
            ...this._getMask(preset, mode),
            ...this._getLabel(preset, mode),
            type: this._getType(preset),
            execute: (row, detail) => this._execute(item, row, detail),
            hideIcon: true,
            items: preset.items || []
        };
        return { ...item, ...preset };
    },
    _getQuarterOfMonth(month) {
        return [1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4][parseInt(month) - 1];
    },
    _getMonthsOfQuarter(quart) {
        return [['01', '02', '03'], ['04', '05', '06'], ['07', '08', '09',], ['10', '11', '12']][parseInt(quart) - 1];
    },
    _getQuarterNumOfQuarterName(quartName) {
        return parseInt(quartName.split('q')[1]);
    },
    _getType({ name }) {
        return /^(\d{4})$/.test(name) ? 'year' :
            /^(\d{4})[\/\-](q[1-4])$/.test(name) ? 'quarter' :
                /^(\d{4})[\/\-](0[1-9]|1[012])$/.test(name) ? 'month' :
                    /^(\d{4})[\/\-](0[1-9]|1[012])[\/\-](0[1-9]|[12][0-9]|3[01])$/.test(name) ? 'day' :
                        name.length > 1 ? 'field' : 'alpha';
    },
    _getLabel(preset, mode) {
        const name = preset.name;
        let dt = new Date(name);
        let label = name === 'pack' || name === '' ? '?' : name.toUpperCase();
        // let packets = this.packs.find(p => p.name === name);
        // if (/alpha/i.test(mode) && name !== 'All' && findedPack && findedPack.$packet && findedPack.$packet.Root && findedPack.$packet.Root.count)
        //     label = `${label} [${this.packs.find(p => p.name === name).$packet.Root.count}]`;
        if ((dt instanceof Date && !isNaN(dt)) || this._getType(preset) === 'quarter') {
            switch (this._getType(preset)) {
                case 'year':
                    label = dt.toLocaleDateString(ODANT.language, { year: 'numeric' });
                    break;
                case 'quarter':
                    const year = name.split('-')[0];
                    const quart = parseInt(name.split('-')[1].substr(1));
                    label = `${quart}/4 ${year}`;
                    break;
                case 'month':
                    label = dt.toLocaleDateString(ODANT.language, { year: 'numeric', month: 'long' });
                    break;
                case 'day':
                    label = dt.toLocaleDateString(ODANT.language, {
                        year: 'numeric',
                        month: 'long',
                        day: 'numeric'
                    });
                    break;
            }
        }
        return { label };
    },
    _getMask(preset, mode) {
        const name = preset.name;
        let mask = name === '' ? '' : name + '*';
        // let mask = name === '' ? '' : name;
        if (this._getType(preset) === 'quarter' && !/quart/i.test(mode)) {
            const year = name.split('-')[0];
            const quart = parseInt(name.split('-')[1].substr(1));
            switch (quart) {
                case 1:
                    mask = `${year}-01*|${year}-02*|${year}-03*`;
                    break;
                case 2:
                    mask = `${year}-04*|${year}-05*|${year}-06*`;
                    break;
                case 3:
                    mask = `${year}-07*|${year}-08*|${year}-09*`;
                    break;
                case 4:
                    mask = `${year}-10*|${year}-11*|${year}-12*`;
                    break;
            }
        }
        return { mask };
    },
    _execute(item, row, detail) {
        // const _ctrlKey = detail && detail.sourceEvent && detail.sourceEvent.ctrlKey;
        // this.async(() => {
        //     selector = selector || this.children.find(i => i.item === item);
        //     if (selector) {
        //         if (_ctrlKey) selector.selected = !selector.selected;
        //         // else this._setSelected(selector);
        //         selector.label = item.label || item.name;
        //     }
        // }, 10);
        // let resultPacks = (this._getType(item) === 'quarter' && !/quart/i.test(this.mode)) ? item.items.reduce((a, c) => a.concat(this.packs.filter(pack => pack.name.includes(c.name))), []) : this.packs.filter(pack => pack.name.includes(item.name)),
        //     resultArray = resultPacks.reduce((a, c) => a.concat(c.items.filter(packItem => !a.some(i => i.name === packItem.name))), []);
        // this.selectedChildren = item.mask === '*' ? this.packs.reduce((a, c) => a.concat(c.items.filter(packItem => !a.some(i => i.name === packItem.name))), []) : _ctrlKey ? this.selectedChildren.concat(resultArray) : resultArray;
        // row.mask = row.mask ? row.mask + '|' + item.mask : item.mask;
        row.mask = item.mask;
        // if (item.name !== 'All' && /alpha/i.test(this.mode))
        //     this.__write(`${this.contextItem.fullId.slice(1)}/${this.mode}/saved`, item);
    },
    _sortArray(arr) {
        return arr.sort((a, b) => {
            if (a.name === 'pack' || a.name === '') return 1;
            if (b.name === 'pack' || b.name === '') return -1;
            if (/q/.test(a.name + b.name)) {
                const aa = a.name.split('-');
                const bb = b.name.split('-');
                if (aa[0] === bb[0]) {
                    const aaa = parseInt(aa[1].toString().replace('q', ''));
                    const bbb = parseInt(bb[1].toString().replace('q', ''));
                    if (!isNaN(aaa) && !isNaN(bbb)) {
                        return aaa - bbb;
                    }
                }
            }
            return a.name.localeCompare(b.name);
        })
    },
    _hideBar() {
        this.mask = '*';
        this.hidden = true;
    }
});
ODA({
    is: 'oda-packets-bar-row', template: /*html*/`
    <style>
    :host {
        @apply --flex;
        @apply --horizontal;
        flex: 0 0 auto;
        border-left: 3px #999 solid;
        align-items: center;
        padding: 4px;
    }
    </style>
    <div refs="selectors" ~is="item.is || tmpl" :menu="$refs.menu" :title="'by ' + mode" ref="selectors" ~for="(item, key) in items" :item="item" :key="key"></div>
    `,
    props: {
        items: {
            type: Array,
            set(items, old) {
                if (!old) items[0].execute(this);
            }
        },
        selection: Array,
        meta: Object,
        mask: {
            type: String,
            set(mask, old) {
                this.fire('mask-changed');
            }
        },
        mode: {
            type: String,
            enum: ['Alpha', 'Field', 'Day', 'Month', 'Quart', 'Year']
        },
        _isDate: false,
        _selectors: [],
        tmpl: 'oda-packets-bar-selector'
    },
    _setSelected(selector) {
        this.$refs.selectors.forEach(i => {
            i.selected = selector === i;
            i.label = i.item && (i.item.label || i.item.name);
        });
    }
});
import '/web/oda/components/menus/menu/menu.js';
import '/web/oda/components/buttons/button/button.js';
ODA({ is: 'oda-packets-bar-menu', extends: 'oda-menu', template: /*html*/`
    <oda-button icon="icons:check" icon-size="14" @tap="_onTap" />
    `,
    props: {
        selector: HTMLElement
    },
    _onTap() {
        const _selectedFunc = (items) => {
            return items.reduce((acc, i) => {
                if(i.items.length > 0) {
                    const selectedChild = _selectedFunc(i.items);
                    acc.push(...selectedChild);
                }
                if(i.selected) acc.push(i);
                return acc;
            }, []);
        };
        const result = _selectedFunc(this.items);
        debugger;
        result.forEach(i => delete i.selected);
        this.selector.domHost.mask = result.map(i => i.mask).join('|');
    }
});
ODA({
    is: 'oda-packets-bar-item', template: /*html*/`
    <style>
        :host {
            padding: 6px;
            cursor: pointer;
            font-size: 14px;
            white-space: nowrap;
            max-height: 30px;
            box-sizing: border-box;
        }

        :host([selected]) {
            @apply --selected;
            color: var(--success-color) !important;
            border: 1px solid var(--success-color);
            text-decoration: underline;
        }
    </style>
    `,
    props: {
        item: {
            type: Object,
            set(item) {
                this.label = item.fullLabel || item.label || item.name;
            }
        },
        selected: {
            type: Boolean,
            reflectToAttribute: true,
            default: false
        },
        label: '',
        count: Number
    },
    listeners: {
        'tap': '_onTapOrDblClick',
        'dblclick': '_onTapOrDblClick'
    },
    async _onTapOrDblClick(e) {
        const firstItem = Object.assign({}, this.item);
        firstItem.items = [];
        let menuItems = [firstItem, ...this.item.items];
        // if (e.type === 'dblclick') this.item.execute(this.domHost);
        // this.debounce('tap-or-dblclick', async () => {
        //     if (e.type === 'tap') {
        if (!this.item.items || (this.item.items && this.item.items.length === 1 && !this.item.items[0].items.length) || (this.item.items && this.item.items.length === 0)) {
            this.domHost._setSelected(this);
            this.item.execute(this.domHost, e.detail);
        } else {
            e.stopPropagation();
            const result = { focusedItem: null };
            await ODA.showDropdown('oda-packets-bar-menu', {
                items: menuItems,
                root: result,
                template: 'oda-packets-bar-context-menu-item',
                selector: this,
                _tap(e) { this.focusedItem = e.currentTarget.item; }
            }, { parent: this });
            this.domHost._setSelected(this);
            // this.label = this.item.label || this.item.name;
            if(result.focusedItem) result.focusedItem.execute(this.domHost, e.detail);
        }
            // }
        // }, 250);
    }
});
ODA({
    is: 'oda-packets-bar-selector', extends: 'oda-packets-bar-item', template: /*html*/`
    <style>
        :host {
            @apply --header;
        }
    </style>
    {{label}}
    `
});
ODA({
    imports: [ '/web/oda/components/buttons/button/button.js' ],
    is: 'oda-packets-bar-now-selector', extends: 'oda-packets-bar-item', template: /*html*/`
    <style>
        :host {
            @apply --horizontal;
        }
    </style>
    <div>{{label}}</div>
    <oda-button icon="icons:arrow-drop-down" @tap="_onHistory" style="padding: 0; min-height: auto; min-width: auto;" icon-size="22" />
    `,
    _onHistory() {
        // ODA.error(this, 'History button tap');
    }
});
import '/web/oda/components/buttons/checkbox/checkbox.js';
ODA({
    is: 'oda-packets-bar-context-menu-item', template: /*html*/`
    <style>
        :host {
            @apply --label;
            @apply --horizontal;
            @apply --flex;
            align-items: center;
            max-width: 480px;
            text-overflow: ellipsis;
        }
        :host>*{
            padding: 8px;
        }
    </style>
    <oda-checkbox ~if="item" icon-size="18" ::value="item.selected"></oda-checkbox>
    <div class="label" :title="item?.label">{{item?.label}}</div>`,
    properties: {
        item: Object
    }
});