(class extends odaAction {
    static get properties() {
        return {
            allowAccess: 'A',
            icon: 'icons:add',
            label: 'Add action',
            category: 'developer',
        };
    }
    async execute() {
        const control = await ODA.createComponent(this.resolveUrl('add-action-dialog.html'));
        const { name, label, area, type } = await ODA.showDialog(control, { title: 'New action' });
        let path = `/${area}`;
        if (area === 'server') {
            switch (type) {
                case 'task': path += '/tasks'; break;
                case 'method': path += '/methods'; break;
                case 'trigger': path += '/triggers'; break;
            }
        }
        const actionsRoot = await this.$item.getFolder(path, true);
        /** * @type odaFolder*/
        const actionFolder = await actionsRoot.createFolder(name, true);
        let actionName = '';
        switch (type) {
            case 'button': actionName = 'odaButton'; break;
            case 'service': actionName = 'odaService'; break;
            case 'page': actionName = 'odaPage'; break;
            case 'view': actionName = 'odaView'; break;
            case 'method':
            default: actionName = 'odaAction'; break;
        }
        const jsBody =
            `(class extends ${actionName} {

    static get properties() {
        return {
            label: '${label}',
            allowAccess: 'A',
            allowUse: true,
            allowExport: false,
            allowArrayContext: false,
            icon: 'icons:settings-applications',
        };
    }
}).register();`;
        await actionFolder.saveFile(jsBody, name + '.js', true);
        if (area === 'client' && ['view', 'page'].includes(type)) {
            const htmlBody =
                `<script src="/web/core/oda.js"></script>
<oda-module id="oda-${name}">
    <template>
        <style>
            
        </style>
        
    </template>
    <script>
        ODA({
            is: 'oda-${name}',
            properties:{
                
            },
            ${type === 'page' ? `show() { document.body.appendChild(this.host);}` : ''}
        })
    </script>
</oda-module>`;
            await actionFolder.saveFile(htmlBody, name + '.html', true);
        }
        if (!window.location.href.includes('/client/app-studio/')) {
            actionFolder.open('app-studio');
        }
    }
}).register();