export default odaPage({
    props: {
        label: 'App studio',
        category: 'developer',
        icon: 'icons:build',
        allowAccess: 'A',
        allowUse: function () {
            let contextFolder = this.$owner instanceof odaFolder ? this.$owner : this.$owner.$folder;
            if (!contextFolder || !(contextFolder instanceof odaFolder)) {
                return false;
            }
            return true;
        }
    }
})
// <link rel="import" href="./app-studio-file-tree.html">
//     <link rel="import" href="/web/lib/app-layout/app-layout.html">
//     <link rel="import" href="/web/tools/app-studio-toolbox/app-studio-toolbox.html">
//     <link rel="import" href="/web/lib/scroll-box/scroll-box.html">
ODANT({
    is: "app-studio",
    imports: [
        "@oda/button",
        "@oda/app-layout",
        "@lib/item-node",
        "@oda/scroll-box"

    ],
    extends: 'oda-app-layout',
    template: /*html*/`
    <div slot="title" class="horizontal bold dark flex" style="justify-content: center; align-self: center; padding: 2px;">
    <oda-item-node hide-prefix allow-expand :size="iconSize*.75" class="no-flex" :context-item.prop="contextItem"></oda-item-node>
    </div>
    <div slot="left-drawer" class="vertical flex" style="overflow: hidden;">
        <oda-app-studio-file-tree ::focused-item.prop="focusedItem" :context-item.prop="contextItem"></oda-app-studio-file-tree>
    </div>
    <div slot="top" class="flex horizontal header" style="align-items: center; max-width: 100%;">
        <oda-scroll-box>
            <oda-app-studio-tab ~for="(page, i) in pages" key="page.fullId" :page.prop="page" :idx="i" :class="{content: pageIndex === i}" :style="{ borderBottom: \`2px solid \${pageIndex === i ? 'blue' : 'transparent'}\`}" @tap.stop="focusedItem = page" :size="iconSize"></oda-app-studio-tab>
        </oda-scroll-box>
        <oda-button ~show="pages.length" icon="icons:close" :size="iconSize" @tap.stop="pages.splice(0); render()" class="header"></oda-button>
    </div>
    <div slot="main" class="vertical flex" :style="{background: \`url(\${contextItem && contextItem.url + '/~/background.jpg'})\`}">
        <iframe ~for="(page, i) in pages" :key="i" ~show="i === pageIndex" class="flex" :src="page.url+'/~/client/form/'"></iframe>
    </div>
    <div slot="right-drawer" class="vertical flex" style="overflow: hidden;">
        <oda-app-studio-toolbox ref="toolbox" :context-item.prop="contextItem || focusedItem"></oda-app-studio-toolbox>
    </div>
    `,
    props: {
        showTitle: true,
        modal: false,
        pages: {
            default: [],
            watch(n) {
                sessionStorage.setItem(`${this.settingsId}/pages`, JSON.stringify(n));
            }
        },
        contextItem: {
            type: Object,
        },
        focusedItem: {
            type: Object,
            watch(n) {
                if (n) {
                    if (n instanceof odaFolder)
                        return;
                    this.pageIndex = this.pages.findIndex(i => i === n);
                    if (this.pageIndex < 0) {
                        this.pages.push(n);
                        this.pageIndex = this.pages.length - 1;
                    }

                }
                else {
                    this.pageIndex = -1;
                }
            }
        },
        pageIndex: {
            default: -1,
            watch(n) {
                if (~n) {
                    sessionStorage.setItem(`${this.settingsId}/index`, String(n));
                }
            }
        },
        files: [],
    },
    listeners: {
        'close-tab': '_closeTab'
    },
    // attached() {
    //     let pages = sessionStorage.getItem(`${this.settingsId}/pages`);
    //     if (pages) {
    //         pages = JSON.parse(pages);
    //         if (pages.length) {
    //             this.pages = pages;
    //         }
    //     }
    //     const index = sessionStorage.getItem(`${this.settingsId}/index`);
    //     if (index) {
    //         this.index = Number(index) || 0;
    //     }
    // },
    _onDragOver(e) {
        e.preventDefault();
        const dt = e.dataTransfer;
        if (dt.types.includes('odant/oda-component')
        ) {
            if (e.ctrlKey) {
                dt.dropEffect = 'copy';
            } else {
                dt.dropEffect = 'move';
            }
        } else {
            dt.dropEffect = 'none';
        }
    },
    _onDrop(e) {
        console.log('drop: ', e);
    },
    _openFile(file) {
        if (!file || file instanceof odaFolder) return;
        let idx = this.pages.findIndex(f => f.item === file.item);
        if (!~idx) {
            this.pages.push({ item: file });
            this.listen(file, 'delete', '_closeTabByFile');
            idx = this.pages.length - 1;
        }
        this.pageIndex = idx;
    },
    _clearFilter() {
        this.filter = '';
    },
    _closeTabByFile(file) {
        this.unlisten(file, 'delete', '_closeTabByFile');
        const idx = this.pages.findIndex(p => p.item.fullId === file.fullId);
        this._closeTab({ detail: { src: idx } });
    },
    _closeTab(e) {
        let idx = e.detail.src.idx;
        this.pages.splice(idx, 1);
        // this.pageIndex = -1;
        while (idx >= this.pages.length)
            idx--;
        this.pageIndex = idx;
        this.focusedItem = this.pages[idx];
        this.render();
    },
    _getOffset(screenWidth, wx) {
        if (wx > screenWidth) return wx - (wx - screenWidth);
        if (wx < 0 && wx > (screenWidth * -1)) return (screenWidth * -1);
        return 0;
    },
});