const modes = {
    js: 'javascript',
    oml: 'xml',
    ocl: 'xml',
    xqi: 'xml',
    xq: 'xquery'
};

export default odaView({
    props: {
        category: 'developer',
        label() {
            if (this.$owner instanceof odaFile)
                return this.$owner.ext.toUpperCase() + ' editor';
            return 'XML editor';
        },
        icon() {
            if (this.$owner instanceof odaFile)
                return this.$owner.icon;
            return 'files:xml';
        },
        allowSave: true,
        allowButtons: false,
        allowUse() {
            if (this.$owner instanceof odaFolder || this.$owner instanceof odaHost)
                return false;
            return true;
        }
    }
})

ODANT({
    is: 'code-editor',
    imports: [
        '@oda/ace-editor',
        '@oda/button',
        '@oda/splitter'
    ],
    template: /*html*/`
    <script src="/web/core/item/client/views/code-editor/sax-min.js"></script>
    <script src="/web/core/item/client/views/code-editor/oda-editor.js"></script>
    <oda-button slot="top-right" :icon-size="iconSize" ::toggled="wrap" icon="editor:wrap-text" title="Wrap text" allow-toggle></oda-button>
    <oda-ace-editor ::wrap="wrap" :mode="mode" ::value="value" ref="editor"></oda-ace-editor>
    <iframe ref="demoFrame" slot="right-drawer" :hidden="!allowDemo" frameborder="0" style="width: 100%; height: 100%;"></iframe>
    `,
    props: {
        iconSize: 24,
        wrap: true,
        contextItem: {
            type: Object,
            async set(item, old) {
                if (old) {
                    // old.removeEventListener('after-save', _itemChanged);
                    this.listen('event', '_itemChanged', { target: old});
                }
                if (item) {
                    let data;
                    if (item instanceof odaFile) {
                        this.mode = modes[item.ext] || item.ext;
                        data = await item.load();
                    }
                    else if (item instanceof odaItem) {
                        this.mode = 'xml';
                        data = await item.load({ format: 'xml' });
                    }
                    if (typeof data === 'object')
                        this.value = JSON.stringify(data, null, '\t');
                    else if (this.mode === 'xml') {
                        this._parseXML(data).then(res => {
                            this.value = res;
                        })
                    }
                    else {
                        this.value = data;
                    }
                    if (this.value && item.ext === 'html') {
                        // item.addEventListener('after-save', _itemChanged);
                        this.listen('event', '_itemChanged', { target: item});
                    }
                }
            }
        },
        allowDemo: Boolean,
        mode: '',
        value: {
            type: String,
            set(n) {
                if (this.contextItem) {
                    this.contextItem.data = n;
                    if (this.contextItem.ext === 'html') {
                        this.allowDemo = true;
                        this.rightDrawer = {
                            icon: 'av:play-circle-outline',
                        };
                        if (!this.contextItem.data.includes('/web/core/oda.js')) {
                            this.$refs.demoFrame.src = `data:text/html;base64,${btoa(this.contextItem.data)}`;
                        } else {
                            const newDemoSrc = ODA.origin + this.contextItem.path + this._randomParam;
                            if (newDemoSrc !== this.$refs.demoFrame.src) {
                                this.$refs.demoFrame.src = newDemoSrc;
                            }
                        }
                    } else {
                        this.allowDemo = false;
                        this.rightDrawer = null;
                        this.$refs.demoFrame.src = '';
                    }
                }
            }
        },
        _randomParam: { // нужен для сброса кеша
            type: String,
            default: `?${String(Math.random()).slice(3)}`,
        },
        rightDrawer: Object
    },
    listeners: {
        drop: '_drop',
        dragover: '_dragOver'
    },
    _itemChanged(e) {
        if (this.contextItem && e.source === 'after-save') {
            this._randomParam = `?${String(Math.random()).slice(3)}`;
            this.$refs.demoFrame.src = ODA.origin + this.contextItem.path + this._randomParam;
            this.contextItem.data = this.value; //иначе iframe не перезагружается O_o
        }

    },
    async _parseXML(xml) {
        if (!window.sax) {
            return new Promise(resolve => {
                const script = document.createElement('script');
                script.src = '/web/oda/components/editors/xml-editor/sax-min.js';
                script.onload = () => resolve(this._parseXML(xml));
                document.head.appendChild(script);
                console.log();
            });
        }
        return new Promise((resolve) => {
            if (!xml)
                this.value = '';
            try {
                const parser = sax.parser(true);
                parser.level = 0;
                let res = ``;
                parser.indent = () => {
                    res += ' '.repeat(parser.level * 4);
                };

                function entity(str) {
                    return str.replace(/"/g, '&quot;')
                }

                parser.onprocessinginstruction = function (e) {
                    res += `<?${e.name} ${e.body}?>`;
                };

                parser.onopentag = function (tag) {
                    this.indent();
                    res += ('<' + tag.name);

                    for (let i in tag.attributes) {
                        res += ' ' + i + '="' + entity(tag.attributes[i]) + '"';
                    }
                    if (tag.isSelfClosing)
                        res += '/>\n';
                    else {
                        res += ('>\n');
                        parser.level++;
                    }
                };

                parser.ontext = ontext;
                parser.ondoctype = ontext;

                function ontext(text) {
                    this.indent();
                    res += (text);
                }

                parser.onclosetag = function (tag) {
                    if (!parser.tag.isSelfClosing) {
                        parser.level--;
                        this.indent();
                        // if (parser.level === 0){
                        //     res+=`\n`;
                        // }

                        res += '</' + tag + '>\n';
                        // if (parser.level<1)
                        //     res+='\n'
                    }
                };

                parser.oncdata = function (data) {
                    this.indent();
                    res += '<![CDATA[' + data + ']]>';

                };

                parser.onend = () => {
                    resolve(res);
                };

                parser.write(xml).close();
            }
            catch (e) {
                console.error(e);
                return xml;
            }
        })
    },
    _dragOver(e) {
        e.preventDefault();
    },
    _drop(e) {
        if (this.mode !== 'html') {
            return;
        }
        let jsonData = e.dataTransfer.getData('json/data');
        if (!jsonData) {
            return;
        }

        let data = JSON.parse(jsonData);

        let _getLinkHeatchery = (docRoot) => {
            let anLink = docRoot.querySelector('link');
            if (anLink) {
                return anLink.parentNode || docRoot;
            }

            return docRoot.querySelector('head') || docRoot;
        };

        let _isLinkExists = (linkHeatchery, link) => {
            let _href = link.getAttribute('href');
            let links = Array.from(linkHeatchery.querySelectorAll('link'));
            return links.find(
                (item) => {
                    return item.getAttribute('href') === _href;
                }
            );
        };

        let _getBeforeElment = (heatchery, tagName) => {
            let _tagName = tagName.toLowerCase();
            let _lastElement;
            for (let i = heatchery.children.length - 1; i > -1; i--) {
                if (heatchery.children[i].nodeName.toLowerCase() === _tagName) {
                    return _lastElement;
                }

                _lastElement = heatchery.children[i];
            }
        };

        if (data.location) {
            ODA.loadLink(data.location.base + data.location.index).then(
                (link) => {
                    if (!link.import) {
                        return;
                    }

                    let componentModul = link.import.body.querySelector('dom-module');

                    if (componentModul) {
                        this.$refs.editor.insertValueIntoCursor((data.location.pattern || `<${componentModul.id}></${componentModul.id}>`) + "\n");
                    }
                    let _document = (new DOMParser).parseFromString(
                        this.value,
                        "text/html"
                    );

                    // append link

                    let linkHeatchery = _getLinkHeatchery(_document);
                    if (!_isLinkExists(linkHeatchery, link)) {
                        let anLink = linkHeatchery.querySelector('link');

                        if (anLink) {
                            linkHeatchery.insertBefore(
                                link,
                                _getBeforeElment(linkHeatchery, 'link')
                            );
                        } else {
                            linkHeatchery.insertBefore(link, linkHeatchery.firstChild);
                        }
                    }

                    // detect parse errors
                    let err;
                    if ((err = _document.querySelector('parsererror'))) {
                        ODA.notify(err);
                    } else {
                        let html = (_document.doctype)
                            ? (new XMLSerializer()).serializeToString(_document.doctype) + "\n" + _document.querySelector('html').outerHTML
                            : _document.querySelector('head').innerHTML.trim() + "\n" + _document.querySelector('body').innerHTML;

                        this.value = html;
                    }
                }
            ).catch(console.log);
        }
    }
})