export default odaView({
    props: {
        label: "File explorer",
        defaultViewName: 'odant-file-explorer',
        category: 'system',
        allowAccess: 'R',
        allowUse: function () {
            return (this.$owner.typeName !== 'file');
        },
        icon: 'enterprise:folder-search'
    }
});

ODANT({
    is: 'file-explorer',
    imports: [
        // '@lib/file-tree',
        // '@lib/file-tile',
        '/web/core/folder/lib/file-tree/file-tree.js',
        '/web/core/folder/lib/file-tile/file-tile.js',
        '@oda/button'
    ],
    // extends: 'odant-view',
    template: /*html*/`
        <style>
            :host {
                @apply --vertical;
                height: 100%;
            }
            .right-pane {
                @apply --flex;
                @applay --vertical;
                overflow: hidden;
                border-right: 1px solid gray;
            }
            .main-pane {
                flex: 9;
            }
        </style>
        <div class="horizontal flex center between" style="flex: 0; align-self: start;" slot="control-panel">
            <div class="horizontal flex">
                <oda-button :disabled="_getGoBackDisabled(_historyIndex)" icon="icons:arrow-back" title="Go back" @tap="_goBack"></oda-button>
                <oda-button :disabled="_getGoForwardDisabled(_historyIndex)" icon="icons:arrow-forward" title="Go forvard" @tap="_goForward"></oda-button>
                <oda-button :disabled="_getGoUpDisabled(focusedItem, contextItem)" icon="icons:reply" title="Go up" @tap="_goUp"></oda-button>
            </div >
            <oda-button icon="icons:file-upload" @tap="_upload" ></oda-button>
        </div >
        <odant-file-tree slot="left-drawer" items-list="$$folders" :context-item="contextItem" :icon-size="iconSize" ::focused-node="focusedNode" ::focused-item="focusedItem" allow-focus></odant-file-tree>
        <odant-file-tile ::context-item="focusedNode" :icon-size="Math.round(iconSize * 1.33)" items-list="$$folders, $$files"></odant-file-tile>
    `,
    props: {
        iconSize: 40,
        focusedNode: {
            type: Object,
            watch(n) {
                if (n) {
                    this.focusedItem = n.$item;
                }
            }
        },
        focusedItem: {
            type: Object,
            // notify: true,
            watch(n) {
                if (!this._history || !n) return;
                if ((this._historyIndex < 0) || this._history[this._historyIndex].path !== n.path) {
                    this._history.splice(this._historyIndex + 1, this._history.length - this._historyIndex);
                    this._history[this._historyIndex + 1] = n;
                    this._historyIndex += 1;
                }
            }
        },
        _dataSet: [],
        _historyIndex: -1,
        _history: []
    },

    observers: [
        async function _setFocusedItem(contextItem) {
            if (contextItem) {
                this.focusedNode = {$item: await contextItem.$folder, items: [{}]};
            }
        }
    ],

    _getGoBackDisabled(_historyIndex) {
        if (!this._history || this._history.length === 0) return true;
        return _historyIndex < 1;
    },
    _getGoForwardDisabled(_historyIndex) {
        if (!this._history || this._history.length === 0) return true;
        return _historyIndex >= this._history.length - 1;
    },
    _getGoUpDisabled(focusedItem, contextItem) {
        return !focusedItem || focusedItem.path === '~' || focusedItem === contextItem;
    },
    _goBack() {
        this._goToHistoryIndex(this._historyIndex - 1);
    },
    _goForward() {
        this._goToHistoryIndex(this._historyIndex + 1);
    },
    async _goUp() {
        if (!this.focusedItem || this.focusedItem.path === '~' || this.focusedItem === this.contextItem) return;
        const ownerFolder = await this.focusedItem.$owner;
        if (ownerFolder) {
            this.focusedNode = {$item: ownerFolder};
        }
    },
    async _upload(e) {
        // if (this.contextItem && this.contextItem.$folder) {
        //     const res = await this.contextItem.$folder.executeAction('upload');
        // }
    },
    _goToHistoryIndex(historyIndex) {
        if (!this._history || (this._history.length === 0) || isNaN(Number(historyIndex)) || historyIndex === this._historyIndex) return;
        if (this._history[historyIndex]) {
            this._historyIndex = historyIndex;
            this.focusedNode = { $item: this._history[historyIndex]};
        }
    }
});