export default odaView({
    props:{
        label: 'XQuery',
        icon: 'xquery'
    }
})

ODANT({
    is: 'odant-xquery',
    imports: [
        '@oda/button',
        '/web/core/item/lib/item-node/item-node.js',
        '/web/core/item/client/views/xquery/sources.js',
        '/web/core/item/client/views/xquery/tools.js',
        '@oda/ace-editor',
        '@oda/xml-editor',
        '@oda/splitter'
    ],
    template: /*html*/ `
    <div slot="left" class="vertical flex" style="overflow: hidden;">
        <oda-xquery-sources :context-item="contextItem" ::focused-item="focusedItem"></oda-xquery-sources>
    </div>

    <div slot="top-left" class="flex horizontal header">
        <div class="flex horizontal">
            <div ~for="(page, i) in pages" class="horizontal no-flex" ~class="{content: pageIndex === i}" style="align-items: center; cursor: pointer; max-width: 150px; overflow: hidden;" :style="_getStyle(i)">
                <oda-item-node hide-prefix class="flex" @tap.stop="focusedItem = page" :context-item="page" :icon-size="16" style="overflow: hidden; margin-left: 4px;"></oda-item-node>
                <oda-icon v-if="i>0" icon="icons:close" :size="iconSize * .75" :idx="i" @tap.stop="_closeTab" style="padding: 4px;"></oda-icon>
            </div>
        </div>
        <oda-button v-show="pages.length" icon="icons:close" :size="iconSize" @tap.stop="pages.splice(1); render()" class="header"></oda-button>
    </div>
    <div slot="service-bar" class="horizontal flex" style="justify-content: flex-end; padding: 0px 8px;">
        <oda-button :size="iconSize * 2" allow-toggle ::toggled="run" style="border-radius: 50%" :style="{backgroundColor: run?'red':'green', marginTop: -iconSize+'px'}" :icon="run?'av:pause-circle-outline':'av:play-circle-outline'" title="Execute" ></oda-button>
    </div>
    <div slot="top-right">
        <oda-button :size="iconSize" ::toggled="wrapEditor" icon="editor:wrap-text" title="Wrap text" allow-toggle></oda-button>
    </div>
    <div class="flex vertical" style="overflow: hidden;">
        <oda-xquery-panel ref="panel" v-for="(item, i) in pages" :context-item="item" v-show="i === pageIndex" :focused="i === pageIndex" :index="i" :wrap-editor="wrapEditor"></oda-xquery-panel>
    </div>
    <div slot="right-drawer" style="overflow: hidden;">
        <oda-xquery-tools :context-item="focusedItem"></oda-xquery-tools>
    </div>`,
    props: {
        iconSize: Number,
        pages: [],
        pageIndex: -1,
        wrapEditor: false,
        run: {
            default: false,
            async set(e){
                if (e){
                    const p = this.$refs.panel.find(i=>i.index === this.pageIndex);
                    await p.execute();
                    this.run = false;
                }
            }
        },
        contextItem:{
            set(e){
                if (e){
                    this.focusedItem = e;
                }
            }
        },
        focusedItem:{
            set(e){
                if (e){
                    this.pageIndex = this.pages.indexOf(e);
                    if (this.pageIndex<0){
                        this.pages.push(e);
                        this.pageIndex = this.pages.length-1;
                    }
                }
            }
        }
    },
    _getStyle(i){return { borderBottom: `2px solid ${this.pageIndex === i ? 'blue' : 'transparent'}`}}
})

ODA({
    is: 'oda-xquery-panel',
    template: /*html*/`
    <style>
        :host{
            @apply --flex;
            @apply --vertical;
        }
    </style>
    <div class="vertical" :style="{height: h+'px'}">
        <oda-ace-editor ref="editor" :wrap="wrapEditor" class="flex" mode="xquery" ::value="query"></oda-ace-editor>
        <oda-splitter ::height="h" :align="'horizontal'"></oda-splitter>
    </div>
    <oda-xml-editor :wrap="wrapEditor" class="flex" readonly  :xml="result"></oda-xml-editor>`,
    props:{
        focused:{
            default: false,
            set(e){
                if (e){
                    this.async(()=>{
                        this.$refs.editor.focus();
                    })
                }
            }
        },
        h: 200,
        result: '',
        query: '*',
        contextItem: {},
        wrapEditor: false,
    },
    async execute(){
        this.$refs.editor.focus();
        try{
            this.result = await this.contextItem.XQuery(this.query, {format: 'xml'});
        }
        catch (e) {
            this.result = e;
        }
        finally {
            this.$refs.editor.focus();
        }
    },
    keyBindings:{
        F9: function(e){
            this.execute();
        }
    }
})
