import '../item-icon/item-icon.js';
import '../item-menu/item-menu.js';
ODA({
    is: 'odant-action-bar',
    template: /*html*/
`<style>
    :host {
        @apply --horizontal;
        @apply --no-flex;
        align-items: center;
        margin-right: 4px;
    }
    :host>div:active {
        @apply --active;
    }
    :host>div:hover{
        @apply --active;
        opacity: 1;
    }
    :host>div[active], div:active{
        @apply --active;
        opacity: 1;
        outline: 1px solid rgba(0,0,0,0.5) !important;
    }
    :host>div{
        cursor: pointer;
        opacity: .8;
        margin-right: 4px;
        padding: 4px;
        align-items: center;
    }
</style>
<oda-button ~if="!queryMatches && actions.length" :size="size" :icon="icon" @tap="_showContextMenu" class="no-flex"></oda-button>
<div ~if="queryMatches || !actions.length" ~for="action in actions" class="horizontal no-flex" @tap="_onTap" :action="action">
    <odant-item-icon :context-item="action" :size="size" :title="action.label" class="no-flex"></odant-item-icon>
    <span ~if="allowLabels" class="label">{{action.label}}</span>
    <oda-button ~if="action.subMenu" :size="size" icon="icons:arrow-drop-down" :action="action" @tap="_dropDownTap"></oda-button>
</div>`,
    props: {
        contextItem: {
            type: Object
        },
        queryMatches: true,
        icon: 'icons:more-vert',
        actions: [],
        size: 24,
        allowLabels: false,
        groups: '*'
    },
    observers: [
        function _getActions(contextItem, groups) {
            try {
                if (contextItem && contextItem.getActions) {
                    this.debounce('get-actions', async () => {
                        const actions = await contextItem.getActions(groups);
                        actions.sort((a, b) => ((a.zIndex <= b.zIndex && a.name.toLowerCase() < b.name.toLowerCase()) ? -1 : 1));
                        this.actions.splice(0, this.actions.length, ...actions);
                    }, 300)
                }
                else {
                    this.actions.splice(0, this.actions.length);
                }
            }
            catch (e) {
                console.error(e, contextItem)
            }
        }
    ],
    _showMenuButton(queryMatches, actions) {
        return !queryMatches && actions.base.length > 1;
    },
    _showContextMenu(e) {
        e.stopPropagation();
        const menu = this.create('odant-item-menu', {
            contextItem: this.contextItem,
            groups: this.groups
        });
        menu.open({ parent: this, hideTitle: false });
    },
    _hasMenu(item) {
        return Array.isArray(item.subMenu) && item.subMenu.length > 0;
    },
    _getLabel(item, allowLabels) {
        if (allowLabels)
            return item && item.label;
    },
    async _dropDownTap(e) {
        e.stopPropagation();
        const currentAction = e.target.action;
        const title = await currentAction.label;
        const icon = await currentAction.icon;
        const items = await currentAction.subMenu
        const res = await ODA.showContextMenu({ title, icon, items });
        res.execute();
    },
    async _onTap(e) {
        e.stopPropagation();
        e.currentTarget.action.execute();
    }
});