import '/web/oda/components/menus/menu/menu.js';
import '/web/oda/components/buttons/button/button.js';
ODANT({
    is: 'odant-item-menu',
    extends: 'oda-menu',
    props: {
        collapseLimit: 0,
        iconSize: 24,
        template: 'odant-item-menu-template',
        groups: '*',
        treeMode: false,
        flateLimit: 3,
        contextItem: {
            type: Object,
            async set(contextItem, o) {
                if (contextItem) {
                    if (Array.isArray(contextItem)) {
                        if (contextItem.length === 1)
                            contextItem = contextItem[0];
                        else
                            contextItem = new odaItemList({ items: contextItem });
                    }
                    this.title = 'loading...';
                    this.icon = CORE.waitIcon;
                    const actions = (await contextItem.getActions()).filter(a => !a.Hidden && (this.groups === '*' || a.typeName === this.groups));
                    this.title = contextItem.label;
                    this.icon = contextItem.icon;

                    let resultActions = actions.reduce((res, a) => {
                        const groupName = a.category || 'other';
                        let group = res.find(i => i.label === groupName);
                        if (!group) {
                            group = {};
                            group.hideIcon = true;
                            group.label = groupName;
                            group.folder = a.scriptFolder;
                            group.items = [];
                            group.menu = this;
                            group.is = 'odant-item-menu-group-template';
                            group.isCategory = true;
                            group.disabled = true;
                            res.push(group);
                        }
                        a.defaulticon = 'icons:eject';
                        a.menu = this;
                        group.items.push(a);
                        return res;
                    }, [])
                        .sort((a, b) => (a.label || a).localeCompare(b.label || b))
                        .reduce((res, g) => {
                            if(!this.treeMode) res.push(g);
                            g.items.sort((a, b) => ((a.label.toLowerCase() < b.label.toLowerCase()) ? -1 : 1)).sort((a, b) => {
                                if (a.zIndex !== b.zIndex)
                                    return a.zIndex < b.zIndex ? 1 : -1;
                                return 0;
                            });
                            if((this.treeMode && g.items.length < this.flateLimit) || !this.treeMode) {
                                res.push(...g.items);
                                delete g.items;
                            } else if((this.treeMode && g.items.length > this.flateLimit - 1)) {
                                res.push(g);
                                g.is = 'odant-item-menu-tree-group-template';
                                delete g.hideIcon;
                            }
                            return res;
                        }, []);
                    this.items = this.treeMode
                     ? resultActions.sort((a, b) => a.isCategory && !b.isCategory ? 1 : -1)
                                    .sort((a, b) => !a.isCategory && !b.isCategory && (a.label || a).localeCompare(b.label || b))
                                    .sort((a, b) => a.isCategory && b.isCategory && (a.label || a).localeCompare(b.label || b))
                     : resultActions;
                }

            }
        }
        // opened: false
    },
    // listeners: {
    //     tap(e) {
    //         this.close();
    //         e.detail.value.execute(this.parent);
    //     }
    // }
});

ODA({
    is: 'odant-item-menu-template',
    template: /*html*/`
    <style>
        :host {
            @apply --horizontal;
            @apply --flex;
            align-items: center;
            @apply --content;
            text-overflow: ellipsis;
        }
        :host > oda-button {
            opacity: 0.25;
            margin-left: 4px;
        }
        :host > oda-button:hover{
            transition: opacity 0.25s;
            opacity: 0.8;
        }
        :host>oda-icon{
            padding: 4px;
        }
    </style>
    <span class="label flex" :title="item?.label">{{item?.label}}</span>
    <span class="label no-flex" style="font-size: x-small; opacity: .5">{{item?.shortcut}}</span>
    <oda-button ~if="item?.typeName === 'view'"  @tap.stop="_open" icon="icons:open-in-new"></oda-button>
    <!--<oda-button ~if="item?.access?.allowAdmin"  @tap.stop="_tap" icon="icons:build"></oda-button>-->
    <oda-button @tap.stop="_tap" icon="icons:build"></oda-button>`,
    props: {
        iconSize: {
            shared: true
        },
        item: {
            type: Object,
            shared: true
        },
    },
    _tap(e) {
        this.item.scriptFolder.open('app-studio');
    },
    _open(e) {
        this.item.execute();
    },
    listeners: {
        tap(e) {
            this.item.execute(this);
        }
    }

});

ODA({
    is: 'odant-item-menu-group-template',
    template: /*html*/`
    <style>
        :host {
            background-color: silver;
            @apply --horizontal;
            @apply --flex;
            cursor: default;
            align-items: baseline;
            padding: 2px 4px;
        }
    </style>
    <label class="flex" style="font-size: small; color: white">{{item?.label}}</label>`,
    props: {
        item: {},
        iconSize: 24
    },
    listeners: {
        tap(e) {
            e.stopPropagation();
        }
    }
});

ODA({
    is: 'odant-item-menu-tree-group-template',
    template: /*html*/`
    <style>
        :host {
            @apply --horizontal;
            @apply --flex;
            cursor: pointer;
            align-items: center;
            text-overflow: ellipsis;
            padding: 7px 0;
            background-color: whitesmoke;
        }
    </style>
    <span class="label bold flex" :title="_getTitle()">{{item?.label}}</span>`,
    props: {
        item: Object,
    },
    listeners: {
        tap(e) {
            e.stopPropagation();
            this.domHost._open(e);
        }
    },
    _getTitle() {
        if (this.item?.items?.length) {
            return this.item?.items.map(i => i.label).join(',\r\n');
        } else {
            return '';
        }
    }
});
