ODANT({
    is: 'odant-item-node',
    imports: '@lib/item-icon',
    extends: 'odant-item-icon',
    template: /*html*/`
    <style>
        :host {
            @apply --horizontal;
            @apply --no-flex;
            align-items: center;
            position: relative;
            cursor: pointer;
            overflow: visible;
            margin: 4px;
        }
        :host(:not([expanded])){
            cursor: default;
        }
        :host([expanded])>svg:hover {
            outline: 1px solid blue;
        }
        :host([active]){
            @apply --active;
        }
        :host oda-icon {
            opacity: .8;
            transition: all .3s;
        }
        :host oda-icon:hover {
            opacity: 1;
        }
        :host([expanded]) oda-icon {
            opacity: 1;
            transform: rotate(180deg);
        }
        :host div{
            overflow: hidden;
        }
        :host .next {
            @apply --horizontal;
            position: relative;
            align-items: center;
        }
        :host .label {
            display: flex;
            cursor: inherit;
        }
        odant-item-node{
            margin: 0px 4px !important;
        }
    </style>
    <style>{{_itemStyle}}</style>
    <style>
        :host label{
            font-size: {{iconSize/2}}px;
        }
        .access{
            font-weight: bolder;
            color: var(--content-color);
        }
    </style>
    <div ref="mainLabel" ~if="!expanded" class="vertical flex" :style="_getStyle(contextItem)" :disabled="contextItem?.disabled">
        <span :style="{fontSize: prefSize + 'px', minHeight: prefSize + 'px'}">{{!hidePrefix && pref || ''}}</span>
        <div class="flex horizontal-center" :style="{ fontSize: labelSize + 'px', height: labelSize + 'px'}">
            <label class="label" ~text="label"></label>
            <label ~if="showName" class="label" style="opacity: .5; margin-left: 4px" ~text="contextItem?.name ? [contextItem.name] : ''"></label>
            <label ~if="contextItem?.isRef && (contextItem?.access.level >= 6)" class="access" ~text="accessLevel"></label>
        </div>
    </div>
    <div class="next" :class="{flex: expanded}" ~if="allowExpand && contextItem && contextItem.$owner" style="align-items: center; overflow: visible;" @tap.stop>
        <oda-icon ~if="!expanded"  :icon-size @tap.stop="expanded = !expanded" icon="icons:chevron-right" style="opacity: .5; cursor: pointer;" :style="{height: iconSize+'px'}"></oda-icon>
        <odant-item-node ref="next" class="flex"  ~if="expanded" :hide-icon="hideIcon" :hide-prefix="hidePrefix" :allow-expand="allowExpand" :admin-mode="adminMode" :icon-size style="align-items: center" :context-item="contextItem.$owner"></odant-item-node>
    </div>`,
    props: {
        currentItem: {
            type: Object,
            notify: true,
        },
        hidePrefix: false,
        adminMode: Boolean,
        item: {
            type: Object,
            watch(n) {
                this.contextItem = (n && (n.item || n.$item)) || null;
            }
        },
        showName: false,
        allowExpand: false,
        expanded: {
            default: false,
            reflectToAttribute: true,
            notify: true
        },
        label: '',
        pref: '',
        prefSize: {
            type: Number,
            get(){
                return Math.round(this.iconSize / 3);
            }
        },
        labelSize: {
            type: Number,
            get(){
                return Math.round((this.iconSize / 3) * 2);
            }
        },
        _itemStyle: {
            default: '',
            get() {
                if (this.contextItem) {
                    const size = this.iconSize / 2;
                    let style = '';
                    if (this.adminMode && this.contextItem instanceof odaClass && this.contextItem?.access?.letter) {
                        style += `
                        :host:after{
                            z-index: 1;
                            left: -${size / 3}px;
                            top: ${size}px;
                            // width: ${size}px;
                            // height: ${size}px;
                            font-weight: bold;
                            position: absolute;
                            content: '${this.contextItem.access.letter}';
                            font-size: ${size * .9}px;
                            text-align: center;
                            color: ${this.item ? this.contextItem.access.color : 'inherit'};
                            text-shadow: 1px 1px silver;
                        }`
                    }
                    return style;
                }
            }
        },
        _accessLevelMap: {
            type: Object,
            default: {
                0: 'None',
                1: '',
                2: 'R',
                3: 'RW',
                4: 'RWC',
                5: 'RWCD',
                6: 'Admin',
            }
        },
        accessLevel: ''
    },
    observers: [
        function _setFocusedItem(expanded, contextItem) {
            if (!contextItem) return;
            this.currentItem = expanded ? this.$refs?.next?.currentItem || contextItem.$owner || contextItem : contextItem;
        }
    ],
    async _itemChanged(e) {
        this.expanded = false;
        if (this.contextItem) {
            this.$super('odant-item-icon', '_itemChanged');
            this.label = await this.contextItem.label;
            this.accessLevel = this._getAccessLevelLabel();
            const getPrefix = async (item) => {
                switch (item.typeName) {
                    case 'base': return this.adminMode && item.baseCount || '';
                    case 'pack':
                    case 'class': return (await item.objectCount) || '';
                    case 'category':
                    case 'field': return item.ItemCount || '';
                }
            };
            const pref = await getPrefix(this.contextItem);
            if (typeof pref === 'number') {
                this.pref = (pref > 0) ? pref.toLocaleString() : '';
            } else {
                this.pref = pref || '';
            }

            this.adminMode = (this.contextItem?.access?.level >= 6) && this.item;
        }

    },
    listeners: {
        tap(e) {
            this.expanded = false;
        }
    },
    _getAccessLevelLabel(contextItem = this.contextItem) {
        if (contextItem.access) {
            return ` - <${this._accessLevelMap[contextItem.access.level]}>`;
        }
        return '';
    },
    _getStyle(item) {
        if (item) {
            let style = `color: ${item.color}; fill: ${item.fill};`;
            if (item instanceof odaDomain || item instanceof odaHost || item instanceof odaFolder) {
                style += `font-size: 110%; font-weight: bold;max-height: ${this.iconSize}px;`;
            }
            return style;
        }
    }
});