ODANT({
    is: 'odant-item-tree',
    imports: `@oda/tree,  @lib/item-node`,
    extends: 'oda-tree',
    props: {
        iconSize: {
            shared: true
        },
        idName: '$item',
        itemsList: String,
        expandNames: String,
        allowFocus: true,
        focusedNode: {
            watch(node) {
                this.focusedItem = node && node.$item;
            }
        },
        focusedItem: {
            type: Object,
            async watch(item) {
                if (item) {
                    const owners = [];
                    let owner = item.$owner;
                    while (owner) {
                        owners.unshift(owner);
                        owner = owner.$owner;
                    }
                    for (let i = 0; i < owners.length; i++) {
                        const owner = owners[i];
                        const node = this.find(i => i && i.$item === owner);
                        if (node && !node.$expanded) {
                            node.$expanded = true;
                            node.items = await this._beforeExpand(node);
                            // this._refresh();
                            // await this._beforeExpand(node); // to wait the refreshing of tree's items
                        }
                    }
                    // this._refresh();
                    // this.async(() => {
                        this.selectItem(this.find(i => {
                            return i.$item === item;
                        }))
                    // }, 50)
                }
                else {
                    this.focusedNode = null;
                    this.clearSelection();
                }
            }
        },
        defaultTemplate: 'odant-item-node'
    },
    observers:[
        async function refreshDataset(contextItem, itemsList) {
            if (this.hideRoot) {
                const obj = { $item: contextItem };
                obj.items = await this._beforeExpand(obj);
                this.dataSet = [obj];
            } else {
                this.dataSet = [{$item: contextItem}];
            }
            this.focusedItem = contextItem;
        }
    ],
    async _beforeExpand(node, deep = 1) {
        let res = this.itemsList.split(',').map(async src => {
            src = src.trim();
            return node.$item && (await node.$item[src] || []);
        });
        res = (await Promise.all(res)).flat();
        res = res.map(async i => {
            const obj = { $item: i, items: [] };
            if (deep) {
                const ch = await this._checkChildren(obj);
                if (ch) {
                    obj.items = [{}];
                    if (i && (this.expandNames.includes(i.name)))
                        obj.$expanded = 'expanding';
                }
            }
            return obj;
        });
        res = (await Promise.all(res)).filter(this._filter, this).sort(this._sort_);
        return res;
    },
    async  _checkChildren(node) {
        const items = await this._beforeExpand(node, 0);
        return items.length > 0;
    },
    _filter(item) {
        return true;
    },
    _sort_(a, b) {
        a = a?.$item;
        b = b?.$item;
        if (a === undefined) return -1;
        if (b === undefined) return 1;
        return ((a.zIndex < b.zIndex) || (a.zIndex === b.zIndex && a.label && b.label && a.label.toLowerCase() < b.label.toLowerCase())) ? -1 : 1;
    }
});