import '/web/ext/transliteration/lib/browser/transliteration.js';
import '/web/oda/components/buttons/button/button.js';

const YANDEX_TRANSLATE_KEY = 'trnsl.1.1.20170815T113812Z.3867b739e14a8066.c581f62184ab231363d6af49ad885cdbcd9d158c';
const TRANSLATE_URL = 'https://translate.yandex.net/api/v1.5/tr.json/translate?key=' + YANDEX_TRANSLATE_KEY;

ODA({
    is: 'oda-label-to-name',
    template: /*html*/`
    <style>
        :host {
            @apply --vertical;
            min-width: 300px;
            padding: 8px;
        }
        label{
            margin: 2px 4px;
            transition: transform ease 0.25s;
            max-width: 50px;
        }
        label[disabled]{
            @apply --disabled;
        }
        input{
            padding: 2px 4px;
            min-height: 24px;
        }
        .container{
            @apply --horizontal;
            align-items: center;
            margin: 4px 8px;
        }
        .container>*{
            @apply --flex;
        }
        [disabled]{
            @apply --disabled;
        }
    </style>
    <div class="container">
        <label for="label">Label:</label>
        <input id="label" ref="label" ::value="label">
    </div>
    <div ~if="!hideName" class="container">
        <label for="name">Name:</label>
        <input id="name" ::value="name" ref="name" :disabled="!_toggled" style="opacity: initial;">
        <oda-button size="16" icon="image:edit" allow-toggle ::toggled="_toggled" class="no-flex"></oda-button>
    </div>
    `,
    props: {
        label: {
            type: String,
            set(label, o) {
                if (!this.toggled) {
                    this.debounce('translate', async () => {
                        if (label) {
                            try {
                                const res = await this._translate(label.trim());
                                this.name = res.trim();
                            } catch (e) {
                                this.name = label.trim();
                            }
                        }
                        else {
                            this.name = null;
                        }
                    }, 300)
                }
            }
        },
        hideName: false,
        name: {
            type: String,
            set(name, o) {
                this.debounce('name-changed', () => {
                    if (name) {
                        if (!this._toggled) name = name.toLowerCase();
                        name = name.split(' ').map((s, i) => {
                            if (i === 0) {
                                if (s === 'the')
                                    return null;
                                return s;
                            }
                            if (s.length < 7)
                                return s;
                            return s.substring(0, 4);
                        });
                        name = name.join('-');
                        // while (name.includes('--')) {
                        //     name = name.replace(/-/g, '--');
                        // }
                        name = name.replace(/-{2,}/g, '-');
                        name = name.replace(/(^\d)/, '_$1');
                        name = name.replace(/\./g, '');
                        let last = name[name.length - 1];
                        if (last !== ' ' && last !== '-')
                            last = '';
                        this.name = slugify(name) + last;
                    }
                }, 100); 
            }
        },
        _toggled: {
            type: Boolean,
            set(n, o) {
                if (!n) {
                    this.$refs.label.focus();
                    this.$refs.label.selectionStart = 0;
                    this.$refs.label.select();
                }
                else {
                    this.$refs.name.focus();
                    this.$refs.name.selectionStart = 0;
                    this.$refs.name.select();
                }
            }
        },
        autofocus: false
    },
    attached() {
        this.async(() => {
            if (this.autofocus) {
                this.$refs.label.focus();
                this.$refs.label.selectionStart = 0;
                this.$refs.label.select();
            }
        })
    },
    async _translate(text) {
        const url = encodeURI(TRANSLATE_URL + '&lang=' + ('en') + '&text=' + text);
        const request = await fetch(url);
        if (request.status === 200) {
            const json = await request.json();
            return json.text && json.text.length > 0 && json.text[0];
        }
    }
})