if (!globalThis.ODANT) {
    const prefixMap = {
        Name: 'name',
        Label: 'label',
        Type: 'type'
    }
    const configMap = {
        i: 'id',
        l: 'label',
        n: 'name',
        t: 'type',
        a: 'accessLevel',
        v: 'version',
    }
    const oldMOdulesExclude = [
        'demo', 'lib'
    ]
    const targetStack = [];
    const SERVER_MODE = typeof globalThis.process !== 'undefined';
    const DEFAULTORIGIN = 'https://current.odant.org';
    const proxyActionHandler = {
        get: (target, key) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (!prop || (typeof prop.value === 'function'))
                return target[key];
            return target.getAttr(key, target[key], prop.value && prop.value.type);
        },
        set: (target, key, value) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (prop && !prop.readOnly && (prop.set || prop.value) && target.$class) {
                // TODO
                if (!target.ROOT) {
                    let action;
                    if (target.$module) {
                        if (!target.$class.ROOT.$MODULES) {
                            target.$class.ROOT.$MODULES = [{ $MODULE: [{ id: target.$module.id, $ACTION: [] }] }];
                            action = this.$class?.ROOT?.$MODULES?.[0].$MODULE[0].$ACTION;
                        } else {
                            let module = this.$class?.ROOT?.$MODULES?.[0].$MODULE.find((m) => m.id === target.$module.id);
                            if (!module) {
                                const l = this.$class?.ROOT?.$MODULES?.[0].$MODULE.push({ id: target.$module.id, $ACTION: [] })
                                module = this.$class?.ROOT?.$MODULES?.[0].$MODULE[l - 1];
                            }
                            action = module.$ACTION;
                        }

                    } else {
                        if (!target.$class.ROOT.$ACTIONS) {
                            target.$class.ROOT.$ACTIONS = [{ $ACTION: [] }];
                        }
                        action = target.$class.ROOT.$ACTIONS[0].$ACTION;
                    }
                    action.push({ id: target.id });
                }
                // delete if equal default value
                target.setAttr(key, (target[key] !== value) ? value : undefined)
                target.$class.fireChanged();
            }
            else {
                target[key] = value;
            }
            return true;
        }
    }
    globalThis.RequestError = globalThis.RequestError || class RequestError extends Error {
        constructor(response, message) {
            super(message);

            this.errorCode = response.headers.get('odant-error-code');
            this.status = response.status;
            this.statusText = response.statusText;
            this.url = response.url;
        }
    };
    globalThis.ODANT = function (prototype) {
        const parents = (typeof prototype.extends === 'string'
            ? prototype.extends.replace(/ /g, '').split(',').map((e) => {
                if (globalThis[e])
                    return globalThis[e];
                throw Error(`Parent class '${e}' for inherit to '${prototype.is}' not found!`)
            })
            : [prototype.extends]).filter(Boolean)
        prototype.props = prototype.props || {};
        for (let key in prototype.props) {
            let prop = prototype.props[key];
            if (typeof prop === 'function')
                prop = (prop.name === key) ? { get: prop } : { type: prop };
            if (Array.isArray(prop))
                prop = { default: prop, type: Array };
            else if (prop === null || typeof prop !== "object")
                prop = { default: prop, type: prop ? prop.__proto__.constructor : Object };
            else if (Object.keys(prop).length === 0 || (!prop.get && !prop.set && prop.default === undefined && !prop.type))
                prop = { default: prop, type: Object };
            if (typeof prop.get === 'string')
                prop.get = prototype[prop.get];
            if (typeof prop.set === 'string')
                prop.set = prototype[prop.set];
            if (!prop.type && prop.default !== undefined)
                prop.type = prop.default.__proto__.constructor;
            prototype.props[key] = prop;
        }

        if (parents) {
            for (let parent of parents) {
                for (let key in parent.$prototype.props) {
                    const parentProp = parent.$prototype.props[key];
                    //if (parentProp) continue;
                    const targetProp = prototype.props[key];
                    if (targetProp) {
                        for (let i in parentProp) {
                            if (i === 'get' && 'default' in targetProp) continue;
                            if (i === 'default' && 'get' in targetProp) continue;
                            targetProp[i] = targetProp[i] || parentProp[i];
                        }
                    }
                    prototype.props[key] = targetProp || parentProp;
                }
                if (parents.indexOf(parent) === 0)
                    continue;
                for (let key in parent.$prototype) {
                    const targetMethod = Object.getOwnPropertyDescriptor(prototype, key);
                    if (targetMethod && targetMethod.value)
                        continue;
                    const parentMethod = Object.getOwnPropertyDescriptor(parent.$prototype, key);
                    if (parentMethod && (parentMethod.get || typeof parentMethod || typeof parentMethod.value === 'function'))
                        Object.defineProperty(prototype, key, parentMethod);
                }
            }
        }

        const body = `with (globalThis) {return class ${prototype.is || ''} extends (p || Object) {
            __cache__ = {}
            __customCache__ = {}
            __items__ = {}
            __deps__ = {}
            constructor(props){
                super(props);
                for (let p in this.constructor.defaults){
                    let val = this.constructor.defaults[p];
                    if (val && typeof val === 'object'){
                        if (Array.isArray(val))
                            val = Array.from(val);
                        else
                            val = Object.assign({}, val);
                    }
                    this.__cache__[p] = val;
                }
                for (let p in props){
                    let value = props[p];
                    const prop = this.props[p];
                    if (value !== undefined && prop){
                        if (prop.type === Boolean)
                            value = (value === 'true') || !!value;
                        else if (prop.type === Number)
                            value = +value;
                    }
                    this.__cache__[p] = value;
                }
                if (prototype.created)
                    prototype.created.call(this);
            }
            get props(){
                return prototype.props;
            }
        }}`;
        const cls = (new Function('p, prototype', body))(parents?.[0], prototype);
        cls.$prototype = prototype;
        if (prototype.is && !globalThis[prototype.is]) {
            globalThis[prototype.is] = cls;
        }
        cls.defaults = {};
        for (let key in prototype.props) {
            const prop = prototype.props[key];
            const parProp = parent?.$prototype?.props[key];
            if (prop.default !== undefined)
                cls.defaults[key] = prop.default;
            const description = { enumerable: !key.startsWith('_'), configurable: false };
            description.get = function () {
                const stack = targetStack ? [...targetStack].reverse() : [];
                for (let i of stack) {
                    if (i.target === this && i.prop === key)
                        continue;
                    let deps = this.__deps__[key];
                    if (!deps)
                        deps = this.__deps__[key] = [];
                    if (deps.find(d => {
                        return d.target === i.target && d.prop === i.prop;
                    })) continue;
                    deps.push(i);
                }
                // if (this.__target__ && this.__target__ !== key){
                //     let deps = this.__deps__[key];
                //     if (!deps)
                //         deps = this.__deps__[key] = [];
                //     if (!deps.includes(this.__target__))
                //         deps.push(this.__target__);
                // }
                if (prop.get && !(key in this.__cache__)) {
                    targetStack.push({ target: this, prop: key })
                    //const before = this.__target__;
                    // this.__target__ = key;
                    this.__cache__[key] = prop.get.call(this);
                    targetStack.pop();
                    // this.__target__ = before;
                    const deps = this.__deps__[key] || [];
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                }
                return this.__cache__[key];
            }
            description.set = prop.set ?
                function (val) {
                    this.__cache__[key] = val;
                    prop.set.call(this, val);
                    const deps = this.__deps__[key] || []
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                } : (prop.readOnly ? undefined :
                    function (val) {
                        this.__cache__[key] = val;
                        const deps = this.__deps__[key] || []
                        for (let d of deps) {
                            delete d.target.__cache__[d.prop];
                        }
                    })
            Object.defineProperty(cls.prototype, key, description);
        }
        for (let key in prototype) {
            const d = Object.getOwnPropertyDescriptor(prototype, key);
            if (typeof d.value === 'function') {
                cls.prototype[key] = function (...args) {
                    return d.value.call(this, ...args);
                }
            }
            else if (d.get || d.set)
                Object.defineProperty(cls.prototype, key, d);
        }
        const prefix = prototype.props?.prefix?.default;
        if (prefix) {
            ODANT.__factory__[prefix] = ODANT.__factory__[prefix] || cls;
        }
        return cls;
    }
    ODANT.waitIcon = 'odant:spin';
    ODANT.getContextFromUrl = function (url) {
        if (url) {
            url = decodeURI(url);
            url = url.split('?')[0];
            url = url.split('#')[0];
            let i = url.indexOf('/api/');
            if (i < 0)
                return null;
            url = url.substring(i + 4);
            i = url.lastIndexOf('/~');
            if (i < 0) {
                url = url.split('/').filter(s => {
                    return s.indexOf(':') === 1;
                }).join('/');
            }
            else {
                url = url.substring(0, i);
            }

            return url;
        }
        return '';
    }
    ODANT.__factory__ = {};
    ODANT.getFactory = function (prefix) {
        return ODANT.__factory__[prefix];
    }
    if (!SERVER_MODE && !document.location.host.startsWith('localhost'))
        ODANT.URL = new URL(document.location.origin);
    else
        ODANT.URL = new URL(DEFAULTORIGIN);

    ODANT({
        is: 'odaAccess',
        props:{
            level: Number,
            label(){
                switch (this.level){
                    case 6: return 'admin';
                    case 5: return 'delete';
                    case 4: return 'create';
                    case 3: return 'write';
                    case 2: return 'read';
                    case 1: return 'preview';
                    case 0: return 'none';
                    default: return 'unknown';
                }
            },
            letter(){
                return this.label[0].toUpperCase();
            }
        }
    })
    ODANT({
        is: 'odaIndexDataSet',
        props: {
            $index: Object,
            mask: String,
            filter: String,
        },
        get Items() {
            return (async () => {
                let data = await this.$index.request((this.filter) ? 'search_index' : 'xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    this.filter || '*/*');
    
                data = data ? data[Object.keys(data)[0]] : [];
    
                return data;
            })();
        },
        get metadata() {
            return this.$index.$$fields;
        },
        async getItems(from = 1, length = 100) {
            try {
                const data = await this.$index.request('xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    `subsequence(*/*, ${from}, ${length})`);
                return data[Object.keys(data)[0]];
            } catch (e) {
                console.error(e);
                return [];
            }
        },
        async getRowItem(row) {
            if (row) {
                let rows = Array.isArray(row) ? row : [row];
                if (rows.length === 0)
                    return Promise.resolve(null);
    
                const item = await this.$index.$class.getObject(rows[0].oid);
                if (rows.length === 1)
                    return item;
                return null;
                // TODO odaItemList
                //return new odaItemList({ $owner: this.$index.$class, $item: item, $$items: rows.map(i => { return i.oid }) });
            }
            return Promise.reject();
        }
    })
    console.log(ODANT.URL)
    ODANT({
        is: 'odaItem',
        props: {
            $part() {
                return this.$owner.$path;
            },
            id: String,
            typeName: String,
            $class() {
                // No $class in host
                return this.$owner?.$class;
            },
            $module() {
                return this.$owner?.$module;
            },
            $base() {
                // No $base in host
                return this.$owner?.$base;
            },
            $host() {
                return this.$owner.$host;
            },
            $owner: null,
            name() {
                return this.id;
            },
            label() {
                return this.name;
            },
            icon: 'odant:item',
            color: 'black',
            fill() {
                return this.color;
            },
            defaultViewName: 'editor',
            accessLevel: undefined,
            access: {
                category: 'system',
                get() {
                    if (!this.__cache__.accessLevel && this.$owner && this.$part && this.$part.id === 'ROOT')
                        return this.$owner.access;
                    return new odaAccess({level: this.__cache__.accessLevel});
                }
            },
            actionsPath() {
                return `web/${this.typeName}/client/*/*`;
            }
        },
        get $$items() {
            return this.cache('$$items', async () => {
                let items = [];
                for (let i in this) {
                    if (i.startsWith('$$') && i !== '$$items')
                        items.push(...(await this[i]));
                }
                return items;
            })
        },
        import(path) {
            return import(this.url + '/~/' + path);
        },
        async getActions($item = this) {
            return this.cache('$$actions' + $item?.typeName, async () => {
                if (!$item.config) await $item.getConfig?.();
                await $item?.$class?.body;
                const path = $item.actionsPath;
                const folders = await this.$folder.getFolders(path, {});
                let actions = folders.map(async i => {
                    if (oldMOdulesExclude.includes(i.name)) // костыль для старой структуры модулей
                        return undefined;
                    const module = { $folder: i, id: i.name, path: i.url, $owner: $item, $source: this };
                    let action;
                    try {
                        const result = await import(i.url + '/' + i.name + '.js');
                        action = new result.default(module);
                    }
                    catch (e) {
                        console.error(i.path, e)
                        action = new odaActionError(module);
                    }
                    action.category = action.category || i.$owner.name;
                    return action;
                })
                actions = (await Promise.all(actions));
                actions = actions.filter(Boolean);
                if (!(this.$owner instanceof odaModule)) {
                    const exports = await this.$base?.getActionsExp(this) || [];
                    console.dir(exports);
                    actions.push(...exports);
                }
                //todo фильтрация конечного списка перед выводом
                return actions.map(a => new Proxy(a, proxyActionHandler)).filter(a => a.active);
            });
        },
        async getActionsExp($item) {
            const actions = await this.getActions($item);
            return actions.filter(i => {
                return i.export;
            })
        },
        async findAction(name) {
            const actions = await this.getActions();
            return actions.find(i => i.name === name);
        },
        cache(key, callback) {
            let result = this.__customCache__[key];
            if (result === undefined) {
                if (typeof callback === 'function')
                    result = callback();
                else
                    result = callback;
                this.__customCache__[key] = result;
            }
            return result;
        }
    })
    ODANT({
        is: 'odaVirtualItem',
        extends: 'odaItem',
    })

    ODANT({
        is: 'odaFieldsOwner',
        extends: 'odaItem',
        get $$fields() {
            return this.cache('$$fields', async () => {
                let attrs;
                await this.body;
                let $ATTR = (this.METADATA || this.ROOT)?.$ATTR;
                if (this.$typeClass) {
                    await this.$typeClass.body;
                    attrs = $ATTR;
                    if (attrs) {
                        this.$typeClass.METADATA?.$ATTR.forEach((attr) => {
                            const a = attrs.find(a => a.Name === attr.Name);
                            if (!a) {
                                attrs.push(attr);
                            }
                            else {
                                for (const i in attr) {
                                    a[i] = a[i] || attr[i];
                                }
                            }
                        })
                    } else {
                        attrs = this.$typeClass.METADATA?.$ATTR;
                    }
                } else {
                    attrs = $ATTR;
                }
                const prom = (attrs || []).map(async (i) => {
                    const conf = { ROOT: i };
                    for (let p in i) {
                        const s = prefixMap[p];
                        if (s === undefined) continue;
                        conf[s] = i[p];
                    }
                    conf.$owner = this;
                    if (!i.Type) return new odaField(conf);

                    const typeCls = (await this.$base.findItem(i.Type)) || (await this.$host.findItem(i.Type));
                    if (!typeCls) return new odaField(conf);

                    conf.$typeClass = typeCls;
                    const url = `${typeCls.url}/~/fields/${!typeCls.isAbstract ? '' : 'links/'}extend.js`;
                    const imp = await import(url);
                    return new imp.default(conf);
                })
                return await Promise.all(prom);
            });
        }
    })
    ODANT({
        is: 'odaIndexesOwner',
        extends: 'odaItem',
        get $$indexes() {
            return this.cache('$$indexes', async () => {
                await this.body;
                let folders = await this.getFolders('index/*')
                if (!folders) return [];
                const indexes = folders.map(async (f) => {
                    // TODO временно через файл
                    const file = await f.getFile(f.name + '.xq');
                    if (!file) return null;
                    const result = await file.request('xquery', { format: 'json' }, '*');
                    const conf = { id: f.name, name: f.name, $owner: this };
                    if (result?.$XQUERY) {
                        conf.ROOT = result.$XQUERY[0];
                        for (let p in conf.ROOT) {
                            const s = prefixMap[p];
                            if (s === undefined) continue;
                            conf[s] = conf.ROOT[p];
                        }
                    }
                    return new odaIndex(conf);
                });
                return (await Promise.all(indexes)).filter(Boolean);

            });
        },
        getIndex(name, XQ, type) {
            return this.cache('Index:' + name, async () => {
                if (XQ) {
                    const param = { name: name };
                    if (type) param.type = type;
                    await this.API.POST('create_index', param, XQ);
                }
                return (await this.$$indexes).find(i => i.name === name);
            })
        }
    })
    ODANT({
        is: 'odaFilesNFoldersOwner',
        extends: 'odaItem',
        props: {
            $$files: {
                type: Array,
                get() {
                    return this.getFiles();
                }
            },
            $$folders: {
                type: Array,
                get() {
                    return this.getFolders();
                }
            }
        },
        async getDirList() {
            return this.cache('get_dirlist', async () => await this.request('get_dirlist'));
        },
        async getFolders(path = '*') {
            return this.cache('get-folder-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        return (files?.$DIR || []).map(i => {
                            let folder = this.__items__[i.name];
                            if (!folder) {
                                i.$owner = this;
                                folder = this.__items__[i.name] = new odaFolder(i);
                            }
                            return folder;
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                }
                else {
                    const steps = path.split('/');
                    path = steps.shift();
                    // todo переделать на маски
                    let folders = await this.getFolders();
                    if (path !== '*')
                        folders = folders.filter(i => i.name === path);
                    if (steps.length) {
                        path = steps.join('/');
                        folders = folders.map(async f => {
                            return (await f.getFolders(path)).flat();
                        })
                    }
                    return (await Promise.all(folders)).flat();
                }
            })
        },
        async getFiles(path = '*') {
            return this.cache('get-files-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        return (files.$FILE || []).map(i => {
                            i.$owner = this;
                            i.path;
                            return new odaFile(i);
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                } else {
                    // const steps = path.split('/');
                    // path = steps.shift();
                    // // todo переделать на маски
                    // let folders = await this.getFolders();
                    // if (path !== '*')
                    //     folders = folders.filter(i => i.name === path);
                    // if (steps.length) {
                    //     path = steps.join('/');
                    //     folders = folders.map(async f => {
                    //         return (await f.getFolders(path)).flat();
                    //     })
                    // }
                    // folders = (await Promise.all(folders)).flat();

                    // TODO продумать *
                    const folders = await this.getFolders(path)
                    const files = folders.map(async f => {
                        return (await f.getFiles('*')).flat();
                    });
                    return (await Promise.all(files)).flat();
                }
            })
        },
        async getFolder(path) {
            return await this._getFromPath(path, false);
        },
        async getFile(path) {
            return await this._getFromPath(path, true);
        },
        async _getFromPath(path, isFile) {
            const steps = path.split('/');
            const name = steps.pop();
            path = steps.join('/');
            const items = await this[isFile ? 'getFiles' : 'getFolders'](path || '*');
            return items.find(i => i.name === name);
        },
        async show(view, params = {}, dialog = false) {
            view = view || this.defaultViewName;
            const action = await this.findAction(view);
            // TODO load
            const form = await ODA.createComponent('odant-form', params);
            form.viewName = view;
            form.contextItem = this;
            form.unique = this.path + "#" + form.viewName;
            return form.show(dialog);
        },
        saveFile(fileBody, path = '', inherit = false, mime) {
            let name = path.split('/').pop();
            switch (fileBody.constructor.name.toLowerCase()) {
                case 'string':
                    //if (ODANT.webMode) {
                        fileBody = new File([fileBody], name, {type: mime || "text/plain"});
                    //}
                    break;
                case 'array':
                case 'object':
                    //if (ODANT.webMode) {
                        fileBody = new File([JSON.stringify(fileBody)], name, {type: mime || "application/json"});
                    //}
                    break;
                case 'odafile': // todo <-- Еще смотреть как определять тип файла (MIME)
                    name = fileBody.id;
                    return fileBody.load().then(fileBody=>{
                        return this.saveFile(fileBody, path || name);
                    });
                case 'file':
                case 'blob':
                case 'readstream':
                    break;
                default:
                    throw new Error('Unknown file type')
            }
            name = name || fileBody.name;
            const formData = new FormData();
            formData.append("file", fileBody, name);
            const url = inherit?this.url:this.url.replaceAll('/~/', '/');
            return fetch(url + '/' + path, { method: 'POST', body: formData, credentials: 'include' }).then((res)=>{
                this.reset?.();
                return this.getFile(path, true).then(file => {
                    file.$owner.fireChanged?.(file);
                    file.$owner.fire?.('create', file);
                    return file;
                });
            })
        }
    })
    ODANT({
        is: 'odaServerItem',
        extends: 'odaItem',
        props: {
            version: 0,
            prefix: String,
            body: Object,
            id: undefined,
            name() {
                return this.ROOT?.name || this.id;
            },
            label() {
                return this.ROOT?.label || this.name;
            },
            icon() {
                if (this.config.ico)
                    return this.url + '/~/' + this.config.ico;
                return this.__cache__.icon;
            },
            path() {
                return (this.$owner ? (this.$owner.path + '/') : '') + (this.prefix ? this.prefix + ':' : '') + this.id;
            },
            url() {
                return this.$host.origin + '/api' + (this.id ? ('/' + this.path) : '');
            },
            $folder() {
                return new odaFolder({ $owner: this, id: '~' });
            }
        },
        resolveUrl(path) {
            return this.url + '/' + path;
        },
        async request(method = '', params = {}, postObject) {
            try {
                let url = encodeURI(this.url + (method ? '?' + method : '') + Object.keys(params).reduce((res, p) => {
                    res += ((res || method) ? '&' : '?') + p + '=' + params[p];
                    return res;
                }, ''))
                params = { method: postObject ? 'POST' : 'GET', body: postObject, credentials: 'include' };
                const response = await fetch(url, params)
                switch (response.status) {
                    case 200: {
                        let content_type_header = response.headers.get('Content-Type');
                        let res = content_type_header ? content_type_header.split(';')[0] : content_type_header;
                        switch (res) {
                            case 'application/x.odant.async':
                            case 'application/x.odant.async+json':
                            case 'text/x-json':
                            case 'application/json':
                                return response.json();
                            case 'text/cmd':
                            case 'text/css':
                            case 'text/csv':
                            case 'text/javascript':
                            case 'application/javascript':
                            case 'text/php':
                            case 'text/html':
                            case 'text/plain':
                            case 'text/xml':
                                return response.text();
                            case 'image/png':
                            case 'image/gif':
                            case 'image/jpg':
                            case 'image/jpeg':
                            case 'application/octet-stream':
                                return ODANT.webMode ? response.blob() : response.buffer();

                            default: {
                                throw new RequestError(response, 'Unresolved fetch mime-type: ' + res);
                            }
                        }
                    }
                    case 500: {
                        const text = await response.text();
                        throw new RequestError(response, text);
                    }
                    default: {
                        // throw new RequestError(response, response.statusText);
                    }
                }
            }
            catch (e) {
                // console.error(e)
            }
        },
        async load(params, fetchParams) {
            try {
                const res = await this.request(undefined, params, fetchParams);
                if (typeof res === 'object')
                    this.body = res;
                else
                    this.data = res;
                return res;
            }
            catch (e) {
                this.error = e;
                console.error(e);
            }
        },
        async findItem(path, params = { path: path, chain: 'full', prefix: true }) {
            const cacheKey = 'find-item:' + Object.keys(params).reduce((res, key) => {
                return res + `${key}=${JSON.stringify(params[key])}`;
            }, '');
            return this.cache(cacheKey, async () => {
                try {
                    const path = await this.request('find_item', params);
                    return this.getItem(path.result);
                }
                catch (e) {
                    console.error(e)
                }
            });
        },
        async getItem(path) {
            return this.cache('get-item:' + path, async () => {
                if (this.config === undefined) {
                    await this.getConfig?.();
                }

                let step = this.prefix + ':' + this.id;
                let idx = path.indexOf(step);
                if (idx < 0)
                    throw new Error('Undefined search context: ' + path);
                path = path.substring(++idx + step.length);
                if (!path)
                    return this;
                step = path.split('/')[0];
                const prefix = step.split(':')[0];
                const id = step.split(':')[1];
                const items = this.config['$' + prefix];
                const config = items && items.find(i => i.i === id);
                const url = this.url + '/' + step + '/~/extend.js';
                const imp = await import(url);
                const item = new imp.default({ id, prefix, $owner: this });
                item.config = config || (await item.getConfig?.());
                return item.getItem(path);
            });
        },
        getAttr(name, def, type) {
            let res = undefined;
            if (this.ROOT) {
                name = name.split('|');
                while (!res && name.length) {
                    const n = name.shift().trim();
                    res = this.ROOT[n];
                }

                if (res === undefined || res === '' || res === null) return def;
                switch (type) {
                    case Boolean: {
                        res = String(res).toLowerCase() === 'true';
                    } break;
                    case Number: {
                        res = Number(res);
                    } break;
                    case Date: {
                        res = Date.parse(res);
                    } break;
                }
                return res;
            }
            return def;
        },
        setAttr(name, v) {
            if (this.ROOT && this.ROOT[name] !== v) {
                if (v === undefined || v === null)
                    delete this.ROOT[name];
                else
                    this.ROOT[name] = v;
                //this.fireChanged();
            }
        }
    })
    ODANT({
        is: 'odaStructureItem',
        extends: 'odaServerItem, odaFilesNFoldersOwner',
        props: {
            config: {
                set(v) {
                    if (v) {
                        for (let p in v) {
                            const s = configMap[p];
                            if (s === undefined) continue;
                            this[s] = v[p];
                        }
                    }
                }
            }
        },
        async getConfig(params = { prefix: true, deep: 3, group: false, byType: false }) {
            return this.cache('get-config', async () => {
                // try{
                const res = await this.request('get_config', params);
                this.config = res['$' + this.prefix][0];
                return this.config;
                // }
                // catch (e){
                //     console.error(e);
                // }
            })
        },
        async _getItems(prefix) {
            const config = await this.getConfig()
            return Promise.all((config['$' + prefix] || []).map(i => {
                return this.findItem(prefix + ':' + i.i);
            }));
        }
    })
    ODANT({
        is: 'odaHost',
        extends: odaStructureItem,
        props: {
            typeName: 'host',
            origin: String,
            $host() {
                return this;
            },
            prefix: 'H',
            icon: 'odant:host',
            defaultViewName: 'desktop-ext',
            uid: String,
            actionsPath: 'web/core/structure/client/*/*'
        },
        async created() {

            if (!SERVER_MODE) {
                try {
                    this.uid = (await this.request('check_ssid')).result; //todo убрать result
                }
                catch (e) {
                    const security = JSON.parse(localStorage.getItem('security'));
                    if (!security || security.login === 'GUEST') return;
                    if (security.login && security.ticket) {
                        const ts = JSON.stringify(security.ticket);
                        try {
                            const response = await this.request('authorize', {}, ts)
                            security.ssid = response.result;
                            // return this.loginIn();
                        }
                        catch (e) {
                            // ODANT.push(e);
                            localStorage.removeItem('security');
                        }
                        this.request('check_ssid').then(res => {
                            this.uid = res?.result
                        })
                        // this.uid = (await this.request('check_ssid')).result; //todo убрать result
                    }
                }
            }
            await this.getConfig();
        }
    })
    ODANT({
        is: 'odaClass',
        extends: 'odaStructureItem, odaFieldsOwner, odaIndexesOwner',
        props: {
            $class() {
                return this;
            },
            body() {
                return this.cache('body', async () => {
                    const body = await this.request();
                    this.ROOT = body?.$CLASS?.[0];
                    this.METADATA = this.getRootElement('METADATA');
                    this.ACTIONS = this.getRootElement('ACTIONS');
                    return body;
                })
            },
            ROOT: Object,
            METADATA: Object,
            ACTIONS: Object,
            icon: 'odant:class',
            prefix: 'C',
            typeName: 'class',
            defaultViewName: 'grid',
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         if (this.isAbstract) {
            //             if (this.access.allowAdmin)
            //                 return 'configurator';
            //             return 'qr-code';
            //         }
            //         return this.getAttr('default-class-view', 'qr-code');//'grid');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
            defaultObjectView: {
                //category: 'Views',
                get() {
                    return this.getAttr('default-object-view', 'editor');
                },
                set(v) {
                    this.setAttr('default-object-view', v);
                }
            },

        },
        get $$classes() {
            return this._getItems('C');
        },
        get $$packets() {
            return (async () => {
                const json = await this.request('get_pack_list');
                return json.$FOLDER[0].$FILE ? json.$FOLDER[0].$FILE.reduce((res, v) => {
                    const names = v.Name.split('#');
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.Count;
                    if (names.length)
                        p.items.push({ count: v.Count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, ROOT: i });
                }) : [];
            })();
        },
        getRootElement(name) {
            return this.ROOT?.[`$${name}`]?.[0];
        }
    })
    ODANT({
        is: 'odaPack',
        extends: 'odaServerItem',
        props: {
            icon: 'av:library-books',
            ROOT() {
                return this.cache('ROOT', async () => {
                    return await this.body;
                })
            },
            mask() {
                const prefix = this.$owner.mask;
                return (prefix ? prefix + '/' : '') + this.name;
            }
        },
        get hasPacks() {
            return this.$$packets.length > 0;
        },
        get $$packets() {
            return this.cache('packets', () => {
                return this.ROOT.items.reduce((res, v) => {
                    const names = v.names;
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.count;
                    if (names.length)
                        p.items.push({ count: v.count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, ROOT: i })
                });
            });
        }
    })
    ODANT({
        is: 'odaGroup',
        extends: 'odaServerItem',
        props: {
            id() {
                return this.$owner?.id + '_G_' + this.name;
            },
            // access() {
            //     return this.$owner?.access;
            // },
            icon: 'odant:folder',
            prefix: 'G',
            type: 'CATEGORY',
            $owner: String,
            fill() {
                switch (this.type?.toLowerCase()) {
                    case 'module': return 'magenta';
                    case 'workplace': return 'darkblue';
                    case 'group':
                    case 'category': return 'orange';
                    default: return 'black'
                }
            }
        },
        async getItem(path) {
            return await this.$owner?.getItem(path);
        },
        async findItem(path, params) {
            return await this.$owner?.findItem(path, params);
        }
    })
    ODANT({
        is: 'odaDomain',
        extends: 'odaClass',
        props: {
            icon() {
                return this.isSystem ? 'icons:lock' : 'odant:domain';
            },
            prefix: 'D',
            typeName: 'domain',
            isSystem() {
                return this.id === '000000000000000';
            },
            color: 'gray',
            defaultViewName: 'desktop-ext'
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         return this.getAttr('default-class-view', 'desktop');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
        }
    })
    ODANT({
        is: 'odaBase',
        extends: 'odaDomain',
        props: {
            icon: 'odant:base',
            prefix: 'B',
            $base() {
                return this;
            },
            typeName: 'base',
            color: 'brown',
            $module() {
                return undefined;
            }
        },
        get $$modules() {
            return this._getItems('M');
        },
        get $$bases() {
            return this._getItems('B');
        },
        async getActionsExp($item) {
            const modules = await this.$$modules;
            let actions = await Promise.all(modules.map((module) => {
                return module.getActionsExp($item);
            }));
            actions = actions?.flat();
            console.dir(actions);
            return actions;
        }
    })
    ODANT({
        is: 'odaPart',
        extends: 'odaBase',
        props: {
            $part() {
                return this;
            },
            icon() {
                switch (this.id) {
                    case 'ROOT':
                        return 'odant:root';
                    case 'WORK':
                        return 'odant:structure';
                }
                return this.id.toLowerCase();
            },
            prefix: 'P',
            typeName: 'part'
        }
    })
    ODANT({
        is: 'odaModule',
        extends: 'odaDomain',
        props: {
            icon: 'odant:module',
            prefix: 'M',
            typeName: 'module',
            color: 'magenta',
            defaultViewName: 'module-info',
            $module() {
                return this;
            }
        },
        async getActionsExp($item) {
            const classes = await this.$$classes;
            let actions = await Promise.all(classes.map((cls) => {
                return cls.getActionsExp($item);
            }))
            actions = actions?.flat();
            console.dir(actions);
            return actions;
        }
    })
    ODANT({
        is: 'odaWorkPlace',
        extends: 'odaDomain',
        props: {
            icon: 'odant:workplace',
            prefix: 'W',
            typeName: 'work place',
            color: 'steelblue'
        }
    })
    ODANT({
        is: 'odaFile',
        extends: 'odaServerItem, odaFilesNFoldersOwner',
        props: {
            id() {
                return this.name;
            },
            icon() {
                return 'files:' + this.ext;
            },
            ext() {
                return this.id.split('.').pop();
            },
            typeName: 'file'
        }
    })
    ODANT({
        is: 'odaFolder',
        extends: 'odaFile, odaFilesNFoldersOwner',
        props: {
            icon: 'odant:folder',
            typeName: 'folder'
        }
    })
    ODANT({
        is: 'odaIndex',
        extends: 'odaServerItem, odaFieldsOwner, odaIndexesOwner, odaFilesNFoldersOwner',
        props: {
            icon: {
                default: 'odant:index',
                get() {
                    switch (this.type) {
                        case 'Union': return 'odant:index';
                        case 'Cascade': return 'odant:index';
                        default: return 'odant:index';
                    }
                }
            },
            prefix: 'I',
            typeName: 'index',
            type: String,
            ROOT: Object,
            METADATA() {
                return this.ROOT?.$METADATA?.[0] || this.$class.METADATA;
            }
        },
        getRows(mask, from, length, post){
            if (typeof post === 'object')
                post = JSON.stringify(post);
            //return this.request('dataset', {loadmask: mask, mask: mask, from: from, length: length, async: true}, post);
            return this.request('dataset', {loadmask: mask, mask: mask, from: from, length: length}, post);
        },
        getDataSet(mask = '*', filter, clear){
            if(clear) this.reset();
            return this.$('DataSet-'+mask + filter, ()=>{
                return new IndexDataSet(this, mask, filter);
            }, 100000) //todo Предложение по сбросу кэша
        }
    })
    ODANT({
        is: 'odaObject',
        extends: 'odaServerItem, odaFilesNFoldersOwner, odaIndexesOwner',
        props: {
            icon: 'odant:object',
            prefix: 'O',
            typeName: 'object',
            ROOT: Object,
            body() {
                return this.cache('body', async () => {
                    const body = await this.request();
                    this.ROOT = body?.$OBJECT?.[0];
                    //this.ACTIONS = this.getRootElement('ACTIONS');
                    return body;
                })
            },
            $fields() {
                this.$class.$fields;
            },
            defaultViewName() {
                return this.$class.defaultObjectView;
            },
        }
    })
    ODANT({
        is: 'odaAction',
        extends: 'odaServerItem',
        props: {
            active() {
                return !this.export;
            },
            icon: 'odant:action',
            typeName: 'action',
            $source: Object,
            defaultViewName() {
                return this.name.toLowerCase() + '.js';
            },
            ROOT() {
                const metaRoot = this.$class?.$module
                    ? this.$class?.ROOT?.$MODULES?.[0].$MODULE?.find(e => e.id === this.$class?.$module?.id)
                    : this.$class?.ROOT?.$ACTIONS?.[0];
                return metaRoot?.$ACTION?.find(e => e.id === this.id);
            }
        },
        execute() {
            throw new Error(`Action "${this.label}" not implemented!`)
        }
    })
    ODANT({
        is: 'odaView',
        extends: 'odaAction',
        props: {
            typeName: 'view',
            url() {
                return this.path;
            }
        },
        async loadView(){

        },
        async execute(parent) {
            while (parent && !parent.showView) {
                parent = parent.parent || parent.domHost || parent.parentElement;
            }
            if (parent) {
                await this.loadView();
                parent.showView(this);
            }
            else {
                window.open(this.$owner.url + '/~/client/pages/form/index2.html?view=' + this.id);
            }
        },
        get _control() {
            return this.createComponent();
        },
        async createComponent(component) {
            await import(`${this.url}/${this.name}.js`);
            const control = await ODA.createComponent(component || this.defaultViewName);
            control.contextItem = this.$owner;
            control.sourceItem = this.$source;
            return control;
        }
    })
    ODANT({
        is: 'odaActionError',
        extends: 'odaAction',
        props: {
            icon: 'icons:error'
        },
        execute() {
            window.open(this.url + '/~/client/app-studio/index2.html');
        }
    })
    ODANT({
        is: 'odaField',
        extends: 'odaVirtualItem, odaFieldsOwner',
        props: {
            $folder() {
                return (this.$typeClass || this.$class).$folder
            },
            typeName: 'field',
            icon: 'odant:field',
            ROOT: Object,
            $typeClass: Object,
            actionsPath() {
                return 'fields/*/*';
            },
            isTable: false,
            hasFields() {
                return (this.$typeClass && this.$typeClass.isAbstract && this.$typeClass.hasFields) || (this.ATTRS && (this.ATTRS.length > 0));
            },
            isLink() {
                return (this.$typeClass && !this.$typeClass.isAbstract);
            },
        },
    })
    ODANT.host = new odaHost({ origin: ODANT.URL.origin });
}
export default globalThis.ODANT;