import './core.js';
import '/web/oda/oda.js';
const odantComponentRegExp = /(?<=\<)(odant-[a-z\-]+)/gim;
const regexUrl = /https?:\/\/(?:.+\/)[^:?#&]+/g
globalThis.ODANT = globalThis.ODANT || async function (prototype) {
    const matches = (new Error()).stack.match(regexUrl);
    prototype.url = prototype.url || matches[matches.length - 1];
    ODANT.modules[prototype.url] = ODANT.modules[prototype.url] || [];
    if (!prototype.is || prototype.is in ODANT.deferred) {
        console.warn(prototype)
        return;
    }
    ODANT.modules[prototype.url].push(prototype.is);
    if (!prototype.is.startsWith('odant-')) prototype.is = 'odant-' + prototype.is;
    prototype.extends = Array.isArray(prototype.extends) ? prototype.extends : prototype.extends?.split(',') || [];
    if (!prototype.extends.includes('odant-context')) {
        prototype.extends.unshift('odant-context');
    }
    prototype.imports = Array.isArray(prototype.imports) ? prototype.imports : prototype.imports?.split(',') || [];
    ODANT.deferred[prototype.is] = {
        prototype,
        deps: prototype.template?.match(odantComponentRegExp) || [],
    };

}
ODANT.modules = {}
ODANT.deferred = {};

ODANT.regComponent = async function(is, origin, context) {
    if (window.customElements.get(is)) return;
    if (!(is in ODANT.deferred)){
        console.warn(`Prototype ${is} not found!`);
        return;
    }
        // throw new Error(`Prototype ${is} not found!`);
    const prototype = ODANT.deferred[is].prototype;
    const modules = ODANT.modules[prototype.url] || [];
    delete ODANT.modules[prototype.url];
    await Promise.all(modules.filter(i=>i !== is).map(async k=>{
        return ODANT.regComponent(k, origin);
    }))

    const starts = Object.keys(ODANT.deferred);
    await Promise.all((prototype.imports).map(async i => {
        let url = i.trim();
        if (url.startsWith('@')) url = context && !url.startsWith('@oda') ? `/api/${context}/${url}` : `/${url}`;
        if (origin) url = origin + '/' + url;
        url = url.replace(/\/\//g, '/');
        return import(url);
    }));
    for (const k of Object.keys(ODANT.deferred)) {
        if (k === is) continue;
        if (!starts.includes(k) || ODANT.deferred[is].deps.includes(k) || prototype.extends.includes(k)) {
            await ODANT.regComponent(k, origin);
            starts.push(k);
        }
    }

    await ODA(prototype, origin, context);
}

ODANT.createComponent = async function (id, props) {
    await ODANT.regComponent(id, ODANT.origin, undefined);
    return document.createElement(id, props);
}
ODA({
    is: 'odant-context',
    props: {
        context: {
            type: String,
            async set(context) {
                this.contextItem = (context) ? await CORE.host.findItem(context) : undefined;
            }
        },
        contextItem: Object
    }
});
[
    { name: '--inherit-owner', syntax: '<color>', initialValue: 'orangered', inherits: true, }, //#ff4500
    { name: '--inherit-parent', syntax: '<color>', initialValue: 'blue', inherits: true, }, //#0000ff
    { name: '--inherit-type', syntax: '<color>', initialValue: 'green', inherits: true, }, //#008000
    { name: '--inherit-core', syntax: '<color>', initialValue: 'gray', inherits: true, }, //#808080
    { name: '--inherit-service', syntax: '<color>', initialValue: 'red', inherits: true, }, //#ff0000
    { name: '--inherit-self', syntax: '<color>', initialValue: 'black', inherits: true, }, //#ff0000

    { name: '--admin-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--rwcd-color', syntax: '<color>', initialValue: 'red', inherits: true, },
    { name: '--rwc-color', syntax: '<color>', initialValue: 'green', inherits: true, },
    { name: '--rw-color', syntax: '<color>', initialValue: 'blue', inherits: true, },
    { name: '--r-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--p-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--no-access-color', syntax: '<color>', initialValue: 'black', inherits: true, },

    { name: '--item-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--domain-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--host-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--part-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--base-color', syntax: '<color>', initialValue: 'brown', inherits: true, },
    { name: '--module-color', syntax: '<color>', initialValue: 'magenta', inherits: true, },
    { name: '--workplace-color', syntax: '<color>', initialValue: 'steelblue', inherits: true, },
    { name: '--folder-color', syntax: '<color>', initialValue: 'orange', inherits: true, },
    { name: '--file-color', syntax: '<color>', initialValue: 'black', inherits: true, },
].forEach(s => CSS.registerProperty(s));
document.addEventListener('DOMContentLoaded', () => {
    const ss = Array.from(document.body.querySelectorAll('*')).filter(i => {
        return i.localName.startsWith('odant-');
    }).forEach(i => {
        ODANT.regComponent(i.localName, ODANT.origin, i.context || i.getAttribute('context') || undefined);
    });
});
ODANT.origin = location.origin.match(/https?:\/\/localhost/) ? 'https://current.odant.org' : undefined;//location.origin;
export default globalThis.ODANT;