(class extends odaAction {
    static get properties() {
        return {
            allowAccess: 'A',
            icon: 'icons:add',
            label() {
                switch (this.$item.itemTypeLabel) {
                    case 'module':
                        return 'Add class';
                    case 'workplace':
                    case 'work place':
                        return 'Add link';
                    case 'object':
                        return 'Add object';
                    case 'root':
                    case 'work':
                    case 'base':
                    case 'class':
                    default:
                        return 'Add item';
                }
            },
            category: 'developer',
            zIndex: 100,
        };
    }
    async execute() {
        let name, label, type, inheritItem;
        type = await this.chooseItemType(this.$item);
        if (!type) return;

        if (type === 'link') {
            await this._createLink(this.$item);
            return;
        }
        
        const method = (type === 'class' || type === 'module')
                     ? await this.chooseMethodType()
                     : 'new';
        if (!method) return;

        if (method === 'new') {
            const count = await this._getCount(this.$item, type);

            const control = await ODA.createComponent('/web/lib/dialogs/label-to-name/label-to-name.html');
            control.label = `${type}${count ? ` ${String(count + 1)}` : ''}`;
            const dialogResult = await ODA.showDialog(control, { title: 'Set name & label', icon: 'icons:build' });
            if (!dialogResult) return;

            name = dialogResult.name;
            label = dialogResult.label;
        } else {
            inheritItem = await this.chooseInheritItem(type, this.$item);
        }

        let item;
        if (type === 'class') {
            item = inheritItem
                   ? await this.$item._createClass(inheritItem.name, JSON.stringify(this.getBody(inheritItem)))
                   : await this.$item._createClass(label);
        } else {
            let prefix;
            switch (type) {
                case 'module': prefix = 'M'; break;
                case 'part': prefix = 'P'; break;
                case 'workplace': prefix = 'W'; break;
                case 'base':
                default: prefix = 'B'; break;
            }
            item = inheritItem
                   ? await this.inheritDomain(this.$item, inheritItem)
                   : await this.$item._createDomain(label, prefix);
        }

        this._pushNote(item, type);
    }
    async inheritDomain(owner, inheritItem) {
        const item = await owner._createDomain(inheritItem.name, inheritItem.prefix, JSON.stringify(this.getBody(inheritItem)));
        const classes = await inheritItem.$classes;
        for (const child of classes) await this.inheritClass(item, child);
        
        owner.fire('update');
        return item;
    }
    async inheritClass(owner, inheritItem) {
        const item = await owner._createClass(inheritItem.name, JSON.stringify(this.getBody(inheritItem)));
        const classes = await inheritItem.$classes;
        for (const child of classes) await this.inheritClass(item, child);
        item.save();
        return item;
    }
    async chooseItemType($item) {
        const cls = { label: 'Class', icon: 'odant:class', special: 'class' };
        const mod = { label: 'Module', icon: 'odant:module', special: 'module' };
        //const obj = { label: 'Object', icon: 'odant:object', special: 'object' };
        const bas = { label: 'Base', icon: 'odant:base', special: 'base' };
        const wrk = { label: 'Workplace', icon: 'odant:workplace', special: 'workplace' };
        //const lnk = { label: 'Link', icon: 'icons:link', special: 'link' };

        let buttons;
        switch ($item.itemTypeLabel) {
            case 'module': return 'class';//buttons = [cls]; break;
            case 'class': buttons = [cls, mod]; break;
            case 'object': return 'object';//buttons = [obj]; break;
            case 'root':
            case 'work': buttons = [bas, mod, wrk]; break;
            case 'base': buttons = [cls, bas, mod, wrk]; break;
            case 'work place': 
            case 'workplace': return 'link';//buttons = [lnk]; break;
            default: return 'unavailable';
        }

        const dialog = {
            title: 'Select item type',
            icon: 'icons:add-circle',
            buttons: buttons
        };

        const dialogResult = await ODA.showDialog(null, dialog);
        return (dialogResult) ? dialogResult.special : null;
    }
    async _createLink($item) {
        const control = await ODA.createComponent('/web/core/structure/lib/select-class/select-class.html');
        control.contextItem = $item;
        const dialogResult = await ODA.showDialog(control, { title: 'Select class for link', icon: 'icons:link' });
        if (!dialogResult) return;

        await $item.createLink(dialogResult.item);
        $item.$owner.fire('update');
    }
    async chooseMethodType() {
        const buttons = [
            { label: 'New', icon: 'av:new-releases', special: 'new' },
            { label: 'Inherit', icon: 'odant:inherit', special: 'inherit' }
        ];

        const dialog = {
            title: 'Choose creating method',
            icon: 'odant:module',
            buttons: buttons
        };
        
        const dialogResult = await ODA.showDialog(null, dialog);
        return (dialogResult) ? dialogResult.special : null;
    }
    async chooseInheritItem(type, $item) {
        const path = (type === 'class')
                       ? '/web/core/structure/lib/select-class/select-class.html'
                       : '/web/core/structure/lib/select-module/select-module.html';
        const control = await ODA.createComponent(path);
        control.contextItem = $item;
        const dialogResult = await ODA.showDialog(control, { title: 'Select an item to inherit', icon: 'odant:inherit' });
        return (dialogResult) ? dialogResult.item : null;
    }
    async _getCount(item, type) {
        if (!item) return 0;
        let pName = '';
        switch (type) {
            case 'workplace': pName = 'WorkPlaces'; break;
            case 'base': pName = 'Bases'; break;
            case 'class':
            default: pName = 'Classes'; break
        }
        const items = await item[pName];
        return items && items.length || 0;
    }
    getBody($item) {
        const body = {
            $CLASS: [{
                ClassId: $item.id,
                Name: $item.name,
                Label: $item.label,
            }]
        };

        if ($item instanceof odaDomain) body.$CLASS[0].Type = $item.type;
        if ($item instanceof odaClass) body.$CLASS[0].Parent = $item.id;
        return body;
    }
    _pushNote(item, type) {
        ODA.push(
            `A ${item.itemTypeLabel} named "${item.label}" was created`,
            {
                tag: `create-${type}-${item.fullId}`,
                body: `A new ${type} "${item.label}" was created in the ${item.$owner.itemTypeLabel} "${item.$owner.label}".`,
                icon: item.icon
            }
        );
    }
}).register();