export default odaPage({
    props: {
        label: 'Navigate',
        icon: 'icons:apps'
    }
});

ODANT({
    imports: [
        '@oda/app-layout',
        '@lib/structure-tree',
        '/web/core/user/lib/user-button/user-button.js',
        '@pages/form',
        '@oda/scroll-box',
        '@oda/tabs'
    ],
    is: 'odant-navigator',
    extends: 'oda-app-layout',
    template: /*html*/`
    <style>
        app-layout-header{
            @apply --header;
        }
    </style>
    <div slot="top-left" class="flex header horizontal" style="justify-content: center; align-items: center; overflow: hidden">
         <oda-tabs template="odant-item-node" ref="tabs" :items="pages" class="horizontal flex" :focused-index="index" @focused-index-changed="_focusedIndexChanged" design-mode="false"></oda-tabs>
    </div>
    <oda-button slot="top-right" ~show="pages.length>1" icon="icons:close" @tap.stop="pages.splice(1); index = 0" style="order: -1;"></oda-button>
    <odant-user-button slot="top-right" ></odant-user-button>
    <div slot="left-drawer" class="vertical flex" style="overflow: hidden; height: 100%;">
        <odant-structure-tree allow-drag allow-focus allow-search ::focused-node="focusedItem" :context-item="contextItem" style="overflow: hidden; height: 100%;"></odant-structure-tree>
    </div>
    <div slot="main" class="vertical flex" style="overflow: hidden; width: 100%; height: 100%;">
        <div ~for="(page, i) in pages" :is="page.tag" :param="page.param" :page="page" :context-item="page.$item" ~show="i === index" class="flex" :ref="_getPageSrc(page)" :src="_getPageSrc(page)"  style="border: none; overflow: hidden; width: 100%; height: 100%;" @load="_frameLoad($event, page)"></div>
    </div>
    <div slot="right-drawer" class="vertical flex" style="overflow: hidden;">
        <iframe ~if="right?.opened" :src="support" style="height: 100%;" class="flex"></iframe>
    </div>`,
    props: {
        support: 'https://www.odant.ru',
        iconSize: {
            default: 24,
            shared: true
        },
        rightIcon: "icons:language",
        focusedPageItem: {
            type: Object,
            get() {
                if (this.pages.length === 0)
                    return null;
                const i = this.pages[this.index];
                return i?.$item;
            }
        },
        focusedItem: {
            type: Object,
            watch(n, o) {
                if (n) {
                    if (!o || n.$item.path !== this.pages[this.index]?.$item.path)
                        this.showItem(n.$item, n.view, n.tag, n.param);
                }
                else {
                    this.index = -1;
                }
            }
        },
        _user: {
            type: Object,
        },
        index: {
            default: -1,
            watch(n, o) {
                this.focusedItem = this.pages[n];
            }
        },
        pages: [],
        contextItem: {
            type: Object,
            async set(n, o){
                if (n){
                    this.contextItem = await n.getNearestAccessibleItem();
                    this.showItem(this.contextItem, '', 'odant-form');
                    this.support = this.contextItem.$host.supportUrl + '/~/client/pages/support/index2.html';
                }
            }
        },
        // _contextItem: {
        //     type: Object,
        //     async set(n, o) {
        //         if (o) {
        //             this.unlisten('message', '_onmessage', { target: window });
        //             // window.removeEventListener('message', this._onmessage);
        //         }
        //         if (n) {
        //             this.listen('message', '_onmessage', { target: window });
        //             // window.addEventListener('message', this._onmessage);
        //             this.showItem(n, '', 'odant-form');
        //             //this._user = await n.$connection.getUser();
        //
        //         }
        //     }
        // }
    },
    // observers: [
    //     async function _onContextItemChanged(contextItem) {
    //         this._contextItem = (await contextItem.getNearestAccessibleItem()) || contextItem;
    //     }
    // ],
    _focusedIndexChanged(e) {
        this.index = e.detail.value;
    },
    _frameLoad(e, page) {
        if(page) page.window = e.target.contentWindow;
    },
    async _onmessage(e) {
        if (e.data) {
            switch (e.data.action) {
                case 'form-close':
                    const idx = this.pages.findIndex(p => p.$item.path === e.data.id);
                    this._closeTabFinally(idx);
                    break;
                case 'show-item':
                    if (e.data.id) {
                        const item = await CORE.findItem(e.data.id);
                        this.showItem(item);
                    }
                    break;
                case 'show-view':
                    if (e.data.id && e.data.view) {
                        const id = e.data.id;
                        const viewName = e.data.view;
                        const item = await CORE.findItem(id);
                        const view = await item.getView(viewName);
                        this.showItem(item, view);
                    }
                    break;
            }
        }
    },
    async _closeTab(idx) {
        try {
            const page = this.pages[idx];
            if (page.tag === 'iframe') {
                page.window.postMessage({ action: 'close', src: this._getPageSrc(page), id: page.$item.path }, this._getPageSrc(page));
                return;
            }
            if (page.view && page.view.allowSave && page.item.isChanged) {
                const ctrl = this.create('odant-form-close-confirm');
                const buttons = [
                    {
                        label: 'Yes',
                        left: true,
                        style: { 'color': 'var(--success-color)' },
                        action: async () => {
                            await this.page.$item.save();
                        }
                    },
                    {
                        label: 'No',
                        action: async () => {
                            page.$item.reset();
                            page.$item.reset(['body', 'Root']);
                            page.$item.reload && await page.$item.reload();
                        }
                    }
                ];
                const res = await ODA.showDialog(ctrl, { buttons, title: page.item.label, fixWidth: true });
                await res.button.action();
            }
        } catch (e) {
            return;
        }
        this._closeTabFinally(idx);
    },
    _closeTabFinally(idx) {
        this.pages.splice(idx, 1);
        if (this.index >= idx) this.index--;
    },
    _getPageSrc(page) {
        return `${page.$item.url}/~/client/pages/form/index2.html${page.view ? '?view=' + page.view.name : ''}`;
    },
    showView(view, tag) {
        view && this.showItem(view.$owner, view, tag);
    },
    // _showItem(e) {
    //     const page = this.pages[this.$refs.tabs.focusedIndex];
    //     this.showItem(page.$item, page.view);
    // },
    showItem(item, view = '', tag = 'iframe', param = '') {
        if (item.noSelect) return;
        const index = this.pages.findIndex(p => {
            return (p.$item === item || p.$item.path === item.path) && p.view === view;
        });
        if (index < 0) {
            const i = { $item: item, view, tag, param, designMode: this.pages.length > 0, bs: {} };
            this.pages.push(i);
            this.index = this.pages.length - 1;
        } else {
            this.index = index;
        }
        if (this.left.persistent) {
            this.left.opened = false;
        }
        this.render();
    },
});