import './desktop-ext-shortcut.js';
import '/web/core/item/lib/item-menu/item-menu.js';
function getGuid () {
    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1);
    }

    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
        s4() + '-' + s4() + s4() + s4();
};
ODANT({
    is: 'odant-desktop-ext-grid',
    template: /*html*/ `
        <style>
            :host{
                --shortcut-width: 118px;
                --shortcut-height: 137px;
            }
            :host{
                @apply --flex;
                overflow: auto;
                display: grid;
                padding: 25px;
                gap: 25px;
                grid-template-rows: repeat(auto-fll, var(--shortcut-height));
                grid-template-columns: repeat(auto-fll, var(--shortcut-width));
                grid-auto-rows: var(--shortcut-height);
                grid-auto-columns: var(--shortcut-width);
                background-attachment: local;
            }
            .hidden, [hidden]{
                display: none !important;
            }
        </style>
        <div is="style">
            :host{
                {{_bgStyle}}
            }
        </div>
        <odant-desktop-ext-shortcut
            :context-item="contextItem"
            :host-id="_hostId"
            ~if="shortcuts && shortcuts.length"
            ~for="s in shortcuts"
            :icon-size="iconSize"
            :style="{gridColumn: s.x, gridRow: s.y}"
            :key="s.key"
            :item="s"
            draggable="true"
            :allow-admin="_allowAdmin"></odant-desktop-ext-shortcut>
    </template>
    `,
    props: {
        contextItem: {
            type: Object,
            async watch(ctx, old) {
                if (old) {
                    this.settingsId = 'oda-desktop-ext';
                    this.shortcuts.splice(0, this.shortcuts.length);
                }
                if (ctx) {
                    this.settingsId = `${ctx.path.replace(/^\//, '')}`;
                    this._hostId = await ctx.$host.id;
                    this._loadShortcuts();
                    this._loadSettings();
                }
            }
        },
        _hostId: '',
        bgSettings: {
            type: Object,
            watch(n, o) {
                this._buildStyle(n);
            }
        },
        _defaultBgSettings: {
            type: Object,
            default: {
                image: 'background.jpg',
                color: 'white',
                repeatX: 'repeat',
                repeatY: 'repeat',
                positionX: 'left',
                positionY: 'top',
                size: 'auto',
            }
        },
        _bgStyle: '',
        _allowAdmin: {
            type: Boolean,
            computed() {
                return this.contextItem &&
                    this.contextItem.access &&
                    (
                        (this.contextItem.access.level >= 6) ||
                        this.contextItem.$base &&
                        (this.contextItem.$base.access.level >= 6)
                    ) || false;
            }
        },
        settingsId: 'oda-desktop-ext',
        settingsPath: 'settings/desktop-ext-settings.json',
        shortcutsPath: 'settings/desktop-ext.json',
        iconSize: 48,
        shortcuts: []
    },
    listeners: {
        async drop(e) {
            while (e.sourceEvent) {
                e = e.sourceEvent;
            }
            const dt = e.dataTransfer;
            const pos = { x: Math.round(e.layerX / (118 + 25)), y: Math.round(e.layerY / (137 + 25)) };

            if (dt.types.includes('odant/odaitem')) {
                const data = JSON.parse(dt.getData('odant/odaitem'));
                if (data.typeName === 'group') return;
                const shortcut = await this._createShortcut(data, pos);
                this._fixPos(shortcut);

                if (this._allowAdmin) {
                    const buttons = [
                        {
                            icon: 'social:share',
                            label: 'Shared',
                        },
                        {
                            icon: 'maps:local-offer',
                            label: 'Local',
                        },
                    ];
                    const ctrl = await ODANT.createComponent('odant-create-shortcut-dialog');
                    const res = await ODA.showModal(ctrl);
                    shortcut.global = res?.shared;
                } else {
                    await ODA.confirm('Create shortcut?', 'Are you sure?');
                }

                this.shortcuts.push(shortcut);
                this._saveShortcuts();
            }
            if (dt.types.includes('odant/desktop-ext-shortcut')) {
                const data_s = dt.getData('odant/desktop-ext-shortcut')
                const data = JSON.parse(data_s);
                const shortcut = this.shortcuts.find(s => JSON.stringify(s) === data_s);
                if (shortcut) {
                    if (shortcut.global && !this._allowAdmin) {
                        await ODA.confirm('Override global shortcut?', 'Confirm');
                        //this.set(shortcut, 'global', false);
                        shortcut.global = false;
                    }
                    [shortcut.x, shortcut.y] = [pos.x, pos.y];
                    this._fixPos(shortcut);
                    this._saveShortcuts();
                }
            }
        },
        dragover(e) {
            const dt = e.dataTransfer;
            if (
                dt.types.includes('odant/odaitem') ||
                dt.types.includes('odant/desktop-ext-shortcut')
            ) {
                e.preventDefault();
            }
        },
        async contextmenu(e) {
            e.preventDefault();
            e.stopPropagation();
            const items = [
                {
                    icon: 'image:image',
                    label: 'Background settings',
                    action: async () => {

                        await import('/web/core/structure/client/views/desktop-ext/lib/desktop-ext-settings.js');
                        const ctrl = await ODA.createComponent('odant-desktop-ext-settings');
                        const buttons = [
                            {
                                label: 'Save',
                                icon: 'icons:save',
                                action: async (settings) => {
                                    return this._saveSettingsLocal(settings);
                                }
                            },
                            {
                                label: 'Reset',
                                icon: 'icons:autorenew',
                                action: async (settings) => {
                                    return this._saveSettingsLocal({});
                                }
                            },
                        ];
                        const uploadImage = async (e) => {
                            const img = e.detail.value[0];
                            await this.contextItem.saveFile(img, `~/background.jpg`);
                            this._buildStyle();
                        };
                        const resetImage = async (e) => {
                            await this.contextItem.deleteFile(`~/background.jpg`);
                            this._buildStyle();
                        };
                        if (this._allowAdmin) {
                            buttons.push(...[
                                {
                                    label: 'Share',
                                    icon: 'social:share',
                                    action: async (settings) => {
                                        return this._saveSettingsGlobal(settings);
                                    }
                                },
                                {
                                    label: 'Reset global',
                                    icon: 'icons:autorenew',
                                    action: async (settings) => {
                                        return this._saveSettingsGlobal({});
                                    }
                                },
                            ]);
                            this.listen(ctrl, 'upload-new-background-image', uploadImage);
                            this.listen(ctrl, 'reset-background-image', resetImage);
                        }
                        ctrl.settings = { ...this._defaultBgSettings, ...this.bgSettings };
                        ctrl.allowAdmin = this._allowAdmin;
                        const res = await ODA.showDialog(ctrl, { buttons });

                        this.unlisten(ctrl, 'upload-new-background-image', uploadImage);
                        this.unlisten(ctrl, 'reset-background-image', resetImage);

                        await res.button.action(ctrl.settings);
                        this._loadSettings();
                        this._buildStyle();
                    },
                },
            ];
            const res = await ODA.showContextMenu({
                title: 'Configure background',
                icon: 'enterprise:settings',
                items,
                iconSize: 32,
                template: 'oda-desktop-ext-menu-item'
            });
            res.action();
        },
        'drag-start'(e) {
            e.detail.value.sourceEvent.dataTransfer.setData('odant/desktop-ext-shortcut', JSON.stringify(e.detail.value.item));
        },
        'change-prop'(e) {
            const { item, prop, value } = e.detail.value;
            //this.set(item, prop, value);
            item.prop = value;
            this._saveShortcuts();
        },
        'reset'(e) {
            const { item, prop } = e.detail.value;
            if (prop)
                item[prop] = '';
            else
                for (const key in item) {
                    if (
                        key === 'id' ||
                        key === 'global' ||
                        key === 'name' ||
                        key === 'x' ||
                        key === 'y'
                    ) continue;
                    item[key] = '';
                }
            this._saveShortcuts();
        },
        'delete'(e) {
            const { item } = e.detail.value;
            const idx = this.shortcuts.findIndex(s => s === item);
            if (~idx) {
                this.shortcuts.splice(idx, 1);
                this._saveShortcuts();
                this.async(() => {
                    this._loadShortcuts();
                }, 500);
            }
        },
        'share'(e) {
            const { item } = e.detail.value;
            const idx = this.shortcuts.findIndex(s => s === item);
            if (~idx) {
                this.shortcuts[idx].global = true;
                this._saveShortcuts();
                this.async(() => {
                    this._loadShortcuts();
                }, 500);
            }
        },
    },
    _saveShortcuts() {
        this.debounce('save-shortcuts', () => {
            const local = this.shortcuts.filter(s => !s.global).map(s => {
                const o = Object.assign({}, s);
                delete o.contextItem;
                return o;
            });
            this.__write(`${this.settingsId}/shortcuts`, local);
            if (this._allowAdmin) {
                const global = this.shortcuts.filter(s => s.global);
                this.contextItem.saveFile(global, this.shortcutsPath)
                    .then(r => this._loadShortcuts());
            }
        }, 300);
    },
    async _loadShortcuts() {
        const local = await this._getLocalShortcuts();
        const global = (await this._getGlobalShortcuts()).filter(g => local.every(l => l.key !== g.key));
        if (!this._allowAdmin) {
            local.forEach(l => {
                const t_l = JSON.stringify(l);
                const g = global.find(e => t_l === JSON.stringify(e));
                if (g) {
                    g.hidden = true;
                }
            });
        }

        this.shortcuts.splice(0, this.shortcuts.length, ...[...local, ...global]);
    },
    async _loadSettings() {
        const local = await this._getLocalSettings();
        const global = await this._getGlobalSettings();
        this.bgSettings = { ...this._defaultBgSettings, ...global, ...local };
    },
    async _getLocalShortcuts() {
        return this.__read(`${this.settingsId}/shortcuts`) || [];
    },
    async _getGlobalShortcuts() {
        return (await this.contextItem.getFile(this.shortcutsPath, undefined, { 'cache': 'no-store' })
            .then(f => f.load())
            .catch(e => void (0))) || []
    },
    async _getLocalSettings() {
        return this.__read(`${this.settingsId}/settings`) || { ...this._defaultBgSettings };
    },
    async _getGlobalSettings() {
        return (await this.contextItem.getFile(this.settingsPath, undefined, { 'cache': 'no-store' })
            .then(f => f.load())
            .catch(e => void (0))) || { ...this._defaultBgSettings };
    },
    _saveSettingsLocal(settings) {
        const _settings = this._filterDefaultSettings(settings);
        this.__write(`${this.settingsId}/settings`, _settings);
    },
    async _saveSettingsGlobal(settings) {
        const _settings = this._filterDefaultSettings(settings);
        await this.contextItem.saveFile(_settings, this.settingsPath);
    },
    _filterDefaultSettings(settings = this.bgSettings, def = this._defaultBgSettings) {
        for (const key in def) {
            if (settings[key] === def[key]) delete settings[key];
        }
        return settings;
    },
    async _createShortcut(item, pos) {
        const remote = this._checkRemote(item);
        const shortcut = {
            id: remote ? item.path : item.path.split('/').filter(s => !['H', 'P'].includes(s[0])).join('/'),
            // id: remote ? item.path : this.contextItem.getRelativeId(item),
            key: getGuid(),
            name: item.name,
            label: '',
            url: item.url,
        };
        if (remote) {
            shortcut.$item = item;
            shortcut.icon = item.icon;
        }

        [shortcut.x, shortcut.y] = [pos.x, pos.y];

        return shortcut;
    },
    _checkRemote(item) {
        const hostId = item.path.match(/^\/?H:(.{15})\//i)[1];
        return this._hostId !== hostId;
    },
    _fixPos(shortcut) {
        let x = true;
        while (this.shortcuts.some(sh => sh !== shortcut && sh.x === shortcut.x && sh.y === shortcut.y)) {
            if (x) shortcut.x++;
            if (!x) shortcut.y++;
            x = !x;
        }
    },
    _buildStyle(settings = this.bgSettings) {
        if (settings && Object.keys(settings).length && this.contextItem) {
            const ctx = this.contextItem;
            const image = (
                settings.image &&
                settings.image !== 'none' &&
                ctx && ctx.url &&
                `url(${ctx.url + `/~/${settings.image}?rnd=${Math.random().toString().slice(3)}`})`
            ) || 'none';
            this._bgStyle = `
                        background-image: ${image};
                        background-color: ${settings.color || 'white'};
                        background-repeat-x: ${settings.repeatX || 'repeat'};
                        background-repeat-y: ${settings.repeatY || 'repeat'};
                        background-position-x: ${settings.positionX || 'left'};
                        background-position-y: ${settings.positionY || 'top'};
                        background-size: ${settings.size || 'auto'};
                    `;
        } else {
            this._bgStyle = '';
        }
    }
})
ODANT({
    is: 'create-shortcut-dialog',
    template: /*html*/`
    <style>
        :host{
            @apply --vertical;
            justify-content: space-between;
        }
        :host .title{
            @apply --header;
            align-items: center;
        }
        :host .buttons{
            justify-content: space-around;
        }
        :host .body, :host .title, :host .buttons{
            padding: 2px;
        }
        :host .buttons > oda-button{
            @apply --flex;
            @apply --raised;
            padding: 4px;
            margin: 2px;
        }
    </style>
    <div class="title horizontal">
    <div class="flex">Create shortcut</div>
    <oda-button class="raised" icon="icons:close" @tap="parentElement.fire('close')"></oda-button></div>
    <div class="flex body">What type of shortcut do you want to create?</div>
    <div class="horizontal buttons header">
        <oda-button icon="social:share" label="Shared" @tap="shared = true; parentElement.fire('ok')"></oda-button>
        <oda-button icon="maps:local-offer" label="Local" @tap="shared = false; parentElement.fire('ok')"></oda-button>
        <oda-button icon="icons:close" fill="red" label="Cancel" @tap="parentElement.fire('close')"></oda-button>
    </div>`,
    props: {
        shared: false
    }
})