import '/web/oda/components/buttons/button/button.js';
import '/web/oda/tools/property-grid/property-grid.js';
globalThis.DesktopSettings = globalThis.DesktopSettings || class DesktopSettings {
    constructor(origin) {
        const props = {};
        for (const key in origin) {
            props[key] = {
                name: key,
                get() {
                    return origin[key];
                },
                set(v) {
                    origin[key] = v;
                }
            };
        }
        Object.defineProperties(this, props);
        this.properties = {
            image: {
                type: String,
                enum: ['background.jpg', 'cover.png', 'none']
            },
            color: {
                editor: '/web/lib/colors/picker/picker.html',
                type: String,
            },
            repeatX: {
                type: String,
                enum: ['no-repeat', 'repeat', 'repeat-x', 'repeat-y', 'round', 'space', 'initial'],
                default: 'repeat'
            },
            repeatY: {
                type: String,
                enum: ['no-repeat', 'repeat', 'repeat-x', 'repeat-y', 'round', 'space', 'initial'],
                default: 'repeat'
            },
            positionX: {
                type: String,
                enum: ['left', 'center', 'right', '0%', 'initial'],
                default: 'repeat'
            },
            positionY: {
                type: String,
                enum: ['left', 'center', 'right', '0%', 'initial'],
                default: 'repeat'
            },
            size: {
                type: String,
                enum: ['auto', 'contain', 'cover', 'initial'],
                default: 'initial'
            }
        };
    }
}

ODANT({
    is: 'desktop-ext-settings',
    template: /*html*/ `
        <style>
            :host{
                @apply --vertical;
                min-width: 270px;
                overflow: hidden;
                text-overflow: ellipsis;
            }
        </style>
        <oda-property-grid ~if="_settings" :inspected-object="_settings"></oda-property-grid>
        <div ~if="_settings && allowAdmin" class=" flex horizontal">
            <oda-button class="raised flex" style="margin: 4px;" label="upload new background image" icon="icons:file-upload" @tap="_tapNew"></oda-button>
            <oda-button class="raised flex" style="margin: 4px;" label="reset background image" icon="icons:autorenew" @tap="_tapReset"></oda-button>
        </div>`,
    props: {
        settings: {
            type: Object,
            notify: true,
            set(n) {
                this._settings = new DesktopSettings(n);
            }
        },
        _settings: Object,
        allowAdmin: false,
    },
    async _tapNew() {
        this.fire('upload-new-background-image', await ODA.showFileDialog('image/*', false));
    },
    async _tapReset() {
        this.fire('reset-background-image');
    }
})