import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/buttons/button/button.js';
import '../../../item/lib/item-node/item-node.js';
import '../select-action-dropdown/select-action-dropdown.js';
ODA({
    is: 'odant-action-finder',
    template: `
    <style>
        :host{
            @apply --horizontal;
            @apply --flex;
        }
        :host>odant-item-node, :host>span{
            @apply --flex;
            margin-left: 8px;
        }
        :host>input{
            border: none;
            padding: 4px 8px;
        }
        [disabled]{
            @apply --disabled;
        }
    </style>
    <odant-item-node ~if="$action" :context-item="$action"></odant-item-node>
    <input ~if="actions.length > 0 && !$action" placeholder="none" ref="input" class="flex" @tap.stop :value="$action && $action.name" @blur="actionName = ''" @keydown.enter.stop @keyup.enter.stop="_findByName">
    <div ~if="!$action && actions.length === 0" class="flex"></div>
    <oda-button :disabled="actions.length === 0" :icon="$action ? 'icons:close' : 'icons:arrow-drop-down'" :toggled="dd && dd.opened" @tap.stop="_tap"></oda-button>`,
    props: {
        contextItem: {
            type: Object,
            async set(n) {
                if (n) {
                    this.actions = await n.getActions();
                }
            }
        },
        actions: [],
        filter: {
            type: String,
            set(n) {
                if (this.dd) this.dd.filter = n;
            }
        },
        includes: [],
        excludes: [],
        groups: [],
        by: 'name',
        dd: HTMLElement,
        $action: {
            type: Object,
            notify: true
        },
        actionName: {
            type: String,
        }
    },
    _tap() {
        if (this.$action) {
            this.$action = null;
        } else {
            this.showDD();
        }
    },
    async showDD() {
        if (!this.dd || !this.dd.opened) {
            this.dd = ODA.createElement('odant-select-action-dropdown', {
                parent: this.host,
                filter: this.filter,
                contextItem: this.contextItem,
                includes: this.includes,
                excludes: this.excludes,
                groups: this.groups,
                by: this.by
            });
            const res = await this.dd.show(this.$action).catch(e => void (0));
            this.$action = res || this.$action;
        }
        this.dd.close();
        this.dd = null;
        this.filter = '';
    },
    async _findByName() {
        const input = this.$refs.input;
        const filter = input && input.value;
        if (filter) {
            input.disabled = true;
            try {
                const action = this.actions.filter(a => this.excludes.every(ex => ex.toLowerCase() !== a[this.by].toLowerCase())).find(a => a[this.by].toLowerCase().includes(filter.toLowerCase()));
                this.$action = action || null;
            } catch (e) {
                console.warn(`Action with filter: "${filter}" was not found.`);
            } finally {
                if (!this.$action) {
                    this.filter = filter;
                    input.value = '';
                    this.showDD();
                }
                input.disabled = false;
            }
        }
    }
});