import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/layouts/dropdown/dropdown.js';
import '../../../../oda/components/buttons/button/button.js';
import '../../../item/lib/item-node/item-node.js';
import '../select-class/select-class.js';

ODA({
    is: 'odant-class-finder',
    template: `
    <style>
        :host {
            @apply --horizontal;
            overflow: hidden;
            text-overflow: ellipsis;
        }

        :host>odant-item-node,
        :host>span {
            @apply --flex;
            margin-left: 8px;
        }

        :host>input {
            border: none;
            padding: 4px 8px;
            min-width: 0;
        }

        [disabled] {
            @apply --disabled;
        }
    </style>
    <odant-item-node ~if="$class" :context-item="$class"></odant-item-node>
    <input ~if="contextItem && !$class && !readOnly" :disabled="classList && classList.length > 0"
        placeholder="none" ref="input" class="flex" @tap.stop :value="$class && $class.name" @blur="className = ''"
        @keydown.enter.stop @keyup.enter.stop="_findByName">
    <oda-button ~if="!readOnly" :disabled="!contextItem" :icon="$class ? 'icons:close' : 'icons:arrow-drop-down'"
        :toggled="dd && dd.opened" @tap.stop="_tap"></oda-button>`,
    props: {
        contextItem: Object,
        filter: {
            type: String,
            set(n) {
                if (this.dd) this.dd.filter = n;
            }
        },
        dd: HTMLElement,
        $class: {
            type: Object,
            notify: true
        },
        classList: Array,
        className: String,
        readOnly: false
    },
    _tap() {
        if (this.$class) {
            this.$class = null;
        } else {
            this.showDD();
        }
    },
    async showDD() {
        if (!this.dd || !this.dd.opened) {
            if (this.classList && this.classList.length > 0) {
                this.dd = ODA.createElement('odant-class-tree-dropdown',
                    { parent: this.host, contextItem: this.contextItem, _items: this.classList });
            } else {
                this.dd = ODA.createElement('odant-select-class-dropdown',
                    { parent: this.host, contextItem: this.contextItem, filter: this.filter });
            }
            const res = await this.dd.show(this.$class).catch(e => void (0));
            
            this.$class = res || this.$class;
        }
        this.dd.close();
        this.dd = null;
        this.filter = '';
    },
    async _findByName() {
        const input = this.$refs.input;
        const filter = input && input.value;
        if (filter) {
            input.disabled = true;
            try {
                const cls = await this.contextItem.$class.findItem(filter);
                this.$class = cls || null;
            } catch (e) {
                console.warn(`Class with filter: "${filter}" was not found.`);
            } finally {
                if (!this.$class) {
                    this.filter = filter;
                    input.value = '';
                    this.showDD();
                }
                input.disabled = false;
            }
        }
    }
})

ODA({
    is: 'odant-select-class-dropdown',
    extends: 'oda-dropdown, odant-select-class',
    template: `
    <style>
        :host {
            position: absolute !important;
        }

        :host>input {
            padding: 4px 8px;
            min-width: 0;
        }
    </style>
    <input class="flex" ref="filterInput" tabindex="1" placeholder="Filter" :value="filterVal"
        @input="_setFilterValue">`,
    props: {
        parent: {
            type: [HTMLElement, Object],
            set(n) {
                if (n) {
                    this.host.style.maxWidth = (n.host || n).offsetWidth;
                } else {
                    this.host.style.maxWidth = '';
                }
            }
        },
    },
    async show() {
        this.opened = true;
        if (this.$refs.input) this.async(() => {
            this.$refs.input.focus();
        }, 300);
        return new Promise((resolve, reject) => {
            const callback = (e) => {
                this.unlisten(this, 'result', callback);
                this.unlisten(this, 'cancel', cancel);
                this.opened = false;
                resolve(this.item);
            };
            const cancel = (e) => {
                this.unlisten(this, 'result', callback);
                this.unlisten(this, 'cancel', cancel);
                this.opened = false;
                reject();
            };
            this.listen(this, 'result', callback);
            this.listen(this, 'cancel', cancel);
        });
    }
})

ODA({
    is: 'odant-class-finder',
    extends: 'oda-dropdown, this, oda-tree',
    template: ``,
    props: {
        allowFocus: true,
        lazy: true,
        mode: 'class',
        _items: [],
        filterVal: '',
        focusedNode: {
            type: Object,
            async set(n) {
                if (n && n.fullId) {
                    this.item = await ODANT.findItem(n.fullId);
                }
            }
        },
        item: {
            type: Object,
            set(n) {
                if (n) {
                    this.fire('result');
                }
            }
        },
        defaultTemplate: 'oda-class-tree-cell',
        parent: {
            type: [HTMLElement, Object],
            set(n) {
                if (n) {
                    this.host.style.maxWidth = (n.host || n).offsetWidth;
                } else {
                    this.host.style.maxWidth = '';
                }
            }
        },
    },
    observers: [
        async function _itemsFilterChanged(_items, filterVal) {
            const filter = filterVal.toLowerCase();
            this.dataSet = [{ template: 'oda-table-loading' }];

            const prom = await _items.map(async (i) => {
                const context = i.$domain;
                const xq = `declare function local:func($node as node()*) as node()*{
if (${filter ? `contains(lower-case(concat($node/@n, '|', $node/@l)), '${filter}')` : '1'})
then
element items {
    $node/@*[name() != 'i'],
    attribute fullId { concat('${context.fullId}/C:', $node/@i) },
    for $a in $node/*
    return local:func($a)
}    
else
for $a in $node/*
return local:func($a)
};

local:func(C[1])`;

                const res = await ODANT.API.post(i.url + '/*', {
                    method: 'xquery',
                    prefix: 'true',
                    format: 'json-2'
                }, xq);
                return res;
            });

            try {
                const result = (await Promise.all(prom)).filter(Boolean);
                const data = result.reduce((array, item) => array.concat(item.items), []);

                this.dataSet = data && data.length ? data : [{ template: 'oda-empty-tree-cell' }];
            } catch (e) {
                this.dataSet = [{template: 'oda-empty-tree-cell'}];
            }
        }
    ],
    attached() {
        this.async(() => {
            this.$refs.filterInput.focus();
        }, 300);
    },
    async show() {
        this.opened = true;
        return new Promise((resolve, reject) => {
            const callback = (e) => {
                this.unlisten(this, 'result', callback);
                this.unlisten(this, 'cancel', cancel);
                this.opened = false;
                resolve(this.item);
            };
            const cancel = (e) => {
                this.unlisten(this, 'result', callback);
                this.unlisten(this, 'cancel', cancel);
                this.opened = false;
                reject();
            };
            this.listen(this, 'result', callback);
            this.listen(this, 'cancel', cancel);
        });
    },
    _setFilterValue(e) {
        this.debounce('filtering', () => {
            this.filterVal = this.$refs.filterInput.value;
        }, 300);
    }

})