import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/grids/tree/tree.js';
import '../../../item/lib/item-node/item-node.js';

ODA({
    is: 'odant-class-tree',
    extends: 'oda-tree',
    template: `
    <style>
        :host {
            @apply --vertical;
            @apply --flex;
            min-width: 300px;
        }
    </style>`,
    props: {
        contextItem: Object,
        lazy: true,
        mode: 'class',
        _items: [],
        filterVal: '',
        defaultTemplate: 'odant-class-tree-cell',
        allowFocus: true
    },
    observers: [
        '_load(contextItem, filterVal)',
    ],
    async _load(context = this.contextItem, filterVal = this.filterVal) {
        if (context) {
            const prefixes = context.itemType === odaItem.Types.odaClass ? 'C, B, P' : 'B, P';
            const xq = `
declare function local:func($node as node()*, $modId as xs:string?) as node()*{
if (${filterVal ? `contains(lower-case(concat($node/@n, '|', $node/@l)), '${this._getFilter()}')` : '1'})
then
element items {
    $node/@*[name() != 'i'],
    attribute fullId { if ($modId) then concat('${context.fullId}/M:', $modId, '/C:', $node/@i) else concat('${context.fullId}/C:', $node/@i) },
    for $a in $node/*
    return local:func($a, $modId)
}
else
for $a in $node/*
return local:func($a, $modId)
};

let $r1 :=
for $a in (${prefixes})[@i='${context.id}']/C
return local:func($a, '')

let $r2 :=
for $b in (${prefixes})[@i='${context.id}']/M
for $c in $b/C
    return local:func($c, $b/@i)

return ($r1, $r2)`;
            try {
                this.dataSet = [{ template: 'oda-table-loading' }];
                const res = await ODANT.API.post(context.url + '/*', {
                    method: 'xquery',
                    prefix: 'true',
                    format: 'json-2'
                }, xq);
                this.dataSet = res && res.items && res.items.length ? res.items : [{ template: 'oda-empty-tree-cell' }];
            } catch (e) {
                this.dataSet = [{ template: 'oda-empty-tree-cell' }];
            }
        }
    },
    _getFilter(filter = this.filterVal) {
        return (filter ? filter.replace(/\s{2}/g, ' ').trim() : '').toLowerCase();
    }
});

ODA({
    is: 'odant-class-tree-cell',
    extends: 'oda-table-cell-base',
    template: `<odant-item-node ~if="_intersect" :context="item.fullId"></odant-item-node>`,
    props: {
        hidePrefix: true,
        hideSuffix: true,
    }
});