import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/grids/tree/tree.js';
import '../../../item/lib/item-node/item-node.js';

ODA({
    is: 'odant-inherit-tree',
    extends: 'oda-tree',
    template: `
    <style>
        :host {
            @apply --vertical;
            @apply --flex;
            min-width: 300px;
        }
    </style>`,
    props: {
        contextItem: Object,
        lazy: true,
        mode: 'class',
        _items: [],
        filterVal: '',
        defaultTemplate: 'odant-inherit-tree-cell',
        allowFocus: true
    },
    observers: [
        '_load(contextItem, filterVal)',
    ],
    async _load(contextItem = this.contextItem) {
        if (contextItem) {
            this.async(async () => {
                this.mode = contextItem.itemTypeLabel;
                const func = `declare function local:func($node as node()*, $ids as xs:string?) as node()*{
                                for $a in $node/C[not(contains($ids, @i))]${this.filterVal && this.filterVal !== '*' ? `[contains(@n, '${this._getFilter()}')]` : ''}
                                    group by $a/@i
                                    let $last := oda:last($a)
                                return element items{
                                    attribute fullId {string-join(reverse($last/ancestor-or-self::*/concat(name(),':',@i)), '/')},
                                    attribute id {$last/@i},
                                    attribute itemName {$last/@n},
                                    local:func($last, $ids)
                                }
                            };`;
                const queryMap = {
                    class: `${func}

                            let $cid := '${contextItem.id}'
                            let $bid := '${contextItem.$base.id}'
                            let $context := //B[@i=$bid]/(C, C//C, M//C)[@i=$cid]
                            let $ancestors := $context/(ancestor::B, ancestor::P)/(C, C//C, M//C)[@i=$cid]
                            let $ids := string-join($context//C/@i, '|')
                            return local:func($ancestors, $ids)`,
                    base: `${func}

                            let $bid := '${contextItem.id}'
                            let $context := //B[@i=$bid]
                            let $ancestors := $context/(ancestor::B, ancestor::P)/B
                            let $ids := string-join($context//C/@i, '|')
                            return local:func($ancestors, $ids)`,
                };
                const xq = queryMap[this.mode] || queryMap['class'];
                try {
                    const res = await ODANT.API.post(this.contextItem.$connection.url + '/P:ROOT/*', {
                        method: 'xquery',
                        prefix: 'true',
                        format: 'json-2'
                    }, xq);
                    
                    this.dataSet = res.items;
                } catch (e) {
                    this.dataSet = [];
                }
            }, 100);
        }
    },
    _getFilter(filter = this.filterVal) {
        return filter ? filter.replace(/\s{2}/g, ' ').trim() : '';
    },
    getChecked(items = this.dataSet) {
        if (!items || items.length === 0) return [];
        return items.reduce((res, i) => {
            if (i.checked) {
                const obj = Object.assign({}, i);
                if (obj.items && obj.items.length > 0) {
                    obj.items = this.getChecked(obj.items);
                }
                res.push(obj);
            }
            return res;
        }, [])
    }
})

ODA({
    is: 'odant-inherit-tree-cell',
    extends: 'oda-table-cell-base',
    template: `<odant-item-node ~if="_intersect" :context="item.fullId"></odant-item-node>`,
    props: {
        hidePrefix: true,
        hideSuffix: true,
    }
})