import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/grids/table/table.js';
import './item-list-editor.js';

ODA({
    is: 'odant-item-list-editor',
    extends: 'this, oda-table',
    template: `
    <style>
        :host {
            min-height: 300px;
            padding-left: 40px;
            @apply --flex;
        }
        [disabled] {
            @apply --disabled;
        }
        .horizontal {
            @apply --horizontal;
        }
        .vertical {
            @apply --vertical;
        }
    </style>
    <div class="no-flex header vertical" style="position: absolute; height: 100%; left: 0; top: 0;">
        <div class="vertical no-flex">
            <oda-button :size="iconSize" title="Add" @tap="_addRow" icon="icons:add"></oda-button>
            <oda-button :size="iconSize" title="Remove" @tap="_removeRow" icon="icons:delete"></oda-button>
        </div>
    </div>`,
    props: {
        allowFocus: true,
        allowSelection: true,
        autoWidth: true,
        showHeader: false,
        showFooter: false,
        colLines: false,
        rowLines: true,
        defaultTemplate: 'odant-item-cell',
        contextItem: Object,
        dataSet: Array,
        columns: [
            { name: 'link', template: 'odant-item-cell'  }
        ],
    },
    async _addRow(e) {
        this.dataSet.push({});
        this.selectItem(this.rows[this.rows.length - 1]);
    },
    _removeRow(e) {
        let index = 0;
        this.selection.forEach(i => {
            const idx = this.dataSet.indexOf(i);
            if (!index)
                index = idx;
            else
                index = Math.min(index, idx);
            this.dataSet.splice(idx, 1);
        });
        this.clearSelection();
        this.async(() => {
            if (index >= this.dataSet.length)
                index = this.dataSet.length - 1;
            this.selectItem(this.dataSet[index]);
        }, 100)
    }
})

ODA({
    is: 'odant-item-list-editor',
    extends: 'this, oda-table'
})

ODA({
    is: 'odant-item-cell',
    extends: 'oda-table-cell-base',
    template: `
    <style>
        :host>odant-type-editor {
            overflow: hidden;
            text-overflow: ellipsis;
        }
    </style>
    <odant-type-editor class="flex" :context-item="contextItem" :$class="$class" @$class-changed="_classChanged"></odant-type-editor>`,
    props: {
        item: Object,
        initialized: false,
        contextItem: Object,
        $class: Object,
        table: Object
    },
    observers: [
        function _itemContextItemChanged(item, contextItem) {
            this.debounce('odant-item-cell-init', async () => {
                if (!item || !contextItem) return;

                try {                        
                    const link = item[this.column.name];
                    if (link) {
                        this.$class = await this.contextItem.findItem(link);
                    }
                } catch (e) {}

                this.initialized = true;
            }, 300);
        }
    ],
    attached() {
        this.table = this.domHost.table;
        this.contextItem = this.table.contextItem;
    },
    _classChanged(e) {
        if (this.initialized) {
            this.$class = e.target.$class;
            const link = (e.target.$class) ? this.contextItem.getRelativeId(e.target.$class) : '';
            this.item[this.column.name] = link;
        }
    }
})