import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/buttons/button/button.js';

ODA({
    is: 'odant-item-list-editor',
    template: `
    <style>
        :host {
            @apply --horizontal;
            @apply --flex;
        }

        label {
            align-self: center;
            border: none;
            background-color: transparent;
            overflow: hidden;
            text-overflow: ellipsis;
            width: 100%;
            padding: 4px;
            margin-left: 2px;
            margin-right: 2px;
            font-family: monospace;
        }
    </style>
    <label class="flex">{{label}}</label>
    <oda-button class="no-flex" :size="size" icon="icons:more-horiz" @tap="_onTap"></oda-button>`,
    props: {
        size: 24,
        value: {
            type: String,
            set(v) {
                this.label = (v) ? `[${v.split('|').length}]` : `[0]`;
            }
        },
        contextItem: Object,
        label: {
            type: String,
            default: '[0]'
        },
        item: {
            type: Object,
            async set(i) {
                if (!i) return;
                i.io = null;
                i.items = null;
            }
        },
    },
    attached() {
        this.table = this.domHost.table;
        this.contextItem = this.table.inspectedObject;
    },
    async _onTap(e) {
        const url = this.resolveUrl('item-list-dialog.js');
        const selector = await ODA.createComponent(url);
        selector.contextItem = this.contextItem.$base;
        const data = this.item.owner[this.item.name];
        selector.dataSet = (data) ? data.split('|').map((link) => ({ link: link })) : [];

        const dialog = {
            title: 'Select classes',
            icon: 'odant:class',
            ok: {
                show: true,
                disable: false
            },
        };
        const dialogResult = await ODA.showDialog(selector, dialog).catch((e) => {});
        if (dialogResult) {                    
            const value = selector.dataSet.reduce((acc, row) => {
                if (!row || !row.link) return acc;

                if (acc.length > 0) {
                    acc += '|';
                }
                
                return acc + row.link;
            }, '');
            this.set(this, 'value', value);
            this.set(this.item.owner, this.item.name, value);
        } else {

        }
    }
})