import ODA from '../../../../oda/oda.js';
import '../../../item/lib/item-node/item-node.js';
import '../../../../oda/components/buttons/button/button.js';

ODA({
    is: 'odant-item-walker',
    extends: 'oda-tree',
    template: `
    <style>
        :host{
            @apply --horizontal;
        }
        [disabled]{
            @apply --disabled;
        }
    </style>
    <odant-item-node class="flex" :context-item="current"></odant-item-node>
    <oda-button :disabled="_path.indexOf(current) === 0" icon="icons:forward:90" @tap="_goDown"></oda-button>
    <oda-button :disabled="current && (current.id.endsWith('ROOT') || current.constructor.name === 'odaHost')" icon="icons:forward:270" @tap="_goUp"></oda-button>`,
    props: {
        contextItem: Object,
        current: {
            type: Object,
            notify: true
        },
        _path: {
            type: Array,
            default: []
        },
        itemKey: {
            type: String,
            default: '$base',
            enum: ['$base', '$class', '$owner', '$parent', '$domain', '$module']
        }
    },
    observers: [
        async function initCurrent(contextItem, itemKey) {
            if (contextItem) {
                if (['$base', '$domain'].includes(this.itemKey)) {
                    this.current = await this._getItem(contextItem, odaDomain);
                } else {
                    this.current = contextItem;
                }
                this._path.push(this.current);
            }
        }
    ],
    async _goUp() {
        const idx = this._path.indexOf(this.current);
        if (~idx && this._path[idx + 1]) {
            this.current = this._path[idx + 1];
        } else {
            switch (this.itemKey) {
                case '$owner':
                // case '$parent':
                case '$module': this.current = this.current[this.itemKey]; break;
                case '$class': this.current = await this._getItem(this.current.$owner, odaClass); break;
                case '$base':
                case '$domain':
                default: this.current = await this._getItem(this.current.$owner, odaDomain); break;
            }
            this._path.push(this.current);
        }
    },
    async _goDown() {
        const idx = this._path.indexOf(this.current);
        if (~idx) this.current = this._path[idx - 1];
    },
    async _getItem(item, constructor) {
        const type = item.itemType;
        const types = odaItem.Types;
        if (item instanceof constructor) {
            return item;
        } else if (type === types.odaHost) {
            return await item.findItem('P:ROOT').catch(e => console.error(e.name, e));
        } else {
            return item[this.itemKey];
        }
    },
});