import ODA from '../../../../oda/oda.js';
import '../../../../oda/components/grids/tree/tree.js';
import '../../../item/lib/item-node/item-node.js';
import '../../../../oda/components/layouts/dropdown/dropdown.js';
//import '../search-input.js'; // TODO
ODA({
    is: 'oda-item-search',
    template: `<style>
    item-search-spin {
        padding: 5px
    }
    .tool {
        margin: 5px
    }
</style>
<oda-search-input @search="_search"></oda-search-input>
<div ref="search">
    <div ~if="notFound" class="tool label"> not found</div>
    <oda-search-tree ref="dropdown" :parent="$refs.search" @opened-changed="_dropDownClose"
        vertical-align="bottom" style="background: white;"
        @focused-item-changed="focusedItemTree = $event.detail.value"
        class="flex"
        :context-item="contextItem"
        :data-set="dataSet">
    </oda-search-tree>
</div>`,
    properties: {
        contextItem: Object,
        dataSet: [],
        selectedItem: {
            type: Object,
            notify: true,
        },
        focusedItemTree: {
            type: Object,
            watch(v) {
                if (v && !v.$group) {
                    this.getSelectedItem(v.fullPath)
                }
            }
        },
        isLoading: false,
        notFound: false,
        findXQ: `declare function local:path($n as node()) as xs:string {
    let $p := $n/..
    return
        if ($p and $p/name())
        then concat(local:path($p),'/', $n/name(),':',$n/@i)
        else concat($n/name(),':',$n/@i)
};

declare function local:labelPath($n as node()) as xs:string {
    let $p := $n/..
    return
        if ($p and $p/name())
        then concat(local:labelPath($p),'/', if ($n/@l) then ($n/@l) else ($n/@n))
        else ''
};

let $d :=
    for $a in //*[contains(lower-case(concat(@l,@n)), lower-case('#FILTER#'))]
    return
        element x {
            attribute fullPath { local:path($a) },
            attribute label  {if ($a/@l) then ($a/@l) else ($a/@n)},
            attribute type {
                if($a/@t) then(
                    if($a/@t = "ORGANIZATION")
                        then ("BASE")
                        else ($a/@t)
                )
                else ("CLASS")
            },
            attribute labelPath{local:labelPath($a)}
        }
return
    for $type in distinct-values($d/@type)
    let $group := $d[@type = $type]
    return
        element x {
            attribute type {$type},

            for $el in $group
            order by $el/@label
            return
                element items {
                    $el/(
                        @fullPath,
                        @type,
                        @label,
                        @labelPath
                }
        }`
    },
    _dropDownClose(e) {
        if (!e.detail.value) {
            this.dataSet = [];
        }
    },
    async getSelectedItem(path) {
        if (!(this.contextItem instanceof odaHost)) {
            path = `${this.contextItem.fullId.replace(`/${this.contextItem.prefix}:${this.contextItem.id}`, '')}/${path}`;
        }
        this.selectedItem = await ODANT.findItem(path);
        this.$refs.dropdown.close();
    },
    async _search(e) {
        this.$refs.dropdown.close();
        this.dataSet = [];
        const value = e.detail.value;
        if (value) {
            const regExp = /^\/?(?:(?:[HPBSWDMCO]:(?:(?:[0-9A-F]{15})|(?:WORK)|(?:ROOT)|(?:USER)))\/?)*$/;
            if (regExp.test(value)) {
                this.selectedItem = await ODANT.findItem(value);
            } else {
                const xq = findXQ.replace('#FILTER#', value.trim());
                try {
                    this.isLoading = true;
                    const res = await this.contextItem.getConfig(xq);
                    console.log(res);
                    let dataSet = res.$x.reduce((results, item) => {
                        if (item.type === "SYSTEM" || item.type === "PART" || item.type === "EXECUTABLE")
                            return results;
                            
                        results.push(...item.$items);
                        return results;
                    }, []);
                    dataSet.sort((a, b) => a.type > b.type);
                    this.dataSet = dataSet;
                    this.$refs.dropdown.open();
                } catch (e) {
                    this.notFound = true;
                    this.debounce('show-error', () => {
                        this.notFound = false;
                    }, 1500);
                } finally {
                    this.isLoading = false;
                }
            }
        }
    },
});

ODA({
    is: "oda-item-search-item",
    extends: 'odant-item-node',
    observers: [
        function _setContext(item) {
            if (item.fullPath) {
                this.context = item.fullPath;
            }
        }
    ]
});

ODA({
    is: "oda-item-search-group",
    extends: 'oda-table-cell-group',
    attached() {
        if (this.item.items && ((this.item.items.length < 12) || (this.table.items.length === 1))) {
            this.set(this.item, '$expanded', true);
            this.table._refresh();
        }
    }
});

ODA({
    is: "oda-search-tree",
    extends: 'oda-tree, oda-dropdown',
    properties: {
        allowFocus: true,
        lazy: true,
        groups: [{ name: 'type' }],
        defaultGroupTemplate: 'oda-item-search-group',
        defaultTemplate: 'oda-item-search-item',
        contextItem: Object,
    }
});