import ODA from '../../../../oda/oda.js';
import '../class-finder/class-finder.js';
ODA({
    is: 'odant-type-editor',
    template: `
    <style>
        :host {
            @apply --horizontal;
        }
        :host > odant-class-finder {
            @apply --flex;
        }
    </style>
    <odant-class-finder :context-item="contextItem" :$class="$class" @$class-changed="_setClass($event.detail.value)"></odant-class-finder>`,
    props: {
        value: {
            type: String,
            notify: true,
        },
        item: {
            type: Object,
            set(n) {
                if (n && n.owner) {
                    this.contextItem = n.owner;
                }
            }
        },
        contextItem: {
            type: Object,
            async set(n) {
                if (n && n.type) {
                    this.value = n.type;
                    if (n.TypeClass) this.$class = n.TypeClass;
                    else {
                        this.$class = await this._findClass(n.type);
                    }
                } else {
                    this.$class = null;
                }
            }
        },
        $class: {
            type: Object,
            default: undefined,
            notify: true,
            set(n) {
                this.focused = true;
                if (n) {
                    this.value = this.contextItem.getRelativeId(n) || n.id;
                } else {
                    this.value = undefined;
                }
                this.async(() => this.focused = false);
            }
        },
        focused: false,
    },
    async _findClass(id, force = false) {
        try {
            if (id) {
                const source = force ? ODANT : this.contextItem.$class;
                const cls = await source.findItem(id);
                return cls || (!force && this._findClass(id, true));
            }
        } catch (err) {
            console.error(err);
        }
        return null;
    },
    _setClass(value) {
        if (this.contextItem) {
            this.$class = value;
        }
    }
})