const odant = require('../../../../odant.js');

odant.then(() => {

    ODANT.ssid = 'AE0D03565DDD28A3429EA32FB9F1F769';

    // Test class

    class ItemCopyAction extends odaAction {

        execute(params = {}) {
            console.log('methods/copy', params);
            console.log(new Date(Date.now()));

            if (!params.source) {
                throw new Error('Target missed');
            }

            return ODANT.findItem(params.source)
            .then((sourceItem) => {
                return this.$item.Items
                .then((children) => {
                    if (children.find(child => child.id === sourceItem.id)) {
                        throw new Error('Item allready exists');
                    }

                    const options = params.options || {};

                    return this.createItem(sourceItem)
                    .then((newItem) => {
                        if (options.copyFiles) {
                            return this.copyFiles(sourceItem, newItem, options)
                            .then(() => {
                                return newItem;
                            });
                        }

                        return newItem;
                    })
                    .then((newItem) => {
                        if (options.copyChildren) {
                            return this.copyChildren(sourceItem, newItem, options)
                            .then(() => {
                                return newItem;
                            })
                        }

                        return newItem;
                    })
                    .then((newItem) => {
                        return 'copy finished';
                    })
                    .catch((err) => {
                        console.dir(err);
                        return 'copy failed';
                    });
                });
            });
        }

        createItem(sourceItem) {
            console.log('createItem');

            const params = { name: sourceItem.name };

            let command = 'create_class';

            if (sourceItem.prefix !== 'C') {
                command = 'create_domain';
                params.type = this.$item._convertPrefixToType(sourceItem.prefix);
                params.prefix = true;
            }

            return this.$item.API.POST(command, params, '{"$CLASS":[' + JSON.stringify(sourceItem.Root) + ']}')
            .then((res) => {
                if (sourceItem.prefix === 'C') {
                    return this.$item.getClass(res.$C[0].i);
                }

                return this.$item.getDomain(res.i, sourceItem.prefix);
            })
            .catch((err) => err);
        }

        copyFiles(source, target, options) {
            console.log('copyFiles');

            return source.getFilesAndFolders()
            .then((children) => {
                const exclude = ['class.ocl']
                return children.filter(child => child.inherit.none && (exclude.findIndex(excl => excl === child.id)) === -1);
            })
            .then((children) => {
                return Promise.all(children.map((child) => {
                    return 'ok';
                }));
            });
        }

        copyChildren(sourceItem, target, options) {
            console.log('copyChildren');

            return sourceItem.Items
            .then((children) => {
                return children.filter((child) => child.id !== '000000000000000');
            })
            .then((children) => {
                return Promise.all(children.map((child) => {
                    return ODANT.executeAction(target.fullId, 'methods/copy', { options, source: child.fullId, });
                }));
            });
        }
    }

    // copyChildren(sourceItem, target, options) {
    //     console.log('copyChildren');
    //
    //     return sourceItem.Items
    //     .then((children) => {
    //         return children.filter((child) => child.id !== '000000000000000');
    //     })
    //     .then((children) => {
    //         return Promise.all(children.map((child) => {
    //             return ODANT.executeAction(target.fullId, 'methods/copy', { options, source: child.fullId, });
    //         }));
    //     });
    // }

})

/* http://www.odant.org/api/H:1D03A3F3B5863BB/D:WORK/D:1D151E87652DEA0/C:1CCDF1802C35D4E/help.php?method=move_file?oldfilename=http://www.odant.org/api/H:1D03A3F3B5863BB/D:WORK/D:1D151E87652DEA0/C:1CCDF1802C35D4E/help.htm&newfilename=http://www.odant.org/api/H:1D03A3F3B5863BB/D:WORK/D:1D151E87652DEA0/C:1CCDF1802C35D4E/index.html */

// Mocking execute

const params =
    {
        source: 'H:1D03A3F3B5863BB/P:WORK/B:1D2FAD813588530/C:1D2FAD81F788630',
        options: {
            copyFiles: true,
            copyChildren: true,
        }
    };

ODANT.findItem('H:1D03A3F3B5863BB/P:WORK/B:1D3629AC5F60260')
.then((item) => {
    return new ItemCopyAction({ $owner: item });
})
.then((action) => {
    return action.execute(params);
})
.then((ok) => {
    console.log(ok);
})
.catch((err) => {
    console.error(err);
})

