class ChartDataPacker {
    _data = [];
    constructor(data) {
        if (data) {
           this.setData(data)
        }
    };
    get data() {
        return this._data;
    };
    setData(data) {
        if (Array.isArray(data)) {
            this._data = data.slice();
        } else throw new Error('Error. Data = Array is expected in ChartDataPack(data).');
    }
    // pack(newSize, agregate, k, d): newSize - размер конечного массива, agregate - агрегирующая функция, k - масштаб, d - относительный сдвиг
    pack(agregate, newSize, k, agregateKind, data) {
        if (!newSize) newSize = this._data.length;
        if (newSize < 0) throw new Error('ChartDataPacker.pack(agregate, newSize, k, agregateKind, data): newSize < 0.');
        if (!agregate) agregate = this.setAgregateKind(agregateKind);
        const arr = this.scale(k, data);
        newSize = newSize > arr.length ? arr.length : newSize; // размер конечного массива должен быть меньше размера исходного массива
        const kI = newSize / arr.length; // Внимание! Возможна ошибка рассчетов из-за прогрешностей деления.
        let newArr = new Array(newSize);
        arr.map((item, i) => {
            const newI = Math.ceil(i * kI);
            newArr[newI] = agregate(newArr[newI], item);
        });
        return  newArr;
    };
    setAgregateKind(agregateKind) {
        switch (agregateKind) {
            case 'array': return this.agregateToArray;
            case 'min': return this.agregateToMin;
            case 'max': return this.agregateToMax;
            case 'first': return this.agregateToFirst;
            case 'last': return this.agregateToLast;
            case 'medium': return this.agregateToMedium;
            case 'sum': return this.agregateToSum;
            default: return this.agregateToArray;
        }
    }
    // agregateToArray(newValue, [ oldValue ]) newValue=String or Number, oldValue = String/Number/Array. Return: Array
    agregateToArray(oldValue, newValue) {
        if (!oldValue) return [newValue];
        else if (Array.isArray(oldValue)) {
            oldValue.push(newValue);
            return oldValue.slice();
        } else return [oldValue, newValue];
    };
    agregateToMin(oldValue, newValue) {
        oldValue = +oldValue;
        newValue = +newValue;
        return (oldValue > newValue) ? newValue : oldValue;
        
    };
    agregateToMax(oldValue, newValue) {
        oldValue = +oldValue;
        newValue = +newValue;
        return (oldValue > newValue) ? oldValue : newValue;
    };
    agregateToFirst(oldValue, newValue) {
        return oldValue;
    };
    agregateToLast(oldValue, newValue) {
        return newValue;
    };
    agregateToMedium(oldValue, newValue) {
        oldValue = +oldValue;
        newValue = +newValue;
        return (oldValue + newValue) / 2;
    };
    agregateToSum(oldValue, newValue) {
        oldValue = +oldValue;
        newValue = +newValue;        
        return oldValue + newValue;
    };
    // // k - коэффициент масштабирования, d - сдвиг относительно размера базового массива. Возвращает массив, размером = data.length
    // extractRange(k, d) {
    //     const arr = this._data;
    //     const size = arr.length;
    //     const absD = d * size;
    //     k = k >= 1 ? k : 1; //  при k < 1 функция будет работать некорректно
    //     let newArr = new Array(size);
    //     arr.map((item, i) => {
    //         let newIndex = Math.floor(i * k + absD);
    //         if ((newIndex >= 0) && (newIndex) < size) newArr[newIndex] = item;
    //     });
    //     return newArr;
    // };
    scale(k, data) { // сделать: пропуск вычислений при k = 1
        if (!k || k < 0) k = 1;
        if (data === undefined) data = this._data;
        const arr = data;
        const newSize = Math.round(arr.length * k);
        k = k >= 1 ? k : 1; //  при k < 1 функция будет работать некорректно
        let newArr = new Array(newSize);
        arr.map((item, i) => {
            let newIndex = Math.floor(i * k);
            newArr[newIndex] = item;
        });
        return newArr;
    };
    // packRanged(start, end, newSize) return array of packed range of data from index=start to index=end, size = newSize
    packRanged(agregate, newSize, start, end, agregateKind, data) { // not tested
        if (data === undefined) data = this._data;
        return pack(agregate, newSize, 1, agregateKind, this.range(start, end));
    };
    range(start, end) { // not tested
        try {
            if (isNaN(start) || isNaN(end)) throw new Error('Error. ChartDataPacker.range(start, end): start and end must to have a Number type. ');
            if ((start < 0) || (end < 0)) throw new Error('Error. ChartDataPacker.range(start, end): start and end must to be >= 0.');
            if (start > end) throw new Error('Error. ChartDataPacker.range(start, end): start must be <= end.');
            if (end >= this._data.length) throw new Error('Error. ChartDataPacker.range(start, end): end is out of data range.');
            return this._data.slice(start, end + 1);
        } catch (e) {
            console.log(e);
        }
    };
}