exports.DataManipulator = class DataManipulator {
    constructor(data = [], baseName = '', targetName = '', restrictorName = '', restrictorValues = [], toNumber = true) {
        if (data) {
            this._data = this._setData(data);
            this._baseName = this._setBaseName(baseName);
            this._targetName = this._setTargetName(targetName);
            this._restrictorName = this._setRestrictorName(restrictorName);
            this._restrictorValues = this._setRestrictorValues(restrictorValues);
            this._extractFields(this._data, this._baseName, this._targetName, this._restrictorName, this._restrictorValues, toNumber);
        }
    };
    get data() {
        return this._data;
    }
    get size() {
        return this._data.length;
    }
    get base() {
        return this._data.map(item => item.base);
    }
    get items() {
        let result = new Array(this._data[0].items.length).fill(new Array(1));
        result = result.map(() => []);
        this._data.map(item => {
            item.items.map((value, i) => result[i].push(value));
        });
        return result;
    }
    get incorrects() {
        return this._data.map(item => item.incorrects);
    }
    _setData(data) {
        if (!data) throw new Error('DataManipulator._setData(data): data is not defined.');
        if (!Array.isArray(data)) throw new Error('DataManipulator._setData(data): data is not an Array type.');
        return data.slice();
    }
    _setBaseName(fieldName) {
        if (typeof fieldName === 'string') return fieldName;
        else return new Error('_setBaseName(fieldName)');
    }
    _setTargetName(fieldName) {
        if (typeof fieldName === 'string') return fieldName;
        else return new Error('_setTargetName(fieldName)');
    }
    _setRestrictorName(fieldName) {
        if (typeof fieldName === 'string') return fieldName;
        else return new Error('_setRestrictorName(fieldName)');
    }
    _setRestrictorValues(array) {
        if (Array.isArray(array)) return array.slice();
        else return new Error('_setRestrictorValues(array)');
    }
    _extractFields(data, baseName, targetName, restrictorName, restrictorValues, toNumber) {
        if (!data) return console.log('ERROR in ChartDataManupulate.extractFields(fields): _data is not defined.');
        const base = this._setBaseName(baseName);
        const target = this._setTargetName(targetName);
        const restrictor = this._setRestrictorName(restrictorName);
        const rValues = this._setRestrictorValues(restrictorValues);
        let newData = data.map(item => {
            let obj = {};
            obj.base = item[base];
            obj.items = [];
            obj.incorrects = [];
            if (!restrictor || !rValues || !rValues.length) {
                if (toNumber) {
                    const value = this._toNumber(item[target]);
                    obj.incorrects.push(value === null ? true : false);
                    obj.items.push(value);
                } else obj.items.push(item[target]);
            } else {
                const targetValue = item[target];
                const currentRestiction = item[restrictor];
                rValues.map(restrictorValue => {
                    if (toNumber) {
                        const value = this._toNumber(targetValue);
                        obj.incorrects.push(value === null ? true : false);
                        obj.items.push((restrictorValue == currentRestiction) ? value : null);
                    } else obj.items.push((restrictorValue == currentRestiction) ? targetValue : null);
                });
            }
            return obj;
        });
        this._data = newData;
        return newData;
    }
    sort() {
        this._data.sort(compareNumberBase);
        function compareNumberBase(a, b) {
            a = +a.base;
            b = +b.base;
            if (a > b) return 1;
            else if (a < b) return -1;
            else return 0;
        }
    }
    // agregate(func) return array that is map of data agregated by func function
    agregate(func) {
        this._data = this._data.reduce((acc, curr) => {
            const index = acc.findIndex(item => item.base === curr.base);
            if (index == -1) acc.push(curr);
            else acc[index].items = acc[index].items.map((item, i) => func(item, curr.items[i]));
            return acc;
        }, []);
    }
    _toNumber(value) {
        const formattedValue = +String(value).replace(',', '.')
        return isNaN(formattedValue) ? null : formattedValue;
    }
}