const assert = chai.assert;

function compareArrays(arr1, arr2) {
    if (Array.isArray(arr1) && Array.isArray(arr2)) {
        if (arr1.length === arr2.length) {
            let result = true
            for (let i = 0; i < arr1.length; i++) {
                if (Array.isArray(arr1[i])) result = result && compareArrays(arr1[i], arr2[i]);
                else result = result && (arr1[i] === arr2[i]);
            } 
            return result;
        } else return false
    } else return false
        
}

describe('ChartDataPacker', function () {
    it('Type of ChartDataPacker = Object', function () {
        assert.equal('object', typeof new ChartDataPacker);
    });
    it('Type of ChartDataPacker.data = Array', function () {
        let a = new ChartDataPacker;
        let d = a.data;
        assert.equal(true, Array.isArray(d));
    });
});


describe('agregateToArray(oldValue, newValue)', function () {
    const a = new ChartDataPacker();

    it('newValue=1, oldValue=Undefined return [1]', function () {
        let newValue = 1;
        let returned = a.agregateToArray(null,1);
        let expected = [1];
        assert.equal(true, compareArrays(expected, returned));
    });
    it('newValue=1, oldValue=2 return [2,1]', function () {
        let newValue = 1;
        let returned = a.agregateToArray(2,1);
        let expected = [2,1];
        assert.equal(true, compareArrays(expected, returned));
    });
    it('newValue=1, oldValue=[2] return [2, 1]', function () {
        let newValue = 1;
        let returned = a.agregateToArray([2], 1);
        let expected = [2, 1];
        assert.equal(true, compareArrays(expected, returned));
    });
    it('newValue=1, oldValue=[2, 3] return [2, 3, 1]', function () {
        let newValue = 1;
        let returned = a.agregateToArray([2, 3], 1);
        let expected = [2, 3, 1];
        assert.equal(true, compareArrays(expected, returned));
    });
});
describe('extractRange(k, d) for data=[1, 2, 3, 4, 5]', function () {
    let a = new ChartDataPacker([1, 2, 3, 4, 5]);
    it('k = 0, d = 0 return [1, 2, 3, 4, 5]', function () {
        const returned = a.extractRange(0, 0);
        const expected = [1, 2, 3, 4, 5];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 1, d = 0 return [1, 2, 3, 4, 5]', function () {
        const returned = a.extractRange(1, 0);
        const expected = [1, 2, 3, 4, 5];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 1, d = 1 return [undefined, undefined, undefined, undefined, undefined]', function () {
        const returned = a.extractRange(1, 1);
        const expected = [undefined, undefined, undefined, undefined, undefined];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 1, d = 0.5 return [undefined, undefined, 1, 2, 3]', function () {
        const returned = a.extractRange(1, 0.5);
        const expected = [undefined, undefined, 1, 2, 3];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 2, d = 0 return [1, undefined, 2, undefined, 3]', function () {
        const returned = a.extractRange(2, 0);
        const expected = [1, undefined, 2, undefined, 3];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 2, d = 0.2 return [undefined, 1, undefined, 2, undefined]', function () {
        const returned = a.extractRange(2, 0.2);
        const expected = [undefined, 1, undefined, 2, undefined];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 2, d = 0.3 return [undefined, 1, undefined, 2, undefined]', function () {
        const returned = a.extractRange(2, 0.3);
        const expected = [undefined, 1, undefined, 2, undefined];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 2, d = 0.4 return [undefined, undefined, 1, undefined, 2]', function () {
        const returned = a.extractRange(2, 0.4);
        const expected = [undefined, undefined, 1, undefined, 2];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
    it('k = 2, d = -0.3 return [2, undefined, undefined, 3, undefined]', function () {
        const returned = a.extractRange(2, -0.3);
        const expected = [2, undefined, 3, undefined, 4];
        let result = compareArrays(returned, expected);
        assert.equal(true, result);
    });
});
describe('pack(newSize, agregate, k)', function () {
    describe("pack(nNew) size return on input array size 101 (k=undefined", function () {
        const a = new ChartDataPacker(new Array(101).fill('hi'));

        it('nNew = 101 return 101', function () {
            assert.equal(101, a.pack(null, 101).length);
        });
        it('nNew = 100 return 100', function () {
            assert.equal(100, a.pack(null,100).length);
        });
        it('nNew = 1 return 1', function () {
            assert.equal(1, a.pack(null,1).length);
        });
        it('nNew = 1000 return 1000', function () {
            assert.equal(1000, a.pack(null,1000).length);
        });
    });
    describe('pack(agregate, newSize, k), k=undefined  for any data', function () {
        it('for data=[1,2,3,4,5,6,7,8,9,10,11], nNew=10 returns [[1,2], 3, 4, 5, 6, 7, 8, 9, 10, 11]', function () {
            let a = new ChartDataPacker([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]);
            let returned = a.pack(null, 10);
            const expected = [[1, 2], [3], [4], [5], [6], [7], [8], [9], [10], [11]];
            let result = compareArrays(returned, expected);
            assert.equal(true, result);
        }); 
        it('for data=[1,2,3,4,5,6,7,8,9,10,11], nNew=1 returns [[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]]', function () {
            let a = new ChartDataPacker([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]);
            let returned = a.pack(null, 1);
            const expected = [[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]];
            let result = compareArrays(returned, expected);
            assert.equal(true, result);
        }); 
        it('for data=[1,2,3,4,5,6,7,8,9,10,11], nNew=2 returns [[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11]]', function () {
            let a = new ChartDataPacker([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]);
            let returned = a.pack(null,2);
            const expected = [[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11]];
            let result = compareArrays(returned, expected);
            assert.equal(true, result);
        }); 
    });
});