var odaString = odaString || class odaString extends String {
    static get FORMATS_STRING() {
        return {
            'string': ['строка', 'string', 'normal'],
            'lower': ['lowercase', 'lower', 'low'],
            'upper': ['UPPERCASE', 'uppercase', 'upper', 'up'],
            'camel': ['CamelCase', 'camelcase', 'camel'],
            'lowercamel': ['lowerCamelCase', 'lowercamelcase', 'lowercamel'],
            'kebab': ['kebab-case', 'kebabcase', 'kebab'],
            'camelkebab': ['Camel-Kebab-Case', 'camel-kebab-case', 'camelkebabcase', 'camel-kebab', 'camelkebab'],
            'snake': ['snake_case', 'snakecase', 'snake'],
            'camelsnake': ['Camel_Snake_Case', 'camel_snake_case', 'camelsnakecase', 'camelsnake'],
            'screaming': ['SCREAMING_SNAKE_CASE', 'screaming_snake_case', 'screamingsnakecase', 'screamingsnake', 'screaming'],
        }
    }

    static get FORMATS_STRING_MASKS() {
        return [
            '# - any character',
            '0 - number',
            'a - letter',
            'A - letter',
            'b - letter, forced to lower case when entered',
            'B - letter, forced to upper case when entered',
            '~ - alphanumeric',
            '> - alphanumeric, forced to upper case when entered',
            '< - alphanumeric, forced to lower case when entered',
            '+7 (000) 000-00-00 - telephone number',
            'B 000 BB 00rus - avto number'
        ];
    }

    static getFormattedValue (value, args) {
        if (args.mask) value = this.getMaskedValue(value, args);
        if (!args.format) return value;
        args.separator = args.separator || '_';
        let frmt = (typeof args.format === 'string') ? args.format.trim().toLowerCase() : 'string'
        frmt = Object.keys(odaString.FORMATS_STRING).find(key => odaString.FORMATS_STRING[key].includes(frmt));
        frmt = frmt || 'string';
        switch (frmt) {
            case 'string':
                return value;
            case 'lower':
                return value.toLowerCase();
            case 'upper':
                return value.toUpperCase();
            case 'camel':
                value = value.toLowerCase();
                return value[0].toUpperCase() + value.slice(1).replace(/[-_\s]+([a-zа-яё])/g, (all, letter) => letter.toUpperCase());
            case 'lowercamel':
                value = value.toLowerCase();
                return value[0].toLowerCase() + value.slice(1).replace(/[-_\s]+([a-zа-яё])/g, (all, letter) => letter.toUpperCase());
            case 'kebab':
                return value.toLowerCase().replace(/[-_\s]+/g, '-');
            case 'camelkebab':
                value = value.toLowerCase().replace(/[-_\s]+/g, '-');
                return value[0].toUpperCase() + value.slice(1).replace(/-([a-zа-яё])/g, (all, letter) => '-' + letter.toUpperCase());
            case 'snake':
                return value.toLowerCase().replace(/[-_\s]+/g, '_');
            case 'camelsnake':
                value = value.toLowerCase().replace(/[-_\s]+/g, '_');
                return value[0].toUpperCase() + value.slice(1).replace(/_+([a-zа-яё])/g, (all, letter) => '_' + letter.toUpperCase());
            case 'screaming':
                return value.toUpperCase().replace(/[-_\s]+/g, '_').replace(/_+([a-zа-яё])/g, (all, letter) => '_' + letter);
        }
        return value;
    }

    static getMaskedValue (value, args) {
        args.separator = args.separator || '_';
        this.countSymbol = -1;
        let isSpecialSymbol = false;
        let maskedValue = '';
        let arrChar = args.mask.split('');
        arrChar.map((symb) => {
            if (isSpecialSymbol) {
                maskedValue += symb;
                isSpecialSymbol = false;
            } else if (symb === '`') {
                isSpecialSymbol = true;
            } else {
                maskedValue += this._getSymbol(value, symb, args.separator);
            }
        });
        return maskedValue;
    }

    static _getSymbol (value, symb, separator) {
        if (/[^0aAbB~><#]/.test(symb)) return symb;
        this.countSymbol += 1;
        if (value && value[this.countSymbol]) return this.checkSymbol(symb, value[this.countSymbol], separator);
        return separator;
    }

    static checkSymbol (symb, val, separator) {
        switch (symb) {
            case '#':                                                                   // any character
                return val;
            case '0':                                                                   // number
                return (/\d/.test(val)) ? val : separator;
            case 'a':                                                                   // letter
            case 'A':
                return (/[a-zа-яё]/i.test(val)) ? val : separator;
            case 'b':                                                                   // letter, forced to lower case when entered
                return (/[a-zа-яё]/i.test(val)) ? val.toLowerCase() : separator;
            case 'B':                                                                   // letter, forced to upper case when entered
                return (/[a-zа-яё]/i.test(val)) ? val.toUpperCase() : separator;
            case '~':                                                                   // alphanumeric
                return (/[a-zа-яё0-9]/i.test(val)) ? val : separator;
            case '>':                                                                   // alphanumeric, forced to upper case when entered
                return (/[a-zа-яё0-9]/i.test(val)) ? val.toUpperCase() : separator;
            case '<':                                                                   // alphanumeric, forced to lower case when entered
                return (/[a-zа-яё0-9]/i.test(val)) ? val.toLowerCase() : separator;
        }
        return '';
    }
}

if (!String['formats']) {
    Object.defineProperty(String, 'formats', {
        get() {
            return Object.keys(odaString.FORMATS_STRING);
        }
    })
}

if (!String['masks']) {
    Object.defineProperty(String, 'masks', {
        get() {
            return odaString.FORMATS_STRING_MASKS;
        }
    })
}

if (!String['getFormattedValue']) {
    Object.defineProperty(String.prototype, 'getFormattedValue', { enumerable: false, value: function (format = '', mask = '', separator = '') {
        let args = typeof format === 'string' ? { format, mask, separator } : format || {};
        return odaString.getFormattedValue(this.valueOf(), args)
    }});
}