import '../../components/buttons/button/button.js';

ODA({
    is: 'oda-plan-calculator', template: `
    <style>
        :host {
            overflow: auto;
        }
    </style>
    <div align="center" style="font-size:32px; color:gray; margin-top:16px;"><b>Select Your Plan</b></div>
    <div style="display:flex; flex-wrap:wrap; justify-content:center;">
        <oda-plan-calculator-block ~for="p in plans" :sets="p" :selected="selected" 
                :selectedColor="selectedColor" @down="selected=p.name"></oda-plan-calculator-block>
    </div>
    <div style="display:flex; align-items:center; justify-content:center">
        <oda-button :disabled="!selected" :style="'background-color:'+selectedColor" style="width:200px; font-size:28px; 
                padding:4px; margin:16px; border-radius:4px; border:1px solid darkgray">ORDER NOW</oda-button>
    </div>
    <div align="center" style="font-size:32px; color:gray; margin-top:16px;"><b>Custom Plan Calculator</b></div>
    <oda-plan-calculator-block :selected="selected" :selectedColor="selectedColor" @down="selected='CUSTOM PLAN'"></oda-plan-calculator-block>
    `,
    props: {
        plans: [
            { name: 'TEST', values: [1, 1024, 8, 1], custom: false },
            { name: 'SMALLBIZ', values: [2, 2048, 16, 3], custom: false },
            { name: 'PRO', values: [4, 4096, 32, 7], custom: false },
            { name: 'PREMIER', values: [6, 6144, 48, 10], custom: false },
            { name: 'ENTERPRICE', values: [8, 8192, 64, 14], custom: false },
        ],
        selected: '',
        selectedColor: '#00A9E5'
    },
});

ODA({
    is: 'oda-plan-calculator-block', template: `
    <style>
        .slider {
            -webkit-appearance: none;
            width: 100%;
            height: 2px;
            background: darkgray;
            outline: none;
            opacity: 0.7;
            -webkit-transition: .2s;
            transition: opacity .2s;
        }
        .slider:hover {
            opacity: 1;
        }
        .slider::-webkit-slider-thumb {
            -webkit-appearance: none;
            appearance: none;
            width: 20px;
            height: 20px;
            border-radius: 50%;
            background: darkgray;
            cursor: pointer;
        }
    </style>
    <div style="display:flex; flex-wrap:wrap; justify-content:center;">
        <div :style="selected===plan.name?'border-color:'+selectedColor:''" style="cursor:pointer; 
                        display:flex; flex-direction:column; border:1px solid darkgray; height:360px; min-width:288px; border-radius:4px; margin:8px">
            <div style="height:45px; border-bottom:1px solid darkgray; background-color:lightgray; display:flex; align-items:center;font-size:24px" 
                    :style="selected===plan.name?'border-color:'+selectedColor+';background-color:'+selectedColor:''">
                <oda-icon :fill="selected===plan.name?'white':''" :icon="selected===plan.name?'icons:check-box':'icons:check-box-outline-blank'" 
                        style="padding:8px" size=32></oda-icon><b>{{plan.name}}</b>
            </div>
            <div style="flex:1; padding:4px;">
                <div ~for="s in plan.sets" style="display:flex; align-items:center;">
                    <div style="display:flex; align-items:center; justify-content:center">
                        <div style="display:flex; align-items:center; justify-content:left; width:300px">
                            <oda-icon :icon="s.icon" style="padding:2px"></oda-icon>
                            <b>
                                <div>{{s.value}}{{s.label}}
                            </b>{{s.text}}
                        </div>
                    </div>
                    <div ~if="plan.custom" style="margin:0px 20px">
                        <input type="range" :min="s.min" :max="s.max" ::value="s.value" :step="s.step" class="slider">
                    </div>
                    <div ~if="plan.custom" style="width:60px">
                        <div align="right"><b>{{Math.floor((s.value-s.step)*s.price)}} $</b></div>
                    </div>
                </div>
            </div>
        </div>
        <div style="height:90px;border-top:1px solid darkgray;display:flex;align-items:center;justify-content:center" 
                :style="selected===plan.name?'border-color:'+selectedColor:''">
            <div style="font-size:28px;color:gray;"><b>\${{_getPrice(plan)}}</b>/mo</div>
        </div>
    </div>
    `,
    props: {
        sets: {
            type: Object,
            set(n) {
                if (n.name) this.plan.name = n.name;
                this.plan.custom = n.custom;
                for (let i = 0; i < n.values.length; i++) {
                    this.plan.sets[i].value = n.values[i] || this.plan.sets[i].value;
                }
            }
        },
        plan: {
            name: 'CUSTOM PLAN',
            sets: [
                { icon: 'enterprise:computer', label: ' CPU', text: ' number processors', min: 1, max: 64, step: 1, value: 1, price: 5 },
                { icon: 'hardware:memory', label: ' MB', text: ' max use memory', min: 1024, max: 32768, step: 1024, value: 1024, price: .02 },
                { icon: 'device:storage', label: ' GB', text: ' disk space', min: 8, max: 4096, step: 8, value: 8, price: .1 },
                { icon: 'office-set:telephone', label: ' calls', text: ' number of calls per month', min: 1, max: 99, step: 1, value: 1, price: 2 },
            ],
            custom: true
        },
        selected: '',
        selectedColor: '#00A9E5'
    },
    _getPrice(plan = this.plan) {
        let total = 0;
        plan.sets.forEach(p => total += (p.value - p.step) * p.price) || 0;
        total = Math.round(total) + ' ';
        return total;
    }
});