import '../icon/icon.js';
export default ODA({is: "oda-checkbox", template:`
    <style>
        :host {
            @apply --horizontal;
            justify-content: center;
            align-items: center;
        }
    </style>
    <oda-icon class="no-flex" :icon="icon" style="cursor: pointer"></oda-icon>`,
    props: {
        iconSize: {
            default: 24,
            shared: true
        },
        icon:{
            get(){
                return this.icons[this.state];
            }
        },
        icons:{
            freeze: true,
            default:{
                checked: 'icons:check-box',
                unchecked: 'icons:check-box-outline-blank',
                indeterminate: 'icons:check-box-indeterminate'
            }
        },
        value: {
            type: Boolean,
            notify: true,
            set(newVal) {
                if (!this.threeStates)
                    this.state = newVal ? 'checked' : 'unchecked';
            }
        },
        state: {
            list: ['unchecked', 'checked', 'indeterminate'],
            notify: true,
            default: 'unchecked',
            set(n){
                this.value = n === 'checked';
            }
        },
        threeStates: false,
        _states:{
            type: Array,
            get(){
                return this.properties.state.list
            }
        }
    },
    listeners: {
        tap(e) {
            e.stopPropagation();
            if (this.threeStates) {
                let states = this._states;
                let newStateIndex = (this._getIndexOfState(this.state) + 1) % 3;
                this.state = states[newStateIndex];
            } else
                this.value = !this.value;
        }
    },
    observers: [
        function _setState(state, threeStates) {
            if (!threeStates && state === 'indeterminate') {
                this.state = 'unchecked';
            }
        }
    ],
    _getIndexOfState(state) {
        return this._states.indexOf(state);
    }
});
