import ODA from '../../../oda.js';
let im = import.meta;
let modulepath = import.meta.url;
modulepath = modulepath.slice(0, modulepath.lastIndexOf('/'));
const libs = {};
const icons = {};
const pars = new DOMParser();
async function loadIcons(name) {
    let content = libs[name];
    if (!content) {
        try {
            const doc = await ODA.loadHTML(ODA.rootPath + '/icons/svg/' + name + '.html');
            const tmpl = doc.querySelector('template');
            content = tmpl.content;
            content.size = +tmpl.getAttribute('size') || 0;
            libs[name] = content;
        }
        catch (e) {
            console.error(e);
            return;
        }
    }
    return content;
}
async function getIcon(n) {
    const key = n;
    if (!n.includes(':')) {
        if (!n.includes('.')) n += '.png';
        if (!n.includes('/')) n = ODA.rootPath + '/icons/png/' + n;
    }
    switch (_getIconType(n)) {
        case 'svg': {
            let obj = icons[key];
            if (!obj) {
                n = n.split(':');
                let name = n.shift();
                let content = await loadIcons(name);
                n = n[0];
                const g = content.getElementById(n);
                if (!g) return {};
                obj = { body: g.outerHTML, size: (+g.getAttribute('size') || content.size) };
            }
            return obj;
        } break;
        case 'img':
        default: {
            if (n) {
                try {
                    if (icons[key]) return icons[key];
                    let url = /* _computeImageUrl */(n);
                    // url = await CORE.API.checkUrl(url); // CORE?
                    if (url) {
                        const reader = new FileReader();
                        icons[key] = await new Promise(async resolve => {
                            reader.readAsDataURL(await (await fetch(url)).blob());
                            reader.onloadend = function () { resolve(reader.result); }
                        });
                    }
                    return icons[key];
                }
                catch (err) {
                    console.warn(err);
                }
            }
        } break;
    }
}
function _getIconType(str) {
    if (str.includes(':') && !str.includes('/'))
        return 'svg';

    return 'img'
}
function _computeImageUrl(str) {
    let url = str;
    if (url.startsWith('./')) {
        url = url.substring(2);
        let host = this;
        while (host && host.domHost) {
            host = host.domHost;
        }
        url = host.resolveUrl(url);
    } else if (!url.includes(':') && !url.includes('/web/')) {
        url = '/web/res/icons/' + url;
        if (!url.endsWith('.png') && !url.endsWith('.jpg'))
            url += '.png';
    }

    return url;
}
ODA({
    is: 'oda-icon', template: /* html */`
    <style>
        :host {
            display: flex;
            @apply --no-flex;
        }
        :host > svg{
            min-width: {{iconSize}}px;
            min-height: {{iconSize}}px;
            max-width: {{iconSize}}px;
            max-height: {{iconSize}}px;
        }
    </style>
    <svg :show="_icon && !hideIcon" :style="_style" :stroke="stroke" :fill="fill" xmlns="http://www.w3.org/2000/svg" preserveAspectRatio="xMidYMid meet" :view-box="\`0 0 \${_svgSize} \${_svgSize}\`">
        <defs ~if="blink">
            <g is="style" type="text/css">
                @keyframes blinker { 100% { opacity: 0; } }
                g { animation: {{blink}}ms ease blinker infinite; }
            </g>
        </defs>
        <g ~html="_icon && _icon.body"></g>
    </svg>`,
    props: {
        rotate: 0,
        _icon: null,
        _rotate: 0,
        _svgSize() {
            return (this._icon && this._icon.size) || 0;
        },
        icon: {
            reflectToAttribute: true,
            set(n) {
                if (n) {
                    if (n in icons) this._icon = icons[n];
                    const id = setTimeout(() => {
                        this._icon = '';
                    }, 150);
                    if (!n) {
                        return;
                    }
                    new Promise(async () => {
                        if (/\:[0-9]+$/.test(n)) {
                            let s = n.match(/\:[0-9]+$/)[0];
                            n = n.replace(s, '');
                            this._rotate = +s.substring(1);
                        }
                        else
                            this._rotate = 0;

                        this._icon = await getIcon(n);
                        clearTimeout(id);
                    });
                }

            }
        },
        hideIcon: false,
        iconSize: {
            default: 24,
            reflectToAttribute: true,
        },

        stroke: '',
        fill: {
            default: '',
        },
        blink: 0,
        _url: '',
        _svg: null,
        _style: {
            freeze: true,
            get() {
                const s = this.iconSize + 'px';
                const obj = { width: s, /*minWidth: s, */height: s, minHeight: s };
                const r = this.rotate + this._rotate;
                if (r)
                    obj.transform = `rotate(${r}deg)`;
                if (typeof this._icon === 'string') {
                    obj.backgroundImage = `url("${this._icon}")`;
                    obj.backgroundRepeat = 'no-repeat';
                    obj.backgroundSize = 'contain';
                    obj.backgroundPosition = 'center';
                }
                return obj;
            }
        }
    }
});