import '../../buttons/button/button.js';
import './ace-custom-elements.js';

ODA({
    is: 'oda-ace-editor', template: `
        <style> :host { display: block; position: relative; overflow: auto; } </style>
        <ace-custom-elements ref='editor'></ace-custom-elements>
        <oda-button ~if="showCopyButton" icon="icons:content-copy" style="position: absolute; right: 12px; top: 4px;" @tap="_copy"></oda-button>
    `,
    props: {
        editor: Object,
        default: {
            mode: 'ace/mode/javascript',
            theme: 'ace/theme/chrome',
            highlightActiveLine: true,
            highlightSelectedWord: true,
            readOnly: false,
            cursorStyle: 'slim', // "ace" | "slim" | "smooth" | "wide"
            //mergeUndoDeltas: false | true | "always"
            //behavioursEnabled: boolean,
            //wrapBehavioursEnabled: boolean,
            autoScrollEditorIntoView: true,
            copyWithEmptySelection: false,
            useSoftTabs: false,
            navigateWithinSoftTabs: false,
            enableMultiselect: true,
            hScrollBarAlwaysVisible: false,
            vScrollBarAlwaysVisible: false,
            highlightGutterLine: false,
            animatedScroll: true,
            showInvisibles: false,
            showPrintMargin: true,
            printMarginColumn: 80,
            //printMargin: false | number
            fadeFoldWidgets: false,
            showFoldWidgets: true,
            showLineNumbers: true,
            showGutter: true,
            displayIndentGuides: true,
            fontSize: "13pt",
            //fontFamily: css font-family value
            maxLines: Infinity,
            minLines: 3,
            //scrollPastEnd: number | boolean // number of page sizes to scroll after document end (typical values are 0, 0.5, and 1)
            fixedWidthGutter: false,
            firstLineNumber: 1,
            //overwrite: boolean,
            //newLineMode: "auto" | "unix" | "windows"
            useWorker: false,
            //useSoftTabs: boolean,
            //tabSize: number,
            wrap: true,
            foldStyle: 'markbeginend' //"markbegin" | "markbeginend" | "manual",
        },
        options: {},
        src: {
            default: '',
            set(n) { this.value = n }
        },
        value: {
            type: String,
            notify: true,
            set(n) {
                if (this.editor && !this._valueChangedOnEditor) {
                    this.editor.setValue(n);
                    this.editor.session.selection.clearSelection();
                }
            }
        },
        lines: {
            type: Number,
            set(n) { if (this.editor) { this.editor.setOptions({ maxLines: n, minLines: n }) } }
        },
        mode: {
            type: String,
            default: '',
            list: [
                'abap', 'abc', 'actionscript', 'ada', 'apache_conf', 'apex', 'applescript', 'aql', 'asciidoc', 'asl', 'assembly_x86',
                'autohotkey', 'batchfile', 'bro', 'c9search', 'cirru', 'clojure', 'cobol', 'coffee', 'coldfusion',
                'crystal', 'csharp', 'csound_document', 'csound_orchestra', 'csound_scope', 'csp', 'css', 'curly',
                'c_cpp', 'd', 'dart', 'diff', 'django', 'dockerfile', 'dot', 'drools', 'edifact',
                'eiffel', 'ejs', 'elixir', 'elm', 'erlang', 'forth', 'fortran', 'fsl', 'ftl', 'gcode', 'gherkin', 'gitignore',
                'glsl', 'gobstones', 'golang', 'graphqlschema', 'groovy', 'haml', 'handlebars', 'haskell', 'haskell_cabal', 'haxe',
                'hjson', 'html', 'html_elixir', 'html_ruby', 'ini', 'io', 'jack', 'jade', 'java', 'javascript', 'json', 'json5',
                'jsoniq', 'jsp', 'jssm', 'jsx', 'julia', 'kotlin', 'latex', 'less', 'liquid', 'lisp', 'livescript', 'logtalk',
                'live_script', 'logiql', 'lsl', 'lua', 'luapage', 'lucene', 'makefile', 'markdown', 'mask', 'matlab',
                'maze', 'mel', 'mixal', 'mushcode', 'mysql', 'nix', 'nsis', 'objectivec', 'ocaml', 'pascal',
                'perl', 'perl6', 'pgsql', 'php', 'php_laravel_blade', 'pig', 'plain_text', 'powershell', 'praat', 'prolog', 'properties',
                'protobuf', 'puppet', 'python', 'r', 'razor', 'rdoc', 'red', 'redshift', 'rhtml', 'rst', 'ruby', 'rust', 'sass', 'scad', 'scala',
                'scheme', 'scss', 'sh', 'sjs', 'slim', 'smarty', 'snippets', 'soy_template', 'space', 'sparql', 'sql', 'sqlserver',
                'stylus', 'svg', 'swift', 'swig', 'tcl', 'tex', 'text', 'textile', 'toml', 'tsx', 'turtle', 'twig', 'typescript',
                'vala', 'vbscript', 'velocity', 'verilog', 'vhdl', 'visualforce', 'wollok', 'xml', 'xquery', 'yaml', 'zeek'],
            set(n) { if (this.editor) this.editor.session.setMode("ace/mode/" + n) }
        },
        theme: {
            type: String,
            default: 'chrome',
            list: ['ambiance', 'chaos', 'chrome', 'clouds', 'clouds_midnight', 'cobalt', 'crimson_editor', 'dawn', 'dracula',
                'dreamweaver', 'eclipse', 'github', 'gob', 'gruvbox', 'idle_fingers', 'iplastic', 'katzenmilch', 'kr_theme', 'kuroir',
                'merbivore', 'merbivore_soft', 'monokai', 'mono_industrial', 'pastel_on_dark', 'solarized_dark',
                'solarized_light', 'sqlserver', 'terminal', 'textmate', 'tomorrow', 'tomorrow_night', 'tomorrow_night_blue',
                'tomorrow_night_bright', 'tomorrow_night_eighties', 'twilight', 'vibrant_ink', 'xcode'],
            set(n) { if (this.editor) this.editor.setTheme("ace/theme/" + n) }
        },
        fontSize: {
            type: Number,
            set(n) { if (this.editor) { this.editor.setOptions({ fontSize: n }) } }
        },
        commands: {
            default: '',
            list: ['showSettingsMenu', 'openCommandPallete', 'find'],
            set(n) {
                if (n) {
                    this.editor.execCommand(n);
                    this.commands = '';
                }
            }
        },
        readonly: {
            default: false,
            set(n) {
                if (this.editor) this.editor.setOptions({ readOnly: n });
            }
        },
        showCopyButton: {
            type: Boolean,
            reflectToAttribute: true,
            default: false
        }
    },
    attached() {
        this.async(() => {
            this.editor = this.$refs.editor.aceEditor;
            this.editor.setOptions({ ...this.default, ...this.options });
            if (this.value) this.editor.setValue(this.value);
            this.editor.session.selection.clearSelection();
            if (this.readOnly || this.options.readOnly) this.editor.renderer.$cursorLayer.element.style.display = "none";
            this.editor.session.on('change', this._valueChangedHandler.bind(this));
            this.editor.setOptions({ readOnly: this.readonly });
            if (this.theme) this.editor.setTheme("ace/theme/" + this.theme);
            if (this.mode) this.editor.session.setMode("ace/mode/" + this.mode);
            if (this.lines) this.editor.setOptions({ maxLines: this.lines, minLines: this.lines });
            if (this.fontSize) this.editor.setOptions({ fontSize: this.fontSize });
        });
    },
    async _copy() {
        await navigator.clipboard.writeText(this.editor.getValue());
    },
    _valueChangedHandler() {
        this._valueChangedOnEditor = true;
        this.value = this.editor.getValue();
        this._valueChangedOnEditor = false;
    }
});
