import ODA from '../../../oda.js';
import '../../buttons/icon/icon.js';

const VIEW_MODES = [
    {
        icon: 'icons:view-week',
        name: 'tile',
        style: `.sticky { display: flex; flex-direction: horizontal; padding: 15px; flex-wrap: wrap;} .cell{width: 250px; margin:1px;}`
    },
    {
        icon: 'icons:view-headline',
        name: 'list',
        style: '.sticky{display: flex; flex-direction: column; padding: 15px;} .cell{margin:1px;}'
    },
    {
        icon: 'image:view-compact',
        name: 'table',
        style: '.cell{margin:1px;}',
        columns: [{name: 'label', label: 'Name'},{name: 'size', label: 'Size'},{name:'type', label: 'Type'},{name:'time', label: 'Edited'}],
        template: 'oda-tile-table-item',
    },
    {
        icon: 'image:view-comfy',
        name: 'icons',
        style: `.sticky{ display: flex; flex-direction: horizontal; padding: 15px; flex-wrap: wrap;} .cell{min-width: 100px; margin:1px;}`,
        template: 'oda-tile-icon-item'
    },
];

ODA({ // It is addon for components: oda-table, oda-tree
    is: 'oda-tile', template: /*html*/`
        <style>
            :focus {
                outline: 1px dotted silver;
            }
            :hover {
                outline: 1px dotted silver;
            }
            {{_style}}
        </style>
        <div style="display: flex; justify-content: space-between;">
            <slot name="control-panel"></slot>
            <div ~if="showViewModes"  style="display: flex">
                <oda-button ~for="obj in viewModes" :icon="obj.icon" :icon-size="this.iconSize" @tap="viewMode = obj"></oda-button>
            </div>
        </div>
    `,

    props: {
        defaultTemplate: 'oda-tile-item',
        _defaultTemplate: '',
        contextItem: {
            type: Object,
            watch(n, o) {
                if (n && (n !== o) && (n !== this._contextItem)) this.updateContext(n);
            }
        },
        _contextItem: Object,
        cols:{
            type: Array,
            get(){
                return this.columns.length?this.columns:[{name: this.keyName}]
            }
        },
        preventExpanding: true,
        viewMode: {
            type: String,
            list: VIEW_MODES,
            default: VIEW_MODES[0]
        },
        _style: '',
        viewModes: VIEW_MODES,
        tableModeColumns: [],
        showViewModes: true
    },

    created() {
        if (this.defaultTemplate) this._defaultTemplate = this.defaultTemplate;
    },

    observers: [
        function _byViewMode(viewMode, tableModeColumns) {
            this._style = '';
            this.columns = [];
            this.colLines = false;
            this.showHeader = false;

            this._style = viewMode.style;
            if ( viewMode.name === 'table') {
                this.defaultTemplate = viewMode.template || 'oda-tile-table-item';
                this.columns = (tableModeColumns && tableModeColumns.length) ? tableModeColumns : viewMode.columns;
                this.showHeader = true;
                this.colLines = true;
            } else if (viewMode.name === 'icons') {
                this.defaultTemplate = viewMode.template || 'oda-tile-icon-item';
            } else {
                if (this._defaultTemplate) this.defaultTemplate = this._defaultTemplate;
            }
        }
    ],
    listeners: {
        'row-dblclick'(e) {
            e.stopPropagation();
            this.updateContext(e.detail.value);
        }
    },
    async updateContext(item) {
        if (!item) return;
        const items = await this._beforeExpand(item);
        if (items) {
            this.dataSet = items;
            if (this.contextItem !== item) {
                this._contextItem = item;
                this.contextItem = item;
            }
        } else {
            // to do
        }
    }
});

import '../table/table.js';
import '../../buttons/icon/icon.js';
ODA({
    is: 'oda-tile-item', extends: 'oda-table-cell-base',
    template: /*html*/`
        <oda-icon :icon="icon"></oda-icon>
        <div class="label" ~if="label">{{label}}</div>
    `,
    props: {
        icon: {
            type: Object
        },
        focused: false,
        defaultIcon: 'files:document',
        folderIcon: 'icons:folder',
        label: {
            type: String,
            get() {
                const item = this.item;
                return item ? (item.label || item.name) : '';
            }
        }
    },

    observers: [
        function _setIcon(item) {
            if (item) {
                this.icon = item.icon || (item.items ? this.folderIcon : this.defaultIcon);
            }
        }
    ]
});

ODA({
    is: 'oda-tile-table-item', extends: 'oda-table-cell-base',
    template: /*html*/`
        <style>
            :host {
                @apply --horizontal;
            }
        </style>
        <div ~is="_template" ~if="_labelCol" :item="item" style="padding-left: 15px"></div>
        <div ~if="!_labelCol" style="padding-left: 15px">{{_value}}</div>
    `,
    props: {
        _value: '',
        _item: {
            get() {
                return this.item; // for redefining in inherit
            }
        },
        _labelCol: false,
        _template: 'oda-tile-item'
    },
    observers: [
        async function _setValue(_item, column) {
            if (_item && column) {
                this._value = (column.name in _item) ? _item[column.name] : '';
                if ((column.name === 'label') || (column.name === 'name')) {
                    this._labelCol = true;
                }
            }
        }
    ]
});

ODA({
    is: 'oda-tile-icon-item', extends: 'oda-table-cell-base',
    template: /*html*/`
        <style>
            .tile-icon-item-main {
                display: flex;
                flex-direction: column;
                width: 100%;
                margin-top: 5px;
                align-items: center;
                overflow: hidden;
            }
        </style>
        <style>{{_style}}</style>
        <div class="tile-icon-item-main" @mouseover="_showFullLabel = true" @mouseout="_showFullLabel = false">
            <oda-icon :icon="icon" :icon-size="this.iconSize * this.sizeKoef" :fill="(_item && _item.fill) || ''"></oda-icon>
            <div class="label" style="width: 100%; text-align: center;" ~if="label && !_showFullLabel" :alt="label" >{{label}}</div>
            <div ~if="_showFullLabel" class="label" style="overflow: visible; background: white; " >{{label}}</div>
        </div>
    `,
    props: {
        icon: {
            get() {
                return (this._item && (this._item.icon || (this._item.items ? this.folderIcon : this.defaultIcon))) || '';
            }
        },
        focused: false,
        defaultIcon: 'files:document',
        folderIcon: 'icons:folder',
        _item: {
            get() {
                return this.item; // for redefining
            }
        },
        label: {
            type: String,
            get() {
                return this._item ? (this._item.label || this._item.name) : '';
            }
        },
        sizeKoef: 1.5,
        _style: {
            type: String,
            get() {
                return `:host{width: ${(this.iconSize * this.sizeKoef * 4) || 150}px;}`
            }
        },
        _showFullLabel: false,
    }
});