import '../form-layout/form-layout.js';
import '../splitter/splitter.js';
ODA({
    is: 'oda-app-layout', extends: 'oda-form-layout', template: /*html*/`
    <style>
        :host {
            @apply --flex;
            position: relative;
            overflow: hidden;
            height: 100%;
            touch-action: manipulation;
        }
        .drawer {
            transition: transform 0.2s;
            @apply --content;
            @apply --horizontal;
            z-index: 1;
            overflow: hidden;
        }
        .drawer > div {
            overflow: hidden;
        }
        .main {
            transition: margin, filter 0.2s;
            @apply --content;
            overflow: hidden;
        }
        slot {
           min-width: 0px;
           overflow: hidden;
        }
        .header {
            @apply --header;
        }
        .content {
            @apply --content;
        }
        [disabled] {
            @apply --disabled;
        }
        .stop-pointer-events * {
            pointer-events: none;
        }
        slot>*{
            @apply --flex;
        }
        ::slotted(*){
            @apply --flex;
        }
    </style>
    <app-layout-header ref="appHeader" class="horizontal no-flex top" style="transition: margin-top 0.3s ease-in-out; border-bottom: 1px solid silver; align-items: center;" :left="left" :right="right" ~if="!hideToolbar && siteMode">
        <slot name="title"></slot>
    </app-layout-header>
    <div class="flex horizontal" ~class="{'stop-pointer-events': size === 'min'}" style="overflow: hidden" ~style="{'zoom': size === 'min' ? '50%' : ''}">
        <div  ~show="left.visible" @tap.stop="" class="drawer no-flex left" ~style="{transform: \`translateX(-\${left.opened ? 0 : left.width}px)\`, width: left.width + 'px'}">
            <slot name="left-drawer"  @slotchange="_slot" class="flex vertical" ref="leftSlot"></slot>
            <oda-splitter  :max="left.persistent?max:0" ~if="!left.hideResize && !left.persistent" ::width="left.width" @split="_set"></oda-splitter>
        </div>
        <div class="main vertical flex" ~style="{
            marginLeft: \`-\${left.visible && (left.persistent || !left.opened) ? left.width : 0}px\`,
            marginRight: \`-\${right.visible && (right.persistent || !right.opened) ? right.width : 0}px\`,
            filter: locked?'brightness(.3)':'none',
            pointerEvents: locked?'none':'all'}">
            <app-layout-header :icon-size="iconSize" class="horizontal no-flex top" style="border-bottom: 1px solid silver; align-items: center;" :left="left" :right="right" ~if="!hideToolbar && !siteMode"></app-layout-header>
            <slot name="main"  class="vertical flex slot" style="overflow: hidden" ref="mainSlot" ~style="_background"></slot>
            <slot name="bottom"  class="horizontal no-flex slot bottom" ref="bottomSlot"></slot>
        </div>
        <div  ~show="right.visible" @tap.stop="" class="drawer no-flex right" ~style="{transform: \`translateX(\${right.opened ? 0 : right.width}px)\`,width: right.width + 'px'}">
            <oda-splitter ~if="!right.hideResize && !right.persistent" :max="right.persistent?max:0" ::width="right.width" @split="_set"></oda-splitter>
            <slot name="right-drawer"  @slotchange="_slot" class="flex vertical" ref="rightSlot"></slot>
        </div>
    </div>
    <slot name="footer"  class="horizontal no-flex slot bottom"></slot>`,
    _slot(e) {
        let el = e.target.assignedNodes();
        let name = e.target.name.split('-').shift();
        el = el.length > 0 ? el[0] : undefined;
        this[name].visible = !!el && !el.hidden;
        this[name].icon = !!el && el.getAttribute("icon");
        this._resize();
    },
    _drawerStyle(e) {
        let name = e.target.name.split('-').shift();
        let obj = this[name];
        let width = obj.persistent ? this.max : obj.width;
        return {
            transform: `translateX(${obj.opened ? 0 : width}px)`, width: width + 'px'
        }
    },
    props: {
        background: {
            type: Object,
            default: {
                props: {
                    size: {
                        default: 'contain',
                        list: ['contain', 'cover', 'auto', 'inherit', 'initial', 'unset']
                    },
                    repeat: {
                        default: 'no-repeat',
                        list: ['no-repeat', 'repeat-x', 'repeat', 'space', 'space repeat', 'round']
                    },
                    image: {
                        default: '',
                        list: ['none', 'url()']
                    },
                    position: {
                        default: 'center',
                        list: ['center', 'top', 'left', 'right', 'bottom']
                    },
                    color: {
                        default: '',
                        list: ['white', 'red', 'green', 'blue', 'orange']
                    },
                }
            }
        },
        _background: {
            get() {
                return {
                    'background-size': this.background.size || this.background.props.size.default,
                    'background-repeat': this.background.repeat || this.background.props.repeat.default,
                    'background-image': this.background.image || this.background.props.image.default,
                    'background-position': this.background.position || this.background.props.position.default,
                    'background-color': this.background.color || this.background.props.color.default
                };
            }
        },
        siteMode: false,
        max: 300,
        hideToolbar: false,
        locked: {
            type: Boolean,
            get() {
                return (
                    (this.left.visible && this.left.persistent && this.left.opened) ||
                    (this.right.visible && this.right.persistent && this.right.opened)
                );
            }
        },
        left: {
            type: Object,
            get() {
                return {
                    hideResize: false,
                    icon: null,
                    visible: false,
                    opened: false,
                    persistent: false,
                    width: 300,
                    _k: 2
                }
            }
        },
        right: {
            type: Object,
            get() {
                return {
                    hideResize: false,
                    icon: null,
                    visible: false,
                    opened: false,
                    persistent: false,
                    width: 300,
                    _k: 2
                };
            }
        },
        leftIcon: "icons:menu",
        rightIcon: "icons:menu",
        hideOnScroll: {
            default: false,
            set(n) {
                if (n) {
                    if (!this._hideOnScroll) this.listen('wheel', '_scroll', { target: window, useCapture: true });
                    this._hideOnScroll = true;
                }
            }
        }
    },
    listeners: {
        tap(e) {
            e.stopPropagation();
            if (this.left.visible && this.left.persistent && this.left.opened)
                this.left.opened = false;
            if (this.right.visible && this.right.persistent && this.right.opened)
                this.right.opened = false;
            this._resize();
        },
        resize(e) {
            this._resize();
        }
    },
    _rightToggle(e) {
        this.right.opened = !this.right.opened;
    },
    _resize(side) {
        if (!side) {
            this._resize(this.left);
            this._resize(this.right);
            return;
        }
        const v = this.offsetWidth < 300 * side._k;
        if (side.persistent !== v) {
            side.persistent = v;
            if (side.persistent) {
                side._before = side.opened;
                side.opened = false;
            } else side.opened = side._before;
        }
    },
    attached() {
        const me =
            this.unique && this.getModals().find(i => i.unique === this.unique);
        if (me) {
            me._top();
            this.remove();
            return;
        }
        this.left.width = Math.max(300, this.__read(this.settingsId + "/left/width", this.left.width)) || 300;
        this.right.width = Math.max(300, this.__read(this.settingsId + "/right/width", this.right.width)) || 300;
        this._resize();
        this.left.opened = !this.left.persistent && !this.right.persistent;

        if (this.hideOnScroll) {
            this.listen('wheel', '_scroll', { target: window, useCapture: true });
            this._hideOnScroll = true;
        }
    },
    deattached() {
        if (this._hideOnScroll)
            this.unlisten('wheel', '_scroll', { target: window, useCapture: true });
    },
    _scroll(e) {
        this.debounce('hide-header', () => {
            requestAnimationFrame(() => {
                let h = this.$refs.appHeader;
                let t = e.target;
                if (e.detail && e.detail.value === 'clearScroll') {
                    h.style.marginTop = '0';
                    return;
                }
                if (t.slot !== 'main') return;
                if (e.wheelDelta >= 0 || e.detail > 0) {
                    h.style.marginTop = '0';
                } else {
                    h.style.marginTop = `-${h.offsetHeight}px`;
                }
            });
        }, 100);
    },
    _set() {
        this.__write(this.settingsId + "/left/width", Math.max(300, this.left.width || 300));
        this.__write(this.settingsId + "/right/width", Math.max(300, this.right.width || 300));
    }
});

ODA({
    is: 'app-layout-header', template: /*html*/`
    <style>
        :host{
            @apply --flex;
            @apply --horizontal;
        }
    </style>
    <oda-button ~if="left.visible" :size="iconSize" :icon="left.opened ? 'icons:file-download:90' : left.icon || 'icons:menu'" allow-toggle ::toggled="left.opened"></oda-button>
    <slot name="top-left" class="horizontal flex" style="overflow: hidden; justify-content: flex-start;"></slot>
<!--    <slot name="title" class="horizontal flex" ></slot>-->
    <slot name="title" class="horizontal" style="overflow: hidden; justify-content: center;"></slot>
    <slot name="top-right" class="horizontal" style="overflow: hidden; justify-content: flex-end; flex-shrink: 0;"></slot>
    <oda-button ~if="right.visible" :size="iconSize" :icon="right.opened ? 'icons:file-download:270' : right.icon || 'icons:language'" allow-toggle ::toggled="right.opened"></oda-button>`,
    props: {
        left: {},
        right: {},
        iconSize: {
            default: 24,
            shared: true
        }
    }
});

ODA({
    is: 'app-layout-drawer', template: /*html*/`
    <style>
        :host {
            transition: transform 0.2s;
            @apply --content;
            @apply --horizontal;
            z-index: 1;
            overflow: hidden;
        }
    </style>
        <div  ~show="right.visible" @tap.stop class="drawer no-flex right" ~style="{transform: \`translateX(\${right.opened ? 0 : right.width}px)\`,width: right.width + 'px'}">
            <oda-splitter ~if="!right.persistent" bind::width="right.width" @split="_set"></oda-splitter>
            <slot name="right-drawer"  @slotchange="_slot" class="flex vertical"></slot>
        </div>
    `
});
