import '../../../buttons/button/button.js';
import '../../../colors/palette/palette.js';

blocks:{
    ODA({
        is: 'oda-scheme-base-block',
        template: `
            <style>
                :host {
                    @apply --vertical;
                    width: 100%;
                }
                .align {
                    @apply --vertical;
                    align-items: center;
                    justify-content:center;
                    min-width: 50px;
                    height: 100%;
                    margin: 2px;
                    padding: .3rem;
                    border: 1px solid orange;
                }
            </style>
            <div ~show="!isSchemeDesigner||(isSchemeDesigner&&parentSchemeProxy&&!showInnerSchema)" class="align">{{label}}</div>
        `,
        props: {
            model: {
                type: Object,
                notify: true,
                default: {
                    label: 'Базовый блок',
                    top: [],
                    right: [],
                    bottom: [],
                    left: [],
                    staticMenu: {
                        top: [],
                        bottom: []
                    },
                    dynamicMenu: {
                        top: [],
                        bottom: []
                    }
                }
            },
            oid: {
                type: String,
                set(n) {
                    this.model['oid'] = n;
                }
            },
            isSchemeDesigner: {
                type: Boolean,
                get() {
                    return this.tagName.includes('oda-scheme-designer'.toUpperCase());
                }
            }
        },
        getNewDefaultConnector() {
            return {
                shape: {border: '2px solid red', borderRadius: '50% 5%'}, color: 'lime', backgroundColor: 'transparent', size: '16',
                icon: 'icons:link', action(position, index) {
                    this.deleteConnector(position, index);
                }, service: false
            }
        },
        addDefaultConnector(position) {
            this.model[position].push(this.getNewDefaultConnector());
        },
        remove(position, index) {
            if (this.model[position][index].link) this.deleteLink(position, index);
            else this.deleteConnector(position, index);
        },
        deleteConnector(position) {
            this.model[position].pop();
        },
        addLink(start, finish, linkColor = 'blue') {
            const pin = {};
            pin[finish.pos] = finish.i;
            this.model[start.pos][start.i].link = null;
            this.model[start.pos][start.i].link = {id: finish.oid, pin, linkColor, selected: false};
        },
        addSchemeLink(start, finish, linkColor = 'blue') {
            const pin = {};
            pin[finish.pos] = this.oid + '-' + start.pos + '-' + start.i;
            this.model[start.pos][start.i].link = {pin, linkColor, selected: false};
        },
        changeColor(position, index = 0) {
            this.model[position][index].color = this.model[position][index].color === 'yellow' ? 'lime' : 'yellow';
        },
        setSelected(position, index) {
            if (!this.model[position][index].link.hasOwnProperty('selected')) this.model[position][index].link['selected'] = false;
            this.model[position][index].link['selected'] = !this.model[position][index].link['selected'];
        },
        execForAll(callback, params = []) {
            ['top', 'right', 'bottom', 'left'].map((pos) => {
                if (this.model[pos]) this.model[pos].forEach((connector, index) => {
                    this[callback](pos, index, params);
                });
            });
        },
        execForAllSelected(callback, params = []) {
            ['top', 'right', 'bottom', 'left'].map((pos) => {
                if (this.model[pos]) this.model[pos].forEach((connector, index) => {
                    if (connector.link && connector.link.selected) {
                        this[callback](pos, index, params);
                    }
                });
            })
        },
        deleteLink(position, index) {
            this.model[position][index].link = null;
        },
        changeLinkColor(position, index, params = ['red']) {
            this.model[position][index].link.linkColor = params[0];
        },
        resetSelectedLink(position, index) {
            this.model[position][index].link['selected'] = false;
        }
    });
}

designer:{
    ODA({
        is: 'oda-scheme-designer', extends: 'oda-scheme-base-block',
        template: /* html */`
        <style>
            :host {
                --size: 9000px;
            }
            svg {
                width: var(--size);
                height: var(--size);
            }
            .root {
                @apply --border;
                position: relative;
                overflow: hidden;
                box-sizing: border-box;
            }
            .main {
                position: relative;
                width: 100%;
                overflow: auto;
            }
            .grid {
                position: absolute;
                width: var(--size);
                height: var(--size);
            }
            .contextmenu {
                position: absolute;
                @apply --vertical;
                @apply --content;
                @apply --border;
                list-style: none;
                margin: 0;
                padding: 0;
            }
            .contextmenu-item {
                margin: .1rem 0;
                padding: .5rem 1rem;
            }
            .contextmenu-item:hover {
                background-color: dimgrey;
                color: white;
            }
            .scheme-connectors {
                position: absolute;
                display:flex;
                align-items:center;
                justify-content: center;
            }
            .left, .right {
                flex-direction: column;
            }
            .buttons {
                display: grid;
            }
        </style>
        <div ~show="!parentSchemeProxy" class="buttons" style="grid-template-columns: 1fr 1fr 1fr">
            <div class="horizontal" style="justify-self: start">
                <oda-button icon="image:grid-off" title="hide grid" allow-toggle ::toggled="hideGrid"></oda-button>
                <oda-button icon="editor:border-style" title="snap to grid" allow-toggle ::toggled="snapToGrid"></oda-button>
                <oda-button icon="icons:trending-up" title="broken line view" allow-toggle ::toggled="brokenLine"></oda-button>
            </div>
            <div class="horizontal" style="justify-self: center">
                <oda-button icon="icons:zoom-in" title="zoom-in" @tap="zoom*=1.1"></oda-button>
                <oda-button :text="scaleText" title="100%" @tap="zoom=1"></oda-button>
                <oda-button icon="icons:zoom-out" title="zoom-out" @tap="zoom*=.9"></oda-button>
            </div>
            <div class="horizontal" style="justify-self: end">
                <oda-button icon="image:edit" title="edit" allow-toggle ::toggled="edit"></oda-button>
            </div>
        </div>
        <div ~show="!parentSchemeProxy||(parentSchemeProxy&&showInnerSchema)" ref="wrapper" ~style="{display:'flex',justifyContent:'center',alignItems:wrapperHeight<mainHeight?'flex-start':'center',width:wrapperWidth,height:wrapperHeight,boxSizing:'border-box',overflow:'hidden'}">
            <div ref="root" class="vertical root" ~style="{height: rootHeight, width: '100%'}">
                <div ~show="edit" class="content" style="position:absolute;font-size:12px;width:25px;height:25px;display:flex;align-items:center;justify-content:center;" ~text="dimension"></div>
                <div ~show="edit" class="horizontal content" style="position:relative;height:25px;left:30px;margin-bottom:5px">
                    <canvas ref="ruler_h" style="height: 25px"></canvas>
                </div>
                <div class="flex horizontal">
                    <div ~show="edit" class="ruler_v content" style="position:relative;margin-right:5px">
                        <canvas ref="ruler_v" style="width: 25px"></canvas>
                    </div>
                    <div class="main" ref="main" ~style="{height: mainHeight, overflow: parentSchemeProxy ?'hidden':'auto'}">
                        <oda-scheme-grid ~show="edit" ~if="!hideGrid" :zoom="zoom" class="grid" :step="step" :size-big-grid="sizeBigGrid" :size-small-grid="sizeSmallGrid"></oda-scheme-grid>
                        <div ref="scheme" :disabled="!edit" style="position: relative" ~style="{width: width + 'px', height: depth + 'px', zoom: zoom}">
                            <svg style="position:absolute;top:0;left:0;background-color:transparent">
                                <line ~if="line.show" :x1="line.x1" :y1="line.y1" :x2="line.x2" :y2="line.y2" :style="'z-index: 1000;stroke:red; stroke-width:2px;stroke-dasharray:2'"></line>
                                <path ~for="l in links" :d="_link(l)" :style="{strokeWidth:l.selected?'4px':'2px',cursor:'pointer'}" fill="none" fill-opacity="0.05" :stroke="l.color||'red'" @tap.stop="_selectLink($event, l)" @contextmenu.stop.prevent="_showLinksMenu($event,l)"></path>
                            </svg>
                            <oda-scheme-proxy ref="proxy" ~wake ~for="layout" :block="item" :edit :i="index" ~style="{position:'absolute',transform:'translate3d('+item.x+'px,'+item.y+'px,'+'0px)'}" @click.stop="_selectProxy(item)" :selected="item.oid === selectedProxyOid"></oda-scheme-proxy>
                            <ul class="contextmenu" ~if="showLinkMenu" ~style="{transform: 'translate3d('+linkMenuPosition.x+'px,'+linkMenuPosition.y+'px,'+'0px)'}">
                                <li class="contextmenu-item" ~for="linkMenu" ~text="item.name" ~style="{cursor: 'pointer'}" @click.stop="item.action.call(this)"></li>
                            </ul>
                        </div>
                    </div>
                </div>
                <div ~if="model" :disabled="!edit" class="scheme-connectors top" style="width: 100%" :style="{top: edit ? '2rem': '.2rem'}">
                    <oda-scheme-connector ~show="line.show && this.line.y2 < rootHeight/4" pos="top" :model="{icon: 'icons:add', color: 'red', size: '16', addingSchemeConnector: 'true'}" @down.stop :disabled="disabled"></oda-scheme-connector>
                    <oda-scheme-connector ref="top" draggable="true" ~for="model.top||[]" :oid="model.oid" pos="top" :i="index" :model="item" @down.stop :disabled="true"></oda-scheme-connector>
                </div>
                <div ~if="model" :disabled="!edit" class="scheme-connectors left" style="height: 100%" :style="{left: edit ? '2rem': '.2rem'}">
                    <oda-scheme-connector ~show="line.show && this.line.x2 < rootWidth/4" pos="left" :model="{icon: 'icons:add', color: 'red', size: '16', addingSchemeConnector: 'true'}" @down.stop :disabled="disabled"></oda-scheme-connector>
                    <oda-scheme-connector ref="left" draggable="true" ~for="model.left||[]" :oid="model.oid" pos="top" :i="index" :model="item" @down.stop :disabled="true"></oda-scheme-connector>
                </div>
                <div ~if="model" :disabled="!edit" class="scheme-connectors right" style="height: 100%" :style="{right: edit ? '2rem': '.2rem'}">
                    <oda-scheme-connector ~show="line.show && this.line.x2 > rootWidth*3/4" pos="right" :model="{icon: 'icons:add', color: 'red', size: '16', addingSchemeConnector: 'true'}" @down.stop :disabled="disabled"></oda-scheme-connector>
                    <oda-scheme-connector ref="right" draggable="true" ~for="model.right||[]" :oid="model.oid" pos="top" :i="index" :model="item" @down.stop :disabled="true"></oda-scheme-connector>
                </div>
                <div ~if="model" :disabled="!edit" class="scheme-connectors bottom" style="width: 100%" :style="{bottom: edit ? '2rem': '1rem'}">
                    <oda-scheme-connector ~show="line.show && this.line.y2 > rootHeight*3/4" pos="bottom" :model="{icon: 'icons:add', color: 'red', size: '16', addingSchemeConnector: 'true'}" @down.stop :disabled="disabled"></oda-scheme-connector>
                    <oda-scheme-connector ref="bottom" draggable="true" ~for="model.bottom||[]" :oid="model.oid" pos="top" :i="index" :model="item" @down.stop :disabled="true"></oda-scheme-connector>
                </div>
                <oda-palette ~if="showPalette" ref="pal" @tap.stop="paletteHandler()" style="position:absolute;bottom:2rem;right:2rem"></oda-palette>
            </div>
        </div>
    `,
        props: {
            layout: [],
            model: {
                get() {
                    const schemeDesignerModel = {top: [], left: [], right: [], bottom: [], oid: this.oid};
                    const proxyModels = this.domHost && this.domHost.tagName === 'oda-scheme-proxy'.toUpperCase() ? this.getProxyModels() : this.modelsArray;
                    proxyModels.map((model) => {
                        return ['top', 'left', 'right', 'bottom'].map(position => {
                            if (model === undefined) return [];
                            return (model[position] || []).map((connector) => {
                                if (connector.link && !connector.link.hasOwnProperty('id')) {
                                    ['top', 'left', 'right', 'bottom'].map(position => {
                                        if (connector.link.pin.hasOwnProperty(position)) {
                                            schemeDesignerModel[position].splice(0, 0, {icon: 'icons:link', name: connector.link.pin[position], link: connector.link.outer});
                                        }
                                    });
                                    return connector.link.pin;
                                } else return {};
                            })
                        })
                    });
                    ['top', 'left', 'right', 'bottom'].map(position => {
                        schemeDesignerModel[position].sort((a, b) => {
                            if (a.name > b.name) return 1;
                            if (a.name < b.name) return -1;
                            return 0;
                        });
                    });
                    if (this.parentSchemeProxy) {
                        schemeDesignerModel['dynamicMenu'] = {};
                        schemeDesignerModel['dynamicMenu']['top'] = [];
                        schemeDesignerModel['dynamicMenu']['top'].push({
                            icon: 'icons:error', title: 'showInnerSchema', action() {
                                this.showInnerSchema = !this.showInnerSchema;
                            }
                        })
                    }
                    return schemeDesignerModel;
                }
            },
            parentSchemeProxy: {
                get() {
                    return this.domHost && this.domHost.tagName === 'oda-scheme-proxy'.toUpperCase();
                }
            },
            label: '',
            disabled: false,
            showInnerSchema: true,
            selectedProxyOid: String,
            dimension: {
                get() {
                    return this.step === 0.1 ? 'mm' : (this.step === 1 || this.step === 10) ? 'cm' : (this.step === 100 || this.step === 1000 || this.step === 10000) ? 'm' : 'km';
                }
            },
            modelsArray: {
                get() {
                    if (this.domHost && this.domHost.tagName === 'oda-scheme-proxy'.toUpperCase()) return this.getProxyModels();
                    return (this.layout || []).map((prox, i) => {
                        return this.$refs.proxy ? this.$refs.proxy[i].model : undefined;
                    })
                }
            },
            links: {
                get() {
                    return this.modelsArray.map((model, i) => {
                        return ['top', 'left', 'right', 'bottom'].map(position => {
                            if (model === undefined) return [];
                            return (model[position] || [])
                                .map((connector, index) => {
                                    if (connector.link) {
                                        if (!connector.link.hasOwnProperty('id')) {// рассчет для коннекторов схемы
                                            let x1, y1, x2, y2, startPos, startIndex, startId, selected, finishPos, finishIndex, color, finishId;
                                            const firstItem = this.layout.find(item => item.oid === model.oid);
                                            x1 = +firstItem.x + (this.$refs.proxy && this.$refs.proxy[i].getConnectorCenterOffset(position, index).x || 0);
                                            y1 = +firstItem.y + (this.$refs.proxy && this.$refs.proxy[i].getConnectorCenterOffset(position, index).y || 0);
                                            startIndex = index;
                                            startPos = position;
                                            startId = model.oid;
                                            finishId = 'self';
                                            ['top', 'left', 'right', 'bottom'].map(pos => {
                                                if (connector.link.pin[pos] !== undefined) {
                                                    const i = !Number.isInteger(connector.link.pin[pos]) ? this.model[pos].findIndex(item => item.name === connector.link.pin[pos]) : connector.link.pin[pos];
                                                    x2 = (this.$refs[pos] && this.$refs[pos][i] && this.$refs[pos][i].getCenterOffset().x + this.scrollDx - (this.edit && 30)) / this.zoom || 0;
                                                    y2 = (this.$refs[pos] && this.$refs[pos][i] && this.$refs[pos][i].getCenterOffset().y + this.scrollDy - (this.edit && 30)) / this.zoom || 0;
                                                    finishIndex = i;
                                                    const getFinishPosition = pos => {
                                                        switch (pos) {
                                                            case 'top':
                                                                return 'bottom';
                                                            case 'bottom':
                                                                return 'top';
                                                            case 'left':
                                                                return 'right';
                                                            case 'right':
                                                                return 'left';
                                                        }
                                                    }
                                                    finishPos = getFinishPosition(pos);
                                                }
                                            });
                                            color = connector.link.linkColor || 'purple';
                                            selected = connector.link.selected || false;
                                            return {x1, y1, startPos, startIndex, startId, selected, x2, y2, finishPos, finishIndex, color, finishId};
                                        }
                                        if (!this.$refs.proxy || !connector.link.hasOwnProperty('id')) return {};
                                        const firstItem = this.layout.find(item => item.oid === model.oid);// рассчет для коннекторов прокси
                                        const secondItem = this.layout.find(item => item.oid === connector.link.id);
                                        const finishModelIndex = this.modelsArray.findIndex(item => item.oid === connector.link.id);
                                        const x1 = +firstItem.x + (this.$refs.proxy && this.$refs.proxy[i].getConnectorCenterOffset(position, index).x || 0);
                                        const y1 = +firstItem.y + (this.$refs.proxy && this.$refs.proxy[i].getConnectorCenterOffset(position, index).y || 0);
                                        const startIndex = index;
                                        const startPos = position;
                                        let deleteLink = false;
                                        let x2 = 0, y2 = 0, finishPos = '', finishIndex = 0;
                                        ['top', 'left', 'right', 'bottom'].map(pos => {
                                            if (connector.link.pin[pos] !== undefined) {
                                                if (finishModelIndex >= 0 && this.$refs.proxy[finishModelIndex] && this.$refs.proxy[finishModelIndex].model[pos][connector.link.pin[pos]] !== undefined) {
                                                    x2 = secondItem.x + this.$refs.proxy[finishModelIndex].getConnectorCenterOffset(pos, connector.link.pin[pos]).x;
                                                    y2 = +secondItem.y + this.$refs.proxy[finishModelIndex].getConnectorCenterOffset(pos, connector.link.pin[pos]).y;
                                                    finishIndex = connector.link.pin[pos];
                                                    finishPos = pos;
                                                } else {
                                                    deleteLink = true;
                                                    this.async(() => this.$refs.proxy[i].$refs.block.deleteLink(position, index));
                                                }
                                            }
                                        });
                                        if (deleteLink) return {};
                                        const startId = model.oid;
                                        const finishId = connector.link.id;
                                        const color = connector.link.linkColor || 'purple';
                                        const selected = connector.link.selected || false;
                                        return {x1, y1, startPos, startIndex, startId, selected, x2, y2, finishPos, finishIndex, color, finishId};
                                    } else return {};
                                }).filter(i => i.x1);
                        }).reduce((accumulator, next) => {
                            return accumulator.concat(next);
                        }, [])
                    }).reduce((accumulator, next) => {
                        return accumulator.concat(next);
                    }, [])
                }
            },
            zoom: {
                type: Number,
                default: 1,
                set() {
                    this.currentProxyRect = null;
                }
            },
            scaleText: {
                get() {
                    if (this.zoom === 1) return '100%';
                    if (this.zoom < 1) return `${this.zoom < 0.0001 ? Math.round(this.zoom * 100000000) / 1000000 : Math.round(this.zoom * 10000) / 100}%`;
                    return `${Math.round(this.zoom * 100)}%`;
                }
            },
            line: {
                type: Object,
                default: {show: false, mode: '', x1: 100, y1: 100, x2: '', y2: '', dev: {}}
            },
            showLinkMenu: false,
            linkMenuPosition: {},
            linkMenu: [{
                name: 'changeLinkColor', action() {
                    this.changeLinksColor()
                }
            }, {
                name: 'deleteLink', action() {
                    this.deleteSelectedLinks()
                }
            }],
            formulaInputPosition: {},
            showFormulaInput: false,
            showPalette: false,
            selectedLink: {},
            cvWidth: 0,
            cvHeight: 0,
            depth: 9000,
            width: 9000,
            step: 10,
            scrollDy: 0,
            scrollDx: 0,
            sizeBigGrid: 100,
            sizeSmallGrid: 10,
            edit: false,
            hideGrid: false,
            snapToGrid: Boolean,
            brokenLine: false,
            wrapperHeight: {
                get() {
                    const max = Math.max(...(this.layout || []).map(proxy => proxy.y));
                    const index = (this.layout).findIndex(item => item.y === max);
                    const proxyHeight = this.$refs.proxy && this.$refs.proxy[index].getProxyOffsetHeight() || 0;
                    return this.domHost && this.domHost.tagName === 'oda-scheme-proxy'.toUpperCase() ? max + proxyHeight + 100 : window.innerHeight - (this.$refs.wrapper && this.$refs.wrapper.offsetTop || 0) - 1;
                }
            },
            wrapperWidth: {
                get() {
                    const max = Math.max(...(this.layout || []).map(proxy => proxy.x));
                    const index = (this.layout).findIndex(item => item.x === max);
                    const proxyWidth = this.$refs.proxy && this.$refs.proxy[index].getProxyOffsetWidth() || 0;
                    return this.domHost && this.domHost.tagName === 'oda-scheme-proxy'.toUpperCase() ? max + proxyWidth + 100 : '100%';
                }
            },
            mainHeight: {
                get() {
                    return window.innerHeight - this.mainOffsetTop;
                }
            },
            rootWidth: {
                get() {
                    return this.$refs.root.offsetWidth;
                }
            },
            rootHeight: {
                get() {
                    return this.$refs.wrapper ? this.$refs.wrapper.offsetHeight : '100%';
                }
            },

            currentProxyRect: null
        },
        observers: [
            'setStep(zoom, step)',
            'resizeRuler_h(width, zoom, cvWidth, scrollDx , step)',
            'resizeRuler_v(depth, zoom, cvHeight, scrollDy, step)',
        ],
        listeners: {
            click() {
                this.showLinkMenu = false;
                this.selectedProxyOid = null;
                this.showPalette = false;
                this.showFormulaInput = false;
                this._resetAllSelectedLinks();
            },
            moveItem(e) {
                e.stopPropagation();
                this.showFormulaInput = false;
                const edv = e.detail.value;
                if (!this.currentProxyRect || this.currentProxyRect.i !== edv.i) this.currentProxyRect = {i: edv.i, x: this.layout[edv.i].x, y: this.layout[edv.i].y};
                this.currentProxyRect.x += edv.x / this.zoom;
                this.currentProxyRect.y += edv.y / this.zoom;
                if (this.snapToGrid) {
                    this.layout[edv.i].x = this.currentProxyRect.x < this.layout[edv.i].x ? Math.ceil(this.currentProxyRect.x / this.step) * this.step : Math.floor(this.currentProxyRect.x / this.step) * this.step;
                    this.layout[edv.i].y = this.currentProxyRect.y < this.layout[edv.i].y ? Math.ceil(this.currentProxyRect.y / this.step) * this.step : Math.floor(this.currentProxyRect.y / this.step) * this.step;
                } else {
                    this.layout[edv.i].x += edv.x / this.zoom;
                    this.layout[edv.i].y += edv.y / this.zoom;
                }
                if (this.layout[edv.i].x < 0) this.layout[edv.i].x = 0;
                if (this.layout[edv.i].y < 0) this.layout[edv.i].y = 0;
            },
            mousewheel(e) {
                if (e.ctrlKey) {
                    e.stopPropagation();
                    e.preventDefault();
                    const k = 0.9;
                    this.zoom = Math.min(80, e.deltaY > 0 ? this.zoom * k : this.zoom / k);
                }
            },
            dragStartConn(e) {
                const edv = e.detail.value;
                this.line.x2 = this.line.x1 = (edv.e.pageX - this.$refs.root.offsetLeft - this.$refs.main.offsetLeft + this.$refs.main.scrollLeft) / (this.zoom);
                this.line.y2 = this.line.y1 = (edv.e.pageY - this.$refs.root.offsetTop - this.$refs.main.offsetTop + this.$refs.main.scrollTop) / (this.zoom);
            },
            dragConn(e) {
                const edv = e.detail.value;
                this.line.show = true;
                if (this.line.show) {
                    this.line.x2 = (edv.e.pageX - this.$refs.root.offsetLeft - this.$refs.main.offsetLeft + this.$refs.main.scrollLeft) / (this.zoom);
                    this.line.y2 = (edv.e.pageY - this.$refs.root.offsetTop - this.$refs.main.offsetTop + this.$refs.main.scrollTop) / (this.zoom);
                }
            },
            dragEndConn() {
                this.line.show = false;
            },
            showFormulaInput(e) {
                const edv = e.detail.value;
                this.formulaInputPosition.x = this.layout.find(i => i.oid === edv.oid).x + edv.center.x;
                this.formulaInputPosition.y = this.layout.find(i => i.oid === edv.oid).y + edv.center.y;
                this.showFormulaInput = !this.showFormulaInput;
            }
        },
        attached() {
            this._setSizeRuler();
            this.listen('scroll', '_onScroll', {target: this.$refs.main});
            this.listen('resize', '_setSizeRuler', {target: window});
        },
        getProxyModels() {
            return this.$refs.proxy.map(i => i.model);
        },
        addSchemeLinkToProxy(start, finish) {
            const proxyIndex = this.layout.findIndex(i => i.oid === start.oid);
            const block = this.$refs.proxy[proxyIndex].$refs.block;
            block.addSchemeLink(start, finish);
        },
        _selectLink(event, link) {
            if (event.detail.sourceEvent.ctrlKey) {
                this.$refs.proxy[this.layout.findIndex(i => i.oid === link.startId)].$refs.block.setSelected(link.startPos, link.startIndex);
            } else {
                this._resetAllSelectedLinks();
                this.$refs.proxy[this.layout.findIndex(i => i.oid === link.startId)].$refs.block.setSelected(link.startPos, link.startIndex);
            }
        },
        _showLinksMenu(event, link) {
            this.linkMenuPosition = {x: event.offsetX / this.zoom, y: event.offsetY / this.zoom};
            this.showLinkMenu = true;
            if (!link.selected) this.$refs.proxy[this.layout.findIndex(i => i.oid === link.startId)].$refs.block.setSelected(link.startPos, link.startIndex);
        },
        paletteHandler() {
            this.$refs.proxy.forEach(schemeProxy => {
                schemeProxy.$refs.block.execForAllSelected('changeLinkColor', [this.$refs.pal.value]);
            });
        },
        changeLinksColor() {
            this.showLinkMenu = false;
            this.showPalette = true;
        },
        deleteSelectedLinks() {
            this.$refs.proxy.forEach(schemeProxy => {
                schemeProxy.$refs.block.execForAllSelected('deleteLink');
            });
            this.showLinkMenu = false;
        },
        _resetAllSelectedLinks() {
            (this.$refs.proxy || []).forEach(schemeProxy => {
                schemeProxy.$refs.block.execForAllSelected('resetSelectedLink');
            });
        },
        _selectProxy(item) {
            if (item.oid !== this.selectedProxyOid) {
                this.selectedProxyOid = item.oid;
            }
        },
        _link(l) {
            const startPoint = `M${l.x1} ${l.y1}`;
            const finishPoint = `${l.x2} ${l.y2}`;
            let helpPoints = '';
            const stop = 'm 0 0';
            let gap = 40 + (l.finishIndex + 4) * 4;
            const mpl = 4;
            let edge = 4 * (l.startIndex + 4);
            const middleX = l.x1 < l.x2 ? l.x2 - (l.x2 - l.x1) / 2 : l.x1 - (l.x1 - l.x2) / 2;
            const middleY = l.y1 < l.y2 ? l.y2 - (l.y2 - l.y1) / 2 : l.y1 - (l.y1 - l.y2) / 2;
            const arrowWidth = 4, arrowLength = 15;
            const arrowDown = `q ${0} ${-arrowLength} ${-arrowWidth} ${-arrowLength} ${startPoint} q ${0} ${-arrowLength} ${arrowWidth} ${-arrowLength} ${startPoint}`;
            const arrowRight = `q ${-arrowLength} ${0} ${-arrowLength} ${+arrowWidth} ${startPoint} q ${-arrowLength} ${0} ${-arrowLength} ${-arrowWidth} ${startPoint}`;
            const arrowUp = `q ${0} ${+arrowLength} ${-arrowWidth} ${+arrowLength} ${startPoint} q ${0} ${+arrowLength} ${arrowWidth} ${+arrowLength} ${startPoint}`;
            const arrowLeft = `q ${+arrowLength} ${0} ${+arrowLength} ${+arrowWidth} ${startPoint} q ${+arrowLength} ${0} ${+arrowLength} ${-arrowWidth} ${startPoint}`;
            const arrow = {top: arrowDown, right: arrowLeft, bottom: arrowUp, left: arrowRight};
            const startStick = {top: `V ${l.y1 - gap} ${stop}`, right: `H ${l.x1 + gap} ${stop}`, bottom: `V ${l.y1 + gap} ${stop}`, left: `H ${l.x1 - gap} ${stop}`,};
            const finishStick = ` ${stop} L ${finishPoint}`;
            gap = gap + (l.finishIndex + 4) * 4;
            const xDiff = Math.abs(l.x1 - l.x2);
            const yDiff = Math.abs(l.y1 - l.y2);
            const min = Math.min(xDiff, yDiff);
            if (xDiff < 2 * edge || yDiff < 2 * edge) edge = min / 2;
            const r = edge;

            // Кубическая кривая Безье
            const edgeUL = `c ${0} ${-r} ${-edge + r} ${-edge} ${-edge} ${-edge}`;
            const edgeUR = `c ${0} ${-r} ${+edge - r} ${-edge} ${+edge} ${-edge}`;
            const edgeRU = `c ${+r} ${0} ${+edge} ${-edge + r} ${+edge} ${-edge}`;
            const edgeRD = `c ${+r} ${0} ${+edge} ${+edge - r} ${+edge} ${+edge}`;
            const edgeDR = `c ${0} ${+r} ${+edge - r} ${+edge} ${+edge} ${+edge}`;
            const edgeDL = `c ${0} ${+r} ${-edge + r} ${+edge} ${-edge} ${+edge}`;
            const edgeLD = `c ${-r} ${0} ${-edge} ${+edge - r} ${-edge} ${+edge}`;
            const edgeLU = `c ${-r} ${0} ${-edge} ${-edge + r} ${-edge} ${-edge}`;

            if (this.brokenLine) {
                const gap = 50/*, mpl = 3, lineType = 'C'*/;
                // const startHelpPoints = {top: `${lineType} ${l.x1} ${l.y1 - mpl * gap},`, bottom: `${lineType} ${l.x1} ${+l.y1 + mpl * gap},`, left: `${lineType} ${l.x1 - mpl * gap} ${l.y1},`, right: `${lineType} ${+l.x1 + mpl * gap} ${l.y1},`};
                // const finishHelpPoints = {top: ` ${l.x2} ${l.y2 - mpl * gap}, ${l.x2} ${l.y2 - gap}`, bottom: ` ${l.x2} ${+l.y2 + mpl * gap}, ${l.x2} ${+l.y2 + gap}`, left: ` ${l.x2 - mpl * gap} ${l.y2},  ${l.x2 - gap} ${l.y2}`, right: ` ${+l.x2 + mpl * gap} ${l.y2}, ${+l.x2 + gap} ${l.y2}`}
                // helpPoints = `${startHelpPoints[l.startPos]} ${finishHelpPoints[l.finishPos]}`;
                const finishHelpPoints = {top: ` ${l.x2} ${l.y2 - gap}`, bottom: ` ${l.x2} ${+l.y2 + gap}`, left: ` ${l.x2 - gap} ${l.y2}`, right: ` ${+l.x2 + gap} ${l.y2}`}
                helpPoints = `L ${finishHelpPoints[l.finishPos]}`;
            } else {
                if (l.startPos === 'top') {
                    if (l.finishPos === 'top') helpPoints = l.y1 <= l.y2 ? l.x1 <= l.x2 ? `${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}` : `${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}` :
                        l.x1 <= l.x2 ? `V ${l.y2 - gap} ${stop} ${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}` : `V ${l.y2 - gap} ${stop} ${edgeUL} ${stop} H ${l.x2 + edge} ${stop}  ${edgeLD}`;
                    else if (l.finishPos === 'left') helpPoints = l.y1 - gap + edge >= l.y2 ? l.x1 <= l.x2 ? `L ${l.x1} ${l.y2 + edge} ${stop} ${edgeUR}` : `${edgeUL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}` :
                        (l.x1 <= l.x2 ? `${edgeUR} ${stop} H ${middleX - edge} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` : `${edgeUL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}`);
                    else if (l.finishPos === 'right') helpPoints = l.y1 - gap + edge >= l.y2 ? l.x1 >= l.x2 ? `L ${l.x1} ${l.y2 + edge} ${stop} ${edgeUL}` : `${edgeUR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}` :
                        l.x1 <= l.x2 ? `${edgeUR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}` : `${edgeUL} ${stop} H ${middleX + edge} ${stop} ${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}`;
                    else if (l.finishPos === 'bottom') helpPoints = l.y1 >= l.y2 ? l.x1 <= l.x2 ? `${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU} ${stop} ` : `${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU} ${stop} ` :
                        l.x1 <= l.x2 ? `${edgeUR} ${stop} H ${middleX - edge} ${stop} ${edgeRD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}` : `${edgeUL} ${stop} H ${middleX + edge} ${stop} ${edgeLD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU}`;
                } else if (l.startPos === 'right') {
                    if (l.finishPos === 'right') helpPoints = l.y1 <= l.y2 ? l.x1 <= l.x2 ? `H ${l.x2 + gap} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}` : `${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}` :
                        l.x1 <= l.x2 ? `H ${l.x2 + gap} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}` : `${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}`;
                    else if (l.finishPos === 'bottom') helpPoints = (l.x1 + gap - edge / 2) <= l.x2 ? l.y1 >= l.y2 ? `L ${l.x2 - edge} ${l.y1} ${stop} ${edgeRU}` : `${edgeRD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}` :
                        l.y1 <= l.y2 ? `${edgeRD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU}` : `${edgeRU} ${stop} V ${middleY + edge} ${stop} ${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU}`;
                    else if (l.finishPos === 'top') helpPoints = l.x1 + gap - edge <= l.x2 ? l.y1 <= l.y2 ? `L ${l.x2 - edge} ${l.y1} ${stop} ${edgeRD}` : `${edgeRU} ${stop} V ${l.y2 - gap} ${stop} ${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}` :
                        l.y1 <= l.y2 ? `${edgeRD} ${stop} V ${middleY - edge} ${stop} ${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}` : `${edgeRU} ${stop} V ${l.y2 - gap} ${stop} ${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}`;
                    else if (l.finishPos === 'left') helpPoints = (l.x1 + 2 * gap) <= l.x2 ? l.y1 <= l.y2 ? `H ${l.x2 - gap - edge} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` : `H ${l.x2 - gap - edge} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}` :
                        l.y1 <= l.y2 ? `${edgeRD} ${stop} V ${middleY - edge} ${stop} ${edgeDL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` : `${edgeRU} ${stop} V ${middleY + edge} ${stop} ${edgeUL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}`;
                } else if (l.startPos === 'bottom') {
                    if (l.finishPos === 'bottom') helpPoints = l.y1 <= l.y2 ? l.x1 <= l.x2 ? `V ${l.y2 + gap} ${stop} ${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}` : `V ${l.y2 + gap} ${stop} ${edgeDL} ${stop} H ${l.x2 + edge} ${stop}  ${edgeLU}` :
                        l.x1 <= l.x2 ? `${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}` : `${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU}`;
                    else if (l.finishPos === 'left') helpPoints = l.y1 + gap - edge <= l.y2 ? l.x1 <= l.x2 ? `L ${l.x1} ${l.y2 - edge} ${stop} ${edgeDR}` : `${edgeDL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` :
                        l.x1 <= l.x2 ? `${edgeDR} ${stop} H ${l.x2 - gap - edge} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}` : `${edgeDL} ${stop} H ${l.x2 - gap} ${stop} ${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}`;
                    else if (l.finishPos === 'top') helpPoints = l.y1 <= l.y2 ? l.x1 <= l.x2 ? `${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}` : `${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}` :
                        l.x1 <= l.x2 ? `${edgeDR} ${stop} H ${l.x2 + mpl * edge} ${stop} ${edgeRU} ${stop} V ${l.y2 - gap} ${stop} ${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}` : `${edgeDL} ${stop} H ${l.x2 - mpl * edge} ${stop} ${edgeLU} ${stop} V ${l.y2 - gap} ${stop} ${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}`;
                    else if (l.finishPos === 'right') helpPoints = l.y1 <= l.y2 ? l.x1 >= l.x2 ? `L ${l.x1} ${l.y2 - edge} ${stop} ${edgeDL}` : `${edgeDR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}` :
                        l.x1 <= l.x2 ? `${edgeDR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}` : `${edgeDL} ${stop} H ${l.x2 + gap + 2 * edge} ${stop} ${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}`;
                } else if (l.startPos === 'left') {
                    if (l.finishPos === 'left') helpPoints = l.y1 <= l.y2 ? l.x1 <= l.x2 ? `${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` : `H ${l.x2 - gap} ${stop} ${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDR}` :
                        l.x1 <= l.x2 ? `${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}` : `H ${l.x2 - gap} ${stop} ${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUR}`;
                    else if (l.finishPos === 'bottom') helpPoints = l.x1 - gap + edge >= l.x2 ? l.y1 >= l.y2 ? `H ${l.x2 + edge} ${stop} ${edgeLU}` : `${edgeLD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLU}` :
                        l.y1 >= l.y2 ? `${edgeLU} ${stop} V ${l.y2 + gap + edge} ${stop} ${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}` : `${edgeLD} ${stop} V ${l.y2 + gap} ${stop} ${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRU}`;
                    else if (l.finishPos === 'top') helpPoints = l.x1 - gap + edge >= l.x2 ? l.y1 <= l.y2 ? `H ${l.x2 + edge} ${stop} ${edgeLD}` : `${edgeLU} ${stop} V ${l.y2 - edge - gap} ${stop} ${edgeUL} ${stop} H ${l.x2 + edge} ${stop} ${edgeLD}` :
                        l.y1 >= l.y2 ? `${edgeLU} ${stop} V ${l.y2 - gap} ${stop} ${edgeUR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}` : `${edgeLD} ${stop} V ${middleY - edge} ${stop} ${edgeDR} ${stop} H ${l.x2 - edge} ${stop} ${edgeRD}`;
                    else if (l.finishPos === 'right') helpPoints = l.x1 >= l.x2 ? l.y1 <= l.y2 ? `${edgeLD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}` : `${edgeLU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}` :
                        l.y1 <= l.y2 ? `${edgeLD} ${stop} V ${l.y2 + mpl * edge} ${stop} ${edgeDR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRU} ${stop} V ${l.y2 + edge} ${stop} ${edgeUL}` : `${edgeLU} ${stop} V ${l.y2 - mpl * edge} ${stop} ${edgeUR} ${stop} H ${l.x2 + gap} ${stop} ${edgeRD} ${stop} V ${l.y2 - edge} ${stop} ${edgeDL}`;
                }
            }
            return `${startPoint} ${arrow[l.startPos]} ${startStick[l.startPos]} ${helpPoints} ${finishStick}`;
        },
        _setSizeRuler() {
            this.cvWidth = this.$refs.main.offsetWidth;
            this.cvHeight = this.$refs.main.offsetHeight;
        },
        resizeRuler_h(width, zoom, cvWidth, scrollDx, step) {
            const canvas = this.$refs.ruler_h;
            const ctx = canvas.getContext('2d');
            const height = 25;
            this.initRuler(canvas, cvWidth, height);
            const smallLineHeight = 5;
            const middleLineHeight = 12;
            const highLineHeight = 20;
            let k = Math.round(cvWidth / step / zoom);
            for (let i = Math.round(scrollDx / zoom / step); i < Math.round(k + scrollDx / zoom / step); i++) {
                if (!(i % (10))) {
                    ctx.fillRect(Math.round(i * zoom * step - (scrollDx)), 5, 1, highLineHeight);
                    ctx.fillText(this._getTextValue(i * step, step), Math.round((i * zoom * step - scrollDx) + 5), 9, 25);
                    continue;
                }
                if (!(i % (5)) && step !== 0.1) {
                    ctx.fillRect(Math.round(i * zoom * step - scrollDx), 15, 1, middleLineHeight);
                    continue
                }
                if (step !== 0.1) {
                    ctx.fillRect(Math.round(i * zoom * step - scrollDx), 20, 1, smallLineHeight);
                }
            }
        },
        _onScroll(e) {
            e.preventDefault();
            e.stopPropagation();
            this.scrollDy = this.$refs.main.scrollTop;
            this.scrollDx = this.$refs.main.scrollLeft;
        },
        /**
         * Перерисовывает Вертикальную линейку
         * @param {number} depth.  высота элементов
         * @param {number} zoom. Масштаб
         * @param {number} cvHeight. высота линейки
         * @param {number} scrollDy. Положение вертикального scroll
         * @param {number} step. шкала линейки
         */
        resizeRuler_v(depth, zoom, cvHeight, scrollDy, step) {
            const canvas = this.$refs.ruler_v;
            const ctx = canvas.getContext('2d');
            const width = 25;
            this.initRuler(canvas, width, cvHeight);
            const smallLineHeight = 5;
            const middleLineHeight = 12;
            const highLineHeight = 20;
            ctx.rotate(.5 * Math.PI);

            let k = Math.round(cvHeight / step / zoom);

            for (let i = Math.round(scrollDy / zoom / step); i < Math.round(k + scrollDy / zoom / step); i++) {
                if (!(i % (10))) {
                    ctx.fillRect(Math.round(i * step * zoom - scrollDy), 0, 1, -highLineHeight - i);
                    ctx.fillText(this._getTextValue(i * step, step), Math.round(i * step * zoom - scrollDy) + 5, -2, 30);
                    continue;
                }
                if (!(i % 5) && step !== 0.1) {
                    ctx.fillRect(Math.round(i * step * zoom - scrollDy), -15, 1, -middleLineHeight);
                    continue
                }
                if (step !== 0.1) {
                    ctx.fillRect(Math.round(i * step * zoom - scrollDy), -20, 1, -smallLineHeight);
                }
            }
        },
        initRuler(canvas, width, height) {
            const ctx = canvas.getContext('2d');
            ctx.clearRect(0, 0, width, height);
            canvas.width = width;
            canvas.height = height;
            ctx.font = "normal 11px 'Lucida Grande', sans-serif";
            ctx.fillStyle = "#000";
        },
        setStep(zoom, step) {
            if (zoom) {
                if ((step * zoom) > 50) step = Math.min(0.1, step / 10);
                else if ((step * zoom) < 5) step = step * 10;
                if (step !== this.step) this.step = step
            }
        },
        _getTextValue(value, step) {//Преобразование значения линейки к  шкале
            return step === 0.1 ? value : (step === 1 || step === 10) ? value / 10 : (step === 100 || step === 1000 || step === 10000 ? value / 1000 : value / 1000000);
        },
    });

    ODA({
        is: 'oda-scheme-proxy',
        template: /* html */`
        <style>
            :host {
                position: absolute;
                cursor: pointer;
                @apply --vertical;
                grid-column: span 4;
                grid-row: span 2;
            }
            .wrapper {
                display: grid;
                grid-template-columns: auto auto auto;
                grid-template-rows: auto auto auto;
                grid-template-areas: ". top ."
                                     "left content right"
                                     ". bottom .";
            }
            .top {
                grid-area: top;
            }
            .left {
                grid-area: left;
            }
            .right {
                grid-area: right;
            }
            .bottom {
                grid-area: bottom;
            }
            .block {
                @apply --border;
                @apply --content;
                position:relative;
                width: 100%;
                @apply --vertical;
                justify-content: center;
                grid-area: content;
                box-sizing: border-box;
            }
            .connectors {
                display:flex;
                position: relative;
                align-items:center;
                justify-content: center;
            }
            .left, .right {
                flex-direction: column;
            }
            .top-menu, .bottom-menu, .dynamic {
                @apply --horizontal;
                justify-content: flex-start;
            }
            .menu-item {
                margin: 2px;
            }
            .menu-item:hover {
                background-color: rgba(255,165,0,.2);
            }
            .dynamic-top {
                position: absolute;
                top: 0;
                right: 0;
                z-index: 1;
            }
            .dynamic-bottom {
                position: absolute;
                bottom: 0;
                right: 0;
            }
        </style>
        <div class="flex wrapper">
            <div  class="connectors top" ~if="model">
                <oda-scheme-connector ref="top" draggable="true" ~for="model.top||[]" :oid="model.oid" pos="top" :i="index" :model="item" @down.stop :disabled="disabled" ></oda-scheme-connector>
            </div>
            <div  class="connectors left" ~if="model">
                <oda-scheme-connector ref="left" draggable="true" ~for="model.left||[]" :oid="model.oid" pos="left" :i="index" :model="item" @down.stop :disabled="disabled" ></oda-scheme-connector>
            </div>
            <div class="block" ~style="selected?{boxShadow: '0 0 9px orange'}:''">
                <div  class="dynamic-top" ~if="_getArrayOfMenu('dynamicMenu', 'top').length&&selected"><oda-icon class="dynamic-item" ~for="_getArrayOfMenu('dynamicMenu', 'top')" :icon="model.dynamicMenu['top'][index].icon" :size="16" :title="model.dynamicMenu['top'][index].title" :stroke="'red'" @tap.stop="_menuHandler('dynamicMenu', 'top', index)"></oda-icon></div>
                <div  class="top-menu" ~if="_getArrayOfMenu('staticMenu', 'top').length"><oda-icon class="menu-item" ~for="_getArrayOfMenu('staticMenu', 'top')" :icon="model.staticMenu['top'][index].icon" :size="12" :title="model.staticMenu['top'][index].title" @tap.stop="_menuHandler('staticMenu', 'top', index)"></oda-icon></div>
                <div ref="block" ~is="block.tag" :oid="block.oid"></div>
                <div  class="bottom-menu" ~if="_getArrayOfMenu('staticMenu', 'bottom').length"><oda-icon class="menu-item" ~for="_getArrayOfMenu('staticMenu', 'bottom')" :icon="model.staticMenu['bottom'][index].icon" :size="12" :title="model.staticMenu['bottom'][index].title" @tap.stop="_menuHandler('staticMenu', 'bottom', index)"></oda-icon></div>
                <div  class="dynamic-bottom" ~if="_getArrayOfMenu('dynamicMenu', 'bottom').length&&selected"><oda-icon class="dynamic-item" ~for="_getArrayOfMenu('dynamicMenu', 'bottom')" :icon="model.dynamicMenu['bottom'][index].icon" :size="16" :title="model.dynamicMenu['bottom'][index].title" :stroke="'red'" @tap.stop="_menuHandler('dynamicMenu', 'bottom', index)"></oda-icon></div>
            </div>
            <div  class="connectors right" ~if="model">
                <oda-scheme-connector ref="right" draggable="true" ~for="model.right||[]" :oid="model.oid" pos="right" :i="index" :model="item" @down.stop  :disabled="disabled" ></oda-scheme-connector>
            </div>
            <div  class="connectors bottom" ~if="model">
                <oda-scheme-connector ref="bottom" draggable="true" ~for="model.bottom||[]" :oid="model.oid" pos="bottom" :i="index" :model="item" @down.stop  :disabled="disabled"  ></oda-scheme-connector>
            </div>
        </div>
    `,
        props: {
            block: {},
            selected: Boolean,
            disabled: false,
            i: {
                type: Number
            },
            model: {
                get() {
                    return this.$refs.block ? this.$refs.block.model : {};
                }
            },
        },
        listeners: {
            track(e) {
                switch (e.detail.state) {
                    case 'track':
                        this.fire('moveItem', {i: this.i, x: e.detail.ddx, y: e.detail.ddy});
                        break;
                }
            },
            dragStartConn(e) {
                if (e.detail.value.oid === this.model.oid) this.disabled = true;
            },
            dragEndConn() {
                this.disabled = false;
            },
        },
        _getArrayOfMenu(menu, pos) {
            if (this.model && this.model.hasOwnProperty(menu) && this.model[menu].hasOwnProperty(pos)) return this.model[menu][pos];
            else return [];
        },
        _menuHandler(menu, position, index) {
            this.model[menu][position][index].action.call(this.$refs.block);
        },
        getConnectorCenterOffset(position, index) {
            return this.$refs[position] ? this.$refs[position][index].getCenterOffset() : {x: 0, y: 0};
        },
        getProxyOffsetWidth() {
            return this.offsetWidth;
        },
        getProxyOffsetHeight() {
            return this.offsetHeight;
        },
    });

    ODA({
        is: 'oda-scheme-connector',
        // hostAttributes:{
        //     draggable: 'true'
        // },
        template: /* html */`
            <style>
                .connector {                    
                    margin: 2px;
                    cursor: pointer;
                    transition: transform 100ms;
                }
            </style>
            <div class="connector" ~style="{border: model.shape&&model.shape.border,borderRadius: model.shape&&model.shape.borderRadius, background: model.backgroundColor || 'transparent', width: model.size || defaultSize, height: model.size || defaultSize, transform: dragover ?'scale(2)':'scale(1)'}">
                <oda-icon ~if="model.icon" :icon="model.icon" :size="model.size || defaultSize" :stroke="model.color"></oda-icon>
            </div>
        `,
        props: {
            oid: '',
            pos: '',
            i: 0,
            model: {},
            defaultSize: 16,
            dragover: false,
        },
        listeners: {
            mousedown(e) {
                if (this.model.service) e.preventDefault();
            },
            click(e) {
                e.preventDefault();
                e.stopPropagation();
                if (this.model.action) {
                    this.model.action.call(this.domHost.$refs.block, this.pos);
                }
            },
            dragstart(e) {
                const str = JSON.stringify({oid: this.oid, pos: this.pos, i: this.i});
                e.dataTransfer.setData('text/plain', str);
                this.fire('dragStartConn', {e, oid: this.oid, i: this.i, pos: this.pos, x: this.getCenterOffset().x, y: this.getCenterOffset().y});
            },
            drag(e) {
                this.fire('dragConn', {e, oid: this.oid, x: e.offsetX, y: e.offsetY});
            },
            dragend(e) {
                this.fire('dragEndConn', {e});
            },
            dragover(e) {
                if (this.model.addingSchemeConnector || !this.model.service && !this.domHost.tagName.includes('oda-scheme-designer'.toUpperCase())) e.preventDefault();
                this.dragover = true;
            },
            dragleave() {
                this.dragover = false;
            },
            drop(e) {
                e.preventDefault();
                e.stopPropagation();
                this.dragover = false;
                if (this.model.service) return;
                const target = JSON.parse(e.dataTransfer.getData('text'));
                const origin = {oid: this.oid, pos: this.pos, i: this.i};
                if (this.domHost.tagName.includes('oda-scheme-designer'.toUpperCase())) {
                    return this.domHost.addSchemeLinkToProxy(target, origin);
                }
                this.domHost.$refs.block.addLink(origin, target);
            },
        },
        getCenterOffset() {
            let x = this.offsetLeft + this.offsetWidth / 2 + this.parentElement.offsetLeft;
            let y = this.offsetTop + this.offsetHeight / 2 + this.parentElement.offsetTop;
            return {x, y}
        }
    });

    ODA({
        is: "oda-scheme-grid",
        template: /* html */ `
        <style>
            :host {
                width: 100%;
                height: 100%;
            }
        </style>
        <svg width="100%" height="100%" :preserve-aspect-ratio="'xMinYMin meet'">
            <defs>
                <pattern id="smallGrid" patternUnits="userSpaceOnUse" :width="realSizeSmallGrid" :height="realSizeSmallGrid">
                    <line x1="0" y1="0" x2="0" fill="none" stroke="gray" stroke-width="0.5" vector-effect="non-scaling-stroke" :y2="realSizeSmallGrid"></line>
                    <line x1="0" y1="0" y2="0" fill="none" stroke="gray" stroke-width="0.5" vector-effect="non-scaling-stroke" :x2="realSizeSmallGrid"></line>
                </pattern>
                <pattern id="grid" :width="realSizeBigGrid" patternUnits="userSpaceOnUse" :height="realSizeBigGrid">
                    <rect width="100%" height="100%" fill="url(#smallGrid)"></rect>
                    <line x1="0" y1="0" x2="0" fill="none" stroke="gray" stroke-width="1" vector-effect="non-scaling-stroke" :y2="realSizeBigGrid"></line>
                    <line x1="0" y1="0" y2="0" fill="none" stroke="gray" stroke-width="1" vector-effect="non-scaling-stroke" :x2="realSizeBigGrid"></line>
                </pattern>
            </defs>
            <rect width="100%" height="100%" fill="url(#grid)" vector-effect="non-scaling-stroke"></rect>
        </svg>
    `,
        props: {
            zoom: 1,
            step: 10,
            sizeBigGrid: 100,
            sizeSmallGrid: 10,
            realSizeBigGrid: {
                type: Number,
                get() {
                    return this.sizeBigGrid * this.zoom * this.step / 10;
                }
            },
            realSizeSmallGrid: {
                type: Number,
                get() {
                    if (this.step === 0.1) return 0;
                    else return this.realSizeBigGrid / this.sizeSmallGrid;
                }
            }
        }
    })
}
