import '../../buttons/button/button.js';
import '../../colors/palette/palette.js';

class BlockItem {
    constructor(item, uuid) {
        if (uuid) {
            this._uuid = uuid;
            _bus[this.uuid].$root = this;
        }
        this.$item = item;
        this._id = item.id || item.name || getUUID();
        this.selected = false;
        this.disabled = false;
        this.tag = item.tag || 'div';
    }
    get uuid() { return this._uuid }
    get _bs() { return _bus[this.uuid] || {} }
    set _bs(v) { return }
    get id() { return this._id }
    get label() { return this.$item.label || this.$item.name || 'block' }
    get items() {
        if (!this._items) {
            this._items = (this.$item.items || []).map(i => new BlockItem(i));
            if (this._items.length) {
                this._items.forEach(i => {
                    i._uuid = this._uuid;
                    i._owner = this;
                })
            }
        }
        return this._items;
    }
    get $owner() { return this._owner || this }
    get model() { return this.$item.model }
    get transform() {
        let x = (this.$item.x * (this._bs.zoom || 1));
        let y = (this.$item.y * (this._bs.zoom || 1));
        return `translate3d(${x}px, ${y}px, 0px)`
    }
    select(e) {
        clearSelectedBlocks(this);
        this._bs.selectedBlock = this;
        this.selected = true;
    }
    move(x, y) {
        this.$item.x += x / (this._bs.zoom || 1);
        this.$item.y += y / (this._bs.zoom || 1);
    }
    addDefaultConnector(position) {
        if (!this._bs.editMode) return;
        this.model[position].splice(this.model[position].length, 0, defaultConnector);
    }
    addConnector(position, index) {
        if (!this._bs.editMode) return;
        this.model[position].splice(this.model[position].length, 0, this.model[position][index].connector || defaultConnector);
    }
    deleteConnector(position, index) {
        if (!this._bs.editMode) return;
        this.model[position].splice(index, 1);
    }
    setConnectors(action = '') {
        this.$owner.items.forEach(i => {
            if (i.model)
                ['top', 'bottom', 'left', 'right'].forEach(p => {
                    let count = 0;
                    if (i.model[p])
                        i.model[p].forEach(c => {
                            c.index = count;
                            c._disabled = c.disabled || (action === 'dragStart' && this._bs?.line?.show && c.disableDrag);
                            if (action === 'deleteAllLinks' && c.link && c.link.id === this.id) delete c.link;
                            count++;
                        })
                })
        })
        if (this._bs._main) this._bs._main.render();
    }
    deleteBlock() {
        ODA[('showDialog')]('oda-scheme-ask', { text: 'Delete current block?' }, { parent: this, flex: 'noflex' }).then(res => {
            if (res) {
                this.$owner.items.splice(this.$owner.items.indexOf(this), 1);
                clearSelectedBlocks(this.$owner.items[0]);
                this._bs._main.render();
            }
        }).catch(e => { })
    }
    deleteAllLinks() {
        ODA[('showDialog')]('oda-scheme-ask', { text: 'Delete all links?' }, { parent: this, flex: 'noflex' }).then(res => {
            if (res) {
                this.setConnectors('deleteAllLinks');
            }
        }).catch(e => { })
    }
    getBlock(id) {
        for (const i of this.$owner.items)
            if (i.id === id)
                return i;
    }
}
const defaultConnector = {
    type: 'default', icon: 'icons:link', action: 'deleteConnector', size: '16', service: false,
    style: { border: '2px solid red', 'border-radius': '50% 5%', 'background-color': 'transparent' }, color: 'lime'
}
function clearSelectedBlocks(bl) {
    bl.$owner.items.forEach(i => {
        i.selected = false;
        i._bl.render();
    })
}

const _bus = {};
const getUUID = function b(a) { return a ? (a ^ Math.random() * 16 >> a / 4).toString(16) : ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, b) };

ODA({
    is: 'oda-scheme-ask', template: `
        <style>
            :host {
                @apply --vertical;
                justify-conent: center;
                align-items: center;
                color: red;
                font-size: 20px;
                padding: 16px;
            }
        </style>
        <div class="flex">{{text}}</div>
    `,
    props: {
        text: ''
    }
})

ODA({
    is: 'oda-scheme-block', template: `
        <style>
            :host {
                @apply --vertical;
                position: absolute;
                left: 0;
                top: 0;
                cursor: pointer;
                transform: {{bl?.transform}};
                z-index: {{bl?.selected ? 1 : 0}};
            }
            .container {
                zoom: {{bl?._bs?.zoom || 1}};
            }
            .block {
                @apply --vertical;
                @apply --border;
                @apply --content;
                position:relative;
                align-items: center;
                justify-content: space-between;
                width: 100%;
                box-shadow: {{bl?.selected ? '0 0 9px orange' : ''}};
                padding: 2px;
            }
            .info-block {
                border-radius: 2px;
                padding: 4px;
                display: flex;
                justify-content: center;
                align-items: center;
            }
            .connectors {
                display: flex;
                position: relative;
                align-items: center;
                justify-content: center;
                cursor: {{bl?._bs?.editMode? 'pointer':'default'}};
            }
        </style>
        <div class="container">
            <div class="connectors top horizontal" ~if="bl?.model">
                <div ~if="bl?.model?.left?.length" ~style="{width:$refs && $refs.right && $refs.left.offsetWidth || 0 + 'px'}"></div>
                <div class="connectors" ref="top">
                    <oda-scheme-connector ~if="item.editMode?editMode:!item.editMode" ~for="bl?.model?.top||[]" :item :bl position="top" :index></oda-scheme-connector>
                </div>
                <div ~if="bl?.model?.right?.length" ~style="{width:$refs && $refs.right && $refs.right.offsetWidth || 0 + 'px'}"></div>
            </div>
            <div class="horizontal">
                <div class="connectors vertical" ref="left" ~if="bl?.model">
                    <oda-scheme-connector ~if="item.editMode?editMode:!item.editMode" ~for="bl?.model?.left||[]" :item :bl position="left" :index></oda-scheme-connector>
                </div>
                <div class="block vertical flex" style="position: relative;justify-content:center;align-items:center">
                    <div class="info-block" ~is="bl?.tag" ~style="bl?.$item.style">{{bl?.label}}</div>
                    <div class="horizontal" style="width: 100%; justify-content: space-between;position:absolute;top:0">
                        <oda-icon ~if="i.editMode?editMode:!i.editMode" ~for="i in bl?.model?.staticMenu?.top||[]" :icon="i.icon" :icon-size="i.size||defaultSize" ~style="i.style" :title="i.title" @tap="_tap(i.action)" :disabled="i.disabled"></oda-icon>
                        <div class="flex"></div>
                        <oda-icon ~if="bl?.selected && i.editMode?editMode:bl?.selected" ~for="i in bl?.model?.dynamicMenu?.top||[]" :icon="i.icon" :icon-size="i.size||defaultSize" ~style="i.style" :title="i.title" @tap="_tap(i.action)" :disabled="i.disabled"></oda-icon>
                    </div>
                    <div class="horizontal" style="width: 100%; justify-content: space-between;position:absolute;bottom:0">
                        <oda-icon ~if="i.editMode?editMode:!i.editMode" ~for="i in bl?.model?.staticMenu?.bottom||[]" :icon="i.icon" :icon-size="i.size||defaultSize" ~style="i.style" :title="i.title" @tap="_tap(i.action)" :disabled="i.disabled"></oda-icon>
                        <div class="flex"></div>
                        <oda-icon ~if="bl?.selected && i.editMode?editMode:bl?.selected" ~for="i in bl?.model?.dynamicMenu?.bottom||[]" :icon="i.icon" :icon-size="i.size||defaultSize" ~style="i.style" :title="i.title" @tap="_tap(i.action)" :disabled="i.disabled"></oda-icon>
                    </div>
                </div>
                <div class="connectors vertical" ref="right" ~if="bl?.model">
                    <oda-scheme-connector ~if="item.editMode?editMode:!item.editMode" ~for="bl?.model.right?||[]" :item :bl position="right" :index></oda-scheme-connector>
                </div>
            </div>
            <div class="connectors" ~if="bl?.model">
                <div ~if="bl?.model?.left?.length" ~style="{width:$refs && $refs.right && $refs.left.offsetWidth || 0 + 'px'}"></div>
                <div class="connectors" ref="bottom">
                    <oda-scheme-connector ~if="item.editMode?editMode:!item.editMode" ~for="bl?.model.bottom?||[]" :item :bl position="bottom" :index></oda-scheme-connector>
                </div>
                <div ~if="bl?.model?.right?.length" ~style="{width:$refs && $refs.right && $refs.right.offsetWidth || 0 + 'px'}"></div>
            </div>
        </div>
    `,
    props: {
        bl: {
            type: Object,
            set(n) {
                if (n)
                    n._bl = this;
            }
        },
        defaultSize: 24,
        editMode: false
    },
    listeners: {
        track(e) {
            if (e.detail.state === 'track')
                this.bl.move(e.detail.ddx, e.detail.ddy);
        },
        down(e) {
            e.stopPropagation();
            this.bl.select(e);
        }
    },
    _tap(action) {
        if (!action || !this.bl?._bs?.editMode) return;
        if (typeof action === 'string') {
            if (this.bl[action])
                this.bl[action]();
        } else
            action.call(this.bl);
        this.render();
    },
    _getConnector(p, i) {
        if (this.$refs[p] && this.$refs[p].children)
            return this.$refs[p].children[i];
    }
})

ODA({
    is: 'oda-scheme-connector', template: `
        <style>
            :host {
                margin: 2px;
                transition: transform 100ms;
                transform: {{dragover ?'scale(2)':'scale(1)'}};
                z-index: {{dragover? 1 : 0}};
            }
        </style>
        <div ~style="item.style" :draggable :disabled="item?.disabled || item?._disabled || bl?._bs?.line?.bl === bl" :title="item?.title">
            <oda-icon ~if="item.icon" :icon="item.icon" :icon-size="item.size || defaultSize" :stroke="item.color"></oda-icon>
        </div>
    `,
    props: {
        item: {}, defaultSize: 16, dragover: false, index: 0, position: '', bl: Object,
        draggable() { return this.item?._disabled || this.item?.disableDrag || !this.bl?._bs?.editMode ? 'false' : 'true'; }
    },
    listeners: {
        down(e) {
            e.stopPropagation();
        },
        tap() {
            if (!this.item.action || !this.bl?._bs?.editMode) return;
            if (typeof this.item.action === 'string') {
                if (this.bl[this.item.action])
                    this.bl[this.item.action](this.position, this.index);
            } else
                this.item.action.call(this.bl, this.position, this.index);
            this.bl._bs = undefined;
        },
        dragstart(e) {
            let bs = this.bl._bs;
            bs.line.bl = this.bl;
            bs.line.item = this;
            bs.line.x1 = bs.line.x2 = e.x - bs._main.offsetLeft + bs._main.scrollLeft;
            bs.line.y1 = bs.line.y2 = e.y - bs._main.offsetTop + bs._main.scrollTop;
            bs.line.show = true;
            this.bl.setConnectors('dragStart');
            //this.bl._bs = undefined;
        },
        drag(e) {
            let bs = this.bl._bs;
            if (bs.line.show && e.x !== 0) {
                bs.line.x2 = e.x - bs._main.offsetLeft + bs._main.scrollLeft;
                bs.line.y2 = e.y - bs._main.offsetTop + bs._main.scrollTop;
                this.bl._bs = undefined;
            }
        },
        dragover(e) {
            if (this.bl._bs.line.show && this.draggable && !this.item._disabled && !this.item.disableDrag && this.bl?._bs?.line?.bl !== this.bl) {
                e.preventDefault()
                this.dragover = true;
            } else
                this.dragover = false;
        },
        dragleave() {
            this.dragover = false;
        },
        dragend(e) {
            this._dragend();
        },
        drop() {
            const l = this.bl._bs.line.item;
            this.item.link = { id: l.bl.id, position: l.position, index: l.item.index };
            this._dragend();
        }
    },
    _dragend() {
        let bs = this.bl._bs;
        bs.line.x1 = bs.line.x2;
        bs.line.y1 = bs.line.y2;
        bs.line.bl = undefined;
        bs.line.show = false;
        this.dragover = false;
        this.bl.setConnectors();
        this.bl._bs = undefined;
    }
})

ODA({
    is: 'oda-scheme-designer', template: `
        <style>
            :host {
                @apply -- vertical;
            }
            .tops {
                border-bottom: 1px solid gray;
                padding: 2px;
            }
        </style>
        <div class="tops horizontal flex"><div ~for="i in _tops" ~is="i.is" :icon="i.icon" :title="i.title" :style="i.style" allow-toggle :toggled="_toggled(i.toggled)||false" @tap="_tap(i.toggled,i.zoom)">{{i.text?_percent:''}}</div></div> 
        <oda-scheme-grid ::zoom :edit-mode="editMode" :hide-grid="hideGrid" :_bs :_width="width*10" :_height="height*10">
            <div slot="scheme-grid-main">
                <svg :width="width*10" :height="height*10" style="position:absolute;top:0;left:0;background-color:transparent;">
                    <path ~for="l in links" :d="_link(l)" :style="{strokeWidth:l.selected?'4px':'2px',cursor:'pointer','z-index':'-1'}" fill="none" opacity="0.5" :stroke="l.color||'red'"></path>
                </svg>
                <oda-scheme-block ~for="bl in block?.items" :bl :edit-mode=editMode></oda-scheme-block>
                <svg :width="width*10" :height="height*10" style="position:absolute;top:0;left:0;background-color:transparent;pointer-events:none;z-index:9">
                    <line ~if="_bs?.line?.show" :x1="_bs?.line?.x1" :y1="_bs?.line?.y1" :x2="_bs?.line?.x2" :y2="_bs?.line?.y2" style="stroke:red; stroke-width:2px;stroke-dasharray:2"></line>
                </svg>
            </div>
        </oda-scheme-grid>
    `,
    props: {
        item: Object, block: Object,
        editMode: { default: true, save: true }, hideGrid: { default: false, save: true },
        snapToGrid: false, brokenLine: false, zoom: 1, width: 1000, height: 1000, _bs: {},
        _tops: [
            { is: 'oda-button', icon: 'image:grid-off', title: 'hide grid', toggled: 'hideGrid' },
            { is: 'oda-button', icon: 'editor:border-style', title: 'snap to grid', toggled: 'snapToGrid' },
            { is: 'oda-button', icon: 'icons:trending-up', title: 'broken line view', toggled: 'brokenLine' },
            { is: 'div', style: 'flex:1;' },
            { is: 'oda-button', icon: 'icons:zoom-in', title: 'zoom-in', zoom: 1.1 },
            { is: 'oda-button', title: '100%', zoom: 1, text: 'ok' },
            { is: 'oda-button', icon: 'icons:zoom-out', title: 'zoom-out', zoom: 0.9 },
            { is: 'div', style: 'flex:1;' },
            { is: 'oda-button', icon: 'image:edit', title: 'editMode', toggled: 'editMode' }
        ],
        _percent() {
            return this.zoom === 1 ? '100%'
                : this.zoom < 1 ? `${this.zoom < 0.0001 ? Math.round(this.zoom * 100000000) / 1000000 : Math.round(this.zoom * 10000) / 100}%`
                    : `${Math.round(this.zoom * 100)}%`;
        },
        links: {
            reactive: true,
            get(){
                if (!this.block || !this._bs || !this._bs._main) return [];
                const links = [];
                let color = 0;
                this.block.items.map(b => {
                    ['top', 'bottom', 'left', 'right'].forEach(p => {
                        if (b._bl && b.model && b.model[p])
                            for (let c = 0; c < b.model[p].length; c++) {
                                const i = b.model[p][c];
                                if (i.link && i.link.id && i.link.position && (i.link.index || i.link.index === 0)) {
                                    let s = b._bl._getConnector(p, c);
                                    if (s) {
                                        s = s.getBoundingClientRect();
                                        const block = b.getBlock(i.link.id)
                                        let l = block._bl._getConnector(i.link.position, i.link.index);
                                        if (l) {
                                            l = l.getBoundingClientRect();
                                            links.push({
                                                x1: (l.x + l.width / 2) * (this._bs.zoom || 1) - this._bs._main.offsetLeft + this._bs._main.scrollLeft,
                                                y1: (l.y + l.height / 2) * (this._bs.zoom || 1) - this._bs._main.offsetTop + this._bs._main.scrollTop,
                                                x2: (s.x + s.width / 2 - (this.brokenLine ? 14 : 0)) * (this._bs.zoom || 1) - this._bs._main.offsetLeft + this._bs._main.scrollLeft,
                                                y2: (s.y + s.height / 2) * (this._bs.zoom || 1) - this._bs._main.offsetTop  + this._bs._main.scrollTop,
                                                selected: false, color: i.link.color || `hsla(${color}, 90%, 40%, 1)`, path: p + '-' + i.link.position
                                            })
                                            color = i.link.color ? color : color + 73;
                                        }
                                    }
                                }
                            }
                    })
                })
                return links;
            }
        },
    },
    observers: [
        '_initLayout(item)'
    ],
    _initLayout(item) {
        this.uuid = this.item.id = this.item.id || this.id || getUUID();
        _bus[this.uuid] = {
            fileName: 'oda-scheme-designer-' + this.uuid, zoom: 1,
            editMode: this.editMode, selectedBlock: undefined, line: { show: false, x1: 0, y1: 0, x2: 0, y2: 0 }
        };
        this.block = new BlockItem(item, this.uuid);
        this._bs = this.block._bs;
        this.block.items[0].setConnectors();
    },
    _toggled(e) { return this[e] },
    _tap(toggled, zoom) {
        if (toggled) this._bs[toggled] = this[toggled] = !this[toggled];
        if (zoom) this._bs.zoom = this.zoom = zoom === 1 ? 1 : zoom > 1 ? Math.min(400, this.zoom *= zoom) : Math.max(1 / 100000000, this.zoom *= zoom);
    },
    _link(l) {
        if (this.brokenLine) return `M${l.x1} ${l.y1} L ${l.x1 + 27.5} ${l.y1} ${l.x2 - 27.5} ${l.y2} ${l.x2} ${l.y2} ${l.x2 - 6} ${l.y2 - 3} ${l.x2 - 6} ${l.y2 + 3} ${l.x2} ${l.y2}`;
        if (l.path === 'left-right' || l.path === 'right-left') return `M${l.x1},${l.y1} Q${(l.x1 + l.x2) / 2},${l.y1} ${(l.x1 + l.x2) / 2},${(l.y1 + l.y2) / 2}  T${l.x2},${l.y2} `;
        return `M${l.x1},${l.y1} Q${l.x1},${l.y2} ${(l.x1 + l.x2) / 2},${(l.y1 + l.y2) / 2}  T${l.x2},${l.y2}`;
    }
});

ODA({
    is: "oda-scheme-grid", template: `
        <style>
            :host {
                @apply -- vertical;
            }
        </style>
        <div  ~show="!hideGrid" class="horizontal" style="height:25px;margin-bottom:5px">
            <div style="font-size:12px;min-width:30px;max-width:30px">{{_unit}}</div>
            <canvas ref="ruler_h"></canvas>
        </div>
        <div class="horizontal">
            <div ~show="!hideGrid" style="width:25px;margin-right:5px"><canvas ref="ruler_v"></canvas></div>
            <div class="main" ref="main" ~style="{height:_mainHeight}" style="position:relative;overflow:auto">
                <svg ~show="hideGrid" :width="_width" :height="_height"></svg>
                <svg ~show="!hideGrid" :width="_width" :height="_height" :preserve-aspect-ratio="'xMinYMin meet'">
                    <defs>
                        <pattern id="smallGrid" patternUnits="userSpaceOnUse" :width="_sizeSmall" :height="_sizeSmall">
                            <line x1="0" y1="0" x2="0" fill="none" stroke="gray" stroke-width="0.5" vector-effect="non-scaling-stroke" :y2="_sizeSmall"></line>
                            <line x1="0" y1="0" y2="0" fill="none" stroke="gray" stroke-width="0.5" vector-effect="non-scaling-stroke" :x2="_sizeSmall"></line>
                        </pattern>
                        <pattern id="grid" patternUnits="userSpaceOnUse" :width="_sizeBig" :height="_sizeBig">
                            <rect width="100%" height="100%" fill="url(#smallGrid)"></rect>
                            <line x1="0" y1="0" x2="0" fill="none" stroke="gray" stroke-width="1" vector-effect="non-scaling-stroke" :y2="_sizeBig"></line>
                            <line x1="0" y1="0" y2="0" fill="none" stroke="gray" stroke-width="1" vector-effect="non-scaling-stroke" :x2="_sizeBig"></line>
                        </pattern>
                    </defs>
                    <rect width="100%" height="100%" fill="url(#grid)" vector-effect="non-scaling-stroke"></rect>
                </svg>
                <slot name="scheme-grid-main"></slot>     
            </div> 
        </div>
    `,
    props: {
        editMode: true, hideGrid: false, zoom: 1, _width: 10000, _height: 10000, _step: 10, _scrollDx: 0, _scrollDy: 0,
        _unit() { return this._step === 0.1 ? 'mm' : (this._step === 1 || this._step === 10) ? 'cm' : (this._step === 100 || this._step === 1000 || this._step === 10000) ? 'm' : 'km' },
        _mainHeight() { return window.innerHeight - (this.$refs.main && this.$refs.main.offsetTop || 0) },
        _sizeBig() { return this.zoom * this._step / 10 * 100 },
        _sizeSmall() { return this._step === 0.1 ? 0 : this._sizeBig / 10 },
        _cvWidth() { return this.$refs && this.$refs.main && this.$refs.main.offsetWidth || 0 },
        _cvHeight() { return this.$refs && this.$refs.main && this.$refs.main.offsetHeight || 0 },
        _bs: {
            type: Object,
            set(n) { if (n) n._main = this.$refs.main; }
        },
    },
    observers: [
        '_setStep(zoom, _step)',
        '_resizeRuler_h(_width, zoom, _cvWidth, _scrollDx , _step)',
        '_resizeRuler_v(_height, zoom, _cvHeight, _scrollDy, _step)'
    ],
    listeners: {
        mousewheel(e) {
            if (!e.ctrlKey) return;
            e.stopPropagation();
            e.preventDefault();
            const k = 0.9;
            this._bs.zoom = this.zoom = e.deltaY > 0 ? Math.min(400, this.zoom / k) : Math.max(1 / 100000000, this.zoom * k);
        },
        down(e) { clearSelectedBlocks(this._bs.$root); }
    },
    attached() {
        this.listen('scroll', '_onScroll', { target: this.$refs.main });
        //this.listen('resize', '_setSizeRuler', { target: window });
    },
    _onScroll(e) {
        e.preventDefault();
        e.stopPropagation();
        this._scrollDy = this.$refs.main.scrollTop;
        this._scrollDx = this.$refs.main.scrollLeft;
    },
    _setStep(zoom, _step) {
        zoom = zoom > 1 ? Math.min(400, zoom) : Math.max(1 / 100000000, zoom);
        if (zoom === 400 || zoom === 1 / 100000000) {
            this._bs.zoom = this.zoom = zoom;
        } else {
            if ((_step * zoom) > 50) _step = _step / 10;
            else if ((_step * zoom) < 5) _step = _step * 10;
            if (_step !== this._step) this._step = _step;
        }
    },
    _resizeRuler_h(_width, zoom, _cvWidth, _scrollDx, _step) {
        const ctx = this._initRuler(this.$refs.ruler_h, _cvWidth, 25);
        let k = Math.round(_cvWidth / _step / zoom);
        for (let i = Math.round(_scrollDx / zoom / _step); i < Math.round(k + _scrollDx / zoom / _step); i++) {
            if (!(i % (10))) {
                ctx.fillRect(Math.round(i * zoom * _step - _scrollDx), 5, 1, 20);
                ctx.fillText(this._getTextValue(i * _step, _step), Math.round((i * zoom * _step - _scrollDx) + 5), 9, 25);
            } else if (!(i % (5)) && _step !== 0.1) ctx.fillRect(Math.round(i * zoom * _step - _scrollDx), 15, 1, 12);
            else if (_step !== 0.1) ctx.fillRect(Math.round(i * zoom * _step - _scrollDx), 20, 1, 5);
        }
    },
    _resizeRuler_v(_height, zoom, _cvHeight, _scrollDy, _step) {
        const ctx = this._initRuler(this.$refs.ruler_v, 25, _cvHeight);
        ctx.rotate(.5 * Math.PI);
        let k = Math.round(_cvHeight / _step / zoom);
        for (let i = Math.round(_scrollDy / zoom / _step); i < Math.round(k + _scrollDy / zoom / _step); i++) {
            if (!(i % (10))) {
                ctx.fillRect(Math.round(i * _step * zoom - _scrollDy), 0, 1, -20 - i);
                ctx.fillText(this._getTextValue(i * _step, _step), Math.round(i * _step * zoom - _scrollDy) + 5, -2, 30);
            } else if (!(i % 5) && _step !== 0.1) ctx.fillRect(Math.round(i * _step * zoom - _scrollDy), -15, 1, -12);
            else if (_step !== 0.1) ctx.fillRect(Math.round(i * _step * zoom - _scrollDy), -20, 1, -5);
        }
    },
    _initRuler(canvas, width, height) {
        const ctx = canvas.getContext('2d');
        ctx.clearRect(0, 0, width, height);
        canvas.width = width;
        canvas.height = height;
        ctx.font = "normal 11px 'Lucida Grande', sans-serif";
        ctx.fillStyle = "#000";
        return ctx;
    },
    _getTextValue(value, _step) {
        return _step === 0.1 ? value : (_step === 1 || _step === 10) ? value / 10 : (_step === 100 || _step === 1000 || _step === 10000 ? value / 1000 : value / 1000000);
    },
})
