import ODA from '../../../oda.js';
import '../../layouts/title/title.js';
import '../../buttons/icon/icon.js';
ODA({
    is: 'oda-menu', template: /*html*/`
        <style>
            :host {
                min-width: 100px;
                max-width: 100vw;
                max-height: 100vh;
                @apply --vertical;
                background-color: whitesmoke;
                overflow: hidden;
            }
            :host>div{
                overflow-y: auto;
            }
            :host>div>div{
                align-items: center;
                cursor: pointer;
                font-size: medium;
            }
            :host>div>div:hover{
                outline: 1px dotted gray;
                outline-offset: -1px;
                @apply --active;
            }
            oda-icon{
                opacity: 0.75;
            }
        </style>
        <oda-title ~show="title" :title :icon></oda-title>
        <div class="vertical">
            <div ~for="items" class="horizontal no-flex" @tap.stop="_tap" :item style="align-items: center;">
                <oda-icon ~if="!item.hideIcon" :icon="item?.icon" :item="item" :icon-size="iconSize" ~style="{padding: \`\${Math.floor(iconSize * 0.2)}px\`}"></oda-icon>
                <div class="content flex horizontal" style="align-items: stretch;">
                    <div ~is="item.is || item.template || template" :item="item" class="flex" ~style="{marginRight: (item.items && item.items.length)?-iconSize:0 +'px',
                            paddingRight: item.items && item.items.length?iconSize:0,  borderTop: item.group === true ? '1px solid gray' : 'none'}"></div>
                    <oda-icon ref="ddPlace" ~if="item?.items?.length" :item="item" style="align-items: center; justify-content: center;" icon="icons:arrow-drop-up:90" @tap.stop="_open"></oda-icon>
                </div>
            </div>
        </div>
    `,
    props: {
        icon: '',
        title: '',
        template: 'oda-menu-item',
        items: [],
        iconSize: {
            default: 24,
            shared: true
        },
        focusedItem: {
            set(n) {
                if (n) {
                    if (this.root)
                        this.root.focusedItem = n;
                    this.parentElement.fire('ok');
                }
            }
        },
        submenu: null,
        root: null,
    },
    async _open(e) {
        if (this.submenu && this.submenu === e.target) {
            this.submenu = null;
            return;
        }
        this.submenu = e.target;
        try {
            const target = this.$refs.ddPlace.find(i => i.item === e.target.item);
            await ODA.showDropdown('oda-menu', { items: e.target.item.items, root: this, template: this.template }, { parent: target });
        }
        catch (e) { }
        finally {
            // this.async(() => {
            //     this.submenu = null;
            // }, 300)
        }
    },
    _tap(e) {
        this.focusedItem = e.currentTarget.item;
    }
});
ODA({
    is: 'oda-menu-item', template: `
        <style>
            :host {
                @apply --label;
                @apply --horizontal;
                @apply --flex;
                align-items: center;
                text-overflow: ellipsis;
            }
            :host>*{
                padding: 4px;
            }
        </style>
        <span class="label" :title="item?.label">{{item.label}}</span>
    `,
    props: {
        item: {}
    }
});
