Команда **showDialog** позволяет выводить любой компонент в отдельном диалоговом окне.

Она имеет следующий синтаксис:

```javascript
ODA.showDialog(component, props = {}, hostProps = {})
```

Ей передаются 3 параметра:

1. **component** — определяет компонент, который будет отображаться в диалоговом окне. Например, **'oda-icon'**.
1. **props** — определяет свойства компонента, который будет отображаться в диалоговом окне. Например, **{icon: 'icons:android', iconSize: 50}**.
1. **hostProps** — определяет свойства самого диалогового окна. Например, **{animation: 500}**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500});
    }
});
```

Данная команда возвращает промис, исполнитель которого вызывает функцию **resolve**, если пользователь нажал на кнопку «**Ok**» или **reject** (если им была нажата кнопка «**Cancel**»). В обработчиках этих функций можно предусмотреть необходимые действия, которые должны выполняться в том или в ином случае.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажим на меня</button>
        <span>Вы нажали кнопку: {{res}}</span>
    `,
    props: {
        res: ''
    },
    onTap() {
        ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500}).then(res=>this.res = 'Ok', () => this.res = 'Cancel');
    }
});
```

Колбеку функции **resolve** в качестве параметра будет передаваться компонент, который отображался в диалогом окне. Через этот параметр можно узнать какие изменения в него внес пользователь.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажим на меня</button>
        <span>Возвращаемый результат: {{res}}</span>
    `,
    props: {
        res: ''
    },
    onTap() {
        ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500}).then(res=>this.res = String(res), () => this.res =  'Не предусмотрено');
    }
});
```

```error_md
В настоящее время значение компонента не возвращается, но этот функционал будет реализован в будущем.
```

Колбэку **reject** никакого параметра не передается.

В качестве первого параметра функции showDialog **component** можно передавать не имя компонента, а сам компонент, который необходимо отобразить в диалоговом окне. В этом случае он не будет повторно создаваться функцией **showDialog**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/button/button.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-button ref="btn" label="Нажми на меня" icon="icons:favorite" icon-size="50" @tap="onTap" blink="1000"></oda-button>
    `,
    props: {
        value: ''
    },
    onTap() {
        ODA.showDialog(this.$refs.btn, {icon: 'icons:android', iconSize: 50}, {animation: 500});
    }
});
```

Свойства, передаваемые компоненту вторым параметром, не будут в этом случае ему присваиваться. Считается, что они у него уже были заданы.

Например, в предыдущим примере второй параметр можно не передавать. В этом случае он по умолчанию будет задан пустым объектом.

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/button/button.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-button ref="btn" label="Нажми на меня" icon="icons:favorite" icon-size="50" @tap="onTap" blink="1000"></oda-button>
    `,
    props: {
        value: ''
    },
    onTap() {
        ODA.showDialog(this.$refs.btn, undefined, {animation: 500});
    }
});
```
