Команда **ODA.showDropdown** позволяет выводить любой компонент в отдельном выпадающем окне.

Она имеет следующий синтаксис:

```javascript
ODA.showDropdown(component, props = {}, hostProps = {})
```

Ей передаются 3 параметра:

1. **component** — определяет компонент, который будет отображаться в выпадающем окне. Например, **'oda-icon'**.
1. **props** — определяет свойства компонента, который будет отображаться в выпадающем окне. Например, **{icon: 'icons:android', iconSize: 50}**.
1. **hostProps** — определяет свойства самого выпадающего окна. Например, **{minWidth: 50}**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=80_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        ODA.showDropdown('oda-icon', {icon: 'icons:android', iconSize: 50}, {minWidth: 50});
    }
});
```

Выпадающее окно закрывается автоматически при щелчке по любой области. При этом возвращается промис, исполнитель которого при нажатии на область внутри выпадающего окна вызывает функцию **resolve**, или функцию **reject** при щелчке по области вне окна. В обработчиках этих функций можно предусмотреть необходимые действия, которые должны выполняться в том или в ином случае.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=80_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажим на меня</button>
        <span>Вы щелкнули {{res}} окна</span>
    `,
    props: {
        res: 'по неизвестно какой области'
    },
    onTap() {
        ODA.showDropdown('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500}).then(res=>this.res = 'внутри', () => this.res = 'снаружи');
    }
});
```

Колбеку функции **resolve** в качестве параметра предается компонент, который был отображен в диалогом окне. Через этот параметр можно узнать, какие изменения в нем совершил пользователь.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажим на меня</button>
        <span>Возвращаемый результат: {{res}}</span>
    `,
    props: {
        res: ''
    },
    onTap() {
        ODA.showDropdown('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500}).then(res=>this.res = res.localName, () => this.res =  'Не предусмотрено');
    }
});
```

Выпадающее окно можно отобразить в статическом режиме. Для этого его содержимое необходимо разместить внутри тега компонента **oda-dropdown2** и вызвать метод **show** для его отображения.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/tools/containers/dropdown/dropdown.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-dropdown ref="dropdown">
            <span>Статический режим отображения</span>
        </oda-dropdown>
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        this.$refs.dropdown.parent = this;
        this.$refs.dropdown.show();
    }
});
```

Для использования этого режима необходимо подключить модуль **dropdown-2.js**, в котором располагается сам код компонента **oda-dropdown-2**.

У самого выпадающего окна существуют следующие свойства:

1. **parent** — указатель на родителя окна. Значение по умолчанию: **нет**.
1. **minWidth** — минимальная ширина окна. Значение по умолчанию: **100** пикселей.
1. **maxWidth** — максимальная ширина окна. Значение по умолчанию: **0** пикселей.
1. **maxHeight** — максимальная высота окна. Значение по умолчанию: **0** пикселей.
1. **intersect** — разрешает перекрывать область родителя. Значение по умолчанию: **false**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <label>Минимальная ширина <input type="number" ::value="minWidth" step="50"></label><br>
        <label>Перекрыть родителя<input type="checkbox" ::value="intersect"></label><br>
        <button @tap="onTap">Нажим на меня</button>
    `,
    props: {
        minWidth: '100',
        intersect: false
    },
    onTap() {
        ODA.showDropdown('oda-icon', {icon: 'icons:android', iconSize: 50}, {minWidth: this.minWidth, intersect: this.intersect});
    }
});
```

В настоящее время свойство **intersect** не оказывает никакого влияния на отображение компонента.
