Команда **ODA.showModal** позволяет выводить любой компонент в отдельном модальном окне.

Она имеет следующий синтаксис:

```javascript
ODA.showModal(component, props = {}, hostProps = {})
```

Ей передаются 3 параметра:

1. **component** — определяет компонент, который будет отображаться в диалоговом окне. Например, **'oda-icon'**.
1. **props** — определяет свойства компонента, который будет отображаться в диалоговом окне. Например, **{icon: 'icons:android', iconSize: 50}**.
1. **hostProps** — определяет свойства самого модального окна. Например, **{animation: 500, title: 'Модальное окно'}**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        ODA.showModal('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: 500});
    }
});
```

Данная команда ничего не возвращает, а модальное окно будет автоматически закрываться при нажатии на область вне окна.

Модальное окно можно отобразить в статическом режиме. Для этого необходимо разместить его содержимое внутри тега компонента **oda-modal** и вызвать метод **show** для его отображения.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/tools/containers/modal/modal.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-modal ref="modal">
            <span>Статический режим отображения</span>
        </oda-modal>
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        this.$refs.modal.show();
    }
});
```

Для использования этого режима необходимо подключить модуль **modal.js**, в котором располагается сам код компонента **oda-modal**.

Для закрытия модального окна программным способом нужно вызвать его метод **close**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
import '/tools/containers/modal/modal.js';
ODA({
    is: 'my-component',
    template: `
        <oda-modal ref="modal">
            <span>Я закроюсь сама</span>
        </oda-modal>
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        this.$refs.modal.show();
        let myClose = this.$refs.modal.close.bind(this.$refs.modal);
        setTimeout(myClose , 2000);
    }
});
```

``` info_error_md
Через секунду после появления окно должно закрыться, но так как дочерний элемент внутри него не удален, то будет происходить ошибка.
```

У самого модального окна существуют следующие свойства:

1. **title** — задает заголовок окна.
1. **titleMode** — задает режим отображение заголовка окна.
    Оно может принимать следующие значения:
    1. **none** — заголовок не отображается.
    1. **auto** — заголовок отображается по ширине содержимого.
    1. **full** — заголовок отображается по ширине всего окна.
    Например, в предыдущем примере второй параметр можно не передавать. В таком случае он по умолчанию будет задан пустым объектом.

    ```javascript run_line_edit_loadoda_[my-component.js]_h=100_
    import '/components/buttons/button/button.js';
    ODA({
        is: 'my-component',
        template: `
            <oda-button ref="btn" label="Нажми на меня" icon="icons:favorite" icon-size="50" @tap="onTap" blink="1000"></oda-button>
        `,
        props: {
            value: ''
        },
        onTap() {
            ODA.showModal(this.$refs.btn, undefined, {title: 'Заголовок', titleMode: 'full'});
        }
    });
    ```

1. **animation** — задает продолжительность анимаций открытия и закрытия окна в миллисекундах.

    ```javascript run_line_edit_loadoda_[my-component.js]_h=100_
    import '/components/buttons/icon/icon.js';
    ODA({
        is: 'my-component',
        template: `
            <label> Продолжительность анимации  <input type="number" ::value="animation" step="100">, мс</label>
            <button @tap="onTap">Нажми на меня</button>
        `,
        props: {
            animation: 500
        },
        onTap() {
            ODA.showModal('oda-icon', {icon: 'icons:android', iconSize: 50}, {animation: this.animation});
        }
    });
    ```

1. **iconSize** — задает размер иконки в пикселях. Это свойство в данный момент не используется.
