**async** — это метод, который позволяет вызвать другой метод или функцию с заданной временной задержкой.

Он объявляется внутри компонента следующим образом:

```javascript
async (handler, delay = 0)
```

Ему передаются 2 параметра:

1. **handler** — вызываемая функция или метод.
1. **delay** — задержка вызова метода или функции в миллисекундах.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня'
    },
    onTap() {
        this.async(() => {this.label='Я выполняюсь только через секунду'}, 1000);
        this.async(() => {this.label='Нажми на меня'}, 2000);
    }
});
```

В данном примере вызываются стрелочные функции через одну и две секунды соответственно. Указатель **this** у них привязан к контексту текущего компонента **my-component**, так как они объявляются в его методе **onTap**. По этой причине они имеют доступ к свойству компонента **label** через указатель **this**.

При передаче метода в списке параметров необходимо явно привязать к его контексту компонент с помощью метода **bind**, так как по умолчанию он автоматически ему передаваться уже не будет. Указатель **this** в этом случае внутри метода будет ссылаться уже не на сам компонент, а на глобальный объект **window**.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня'
    },
    up() {
        this.label = 'Я выполняюсь только через секунду';
    },
    down() {
        this.label = 'Нажми на меня';
    },
    down
    onTap() {
        this.async(this.down.bind(this), 1000);
        this.async(this.up.bind(this), 2000);
    }
});
```

Аналогичная ситуация происходит при вызове анонимной функции, контекст которой зависит от способа её вызова. Она вызывается внутри метода **setTimeout** в виде функции, поэтому указатель **this** внутри нее будет ссылаться на глобальный объект **window**.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня',
        count: 0
    },
    onTap() {
        this.async(function() {this.label="Я выполняюсь только через секунду";}.bind(this), 1000);
        this.async(function() {this.label="Нажми на меня";}.bind(this), 2000);
    }
});
```

Если в последних двух примерах не указывать метод **bind**, то все изменения будут происходить с глобальным объектом **window**, а не с компонентом **my-component**.
