**debounce** — это метод, который ограничивает частоту вызова другого метода или функции.

Он объявляется внутри компонента следующим образом:

```javascript
debounce (key, handler, delay = 0)
```

Ему передаются 3 параметра:

1. **key** — уникальный идентификатор, под которым регистрируется вызываемая функция.
1. **handler** — указатель на функцию, которую необходимо вызвать.
1. **delay** — время, ограничивающее вызов функции.

Фактически передаваемая функция **handler** будет вызываться только по истечении указанного времени **delay** после ее последнего вызова. Если произойдет ее повторный вызов до истечения этого времени, то предыдущий вызов будет отменен, а отсчет задержки времени нового вызова начнется заново.

Такой подход позволяет ограничить частоту вызова функции и повысить производительность работы браузера, предотвратив многократную обработку событий, вызванных пользователем при ряде действий: изменении размера окна, скроллинге страницы, нажатии клавиш клавиатуры, перемещении указателя мыши и так далее.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <label>Нажми любую клавишу <input @keydown="onKeyDown" ref="typewriter"> </label> <button @tap="onTap">Очистить</button><br>
        <label>Временная задержка <input @mouseleave="$refs.typewriter.focus()" type="number" ::value="delay" step="10">, мс</label>
        <div>Количество событий нажатия: {{countEvent}}</div>
        <div>Количество обработок нажатия: {{count}}</div>
    `,
    props: {
        count: 0,
        countEvent: 0,
        delay: 0
    },
    onTap() {
        this.count=0;
        this.countEvent=0;
        this.$refs.typewriter.value = '';
        this.$refs.typewriter.focus();
    },
    onKeyDown() {
        this.countEvent++;
        this.debounce('my-keydown', ()=>this.count++, this.delay);
    }
});
```

Если в данном примере нажать и не отпускать любую клавишу клавиатуры внутри первого элемента **input**, то событие **keydown** будет происходить довольно часто, и каждый раз оно будет обрабатываться в методе **onKeyDown**. Однако если увеличить задержку, то, начиная с небольшого значения (20-30 мc), его обработка методом **debounce** будет происходить не всегда, а только когда после последнего нажатия на клавишу пройдет достаточное время, чтобы метод **debounce** смог запустить его обработку. Если времени пройдет меньше чем задано в параметре **delay**, то обработка события не начнется, а таймер его ожидания заново обнулится.

Если в качестве задержки указать значение 0 (ноль), то обработка события произойдет не мгновенно, а только тогда, когда браузер обновит страницу в следующий раз, так как метод **debounce** в этом случае выполняет указанную функцию или метод в контексте метода **requestAnimationFrame**, который вызывается браузером примерно 60 раз в секунду при стандартной частоте обновления кадров 60 fps. При большей загрузке браузера эта частота может уменьшаться.
