Хуки (**Hooks**) – это методы, которые вызываются автоматически в определенные моменты жизненного цикла компонента.

Диаграмму жизненного цикла компонента с моментами вызова хуков можно представить следующим образом:

![Диаграмма жизненного цикла компонента](./learn/images/life-cycle-title.svg "Жизненный цикл компонента")

Имена хуков строго определены и имеют следующие назначения:

| Имя хука           | Назначение |
| :----------------  | :--------- |
| created           | Хук created можно использовать для выполнения кода после создания компонента. |
| ready            | Хук ready вызывается при инициализации компонента перед появлением в DOM. |
| attached              | Хук attached выполняется во время появления компонента на странице. |
| detached              | Хук detached выполняется когда компонент пропадает со страницы. |

Например:

```javascript _run_line_edit_console_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div>Hooks ...</div>
    `,
    created() {
        console.log('created');
    },
    ready() {
        console.log('ready');
    },
    attached() {
        console.log('attached');
    }
    detached() {
        console.log('detached');
    }
});

Диаграмму жизненного цикла компонента с моментами вызова этих хуков можно представить следующим образом:

![Диаграмма жизненного цикла компонента](./learn/images/life-cycle-title.svg "Жизненный цикл компонента")

Жизненный цикл компонента начинает с момента его создания функцией ODA. Она создает новый компонент на основе его прототипа, указанного в списке ее параметров.

Любой компонент представляет собой объект с типом пользовательского класса **odaComponent**, который является наследником нативного класса языка JS **HTMLElement**. В результате этого он наследует все его [элементы](https://html.spec.whatwg.org/multipage/dom.html#htmlelement "interface HTMLElement").

В конструкторе этого класса назначаются свойства компонента на основе информации о них из раздела **props** его прототипа, осуществляется регистрация их реактивности с помощью метода **makeReactive** и создается теневое дерево методом **attachShadow**, который возвращает указатель на его корень в закрытом режиме. После этого вызывается хук **created**, если он предусмотрен в компоненте.

```javascript _run_line_edit_console_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div>{{text}}</div>
    `,
    props: {
        text: "Привет, Хук created!"
    }
    created() {
        console.log('created');
        console.log(this.text);
        console.log(this.$core.shadowRoot);
        console.log(JSON.stringify(this.$core.shadowRoot));
    }
});
```

В этом хуке значение свойств компонента уже определены с учетом их реактивности, а теневой дерево создано. Однако оно является пустым. Шаблон компонента еще не разобран. Компонент на странице не зарегистрирован.

После хука **created** продолжается выполняться конструктор компонента. Здесь в специальный объект телеметрии записывается прототип компонента и продолжается выполнятся. В нем созданный компонент ре

Изначальное теневое дерево является пустым.

```javascript_run_line_edit_console_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div>Hooks ...</div>
    `,
    created() {
    	console.log('created');
    },
    ready() {
    	console.log('ready');
    },
    attached() {
    	console.log('attached');
    }
});
```
</p>