Директива : используется для динамического связывания атрибута или свойства компонента с указанным выражением.

```javascript_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <input :value="text"></span>
        <button @tap="_onTap">Bind</button>
    `,
    props: {
        text: 'A'
    },
    _onTap() {
        if (this.text ==='A') {
            this.text = 'B';
         }
        else {
             this.text = 'A';
        }
    }
});
```

В HTML-элемент атрибут или свойство, записанное после символа двоеточия будет связан с выражением, идущим после знака равно и записанным в кавычках.

В этом случает любое изменение указанного выражения будет приводить к изменению указанного атрибута или свойства компонента, используя механизм реактивности.

В данном примере изменение свойства **text** будет реактивно приводить к изменению содержимого компонент.

```like
Если имя свойства элемента совпадает с именем свойства указанного в выражении, то выражение в директиве **:** можно не указывать.
```

```javascript_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <input :value></span>
        <button @tap="_onTap">Bind</button>
    `,
    props: {
        value: 'A'
    },
    _onTap() {
        if (this.value ==='A') {
            this.value = 'B';
        }
         else {
            this.value = 'A';
        }
    }
});
```

В этом примере используется синтаксическое упрощение ("сахар"), так как имя свойства **value** элемента **input** совпадает с именем свойства самого компонента и его можно указать только один раз.

```like
Связывать можно не только свойства компонентов, но и атрибуты.
```

```javascript_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <a :href="url">Привет, атрибут</a>
    `,
    props: {
        url: 'http://htmlbook.ru/example/knob.html'
    }
});
```

В этом примере свойство компонента **url** будет связано со атрибутом **href** элемента **a**.
