Директива **~for** используется для создания последовательности HTML-элементов на основе массива данных.

```warning_md
В этой директиве обязательно должно быть указано как имя самого массива, так и ссылка на его текущий элемент.
```
Пример 1:
```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="item in items">Элемент: {{item}} </div>
    `,
    props: {
        items: [
            'A', 'B', 'C'
        ]
    }
});
```

В приведенном примере из массива **items** будут последовательно извлечены все элементы и для каждого из них будет создан свой собственный HTML-элемент **div**, в котором будет выводиться значение текущего элемента массива с помощью указателя на него **item**.

В расширенной форме записи, помимо значения текущего элемента массива, можно указать еще и ссылку на его индекс.

Пример 2:
```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="item, index in items">Элемент[{{index}}]: {{item}}</div>
    `,
    props: {
        items: [
            'A', 'B', 'C'
        ]
    }
});
```

В этом примере для ссылки на индекс элемента массива использовалось имя **index**.

```info_md
Кроме структуры цикла **for...in**, можно использовать структуру **for...of**.
```

Пример 3:
```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="item, index of items">Элемент[{{index}}]: {{item}}</div>
    `,
    props: {
        items: [
            'A', 'B', 'C'
        ]
    }
});
```

Результат выполнения этого примера будет точно таким же, как и у предыдущего.

Кроме этого, существует упрощенная форма записи директивы **~for**, в которой можно не указывать ссылки на элемент массива и его индекс.

Пример 4:
```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="items">Элемент[{{index}}]: {{item}}</div>
    `,
    props: {
        items: [
            'A', 'B', 'C'
        ]
    }
});
```

В этом случае в значение директивы будет достаточно записать только имя самого массива, а имена ссылок **item** и **index** будут добавлены автоматически. В результате этого обращаться к ним можно будет точно так же, как и при полной форме записи.

```info_md
При использовании упрощенной формы имена ссылок **item** и **index** задаются по умолчанию. Если у компонента уже существуют элементы с такими же именами, то именно эти элементы будут использоваться в качестве параметров цикла вместо ссылок по умолчанию.
```

Пример 5:
```javascript _error_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="items">Элемент[{{index}}]: {{item}}</div>
    `,
    props: {
        items: [
            'A', 'B', 'C'
        ],
        item: "Ошибка",
        index: -1
    }
});
```

В этом примере значения параметров **item** и **index** не будут меняться в цикле. Они перекрываются значениями свойств самого компонента с теми же самыми именами.

```warning_md
 По этой причине крайне нежелательно использовать имена **item** и **index** для свойств, методов или других элементов компонента.
```

С помощью директивы **~for** можно вывести последовательность целых чисел, начиная с 1, если значение, указанное в JS-выражении, можно привести к целому типу.

Пример 6:
```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~for="count">Элемент[{{index}}]: {{item}}</div>
    `,
    props: {
        count: 10
    }
});
```

Вычисленное значение директивы в этом случае определяет последнее число в генерируемой последовательности. В данном примере это значение равно 10.

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/5grvyQc4-bI?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen 
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>

