﻿Значение выражения, записанное в двойных фигурных скобках {{ Mustache }}, или в директиве **~text** подставляется как простой текст независимо от его внутреннего формата. Если необходимо вместо обычного текста выводить текст с HTML разметкой, то необходимо вместо директивы **~text** использовать специальную директиву **~html**.

```javascript_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <p>{{myText}}</p>
        <p ~html="myText"></p>
    `,
    props: {
        myText: '<span style="color: red"> Текст должен быть красным </span>'
    }
});
```

В первом варианте (тэг **p**) свойство **myText** интерпретируется как обычный текст. Содержимое второго тега **p** будет заменено значением свойства **myText**, которое уже интерпретируется как HTML-код.

Важно помнить, что в HTML-элементах, созданных директивой **~html**, все внутренние привязки игнорируются.

```info_md
Директиву ~html нельзя использовать для вложения шаблонов друг в друга. Вместо этого нужно использовать компоненты, позволяющие объединять и повторно использовать элементы UI.
```

```warning_md
Динамическая отрисовка произвольного HTML-кода на сайте крайне опасна, так как может легко привести к XSS-уязвимостям. Используйте интерполяцию HTML только для доверенного кода, и никогда не подставляйте туда содержимое, создаваемое пользователями.
```

```warning_md
Директиву **~html** использовать не рекомендуется. Явно указывайте все необходимые HTML-элементы в шаблоне компонента, управляйте их отображением с помощью директив **~show** и **~if / ~else / ~else-if** и модифицируйте их вид через управление стилями.
```

Если HTML-элемент содержит вложенные элементы, то директива **~html** игнорируется.

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div ~html="myText">
            <div>Это вложенный HTML-элемент</div>
        </div>
    `,
    props: {
        myText: '<span style="color: red"> Текст должен быть красным </span>'
     }
});
```

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/vZwCdAMvuqw?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen 
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>
