﻿Директива **~save-key** используется для сохранения значений свойств компонентов в Web-хранилище и восстановления их оттуда как при обновлении страницы, так и при перезапуске браузера.

В Web-хранилище сохраняются только свойства, у которых модификатор **save** установлен в значение **true**.

Пример 1

```javascript _line_edit_[my-input-component.js]
ODA({
    is: 'my-input-component',
    template: `
        <input ::value></input>
        <button @tap="_clear">Очистить хранилище</button>
    `,
    props: {
        value: {
            default: 'Введите текст',
            save: true
        }
    },
    _clear() {
        this.clearSaves();
        window.location.reload(true);
    }
});
```

Данные записываются в Web-хранилище в виде пары **ключ/значение**. Значение определяется свойством, в котором указан параметр **save**, а ключ определяется директивой **~save-key** того компонента, чье свойство необходимо сохранить.

Пример 2

```javascript _run_line_edit_[my-component.js]_{my-input-component.js}
ODA({
    is: 'my-component',
    template:`
        <my-input-component ~save-key="'myKey'"></my-input-component>
    `
});
```

Ключ можно задавать не только в виде строкового литерала, но и с помощью JavaScript-выражения.

 Пример 3

```javascript _run_line_edit_[my-component.js]_{my-input-component.js}
ODA({
    is: 'my-component',
    template:`
        <my-input-component ~save-key="'myKey' + 2"></my-input-component>
    `
});
```

Записанные в хранилище данные будут автоматически восстанавливаться как после обновления страницы, так и после перезапуска браузера.

Например, если обновить страницу с этими примерами, то значение, введенное в компоненте **my-component** будет автоматически восстановлено.

Чтобы восстановить значение, заданное в компоненте по умолчанию, необходимо удалить данные из хранилища. Для этого необходимо вызвать метод **clearSaves()**. Метод не имеет параметров и удаляет сохраненные значения всех свойств компонента, из которого он вызван. В примере 1 показано использование этого метода для восстановления значения по умолчанию у свойства **value** компонента **my-input-component**.

```info_md
Удалить из хранилища сохраненные данные всех компонентов для текущего Web-сайта можно командой **window.localStorage.clear()**.
```

При нажатии на эту красную кнопку восстанавливаются значения по умолчанию для всех примеров на данной странице:

```javascript _run_line_edit_nocopy_[my-component.js]
ODA({
    is: 'my-component',
    template:`
        <button @tap="window.localStorage.clear()" style="background-color:red">RESET</button>
        `
});
```

В отличие от технологии куки (**cookie**) данный способ хранения пользовательских данных имеет ряд достоинств:

1. Объекты Web-хранилища не отправляются на сервер при каждом запросе. Поэтому в хранилище можно хранить гораздо больше данных. Большинство браузеров могут сохранить минимум 2 мегабайта данных (или больше), и этот объем можно поменять в настройках.

1. Сервер не может манипулировать объектами хранилища через HTTP-заголовки. Всё делается при помощи JavaScript.

1. Web-хранилище привязано к источнику (домен/протокол/порт). Это значит, что разные протоколы или поддомены определяют разные объекты хранилища и они не могут получить доступ к данным друг друга.

Если в проекте применяется несколько экземпляров одного компонента и у них в директиве **~save-key** указано одно и то же значение ключа, то в хранилище будут записываться только значения свойств компонента, расположенного последним в структуре документа. В этом случае при обновлении страницы сохраненное значение будет восстанавливаться для всех однотипных компонентов, у которых совпадает значение ключа.

Пример 3

```javascript _run_line_edit_nocopy_[my-component.js]_{my-input-component.js}
ODA({
    is: 'my-component',
    template:`
        <my-input-component ~save-key="'key1'"></my-input-component><br>
        <my-input-component ~save-key="'key2'"></my-input-component><br>
        <my-input-component ~save-key="'key1'"></my-input-component><br>
        <button @tap="window.location.reload(true)">Обновить</button>
        `
});
```

В данном примере три одинаковых компонента содержат директиву **~save-key**, но в ней используются два разных ключа: **key1** и **key2**, поэтому сохраняться будут значения только для двух компонентов. Ключ **key1** имеют два компонента, и в хранилище будет записываться одно общее для них значение, которое будет браться из третьего компонента. При обновлении страницы, это значение будет восстанавливаться как для первого, так и для третьего компонента.

```info_md
Обратите внимание, что отсутствие директивы **~save-key** НЕ отменяет сохранение свойств с модификатором **save**. Свойства по-прежнему сохраняются в хранилище, но без ключа. Фактически все компоненты без директивы **~save-key** имеют одно и то же "пустое" значение ключа. Соответственно сохранение и восстановление значений свойств осуществляется также как для компонентов с одинаковыми ключами.
```

Пример 4

```javascript_run_line_edit_nocopy_[my-component.js]_{my-input-component.js}
ODA({
    is: 'my-component',
    template:`
        <my-input-component></my-input-component><br>
        <my-input-component></my-input-component><br>
        <my-input-component></my-input-component><br>
        <button @tap="window.location.reload(true)">Обновить</button>
        `
});
```

В данном примере при сохранении значения свойств не используется директива **~save-key**, поэтому для всех компонентов будет восстанавливаться одно и то же значение, вводимое в третий компонент, так как он является последним в структуре документа.

```warning_md
Будьте внимательны и учитывайте порядок поиска данных в хранилище при восстановлении значений свойств у компонентов, используемых с директивой **~save-key**. Вначале ищутся данные, сохраненные с ключом, указанным в директиве **~save-key**. Если в хранилище данные с указанным ключом отсутствуют, то ищутся данные с "пустым" ключом. Если данные с "пустым" ключом будут найдены, то в хранилище сразу создастся их копия с ключом из директивы **~save-key**, затем данные загрузятся в компонент.
```

Пример 5

```javascript _run_line_edit_nocopy_[my-component.js]_{my-input-component.js}
ODA({
    is: 'my-component',
    template:`
        <span>Значение с ключом:</span>
        <my-input-component ~save-key="'key3'"></my-input-component>
        <br>
        <span>Значение без ключа:</span>
        <my-input-component></my-input-component>
        <br>
        <button @tap="window.location.reload(true)">Обновить</button>
        `
});
```

В данном примере значение, введенное в первой строке ввода, сохраняется с ключом **key3**, а значение из второй строки ввода сохраняется с "пустым" ключом. Если в первой строке ввода ввести какое-либо значение, то оно будет восстанавливаться при перезагрузке страницы независимо от содержимого второй строки ввода. Если удалить значение первой строки ввода из хранилища нажав кнопку "Удалить значение с ключом", то при перезагрузке страницы в первой строке будет восстановлено значение из второй строки.

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/uOQBlyWafe4?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>
