Директива двойного биндинга **::** используется для взаимного связывания свойств двух HTML-элементов.

Любое изменение одного из свойств будет автоматически приводить к изменению другого свойства, и наоборот. Имена связанных свойств указываются в директиве через символ равно **=**, при этом второе свойство, идущее после знака равно, необходимо указывать в кавычках.

Например:

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <input ::value="text"></input>
        <button @tap="_onTap">{{text}}</button>
    `,
    props: {
        text: 'Мы взаимосвязаны'
    },
    _onTap() {
        this.text = this.text==='Мы взаимосвязаны' ?  'Введите что-нибудь' : 'Мы взаимосвязаны';
    }
});
```

В данном примере свойство **value** нативного HTML-элемента **input** связывается со свойством **text** пользовательского компонента. Любое изменение свойства **text** приводит к изменению свойства **value**, и наоборот — любое изменение свойства **value** приведет к изменению свойства **text**. В результате этого текст, введенный в компонент **input**, будет сохранятся в свойстве **text** компонента, а изменения свойства **text** будут автоматически передаваться в элемент **input**.

``` like_md
Если имена взаимосвязанных свойств совпадают, то знак **=** и второе свойство, идущее после него, в целях упрощения можно не указывать.
```

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <input ::value></input>
        <button @tap="_onTap">{{value}}</button>
    `,
    props: {
        value: 'Мы взаимосвязаны'
    },
    _onTap() {
        this.value = this.value==='Мы взаимосвязаны' ?  'Введите что-нибудь' : 'Мы взаимосвязаны';
    }
});
```

В этом примере имя свойства компонента **value** совпадает с именем свойства элемента **input**.  Поэтому в директиве символ **=** и второе свойство, идущее после него, не указано. Фактически эта запись является синтаксическим упрощением (т.н. "сахаром"), позволяющим не записывать дважды в директиве одно и тоже имя для двух разных свойств.

Аналогичным образом можно связывать свойства вложенного и родительского компонентов.

Например:

```javascript _line_edit_[second-component.js]
ODA({
    is: 'second-component',
    template: `
        <span>{{text}}</span>
        <button @tap="_changeText">{{text}}</button>
    `,
    props: {
        text: {
            default: 'Второй компонент',
            notify: true
        }
    },
    _changeText() {
        this.text = 'Второй компонент';
    }
});
```

``` warning_md
Для того чтобы директива двойного связывания работала, необходимо разрешить вложенному компоненту передачу сообщения об изменении значения его свойства внешнему компоненту. Для этого у его свойства необходимо задать модификатор **notify** со значением **true**. У свойства внешнего компонента этот модификатор указывать не нужно. Все изменения во вложенный компонент будут передаваться автоматически, как при обычном связывании.
```

```javascript _run_line_edit_[first-component.js]_{second-component.js}
ODA({
    is: 'first-component',
    template: `
        <button @tap="_changeText">{{text}}</button>
        <span>{{text}}</span>
        <second-component ::text="text"></second-component>
    `,
    props: {
        text: 'Первый компонент'
    },
    _changeText() {
        this.text = 'Первый компонент';
    }
});
```

Этот пример не будет работать из-за того, что директива двойного связывания будет пересекаться с директивой **text**.

```warning_md
С точки зрения HTML, директива двойного биндинга представляет собой атрибут тега и, соответственно, должна строго следовать требованиям, предъявляемым к синтаксису атрибута. HTML является регистронезависимым языком, поэтому нельзя напрямую использовать имена свойств в camelCase в левой части директивы двойного биндинга — необходимо преобразовать имена этих свойств в kebab-case.
```

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/KVf8phiZZ10?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen 
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>
