Глобальная стилизация задает CSS-стили, которые автоматически добавляются к заголовку HTML-страницы, к которой подключена библиотека **oda.js**.

Для этого в обработчике события **load** создается специальный компонент «**oda-style**», который сразу после окончания загрузки HTML-страницы автоматически регистрирует в разделе **head** HTML-документа все предусмотренные в нем стили.

Эти стили условно можно разделить на 6 функциональных групп:

1. **layouts** — это группа предназначена для стилизации оконных компонентов, содержащих слои.
1. **shadow** — это группа предназначена для стилизации теней у элементов.
1. **special** — это группа предназначена для стилизации диалоговых компонентов, выводящих различные сообщения.
1. **effects** — это группа предназначена для стилизации различных состояний HTML-элементов (получение или потеря фокуса, активация и т.д.).
1. **oda** — это группа предназначена для стилизации анимаций различных эффектов у компонентов.
1. **theme** — это группа предназначена для стилизации отображения компонента сайта.

Кроме этого, все глобальные CSS-стили сохраняются в свойстве **styles** самого компонента «**oda-style**», доступ к которому можно получить через свойство **style** глобального объекта фреймворка **ODA**.

Например,

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div>Глобальные стили:</div>
        <div>{{ODA.style.styles['--active']}}</div>
    `,
});
```

Свойство **styles** по своей структуре является объектом, имена свойств которого совпадают с именами CSS-объявлений, предусмотренных во всех функциональных группах глобальной стилизации.

Каждое из таких объявлений начинается с двойного дефиса и может быть либо CSS-свойством, либо CSS-миксином.

Например,

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div>Я CSS-свойство</div>
        <div>{{ODA.style.styles['--success-color']}}</div>
        <div>А я миксин</div>
        <div>{{ODA.style.styles['--horizontal']}}</div>
    `,
});
```

В отличие от миксинов, CSS-свойства имеют только одно CSS-объявление и подключаются в других CSS-объявлениях или миксинах с помощью функции **var**.

Например,

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <style>
            div {
                color: var(--success-color);
            }
        </style>
        <div>
            Контейнер элементов:
            <div>Элемент 1</div>
            <div>Элемент 2</div>
        </div>
    `,
});
```

Миксины объединяют несколько CSS-объявлений и подключаются к другим миксинам или CSS-правилам с помощью директивы **@apply**.

Например,

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <style>
            div {
                @apply --horizontal;
            }
        </style>
        <div>
            Контейнер элементов:
            <div>Элемент 1</div>
            <div>Элемент 2</div>
        </div>
    `,
});
```

У глобальной стилизации существует одна особенность: при указании имени предопределенного миксина в качестве имени CSS-класса в директиве **class**, все его объявления будут автоматически добавлены в раздел **style** в виде одноименного CSS-правила.

Например,

```javascript _run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <div class="horizontal">
            <div>Элемент 1</div>
            <div>Элемент 2</div>
        </div>
    `,
});
```

В данном примере задан CSS-класс с именем глобального миксина **--horizontal**. В результате этого все CSS-объявления этого миксина будут автоматически преобразованы в одноименный CSS-класс следующим образом:

```html
<my-component>
    #shadow-root (closed)
        <style>
            .horizontal, ::slotted(.horizontal) {
                display: flex;
                flex: 1;
            }
        </style>
        <div class="horizontal">
            <div>Элемент 1</div>
            <div>Элемент 2</div>
        </div>
</my-component>
```

Следует отметить, что фреймворк также позволяет использовать вложенные директивы **@apply**.

К стилям, например, можно применить глобальный миксин **--bar**, который объявляется следующим образом:

```css
    --bar {
        @apply --horizontal;
    }
```

В этом случае к элементам **div** будут применены такие же CSS-объявления, как и в предыдущем примере.

Например,

```javascript error_run_line_edit_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <style>
            div {
                @apply --bar;
            }
        </style>
        <div>
            <div>Элемент 1</div>
            <div>Элемент 2</div>
        </div>
    `,
});
```

Элементы **div** здесь будут располагаться не вертикально, а горизонтально, так как это указано в миксине **--horizontal**.

Каждая из существующих групп глобальных стилей со всеми предусмотренными в ней CSS-свойствами и миксинами будет подробно рассмотрена в соответствующем разделе.
