import ODA from '../../../oda.js';
ODA({
    is: "oda-dropdown", template: `
    <style>
        :host{
            visibility: {{_ready?'visible':'hidden'}};
        }    
    </style>
    <slot @down.stop tabindex="1" ref="container" ~show="opened" style="overflow: auto; z-index: 100; position: fixed;" ~style="_size" class="shadow content border flex vertical"></slot>`,
    props: {
        opened: {
            default: false,
            set(opened, closed) {
                this._ready = false;
                if (closed) {
                    this.unlisten('keydown', '_keyDown', {target: document});
                    this.unlisten('mousedown', '_close', {target: document});
                    this.unlisten('resize', 'close', {target: document});
                    this.unlisten('scroll', 'close', {target: document});
                } else if (opened) {
                    this.listen('keydown', '_keyDown', {target: document});
                    this.listen('mousedown', '_close', {target: document});
                    this.listen('resize', 'close', {target: document});
                    this.listen('scroll', 'close', {target: document});
                    this._setSize();
                }
            }
        },
        parent: { type: [HTMLElement, Object] },
        minWidth: 100,
        maxWidth: 0,
        maxHeight: 0,
        intersect: false,
        _size: null,
        _ready: false,
        // alignByParent: {
        //     default: false,
        //     reflectToAttribute: true
        // }
    },
    attached() {
        for (let el of this.$core.shadowRoot.children)
            if (el.localName !== 'style' && el.localName !== 'slot') el.setAttribute('slot', '*');
    },
    open(params = { prev: undefined, control: undefined, parent: undefined }) {
        for (let p in params)
            if (params[p]) this[p] = params[p];
        this.opened = true;
        return new Promise((resolve, reject) => {
            this.listen('close', () => {
                this.unlisten(this, 'close');//Он не будет удалять. У него нет слушателя.
                resolve(this);
            })
        })
    },
    close() {
        this.opened = false;
        this.fire('close');
    },
    toggle() {
        if (this.opened) this.close();
        else this.open();
    },
    _keyDown(e) {
        if (e.keyCode === 27) this.close();
    },
    _setSize(repeat = false) {
        requestAnimationFrame(() => {
            const size = {},
                container = this.$refs && this.$refs.container,
                containerSize = container.getBoundingClientRect(),
                rect = new ODARect(this.parent);
            size.maxHeight = size.maxWidth = size.minHeight = '';
            size.minWidth = ((this.parent && this.parent.offsetWidth > this.minWidth) ? this.parent.offsetWidth : this.minWidth || this._containerSize.width) + 'px';
            const top = this.intersect ? rect.top : rect.bottom,
                bottom = window.innerHeight - (this.intersect ? rect.top : rect.bottom) - 4;
            if (bottom > containerSize.height) {
                size.maxHeight = bottom + 'px';
                size.top = top + 'px';
            } else {
                if (this.parent) size.top = top + 'px';
                size.bottom = '0px';
            }
            const right = window.innerWidth - rect.left;

            if (right > containerSize.width) {
                size.maxWidth = right + 'px';
                size.left = rect.left + 'px';
            } else {
                size.right = '0px';
            }
            if (this.parent) {
                if (this.maxWidth > 0) {
                    if (this.maxWidth < this.minWidth)
                        size.maxWidth = size.minWidth;
                    else
                        size.maxWidth = this.maxWidth + 'px';
                }
                if (this.maxHeight) size.maxHeight = this.maxHeight + 'px';
            }
            this._size = { ...{}, ...size };
            if (this.parent && !repeat) this._setSize(true);
            this._ready = true;
        });
    },
    _close(e) {
        if (e.target !== window && e.target !== document.body && e.path.find(i => (i.localName === this.localName))) return;
        this.close();
    }
});

export const openDropdown = (parent) => {
    const dropdown = document.createElement('oda-dropdown');
    document.body.appendChild(dropdown);
    dropdown.open({ parent: parent }).then(resolve => {
        document.body.removeChild(dropdown);
    });
}
