/*
 * oda-site v1.0
 * (c) 2020 Roman Perepelkin
 * Under the MIT License.
 */
import '../components/layouts/app-layout/app-layout.js';
import '../tools/containers/dropdown/dropdown.js';
import '../components/grids/tree/tree.js';

const statuses = [
    { id: 'no', icon: '', color: 'lightgray' },
    { id: 'temp', icon: 'image:edit', color: 'lightblue' },
    { id: 'check', icon: 'image:remove-red-eye', color: 'green' },
    { id: 'edit', icon: 'editor:border-color', color: 'red' },
    { id: 'ok', icon: 'icons:check', color: 'lightgreen' },
];

function clone(obj){
    const res = Object.assign({}, obj);
    res.hideExpander = true;
    res.items = Object.assign([], res.items).map(i=>clone(i));
    return res;
}
function route(item, idx){
    if (item && item.execute){
        actions[item.execute](item);
    }
    else{
        ODA.router.go("#"+item.$id, idx);
    }
}
const actions = {
    async setStyle(item){
        try{
            ODA.style.update();
            const theme = await ODA.loadJSON(ODA.rootPath+'/'+item.$id+'.json');
            ODA.style.update(theme);
        }
        catch (e) {}
    },
    async setLocale(item){
        try{
            ODA.locale = '';
            const locale = await ODA.loadJSON(ODA.rootPath+'/'+item.$id+'.json');
            ODA.style.update(theme);
        }
        catch (e) {}
    }
};
let prev = {};
function loadInfo(root, inf){
    if (!root.items) {
        root.isOk = true;
        let items = [];
        if (inf.folder) {
            items = inf.folder.map(i=>{
            i.$id = (root.$id?(root.$id + '/'):'') + i.name;
            i.status = statuses.find(s => s.id === (i.status)) || statuses[0];
            i.$expanded = true;
            // i.hideExpander = true;
            prev.next = i;
            i.prev = prev;
            prev = i;
            return (i.name && i.name.includes('.')) || (i.content && i.content.link && i.content.link.includes('.html')) ? i : loadInfo(i, i);
            });
        }
        root.items = items;
        root._statusReady = { count: 0, ok: 0, percent: 0 };
    }
    return root;
}

async function _readODAsrc(){
    let src =  await fetch(ODA.rootPath + '/oda.js');
    src =  await src.text();
    window.odasrc = URL.createObjectURL(new Blob([src], { type: 'text/javascript' }));
};
if (!window.odasrc) {
    _readODAsrc();
};

site:{
    ODA({
        is: 'oda-site',
        extends: 'oda-app-layout',
        hostAttributes:{
            'right.icon': 'icons:warning',
            'left.splitter.hidden': true
        },
        template:`
            <oda-site-header :items="items" slot="top-right" ::part="part">Задвижной header</oda-site-header>
            <div class="no-flex horizontal" slot="top-left" style="align-items: center;">
                <img width="64" class="no-flex" @tap="window.location.hash = '';focusedItem={}" src="icon.png" style="cursor: pointer; margin: 8px;">
                <div class="horizontal">
                    <input class="flex" type="search" @change="_changed">
                    <div style="cursor: pointer;">&#128269;</div>
                    <a href="https://www.youtube.com/channel/UC37BC2N95knlYcDRK4cQoug" target="_blank"><img src="youtube.png" height="20" style="margin: 6px 0 0 6px"/></a>
                </div>
            </div>
            <oda-site-content-tree :part="focusedItem" :slot="!src?'main':'no_slot'" style="overflow:auto"></oda-site-content-tree>
            <oda-site-nav-tree show-tree-lines :slot="(part && part.items && part.items.length>0)?'left-drawer':'no_slot'" :part="part" :focused-node="focusedItem"></oda-site-nav-tree>
            <div :slot="!!part?'right-drawer':'no_slot'">А здесь экосистема и прочая поддержка</div>
            <video ~if="_srcIframe==='video'" @pause="_stop" ref="video" :slot="!focusedItem || part === null?'main':'no_slot'" width="100%" height="100%" style="background-color: black;" src="site/intro.mp4" @tap="_playVideo" poster="site/intro.webp"></video>
            <iframe ~if="_srcIframe!=='video'" :slot="part === null?'main':'no_slot'" :src="_srcIframe" style="width:100%;height:100%;border:none;"></iframe>
            <iframe ref="iframe" class="flex" ~show="(focusedItem && (!focusedItem.items || focusedItem.items.length === 0)) && src" :src="src" slot="main" style="flex: 200 1 auto; border: none;"></iframe>
        `,
        props:{
            left:{
                splitter: 'hidden'
            },
            src: '',
            siteMode: true,
            _editMode:{
                default: false,
                shared: true
            },
            hash:{
                type: String,
                async set(v){
                    this.part = null;
                    if (!v)
                        return;
                    const groups = v.split('#').filter(i=>i);
                    while (groups.length){
                        let part = this;
                        const group = groups.shift();
                        const steps =  group.split('/');
                        while (part && steps.length){
                            const step = steps.shift();
                            if (!part.items) part.items = [];
                             part = part.items.find(i=>i.name === step);
                        }
                        if (this.part)
                            this.focusedItem = part;
                        else{
                            this.focusedItem = null;
                            this.part = part;
                        }

                    }
                }
            },
            part: {
                type: Object,
                freeze: true,
                set(n){
                    if (!this.focusedItem)
                        this.focusedItem = n;
                    // this.async(()=>{
                    //     this.focusedItem = n;
                    // })
                }
            },
            items: Array,
            focusedItem: {
                type: Object,
                set(n) {
                    this._setSrc(n);
                }
            },
            _srcIframe: 'video',
            hideOnScroll: true
        },
        _setSrc(n = this.focusedItem) {
            let url;
            if (n && n.content)
                url = ODA.rootPath + '/' + n.$id + '/' + (n.content.link || n.content.src);
            if (url && !url.endsWith('.md')) {
                setTimeout(() => {
                    this.$refs.iframe.contentDocument._editMode = this._editMode;
                    this.bubbleIframeMouseWheel(this.$refs.iframe);
                }, 500);
                this.src = url;
                return;
            }
            this.src = '';
        },
        bubbleIframeMouseWheel(iframe) {
            if (!this.hideOnScroll) return;
            var existingOnMouseWheel = iframe.contentWindow.onwheel;
            iframe.contentWindow.onwheel = function (e) {
                if (existingOnMouseWheel) existingOnMouseWheel(e);
                iframe.dispatchEvent(new MouseEvent('wheel', {
                    'view': window,
                    'bubbles': true,
                    'cancelable': false,
                    'detail': e.wheelDelta
                }));
            };
        },
        async ready(){
            ODA.router.create('#*,', (hash)=>{
                this.hash = hash;
            });
            document.addEventListener("setTheme", e => {
                let iframeContent = this.$refs.iframe.contentDocument;
                iframeContent.querySelector('style[group=theme]').textContent = e.detail.value;
            });
        },
        _stop(e){
            if(document.fullscreen)
                document.exitFullscreen();
        },
        _playVideo(e){
            if (this._srcIframe !== 'video') return;
            if(!this.$refs.video.src.endsWith('site/intro1.mp4'))
                this.$refs.video.src = 'site/intro1.mp4';
            this.$refs.video.muted = false;
            this.$refs.video.requestFullscreen();
            this.play();
        },
        play(){
            if (this._srcIframe !== 'video') return;
            if (this.$refs.video.currentTime >= this.$refs.video.duration){
                this.$refs.video.currentTime = 0;
            }
            if (this.$refs.video.paused)
                this.$refs.video.play();
            else
                this.$refs.video.pause();
        },
        attached() {
            this.async(async () => {
                if (this._srcIframe === 'video') this.$refs.video.muted = true;
                const path = window.location.pathname;
                let _url = path.endsWith('.html') ? path.replace(/\/\w+.html/, '/learn/_info.js') : path + 'learn/_info.js';
                _url = await import(_url);
                this._srcIframe = _url._srcIframe.src;
                await loadInfo(this, _url.default);
                this._setStatus(this.items[0]);
                this.hash = window.location.hash;
                if (!this.hash && this._srcIframe === 'video'){
                    this.play();
                }
            });
        },
        _changed(e) {
            this._editMode = e.currentTarget.value === '~~~' || e.currentTarget.value === '###';
        },
        _setStatus(root) {
            if (root._statusReady) {
                this._setCount(root, root);
                root._statusReady.percent = root._statusReady.ok * 100 / root._statusReady.count;
            }
            if (root.items) {
                root.items.forEach(e => {
                    this._setStatus(e);
                });
            }
        },
        _setCount(r, root) {
            if (r.items) {
                r.items.forEach(e => {
                    if (e.status && !e._statusReady) {
                        root._statusReady.count += 1;
                        root._statusReady.ok += e.status.id === 'ok' ? 1 : 0;
                    }
                    this._setCount(e, root);
                });
            }
        }
    });
}

content:{
    ODA({is:'oda-site-content-tree', extends: 'oda-tree',
        props:{
            defaultTemplate: 'oda-site-content-cell',
            treeStep: 0,
            rowLines: false,
            part:{
                set(n){
                    // this.async(()=>{
                        if (n){
                            n = clone(n);
                            this.dataSet = (n && [n]) || [];
                        }
                        else
                            this.dataSet = [];
                    // });
                }
            },
            columns: [{ name: 'item', treeMode: false, showLines: false }]
        }
    });
    ODA({is: 'oda-site-content-cell', extends: 'oda-table-cell-base', template:`
        <style>
            :host * {
                @apply --user-select;
                white-space: initial;
            }
            :host {
                display: unset !important;
            }
            .title{
                cursor: pointer;
                text-decoration: underline;
            }
            .title:hover{
                @apply --selected;
            }
        </style>
        <div class="vertical flex" style="padding: 2px 2px;">
            <div :style="h" style="padding-bottom:4px" class="title no-flex" @down="_tap" ~html="value"></div> 
            <oda-md-viewer ~if="abstract" :src="abstract"></oda-md-viewer>
            <oda-md-viewer ~if="item && item.name && item.name.includes('.md') && item.$level === 0" 
                :src="path" class="flex" style="border: none;" :item></oda-md-viewer>
        </div>`,
        props:{
            h(){
                //return 'h3'// + ((this.item.$level || 0) + 2);
                let l = this.item.$level || 0;
                let style = `
                    padding-left: ${2 + l * 30}px;
                    font-size: ${24 - l}px;
                    font-weight: ${550 - l * 50};
                    color: ${this.item && ((this.item.name && this.item.name.includes('.md')) || (this.item.content && this.item.content.link)) ? '#6699cc' : '#336699'};
                `;
                return style;
            },
            value:{
                get(){
                    return this.item.label || this.item.name;
                }
            },
            abstract(){
                return this.item.content && this.item.content.abstract;
            },
            path(){
                return (this.item &&  (this.item.$id || (this.item.content && (this.item.content.link || this.item.content.src)))) || '';
            }
        },
        _tap(e) {
            route(this.item, 1)
        },
        async ready() {
            await import('../components/viewers/md-viewer/md-viewer.js');
        }
    })
}
navigator:{
    ODA({is:'oda-site-nav-tree', extends: 'oda-tree',
        template:`
            <style>
                :host, :host *{
                    @apply --layout;
                }
 
            </style>
        `,
        props:{
            allowFocus: true,
            defaultTemplate: 'oda-site-nav-cell',
            rowLines: false,
            iconSize: 24,
            columns: [],
            part:{
                set(n){
                    this.dataSet = (n && [n]) || [];
                }
            },
            _editMode: {
                default: false,
                set(n){
                    this.columns = n ? [{ name: 'item', width: 24, template: 'oda-status-cell' }, { name: 'item', treeMode: true }] : [];
                }
            }
        }
    });
    ODA({is: 'oda-status-cell', extends: 'oda-table-cell', template:`
        <style>
            :host{
                min-width: 22px;
                max-width: 22px;
            }
            :hover {
                cursor: pointer;
            }

            div {
                width: 14px;
                height: 14px;
                border: 2px solid lightgreen;
                border-radius: 50%;
                cursor: pointer;
            }
        </style>
        <div ~if="item && item._statusReady" :title @tap="_tap(item)" ~style="_style"></div>
        <oda-icon ~if="item && !item._statusReady" :icon-size="20" :icon="item.status && item.status.icon" ~style="{fill:item.status?item.status.color:''}" :title="item && item.$id" @tap="_tap(item)"></oda-icon>`,
        props: {
            title: {
                get() { 
                    let i = this.item;
                    return i ? `${i.$id} [${i._statusReady.ok}/${i._statusReady.count}-${Math.round(i._statusReady.percent)}%]` : ''; 
                }
            },
            _style: {
                get() {
                    return `background:conic-gradient(lightgreen 0% ${this.item && this.item._statusReady.percent}%, white 0% 100%);`;
                }
            }
        },
        async _tap(e) {
            await navigator.clipboard.writeText(e.label);
            let s = (e.$id.startsWith('components') && !e._statusReady) ? e.$id + '/$info/_info.js' : 'learn/_info.js';
            this.async(() => {
                window.open(ODA.rootPath + '/components/editors/md-editor/md-editor.html?src=' + ODA.rootPath + '/' + (e.$id.includes('.') ? e.$id : s));
            });
        }
    });
    ODA({is: 'oda-site-nav-cell', extends: 'this, oda-table-cell-base', template:`
        <style>
            :host{
                @apply --header;
                cursor: pointer;
            }
            :host(:hover){
                @apply --selected;
            }
        </style>
        <oda-icon ~if="icon" :icon="icon" :icon-size="iconSize * .7" style="margin-right: 8px;"></oda-icon>
        <span ~html="value" ~style="{color: item && ((item.name && item.name.includes('.md')) || (item.content && item.content.link)) ? '#6699cc' : '#336699'}"></span>`,
        props:{
            icon(){
                return this.item.icon;
            },
            value:{
                get(){
                    return this.item.label || this.item.name;
                }
            }
        },
        listeners:{
            tap: function (e){
                this.fire('wheel', 'clearScroll')
                route(this.item, 1);
            }
        }
    })
}
header:{
    ODA({is:'oda-site-header', template:`
        <style>
            :host{
                cursor: pointer;
                padding: 8px;
                justify-content: flex-end;
                @apply --flex;
                @apply --horizontal;
                overflow: hidden;
                this._resize();
            }
            div{
                align-items: center;
            }
            .vertical:{
                @apply --vertical;
            }
            .mob:hover {
                filter: invert(100%);
            }
        </style>
        <oda-button ~if="mobile" icon="icons:menu" allow-toggle ::toggled="toggled"></oda-button>
        <div ref="menu" ~if="!mobile" :parent="this" class="no-flex horizontal">
            <oda-site-header-item :mobile="mobile" ~for="items" :focused="item === part" :item="item" style="color: #336699;">{{item.label}}</oda-site-header-item>
        </div>
        <div ~if="mobile && toggled" style="font-size:18px;position:absolute;top:60px;right:0;width:auto;border:1px solid #ccc;z-index:999;background-color:#eeeeee;overflow:auto;max-height:80%">
            <div class="mob" ~for="items" :item="item" @tap="_tap(item)" style="color: #336699;justify-content:left; padding:3px;font-weight:700;">{{item.label}}
                <div class="mob" ~for="i in item.items" :item="i" @tap.stop="_tap(i)" style="color:#336699;padding:2px;margin-left:20px;font-weight:400;">{{i.label}}</div>
            </div>
        </div>
        `,
        props:{
            toggled: false,
            mobile: false,
            items: [],
            part: {
                notify: true,
                type: Object
            }
        },
        attached() {
            this._resize();
        },
        listeners:{
            resize: function (e) { this._resize(); }
        },
        _resize() {
            //this.toggled = false;
            if (this._width && this._width < window.innerWidth) {
                this._width = 0;
                this.mobile = false;
            } else if (this.$refs.menu && window.innerWidth < this.$refs.menu.offsetWidth + 400) {
                this.mobile = true;
                this._width = window.innerWidth;
            }
        },
        _tap(i) {
            this.toggled = false;
            route(i);
        }
    });
    let _selectedHeader = undefined;
    ODA({is: 'oda-site-header-item', template:`
            <style>
                :host{
                    padding: 8px 4px;
                    cursor: pointer;
                    font-size: large;
                    @align --vertical;
                }
            </style>
            <div class="horizontal" style="align-items: center">
                <oda-icon ~if="mobile" icon="icons:chevron-left"></oda-icon>
                {{item.label || item.name}}
            </div>
        `,
        properties: {
            mobile: false,
            item: {}
        },
        listeners: {
            mouseenter: function(e) { this._showDropdown(); },
            tap: function (e) { this._tap(); }
        },
        _showDropdown() {
            window.dispatchEvent(new KeyboardEvent('keydown', { 'keyCode': 27 }));
            this.async(async () => {
                let res = await ODA.showDropdown('oda-site-menu', { items: this.items, item: this.item, mobile: this.mobile }, { parent: this });
                this.value = res && res.value || undefined;
            });
            _selectedHeader = this.item;
        },
        _tap() {
            if (_selectedHeader !== this.item) this._showDropdown();
            else {
                route(this.item);
                _selectedHeader = undefined;
            }
        }
    });
}
menu:{
    ODA({is:'oda-site-menu', template:`
            <style>
                :host{
                    @apply --vertical;
                    overflow-y: auto;
                }
            </style>
            <div @mouseleave="_leave">
                <oda-site-menu-item ~for="i in items" :item="i" :parent="parent" :path="item.name" ~style="(i.items && i.items.length > 0)?'padding: 8px; font-weight: bold; font-size: large;':''"></oda-site-menu-item>
            </div>
        `,
        props: {
            mobile: Boolean,
            parent: HTMLElement,
            opened: false,
            items(){
                return (this.item && (this.item.items || []).filter(i=>!(i.name && i.name.includes('.')))) || [];
            },
            item:{
                default: {},
                set(v) { if (v && !v.items) v.items = []; }
            },
            _leave() {
                window.dispatchEvent(new KeyboardEvent('keydown', { 'keyCode': 27 }));
            }
        }
    });

    ODA({is: 'oda-site-menu-item', template:`
        <style>
            :host{
                cursor: pointer;
                @apply --vertical;
                font-weight: normal;
                color: #336699;
            }
            div:hover{
                @apply --focused;
            }
            span{
                padding: 8px;
                white-space: nowrap;
            }
        </style>
        <div class="horizontal" style="align-items: center" @tap.stop="_tap">
            <oda-icon ~if="item.icon" style="padding-left: 8px;" :icon-size="16" :icon="item.icon"></oda-icon>
            <span ~html="item.label || item.name" ~style="(item.items && item.items.length === 0)?'margin-left:16px':''"></span>
        </div>
        <oda-site-menu-item ~show="!stop" stop ~for="i in items" :path="path+'/'+item.name" :item="i" :parent="parent" style="font-weight: normal; font-size: initial;"></oda-site-menu-item>
        <!-- hr ~if="!stop && items.length>0" style="width: 100%; max-height: 2px;"-->`,
        props:{
            path: String,
            stop: false,
            items(){
                if (this.item && this.item.items)
                    return this.item.items.filter(i=>!i.name.includes('.'));
                return []
            },
            item: {},
            parent: Object
        },
        _tap(e) {
            route(this.item);
        }
    });
}