import '../../components/grids/tree/tree.js';
let iconsPath = import.meta.url;
iconsPath = iconsPath.slice(0, iconsPath.lastIndexOf('/'))+'/../../icons/svg/';
const pars = new DOMParser();
ODA({is: 'oda-icons-tree', extends: 'oda-tree',
    properties: {
        allowFocus: true,
        lazy: true,
        allowDrag: true,
        iconsList:{
            type: Array,
            set(v){
                this.dataSet = v.map(i => {
                    return { icon: i.name, drag: false, items: [{}] }
                })
            }
        },
        columns: [{ name: 'icon', treeMode: true }],
        filterVal: {
            default: '',
            watch(filter) {
                this.set(this.columns[0], '$filter', filter === '*' ? '' : filter);
                this.hideRoot = Boolean(filter);
            }
        }
    },
    async _onDragStart(e) {
        const el = e.path.find(p => p.row);
        if (el && (this.allowDrag || el.row.drag)) {
            try {
                const node = e.target.querySelector('oda-table-cell-tree').__shadowRoot.querySelector('oda-icons-icon');
                e.dataTransfer.setDragImage(node, 0, 0);
            } catch (err) {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
            }
            e.dataTransfer.setData('text/plain', `\r\n<oda-icon icon="${el.row.icon}"></oda-icon>`);
            e.dataTransfer.setData('odant/html-links', JSON.stringify([`<link rel="import" href="/web/lib/icon/icon.html">`]));
            e.dataTransfer.setData('odant/icon', el.row.icon);
        }
    },
    async _beforeExpand(item) {
        if (item.items.length > 1)
            return item.items;
        let file = await fetch(iconsPath+item.icon);
        file = await file.text();
        file = pars.parseFromString(file, 'text/html');
        file = file.querySelector('template');
        file = file.content;
        const icons = Array.from(file.children).map(i => {
            return { label: i.id, icon: item.icon + ':' + i.id, template: 'oda-icons-icon' }
        });
        icons.sort((a, b) => a.label.localeCompare(b.label));
        return icons;
    },
    async ready() {
        this.iconsList = await (await fetch(iconsPath+'info.json')).json();
    }
});

ODA({is: 'oda-icons-icon', extends: 'oda-icon, oda-table-cell-base', template:`
    <style>
        :host(:not(:hover)) > oda-button{
            display: none;
        }
        oda-button{
            opacity: 0.25;
        }
        oda-button:hover{
            opacity: 1;
        }
    </style>
    <label class="label flex" style="cursor: pointer;">{{item.label}}</label>
    <oda-button icon="icons:content-copy" @tap="_copyToClipboard"></oda-button>`,
    properties: {
        icon: {
            get() {
                return this.item.icon;
            }
        }
    },
    listeners: {
        dblclick: '_copyToClipboard'
    },
    _copyToClipboard() {
        const s = this.icon;
        const input = document.createElement('input');
        input.style.opacity = '0';
        input.style.display = 'fixed';
        document.body.appendChild(input);
        input.value = s;
        input.focus();
        input.select();
        document.execCommand("Copy");
        input.remove();
    }
});